import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.text.*;

public class CoronalMenu extends JPanel implements ActionListener {
    private JFrame frame;
    private JCheckBox vr, vt, vp, br, bt, bp, rho, t, p, jr, jt, jp,
                      ar, at, ap;
    private JTextField tmax, outRad, outInt, dtMin, dtMax, lund, visc, density,
                       temp;
    private JButton saveButton;

    public CoronalMenu () {
        super (new GridBagLayout ());

        frame = new JFrame ("Coronal Options");
        setOpaque (true);
        setBorder (BorderFactory.createRaisedBevelBorder ());
        frame.setContentPane (this);
        frame.setResizable (false);

        GridBagConstraints c = new GridBagConstraints ();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.insets = new Insets (5, 5, 5, 5);

        // Output parameters
        JLabel outputParamLabel = new JLabel ("Output Parameters:");
        add (outputParamLabel, c);

        // Output parameters check box container
        JPanel outputParamPanel = new JPanel (new GridLayout (3, 5, 10, 10));
        vr = new JCheckBox ("Vr", true);
        vt = new JCheckBox ("Vt", true);
        vp = new JCheckBox ("Vp", true);
        br = new JCheckBox ("Br", true);
        bt = new JCheckBox ("Bt", true);
        bp = new JCheckBox ("Bp", true);
        rho = new JCheckBox ("Rho", true);
        t = new JCheckBox ("T", true);
        p = new JCheckBox ("P", true);
        jr = new JCheckBox ("Jr", false);
        jt = new JCheckBox ("Jt", false);
        jp = new JCheckBox ("Jp", false);
        ar = new JCheckBox ("ar", false);
        at = new JCheckBox ("at", false);
        ap = new JCheckBox ("ap", false);
        outputParamPanel.add (vr);
        outputParamPanel.add (br);
        outputParamPanel.add (rho);
        outputParamPanel.add (jr);
        outputParamPanel.add (ar);
        outputParamPanel.add (vt);
        outputParamPanel.add (bt);
        outputParamPanel.add (t);
        outputParamPanel.add (jt);
        outputParamPanel.add (at);
        outputParamPanel.add (vp);
        outputParamPanel.add (bp);
        outputParamPanel.add (p);
        outputParamPanel.add (jp);
        outputParamPanel.add (ap);

        add (outputParamPanel, c);

        // Input panel container
        JPanel inputPanel = new JPanel (new GridLayout (9, 2, 10, 10));		

        tmax = new JTextField ("120");
        outRad = new JTextField ("30");
        outInt = new JTextField ("20");
        dtMax = new JTextField ("0.05");
        dtMin = new JTextField ("0.005");
        lund = new JTextField ("1.0e3");
        visc = new JTextField ("0.01");
        density = new JTextField ("2.0");
        temp = new JTextField ("1.8e6");
        inputPanel.add (new JLabel ("Maximum Time:"));
        inputPanel.add (tmax);
        inputPanel.add (new JLabel ("Outer Radial Boundary:"));
        inputPanel.add (outRad);
        inputPanel.add (new JLabel ("Output Interval:"));
        inputPanel.add (outInt);
        inputPanel.add (new JLabel ("dt Max:"));
        inputPanel.add (dtMax);
        inputPanel.add (new JLabel ("dt Min:"));
        inputPanel.add (dtMin);
        inputPanel.add (new JLabel ("Lundquist Number:"));
        inputPanel.add (lund);
        inputPanel.add (new JLabel ("Viscosity"));
        inputPanel.add (visc);
        inputPanel.add (new JLabel ("Density"));
        inputPanel.add (density);
        inputPanel.add (new JLabel ("Temperature"));
        inputPanel.add (temp);

        add (inputPanel, c);

        // Save all of the coronal input parameters
        saveButton = new JButton ("Save Input File Parameters");
        saveButton.addActionListener (this);
        saveButton.setActionCommand ("save");
        add (saveButton, c);

        frame.pack ();
        frame.setVisible (false);
    }

    public String getOutputParams () {
        int counter = 0;
        String outputParams = "";

        if (vr.isSelected ()) {
            outputParams += (counter == 0) ? "'vr'" : ",'vr'";
            counter++;
        }
        if (vt.isSelected ()) {
            outputParams += (counter == 0) ? "'vt'" : ",'vt'";
            counter++;
        }
        if (vp.isSelected ()) {
            outputParams += (counter == 0) ? "'vp'" : ",'vp'";
            counter++;
        }
        if (br.isSelected ()) {
            outputParams += (counter == 0) ? "'br'" : ",'br'";
            counter++;
        }
        if (bt.isSelected ()) {
            outputParams += (counter == 0) ? "'bt'" : ",'bt'";
            counter++;
        }
        if (bp.isSelected ()) {
            outputParams += (counter == 0) ? "'bp'" : ",'bp'";
            counter++;
        }
        if (rho.isSelected ()) {
            outputParams += (counter == 0) ? "'rho'" : ",'rho'";
            counter++;
        }
        if (t.isSelected ()) {
            outputParams += (counter == 0) ? "'t'" : ",'t'";
            counter++;
        }
        if (p.isSelected ()) {
            outputParams += (counter == 0) ? "'p'" : ",'p'";
            counter++;
        }
        if (jr.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jr'" : ",'jr'";
            counter++;
        }
        if (jt.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jt'" : ",'jt'";
            counter++;
        }
        if (jp.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'jp'" : ",'jp'";
            counter++;
        }
        if (ar.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'ar'" : ",'ar'";
            counter++;
        }
        if (at.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'at'" : ",'at'";
            counter++;
        }
        if (ap.isSelected ()) {
            if (counter == 9) {
                counter = 0;
                outputParams += "\n ";
            }
            outputParams += (counter == 0) ? "'ap'" : ",'ap'";
            counter++;
        }
        return outputParams;
    }

    public String getMaxTime () {
        return tmax.getText ();
    }

    public String getOuterRadial () {
        return String.valueOf (Integer.parseInt (outRad.getText ()) - 1);
    }

    public String getOutputInt () {
        return outInt.getText ();
    }

    public String getDtMax () {
        return dtMax.getText ();
    }

    public String getDtMin () {
        return dtMin.getText ();
    }

    public String getLundNum () {
        return lund.getText ();
    }

    public String getViscosity () {
        return visc.getText ();
    }

    public String getDensity () {
        return density.getText ();
    }

    public String getTemperature () {
        return temp.getText ();
    }

    public void showMenu () {
        frame.setVisible (true);
    }

    public void hideMenu () {
        frame.setVisible (false);
    }

    public void reset () {
        vr.setSelected (true);
        vt.setSelected (true);
        vp.setSelected (true);
        br.setSelected (true);
        bt.setSelected (true);
        bp.setSelected (true);
        rho.setSelected (true);
        t.setSelected (true);
        p.setSelected (true);
        jr.setSelected (false);
        jt.setSelected (false);
        jp.setSelected (false);
        ar.setSelected (false);
        at.setSelected (false);
        ap.setSelected (false);

        tmax.setText ("120");
        outRad.setText ("30");
        outInt.setText ("20");
        dtMax.setText ("0.05");
        dtMin.setText ("0.005");
        lund.setText ("1.0e3");
        visc.setText ("0.01");
        density.setText ("2.0");
        temp.setText ("1.8e6");

        // Resave the input parameters
        actionPerformed (new ActionEvent (saveButton,
                         ActionEvent.ACTION_PERFORMED, "save"));
    }

    public void saveInput () {
        int tmaxParse;
        try {
            tmaxParse = Integer.parseInt (tmax.getText ());
        } catch (NumberFormatException exc) {
            tmaxParse = 120;
            tmax.setText (Integer.toString (tmaxParse));
        }
        if (tmaxParse < 0) {
            tmaxParse = 0;
            tmax.setText (Integer.toString (tmaxParse));
        }

        int outRadParse;
        try {
            outRadParse = Integer.parseInt (outRad.getText ());
        } catch (NumberFormatException exc) {
            outRadParse = 30;
            outRad.setText (Integer.toString (outRadParse));
        }
        if (outRadParse > 30) {
            outRadParse = 30;
            outRad.setText (Integer.toString (outRadParse));
        }
        if (outRadParse < 1) {
            outRadParse = 1;
            outRad.setText (Integer.toString (outRadParse));
        }

        int outIntParse;
        try {
            outIntParse = Integer.parseInt (outInt.getText ());
        } catch (NumberFormatException exc) {
            outIntParse = 20;
            outInt.setText (Integer.toString (outIntParse));
        }
        if (outIntParse > tmaxParse) { 
            outIntParse = tmaxParse;
            outInt.setText (Integer.toString (outIntParse));
        }
        if (outIntParse < 1) {
            outIntParse = 1;
            outInt.setText (Integer.toString (outIntParse));
        }

        double dtMaxParse;
        try {
            dtMaxParse = Double.parseDouble (dtMax.getText ());
        } catch (NumberFormatException exc) {
            dtMaxParse = 0.05;
            dtMax.setText (Double.toString (dtMaxParse));
        }
        if (dtMaxParse > 1.0) {
            dtMaxParse = 1.0;
            dtMax.setText (Double.toString (dtMaxParse));
        }
        if (dtMaxParse < 0.01) {
            dtMaxParse = 0.01;
            dtMax.setText (Double.toString (dtMaxParse));
        }

        double dtMinParse;
        try {
            dtMinParse = Double.parseDouble (dtMin.getText ());
        } catch (NumberFormatException exc) {
            dtMinParse = 0.005;
            dtMin.setText (Double.toString (dtMinParse));
        }
        if (dtMinParse > dtMaxParse) {
            dtMinParse = dtMaxParse;
            dtMin.setText (Double.toString (dtMinParse));
        }
        if (dtMinParse < 0.001) {
            dtMinParse = 0.001; 
            dtMin.setText (Double.toString (dtMinParse));
        }

        double lundParse;
        try {
            lundParse = Double.parseDouble (lund.getText ());
        } catch (NumberFormatException exc) {
            lundParse = 1.0e3;
            lund.setText (Double.toString (lundParse));
        }
        if (lundParse < 0.0) {
            lundParse = 0;
            lund.setText (Double.toString (lundParse));
        }

        double viscParse;
        try {
            viscParse = Double.parseDouble (visc.getText());
        } catch (NumberFormatException exc) {
            viscParse = 0.01;
            visc.setText (Double.toString (viscParse));
        }
        if (viscParse < 0.0) {
            viscParse = 0;
            visc.setText (Double.toString (viscParse));
        }

        double densityParse;
        try {
            densityParse = Double.parseDouble (density.getText ());
        } catch (NumberFormatException exc) {
            densityParse = 2.0;
            density.setText (Double.toString (densityParse));
        }
        if (densityParse < 0.0) {
            densityParse = 0.0;
            density.setText (Double.toString (densityParse));
        }

        double tempParse;
        try {
            tempParse = Double.parseDouble (temp.getText ());
        } catch (NumberFormatException exc) {
            tempParse = 1.8e6;
            temp.setText (Double.toString (tempParse));
        }
        if (tempParse < 0.0) {
            tempParse = 0.0;
            temp.setText (Double.toString (tempParse));
        }
    }

    public void actionPerformed (ActionEvent e) {
        if (e.getActionCommand ().equals ("save")) {
            saveInput ();
            hideMenu ();
        }
    }
}
