import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.applet.Applet;
import javax.swing.*;
import java.text.*;
import java.net.*;

public class GUI extends JPanel implements ActionListener {
    private static final int scale = 400;

    private String corhelDir;
    private String[] pal;
    private ImageIcon rawImageIcon, masinImageIcon;
    private JLabel rawImageLabel, masinImageLabel;
    private JTextField ebMinBox, ebMaxBox, ePointBox, scaleMinBox, scaleMaxBox,
                       crBox;
    private JComboBox resBox, palBox, observBox, mmmaxBox;
    private JButton coronaOptButton, processCoronaButton, corhelButton,
                    helioOptButton, resetButton, helpButton, quitButton;
    private JComponent coronalMenu, helioMenu, helpMenu;

    private static void createAndShowGUI (String dirName) {
        JFrame.setDefaultLookAndFeelDecorated (true);
        JFrame frame = new JFrame ("CORHEL");
        frame.setResizable (false);
        frame.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
        JLabel label = new JLabel ("CORHEL");

        JComponent newContentPane = new GUI (dirName);
        newContentPane.setOpaque (true);
        newContentPane.setBorder (BorderFactory.createRaisedBevelBorder ());
        frame.setContentPane (newContentPane);

        frame.pack ();
        frame.setVisible (true);
    }

    public GUI (String dirName) {
        super (new BorderLayout ());

        corhelDir = dirName;

        // Create window for Coronal Options menu
        coronalMenu = new CoronalMenu ();
        // Create window for Heliospheric Options menu
        helioMenu = new HelioMenu ();
        // Create window for Help menu
        helpMenu = new HelpMenu ();

        // Carrington rotation
        JLabel crLabel = new JLabel ("Carrington Rotation");
        crBox = new JTextField ("1913");
        crBox.addActionListener (this);

        // Resolution
        JLabel resLabel = new JLabel ("Resolution");
        String[] res = new String[2];
        res[0] = "low";
        res[1] = "medium";
        resBox = new JComboBox (res);
        resBox.setSelectedIndex (0);
        resBox.addActionListener (this);

        // Observatory
        JLabel observLabel = new JLabel ("Observatory");
        String[] observ = new String[2];
        observ[0] = "Kittpeak/Solis";
        observ[1] = "Wilcox";
        observBox = new JComboBox (observ);
        observBox.setSelectedIndex (0);
        observBox.addActionListener (this);

        // Palette
        JLabel palLabel = new JLabel ("Palette");
        pal = new String[] {"EIT 171", "EIT 195", "EIT 284", "EIT Green",
                            "EIT Blue", "EIT Yellow", "Blue & White",
                            "Blue & White: Low Black", "Hot Metal", "Seismic",
                            "Seismic: Low Black", "Rainbow",
                            "Rainbow: Low Black", "Rainbow Banded",
                            "Rainbow Banded: Low Black", "B&W Contour",
                            "Volcano", "Gray Scale", "Saturn"};
        palBox = new JComboBox (pal);
        palBox.setSelectedIndex (9);
        palBox.addActionListener (this);
        pal = new String[] {"EIT171.pal", "EIT195.pal", "EIT284.pal",
                            "EIT_green.pal", "EIT_blue.pal", "EIT_yellow.pal",
                            "bluewhite.pal", "bluewhite_lowblack.pal",
                            "hotmetal.pal", "seismic.pal",
                            "seismic_lowblack.pal", "rainbow.pal",
                            "rainbow_lowblack.pal", "banded.pal",
                            "banded_lowblack.pal", "contour.pal",
                            "volcano.pal", "gray.pal", "saturn.pal"};

        // Maximum mode
        JLabel mmmaxLabel = new JLabel("Max Mode");
        String[] mmmax = new String[21];
        for (int i = 0; i <= 20; i++)
            mmmax[i] = Integer.toString (i);
        mmmaxBox = new JComboBox (mmmax);
        mmmaxBox.setSelectedIndex (9);
        mmmaxBox.addActionListener (this);

        // Extrapolation band min
        JLabel ebMinLabel = new JLabel(" Th 0");
        ebMinBox = new JTextField ("0.25");
        ebMinBox.addActionListener (this);

        // Extrapolation band max
        JLabel ebMaxLabel = new JLabel(" Th 1");
        ebMaxBox = new JTextField ("0.45");
        ebMaxBox.addActionListener (this);

        // Extrapolation point
        JLabel ePointLabel = new JLabel(" Th R");
        ePointBox = new JTextField ("0.35");
        ePointBox.addActionListener (this);

        // Image scale min
        JLabel scaleMinLabel = new JLabel ("Image Scale Min");
        scaleMinBox = new JTextField ("-40.0");
        scaleMinBox.addActionListener (this);

        // Image scale max
        JLabel scaleMaxLabel = new JLabel ("Image Scale Max");
        scaleMaxBox = new JTextField ("40.0");
        scaleMaxBox.addActionListener (this);

        // Coronal options
        coronaOptButton = new JButton ("Coronal Options");
        coronaOptButton.setActionCommand ("coronaloptions");
        coronaOptButton.addActionListener (this);

        // Heliospheric options
        helioOptButton = new JButton ("Heliospheric Options");
        helioOptButton.setActionCommand ("heliooptions");
        helioOptButton.addActionListener (this);

        // Process corona input
        processCoronaButton = new JButton ("Process Corona Input");
        processCoronaButton.setForeground (Color.RED);
        processCoronaButton.setActionCommand ("processcorona");
        processCoronaButton.addActionListener (this);

        // Run corona
        corhelButton = new JButton ("Run Corhel");
        corhelButton.setActionCommand ("corhel");
        corhelButton.addActionListener (this);

        // Help
        helpButton = new JButton ("Help");
        helpButton.setActionCommand ("help");
        helpButton.addActionListener (this);

        // Reset
        resetButton = new JButton ("Reset");
        resetButton.setActionCommand ("reset");
        resetButton.addActionListener (this);

        // Quit
        quitButton = new JButton ("Quit");
        quitButton.setActionCommand ("quit");
        quitButton.addActionListener (this);

        // Raw image
        JLabel rawLabel = new JLabel ("Raw Image", JLabel.CENTER);
        rawImageIcon = new ImageIcon ("./gif/raw.gif");
        rawImageIcon.setImage ((rawImageIcon.getImage ()).
                    getScaledInstance (scale, scale / 2, Image.SCALE_FAST));
        rawImageLabel = new JLabel (rawImageIcon);
        ImageIcon westAxisImageIcon = new ImageIcon ("./gif/west.png");
        westAxisImageIcon.setImage (westAxisImageIcon.getImage ().
                    getScaledInstance (25, scale / 2, Image.SCALE_FAST));
        JLabel westAxisImageLabel = new JLabel (westAxisImageIcon);
        ImageIcon southAxisImageIcon = new ImageIcon ("./gif/south.png");
        southAxisImageIcon.setImage (southAxisImageIcon.getImage ().
                    getScaledInstance (scale + 25, 25, Image.SCALE_FAST));
        JLabel southAxisImageLabel = new JLabel (southAxisImageIcon);

        // Mas input image
        JLabel masinLabel = new JLabel ("MAS Input", JLabel.CENTER);
        masinImageIcon = new ImageIcon ("./gif/masin.gif");
        masinImageIcon.setImage ((masinImageIcon.getImage ()).
                    getScaledInstance (scale, scale, Image.SCALE_FAST));
        masinImageIcon.setImage (rotate (masinImageIcon.getImage (), scale,
                                         scale));
        masinImageIcon.setImage ((masinImageIcon.getImage ()).
                    getScaledInstance (scale, scale / 2, Image.SCALE_FAST));
        masinImageLabel = new JLabel (masinImageIcon);
        ImageIcon westAxisImageIcon2 = new ImageIcon ("./gif/west.png");
        westAxisImageIcon2.setImage (westAxisImageIcon2.getImage ().
                    getScaledInstance (25, scale / 2, Image.SCALE_FAST));
        JLabel westAxisImageLabel2 = new JLabel (westAxisImageIcon2);
        ImageIcon southAxisImageIcon2 = new ImageIcon ("./gif/south.png");
        southAxisImageIcon2.setImage (southAxisImageIcon2.getImage ().
                    getScaledInstance (scale + 25, 25, Image.SCALE_FAST));
        JLabel southAxisImageLabel2 = new JLabel (southAxisImageIcon2);


        // Image panel container
        JPanel imagePanel = new JPanel (new GridBagLayout ());
        GridBagConstraints c = new GridBagConstraints ();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.insets = new Insets (0, 0, 0, 0);
        imagePanel.add (westAxisImageLabel, c);
        c.gridx = 1;
        c.insets = new Insets (0, 0, 0, 5);
        imagePanel.add (rawImageLabel, c);
        c.gridx = 2;
        c.insets = new Insets (0, 5, 0, 0);
        imagePanel.add (westAxisImageLabel2, c);
        c.gridx = 3;
        c.insets = new Insets (0, 0, 0, 0);
        imagePanel.add (masinImageLabel, c);
        c.gridx = 0;
        c.gridy = 1;
        c.gridwidth = 2;
        c.insets = new Insets (0, 0, 0, 5);
        imagePanel.add (southAxisImageLabel, c);
        c.gridx = 2;
        c.insets = new Insets (0, 5, 0, 0);
        imagePanel.add (southAxisImageLabel2, c);
        c.gridx = 0;
        c.gridy = 2;
        imagePanel.add (rawLabel, c);
        c.gridx = 2;
        imagePanel.add (masinLabel, c);

        // Input panel container
        JPanel leftInputPanel  = new JPanel (new GridLayout (6, 2));
        leftInputPanel.add (new JLabel ());
        leftInputPanel.add (new JLabel ());
        leftInputPanel.add (crLabel);
        leftInputPanel.add (crBox);
        leftInputPanel.add (resLabel);
        leftInputPanel.add (resBox);
        leftInputPanel.add (observLabel);
        leftInputPanel.add (observBox);
        leftInputPanel.add (palLabel);
        leftInputPanel.add (palBox);
        leftInputPanel.add (mmmaxLabel);
        leftInputPanel.add (mmmaxBox);
        ((GridLayout) leftInputPanel.getLayout ()).setVgap (10);

        JPanel rightInputPanel  = new JPanel (new GridLayout (3, 2));
        rightInputPanel.add (new JLabel ());
        rightInputPanel.add (new JLabel ());
        rightInputPanel.add (scaleMinLabel);
        rightInputPanel.add (scaleMinBox);
        rightInputPanel.add (scaleMaxLabel);
        rightInputPanel.add (scaleMaxBox);
        ((GridLayout) rightInputPanel.getLayout ()).setVgap (15);

        JPanel polefitPanel = new JPanel (new GridLayout (3, 2));
        polefitPanel.setBorder (BorderFactory.createTitledBorder
                                ("Polefitting"));
        polefitPanel.add (ebMinLabel);
        polefitPanel.add (ebMinBox);
        polefitPanel.add (ebMaxLabel);
        polefitPanel.add (ebMaxBox);
        polefitPanel.add (ePointLabel);
        polefitPanel.add (ePointBox);
        ((GridLayout) polefitPanel.getLayout ()).setVgap (10);

        JPanel dummyPanel = new JPanel (new GridLayout (1, 3));
        dummyPanel.add (new JLabel ());
        dummyPanel.add (processCoronaButton);
        dummyPanel.add (new JLabel ());

        // Execute panel container
        JPanel execPanel = new JPanel (new GridLayout (2, 3, 5, 5));
        execPanel.add (coronaOptButton);
        execPanel.add (helioOptButton);
        execPanel.add (corhelButton);
        execPanel.add (helpButton);
        execPanel.add (resetButton);
        execPanel.add (quitButton);

        JPanel inputPanel = new JPanel (new GridBagLayout ());
        c = new GridBagConstraints ();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        c.gridy = 0;
        c.gridheight = 2;
        c.insets = new Insets (5, 5, 5, 10);
        inputPanel.add (leftInputPanel, c);
        c.gridx = 1;
        c.gridy = 0;
        c.gridheight = 1;
        c.ipadx = 200;
        c.insets = new Insets (5, 10, 5, 5);
        inputPanel.add (rightInputPanel, c);
        c.gridx = 1;
        c.gridy = 1;
        inputPanel.add (polefitPanel, c);
        c.gridx = 0;
        c.gridy = 2;
        c.gridwidth = 2;
        c.insets = new Insets (10, 5, 10, 5);
        inputPanel.add (dummyPanel, c);
        c.gridx = 0;
        c.gridy = 3;
        inputPanel.add (new JSeparator (), c);
        c.gridx = 0;
        c.gridy = 4;
        inputPanel.add (execPanel, c);

        // Main panel container
        add (imagePanel, BorderLayout.NORTH);
        add (inputPanel, BorderLayout.CENTER);
    }

    public Image rotate (Image inputImage, int x, int y) {
        int size = x * y;
        int buffer[] = new int[size];
        int rotate[] = new int[size];
        int bufferIndex[] = new int[size];
        int rotateIndex[] = new int[size];

        PixelGrabber grabber = new PixelGrabber (inputImage, 0, 0, x, y,
                                                 buffer, 0, x);
        try {
            grabber.grabPixels ();
        } catch (InterruptedException e) {
            e.printStackTrace ();
        }

        for (int i = 0; i < bufferIndex.length; i++)
            bufferIndex[i] = i;

        rotateIndex[size - 1] = bufferIndex[x - 1];
        for (int i = size - 2; i > size - y - 1; i--)
            rotateIndex[i] = rotateIndex[i + 1] + x;

        for (int i = size - 1 - y; i >= 0; i--)
            rotateIndex[i] = rotateIndex[i+y]-1;

        for (int i = 0; i < size; i++)
            rotate[i] = buffer[rotateIndex[i]];

        return createImage (new MemoryImageSource (y, x, rotate, 0, y));
    }

    public void checkInput () {
        int cr;
        try {   
            cr = Integer.parseInt (crBox.getText ());
        } catch (NumberFormatException exc) {
            cr = 1913;
            crBox.setText (Integer.toString (cr));
        }
        if (cr < 1625) {
            cr = 1625;
            crBox.setText (Integer.toString (cr));
        }
        if (cr > 2007) {
            cr = 2007;
            crBox.setText (Integer.toString (cr));
        }

        double smin, smax;
        try {
            smin = Double.parseDouble (scaleMinBox.getText ());
        } catch (NumberFormatException exc) {
            smin = -40.0;
            scaleMinBox.setText ("-40.0");
        }
        try {
            smax = Double.parseDouble (scaleMaxBox.getText ());
        } catch (NumberFormatException exc) {
            smax = 40.0;
            scaleMaxBox.setText ("40.0");
        }
        if (smin > smax) {
            smin = smax;
            scaleMinBox.setText (Double.toString (smin));
        }

        double emin, emax, epoint;
        try {
            emin = Double.parseDouble (ebMinBox.getText ());
        } catch (NumberFormatException exc) {
            emin = 0.25;
            ebMinBox.setText (Double.toString (emin));
        }
        try {
            emax = Double.parseDouble (ebMaxBox.getText ());
        } catch (NumberFormatException exc) {
            emax = 0.45;
            ebMaxBox.setText (Double.toString (emax));
        }
        try {
            epoint = Double.parseDouble (ePointBox.getText ());
        } catch (NumberFormatException exc) {
            epoint = 0.35;
            ePointBox.setText (Double.toString (epoint));
        }
        if (emin < 0.0) {
            emin = 0.0;
            ebMinBox.setText (Double.toString (emin));
        }
        if (emin > 1.57) {
            emin = 1.57;
            ebMinBox.setText (Double.toString (emin));
        }
        if (emax < 0.0) {
            emax = 0.0;
            ebMaxBox.setText (Double.toString (emax));
        }
        if (emax > 1.57) {
            emax = 1.57;
            ebMaxBox.setText (Double.toString (emax));
        }
        if (epoint < emin) {
            epoint = emin;
            ePointBox.setText (Double.toString (epoint));
        }
        if (epoint > emax) {
            epoint = emax;
            ePointBox.setText (Double.toString (epoint));
        }
    }

    public void processCoronaInput () throws IOException,
                                             InterruptedException {
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();
        String palette = pal[palBox.getSelectedIndex ()];
        String mmmax = (String) mmmaxBox.getSelectedItem ();
        String smin = scaleMinBox.getText ();
        String smax = scaleMaxBox.getText ();
        String ebmin = ebMinBox.getText ();
        String ebmax = ebMaxBox.getText ();
        String epoint = ePointBox.getText ();
        String observ = (observBox.getSelectedIndex () == 0) ? "kittpeak"
                                                             : "wilcox";

        String tmp;
        Process p;
        BufferedReader br;
        Runtime run = Runtime.getRuntime ();

        p = run.exec (corhelDir + "/gui/clear");
        p.waitFor ();
        p.destroy ();

        p = run.exec (corhelDir + "/bin/corhel " + cr + " " + res);
        br = new BufferedReader (new InputStreamReader (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();

        p = run.exec (corhelDir + "/bin/prep_corona " + cr + " " + res + " " +
                      observ + " " + mmmax + " " + ebmin + " " + ebmax + " " +
                      epoint);
        br = new BufferedReader (new InputStreamReader (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();

        double rmin = Double.parseDouble (smin) * 2.2;
        double rmax = Double.parseDouble (smax) * 2.2;
        p = run.exec (corhelDir + "/gui/hdf2gif" + " " + cr + " " + res + " " +
                      palette + " " + smin + " " + smax + " " + rmin + " " +
                      rmax);
        br = new BufferedReader (new InputStreamReader (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();
    }

    public void updateImage () {
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();

        rawImageIcon = new ImageIcon (corhelDir + "/run/cr"+ cr + "-" + res +
                                      "/movie/raw" + cr + res + ".gif");
        masinImageIcon = new ImageIcon (corhelDir + "/run/cr"+ cr + "-" + res +
                                        "/movie/masin" + cr + res + ".gif");

        rawImageIcon.setImage ((rawImageIcon.getImage ()).
                    getScaledInstance (scale, scale, Image.SCALE_FAST));
        rawImageIcon.setImage (rotate (rawImageIcon.getImage (), scale,
                                       scale));
        rawImageIcon.setImage ((rawImageIcon.getImage ()).
                    getScaledInstance (scale, scale / 2, Image.SCALE_FAST));
        masinImageIcon.setImage ((masinImageIcon.getImage ()).
                    getScaledInstance (scale, scale, Image.SCALE_FAST));
        masinImageIcon.setImage (rotate (masinImageIcon.getImage (), scale,
                                         scale));
        masinImageIcon.setImage ((masinImageIcon.getImage ()).
                    getScaledInstance (scale, scale / 2, Image.SCALE_FAST));

        rawImageIcon.getImage ().flush ();
        masinImageIcon.getImage ().flush ();

        rawImageLabel.setIcon (rawImageIcon);
        masinImageLabel.setIcon (masinImageIcon);

        this.repaint(new Rectangle (0, 0, 800, 800));
    }

    public void createIMAS () throws IOException, InterruptedException {
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();
        String mmmax = (String) mmmaxBox.getSelectedItem ();

        String modestring = "0";
        int modenum = Integer.valueOf (mmmax).intValue ();
        for (int i = 1; i <= modenum; i++)
            modestring += "," + i;

        CoronalMenu tmp = (CoronalMenu) coronalMenu;
        String imas =
            " &invars\n" +
            " option='streamer'\n" +
            " fldtype='potential' bingauss=.true." +
            " bnfile='br.m9.filt3.dat'\n" +
            " eqtype='parker' np1d=130 onedfile='parker1.8.pw' rhor0=" +
                tmp.getDensity () + "\n" +
            " bcr0type='1dchar' bcr1type='1dchar'\n" +
            " tmax=" + tmp.getMaxTime() + " ntmax=40000" +
            " dtmax=" + tmp.getDtMax() + " dtmin=" + tmp.getDtMin() + "\n" +
            " ifideal=0 slund=" + tmp.getLundNum() + " visc=" +
                tmp.getViscosity() + "\n" +
            " rsifile=' '\n" +
            " rl=" + tmp.getOuterRadial() + ". g0=.823 ifrho=1" +
                " iftemp=1 ifvdgv=1 ifpc=1\n" +
            " rfrac=.4,1. drratio=60.,4. nfrmesh=5\n" +
            " tfrac=.5,1. dtratio=.5,2. nftmesh=5\n" +
            " mmodes=" + modestring + "\n" +
            " tpltxint=" + tmp.getOutputInt() + ". ihistint=1 ifprec=1" +
                " trsdump=20.\n" +
            " upwindv=1. cfl=.4 isitype=1\n" +
            " dformat='hdf'\n" +
            " plotlist=" + tmp.getOutputParams() + "\n" +
            " tnode=0.,10.0 vnode=0.,0. ishearprof=2\n" +
            " arotate=1. brotate=0. crotate=0.\n" +
            " ihst=23 jhst=33 khst=1\n" +
            " parchar=.false. ubzero=.true. nfiltub=3\n" +
            " he_frac=0. radloss=0. tcond=0. ifaw=0\n" +
            " tnode_ch=0.,500. q0phys_ch=0.,0.\n" +
            " tbc0=" + tmp.getTemperature () + " tbc1=0.\n" +
            " upwinda=0. upwindar=1. upwindat=1. upwindap=1.\n" +
            " upwindphi=1. strmax=.4\n" +
            " fdphi=.false.\n" +
            " si_aggressive=.true.\n" +
            " /";

        String filename = corhelDir + "/run/cr" + cr + "-" + res +
                          "/corona/imas";
        Runtime run = Runtime.getRuntime ();
        Process p = run.exec ("touch " + filename);
        p.waitFor ();
        p.destroy ();

        FileWriter fw = new FileWriter (filename);
        fw.write (imas);
        fw.flush ();
        fw.close ();
    }

    public void runCorona () throws IOException, InterruptedException {
        String tmp;
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();
        Runtime run = Runtime.getRuntime ();
        Process p = run.exec (corhelDir + "/bin/corona "+ cr + " " + res);
        BufferedReader br = new BufferedReader (new InputStreamReader
                                                (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();
    }

    public void processHelioInput () throws IOException, InterruptedException {
        String tmp;
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();
        Runtime run = Runtime.getRuntime ();
        Process p = run.exec (corhelDir + "/bin/prep_helio "+ cr + " " + res);
        BufferedReader br = new BufferedReader (new InputStreamReader
                                                (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();
    }

    public void runHelio () throws IOException, InterruptedException {
        String tmp;
        String cr = crBox.getText ();
        String res = (String) resBox.getSelectedItem ();
        Runtime run = Runtime.getRuntime ();
        Process p = run.exec (corhelDir + "/bin/helio_cr "+ cr + " " + res);
        BufferedReader br = new BufferedReader (new InputStreamReader
                                                (p.getInputStream ()));
        while ((tmp = br.readLine ()) != null)
            System.out.println (tmp);
        p.waitFor ();
        p.destroy ();
    }

    public void reset () {
        crBox.setText ("1913");
        resBox.setSelectedIndex (0);
        palBox.setSelectedIndex (9);
        scaleMinBox.setText ("-40.0");
        scaleMaxBox.setText ("40.0");
        mmmaxBox.setSelectedIndex (9);
        ebMinBox.setText ("0.25");
        ebMaxBox.setText ("0.45");
        ePointBox.setText ("0.35");
        ((CoronalMenu) coronalMenu).reset ();
        ((HelioMenu) helioMenu).reset ();
    }

    public void actionPerformed (ActionEvent e) {
        checkInput ();

        if (e.getActionCommand ().equals ("coronaloptions")) {
            ((CoronalMenu) coronalMenu).showMenu ();
        }

        if (e.getActionCommand ().equals ("processcorona")) {
            try {
                processCoronaInput ();
                updateImage ();
            } catch (IOException ioe) {
                System.err.println (ioe.getMessage ());
                System.exit (1);
            } catch (InterruptedException ie) {
                System.err.println (ie.getMessage ());
                System.exit (1);
            }
        }

        if (e.getActionCommand ().equals ("corhel")) {
            try {
                createIMAS ();
                runCorona ();
                processHelioInput ();
                runHelio ();
            } catch (IOException ioe) {
                System.err.println (ioe.getMessage ());
                System.exit (1);
            } catch (InterruptedException ie) {
                System.err.println (ie.getMessage ());
                System.exit (1);
            }
        }

        if (e.getActionCommand ().equals ("heliooptions")) {
            ((HelioMenu) helioMenu).showMenu ();
        }

        if (e.getActionCommand ().equals ("help")) {
            ((HelpMenu) helpMenu).showMenu ();
        }

        if (e.getActionCommand ().equals ("quit")) {
            System.exit (0);
        }

        if (e.getActionCommand ().equals ("reset")) {
            reset ();
        }
    }

    public static void main (String[] args) {
        if (args[0] == null) {
            System.out.println ("\nError: Corhel directory is not specified");
            System.exit (1);
        }

        final String arg = args[0];
        javax.swing.SwingUtilities.invokeLater (new Runnable () {
            public void run () {
                createAndShowGUI (arg);
            }
        });
    }
}
