c
c Program reads in output from mapfl and computes the 
c the radial speed at 30 solar radii based on several 
c methods:
c 1. speed based on distance from CH boundary (SAIC approach)
c 2. Speed based on expansion factor of field lines (Wang-Sheeley-Arge)
c 3. Speed based on combo of dist from CH and Exp. fac (Arge 2002).
c
c-----------------------------------------------------------------------
c
      integer idim(2),ips(2)
      pointer (ipt,t),(ipp,p)
c
      pointer (iprfl0,rfl0)
c
      pointer (iptfl0,tfl0)
c
      pointer (ippfl0,pfl0)
c
      pointer (ipefl1,efl1)
c
      pointer (ipcd,cd)
c
      pointer (iprfl1,rfl1)
c
      pointer (ipv1,v1)
      pointer (ipv2,v2)
      pointer (ipv3,v3)
      pointer (ipv4,v4)
c
c use command-line input for supplying arguments eps and width
c
      integer iargc
      external iargc,getarg
      character*(72) arg1, arg2
      real eps, width
c
c-----------------------------------------------------------------------
c
      call getarg (1,arg1)
      call getarg (2,arg2)
      READ(arg1,FMT='(F10.4)') eps
      READ(arg2,FMT='(F10.4)') width
      write(*,*)
      write(*,*)
      write(*,*)
      write(*,*) '--------------------------'
      write(*,*)
      write(*,*) 'Setting eps/width to:',eps,width
      write(*,*)
      write(*,*) '--------------------------'
      write(*,*)
      write(*,*)
      write(*,*)
c
c-----------------------------------------------------------------------
c
      call rdsds ('hdf','r0.hdf',2,ndim,idim,iprfl0,ifscale,ips,ierr)
      call rdsds ('hdf','t0.hdf',2,ndim,idim,iptfl0,ifscale,ips,ierr)
      call rdsds ('hdf','p0.hdf',2,ndim,idim,ippfl0,ifscale,ips,ierr)
      call rdsds ('hdf','e1.hdf',2,ndim,idim,ipefl1,ifscale,ips,ierr)
      call rdsds ('hdf','ch_dist.hdf',2,ndim,idim,ipcd,ifscale,ips,ierr)
      call rdsds ('hdf','r1.hdf',2,ndim,idim,iprfl1,ifscale,ips,ierr)
c
      nt=idim(1)
      np=idim(2)
c
      ipt=ips(1)
      ipp=ips(2)
c
      nwd=nt*np
      ipv1=igetmem(nwd)
      ipv2=igetmem(nwd)
      ipv3=igetmem(nwd)
      ipv4=igetmem(nwd)
c
       call getv1(nt,np,t,p,rfl0,tfl0,pfl0,rfl1,v1,eps,width)

c      call getv2(nt,np,t,p,rfl0,tfl0,pfl0,cd,v2,eps,width)

c      call getv3(nt,np,t,p,rfl0,tfl0,pfl0,efl1,v3)
c
c      call getv4(nt,np,t,p,rfl0,tfl0,pfl0,efl1,cd,v4,eps,width)
c
      call wrsds ('hdf','v_smooth_'//TRIM(arg1)//'_'//
     &     TRIM(arg2)//'.hdf',2,idim,v1,ifscale,ips,ierr)
c
c      call wrsds ('hdf','v_smooth_precomp_cd_'//TRIM(arg1)//'_'//
c     &     TRIM(arg2)//'.hdf',2,idim,v2,ifscale,ips,ierr)
c
c      call wrsds ('hdf','v_smooth_ws_orig_'//TRIM(arg1)//'_'//
c     &     TRIM(arg2)//'.hdf',2,idim,v3,ifscale,ips,ierr)
c
c      call wrsds ('hdf','v_smooth_ws_rev_'//TRIM(arg1)//'_'//
c     &     TRIM(arg2)//'.hdf',2,idim,v4,ifscale,ips,ierr)
c
      end
c#######################################################################
      subroutine getv1 (nt,np,t,p,rfl0,tfl0,pfl0,rfl1,v1,eps,width)
c
c-----------------------------------------------------------------------
c
      real t(nt),p(np)
      real rfl0(nt,np)
      real tfl0(nt,np)
      real pfl0(nt,np)
      real rfl1(nt,np)
      real v1(nt,np)
      real eps,width
c
c-----------------------------------------------------------------------
c
      parameter (rmin=1.05)
      parameter (rmax=19.5)
c
ccc      parameter (vslow=.625)
ccc      parameter (vfast=1.166)

      parameter (vslow=.521)
      parameter (vfast=1.354)
c
c      parameter (eps=.1)
c      parameter (width=.05)
c
c-----------------------------------------------------------------------
c
      do i=1,nt
        do j=1,np
          r0=rfl0(i,j)
          t0=tfl0(i,j)
          p0=pfl0(i,j)
          if (r0.gt.rmax) then
            v1(i,j)=vslow
          else if (r0.lt.rmin) then
            call s2c (r0,t0,p0,x0,y0,z0)
            dmin=1.e10
            do ii=1,nt
              do jj=1,np
                r1=rfl1(ii,jj)
                if (r1.lt.rmin) then
                  t1=t(ii)
                  p1=p(jj)
                  call s2c (r0,t1,p1,x1,y1,z1)
                  dist=sqrt((x1-x0)**2+(y1-y0)**2+(z1-z0)**2)
                  dmin=min(dmin,dist)
                end if
              enddo
            enddo
ccc            if (dmin.lt.eps) then
ccc              v1(i,j)=vslow
ccc            else
ccc              v1(i,j)=vfast
ccc            end if
ccc            prof=(1.-tanh(dmin/width))
            prof=.5*(1.+tanh((dmin-eps)/width))
            v1(i,j)=vslow+(vfast-vslow)*prof
          else
            write (*,*)
            write (*,*) '### WARNING: Intermediate radius!'
            write (*,*) 'i =',i
            write (*,*) 'j =',j
            write (*,*) 'r0 = ',r0
            v1(i,j)=vslow
          end if
        enddo
      enddo
c
      return
      end

c#######################################################################

      subroutine getv2(nt,np,t,p,rfl0,tfl0,pfl0,cd,v2,eps,width)
c
c modification to original getv subroutine. 
c Uses ch_dist previously calculated so it removes 
c 2 of the loops. Should be much quicker, and since the 
c ch_dist routine is more optimized, the combined process 
c should be more efficient, esp. when other methods are used. 
c
c-----------------------------------------------------------------------
c
      real t(nt),p(np)
      real rfl0(nt,np)
      real tfl0(nt,np)
      real pfl0(nt,np)
      real v2(nt,np)
      real cd(nt,np)
      real eps,width
c
c-----------------------------------------------------------------------
c
      parameter (rmin=1.05)
      parameter (rmax=19.5)
c
      parameter (vslow=.521)
      parameter (vfast=1.354)
c
c-----------------------------------------------------------------------
c
      do i=1,nt
        do j=1,np
          r0=rfl0(i,j)
          t0=tfl0(i,j)
          p0=pfl0(i,j)
          if (r0.gt.rmax) then
            v2(i,j)=vslow
          else if (r0.lt.rmin) then
            call interp (nt,t,t0,ii,iip1,at,ierr)
            if (ierr.ne.0)then
                    write(*,*) "Error from interp"
            endif
            call interp (np,p,p0,jj,jjp1,ap,ierr)
            if (ierr.ne.0)then 
                    write(*,*) "Error from interp"
            endif
            dmin= (1.-at)*((1.-ap)*abs(cd(ii,jj))+ap*abs(cd(ii,jjp1)))
     &     +    at *((1.-ap)*abs(cd(iip1,jj))+ap*abs(cd(iip1,jjp1)))
            prof=.5*(1.+tanh((dmin-eps)/width))
            v2(i,j)=vslow+(vfast-vslow)*prof
          else
            write (*,*)
            write (*,*) '### WARNING: Intermediate radius!'
            write (*,*) 'i =',i
            write (*,*) 'j =',j
            write (*,*) 'r0 = ',r0
            v2(i,j)=vslow
          end if
        enddo
      enddo
c
      return
      end

c#######################################################################

      subroutine getv3(nt,np,t,p,rfl0,tfl0,pfl0,efl1,v3)
c
c this version of getv uses the original Wang-Sheeley-Arge
c prescription for the flow speed based on the expansion 
c factor,  f.  (called e in our notation).
c
c-----------------------------------------------------------------------
c
      real t(nt),p(np)
      real rfl0(nt,np)
      real tfl0(nt,np)
      real pfl0(nt,np)
      real efl1(nt,np)
      real v3(nt,np)
c
c-----------------------------------------------------------------------
c
      do i=1,nt
        do j=1,np
          r0=rfl0(i,j)
          t0=tfl0(i,j)
          p0=pfl0(i,j)
          call interp (nt,t,t0,ii,iip1,at,ierr)
          if (ierr.ne.0)then
                  write(*,*) "Error from interp"
          endif
          call interp (np,p,p0,jj,jjp1,ap,ierr)
          if (ierr.ne.0)then 
                  write(*,*) "Error from interp"
          endif
          fs  = (1.-at)*((1.-ap)*abs(efl1(ii,jj))+ap*abs(efl1(ii,jjp1)))
     &     +    at *((1.-ap)*abs(efl1(iip1,jj))+ap*abs(efl1(iip1,jjp1)))
           v3(i,j) = (285. + 650./(fs**(5./9.)))/481.37
        enddo
      enddo
c
      return
      end

c#######################################################################

      subroutine getv4(nt,np,t,p,rfl0,tfl0,pfl0,efl1,cd,v4,eps,width)
c
c modification to getv subroutine. 
c uses both the expansion factor and the distance 
c from the CH boundary, as specified in equation (2) 
c of Arge et al. (200) [solar wind 10]. 
c
c-----------------------------------------------------------------------
c
      real t(nt),p(np)
      real rfl0(nt,np)
      real tfl0(nt,np)
      real pfl0(nt,np)
      real efl1(nt,np)
      real v4(nt,np)
      real cd(nt,np)
      real eps,width
c
c-----------------------------------------------------------------------
c
      parameter (rmin=1.05)
      parameter (rmax=19.5)
c
      parameter (vslow=.521)
      parameter (vfast=1.354)
c
c-----------------------------------------------------------------------
c
      do i=1,nt
        do j=1,np
          r0=rfl0(i,j)
          t0=tfl0(i,j)
          p0=pfl0(i,j)
          call interp (nt,t,t0,ii,iip1,at,ierr)
          if (ierr.ne.0)then
                  write(*,*) "Error from interp"
          endif
          call interp (np,p,p0,jj,jjp1,ap,ierr)
          if (ierr.ne.0)then 
                  write(*,*) "Error from interp"
          endif
          dmin= (1.-at)*((1.-ap)*abs(cd(ii,jj))+ap*abs(cd(ii,jjp1)))
     &     +    at *((1.-ap)*abs(cd(iip1,jj))+ap*abs(cd(iip1,jjp1)))
          fs  = (1.-at)*((1.-ap)*abs(efl1(ii,jj))+ap*abs(efl1(ii,jjp1)))
     &     +    at *((1.-ap)*abs(efl1(iip1,jj))+ap*abs(efl1(iip1,jjp1)))
          v4(i,j)=(265. + (25./(fs**(2./7.)))*
     &              (5.-1.1*EXP(1.-(dmin/4.)**2))**2
     &      )/481.37
        enddo
      enddo
c
      return
      end

c#######################################################################

      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      ndim=0
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=1
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
      nb=4*nwd
c
c     ip=malloc(%val(nb))
      ip=malloc(nb)
c
      if (ip.eq.0) then
        write (*,*)
        write (*,*) '### ERROR in IGETMEM:'
        write (*,*) '### Could not get the requested memory.'
        write (*,*) 'Number of bytes requested = ',nb
        call exit (1)
      end if
c
      igetmem=ip
c
      return
      end
c#######################################################################
      subroutine c2s (x,y,z,r,t,p)
c
c-----------------------------------------------------------------------
c
c ****** Convert from Cartesian coordinates (X,Y,Z)
c ****** to spherical coordinates (R,T,P).
c
c ****** This routine returns T and P in radians, in the
c ****** following range:
c
c          0. .le. t .le. pi
c          0. .le. p .lt. 2.*pi
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pid2=.5*pi)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
      r=sqrt(x**2+y**2+z**2)
c
      if (r.eq.0.) then
        t=0.
      else
        t=acos(z/r)
      end if
c
      if (x.eq.0.) then
        if (y.ge.0.) then
          p=pid2
        else
          p=-pid2
        end if
      else
        p=atan2(y,x)
      end if
      if (p.lt.0.) p=p+pi2
c
      return
      end
c#######################################################################
      subroutine s2c (r,t,p,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Convert from spherical coordinates (R,T,P) 
c ****** to Cartesian coordinates (X,Y,Z).
c
c ****** This routine assumes that T and P are in radians.
c
c-----------------------------------------------------------------------
c
      st=sin(t)
      x=r*st*cos(p)
      y=r*st*sin(p)
      z=r*cos(t)
c
      return
      end

c#######################################################################
      subroutine interp (n,x,xv,i,ip1,a,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Get the interpolation factor at XV from the table X(N).
c
c-----------------------------------------------------------------------
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I, IP1, and A to get the interpolated
c ****** value.
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      integer :: n
c      real dimension(n) :: x
      real x(n)
      real :: xv
      integer :: i,ip1
      real :: a
      integer :: ierr
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Check if the x-scale has only one point.
c
      if (n.eq.1.and.xv.eq.x(1)) then
        ip1=1
        a=0.
        return
      end if
c
c ****** Find the interval and compute the interpolation factor.
c
      do i=1,n-1
        if (xv.ge.x(i).and.xv.le.x(i+1)) then
          ip1=i+1
          if (x(i).eq.x(i+1)) then
            a=0.
          else
            a=(xv-x(i))/(x(i+1)-x(i))
          end if
          return
        end if
      enddo
c
c ****** ERROR: the value was not found.
c
      write (*,*)
      write (*,*) '### ERROR in INTERP:'
      write (*,*) '### Value not found in table.'
      write (*,*) 'Value requested = ',xv
      write (*,*) 'Min table value = ',x(1)
      write (*,*) 'Max table value = ',x(n)
      write (*,*) 'Number of values in table = ',n
      ierr=1
c
      return
      end
