c
c-----------------------------------------------------------------------
c
c ****** Read an HDF data file and write it as a text file.
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        05/05/91, ZM, Version 1.00:
c
c         - Original version of program.
c
c        10/12/92, ZM, Version 1.01:
c
c         - Installed the new command line syntax parser.
c         - Cleaned up a bit.
c
c        11/04/92, ZM, Version 1.02:
c
c         - Modified for SGI IRIS from CRAY UNICOS.
c
c        07/29/2003, ZM, Version 1.03:
c
c         - This version uses the ZM tools libraries.
c         - The program thus uses the new HDF read/write
c           routines that can handle 32-bit and 64-bit data files.
c
c-----------------------------------------------------------------------
c
c#######################################################################
c
c ****** Note that this tool uses modules from ZM's tools library.
c
c#######################################################################
      module params
c
c-----------------------------------------------------------------------
c ****** Parameters.
c-----------------------------------------------------------------------
c
      implicit none
c
      character(256) :: hdffile
      character(256) :: textfile
c
      end module
c#######################################################################
      program HDF2TXT
c
c-----------------------------------------------------------------------
c
      use params
      use sds_def
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='HDF2TXT'
      character(*), parameter :: cvers='1.03'
      character(*), parameter :: cdate='07/29/2003'
c
c-----------------------------------------------------------------------
c
      type(sds) :: s
      integer :: ierr
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call set_parameters (cname,cvers,cdate)
c
c ****** Read the field from the HDF file.
c
      call rdhdf (hdffile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the HDF data file.'
        write (*,*) 'IERR (from RDHDF) = ',ierr
        write (*,*) 'File name: ',trim(hdffile)
        call exit (1)
      end if
c
c ****** Write the text file.
c
      call wrtxt (textfile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write the text data file.'
        write (*,*) 'IERR (from WRTXT) = ',ierr
        write (*,*) 'File name: ',trim(textfile)
        call exit (1)
      end if
c
      call exit (0)
c
      end
c#######################################################################
      subroutine set_parameters (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use number_types
      use syntax
      use paragraph_def
      use get_usage_line_interface
      use print_par_interface
      use delete_par_interface
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*) :: cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Storage the for usage line.
c
      type(paragraph), pointer :: usage
c
c ****** Storage for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      integer :: ierr
      character(256) :: arg
      logical :: set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (GROUP_A,'hdffile',' ',' ')
      call defarg (GROUP_A,'textfile','hdffile',' ')
c
c ****** Parse the command line.
c
      call parse (errmsg,ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Make a text file from an HDF file.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage line.
c
        call get_usage_line (usage)
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
c
        call print_par (usage)
c
        write (*,*)
        write (*,*) 'If <textfile> is not specified, the HDF file'//
     &              ' <hdffile>.hdf will be read,'
        write (*,*) 'and the text file produced will be named'//
     &              ' <hdffile>.dat.'
c
        call delete_par (usage)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Input file name.
c
      call fetcharg ('hdffile',set,arg)
      hdffile=trim(arg)
c
c ****** Output file name.
c
      call fetcharg ('textfile',set,arg)
      if (set) then
        textfile=trim(arg)
      else
        textfile=trim(hdffile)//'.dat'
        hdffile=trim(hdffile)//'.hdf'
      end if
c
      return
      end
