      program KPSYN
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a Kitt Peak synoptic magnetic field file to
c ****** a new mesh.
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*(*),cdate*(*)
c
      parameter (cname='KPSYN')
      parameter (cvers='1.04')
      parameter (cdate='09/04/2003')
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        09/13/96, ZM, Version 1.00:
c
c         - Original version of program.
c
c        11/15/96, ZM, Version 1.01:
c
c         - Added the ability to fit or specify the polar fields.
c         - Added the longitudinal filter.
c
c        03/19/1999, ZM, Version 1.02:
c
c         - Major clean-up of the code.
c         - Improved the polar fitting of higher-m modes.
c         - Added a command-line interface.
c         - Added more meaningful diagnostics.
c
c        05/06/2003, ZM, Version 1.03:
c
c         - Small fix to the accuracy of the latitude
c           specification of the poles.
c         - Fixed routine HEADING to use standard F90.
c
c        09/04/2003, ZM, Version 1.04:
c
c         - Changed the behavior of the code when MMAX=0 is
c           specified.  Previously, when MMAX=0 was set (i.e.,
c           no longitudinal filtering), the code would not
c           ask for a file name to output the rms of Br after
c           the filter.  Now it does so anyway, even though this
c           is identical to the rms before the filter.  This was
c           done to make it easier to use KPSYN in a script.
c
c-----------------------------------------------------------------------
c
      cliche syntax
c
c-----------------------------------------------------------------------
c
c ****** Group definitions.
c
c        GROUP 1: <kw>
c        GROUP 2: <arg>
c        GROUP 3: <kw> <arg>
c        GROUP 4: <kw> <arg> <arg>
c
      parameter (ngroup=4)
c
      parameter (IGRPK  =1)
      parameter (IGRPA  =2)
      parameter (IGRPKA =3)
      parameter (IGRPKAA=4)
c
      parameter (mxarg=100)
c
      parameter (IXGRP=1)
      parameter (IXSET=2)
      parameter (IXREQ=3)
      parameter (IXKEY=4)
      parameter (IXARG=5)
      parameter (IXNAM=6)
c
      integer narg,info(6,mxarg)
      common /sntx1/ narg,info
c
c ****** Buffer for the usage line.
c
      character*1024 ulbuf
      common /sntx2/ ulbuf
c
c ****** Buffer for the error message.
c
      character*72 errmsg
      common /sntx3/ errmsg
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche globals
c
c-----------------------------------------------------------------------
c
c ****** Kitt Peak data mesh.
c
      common /gkp/ ntkp,npkp
c
c ****** Interpolated data mesh.
c
      common /gdm/ nt,np,npd2
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche kittpeak
c
c-----------------------------------------------------------------------
c
c ****** Kitt Peak data arrays.
c
      real thetakp(ntkp),phikp(npkp)
      real thetahkp(ntkp+1),phihkp(npkp+1)
      real brkp(ntkp,npkp)
      pointer (ipthetakp,thetakp),(ipphikp,phikp)
      pointer (ipthetahkp,thetahkp),(ipphihkp,phihkp)
      pointer (ipbrkp,brkp)
c
c ****** Pointer to the Kitt Peak synoptic file.
c
      real*4 bsyn(npkp,ntkp)
      pointer (ipbsyn,bsyn)
c
      common /kpcom1/ ipthetakp,ipphikp,ipthetahkp,ipphihkp,
     &                ipbrkp,ipbsyn
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche intdata
c
c-----------------------------------------------------------------------
c
c ****** Interpolated data arrays.
c
      real theta(nt),phi(np)
      real br(nt,np)
      pointer (iptheta,theta),(ipphi,phi)
      pointer (ipbr,br)
c
      common /intcom1/ iptheta,ipphi,ipbr
c
c ****** Arrays to hold the Fourier transform of Br.
c
      real brs(nt,npd2),brc(nt,npd2)
      real fm(npd2)
      real brms(npd2)
      pointer (ipbrs,brs),(ipbrc,brc)
      pointer (ipfm,fm)
      pointer (ipbrms,brms)
c
c ****** Temporary arrays.
c
      real btmp1(ntkp,np-1)
      real btmp2(nt-1,np-1)
      pointer (ipbtmp1,btmp1)
      pointer (ipbtmp2,btmp2)
c
      common /intcom2/ ipbrs,ipbrc,ipfm,ipbrms,ipbtmp1,ipbtmp2
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche fitting
c
c-----------------------------------------------------------------------
c
c ****** Arrays for polar fitting.
c
c ****** Utility arrays for least-squares fitting.
c
      real thfit(nfitmax)
      real brsfit(nfitmax)
      real brcfit(nfitmax)
      pointer (ipthfit,thfit)
      pointer (ipbrsfit,brsfit)
      pointer (ipbrcfit,brcfit)
c
c ****** Polar fit coefficients.
c
      real abc_cos_n(3,mxmodes)
      real abc_sin_n(3,mxmodes)
      real abc_cos_s(3,mxmodes)
      real abc_sin_s(3,mxmodes)
      pointer (ipabc_cos_n,abc_cos_n)
      pointer (ipabc_sin_n,abc_sin_n)
      pointer (ipabc_cos_s,abc_cos_s)
      pointer (ipabc_sin_s,abc_sin_s)
c
c ****** Temporary arrays to hold the Fourier transform.
c
      real brs(ntmax)
      real brc(ntmax)
      pointer (ipbrs,brs)
      pointer (ipbrc,brc)
c
      common /fitcom1/ nfitmax,mxmodes,ntmax
c
      common /fitcom2/ ipthfit,ipbrsfit,ipbrcfit,
     &                 ipabc_cos_n,ipabc_sin_n,
     &                 ipabc_cos_s,ipabc_sin_s,
     &                 ipbrs,ipbrc
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
c ****** Command line parameters.
c
      character*256 kpfile
      common /parcom1/ kpfile
c
      logical verbose,polefit
      common /parcom2/ verbose,polefit
      common /parcom3/ ntin,npin
c
c-----------------------------------------------------------------------
c
c ****** Output theta mesh type.
c
      logical wsomsh
c
c-----------------------------------------------------------------------
c
      character*1 reply
c
c-----------------------------------------------------------------------
c
c ****** Get the command line parameters.
c
      call setpar (cname,cvers,cdate)
c
      write (*,*)
      write (*,*) '### ',cname,', version ',cvers,', ',cdate,' ###'
c
c ****** Read the Kitt Peak synoptic map.
c
      call readkp (kpfile)
c
c ****** Allocate memory for the Kitt Peak arrays.
c
      call alloc_kpmem
c
c ****** Set up the the Kitt Peak data meshes.
c
      call set_kpmesh
c
c ****** Get the interpolated data mesh.
c
      call heading ('SELECT OUTPUT MESH')
c
      nt=ntin
      np=npin
c
c ****** Check that NT and NP are valid.
c
      if (nt.le.6.or.nt.gt.10000.or.np.le.4.or.np.gt.10000) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### The number of points specified '//
     &              'for the interpolation'
        write (*,*) '### data mesh is unphysical.'
        write (*,*) 'Number of latitude points  = ',nt
        write (*,*) 'Number of longitude points = ',np
        call exit (1)
      end if
c
c ****** Check that NP is odd.
c
      npb2=np/2
      if (npb2*2.eq.np) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### The number of longitude points specified '//
     &              'for the interpolation'
        write (*,*) '### data mesh must be odd.'
        write (*,*) 'Number of longitude points = ',np
        call exit (1)
      end if
c
c ****** Calculate the number of Fourier modes in longitude.
c
      npd2=(np-1)/2+1
c
      write (*,*)
      write (*,*) 'Specify the type of output latitude mesh:'
      write (*,*)
      write (*,*) 'y ==> use the original "Wilcox data" latitude mesh'
      write (*,*) 'n ==> use a uniform latitude mesh'
      write (*,*)
      write (*,100) 'Use Wilcox latitude mesh? [y|n]: '
  100 format (1x,a,$)
      read (*,'(a)') reply
      if (reply.eq.'y'.or.reply.eq.'Y') then
        wsomsh=.true.
      else
        wsomsh=.false.
      end if
c
c ****** Allocate memory for the interpolated arrays.
c
      call alloc_intmem
c
c ****** Set up the the interpolated data meshes.
c
      call set_intmesh (wsomsh)
c
c ****** Begin processing.
c
      call exec (cname,cvers,cdate)
c
      call exit (0)
      end
c#######################################################################
      subroutine exec (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a 2D magnetic field from a Kitt Peak synoptic 
c ****** file.
c
c-----------------------------------------------------------------------
c
      use globals
      use kittpeak
      use intdata
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Command line parameters.
c
      character*256 kpfile
      common /parcom1/ kpfile
c
      logical verbose,polefit
      common /parcom2/ verbose,polefit
      common /parcom3/ ntin,npin
c
c-----------------------------------------------------------------------
c
c ****** HDF array dimensions and scales.
c
      parameter (mxdim=3)
      integer idim(mxdim),ips(mxdim)
c
c-----------------------------------------------------------------------
c
c ****** Polar fitting.
c
      logical modpole
      logical two_term
c
c-----------------------------------------------------------------------
c
      character*256 fname
      character*1 reply
c
c-----------------------------------------------------------------------
c
c ****** Initialize the BRKP with the Kitt Peak synoptic data.
c ****** Note that this transposes the array and flips the image
c ****** in theta.
c
      do i=1,ntkp
        do j=1,npkp
          brkp(ntkp-i+1,j)=bsyn(j,i)
        enddo
      enddo
c
c ****** Write the (raw) Kitt Peak magnetic field to a file.
c
      call heading ('OUTPUT RAW KITT PEAK DATA')
c
      write (*,*)
      write (*,*) '### Kitt Peak (raw) Br ###'
      write (*,100) 'Enter HDF output file name [CR to skip]: '
  100 format (1x,a,$)
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        idim(1)=ntkp
        idim(2)=npkp
        ips(1)=loc(thetakp)
        ips(2)=loc(phikp)
c
        write (*,*)
        write (*,*) 'Writing Kitt Peak (raw) Br to file: ',
     &              fname(1:lf)
c
        call wrsds ('hdf',fname,2,idim,brkp,1,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not write Kitt Peak (raw) Br.'
          write (*,*) 'IERR (from WRSDS) = ',ierr
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
c ****** Do a fit at the poles, if requested.
c
      modpole=.false.
c
      call heading ('POLAR FIELD FITTING')
c
      write (*,*)
      write (*,100) 'Would you like to fit the polar field? [y|n]: '
      read (*,'(a)') reply
c
      if (reply.eq.'y'.or.reply.eq.'Y') then
c
        modpole=.true.
c
c ****** Get the fit parameters.
c
        write (*,*)
        write (*,*) 'The data in the theta range [TH0,TH1] '//
     &              'is used to fit the lowest-m'
        write (*,*) 'components of Br with a two-term or '//
     &              'three-term geometric Taylor'
        write (*,*) 'expansion near the poles.'
        write (*,*)
        write (*,*) 'Br is reset to the fitted '//
     &              'field for theta in the range [0,THR] '
        write (*,*) 'for each pole.'
c
        write (*,*)
        write (*,*) '### Fitting region ###'
        write (*,100) 'NORTH POLE: Enter TH0, TH1 [radians]: '
        read (*,*) th0_n,th1_n
c
        write (*,*)
        write (*,*) '### Fitting region ###'
        write (*,100) 'SOUTH POLE: Enter TH0, TH1 [radians]: '
        read (*,*) th0_s,th1_s
c
        write (*,*)
        write (*,*) '### Region to reset ###'
        write (*,100) 'NORTH POLE: Enter THR [radians]: '
        read (*,*) thr_n
c
        write (*,*)
        write (*,*) '### Region to reset ###'
        write (*,100) 'SOUTH POLE: Enter THR [radians]: '
        read (*,*) thr_s
c
        write (*,*)
        write (*,*) '### Type of fit ###'
        write (*,*)
        write (*,*) 'y ==> use a two-term Taylor series fit'
        write (*,*) 'n ==> use a three-term Taylor series fit'
        write (*,*)
        write (*,100) 'Use a two-term fit? [y|n]: '
        read (*,'(a)') reply
        if (reply.eq.'y'.or.reply.eq.'Y') then
          two_term=.true.
        else
          two_term=.false.
        end if
c
        write (*,*)
        write (*,*) '### Maximum m-mode ###'
        write (*,100) 'Maximum m-mode to fit: '
        read (*,*) mxm
c
        write (*,*)
        write (*,*) '### Fitting the polar fields ...'
c
        call alloc_fitpoles (ntkp,thetakp,mxm,
     &                       th0_n,th1_n,th0_s,th1_s)
c
        call fitpoles (ntkp,npkp,thetakp,brkp,two_term,mxm,
     &                 th0_n,th1_n,thr_n,th0_s,th1_s,thr_s,verbose)
c
      end if
c
c ****** Subtract the monopole component.
c
      call balance_flux_h (ntkp,npkp,thetahkp,brkp,bmono)
c
      write (*,*)
      write (*,*) '### From Kitt Peak synoptic data:'
      write (*,*)
      write (*,*) 'Br (monopole) = ',bmono
c
      call heading ('DIAGNOSTICS AFTER POLE FITTING')
c
c ****** Write the corrected Br.
c
      write (*,*)
      write (*,*) '### Kitt Peak (corrected) Br ###'
      write (*,100) 'Enter HDF output file name [CR to skip]: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        idim(1)=ntkp
        idim(2)=npkp
        ips(1)=loc(thetakp)
        ips(2)=loc(phikp)
c
        write (*,*)
        write (*,*) 'Writing Kitt Peak (corrected) Br to file: ',
     &              fname(1:lf)
c
        call wrsds ('hdf',fname,2,idim,brkp,1,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not write Kitt Peak (corrected) Br.'
          write (*,*) 'IERR (from WRSDS) = ',ierr
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
      call heading ('INTERPOLATION')
c
c ****** Interpolate the flux from the Kitt Peak mesh.
c
      write (*,*)
      write (*,*) '### Interpolating Br ...'
c
c ****** First, re-bin over phi.
c
      do i=1,ntkp
        do j=1,np-1
          phi0=phi(j)
          phi1=phi(j+1)
          call integh (ntkp,npkp,phihkp,brkp(i,1),
     &                 phi0,phi1,.false.,val)
          btmp1(i,j)=val
        enddo
      enddo
c
c ****** Next, re-bin over theta.
c
      do j=1,np-1
        do i=1,nt-1
          theta0=theta(i)
          theta1=theta(i+1)
          call integh (1,ntkp,thetahkp,btmp1(1,j),
     &                 theta0,theta1,.true.,val)
          btmp2(i,j)=val
        enddo
      enddo
c
c ****** Interpolate the flux to the final mesh.
c
      do j=2,np-1
        do i=2,nt-1
          br(i,j)=.25*( btmp2(i  ,j-1)+btmp2(i  ,j  )
     &                 +btmp2(i-1,j-1)+btmp2(i-1,j  ))
        enddo
      enddo
c
c ****** Enforce periodicity.
c
      do i=2,nt-1
        br(i, 1)=.25*( btmp2(i  , 1)+btmp2(i  ,np-1)
     &                +btmp2(i-1, 1)+btmp2(i-1,np-1))
        br(i,np)=br(i, 1)
      enddo
c
c ****** Divide by the cell area to get Br.
c
      do j=1,np-1
        dp=(phi(j+1)-phi(j))
        do i=2,nt-1
          dt=.5*(theta(i+1)-theta(i-1))
          area=sin(theta(i))*dt*dp
          br(i,j)=br(i,j)/area
        enddo
      enddo
c
      do i=2,nt-1
        br(i,np)=br(i,1)
      enddo
c
c ****** Set the polar values.
c
      flux0=0.
      flux1=0.
      do j=1,np-1
        flux0=flux0+btmp2(   1,j)
        flux1=flux1+btmp2(nt-1,j)
      enddo
c
      dt=(theta(2)-theta(1))
      area0=2.*pi*sin(.5*dt)*dt
      area1=area0
c
      do j=1,np
        br( 1,j)=flux0/area0
        br(nt,j)=flux1/area1
      enddo
c
c ****** Subtract the monopole component.
c
      call balance_flux (nt,np,theta,br,bmono)
c
      write (*,*)
      write (*,*) '### After interpolation:'
      write (*,*)
      write (*,*) 'Br (monopole) = ',bmono
c
c ****** Decompose Br into its Fourier modes.
c
      write (*,*)
      write (*,*) '### Computing the Fourier transform of Br ...'
c
      call get_ft (nt,np,npd2,br,brs,brc)
c
c ****** Get the rms mode amplitudes of Br.
c
      call get_brms (nt,np,npd2,theta,brs,brc,brms,b_rms_g)
c
      write (*,*) 
      write (*,*) '### RMS value of Br (before filter) = ',b_rms_g
c
c ****** Output the rms mode amplitudes of Br.
c
      write (*,*)
      write (*,*) '### Br_rms vs. m (before filter) ###'
      write (*,100) 'Enter text output file name [CR to skip]: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        write (*,*)
        write (*,*) 'Writing Br_rms to file: ',fname(1:lf)
c
        call ffopen (1,fname,'rw')
c
        write (1,*) 'm',char(9),'Br_rms'
        do k=1,npd2
          m=k-1
          write (1,*) m,char(9),brms(k)
        enddo
c
        close (1)
c
      end if
c
c ****** Output the Fourier mode amplitudes of Br.
c
      do k=1,npd2
        m=k-1
        fm(k)=m
      enddo
c
      idim(1)=nt
      idim(2)=npd2
c
      ips(1)=loc(theta)
      ips(2)=loc(fm)
c
      write (*,*)
      write (*,*) '### Sine part of Fourier modes of Br ###'
      write (*,100) 'Enter HDF output file name [CR to skip]: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        write (*,*)
        write (*,*) 'Writing Br_sin to file: ',fname(1:lf)
c
        call wrsds ('hdf',fname,2,idim,brs,1,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not write the sine part of Br.'
          write (*,*) 'IERR (from WRSDS) = ',ierr
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
      write (*,*)
      write (*,*) '### Cosine part of Fourier modes of Br ###'
      write (*,100) 'Enter HDF output file name [CR to skip]: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        write (*,*)
        write (*,*) 'Writing Br_cos to file: ',fname(1:lf)
c
        call wrsds ('hdf',fname,2,idim,brc,1,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not write the cosine part of Br.'
          write (*,*) 'IERR (from WRSDS) = ',ierr
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
      call heading ('LONGITUDINAL FILTER')
c
c ****** Get the filter parameters.
c
      write (*,*)
      write (*,*) '### Longitudinal filter parameters:'
      write (*,*)
      write (*,*) 'Filter type: exp[-(0.7*m/MMAX)**8)]'
      write (*,*)
      write (*,*) 'MMAX specifies the mode that is attenuated to'//
     &            ' 95% of its original amplitude.'
      write (*,*) 'Modes with M smaller than MMAX are not'//
     &            ' attenuated appreciably.'
      write (*,*) 'The mode with M = 1.64*MMAX is attenuated'//
     &            ' to 5% of its original amplitude.'
      write (*,*) 'If filtering is not desired, set MMAX=0.'
c
      write (*,*)
      write (*,100) 'Enter MMAX: '
      read (*,*) mmax
c
c ****** Apply the filter if MMAX.gt.0.
c
c ****** Apply the filter.
c
      write (*,*)
      write (*,*) '### Applying the longitudinal filter ...'
c
      call filter (nt,np,npd2,theta,brs,brc,mmax,br)
c
c ****** Get the rms mode amplitudes of Br.
c
      call get_brms (nt,np,npd2,theta,brs,brc,brms,b_rms_g)
c
      write (*,*) 
      write (*,*) '### RMS value of Br (after filter) = ',b_rms_g
c
c ****** Output the rms mode amplitudes of Br.
c
      write (*,*)
      write (*,*) '### Br_rms vs. m (after filter) ###'
      write (*,100) 'Enter text output file name [CR to skip]: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        write (*,*)
        write (*,*) 'Writing Br_rms to file: ',fname(1:lf)
c
        call ffopen (1,fname,'rw')
c
        write (1,*) 'm',char(9),'Br_rms'
        do k=1,npd2
          m=k-1
          write (1,*) m,char(9),brms(k)
        enddo
c
        close (1)
c
      end if
c
      call heading ('FINAL OUTPUT')
c
c ****** Write the final interpolated Br.
c
      write (*,*)
      write (*,100) 'File name for interpolated Br: '
      read (*,'(a)') fname
c
      if (fname.ne.' ') then
c
        lf=lenstr(fname)
c
        idim(1)=nt
        idim(2)=np
        ips(1)=loc(theta)
        ips(2)=loc(phi)
c
        write (*,*)
        write (*,*) 'Writing interpolated Br to file: ',fname(1:lf)
c
        call wrsds ('hdf',fname,2,idim,br,1,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not write interpolated Br.'
          write (*,*) 'IERR (from WRSDS) = ',ierr
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
      call heading ('DONE')
c
      write (*,*)
c
      call exit (0)
c
      end
c#######################################################################
      subroutine readkp (fname)
c
c-----------------------------------------------------------------------
c
c ****** Read the Kitt Peak synoptic magnetic field from file FNAME.
c
c-----------------------------------------------------------------------
c
      use globals
      use kittpeak
c
c-----------------------------------------------------------------------
c
      character*(*) fname
c
c-----------------------------------------------------------------------
c
c ****** HDF array dimensions and scales.
c
      parameter (mxdim=2)
      integer idim(mxdim),ips(mxdim)
c
c-----------------------------------------------------------------------
c
c ****** Read the Kitt Peak synoptic map.
c
      lf=lenstr(fname)
c
      write (*,*)
      write (*,*) 'Reading file: ',fname(1:lf)
c
      call rdsds ('hdf',fname,mxdim,ndim,idim,ipbsyn,ifscale,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in READKP:'
        write (*,*) '### Could not read the Kitt Peak synoptic file.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
      if (ndim.ne.2) then
        write (*,*)
        write (*,*) '### ERROR in READKP:'
        write (*,*) '### The file does not have a 2D SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
      npkp=idim(1)
      ntkp=idim(2)
c
      write (*,*)
      write (*,*) '### Kitt Peak synoptic file dimensions:'
      write (*,*)
      write (*,*) 'Number of points in longitude = ',npkp
      write (*,*) 'Number of points in latitude  = ',ntkp
c
      return
      end
c#######################################################################
      subroutine alloc_kpmem
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for the Kitt Peak data arrays.
c
c-----------------------------------------------------------------------
c
      use globals
      use kittpeak
c
c-----------------------------------------------------------------------
c
      ipthetakp=igetmemory(ntkp)
      ipphikp=igetmemory(npkp)
      ipthetahkp=igetmemory(ntkp+1)
      ipphihkp=igetmemory(npkp+1)
      ipbrkp=igetmemory(ntkp*npkp)
c
      return
      end
c#######################################################################
      subroutine set_kpmesh
c
c-----------------------------------------------------------------------
c
c ****** Set up the Kitt Peak data meshes.
c
c-----------------------------------------------------------------------
c
      use globals
      use kittpeak
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dst=2./ntkp
c
      do i=1,ntkp+1
        slat=1.-(i-1.)*dst
        thetahkp(i)=.5*pi-asin(slat)
      enddo
c
      do i=1,ntkp
        thetakp(i)=.5*(thetahkp(i)+thetahkp(i+1))
      enddo
c
      dpkp=2.*pi/npkp
c
      do j=1,npkp+1
        phihkp(j)=(j-1.)*dpkp
      enddo
c
      do j=1,npkp
        phikp(j)=.5*(phihkp(j)+phihkp(j+1))
      enddo
c
      return
      end
c#######################################################################
      subroutine alloc_intmem
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for the interpolated data arrays.
c
c-----------------------------------------------------------------------
c
      use globals
      use intdata
c
c-----------------------------------------------------------------------
c
      iptheta=igetmemory(nt)
      ipphi=igetmemory(np)
      ipbr=igetmemory(nt*np)
      ipbrs=igetmemory(nt*npd2)
      ipbrc=igetmemory(nt*npd2)
      ipfm=igetmemory(npd2)
      ipbrms=igetmemory(npd2)
      ipbtmp1=igetmemory(ntkp*(np-1))
      ipbtmp2=igetmemory((nt-1)*(np-1))
c
      return
      end
c#######################################################################
      subroutine set_intmesh (wsomsh)
c
c-----------------------------------------------------------------------
c
c ****** Set up the interpolated data meshes.
c
c-----------------------------------------------------------------------
c
      use globals
      use intdata
c
c-----------------------------------------------------------------------
c
      logical wsomsh
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dpwso=2.*pi/(np-1.)
c
      do j=1,np
        phi(j)=(j-1.)*dpwso
      enddo
c
      if (wsomsh) then
c
        ntwso=nt-6
c
        dst=2./ntwso
        do i=1,ntwso
          slat=1.-(i-.5)*dst
          theta(i+3)=.5*pi-asin(slat)
        enddo
c
c ****** Add theta points near the poles.
c
        dt0=theta(4)/3.
c
        theta(1)=0.
        theta(2)=dt0
        theta(3)=2.*dt0
        theta(ntwso+4)=pi-2.*dt0
        theta(ntwso+5)=pi-dt0
        theta(ntwso+6)=pi
c
      else
c
        dtwso=pi/(nt-1.)
        do i=1,nt
          theta(i)=(i-1.)*dtwso
        enddo
        theta(nt)=min(pi,theta(nt))
c
      end if
c
      return
      end
c#######################################################################
      subroutine balance_flux_h (nt,np,theta,br,bmono)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the total flux, and subtract off the monopole
c ****** component.
c
c ****** This version has THETA defined on a mesh that is staggered
c ****** with respect to the theta mesh of BR.
c
c-----------------------------------------------------------------------
c
      real theta(nt+1)
      real br(nt,np)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the total flux.
c
      dp=2.*pi/(np-1.)
c
      area=0.
      flux=0.
      do i=1,nt
        th=.5*(theta(i)+theta(i+1))
        dt=(theta(i+1)-theta(i))
        da=sin(th)*dt*dp
        do j=1,np
          flux=flux+br(i,j)*da
          area=area+da
        enddo
      enddo
c
c ****** Subtract the monopole component.
c
      bmono=flux/area
c
      do i=1,nt
        do j=1,np
          br(i,j)=br(i,j)-bmono
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine balance_flux (nt,np,theta,br,bmono)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the total flux, and subtract off the monopole
c ****** component.
c
c ****** This version has THETA defined on the same mesh as that
c ****** of BR.
c
c-----------------------------------------------------------------------
c
      real theta(nt)
      real br(nt,np)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the total flux.
c
      dp=2.*pi/(np-1.)
c
      area=0.
      flux=0.
      do i=1,nt
        if (i.eq.1) then
          dt=theta(2)-theta(1)
          da=.125*dt**2*dp
        else if (i.eq.nt) then
          dt=theta(nt)-theta(nt-1)
          da=.125*dt**2*dp
        else
          dt=.5*(theta(i+1)-theta(i-1))
          da=sin(theta(i))*dt*dp
        end if
        do j=1,np
          flux=flux+br(i,j)*da
          area=area+da
        enddo
      enddo
c
c ****** Subtract the monopole component.
c
      bmono=flux/area
c
      do i=1,nt
        do j=1,np
          br(i,j)=br(i,j)-bmono
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine get_ft (nt,np,npd2,br,brs,brc)
c
c-----------------------------------------------------------------------
c
c ****** Get the Fourier transform of the magnetic field.
c
c ****** The sin and cosine parts of the Fourier transform are
c ****** returned in the arrays BRS and BRC.
c
c-----------------------------------------------------------------------
c
      real br(nt,np),brs(nt,npd2),brc(nt,npd2)
c
c-----------------------------------------------------------------------
c
c ****** Decompose Br into modes.
c
      do k=1,npd2
        m=k-1
        call ft (nt,np,br,m,brs(1,k),brc(1,k))
      enddo
c
      return
      end
c#######################################################################
      subroutine get_brms (nt,np,npd2,theta,brs,brc,brms,b_rms_g)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the rms amplitude of Br in each Fourier mode.
c
c-----------------------------------------------------------------------
c
c ****** On input, the sine and cosine Fourier mode amplitudes
c ****** of Br should be in the arrays BRS and BRC, respectively.
c
c ****** Upon return, the rms amplitudes of Br for each mode
c ****** are returned in the array BRMS.
c
c ****** The rms is area-weighted so that the sum of all of the
c ****** squares of the rms amplitudes over all the modes gives
c ****** the overall area-weighted rms of the magnetic field.
c
c ****** This overall (global) rms value of Br is returned
c ****** in B_RMS_G.
c
c-----------------------------------------------------------------------
c
      real theta(nt)
      real brs(nt,npd2),brc(nt,npd2)
      real brms(npd2)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the rms amplitudes of Br in each mode.
c
      do k=1,npd2
        m=k-1
        if (m.eq.0) then
          fcc=2.
        else
          fcc=1.
        end if
        brms(k)=0.
        area=0.
        do i=1,nt
          if (i.eq.1) then
            dt=theta(2)-theta(1)
            da=.125*dt**2
          else if (i.eq.nt) then
            dt=theta(nt)-theta(nt-1)
            da=.125*dt**2
          else
            dt=.5*(theta(i+1)-theta(i-1))
            da=sin(theta(i))*dt
          end if
          brms(k)=brms(k)+(brs(i,k)**2+brc(i,k)**2)*da
          area=area+da
        enddo
        brms(k)=sqrt(.5*fcc*brms(k)/area)
      enddo
c
c ****** Calculate the global rms value of Br.
c
      b_rms_g=0.
      do k=1,npd2
        b_rms_g=b_rms_g+brms(k)**2
      enddo
      b_rms_g=sqrt(b_rms_g)
c
      return
      end
c#######################################################################
      subroutine alloc_fitpoles (nt,theta,mxm,
     &                           th0_n,th1_n,th0_s,th1_s)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for the polar field fitting arrays.
c
c-----------------------------------------------------------------------
c
      use fitting
c
c-----------------------------------------------------------------------
c
      real theta(nt)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for the fitting arrays.
c
c ****** Determine how many points will be used
c ****** for the fit.
c
      nfit_n=0
      do i=1,nt
        th=theta(i)
        if (th.ge.th0_n.and.th.le.th1_n) then
          nfit_n=nfit_n+1
        end if
      enddo
c
      nfit_s=0
      do i=1,nt
        th=pi-theta(i)
        if (th.ge.th0_s.and.th.le.th1_s) then
          nfit_s=nfit_s+1
        end if
      enddo
c
      nfitmax=max(nfit_n,nfit_s)
c
      nwd=nfitmax
      ipthfit=igetmem(nwd)
      if (ipthfit.eq.0) go to 900
      ipbrsfit=igetmem(nwd)
      if (ipbrsfit.eq.0) go to 900
      ipbrcfit=igetmem(nwd)
      if (ipbrcfit.eq.0) go to 900
c
c ****** Allocate memory for the fit coefficients.
c
      mxmodes=mxm+1
c
      nwd=3*mxmodes
      ipabc_cos_n=igetmem(nwd)
      if (ipabc_cos_n.eq.0) go to 900
      ipabc_sin_n=igetmem(nwd)
      if (ipabc_sin_n.eq.0) go to 900
      ipabc_cos_s=igetmem(nwd)
      if (ipabc_cos_s.eq.0) go to 900
      ipabc_sin_s=igetmem(nwd)
      if (ipabc_sin_s.eq.0) go to 900
c
c ****** Allocate memory for the Fourier transform arrays.
c
      ntmax=nt
c
      nwd=ntmax
      ipbrs=igetmem(nwd)
      if (ipbrs.eq.0) go to 900
      ipbrc=igetmem(nwd)
      if (ipbrc.eq.0) go to 900
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in ALLOC_FITPOLES:'
      write (*,*) '### Could not allocate the requested memory.'
      write (*,*) 'Number of words requested = ',nwd
      call exit (1)
c
      end
c#######################################################################
      subroutine fitpoles (nt,np,theta,br,two_term,mxm,
     &                     th0_n,th1_n,thr_n,th0_s,th1_s,thr_s,
     &                     verbose)
c
c-----------------------------------------------------------------------
c
c ****** Fit the polar fields.
c
c-----------------------------------------------------------------------
c
      use fitting
c
c-----------------------------------------------------------------------
c
      real theta(nt)
      real br(nt,np)
c
      logical two_term
      logical verbose
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dp=2.*pi/(np-1.)
c
c ****** Loop through the modes to fit.
c
      do m=0,mxm
c
        k=m+1
c
c ****** Get the Fourier transform of the data.
c
        call ft (nt,np,br,m,brs,brc)
c
c-----------------------------------------------------------------------
c ****** Do the fit at the North pole.
c-----------------------------------------------------------------------
c
c ****** Collect the data points for the fit.
c
        nfit=0
        do i=1,nt
          th=theta(i)
          if (th.ge.th0_n.and.th.le.th1_n) then
            nfit=nfit+1
            thfit(nfit)=th
            brsfit(nfit)=brs(i)
            brcfit(nfit)=brc(i)
          end if
        enddo
c
c ****** Do the least-squares fit.
c
c ****** Cosine component.
c
        call lsqfit (nfit,thfit,brcfit,m,a,b,c,
     &               two_term,emax,erms,ierr)
        if (ierr.ne.0) go to 920
c
        abc_cos_n(1,k)=a
        abc_cos_n(2,k)=b
        abc_cos_n(3,k)=c
c
        if (verbose) then
          write (*,*)
          write (*,*) '### North pole fit, cosine component:'
          write (*,*) 'm = ',m
          write (*,*) 'Number of points in fit = ',nfit
          write (*,*) 'Error in fit: max = ',emax,' rms = ',erms
          write (*,*) 'A = ',a,' B = ',b,' C = ',c
        end if
c
c ****** Sine component.
c
        if (m.eq.0) then
          abc_sin_n(1,k)=0.
          abc_sin_n(2,k)=0.
          abc_sin_n(3,k)=0.
        else
          call lsqfit (nfit,thfit,brsfit,m,a,b,c,
     &                 two_term,emax,erms,ierr)
          if (ierr.ne.0) go to 930
c
          abc_sin_n(1,k)=a
          abc_sin_n(2,k)=b
          abc_sin_n(3,k)=c
c
          if (verbose) then
            write (*,*)
            write (*,*) '### North pole fit, sine component:'
            write (*,*) 'm = ',m
            write (*,*) 'Number of points in fit = ',nfit
            write (*,*) 'Error in fit: max = ',emax,' rms = ',erms
            write (*,*) 'A = ',a,' B = ',b,' C = ',c
          end if
        end if
c
c-----------------------------------------------------------------------
c ****** Do the fit at the South pole.
c-----------------------------------------------------------------------
c
        nfit=0
        do i=1,nt
          th=pi-theta(i)
          if (th.ge.th0_s.and.th.le.th1_s) then
            nfit=nfit+1
            thfit(nfit)=th
            brsfit(nfit)=brs(i)
            brcfit(nfit)=brc(i)
          end if
        enddo
c
c ****** Do the least-squares fit.
c
c ****** Cosine component.
c
        call lsqfit (nfit,thfit,brcfit,m,a,b,c,
     &               two_term,emax,erms,ierr)
        if (ierr.ne.0) go to 940
c
        abc_cos_s(1,k)=a
        abc_cos_s(2,k)=b
        abc_cos_s(3,k)=c
c
        if (verbose) then
          write (*,*)
          write (*,*) '### South pole fit, cosine component:'
          write (*,*) 'm = ',m
          write (*,*) 'Number of points in fit = ',nfit
          write (*,*) 'Error in fit: max = ',emax,' rms = ',erms
          write (*,*) 'A = ',a,' B = ',b,' C = ',c
        end if
c
c ****** Sine component.
c
        if (m.eq.0) then
          abc_sin_s(1,k)=0.
          abc_sin_s(2,k)=0.
          abc_sin_s(3,k)=0.
        else
          call lsqfit (nfit,thfit,brsfit,m,a,b,c,
     &                 two_term,emax,erms,ierr)
          if (ierr.ne.0) go to 950
c
          abc_sin_s(1,k)=0.
          abc_sin_s(2,k)=0.
          abc_sin_s(3,k)=0.
c
          if (verbose) then
            write (*,*)
            write (*,*) '### South pole fit, sine component:'
            write (*,*) 'm = ',m
            write (*,*) 'Number of points in fit = ',nfit
            write (*,*) 'Error in fit: max = ',emax,' rms = ',erms
            write (*,*) 'A = ',a,' B = ',b,' C = ',c
          end if
        end if
c
      enddo
c
c-----------------------------------------------------------------------
c ****** Reset the field to the fitted field near the North pole.
c-----------------------------------------------------------------------
c
      do i=1,nt
        th=theta(i)
        if (th.le.thr_n) then
          do j=1,np
            phi=(j-1.)*dp
            br(i,j)=0.
            do m=0,mxm
              k=m+1
              brc0=( abc_cos_n(1,k)
     &              +abc_cos_n(2,k)*th**2
     &              +abc_cos_n(3,k)*th**4)*th**m
              brs0=( abc_sin_n(1,k)
     &              +abc_sin_n(2,k)*th**2
     &              +abc_sin_n(3,k)*th**4)*th**m
              br(i,j)=br(i,j)+brc0*cos(m*phi)+brs0*sin(m*phi)
            enddo
          enddo
        end if
      enddo
c
c-----------------------------------------------------------------------
c ****** Reset the field to the fitted field near the South pole.
c-----------------------------------------------------------------------
c
      do i=1,nt
        th=pi-theta(i)
        if (th.le.thr_s) then
          do j=1,np
            phi=(j-1.)*dp
            br(i,j)=0.
            do m=0,mxm
              k=m+1
              brc0=( abc_cos_s(1,k)
     &              +abc_cos_s(2,k)*th**2
     &              +abc_cos_s(3,k)*th**4)*th**m
              brs0=( abc_sin_s(1,k)
     &              +abc_sin_s(2,k)*th**2
     &              +abc_sin_s(3,k)*th**4)*th**m
              br(i,j)=br(i,j)+brc0*cos(m*phi)+brs0*sin(m*phi)
            enddo
          enddo
        end if
      enddo
c
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in FITPOLES:'
      write (*,*) '### Least-squares fit at the North pole failed.'
      write (*,*) 'Cosine component, m = ',m
      call exit (1)
c
  930 continue
c
      write (*,*)
      write (*,*) '### ERROR in FITPOLES:'
      write (*,*) '### Least-squares fit at the North pole failed.'
      write (*,*) 'Sine component, m = ',m
      call exit (1)
c
  940 continue
c
      write (*,*)
      write (*,*) '### ERROR in FITPOLES:'
      write (*,*) '### Least-squares fit at the South pole failed.'
      write (*,*) 'Cosine component, m = ',m
      call exit (1)
c
  950 continue
c
      write (*,*)
      write (*,*) '### ERROR in FITPOLES:'
      write (*,*) '### Least-squares fit at the South pole failed.'
      write (*,*) 'Sine component, m = ',m
      call exit (1)
c
      return
      end
c#######################################################################
      subroutine lsqfit (n,x,f,m,a,b,c,two_term,emax,erms,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Perform a least squares fit of the function:
c
c          f = (A + B*x**2 + C*x**4)*x**M
c
c ****** to the data in F(N), X(N), when TWO_TERM=.false..
c
c ****** When TWO_TERM=.true., do a two-term fit (with C=0).
c
c ****** The maximum and rms errors in the fit are returned
c ****** in EMAX and ERMS.
c
c-----------------------------------------------------------------------
c
      logical two_term
c
c-----------------------------------------------------------------------
c
      real x(n),f(n)
c
c-----------------------------------------------------------------------
c
c ****** Normal matrix for least-squares system.
c
      real*8 aa(3,3),ff(3)
c
      real*8 xdp,fdp
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      if (two_term) then
        if (n.le.2) then
          write (*,*)
          write (*,*) '### ERROR in LSQFIT:'
          write (*,*) '### Less than 2 points specified for a '//
     &                'two_term fit.'
          write (*,*) 'Number of points = ',n
          ierr=1
          return
        end if
      else
        if (n.le.3) then
          write (*,*)
          write (*,*) '### ERROR in LSQFIT:'
          write (*,*) '### Less than 3 points specified for a '//
     &                'three-term fit.'
          write (*,*) 'Number of points = ',n
          ierr=1
          return
        end if
      end if
c
      do i=1,3
        ff(i)=0.
        do j=1,3
          aa(i,j)=0.
        enddo
      enddo
c
      do i=1,n
        xdp=x(i)
        aa(1,1)=aa(1,1)+xdp**(2*m)
        aa(1,2)=aa(1,2)+xdp**(2+2*m)
        aa(1,3)=aa(1,3)+xdp**(4+2*m)
        aa(2,2)=aa(2,2)+xdp**(4+2*m)
        aa(2,3)=aa(2,3)+xdp**(6+2*m)
        aa(3,3)=aa(3,3)+xdp**(8+2*m)
        fdp=f(i)
        ff(1)=ff(1)+fdp*xdp**m
        ff(2)=ff(2)+fdp*xdp**(2+m)
        ff(3)=ff(3)+fdp*xdp**(4+m)
      enddo
c
      if (two_term) then
        aa(1,3)=0.
        aa(2,3)=0.
        aa(3,3)=1.0d0
        ff(3)=0.
      end if
c
c ****** The normal matrix is symmetric.
c
      aa(2,1)=aa(1,2)
      aa(3,1)=aa(1,3)
      aa(3,2)=aa(2,3)
c
c ****** Invert the matrix.
c
      call minv (aa)
c
c ****** Get the best-fit parameters.
c
      a=aa(1,1)*ff(1)+aa(1,2)*ff(2)+aa(1,3)*ff(3)
      b=aa(2,1)*ff(1)+aa(2,2)*ff(2)+aa(2,3)*ff(3)
      c=aa(3,1)*ff(1)+aa(3,2)*ff(2)+aa(3,3)*ff(3)
c
c ****** Calculate the error in the fit.
c
      emax=0.
      erms=0.
      do i=1,n
        err=f(i)-(a+b*x(i)**2+c*x(i)**4)*x(i)**m
        emax=max(abs(err),emax)
        erms=erms+err**2
      enddo
      erms=sqrt(erms/float(n))
c
      return
      end
c#######################################################################
      subroutine minv (a)
c
c-----------------------------------------------------------------------
c
c ****** Invert the 3 x 3 matrix A.
c
c ****** Use Gaussian elimination without pivoting.
c
c-----------------------------------------------------------------------
c
      real*8 a(3,3)
c
c-----------------------------------------------------------------------
c
      real*8 det,atemp12
c
c-----------------------------------------------------------------------
c
c ****** Check that the matrix is non-singular.
c
      det= a(1,1)*(a(2,2)*a(3,3)-a(2,3)*a(3,2))
     &    -a(1,2)*(a(2,1)*a(3,3)-a(2,3)*a(3,1))
     &    +a(1,3)*(a(2,1)*a(3,2)-a(2,2)*a(3,1))
c
      if (det.eq.0.) then
        write (*,*)
        write (*,*) '### ERROR in MINV:'
        write (*,*) '### The normal matrix for the least-squares'
        write (*,*) '### solution is singular.'
        call exit (1)
      end if
c
c ****** First, perform the L-U decomposition of the matrix,
c ****** (with the diagonal of the U matrix equal to unity).
c
      a(1,2)=a(1,2)/a(1,1)
      a(1,3)=a(1,3)/a(1,1)
      a(2,2)=a(2,2)-a(2,1)*a(1,2)
      a(3,2)=a(3,2)-a(3,1)*a(1,2)
      a(2,3)=(a(2,3)-a(2,1)*a(1,3))/a(2,2)
      a(3,3)=a(3,3)-a(3,1)*a(1,3)-a(3,2)*a(2,3)
c
c ****** Now, find the columns of the inverse of the L matrix.
c
      a(1,1)=1.0d0/a(1,1)
      a(2,1)=-a(2,1)*a(1,1)/a(2,2)
      a(3,1)=(-a(3,1)*a(1,1)-a(3,2)*a(2,1))/a(3,3)
      a(2,2)=1.0d0/a(2,2)
      a(3,2)=-a(3,2)*a(2,2)/a(3,3)
      a(3,3)=1.0d0/a(3,3)
c
c ****** Finally, find the elements of the inverse matrix.
c
      a(2,2)=a(2,2)-a(2,3)*a(3,2)
      a(2,1)=a(2,1)-a(2,3)*a(3,1)
      a(2,3)=-a(2,3)*a(3,3)
      a(1,1)=a(1,1)-a(1,2)*a(2,1)-a(1,3)*a(3,1)
      atemp12=a(1,2)
      a(1,2)=-a(1,2)*a(2,2)-a(1,3)*a(3,2)
      a(1,3)=-atemp12*a(2,3)-a(1,3)*a(3,3)
c
      return
      end
c#######################################################################
      subroutine filter (nt,np,npd2,theta,brs,brc,mmax,br)
c
c-----------------------------------------------------------------------
c
c ****** Apply a filter along the longitudinal direction.
c
c-----------------------------------------------------------------------
c
c ****** On input, the sine and cosine Fourier mode amplitudes
c ****** of Br should be in the arrays BRS and BRC, respectively.
c
c ****** MMAX controls the amount of filtering applied.
c
c-----------------------------------------------------------------------
c
      real theta(nt)
      real brs(nt,npd2),brc(nt,npd2)
      real br(nt,np)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dp=2.*pi/(np-1.)
c
c ****** Apply the filter to the Fourier coefficients.
c
      do i=1,nt
        do k=1,npd2
c
          m=k-1
c
          if (mmax.gt.0) then
            fmnorm=amax1(4.,mmax*abs(sin(theta(i))))
            filt=exp(-(.7*float(m)/fmnorm)**8)
          else
            filt=1.
          end if
c
          brs(i,k)=filt*brs(i,k)
          brc(i,k)=filt*brc(i,k)
c
        enddo
      enddo
c
c ****** Reconstruct the field from the filtered Fourier
c ****** coefficients.
c
      do j=1,np
        phi=(j-1.)*dp
        do i=1,nt
          br(i,j)=0.
          do k=1,npd2
            m=k-1
            br(i,j)=br(i,j)+brs(i,k)*sin(m*phi)
     &                     +brc(i,k)*cos(m*phi)
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine ft (nt,np,f,m,fs,fc)
c
c-----------------------------------------------------------------------
c
c ****** Take the Fourier transform of the array F for
c ****** all NT points, for Fourier mode M.

c ****** The sin and cosine parts of the Fourier transform are
c ****** returned in arrays FS and FC.
c
c-----------------------------------------------------------------------
c
      real f(nt,np),fs(nt),fc(nt)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      npd2=(np-1)/2+1
c
      dp=2.*pi/(np-1.)
c
      fnorm=1./(np-1.)
c
      if (.not.(m.eq.0.or.m.eq.npd2-1)) then
        fnorm=2.*fnorm
      end if
c
      do i=1,nt
        fs(i)=0.
        fc(i)=0.
        do j=1,np-1
          phi=(j-1.)*dp
          fs(i)=fs(i)+sin(m*phi)*f(i,j)
          fc(i)=fc(i)+cos(m*phi)*f(i,j)
        enddo
        fs(i)=fnorm*fs(i)
        fc(i)=fnorm*fc(i)
      enddo
c
      return
      end
c#######################################################################
      subroutine heading (text)
c
c-----------------------------------------------------------------------
c
c ****** Write a centered heading to the TTY.
c
c-----------------------------------------------------------------------
c
      character(*) :: text
c
c-----------------------------------------------------------------------
c
      nc=len_trim(text)
c
      nl=(76-nc)/2
      nr=76-nc-nl
c
      write (*,*)
      write (*,'(x,a,x,a,x,a)') repeat('#',nl),
     &                          text(1:nc),
     &                          repeat('#',nr)
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do 100 i=len(str),1,-1
        if (str(i:i).ne.' ') go to 200
  100 continue
      i=1
  200 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
      nb=4*nwd
c
c     ip=malloc(%val(nb))
      ip=malloc(nb)
c
      igetmem=ip
c
      return
      end
c#######################################################################
      integer function igetmemory (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** Uses routine IGETMEM to do the actual memory allocation.
c ****** If an error occurs, execution is terminated.
c
c-----------------------------------------------------------------------
c
      igetmemory=igetmem(nwd)
c
      if (igetmemory.eq.0) then 
        write (*,*)
        write (*,*) '### ERROR in IGETMEMORY:'
        write (*,*) '### Could not allocate the requested memory.'
        write (*,*) 'Number of words requested = ',nwd
        call exit (1)
      end if
c
      return
      end
c#######################################################################
      subroutine interp (x,n,xv,i,alpha)
c
c-----------------------------------------------------------------------
c
c ****** Get interpolation factor ALPHA and table index I.
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I and ALPHA to get the interpolated value.
c
c-----------------------------------------------------------------------
c
      dimension x(n)
c
c-----------------------------------------------------------------------
c
      do 100 i=1,n-1
        if (xv.ge.x(i).and.xv.le.x(i+1)) then
          alpha=(xv-x(i))/(x(i+1)-x(i))
          go to 200
        end if
  100 continue
c
c ****** Value not found --- signal error and stop.
c
      write (*,*)
      write (*,*) '### ERROR in INTERP:'
      write (*,*) '### Abscissa not found in table.'
      write (*,*)
      write (*,*) 'Abscissa requested = ',xv
      write (*,*)
      write (*,*) 'Table values:'
      write (*,*) (x(i),i=1,n)
      call exit (1)
c
  200 continue
c
      return
      end
c#######################################################################
      subroutine integh (ldim,n,x,f,x0,x1,wsin,val)
c
c-----------------------------------------------------------------------
c
c ****** Integrate the function defined by the table X(N+1) and F(N)
c ****** with respect to x from X0 to X1.
c
c ****** The function is piecewise constant, with a value given by
c ****** F(i) for x between X(i) and X(i+1).
c
c ****** When WSIN=.true., weight the integrand with sin(x).
c
c-----------------------------------------------------------------------
c
      real x(n+1),f(ldim,n)
      logical wsin
c
c-----------------------------------------------------------------------
c
      call interp (x,n+1,x0,i0,a0)
      call interp (x,n+1,x1,i1,a1)
c
      val=0.
      do i=i0,i1
        if (i.eq.i0) then
          xl=x(i)+a0*(x(i+1)-x(i))
        else
          xl=x(i)
        end if
        if (i.eq.i1) then
          xr=x(i)+a1*(x(i+1)-x(i))
        else
          xr=x(i+1)
        end if
        dx=xr-xl
        if (wsin) then
          wt=sin(.5*(xl+xr))
        else
          wt=1.
        end if
        val=val+f(1,i)*wt*dx
      enddo
c
      return
      end
c#######################################################################
      subroutine setpar (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      character*256 kpfile
      common /parcom1/ kpfile
c
      logical verbose,polefit
      common /parcom2/ verbose,polefit
      common /parcom3/ ntin,npin
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*256 arg
c
c-----------------------------------------------------------------------
c
      logical set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (IGRPKA,'-kpfile',' ','<file>')
      call defarg (IGRPKA,'-fmt','hdf','<format>')
      call defarg (IGRPK ,'-v',' ',' ')
      call defarg (IGRPKA,'-nt','36','<val>')
      call defarg (IGRPKA,'-np','73','<val>')
      call defarg (IGRPK ,'-wsomesh',' ',' ')
      call defarg (IGRPK ,'-polefit',' ',' ')
      call defarg (IGRPKA,'-t0n','.25','<val>')
      call defarg (IGRPKA,'-t1n','.5','<val>')
      call defarg (IGRPKA,'-t0s','.25','<val>')
      call defarg (IGRPKA,'-t1s','.5','<val>')
      call defarg (IGRPKA,'-trn','.35','<val>')
      call defarg (IGRPKA,'-trs','.35','<val>')
      call defarg (IGRPKA,'-mpf','5','<m>')
      call defarg (IGRPK ,'-three_term',' ',' ')
      call defarg (IGRPKA,'-mfilt','0','<m>')
c
c ****** Parse the command line.
c
      call parse (ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Interpolate a Kitt Peak synoptic '//
     &              'magnetic field file.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage.
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
        call wrulin (ulbuf)
        write (*,*)
        write (*,*) 'The file format is specified by -fmt text|hdf '//
     &              '(default=hdf).'
        write (*,*)
        write (*,*) 'INTERPOLATION MESH:'
        write (*,*) '-nt and -np specify the number of mesh points '//
     &              'to use in the interpolation.'
        write (*,*) 'The default is nt=36, np=73 (Wilcox data '//
     &              'resolution).  np must be odd.'
        write (*,*) 'The default is to use a mesh that has uniform '//
     &              'increments in latitude.'
        write (*,*) '-wsomesh indicates that the Wilcox latitude '//
     &              'mesh should be used (uniform'
        write (*,*) 'increments in sine latitude).'
        write (*,*)
        write (*,*) 'POLAR FIELD FIT:'
        write (*,*) 'Use -polefit to specify that polar field '//
     &              'fitting should be performed.'
        write (*,*) 'Fit using data in the latitude range '//
     &              '[t0n,t1n], and reset polar fields in'
        write (*,*) 'the latitude range [0,trn], for the North '//
     &              'pole, and similarly for the'
        write (*,*) 'South pole.  Latitudes are specified in '//
     &              'radians.'
        write (*,*) 'Default for both poles: fit in [.25,.5], '//
     &              'reset in [0,.35].'
        write (*,*) '-mpf sets the maximum Fourier mode m to fit '//
     &              'at the pole (default=5).'
        write (*,*) '-three_term specifies a three-term geometric '//
     &              'Taylor expansion fit.'
        write (*,*) 'Otherwise, a two-term fit is performed '//
     &              '(default).'
        write (*,*)
        write (*,*) 'LONGITUDINAL FILTER:'
        write (*,*) '-mfilt specifies the Fourier mode m at which '//
     &              'longitudinal filtering begins.'
        write (*,*) 'The default, mfilt=0, is to apply no '//
     &              'longitudinal filtering.'
        write (*,*)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Verbose flag.
c
      call fetcharg ('-v',set,arg)
      verbose=set
c
c ****** Kitt Peak synoptic data file.
c
      call fetcharg ('-kpfile',set,arg)
      ls=lenstr(arg)
      kpfile=arg(1:ls)
c
c ****** Interpolation data mesh.
c
      call fetcharg ('-nt',set,arg)
      ntin=intval(arg,'-nt')
c
      call fetcharg ('-np',set,arg)
      npin=intval(arg,'-np')
c
c ****** Pole fitting flag.
c
      call fetcharg ('-polefit',set,arg)
      polefit=set
c
      return
      end
c#######################################################################
      block data bdsntx
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data narg/0/
c
      end
c#######################################################################
      function fpval (avalue,name)
c
c-----------------------------------------------------------------------
c
c ****** Get the value of the floating point number in character
c ****** string AVALUE.
c
c-----------------------------------------------------------------------
c
      character*(*) avalue,name
c
c-----------------------------------------------------------------------
c
      logical iffp
      external iffp
c
c-----------------------------------------------------------------------
c
      ls=lenstr(avalue)
c
      if (.not.iffp(avalue(1:ls),value)) then
        write (*,*)
        write (*,*) '### ERROR in FPVAL:'
        write (*,*) '### Could not interpret a floating point '//
     &              'number while setting: ',name
        write (*,*) 'Invalid format: ',avalue(1:ls)
        call exit (1)
      end if
c
      fpval=value
c
      return
      end
c#######################################################################
      function intval (avalue,name)
c
c-----------------------------------------------------------------------
c
c ****** Get the value of the integer in character string AVALUE.
c
c-----------------------------------------------------------------------
c
      character*(*) avalue,name
c
c-----------------------------------------------------------------------
c
      logical ifint
      external ifint
c
c-----------------------------------------------------------------------
c
      ls=lenstr(avalue)
c
      if (.not.ifint(avalue(1:ls),ivalue)) then
        write (*,*)
        write (*,*) '### ERROR in INTVAL:'
        write (*,*) '### Could not interpret an integer '//
     &              'while setting: ',name
        write (*,*) 'Invalid format: ',avalue(1:ls)
        call exit (1)
      end if
c
      intval=ivalue
c
      return
      end
c#######################################################################
      subroutine wrulin (line)
c
c-----------------------------------------------------------------------
c
c ****** Write the string LINE to the TTY, taking into account
c ****** any carriage returns in the line.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      ls=lenstr(line)
c
      i0=1
  100 continue
c
c ****** Find the next carriage return.
c
      icr=index(line(i0:ls),char(13))
c
      if (icr.gt.0) then
c
c ****** A CR was found.
c
        if (icr.eq.1) then
          write (*,*)
          i0=i0+1
        else
          write (*,*) line(i0:i0+icr-2)
          i0=i0+icr
        end if
        if (i0.le.ls) go to 100
c
      else
c
c ****** A CR was not found.
c
        write (*,*) line(i0:ls)
c
      end if
c
      return
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line.
c
c-----------------------------------------------------------------------
c
c ****** The syntax for the keyword/argument items can be defined
c ****** by using routine DEFARG.
c
c ****** The usage line is constructed in character string ULBUF.
c ****** Use routine WRULIN to write the usage line, since it may
c ****** contain embedded carriage returns.
c
c ****** On return, IERR=0 indicates that the command line was
c ****** parsed successfully.
c
c ****** IERR=1 indicates that no arguments were present.  This
c ****** is usually used to print the usage line.
c
c ****** IERR=2 indicates that a syntax error occured.
c
c ****** IERR=3 indicates that one or more required arguments
c ****** was not supplied.
c
c ****** When IERR=2 or IERR=3, and error message is put into
c ****** character string ERRMSG.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Command line arguments.
c
      integer iargc
      external iargc,getarg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*256 arg
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      ierr=0
      errmsg=' '
c
c ****** Construct the usage line in ULBUF.
c
      ix=1
      ulbuf=' '
c
      call getarg (0,arg)
      ls=lenstr(arg)
      call addtobuf (arg(1:ls),ulbuf,ix)
      nc=ix-1
      ncc=min0(nc,7)
c
      i=1
      itried=0
  100 continue
      if (i.le.narg) then
        ix0=ix
        call addtobuf (' ',ulbuf,ix)
        if (info(IXREQ,i).eq.0) then
          call addtobuf ('[',ulbuf,ix)
        end if
        call getstr (info(IXKEY,i),arg)
        ls=lenstr(arg)
        call addtobuf (arg(1:ls),ulbuf,ix)
        if (info(IXNAM,i).gt.0) then
          call addtobuf (' ',ulbuf,ix)
          call getstr (info(IXNAM,i),arg)
          ls=lenstr(arg)
          call addtobuf (arg(1:ls),ulbuf,ix)
        end if
        if (info(IXREQ,i).eq.0) then
          call addtobuf (']',ulbuf,ix)
        end if
        nc=nc+ix-ix0
        if (nc.gt.78.and.itried.eq.0) then
          itried=1
          ix=ix0
          call addtobuf (char(13),ulbuf,ix)
          ulbuf(ix:ix+ncc-1)=' '
          ix=ix+ncc
          nc=ncc
          go to 100
        end if
        itried=0
        i=i+1
        go to 100
      end if
c
c ****** Get the number of command line arguments.
c
      na=iargc()
      if (na.eq.0) then
        ierr=1
        go to 900
      end if
c
      ia=1
  200 continue
c
      ia0=ia
c
c ****** Process arguments with syntax: <kw> <arg>
c
      if (na-ia+1.ge.2) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw> <arg> <arg>
c
      if (na-ia+1.ge.3) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKAA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            ls=ls+1
            arg(ls:ls)=' '
            ia=ia+1
            call getarg (ia,arg(ls+1:))
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw>
c
      if (na-ia+1.ge.1) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPK,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            info(IXARG,iarg)=0
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <arg>
c
      if (na-ia+1.ge.1) then
        iarg=nxtarg(IGRPA)
        if (iarg.gt.0) then
          call getarg (ia,arg)
          ls=lenstr(arg)
          info(IXARG,iarg)=iputstr(arg(1:ls))
          info(IXSET,iarg)=1
          ia=ia+1
          go to 300
        end if
      end if
c
  300 continue
c
c ****** Check that an argument was found.
c
      if (ia.eq.ia0) then
        ierr=2
        go to 900
      end if
c
c ****** Keep processing arguments until done.
c
      if (na-ia+1.gt.0) go to 200
c
c ****** Check that the required arguments were supplied.
c
      do 400 i=1,narg
        if (info(IXREQ,i).ne.0.and.info(IXSET,i).eq.0) then
          ierr=3
          go to 900
        end if
  400 continue
c
      return
c
c ****** Error exit.
c
  900 continue
c
      if (ierr.eq.2) then
        errmsg='### Syntax error.'
      else if (ierr.eq.3) then
        errmsg='### A required argument was not supplied.'
      end if
c
      return
      end
c#######################################################################
      subroutine defarg (igroup,kw,default,argname)
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax for a command line argument item.
c
c-----------------------------------------------------------------------
c
c ****** IGROUP is the syntax group index.
c ****** KW is the keyword.
c ****** DEFAULT is the default value of the argument.
c ****** ARGNAME is the name of the argument (for use in
c ****** constructing the usage line).
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw,default,argname
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Check the group index.
c
      if (igroup.lt.0.or.igroup.gt.ngroup) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Invalid group index specified.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Check for a null keyword.
c
      if (kw.eq.' ') then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### The keyword is null.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Increment the argument counter.
c
      if (narg.ge.mxarg) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Exceeded number of allowed arguments.'
        write (*,*) 'Maximum number of arguments = ',mxarg
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
      narg=narg+1
c
c ****** Store the group index and keyword.
c
c ****** For group IGRPA (single arguments), the name of the
c ****** argument is stored as the "keyword".
c
      info(IXGRP,narg)=igroup
      ls=lenstr(kw)
      info(IXKEY,narg)=iputstr(kw(1:ls))
c
c ****** Initialize the flag which indicates whether an argument
c ****** has been set.
c
      info(IXSET,narg)=0
c
c ****** If a default argument was supplied, the argument
c ****** does not have to be set.  Use DEFAULT=' ' to
c ****** indicate that an argument is required.
c
c ****** If a default argument has been supplied, store it and
c ****** put its pointer in info(IXARG,narg).  If there is no
c ****** default, set info(IXARG,narg)=0.
c
c ****** Since group IGRPK doesn't have an argument,
c ****** DEFAULT is ignored for this group.
c
      if (igroup.eq.IGRPK) then
        info(IXREQ,narg)=0
        info(IXARG,narg)=0
      else
        if (default.eq.' ') then
          info(IXREQ,narg)=1
          info(IXARG,narg)=0
        else
          info(IXREQ,narg)=0
          ls=lenstr(default)
          info(IXARG,narg)=iputstr(default(1:ls))
        end if
      end if
c
c ****** Store the argument name.  For groups IGRPK (keywords) 
c ****** and IGRPA (single arguments), there is no argument name,
c ****** so ARGNAME is ignored.
c
      if (igroup.eq.IGRPK.or.igroup.eq.IGRPA) then
        info(IXNAM,narg)=0
      else
        ls=lenstr(argname)
        info(IXNAM,narg)=iputstr(argname(1:ls))
      end if
c
      return
      end
c#######################################################################
      subroutine fetcharg (kw,set,arg)
c
c-----------------------------------------------------------------------
c
c ****** Fetch the argument corresponding to keyword KW.
c
c-----------------------------------------------------------------------
c
c ****** If the corresponding argument was set, return SET=.true.,
c ****** and copy the argument to ARG.
c
c ****** If the argument was not set, return SET=.false..
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      logical set
      character*(*) kw,arg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the argument corresponding to keyword KW.
c
      do 100 i=narg,1,-1
        call getstr (info(IXKEY,i),tkw)
        if (tkw.eq.kw) then
          iarg=i
          go to 200
        end if
  100 continue
c
      ls=lenstr(kw)
      write (*,*)
      write (*,*) '### ERROR in FETCHARG:'
      write (*,*) '### Requested keyword could not be matched.'
      write (*,*) 'Keyword = ',kw(1:ls)
      call exit (1)
c
  200 continue
c
      set=info(IXSET,iarg).ne.0
      if (info(IXARG,iarg).gt.0) then
        call getstr (info(IXARG,iarg),arg)
      else
        arg=' '
      end if
c
      return
      end
c#######################################################################
      function matchkw (igroup,kw)
c
c-----------------------------------------------------------------------
c
c ****** Match keyword KW against the list of keywords in
c ****** group IGROUP.
c
c ****** If found, set the function value to the corresponding
c ****** argument number.  Otherwise, set MATCHKW=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
      do 100 i=narg,1,-1
        if (info(IXGRP,i).eq.igroup) then
          call getstr (info(IXKEY,i),tkw)
          if (tkw.eq.kw) then
            matchkw=i
            return
          end if
        end if
  100 continue
c
      matchkw=0
c
      return
      end
c#######################################################################
      function nxtarg (igroup)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the next argument in group IGROUP
c ****** which has not been set.
c
c-----------------------------------------------------------------------
c
c ****** If an empty slot is found, set the function value
c ****** to the corresponding argument number.
c
c ****** Otherwise, set NXTARG=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      do 100 i=1,narg
        if (info(IXGRP,i).eq.igroup) then
          if (info(IXSET,i).eq.0) then
            nxtarg=i
            return
          end if
        end if
  100 continue
c
      nxtarg=0
c
      return
      end
c#######################################################################
      function iputstr (line)
c
c-----------------------------------------------------------------------
c
c ****** Put the string in LINE into internal storage, and return
c ****** a pointer to it as the function value.
c
c ****** The string can be retrieved by calling routine GETSTR
c ****** with the returned pointer.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=4096)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
      ls=len(line)
c
c ****** Check that the string is not empty.
c
      if (ls.le.0) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Trying to store a zero-length string.'
        call exit (1)
      end if
c
c ****** Check that a free pointer exists.
c
      if (nstr.ge.mxstr) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for pointers.'
        write (*,*) 'Maximum number of pointers = ',mxstr
        call exit (1)
      end if
c
      nstr=nstr+1
c
c ****** Check that there is enough room to store the string.
c
      ix1=ix+ls-1
      if (ix1.gt.LENBUF) then
        nb=ix1-LENBUF
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for strings.'
        write (*,*) 'Number of additional bytes needed = ',nb
        call exit (1)
      end if
c
      strbuf(ix:ix1)=line
c
c ****** Set IX to point to the next available character.
c
      ix=ix1+1
      indx(nstr)=ix
c
c ****** Return the pointer to the string as the function value.
c
      iputstr=nstr
c
      return
      end
c#######################################################################
      block data bdstr
c
c-----------------------------------------------------------------------
c
      parameter (mxstr=100)
      common /pstr1/ nstr,ix,indx(mxstr)
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data nstr/0/
      data ix/1/
c
      end
c#######################################################################
      subroutine getstr (ip,line)
c
c-----------------------------------------------------------------------
c
c ****** Get the string associated with pointer IP and put it
c ****** into LINE.
c
c ****** This routine retrieves strings stored with routine IPUTSTR.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=4096)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
c ****** Check that the pointer is valid.
c
      if (ip.lt.0.or.ip.gt.nstr) then
        write (*,*)
        write (*,*) '### ERROR in GETSTR:'
        write (*,*) '### Invalid string pointer.'
        write (*,*) 'Pointer = ',ip
        call exit (1)
      end if
c
c ****** Retrieve the string.
c
      if (ip.eq.1) then
        i0=1
      else
        i0=indx(ip-1)
      end if
      i1=indx(ip)-1
c
      line=strbuf(i0:i1)
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (compress) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          write (*,*)
          write (*,*) '### ERROR in ADDTOBUF:'
          write (*,*) '### Internal buffer overflow.'
          write (*,*) '[This is an internal error.]'
          call exit (1)
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
c#######################################################################
      logical function iffp (alpha,value)
c
c-----------------------------------------------------------------------
c
c ****** Determine if ALPHA represents a floating point number;
c ****** if so, return its value in VALUE.
c
c-----------------------------------------------------------------------
c
c ****** Set IFFP=.TRUE. if ALPHA contains an alphanumeric
c ****** string with the following format:
c
c       ALPHA = '[A][B...B][.][B...B][e[A]B[B...B]]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c ****** The exponent may be denoted by a lower or upper case e.
c ****** The mantissa must have at least one digit, and the
c ****** exponent, if present, must have between 1 and 3 digits.
c ****** Otherwise, set IFFP=.FALSE.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      logical ifpoint,ifexp
c
c-----------------------------------------------------------------------
c
      character*7 fmt
c
c-----------------------------------------------------------------------
c
      iffp=.false.
      ifpoint=.false.
      ifexp=.false.
      nmant=0
      nexp=0
c
      do 100 k=1,len(alpha)
      i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
      if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
      if (i.ge.48.and.i.le.57) then
c
c ****** Count digits in mantissa and exponent.
c
        if (ifexp) then
          nexp=nexp+1
        else
          nmant=nmant+1
        end if
        go to 100
c
      end if
c
c ****** Check for a decimal point.
c
      if (.not.ifpoint.and.i.eq.46) then
c
c ****** Check that we are in the mantissa.
c
        if (.not.ifexp) then
          ifpoint=.true.
          go to 100
        end if
c
      end if
c
c ****** Check for an exponent.
c
      if (.not.ifexp.and.(i.eq.101.or.i.eq.69)) then
        ifexp=.true.
        ke=k
        go to 100
      end if
c
c ****** Check for an exponent sign.
c
      if (ifexp.and.k.eq.ke+1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Failed check: fall through here.
c
      iffp=.false.
c
      return
c
  100 continue
c
c ****** Final check of validity: check number of digits in
c ****** mantissa and exponent.
c
      if (nmant.ge.1) iffp=.true.
      if (ifexp.and.(nexp.lt.1.or.nexp.gt.3)) iffp=.false.
c
c ****** Obtain its numeric value.
c
      fmt='(f  .0)'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (iffp) read (alpha,fmt) value
c
      return
      end
c#######################################################################
      logical function ifint (alpha,ivalue)
c
c-----------------------------------------------------------------------
c
c ****** If ALPHA represents an integer, return IFINT=.true., and
c ****** put its value into IVALUE.
c
c ****** Otherwise, return IFINT=.false..
c
c-----------------------------------------------------------------------
c
c ****** A valid integer has the format:
c
c          ALPHA = '[A]B[B...B]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      integer ivalue
c
c-----------------------------------------------------------------------
c
      character*5 fmt
c
c-----------------------------------------------------------------------
c
      ifint=.false.
c
      do 100 k=1,len(alpha)
c
        i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
        if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
        if (i.ge.48.and.i.le.57) then
          ifint=.true.
          go to 100
        end if
c
c ****** Failed check: fall through here.
c
        ifint=.false.
c
        return
c
  100 continue
c
c ****** Obtain its numeric value.
c
      fmt='(i  )'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (ifint) read (alpha,fmt) ivalue
c
      return
      end
