      program MAPFL2
c
c-----------------------------------------------------------------------
c
c ****** Get the field line mapping for a particular
c ****** magnetic field topology.
c
c modified by taking Router, the point from which you 
c want to trace field lines.
c This replaces R1 so that you don't have to make a 
c special smaller file just to trace the field lines.
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*(*),cdate*(*)
c
      parameter (cname='MAPFL')
      parameter (cvers='1.00')
      parameter (cdate='10/12/1999')
c
c-----------------------------------------------------------------------
c
      cliche globals
c
c-----------------------------------------------------------------------
c
c ****** Global parameters.
c
      common /gldims/ nxbx,nybx,nzbx,nxby,nyby,nzby,nxbz,nybz,nzbz,
     &                nrs,nts,nps,ntss,npss
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche mesh
c
c-----------------------------------------------------------------------
c
c ****** Mesh.
c
      real xbx(nxbx),xby(nxby),xbz(nxbz),
     &     ybx(nybx),yby(nyby),ybz(nybz),
     &     zbx(nzbx),zby(nzby),zbz(nzbz)
      pointer (ipxbx,xbx),(ipxby,xby),(ipxbz,xbz),
     &        (ipybx,ybx),(ipyby,yby),(ipybz,ybz),
     &        (ipzbx,zbx),(ipzby,zby),(ipzbz,zbz)
c
      common /mshcom1/ ipxbx,ipxby,ipxbz,
     &                 ipybx,ipyby,ipybz,
     &                 ipzbx,ipzby,ipzbz
c
c ****** Size of domain.
c
      common /mshcom2/ x0,x1,y0,y1,z0,z1,r0,r1,router
c
c ****** Primary (r,t,p) meshes.
c
      real rs(nrs),ts(nts),ps(nps)
      pointer (iprs,rs),(ipts,ts),(ipps,ps)
c
      common /mshcom3/ iprs,ipts,ipps
c
c ****** Inverse interpolation arrays.
      pointer (ipxbxi,xbxi),(ipybxi,ybxi),(ipzbxi,zbxi)
      pointer (ipxbyi,xbyi),(ipybyi,ybyi)
      pointer (ipxbzi,xbzi),(ipybzi,ybzi)
c
      common /mshcom4/ ipxbxi,ipybxi,ipzbxi,ipxbyi,ipybyi,ipxbzi,ipybzi
c
      real xbxi(*),ybxi(*),zbxi(*)
      real xbyi(*),ybyi(*)
      real xbzi(*),ybzi(*)
c
      common /mshcom5/ nxbxu,nybxu,nzbxu,nxbyu,nybyu,nxbzu,nybzu
      common /mshcom6/ dxbxi,dybxi,dzbxi,dxbyi,dybyi,dxbzi,dybzi
c
c ****** Secondary (t,p) meshes.
c
      real tss(ntss),pss(npss)
      pointer (iptss,tss),(ippss,pss)
c
      common /mshcom7/ iptss,ippss
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche field
c
c-----------------------------------------------------------------------
c
c ****** Magnetic field storage.
c
      real bxd(nxbx,nybx,nzbx),
     &     byd(nxby,nyby,nzby),
     &     bzd(nxbz,nybz,nzbz)
      pointer (ipbxd,bxd),(ipbyd,byd),(ipbzd,bzd)
c
      common /fldcom1/ ipbxd,ipbyd,ipbzd
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche fline
c
c-----------------------------------------------------------------------
c
c ****** Maximum number of field lines and segments.
c
      parameter (nds=50000)
c
c ****** Field line traces.
c
      common /flpcom1/ xfl(nds),yfl(nds),zfl(nds)
c
c ****** Current step size and default step size multiplier.
c
      common /flpcom2/ ds00,dsmult
c
c ****** New mesh flag.
c
      logical newtpmesh
      common /flpcom3/ newtpmesh
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche files
c
c-----------------------------------------------------------------------
c
c ****** File names, format, switches.
c
      character*72 bxfile,byfile,bzfile
      common /setup1/ bxfile,byfile,bzfile
c
      character*72 rffile,tffile,pffile,effile
      common /setup2/ rffile,tffile,pffile,effile
c
      character*72 rbfile,tbfile,pbfile,ebfile
      common /setup3/ rbfile,tbfile,pbfile,ebfile
c
      character*16 bfmt,ofmt
      common /setup4/ bfmt,ofmt
c
      logical trace_fwd,trace_bwd
      common /setup5/ trace_fwd,trace_bwd
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
c ****** Begin main program.
c
c these are parameters to read in Router, the starting point of the trace
c
      integer iargc
      external iargc,getarg
      character*(72) arg1
      real Router_dummy
c
      write (*,*)
      write (*,*) 'Code: ',cname
      write (*,*) 'Version: ',cvers
      write (*,*) 'Date: ',cdate
      write (*,*)

      call getarg (1,arg1)
      READ(arg1,FMT='(F10.4)') Router_dummy

      write(*,*) 'Read in arg1: ',arg1
      write(*,*) 'Read in Router_dummy: ',Router_dummy
c
      call setup(Router_dummy)
c
      end
c#######################################################################
      block data
c
c-----------------------------------------------------------------------
c
      use fline
c
c-----------------------------------------------------------------------
c
      data dsmult/1./
c
      end
c#######################################################################
      subroutine setup(Router_dummy)
c
c-----------------------------------------------------------------------
c
c ****** Set up the run, read the input file, and allocate memory.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
      use fline
      use files
c
c-----------------------------------------------------------------------
c
      common /rflcom/ iprfl,iptfl,ippfl,ipefl
      pointer (iprfl,rfl)
      pointer (iptfl,tfl)
      pointer (ippfl,pfl)
      pointer (ipefl,efl)

      REAL Router_dummy
c
c-----------------------------------------------------------------------
c
c ****** Read the input file.
c
      Router = Router_dummy
      write(*,*) 'Router in setup: ',Router
      call ffopen (1,'mapfl.dat','r')
c
      read (1,*)
      read (1,*) bfmt
      read (1,*)
      read (1,*) bxfile
      read (1,*)
      read (1,*) byfile
      read (1,*)
      read (1,*) bzfile
      read (1,*)
      read (1,*) ofmt
      read (1,*)
      read (1,*) trace_fwd
      read (1,*)
      read (1,*) rffile
      read (1,*)
      read (1,*) tffile
      read (1,*)
      read (1,*) pffile
      read (1,*)
      read (1,*) effile
      read (1,*)
      read (1,*) trace_bwd
      read (1,*)
      read (1,*) rbfile
      read (1,*)
      read (1,*) tbfile
      read (1,*)
      read (1,*) pbfile
      read (1,*)
      read (1,*) ebfile
      read (1,*)
      read (1,*) dsmult
      read (1,*)
      read (1,*) newtpmesh
      read (1,*)
      read (1,*) ntss,npss
c
      close (1)
c
c ****** Read the magnetic field.
c
      call readb
c
c ****** Get the primary (r,t,p) meshes and the mesh limits.
c
      call getrtp
c
c ****** Make new (t,p) meshes.
c
      call newtp
c
      r0=x0
c 
c change this so that the outer distance is Router
c      r1=x1
      write(*,*) 'setting r1 to: ',router     
      r1 = Router
c
c ****** Build the inverse interpolation arrays
c ****** (to increase interpolation efficiency).
c
      nxbxu=100
      call getinv (xbx,nxbx,nxbxu,dxbxi,ipxbxi)
      nybxu=100
      call getinv (ybx,nybx,nybxu,dybxi,ipybxi)
c
      nxbyu=100
      call getinv (xby,nxby,nxbyu,dxbyi,ipxbyi)
      nybyu=100
      call getinv (yby,nyby,nybyu,dybyi,ipybyi)
c
      nxbzu=100
      call getinv (xbz,nxbz,nxbzu,dxbzi,ipxbzi)
      nybzu=100
      call getinv (ybz,nybz,nybzu,dybzi,ipybzi)
c
      nzbxu=100
      call getinv (zbx,nzbx+1,nzbxu,dzbxi,ipzbxi)
c
c ****** Set the default step size.
c
      call setds
c
c ****** Assign memory for the RFL, TFL, and PFL arrays.
c
      nwd=ntss*npss
      iprfl=igetmem(nwd)
      iptfl=igetmem(nwd)
      ippfl=igetmem(nwd)
      ipefl=igetmem(nwd)
c
c ****** Trace the field lines forward, if requested.
c
      if (trace_fwd) then
        call map_forward (rfl,tfl,pfl,efl)
      end if
c
c ****** Trace the field lines backward, if requested.
c
      if (trace_bwd) then
        call map_backward (rfl,tfl,pfl,efl)
      end if
c
      call exit (0)
c
      return
      end
c#######################################################################
      subroutine readb
c
c-----------------------------------------------------------------------
c
c ****** Read the magnetic field files.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
      use files
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      character*80 fname
      integer idim(3),ips(3)
c
c-----------------------------------------------------------------------
c
c ****** Read the magnetic fields.
c
c ****** Bx.
c
      fname=bxfile
      lf=lenstr(fname)
c
      write (*,*)
      write (*,*) 'Reading data file: ',fname(1:lf)
c
      call rdsds (bfmt,bxfile,3,ndim,idim,ipbxd,ifscale,ips,ierr)
c
      if (ierr.ne.0) go to 900
      if (ndim.ne.3) go to 900
      if (ifscale.eq.0) go to 900
      nxbx=idim(1)
      nybx=idim(2)
      nzbx=idim(3)
      ipxbx=ips(1)
      ipybx=ips(2)
      ipzbx=ips(3)
c
c ****** By.
c
      fname=byfile
      lf=lenstr(fname)
c
      write (*,*)
      write (*,*) 'Reading data file: ',fname(1:lf)
c
      call rdsds (bfmt,byfile,3,ndim,idim,ipbyd,ifscale,ips,ierr)
c
      if (ierr.ne.0) go to 900
      if (ndim.ne.3) go to 900
      if (ifscale.eq.0) go to 900
      nxby=idim(1)
      nyby=idim(2)
      nzby=idim(3)
      ipxby=ips(1)
      ipyby=ips(2)
      ipzby=ips(3)
c
c ****** Bz.
c
      fname=bzfile
      lf=lenstr(fname)
c
      write (*,*)
      write (*,*) 'Reading data file: ',fname(1:lf)
c
      call rdsds (bfmt,bzfile,3,ndim,idim,ipbzd,ifscale,ips,ierr)
c
      if (ierr.ne.0) go to 900
      if (ndim.ne.3) go to 900
      if (ifscale.eq.0) go to 900
      nxbz=idim(1)
      nybz=idim(2)
      nzbz=idim(3)
      ipxbz=ips(1)
      ipybz=ips(2)
      ipzbz=ips(3)
c
c ****** Check that the phi coordinates are all the same.
c
      if (nzbx.ne.nzby.or.nzbx.ne.nzbz.or.nzby.ne.nzbz) then
        write (*,*)
        write (*,*) '### ERROR in READB:'
        write (*,*) '### Anomaly in the magnetic field components.'
        write (*,*) 'The components do not have the same phi mesh.'
        write (*,*) 'Number of phi points in Br = ',nzbx
        write (*,*) 'Number of phi points in Bt = ',nzby
        write (*,*) 'Number of phi points in Bp = ',nzbz
        call exit (1)
      end if
c
c ****** Replace the phi coordinate array with one which has
c ****** an additional point at phi=2*pi.
c
      nwd=nzbx+1
      ipzbx=igetmem(nwd)
c
      do i=1,nzbx
        zbx(i)=zby(i)
      enddo
      zbx(nzbx+1)=2.*pi+1.e-6
c
      ipzby=ipzbx
      ipzbz=ipzbx
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in READB:'
      write (*,*) '### Invalid data in file.'
      write (*,*) 'File name: ',fname(1:lf)
      call exit (1)
c
      end
c#######################################################################
      subroutine getrtp
c
c-----------------------------------------------------------------------
c
c ****** Determine the primary (r,t,p) scales and the mesh limits 
c ****** from the type of magnetic field files read.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
c
c-----------------------------------------------------------------------
c
c ****** Check the type of magnetic field files read.
c
      if (nxbx.eq.nxby+1.and.nxbx.eq.nxbz+1.and.
     &    nybx.eq.nyby-1.and.nybx.eq.nybz) then
c
c ****** These are MAS code files.
c
        nrs=nxby
        iprs=ipxby
        nts=nybx
        ipts=ipybx
c
      else if (nxbx.eq.nxby-1.and.nxbx.eq.nxbz-1.and.
     &         nybx.eq.nyby+1.and.nybx.eq.nybz) then
c
c ****** These are POT3D code files.
c
        nrs=nxbx
        iprs=ipxbx
        nts=nyby
        ipts=ipyby
c
      else if (nxbx.eq.nxby.and.nxbx.eq.nxbz.and.
     &         nybx.eq.nyby.and.nybx.eq.nybz) then
c
c ****** These are non-staggered files.
c
        nrs=nxbx
        iprs=ipxbx
        nts=nybx
        ipts=ipybx
c
      else
c
c ****** Invalid file type.
c
        write (*,*)
        write (*,*) '### ERROR in GETRTP:'
        write (*,*) '### Unrecognized magnetic field file staggering.'
        call exit (1)
c
      end if
c
      nps=nzbx+1
      ipps=ipzbx
c
      write(*,*) 'Router in getrtp: ',Router
      x0=rs(1)
      x1=rs(nrs)
      y0=ts(1)
      y1=ts(nts)
      z0=ps(1)
      z1=ps(nps)
c
      return
      end
c#######################################################################
      subroutine newtp
c
c-----------------------------------------------------------------------
c
c ****** Make new (t,p) meshes.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use fline
c
c-----------------------------------------------------------------------
c
      if (newtpmesh) then
        iptss=igetmem(ntss)
        ippss=igetmem(npss)
        call loadtp
      else
        iptss=ipts
        ippss=ipps
        ntss=nts
        npss=nps
      end if
c
      return
      end
c#######################################################################
      subroutine loadtp
c
c-----------------------------------------------------------------------
c
c ****** Make new (t,p) meshes.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use fline
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dtss=pi/(ntss-1.)
      do i=1,ntss
        tss(i)=(i-1.)*dtss
      enddo
      dpss=(ps(nps)-ps(1))/(npss-1.)
      do j=1,npss
        pss(j)=ps(1)+(j-1.)*dpss
      enddo
c
      return
      end
c#######################################################################
      subroutine setds
c
c-----------------------------------------------------------------------
c
c ****** Set the field line integration step size based on the
c ****** mesh on which the magnetic field is defined.
c
c ****** The step size is set to the minimum of the cell dimensions
c ****** over the r, t, and p coordinates, multiplied by the
c ****** factor DSMULT.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use fline
c
c-----------------------------------------------------------------------
c
      drmin=abs(x1-x0)
      do i=1,nrs-1
        dr=abs(rs(i+1)-rs(i))
        drmin=amin1(drmin,dr)
      enddo
c
      dtmin=abs(y1-y0)
      do j=1,nts-1
        dt=abs(ts(j+1)-ts(j))
        dtmin=amin1(dtmin,dt)
      enddo
c
      dpmin=abs(z1-z0)
      do k=1,nps-1
        dp=abs(ps(k+1)-ps(k))
        dpmin=amin1(dpmin,dp)
      enddo
c
      ds00=amin1(drmin,r0*dtmin,r0*dtmin*dpmin)*dsmult
c
      return
      end
c#######################################################################
      subroutine map_forward (rfl,tfl,pfl,efl)
c
c-----------------------------------------------------------------------
c
c ****** Trace field lines outward from r=R0, and store the final
c ****** field line location in the RFL, FTL, PFL arrays.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use fline
      use files
c
c-----------------------------------------------------------------------
c
      real rfl(ntss,npss)
      real tfl(ntss,npss)
      real pfl(ntss,npss)
      real efl(ntss,npss)
c
c-----------------------------------------------------------------------
c
      integer idim(2),ips(2)
c
c-----------------------------------------------------------------------
c
      eps1=1.e-5*abs(r1-r0)
c
c ****** Trace field lines, starting from each (T,P) cell at r=R0,
c ****** until the field line hits r=R1, or goes back to r=R0,
c ****** or exhausts the number of segments allowed.
c
      write (*,*)
      write (*,*) 'Tracing field lines forward ...'
c
      nwarn=0
c
      do k=1,npss-1
        write (*,*) 'k = ',k
        do j=1,ntss
c
          rfl0=r0+eps1
          tfl0=tss(j)
          pfl0=pss(k)
          call s2c (rfl0,tfl0,pfl0,xx0,yy0,zz0)
c
          idir=1
          call tracefl (nds,ds00,xx0,yy0,zz0,xfl,yfl,zfl,npts,idir,ef)
c
c ****** Check that the field line reached R0 or R1.
c
          call c2s (xfl(npts),yfl(npts),zfl(npts),rfl1,tfl1,pfl1)
c
          if (abs(rfl1-r0).lt.eps1) then
            efac=0.
          else if (abs(rfl1-r1).lt.eps1) then
            efac=ef
          else
            write (*,*)
            write (*,*) '### WARNING in MAP_FORWARD:'
            write (*,*) '### A field line did not reach R0 or R1.'
            write (*,*) 'j = ',j
            write (*,*) 'k = ',k
            write (*,*) 'Number of points in field line = ',npts
            write (*,*) 'Final field line radius = ',rfl1
            nwarn=nwarn+1
            efac=0.
          end if
c
          if (nwarn.gt.100) then
            write (*,*)
            write (*,*) '### ERROR in MAP_FORWARD:'
            write (*,*) '### Over 100 field lines did not reach'//
     &                  ' R0 or R1.'
            call exit (1)
          end if
c
c ****** Store the the final field line location. 
c
          rfl(j,k)=rfl1
          tfl(j,k)=tfl1
          pfl(j,k)=pfl1
          efl(j,k)=efac
c
        enddo
      enddo
c
c ****** Extend RFL, TFL, and PFL periodically.
c
      do j=1,ntss
        rfl(j,npss)=rfl(j,1)
        tfl(j,npss)=tfl(j,1)
        pfl(j,npss)=pfl(j,1)
        efl(j,npss)=efl(j,1)
      enddo
c
c ****** Write out the final coordinates.
c
      idim(1)=ntss
      idim(2)=npss
c
      ips(1)=iptss
      ips(2)=ippss
c
      call wrsds (ofmt,rffile,2,idim,rfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_FORWARD:'
        write (*,*) '### Could not write the field line radius file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,tffile,2,idim,tfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_FORWARD:'
        write (*,*) '### Could not write the field line theta file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,pffile,2,idim,pfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_FORWARD:'
        write (*,*) '### Could not write the field line phi file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,effile,2,idim,efl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_FORWARD:'
        write (*,*) '### Could not write the field line EF file.'
        call exit (1)
      end if
c
      return
      end
c#######################################################################
      subroutine map_backward (rfl,tfl,pfl,efl)
c
c-----------------------------------------------------------------------
c
c ****** Trace field lines inward from r=R1, and store the final
c ****** field line location in the RFL, FTL, PFL arrays.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use fline
      use files
c
c-----------------------------------------------------------------------
c
      real rfl(ntss,npss)
      real tfl(ntss,npss)
      real pfl(ntss,npss)
      real efl(ntss,npss)
c
c-----------------------------------------------------------------------
c
      integer idim(2),ips(2)
c
c-----------------------------------------------------------------------
c
      eps1=1.e-5*abs(r1-r0)
c
c ****** Trace field lines, starting from each (T,P) cell at r=R1,
c ****** until the field line hits r=R0, or goes back to r=R1,
c ****** or exhausts the number of segments allowed.
c
      write (*,*)
      write (*,*) 'Tracing field lines backward ...'
c
      nwarn=0
c
      do k=1,npss-1
        write (*,*) 'k = ',k
        do j=1,ntss
c
          rfl0=r1-eps1
          tfl0=tss(j)
          pfl0=pss(k)
          call s2c (rfl0,tfl0,pfl0,xx0,yy0,zz0)
c
          idir=-1
          call tracefl (nds,ds00,xx0,yy0,zz0,xfl,yfl,zfl,npts,idir,ef)
c
c ****** Check that the field line reached R0 or R1.
c
          call c2s (xfl(npts),yfl(npts),zfl(npts),rfl1,tfl1,pfl1)
c
          if (abs(rfl1-r0).lt.eps1) then
            efac=1./ef
          else if (abs(rfl1-r1).lt.eps1) then
            efac=0.
          else
            write (*,*)
            write (*,*) '### WARNING in MAP_BACKWARD:'
            write (*,*) '### A field line did not reach R0 or R1.'
            write (*,*) 'j = ',j
            write (*,*) 'k = ',k
            write (*,*) 'Number of points in field line = ',npts
            write (*,*) 'Final field line radius = ',rfl1
            nwarn=nwarn+1
            efac=0.
          end if
c
          if (nwarn.gt.100) then
            write (*,*)
            write (*,*) '### ERROR in MAP_BACKWARD:'
            write (*,*) '### Over 100 field lines did not reach'//
     &                  ' R0 or R1.'
            call exit (1)
          end if
c
c ****** Store the the final field line location. 
c
          rfl(j,k)=rfl1
          tfl(j,k)=tfl1
          pfl(j,k)=pfl1
          efl(j,k)=efac
c
        enddo
      enddo
c
c ****** Extend RFL, TFL, and PFL periodically.
c
      do j=1,ntss
        rfl(j,npss)=rfl(j,1)
        tfl(j,npss)=tfl(j,1)
        pfl(j,npss)=pfl(j,1)
        efl(j,npss)=efl(j,1)
      enddo
c
c ****** Write out the final coordinates.
c
      idim(1)=ntss
      idim(2)=npss
c
      ips(1)=iptss
      ips(2)=ippss
c
      call wrsds (ofmt,rbfile,2,idim,rfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_BACKWARD:'
        write (*,*) '### Could not write the field line radius file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,tbfile,2,idim,tfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_BACKWARD:'
        write (*,*) '### Could not write the field line theta file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,pbfile,2,idim,pfl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_BACKWARD:'
        write (*,*) '### Could not write the field line phi file.'
        call exit (1)
      end if
c
      call wrsds (ofmt,ebfile,2,idim,efl,1,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in MAP_BACKWARD:'
        write (*,*) '### Could not write the field line EF file.'
        call exit (1)
      end if
c
      return
      end
c#######################################################################
      subroutine tracefl (nds,ds0,xfl0,yfl0,zfl0,xfl,yfl,zfl,n,idir,ef)
c
c-----------------------------------------------------------------------
c
c ****** Trace a field line.
c
c-----------------------------------------------------------------------
c
c ****** NDS is the maximum number of points to use in the trace.
c ****** DS0 is the arc length to use.
c ****** (XFL0,YFL0,ZFL0) are the coordinates of the starting point.
c
c ****** The number of points used in the trace is returned in N,
c ****** and the coordinates of the points are returned in the
c ****** arrays XFL, YFL, ZFL.
c
c-----------------------------------------------------------------------
c
      real xfl(nds),yfl(nds),zfl(nds)
c
c-----------------------------------------------------------------------
c
      logical inside
c
c-----------------------------------------------------------------------
c
c ****** Trace a field line starting at launch point (XFL0,YFL0,ZFL0).
c
      xfl(1)=xfl0
      yfl(1)=yfl0
      zfl(1)=zfl0
      ds=ds0
      n=1
c
c ****** Change the sign of DS if the normal field
c ****** points out of the domain.
c
      bmag0=1.
      bmag=1.
      if (.not.inside(xfl(1),yfl(1),zfl(1))) go to 300
      call getb (xfl(1),yfl(1),zfl(1),bx,by,bz,br)
      bmag0=sqrt(bx**2+by**2+bz**2)
      if (br*ds*idir.lt.0.) ds=-ds
c
      do 200 ids=1,nds-1
        n=n+1
        xfl(n)=xfl(n-1)
        yfl(n)=yfl(n-1)
        zfl(n)=zfl(n-1)
c
c ****** Predictor-corrector loop.
c
        fpc=.5
        dxp=0.
        dyp=0.
        dzp=0.
        do 100 ipc=1,2
c
c ****** Field line push.
c
          call getb (xfl(n),yfl(n),zfl(n),bx,by,bz,br)
          bmag=sqrt(bx**2+by**2+bz**2)
          if (bmag.eq.0.) go to 300
          rbds=fpc*ds/bmag
          dxt=rbds*bx
          dyt=rbds*by
          dzt=rbds*bz
          xfl(n)=xfl(n)+dxt-dxp
          yfl(n)=yfl(n)+dyt-dyp
          zfl(n)=zfl(n)+dzt-dzp
          dxp=dxt
          dyp=dyt
          dzp=dzt
c
c ****** Check whether the field line is inside the
c ****** simulation domain.
c
          if (.not.inside(xfl(n),yfl(n),zfl(n))) go to 300
c
          fpc=1.
  100   continue
  200 continue
c
  300 continue
c
      r0=sqrt(xfl0**2+yfl0**2+zfl0**2)
      r1=sqrt(xfl(n)**2+yfl(n)**2+zfl(n)**2)
      ef=bmag0*r0**2/(bmag*r1**2)
c
      return
      end
c#######################################################################
      logical function inside (x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Check whether point (X,Y,Z) lies inside the
c ****** simulation domain.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
c
c-----------------------------------------------------------------------
c
      inside=.true.
c
      call c2s (x,y,z,rv,tv,pv)
c
      if (rv.lt.r0) then
        rv=r0
        inside=.false.
        call s2c (rv,tv,pv,x,y,z)
      else if (rv.gt.r1) then
        rv=r1
        inside=.false.
        call s2c (rv,tv,pv,x,y,z)
      end if
c
      return
      end
c#######################################################################
      subroutine getb (x,y,z,bx,by,bz,br)
c
c-----------------------------------------------------------------------
c
c ****** Get the field (BX,BY,BZ) at the point (X,Y,Z).
c
c ****** The radial field is also returned in BR.
c
c-----------------------------------------------------------------------
c
c ****** Transform to spherical coordinates.
c
      call c2s (x,y,z,rv,tv,pv)
c
c ****** Get Br, Bt, and Bp.
c
      call brtpint (rv,tv,pv,br,bt,bp)
c
c ****** Transform B to Cartesian coordinates.
c
      st=sin(tv)
      ct=cos(tv)
      sp=sin(pv)
      cp=cos(pv)
      bx=br*st*cp+bt*ct*cp-bp*sp
      by=br*st*sp+bt*ct*sp+bp*cp
      bz=br*ct   -bt*st
c
      return
      end
c#######################################################################
      subroutine brtpint (rv,tv,pv,br,bt,bp)
c
c-----------------------------------------------------------------------
c
c ****** Get the three spherical components of the magnetic field
c ****** at the position (RV,TV,PV).
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
c
c-----------------------------------------------------------------------
c
c ****** Get the phi interpolation factor.
c
      call zifac (pv,k,kp1,az)
c
c ****** Find the r and theta interpolation factors for Br.
c
      call interpi (xbx,nxbx,nxbxu,xbxi,dxbxi,rv,i,ax)
      call interpi (ybx,nybx,nybxu,ybxi,dybxi,tv,j,ay)
c
      br= (1.-ax)*( (1.-ay)*( (1.-az)*bxd(i  ,j  ,k  )
     &                       +    az *bxd(i  ,j  ,kp1))
     &             +    ay *( (1.-az)*bxd(i  ,j+1,k  )
     &                       +    az *bxd(i  ,j+1,kp1)))
     &   +    ax *( (1.-ay)*( (1.-az)*bxd(i+1,j  ,k  )
     &                       +    az *bxd(i+1,j  ,kp1))
     &             +    ay *( (1.-az)*bxd(i+1,j+1,k  )
     &                       +    az *bxd(i+1,j+1,kp1)))
c
c ****** Find the r and theta interpolation factors for Bt.
c
      call interpi (xby,nxby,nxbyu,xbyi,dxbyi,rv,i,ax)
      call interpi (yby,nyby,nybyu,ybyi,dybyi,tv,j,ay)
c
      bt= (1.-ax)*( (1.-ay)*( (1.-az)*byd(i  ,j  ,k  )
     &                       +    az *byd(i  ,j  ,kp1))
     &             +    ay *( (1.-az)*byd(i  ,j+1,k  )
     &                       +    az *byd(i  ,j+1,kp1)))
     &   +    ax *( (1.-ay)*( (1.-az)*byd(i+1,j  ,k  )
     &                       +    az *byd(i+1,j  ,kp1))
     &             +    ay *( (1.-az)*byd(i+1,j+1,k  )
     &                       +    az *byd(i+1,j+1,kp1)))
c
c ****** Find the r and theta interpolation factors for Bp.
c
      call interpi (xbz,nxbz,nxbzu,xbzi,dxbzi,rv,i,ax)
      call interpi (ybz,nybz,nybzu,ybzi,dybzi,tv,j,ay)
c
      bp= (1.-ax)*( (1.-ay)*( (1.-az)*bzd(i  ,j  ,k  )
     &                       +    az *bzd(i  ,j  ,kp1))
     &             +    ay *( (1.-az)*bzd(i  ,j+1,k  )
     &                       +    az *bzd(i  ,j+1,kp1)))
     &   +    ax *( (1.-ay)*( (1.-az)*bzd(i+1,j  ,k  )
     &                       +    az *bzd(i+1,j  ,kp1))
     &             +    ay *( (1.-az)*bzd(i+1,j+1,k  )
     &                       +    az *bzd(i+1,j+1,kp1)))
c
      return
      end
c#######################################################################
      function brint (rv,tv,pv)
c
c-----------------------------------------------------------------------
c
c ****** Get the radial component of the magnetic field at
c ****** the position (RV,TV,PV).
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
c
c-----------------------------------------------------------------------
c
c ****** Get the phi interpolation factor.
c
      call zifac (pv,k,kp1,az)
c
c ****** Find the r and theta interpolation factors.
c
      call interpi (xbx,nxbx,nxbxu,xbxi,dxbxi,rv,i,ax)
      call interpi (ybx,nybx,nybxu,ybxi,dybxi,tv,j,ay)
c
      brint= (1.-ax)*( (1.-ay)*( (1.-az)*bxd(i  ,j  ,k  )
     &                          +    az *bxd(i  ,j  ,kp1))
     &                +    ay *( (1.-az)*bxd(i  ,j+1,k  )
     &                          +    az *bxd(i  ,j+1,kp1)))
     &      +    ax *( (1.-ay)*( (1.-az)*bxd(i+1,j  ,k  )
     &                          +    az *bxd(i+1,j  ,kp1))
     &                +    ay *( (1.-az)*bxd(i+1,j+1,k  )
     &                          +    az *bxd(i+1,j+1,kp1)))
c
      return
      end
c#######################################################################
      function btint (rv,tv,pv)
c
c-----------------------------------------------------------------------
c
c ****** Get the theta component of the magnetic field at
c ****** the position (RV,TV,PV).
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
c
c-----------------------------------------------------------------------
c
c ****** Get the phi interpolation factor.
c
      call zifac (pv,k,kp1,az)
c
c ****** Find the r and theta interpolation factors.
c
      call interpi (xby,nxby,nxbyu,xbyi,dxbyi,rv,i,ax)
      call interpi (yby,nyby,nybyu,ybyi,dybyi,tv,j,ay)
c
      btint= (1.-ax)*( (1.-ay)*( (1.-az)*byd(i  ,j  ,k  )
     &                          +    az *byd(i  ,j  ,kp1))
     &                +    ay *( (1.-az)*byd(i  ,j+1,k  )
     &                          +    az *byd(i  ,j+1,kp1)))
     &      +    ax *( (1.-ay)*( (1.-az)*byd(i+1,j  ,k  )
     &                          +    az *byd(i+1,j  ,kp1))
     &                +    ay *( (1.-az)*byd(i+1,j+1,k  )
     &                          +    az *byd(i+1,j+1,kp1)))
c
      return
      end
c#######################################################################
      function bpint (rv,tv,pv)
c
c-----------------------------------------------------------------------
c
c ****** Get the phi component of the magnetic field at
c ****** the position (RV,TV,PV).
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
      use field
c
c-----------------------------------------------------------------------
c
c ****** Get the phi interpolation factor.
c
      call zifac (pv,k,kp1,az)
c
c ****** Find the r and theta interpolation factors.
c
      call interpi (xbz,nxbz,nxbzu,xbzi,dxbzi,rv,i,ax)
      call interpi (ybz,nybz,nybzu,ybzi,dybzi,tv,j,ay)
c
      bpint= (1.-ax)*( (1.-ay)*( (1.-az)*bzd(i  ,j  ,k  )
     &                          +    az *bzd(i  ,j  ,kp1))
     &                +    ay *( (1.-az)*bzd(i  ,j+1,k  )
     &                          +    az *bzd(i  ,j+1,kp1)))
     &      +    ax *( (1.-ay)*( (1.-az)*bzd(i+1,j  ,k  )
     &                          +    az *bzd(i+1,j  ,kp1))
     &                +    ay *( (1.-az)*bzd(i+1,j+1,k  )
     &                          +    az *bzd(i+1,j+1,kp1)))
c
      return
      end
c#######################################################################
      subroutine zifac (p,k,kp1,ap)
c
c-----------------------------------------------------------------------
c
c ****** Get the interpolation factor at the phi value given by P.
c
c-----------------------------------------------------------------------
c
      use globals
      use mesh
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
      pv=p
c
c ****** Take care of the periodic wrap-around.
c
      if (pv.lt.0.) then
        pv=pv+pi2
      else if (pv.gt.pi2) then
        pv=pv-pi2
      end if
c
c ****** Get the phi interpolation factor.
c
      if (nzbx.eq.1) then
        k=1
        kp1=1
        ap=0.
      else
        call interpi (zbx,nzbx+1,nzbxu,zbxi,dzbxi,pv,k,ap)
        if (k.eq.nzbx) then
          kp1=1
        else
          kp1=k+1
        end if
      end if
c
      return
      end
c#######################################################################
      subroutine c2s (x,y,z,r,t,p)
c
c-----------------------------------------------------------------------
c
c ****** Convert from Cartesian coordinates (X,Y,Z)
c ****** to spherical coordinates (R,T,P).
c
c ****** This routine returns T and P in radians, in the
c ****** following range:
c
c          0. .le. t .le. pi
c          0. .le. p .lt. 2.*pi
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pid2=.5*pi)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
      r=sqrt(x**2+y**2+z**2)
c
      if (r.eq.0.) then
        t=0.
      else
        t=acos(z/r)
      end if
c
      if (x.eq.0.) then
        if (y.ge.0.) then
          p=pid2
        else
          p=-pid2
        end if
      else
        p=atan2(y,x)
      end if
      if (p.lt.0.) p=p+pi2
c
      return
      end
c#######################################################################
      subroutine s2c (r,t,p,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Convert from spherical coordinates (R,T,P) 
c ****** to Cartesian coordinates (X,Y,Z).
c
c ****** This routine assumes that T and P are in radians.
c
c-----------------------------------------------------------------------
c
      st=sin(t)
      x=r*st*cos(p)
      y=r*st*sin(p)
      z=r*cos(t)
c
      return
      end
c#######################################################################
      subroutine interp (x,n,xv,i,alpha)
c
c-----------------------------------------------------------------------
c
c ****** Get interpolation factor ALPHA and table index I.
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I and ALPHA to get the interpolated value.
c
c-----------------------------------------------------------------------
c
      real x(n)
c
c-----------------------------------------------------------------------
c
      do i=1,n-1
        if (xv.ge.x(i).and.xv.le.x(i+1)) then
          alpha=(xv-x(i))/(x(i+1)-x(i))
          return
        end if
      enddo
c
c ****** ERROR: value not found in table.
c
      write (*,*)
      write (*,*) '### ERROR in INTERP:'
      write (*,*) 'Abscissa not found in table.'
      write (*,*)
      write (*,*) 'N = ',n
      write (*,*) 'X = ',(x(i),i=1,n)
      write (*,*)
      write (*,*) 'Abscissa requested = ',xv
      call exit (1)
c
      return
      end
c#######################################################################
      subroutine getinv (x,n,nu,dxi,ipfi) 
c
c-----------------------------------------------------------------------
c
c ****** Build an inverse interpolation table to increase the
c ****** efficiency of table look-up in a nonuniform mesh.
c
c ****** On input, the table X(N) is specified, together with the
c ****** number of points to use in the inverse interpolation
c ****** table, NU.
c
c ****** The output is the inverse of the "inverse interpolation
c ****** table" spacing, DXI, and a pointer to the inverse
c ****** interpolation table, IPFI, which points to a table of
c ****** length NU.
c
c-----------------------------------------------------------------------
c
      real x(n)
      pointer (ipfi,fi(*))
c
c-----------------------------------------------------------------------
c
c ****** Check that the number of points is valid.
c
      if (nu.le.1) then
        write (*,*)
        write (*,*) '### ERROR in GETINV:'
        write (*,*) '### Invalid number of points specified'//
     &              ' for inverse interpolation table.'
        write (*,*)
        write (*,*) 'Number of points = ',nu
        call exit (1)
      end if
c
c ****** Assign memory for the inverse interpolation table.
c
      ipfi=igetmem(nu)
c
c ****** Set the uniform interval to be used in the inverse
c ****** interpolation.
c
      dx=(x(n)-x(1))/(nu-1.)
c
      if (dx.le.0.) then
        write (*,*)
        write (*,*) '### ERROR in GETINV:'
        write (*,*) '### Invalid interval for inverse'//
     &              'interpolation table.'
        write (*,*)
        write (*,*) 'Interval = ',dx
        call exit (1)
      end if
c
      dxi=1./dx
c
c ****** Build the inverse interpolation table.
c
      do k=1,nu
        xv=x(1)+(k-1.)*dx
        xv=max(xv,x(1))
        xv=min(xv,x(n))
        call interp (x,n,xv,i,alpha)
        fi(k)=i+alpha
        fi(k)=max(fi(k),1.)
        fi(k)=min(fi(k),float(n))
      enddo
c
      return
      end
c#######################################################################
      subroutine interpi (x,n,nu,fi,dxi,xv,i,alpha)
c
c-----------------------------------------------------------------------
c
c ****** Get interpolation factor ALPHA and table index I.
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I and ALPHA to get the interpolated value.
c
c ****** This version uses a uniform inverse interpolation table to
c ****** improve the efficiency of the search.
c
c-----------------------------------------------------------------------
c
      real x(n),fi(nu)
c
c-----------------------------------------------------------------------
c
c ****** Get an estimate of the nearest grid point location in
c ****** the (uniform) inverse interpolation table.
c
      xi=1.+(xv-x(1))*dxi
      i=xi
      i=max(i,1)
      i=min(i,nu)
      alpha=xi-i
      fiv=(1.-alpha)*fi(i)+alpha*fi(i+1)
c
c ****** Set IG to be the guess for the nearest grid point.
c
      ig=fiv
      ig=max(ig,1)
      ig=min(ig,n-1)
c
      if (xv.ge.x(ig)) then
c
c ****** Search forwards.
c
        do i=ig,n-1
          if (xv.ge.x(i).and.xv.le.x(i+1)) then
            alpha=(xv-x(i))/(x(i+1)-x(i))
            return
          end if
        enddo
c
      else
c
c ****** Search backwards.
c
        do i=ig-1,1,-1
          if (xv.ge.x(i).and.xv.le.x(i+1)) then
            alpha=(xv-x(i))/(x(i+1)-x(i))
            return
          end if
        enddo
c
      end if
c
c ****** ERROR: value not found in table.
c
      write (*,*)
      write (*,*) '### ERROR in INTERPI:'
      write (*,*) 'Abscissa not found in table.'
      write (*,*)
      write (*,*) 'N = ',n
      write (*,*) 'X = ',(x(i),i=1,n)
      write (*,*)
      write (*,*) 'Abscissa requested = ',xv
      call exit (1)
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=1
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      ndim=0
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
      nb=4*nwd
c
c     ip=malloc(%val(nb))
      ip=malloc(nb)
c
      if (ip.eq.0) then
        write (*,*)
        write (*,*) '### ERROR in IGETMEM:'
        write (*,*) '### Could not get the requested memory.'
        write (*,*) 'Number of bytes requested = ',nb
        call exit (1)
      end if
c
      igetmem=ip
c
      return
      end
