c
c-----------------------------------------------------------------------
c
c ****** Read a free-format text file and write it as an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        09/23/91, ZM, Version 1.00:
c
c         - Original version of program.
c
c        11/25/91, ZM, Version 1.01:
c
c         - Allocate memory dynamically using MALLOC.
c
c        03/18/92, ZM, Version 1.02:
c
c         - Cray UNICOS version.  Changed the call to MALLOC
c           to use HPALLOC.
c
c        04/22/92, ZM, Version 1.03:
c
c         - Changed the format of the text data to include
c           scales and multi-dimensional data.
c
c        05/11/92, ZM, Version 1.04:
c
c         - Cleaned up a bit.
c
c        10/08/92, ZM, Version 1.05:
c
c         - Installed the new command line syntax parser.
c         - Made the error reporting for text file reads
c           more robust.
c
c        11/04/92, ZM, Version 1.06:
c
c         - Modified for SGI IRIS from CRAY UNICOS.
c
c        07/29/2003, ZM, Version 1.07:
c
c         - This version uses the ZM tools libraries.
c         - The program thus uses the new HDF read/write
c           routines that can handle 32-bit and 64-bit data files.
c
c-----------------------------------------------------------------------
c
c#######################################################################
c
c ****** Note that this tool uses modules from ZM's tools library.
c
c#######################################################################
      module params
c
c-----------------------------------------------------------------------
c ****** Parameters.
c-----------------------------------------------------------------------
c
      implicit none
c
      character(256) :: textfile
      character(256) :: hdffile
      logical :: hdf32
c
      end module
c#######################################################################
      program MKHDF
c
c-----------------------------------------------------------------------
c
      use params
      use sds_def
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='MKHDF'
      character(*), parameter :: cvers='1.07'
      character(*), parameter :: cdate='07/29/2003'
c
c-----------------------------------------------------------------------
c
      type(sds) :: s
      integer :: ierr
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call set_parameters (cname,cvers,cdate)
c
c ****** Read the field from the text file.
c
      call rdtxt (textfile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the text data file.'
        write (*,*) 'IERR (from RDTXT) = ',ierr
        write (*,*) 'File name: ',trim(textfile)
        call exit (1)
      end if
c
c ****** Set the precision for the HDF file.
c
      s%hdf32=hdf32
c
c ****** Write the HDF file.
c
      call wrhdf (hdffile,s,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write the HDF data file.'
        write (*,*) 'IERR (from WRHDF) = ',ierr
        write (*,*) 'File name: ',trim(hdffile)
        call exit (1)
      end if
c
      call exit (0)
c
      end
c#######################################################################
      subroutine set_parameters (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use number_types
      use syntax
      use paragraph_def
      use get_usage_line_interface
      use print_par_interface
      use delete_par_interface
      use params
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      character(*) :: cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Storage the for usage line.
c
      type(paragraph), pointer :: usage
c
c ****** Storage for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      integer :: ierr
      character(256) :: arg
      logical :: set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (GROUP_A,'textfile',' ',' ')
      call defarg (GROUP_A,'hdffile','textfile',' ')
      call defarg (GROUP_K,'-64',' ',' ')
c
c ****** Parse the command line.
c
      call parse (errmsg,ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Make an HDF file from a text file.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage line.
c
        call get_usage_line (usage)
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
c
        call print_par (usage)
c
        write (*,*)
        write (*,*) 'When <hdffile> is not specified, the text file'//
     &              ' <textfile>.dat will be read,'
        write (*,*) 'and the HDF file produced will be named'//
     &              ' <textfile>.hdf.'
        write (*,*) '-64 specifies a 64-bit output HDF file.'
c
        call delete_par (usage)
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Input file name.
c
      call fetcharg ('textfile',set,arg)
      textfile=trim(arg)
c
c ****** Output file name.
c
      call fetcharg ('hdffile',set,arg)
      if (set) then
        hdffile=trim(arg)
      else
        hdffile=trim(textfile)//'.hdf'
        textfile=trim(textfile)//'.dat'
      end if
c
c ****** Precision for the HDF file.
c
      call fetcharg ('-64',set,arg)
      hdf32=.not.set
c
      return
      end
