c#######################################################################
      module syntax
c
c-----------------------------------------------------------------------
c
c ****** Group definitions.
c
c        GROUP 1: <kw>
c        GROUP 2: <arg>
c        GROUP 3: <kw> <arg>
c        GROUP 4: <kw> <arg> <arg>
c
      integer, parameter :: ngroup=4
c
      integer, parameter :: IGRPK  =1
      integer, parameter :: IGRPA  =2
      integer, parameter :: IGRPKA =3
      integer, parameter :: IGRPKAA=4
c
      integer, parameter :: mxarg=100
c
      integer, parameter :: IXGRP=1
      integer, parameter :: IXSET=2
      integer, parameter :: IXREQ=3
      integer, parameter :: IXKEY=4
      integer, parameter :: IXARG=5
      integer, parameter :: IXNAM=6
c
      integer :: narg=0
      integer :: info(6,mxarg)
c
c ****** Buffer for the usage line.
c
      character(512) :: ulbuf
c
c ****** Buffer for the error message.
c
      character(72) :: errmsg
c
c-----------------------------------------------------------------------
c
      end module
c#######################################################################
      module gif_buffers
c
c-----------------------------------------------------------------------
c
c ****** GIF file buffer.
c
      integer, parameter :: LENBUF=128
      character(LENBUF) :: buf
c
      integer :: filerec=0
      integer :: ibuf=0
c
c-----------------------------------------------------------------------
c
c ****** GIF bit buffer that can hold 32 bits.
c
      integer :: intbuf
      integer :: iintbuf=0
c
c-----------------------------------------------------------------------
c
c ****** GIF character buffer to write up to 255 bytes into the
c ****** GIF file.
c
      character(255) :: cbuf
      integer :: icbuf=0
c
c-----------------------------------------------------------------------
c
      end module
c#######################################################################
      program MOVIE
c
c-----------------------------------------------------------------------
c
c ****** Ratersize a sequence of HDF data sets to create a set
c ****** of images.
c
c ****** This version is for SGI IRIS.
c
c-----------------------------------------------------------------------
c
c ****** Compile and link with:
c
c     f90 movie.f $LIBDF -o movie
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        06/03/91, ZM, Version 1.00:
c
c         - Original version of program.
c
c        05/26/92, ZM, Version 1.01:
c
c         - Modified to allow the image to have arbitrary
c           dimensions.
c         - The scales in the input SDS are taken into
c           account if present.
c         - The storage is allocated dynamically.
c         - The parameters are accepted as arguments from
c           the command line.
c         - Added the capability to specify a rectangular
c           part of the image to rasterize.
c
c        07/10/92, ZM, Version 1.02:
c
c         - Increased the size of ULBUF to accomodate a longer
c           usage line description.  A minor formatting change
c           to the usage line was also made.
c
c        10/16/92, ZM, Version 1.03:
c
c         - Fixed a bug in SETPAR which prevented detecion
c           of an error when setting -seq.
c         - Updated the SDS read/write routines.
c         - Cleaned up a bit.
c
c        11/25/92, ZM, Version 1.04:
c
c         - Updated RDHDF and NXTHDF so that the HDF routine
c           DFSDrestart is called before reading from the file.
c           This is required because if one tries to read an
c           SDS from the same file twice in a row (as happens
c           when only 1 frame is being processed), an error
c           would occur.
c
c        11/04/97, ZM, Version 1.05:
c
c         - Added the ability to use "nearest-grid-point"
c           interpolation.  To seect this, set the switch -ngp.
c           The default is to use bilinear interpolation,
c           as before.
c
c        11/20/2001, ZM, Version 1.06:
c
c         - Added the ability to write GIF files.
c           This allows GIF movies to be written directly.
c
c        03/05/2002, ZM, Version 1.07:
c
c         - Fixed some very minor bugs.
c
c-----------------------------------------------------------------------
c
      character(*), parameter :: cname='MOVIE'
      character(*), parameter :: cvers='1.07'
      character(*), parameter :: cdate='03/05/2002'
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      logical verbose,group,scaled,ngp,setx0,setx1,sety0,sety1,
     &        gif_output
      common /parcom1/ verbose,group,nxi,nyi,iseq0,iseq1,
     &                 scaled,ngp,fmin,fmax,x0,x1,y0,y1,
     &                 setx0,setx1,sety0,sety1,gif_output
c
      character*256 iroot,itail,oroot,otail,palfile
      common /parcom2/ iroot,itail,oroot,otail,palfile
c
c-----------------------------------------------------------------------
c
c ****** Data arrays and raster image arrays.
c
      pointer (ipf,f),(ipx,x),(ipy,y)
c
      real, dimension(:,:), allocatable :: fi
      real, dimension(:), allocatable :: ax,ay
c
      real, dimension(:,:), allocatable :: ff
      real, dimension(:), allocatable :: xx,yy
c
      character, dimension(:,:), allocatable :: img
c
c-----------------------------------------------------------------------
c
c ****** Palette.
c
      character :: pal(3,0:255)
c
c-----------------------------------------------------------------------
c
      character*72 fname
      character*3 ch3
c
c-----------------------------------------------------------------------
c
c ****** HDF routine declarations.
c
      integer DFTAG_RLE
      parameter (DFTAG_RLE=11)
c
      integer dsgdims,dsgdata,d8pimg,d8aimg,DFPgetpal,d8spal
c
c-----------------------------------------------------------------------
c
      logical first
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call setpar (cname,cvers,cdate)
c
c ****** Check the image dimensions.
c
      if (nxi.le.2.or.nyi.le.2) then
        write (*,*)
        write (*,*) '### ERROR in MOVIE:'
        write (*,*) '### Invalid raster image dimensions specified.'
        write (*,*) 'Dimension 1 = ',nxi
        write (*,*) 'Dimension 2 = ',nyi
        call exit (1)
      end if
c
c ****** Set the flag for the first HDF read.
c
      first=.true.
c
c ****** Check if using absolute scaling.
c
      if (.not.scaled) then
        fmin=0.
        fmax=0.
        go to 150
      end if
c
c ****** Check if the minimum and maximum need to
c ****** be determined from the data (this is indicated
c ****** by setting min=max=0).
c
      if (.not.(fmin.eq.0..and.fmax.eq.0.)) go to 150
c
      if (verbose) then
        write (*,*)
        write (*,*) 'Getting the min and max over all frames ...'
        write (*,*)
      end if
c
c ****** Loop over input files to determine the max and min
c ****** over all frames of the data.
c
      do 100 i=iseq0,iseq1
c
c ****** Create the input file name.
c
        write (ch3,'(i3.3)') i
        fname=trim(iroot)//ch3//trim(itail)
c
        if (verbose) write (*,*) 'Reading from file: ',trim(fname)
c
        call getsds (first,fname,nx,ny,ipf,ipx,ipy)
c
c ****** Find the rectangular region selected and allocate memory
c ****** for the array FF that holds it.
c
        if (first) then
          if (.not.setx0) x0=setval(x,1)
          if (.not.setx1) x1=setval(x,nx)
          if (.not.sety0) y0=setval(y,1)
          if (.not.sety1) y1=setval(y,ny)
          call setrect (nx,ny,x,y,x0,x1,y0,y1,i0,j0,nxx,nyy)
          allocate (ff(nxx,nyy))
          allocate (xx(nxx))
          allocate (yy(nyy))
        end if
c
        first=.false.
c
        call extract (nx,ny,f,x,y,i0,j0,nxx,nyy,ff,xx,yy)
c
        call getmnmx (ff,nxx*nyy,fmn,fmx)
c
        if (i.eq.iseq0) then
          fmin=fmn
          fmax=fmx
        else
          fmin=min(fmin,fmn)
          fmax=max(fmax,fmx)
        end if
c
  100 continue
c
c ****** Write the max and min.
c
      if (verbose) then
        write (*,*)
        write (*,*) 'Minimum value over all frames = ',fmin
        write (*,*) 'Maximum value over all frames = ',fmax
      end if
c
  150 continue
c
      if (verbose) then
        write (*,*)
        write (*,*) 'Minimum used for scaling images = ',fmin
        write (*,*) 'Maximum used for scaling images = ',fmax
      end if
c
c ****** Load the palette.  The default palette is the
c ****** "Rainbow" palette.
c
      if (palfile.eq.' ') then
        call load_rainbow_palette (pal)
      else
        ires=DFPgetpal(palfile,pal)
        if (ires.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in MOVIE:'
          write (*,*) '### Could not read the palette.'
          write (*,*) 'IRES (from DFPgetpal) = ',ires
          write (*,*) 'File name: ',trim(palfile)
          call exit (1)
        end if
      end if
c
c ****** Allocate memory for the raster image arrays.
c
      allocate (fi(nxi,nyi))
      allocate (ax(nxi))
      allocate (ay(nyi))
c
      allocate (img(nxi,nyi))
c
      if (verbose) then
        write (*,*)
        write (*,*) 'Rasterizing the images ...'
        write (*,*)
      end if
c
c ****** Create the raster images.
c
      do 200 i=iseq0,iseq1
c
        write (ch3,'(i3.3)') i
        fname=trim(iroot)//ch3//trim(itail)
c
        call getsds (first,fname,nx,ny,ipf,ipx,ipy)
c
        if (first) then
          if (.not.setx0) x0=setval(x,1)
          if (.not.setx1) x1=setval(x,nx)
          if (.not.sety0) y0=setval(y,1)
          if (.not.sety1) y1=setval(y,ny)
          call setrect (nx,ny,x,y,x0,x1,y0,y1,i0,j0,nxx,nyy)
          allocate (ff(nxx,nyy))
          allocate (xx(nxx))
          allocate (yy(nyy))
        end if
c
        first=.false.
c
        call extract (nx,ny,f,x,y,i0,j0,nxx,nyy,ff,xx,yy)
c
c ****** Create the output file name.
c
        if (group) then
          fname=trim(oroot)//trim(otail)
        else
          fname=trim(oroot)//ch3//trim(otail)
        end if
c
c ****** Interpolate to the new dimensions.
c
        call intrp2 (ngp,nxx,nyy,ff,xx,yy,nxi,nyi,fi,ax,ay,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in MOVIE:'
          write (*,*) '### Could not interpolate the data.'
          write (*,*) 'The image or mesh data is bad.'
          write (*,*) 'Frame sequence number = ',i
          write (*,*) 'File name: ',trim(fname)
          call exit (1)
        end if
c
c ****** Rasterize the image.
c
        call raster (fi,nxi*nyi,fmin,fmax,img)
c
c ****** Output the raster image to a file.
c
        if (verbose) write (*,*) 'Writing to file: ',trim(fname)
c
        if (gif_output) then
c
c ****** Write a GIF file.
c
          if (group) then
            if (i.eq.iseq0) call gif_write_header (1,fname,nxi,nyi,pal)
            call gif_write_image (1,nxi,nyi,img)
            if (i.eq.iseq1) call gif_close_file (1)
          else
            call gif_write_header (1,fname,nxi,nyi,pal)
            call gif_write_image (1,nxi,nyi,img)
            call gif_close_file (1)
          end if
c
        else
c
c ****** Write an HDF raster-8 file.
c
c ****** Set the palette.
c
          if (.not.group.or.i.eq.iseq0) then
            ires=d8spal(pal)
          end if
c
          if (ires.ne.0) then
            write (*,*)
            write (*,*) '### ERROR in MOVIE:'
            write (*,*) '### Could not set the palette.'
            write (*,*) 'IRES (from D8SPAL) = ',ires
            write (*,*) 'Frame sequence number = ',i
            call exit (1)
          end if
c
c ****** Write the image.
c
          if (.not.group.or.i.eq.iseq0) then
            ires=d8pimg(trim(fname),img,nxi,nyi,DFTAG_RLE)
          else
            ires=d8aimg(trim(fname),img,nxi,nyi,DFTAG_RLE)
          end if
c
          if (ires.ne.0) then
            write (*,*)
            write (*,*) '### ERROR in MOVIE:'
            if (.not.group.or.i.eq.iseq0) then
              write (*,*) '### Call to D8PIMG resulted in an error.'
            else
              write (*,*) '### Call to D8AIMG resulted in an error.'
            end if
            write (*,*) 'IRES = ',ires
            write (*,*) 'Frame sequence number = ',i
            write (*,*) 'File name: ',trim(fname)
            call exit (1)
          end if
c
        end if
c
  200 continue
c
      if (verbose) then
        write (*,*)
        write (*,*) 'Dimension 1 of SDS = ',nx
        write (*,*) 'Dimension 2 of SDS = ',ny
        write (*,*)
        write (*,*) 'Dimension 1 of image = ',nxi
        write (*,*) 'Dimension 2 of image = ',nyi
      end if
c
      if (verbose) then
        if (nxx.lt.nx.or.nyy.lt.ny) then
          write (*,*)
          write (*,*) 'Extracted the rectangular region:'
          write (*,*) 'X0 = ',setval(xx,1)
          write (*,*) 'X1 = ',setval(xx,nxx)
          write (*,*) 'Y0 = ',setval(yy,1)
          write (*,*) 'Y1 = ',setval(yy,nyy)
          write (*,*)
          write (*,*) 'Mesh indices of extracted region:'
          write (*,*) 'X: ',i0,i0+nxx-1
          write (*,*) 'Y: ',j0,j0+nyy-1
        end if
      end if
c
      call exit (0)
c
      end
c#######################################################################
      subroutine setpar (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      logical verbose,group,scaled,ngp,setx0,setx1,sety0,sety1,
     &        gif_output
      common /parcom1/ verbose,group,nxi,nyi,iseq0,iseq1,
     &                 scaled,ngp,fmin,fmax,x0,x1,y0,y1,
     &                 setx0,setx1,sety0,sety1,gif_output
c
      character*256 iroot,itail,oroot,otail,palfile
      common /parcom2/ iroot,itail,oroot,otail,palfile
c
c-----------------------------------------------------------------------
c
      logical ifint,iffp
      external ifint,iffp
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
      logical set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (IGRPK  ,'-v',' ',' ')
      call defarg (IGRPK  ,'-single',' ',' ')
      call defarg (IGRPKAA,'-d','100 100','<nx> <ny>')
      call defarg (IGRPKAA,'-seq',' ','<start> <end>')
      call defarg (IGRPK  ,'-scale',' ',' ')
      call defarg (IGRPKA ,'-min','0.','<val>')
      call defarg (IGRPKA ,'-max','0.','<val>')
      call defarg (IGRPK  ,'-ngp',' ',' ')
      call defarg (IGRPA  ,'iroot',' ',' ')
      call defarg (IGRPA  ,'oroot','iroot',' ')
      call defarg (IGRPKA ,'-itail','.hdf','<arg>')
      call defarg (IGRPKA ,'-otail','.r8','<arg>')
      call defarg (IGRPKA ,'-x0','default','<val>')
      call defarg (IGRPKA ,'-x1','default','<val>')
      call defarg (IGRPKA ,'-y0','default','<val>')
      call defarg (IGRPKA ,'-y1','default','<val>')
      call defarg (IGRPK  ,'-gif',' ',' ')
      call defarg (IGRPKA ,'-palette','default','<file>')
c
c ****** Parse the command line.
c
      call parse (ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Rasterize a sequence of HDF data sets.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage.
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
        call wrulin (ulbuf)
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** Verbose flag.
c
      call fetcharg ('-v',set,arg)
      verbose=set
c
c ****** Output to single file flag.
c
      call fetcharg ('-single',set,arg)
      group=.not.set
c
c ****** Image dimensions.
c
      call fetcharg ('-d',set,arg)
      ls=lenstr(arg)
      do 100 i=2,ls-1
        if (arg(i:i).eq.' ') then
          if (.not.ifint(arg(1:i-1),nxi)) go to 200
          if (.not.ifint(arg(i+1:ls),nyi)) go to 200
          go to 300
        end if
  100 continue
  200 continue
      write (*,*)
      write (*,*) '### ERROR in SETPAR:'
      write (*,*) '### Error in setting the image dimensions.'
      write (*,*) 'Invalid format:'
      write (*,*) arg(1:ls)
      call exit (1)
  300 continue
c
c ****** File sequence numbers.
c
      call fetcharg ('-seq',set,arg)
      ls=lenstr(arg)
      do 400 i=2,ls-1
        if (arg(i:i).eq.' ') then
          if (.not.ifint(arg(1:i-1),iseq0)) go to 500
          if (.not.ifint(arg(i+1:ls),iseq1)) go to 500
          go to 600
        end if
  400 continue
  500 continue
      write (*,*)
      write (*,*) '### ERROR in SETPAR:'
      write (*,*) '### Error in setting the file sequence numbers.'
      write (*,*) 'Invalid format:'
      write (*,*) arg(1:ls)
      call exit (1)
  600 continue
c
c ****** Absolute scaling flag.
c
      call fetcharg ('-scale',set,arg)
      scaled=set
c
c ****** Minimum for scaling.
c
      call fetcharg ('-min',set,arg)
      ls=lenstr(arg)
      if (.not.iffp(arg(1:ls),fmin)) then
        write (*,*)
        write (*,*) '### ERROR in SETPAR:'
        write (*,*) '### Error in setting the minimum scaling value.'
        write (*,*) 'Invalid format:'
        write (*,*) arg(1:ls)
        call exit (1)
      end if
c
c ****** Maximum for scaling.
c
      call fetcharg ('-max',set,arg)
      ls=lenstr(arg)
      if (.not.iffp(arg(1:ls),fmax)) then
        write (*,*)
        write (*,*) '### ERROR in SETPAR:'
        write (*,*) '### Error in setting the maximum scaling value.'
        write (*,*) 'Invalid format:'
        write (*,*) arg(1:ls)
        call exit (1)
      end if
c
      if (fmin.gt.fmax) then
        hold=fmax
        fmax=fmin
        fmin=hold
      end if
c
c ****** Flag for "nearest-grid-point" interpolation.
c
      call fetcharg ('-ngp',set,arg)
      ngp=set
c
c ****** Flag to write a GIF output file.
c
      call fetcharg ('-gif',set,arg)
      gif_output=set
c
c ****** Palette.
c
      call fetcharg ('-palette',set,arg)
      ls=lenstr(arg)
      if (trim(arg).eq.'default') then
        palfile=' '
      else
        palfile=trim(arg)
      end if
c
c ****** Input file root.
c
      call fetcharg ('iroot',set,arg)
      ls=lenstr(arg)
      iroot=arg(1:ls)
c
c ****** Output file root.
c
      call fetcharg ('oroot',set,arg)
      if (set) then
        ls=lenstr(arg)
        oroot=arg(1:ls)
      else
        oroot=iroot
      end if
c
c ****** Input file tail.
c
      call fetcharg ('-itail',set,arg)
      ls=lenstr(arg)
      itail=arg(1:ls)
c
c ****** Output file tail.
c
      call fetcharg ('-otail',set,arg)
      if (set) then
        ls=lenstr(arg)
        otail=arg(1:ls)
      else
        if (gif_output) then
          otail='.gif'
        else
          otail='.r8'
        end if
      end if
c
c ****** Extraction rectangle.
c
      call fetcharg ('-x0',set,arg)
      setx0=set
      if (set) then
        ls=lenstr(arg)
        if (.not.iffp(arg(1:ls),x0)) then
          write (*,*)
          write (*,*) '### ERROR in SETPAR:'
          write (*,*) '### Error in setting X0.'
          write (*,*) 'Invalid format:'
          write (*,*) arg(1:ls)
          call exit (1)
        end if
      end if
c
      call fetcharg ('-x1',set,arg)
      setx1=set
      if (set) then
        ls=lenstr(arg)
        if (.not.iffp(arg(1:ls),x1)) then
          write (*,*)
          write (*,*) '### ERROR in SETPAR:'
          write (*,*) '### Error in setting X1.'
          write (*,*) 'Invalid format:'
          write (*,*) arg(1:ls)
          call exit (1)
        end if
      end if
c
      call fetcharg ('-y0',set,arg)
      sety0=set
      if (set) then
        ls=lenstr(arg)
        if (.not.iffp(arg(1:ls),y0)) then
          write (*,*)
          write (*,*) '### ERROR in SETPAR:'
          write (*,*) '### Error in setting Y0.'
          write (*,*) 'Invalid format:'
          write (*,*) arg(1:ls)
          call exit (1)
        end if
      end if
c
      call fetcharg ('-y1',set,arg)
      sety1=set
      if (set) then
        ls=lenstr(arg)
        if (.not.iffp(arg(1:ls),y1)) then
          write (*,*)
          write (*,*) '### ERROR in SETPAR:'
          write (*,*) '### Error in setting Y1.'
          write (*,*) 'Invalid format:'
          write (*,*) arg(1:ls)
          call exit (1)
        end if
      end if
c
      return
      end
c#######################################################################
      subroutine wrulin (line)
c
c-----------------------------------------------------------------------
c
c ****** Write the string LINE to the TTY, taking into account
c ****** any carriage returns in the line.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      ls=lenstr(line)
c
      i0=1
  100 continue
c
c ****** Find the next carriage return.
c
      icr=index(line(i0:ls),char(13))
c
      if (icr.gt.0) then
c
c ****** A CR was found.
c
        if (icr.eq.1) then
          write (*,*)
          i0=i0+1
        else
          write (*,*) line(i0:i0+icr-2)
          i0=i0+icr
        end if
        if (i0.le.ls) go to 100
c
      else
c
c ****** A CR was not found.
c
        write (*,*) line(i0:ls)
c
      end if
c
      return
      end
c#######################################################################
      subroutine getmnmx (f,n,fmn,fmx)
c
c-----------------------------------------------------------------------
c
c ****** Find the min and max of the elements of array F(N).
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      fmn=f(1)
      fmx=f(1)
      do 100 i=1,n
        fmn=amin1(fmn,f(i))
        fmx=amax1(fmx,f(i))
  100 continue
c
      return
      end
c#######################################################################
      subroutine setrect (nx,ny,x,y,x0,x1,y0,y1,i0,j0,nxx,nyy)
c
c-----------------------------------------------------------------------
c
c ****** Find the mesh point indices which are enclosed
c ****** by the rectangle [X0,X1] x [Y0,Y1].
c
c-----------------------------------------------------------------------
c
      real x(nx),y(ny)
c
c-----------------------------------------------------------------------
c
      if (x1.le.x0) go to 900
c
      do 100 i=1,nx
        if (x(i).ge.x0) then
          i0=i
          go to 110
        end if
  100 continue
      go to 900
  110 continue
c
      do 200 i=nx,1,-1
        if (x(i).le.x1) then
          i1=i
          go to 210
        end if
  200 continue
      go to 900
  210 continue
c
      i0=max0(i0,1)
      i0=min0(i0,nx)
      i1=max0(i1,1)
      i1=min0(i1,nx)
c
      nxx=i1-i0+1
c
c ****** Make sure that there are at least two mesh points
c ****** per dimension.
c
      if (nxx.lt.2) go to 900
c
      if (y1.le.y0) go to 900
c
      do 300 j=1,ny
        if (y(j).ge.y0) then
          j0=j
          go to 310
        end if
  300 continue
      go to 900
  310 continue
c
      do 400 j=ny,1,-1
        if (y(j).le.y1) then
          j1=j
          go to 410
        end if
  400 continue
      go to 900
  410 continue
c
      j0=max0(j0,1)
      j0=min0(j0,ny)
      j1=max0(j1,1)
      j1=min0(j1,ny)
c
      nyy=j1-j0+1
c
      if (nyy.lt.2) go to 900
c
      return
c
  900 continue
c
c ****** Error: invalid rectangle specified.
c
      write (*,*)
      write (*,*) '### ERROR in SETRECT:'
      write (*,*) '### Invalid extraction rectangle specified.'
      write (*,*) 'X0 = ',x0
      write (*,*) 'X1 = ',x1
      write (*,*) 'Y0 = ',y0
      write (*,*) 'Y1 = ',y1
      call exit (1)
c
      return
      end
c#######################################################################
      subroutine extract (nx,ny,f,x,y,i0,j0,nxx,nyy,ff,xx,yy)
c
c-----------------------------------------------------------------------
c
c ****** Extract the data from arrays F, X, and Y, and copy it
c ****** into arrays FF, XX, and YY, according to the rectangle
c ****** specified by I0, J0, NXX, and NYY.
c
c-----------------------------------------------------------------------
c
      real f(nx,ny),x(nx),y(ny),ff(nxx,nyy),xx(nxx),yy(nyy)
c
c-----------------------------------------------------------------------
c
      do 110 j=1,nyy
        yy(j)=y(j0+j-1)
        do 100 i=1,nxx
          ff(i,j)=f(i0+i-1,j0+j-1)
  100   continue
  110 continue
c
      do 200 i=1,nxx
        xx(i)=x(i0+i-1)
  200 continue
c
      return
      end
c#######################################################################
      function setval (x,i)
c
c-----------------------------------------------------------------------
c
      real x(*)
c
c-----------------------------------------------------------------------
c
      setval=x(i)
c
      return
      end 
c#######################################################################
      subroutine intrp2 (ngp,nx,ny,f,x,y,nx2,ny2,fi,ax,ay,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate the 2D array F(NX,NY), defined on meshes
c ****** X(NX) and Y(NY), onto a uniform mesh (NX2,NY2)
c ****** covering the same domain as the original mesh.
c ****** The interpolated function is placed in array
c ****** FI(NX2,NY2).
c
c ****** AX(NX2) and AY(NY2) are used as scratch arrays.
c
c ****** When NGP=.true., use "nearest-grid-point" interpolation;
c ****** otherwise, use bilinear interpolation.
c
c-----------------------------------------------------------------------
c
      logical ngp
      integer nx,ny,nx2,ny2,ierr
      real f(nx,ny),x(nx),y(ny),fi(nx2,ny2),ax(nx2),ay(ny2)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the interpolation factors.
c
      call getif (nx,x,nx2,ax,ierr)
      if (ierr.ne.0) return
      call getif (ny,y,ny2,ay,ierr)
      if (ierr.ne.0) return
c
c ****** Do the bilinear interpolation.
c
      do 200 j=1,ny2
        jj=ay(j)
        jp=min0(jj+1,ny)
        if (ngp) then
          ay1=0.
        else
          ay1=ay(j)-jj
        end if
        ay0=1.-ay1
        do 100 i=1,nx2
          ii=ax(i)
          ip=min0(ii+1,nx)
          if (ngp) then
            ax1=0.
          else
            ax1=ax(i)-ii
          end if
          ax0=1.-ax1
          fi(i,j)= ay0*(ax0*f(ii,jj)+ax1*f(ip,jj))
     &            +ay1*(ax0*f(ii,jp)+ax1*f(ip,jp))
  100   continue
  200 continue
c
      return
      end
c#######################################################################
      subroutine getif (n1,x,n2,alpha,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Get the interpolation factors for interpolation from an
c ****** arbitrary nonuniform mesh onto a uniform mesh (covering
c ****** the entire range).
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          N1      : [integer]
c                    Number of points in the input (nonuniform) mesh.
c
c          X       : [real array, dimension N1]
c                    Coordinates of the input mesh points.  The x(i)
c                    values must increase monotonically, i.e.,
c                    x(i+1).gt.x(i) for all i.
c
c          N2      : [integer]
c                    Number of points in the uniform mesh.
c
c ****** Output arguments:
c
c          ALPHA   : [real array, dimension N2]
c                    This array holds the factors for interpolation
c                    from mesh X to the uniform mesh.  Let f(i) be
c                    the value of a function on the nonuniform
c                    mesh x(i).  At mesh point j of the uniform mesh,
c                    the interpolated value fi of f is given by:
c
c                          i=alpha(j)
c                          a=alpha(j)-i
c                          fi=(1.-a)*f(i)+a*f(i+1)
c
c          IERR    : [integer]
c                    IERR=0 is returned if the interpolation was
c                    successful.
c                    IERR=1 indicates that N1 or N2 is invalid.
c                    IERR=2 indicates that X does not contain
c                    monotonically increasing values.
c
c-----------------------------------------------------------------------
c
      integer n1,n2,ierr
      real x(n1),alpha(n2)
c
c-----------------------------------------------------------------------
c
c ****** Statement function for normalizing the input mesh.
c
      xn(i)=1.+(n2-1)*(x(i)-x(1))/(x(n1)-x(1))
c
      ierr=0
c
c ****** Check N1 and N2.
c
      if (n1.lt.2.or.n2.lt.2) then
        ierr=1
        return
      end if
c
c ****** Get the interpolation factors.
c
      x1=1.
      j0=1
      do 200 i=1,n1-1
        x0=x1
        x1=xn(i+1)
        dx=x1-x0
        if (dx.le.0.) then
          ierr=2
          return
        end if
        j1=x1
        if (i.eq.n1-1) j1=n2
        if (j1.ge.j0) then
          do 100 j=j0,j1
            frac=(j-x0)/dx
            alpha(j)=i+frac
  100     continue
          j0=j1+1
        end if
  200 continue
c
      return
      end
c#######################################################################
      subroutine raster (f,n,fmin,fmax,image)
c
c-----------------------------------------------------------------------
c
c ****** Rasterize the data in array F(N) and place the resulting R8
c ****** image in array IMAGE(N).
c
c ****** An R8 image represents each pixel by a byte.
c
c ****** FMIN and FMAX are the minimum and maximum values used for
c ****** the rasterization, unless FMIN=FMAX=0., in which case
c ****** the minimum and maximum over the elements of array
c ****** F are computed and used.
c
c-----------------------------------------------------------------------
c
c ****** Min and max levels to use (min=0, max=255).
c
      parameter (lev0=1,lev1=254)
      parameter (nlev=lev1-lev0)
c
c-----------------------------------------------------------------------
c
      real f(n)
      character*1 image(n)
c
c-----------------------------------------------------------------------
c
c ****** Set the min and max.
c
      if (fmin.eq.0..and.fmax.eq.0.) then
        call getmnmx (f,n,fmn,fmx)
      else
        fmn=fmin
        fmx=fmax
      end if
c
      df=(fmx-fmn)/nlev
c
      if (df.eq.0.) then
        do 100 i=1,n
          image(i)=char(lev0)
  100   continue
        return
      end if
c
      do 200 i=1,n
        n=lev0+(f(i)-fmn)/df
        n=max0(n,lev0)
        n=min0(n,lev1)
        image(i)=char(n)
  200 continue
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do 100 i=len(str),1,-1
        if (str(i:i).ne.' ') go to 200
  100 continue
      i=1
  200 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata,DFSDrestart
      external dsgdims,dsgdisc,dsgdata,DFSDrestart
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Reset the HDF read routines to read from the beginning
c ****** of the file.  This is required, since, if the previous
c ****** read happened to be from the same file, the next 
c ****** read will be from the end of the file, not the beginning.
c
      iret=DFSDrestart()
c
c ****** Get the dimensions of the SDS.
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine nxthdf (fname,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine is used when a sequence of data sets with
c ****** identical dimensions is being read.
c
c ****** The first SDS can be read using routine RDHDF, which
c ****** allocates memory for the arrays.  Subsequent data sets
c ****** can then be read using this routine.  This routine checks
c ****** that the requested SDS has the same dimensions as the
c ****** previous one, and returns an error if it does not.
c
c ****** The data and scales (if present) from the SDS are put in
c ****** the storage pointed to by IPF and IPS.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          NDIM    : [integer]
c                    Number of dimensions that the data set should
c                    have.  An error condition is returned if the
c                    SDS does not have NDIM dimensions.
c
c          IDIM    : [integer array, dimension NDIM]
c                    The NDIM dimensions that the data set should
c                    have.  An error condition is returned if the
c                    SDS does not have these dimensions.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate whether the SDS scales
c                    (axes) should be read.  IFSCALE=1 means that
c                    scales should be present.  IFSCALE=0 means that
c                    scales are ignored.
c
c          IPS     : [integer array, dimension NDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays into which the
c                    scales are put.
c
c ****** Output arguments:
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer ndim,idim(ndim),ifscale,ips(ndim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata,DFSDrestart
      external dsgdims,dsgdisc,dsgdata,DFSDrestart
c
c-----------------------------------------------------------------------
c
c ****** Scratch storage.
c
      parameter (mxdim=3)
      integer ndim2,idim2(mxdim)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
c ****** Make sure that the number of dimensions expected does
c ****** not exceed MXDIM.
c
      if (ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in NXTHDF:'
        write (*,*) '### Number of dimensions expected exceeds'
        write (*,*) '### the maximum allowed.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        call exit (1)
      end if
c
c ****** Reset the HDF read routines to read from the beginning
c ****** of the file.  This is required, since, if the previous
c ****** read happened to be from the same file, the next 
c ****** read will be from the end of the file, not the beginning.
c
      iret=DFSDrestart()
c
c ****** Get the dimensions of the SDS.
c
      iret=dsgdims(fname(1:lf),ndim2,idim2,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in NXTHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check that the dimensions agree with the expected
c ****** dimensions.
c
      if (ndim2.ne.ndim) go to 900
      do 100 i=1,ndim
        if (idim2(i).ne.idim(i)) go to 910
  100 continue
c
c ****** Check to see if the scales should be read in.
c
      if (ifscale.ne.0) then
c
        nscal=0
        do 200 i=1,ndim
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
          if (iret.eq.0) nscal=nscal+1
  200   continue
c
c ****** Check that all scales were read in.
c
        if (nscal.ne.ndim) go to 920
c
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in NXTHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in NXTHDF:'
      write (*,*) '### Invalid number of dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) 'Expected number of dimensions = ',ndim
      write (*,*) 'Actual number of dimensions = ',ndim2
      ierr=4
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in NXTHDF:'
      write (*,*) '### Invalid dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) 'Expected dimensions = ',(idim(i),i=1,ndim)
      write (*,*) 'Actual dimensions = ',(idim2(i),i=1,ndim2)
      ierr=5
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in NXTHDF:'
      write (*,*) '### Error in reading the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=6
c
      return
      end
c#######################################################################
      subroutine getsds (first,fname,nx,ny,ipf,ipx,ipy)
c
c-----------------------------------------------------------------------
c
c ****** Get a 2D SDS from the HDF file named FNAME.
c
c ****** When FIRST=.true., space is allocated for the data
c ****** and scales, and the SDS is read.  The pointers
c ****** IPF, IPX, and IPY are returned to point to the
c ****** data and scales.  The dimensions of the data NX and NY
c ****** are also returned.
c
c ****** When FIRST=.false., a new SDS is read.  It must have
c ****** the same dimensions as those specified by NX and NY.
c ****** It is placed in the storage pointed to by IPF, IPX,
c ****** and IPY.
c
c ****** Use routines RDHDF and NXTHDF.
c
c-----------------------------------------------------------------------
c
      logical first
      character*(*) fname
c
c-----------------------------------------------------------------------
c
      parameter (mxdim=3)
      integer idim(mxdim),ips(mxdim)
      save ifscale
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (first) then
c
c ****** On the first time, use routine RDHDF to allocate the
c ****** space and read the SDS.
c
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in GETSDS:'
          write (*,*) '### Could not read the SDS.'
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
c ****** Check that it is a 2D field.
c
        if (ndim.ne.2) then
          write (*,*)
          write (*,*) '### ERROR in GETSDS:'
          write (*,*) '### The SDS does not contain a 2D array.'
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
        nx=idim(1)
        ny=idim(2)
c
c ****** If the scales are not present, make a set of uniform
c ****** scales.
c
        if (ifscale.eq.0) then
          call uscal (nx,ny,ipx,ipy,ierr)
          if (ierr.ne.0) then
            write (*,*)
            write (*,*) '### ERROR in GETSDS:'
            write (*,*) '### Could not allocate storage for scales.'
            write (*,*) 'Number of words required = ',nx+ny
            call exit (1)
          end if
        else
          ipx=ips(1)
          ipy=ips(2)
        end if
c
      else
c
c ****** For subsequent reads, use routine NXTHDF to read the SDS.
c
        idim(1)=nx
        idim(2)=ny
c
        ips(1)=ipx
        ips(2)=ipy
c
        call nxthdf (fname,2,idim,ipf,ifscale,ips,ierr)
c
        if (ierr.ge.4) then
          write (*,*)
          write (*,*) '### ERROR in GETSDS:'
          write (*,*) '### The SDS does not have the same structure'
          write (*,*) '### as previous members of the sequence.'
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        else if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in GETSDS:'
          write (*,*) '### Could not read the SDS.'
          write (*,*) 'File name: ',fname(1:lf)
          call exit (1)
        end if
c
      end if
c
      return
      end
c#######################################################################
      subroutine uscal (nx,ny,ipx,ipy,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Make a set of uniform scales.
c
c ****** Assign storage for the scales and return the
c ****** pointers in IPX and IPY.
c
c-----------------------------------------------------------------------
c
      integer nx,ny
      pointer (ipx,x(*)),(ipy,y(*))
c
c-----------------------------------------------------------------------
c
      ierr=1
c
      nwd=nx
      ipx=igetmem(nwd)
      if (ipx.eq.0) return
c
      nwd=ny
      ipy=igetmem(nwd)
      if (ipy.eq.0) return
c
      do 100 i=1,nx
        x(i)=i
  100 continue
c
      do 200 j=1,ny
        y(j)=j
  200 continue
c
      ierr=0
c
      return
      end
c#######################################################################
      subroutine gif_write_header (iun,fname,nx,ny,palette)
c
c-----------------------------------------------------------------------
c
c ****** Write the header of a GIF file.
c
c ****** The GIF file is written to the file named FNAME, using
c ****** logical unit number IUN.
c
c ****** The GIF image dimensions are NX x NY, and a global palette
c ****** whose RGB color map values are in PALETTE is set as the
c ****** global palette.
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
      character(*) :: fname
      integer :: nx,ny
      character :: palette(3,0:255)
c
c-----------------------------------------------------------------------
c
c ****** Create the file.
c
      open (iun,file=fname,form='unformatted',
     &      access='direct',recl=LENBUF,status='replace')
c
c ****** Initialize the buffer record pointers.
c
      filerec=0
      ibuf=0
      iintbuf=0
      icbuf=0
c
c ****** Write the GIF signature.
c
      call gif_put_string (iun,'GIF87a')
c
c ****** Write the screen descriptor.
c
c ****** Screen dimensions.
c
      call gif_put_i2 (iun,nx)
      call gif_put_i2 (iun,ny)
c
c ****** Global color map flag, color resolution, and number
c ****** of colors.
c
      i=b'11110111'
      call gif_puti (iun,i)
c
c ****** Background color index.
c
      call gif_puti (iun,0)
c
c ****** Terminator.
c
      call gif_puti (iun,0)
c
c ****** Write the global color map.
c
      do i=0,255
        call gif_putc (iun,palette(1,i))
        call gif_putc (iun,palette(2,i))
        call gif_putc (iun,palette(3,i))
      enddo
c
      return
      end
c#######################################################################
      subroutine gif_write_image (iun,nx,ny,img)
c
c-----------------------------------------------------------------------
c
c ****** Write an image to a GIF file.
c
c ****** The image is written to the file connected to
c ****** unit number IUN.
c
c ****** This routine can be called repeatedly with different
c ****** IMG arrays (with the same NX and NY) to write multiple
c ****** images to the file (i.e., a movie).
c
c ****** It is assumed that IMG(1,1) has the bottom leftmost pixel
c ****** of the image, and that IMG(NX,NY) has the top rightmost
c ****** pixel of the image.
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
      integer :: nx,ny
      character :: img(nx,ny)
c
c-----------------------------------------------------------------------
c
c ****** Write the image descriptor.
c
c ****** Image separator.
c
      call gif_putc (iun,',')
c
c ****** Image position.
c
      call gif_put_i2 (iun,0)
      call gif_put_i2 (iun,0)
      call gif_put_i2 (iun,nx)
      call gif_put_i2 (iun,ny)
c
c ****** Image color map flag.
c
      i=b'00000111'
      call gif_puti (iun,i)
c
c ****** Image code size.
c
      call gif_puti (iun,8)
c
c ****** Image pixel values.
c
      call gif_lzw_compress (iun,nx,ny,img)
c
c ****** Terminator.
c
      call gif_puti (iun,0)
c
      return
      end
c#######################################################################
      subroutine gif_close_file (iun)
c
c-----------------------------------------------------------------------
c
c ****** Close the GIF file connected to unit number IUN.
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
c
c-----------------------------------------------------------------------
c
c ****** Write the GIF terminator.
c
      call gif_putc (iun,';')
c
c ****** Flush the file buffer and close the file.
c
      call gif_flush_file_buffer (iun)
c
      close (iun)
c
      end
c#######################################################################
      subroutine gif_lzw_compress (iun,nx,ny,img)
c
c-----------------------------------------------------------------------
c
c ****** Write the image raster data for the integer pixel values
c ****** in IMG using Lempel-Ziv-Welch compression, as specified in
c ****** the GIF87a standard.
c
c-----------------------------------------------------------------------
c
      integer :: iun
      integer :: nx,ny
      character :: img(nx,ny)
c
c-----------------------------------------------------------------------
c
      integer :: clear_code
      integer :: eoi_code
      integer :: current_code
      integer :: next_code
      integer :: next_char
      integer :: nb
      logical :: first
c
c-----------------------------------------------------------------------
c
c ****** Dictionary table.
c
      integer(2) :: dict(0:255,0:4095)
c
c ****** Array to specify the number of bits for each code.
c
      integer(2) :: nbits(0:4096)
c
      data nbits(0:4096)/513*9,512*10,1024*11,2048*12/
c
c-----------------------------------------------------------------------
c
c ****** Initialize the dictionary.
c
      dict(:,:)=0
c
c ****** Initialize the codes.
c
      clear_code=256
      eoi_code=clear_code+1
      next_code=clear_code+2
c
c ****** Set the initial number of bits.
c
      nb=9
c
c ****** Output the "clear code".
c
      call gif_add_to_bit_buffer (iun,nb,clear_code)
c
c ****** Process all pixels.
c
      first=.true.
c
      do j=ny,1,-1
        do i=1,nx
c
c ****** Get the next character.
c
          next_char=ichar(img(i,j))
c
          if (first) then
            current_code=next_char
            first=.false.
            cycle
          end if
c
c ****** Check if the new string is in the dictionary.
c
          if (dict(next_char,current_code).ne.0) then
c
c ****** The string is in the dictionary.
c ****** Update the string.
c
            current_code=dict(next_char,current_code)
c
          else
c
c ****** The string is not in the dictionary.
c ****** Add it to the dictionary, and output its code.
c
            dict(next_char,current_code)=next_code
c
            nb=nbits(next_code)
            call gif_add_to_bit_buffer (iun,nb,current_code)
c
            current_code=next_char
            next_code=next_code+1
c
c ****** Clear the dictionary if the code table has filled up.
c
            if (next_code.ge.4096) then
              dict(:,:)=0
              nb=nbits(next_code)
              call gif_add_to_bit_buffer (iun,nb,clear_code)
              next_code=clear_code+2
            end if
c
          end if 
c
        enddo
      enddo
c
c ****** Print out the final code.
c
      nb=nbits(next_code)
      call gif_add_to_bit_buffer (iun,nb,current_code)
c
c ****** Output an "End of Information" code.
c
      nb=nbits(next_code)
      call gif_add_to_bit_buffer (iun,nb,eoi_code)
c
c ****** Flush the bit buffer.
c
      call gif_flush_bit_buffer (iun)
c
c ****** Flush the character buffer.
c
      call gif_flush_char_buffer (iun)
c
      return
      end
c#######################################################################
      subroutine gif_add_to_bit_buffer (iun,nb,iadd)
c
c-----------------------------------------------------------------------
c
c ****** Add the integer IADD to the bit buffer, using NB bits to
c ****** represent the number.
c
c ****** The bit buffer is emptied 8 bits at a time by filling the
c ****** character buffer CBUF, which itself is emptied when it
c ****** fills up.
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
      integer :: nb
      integer :: iadd
c
c-----------------------------------------------------------------------
c
      integer :: ival
c
c-----------------------------------------------------------------------
c
c ****** Add IADD to the buffer, using NB bits.
c
      call mvbits (iadd,0,nb,intbuf,iintbuf)
      iintbuf=iintbuf+nb
c
      do
c
c ****** Write an 8-bit word when it becomes available.
c
        if (iintbuf.lt.8) exit
c
c ****** Write the 8 least significant bits first.
c
        ival=0
        call mvbits (intbuf,0,8,ival,0)
c
c ****** Right-shift INTBUF to discard the 8 least significant bits
c ****** that were just written.
c
        intbuf=ishft(intbuf,-8)
        iintbuf=iintbuf-8
c
c ****** Empty the character buffer when it becomes full.
c
        if (icbuf.ge.255) call gif_flush_char_buffer (iun)
c
        icbuf=icbuf+1
        cbuf(icbuf:icbuf)=char(ival)
c
      enddo
c
      return
      end
c#######################################################################
      subroutine gif_flush_bit_buffer (iun)
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
c
c-----------------------------------------------------------------------
c
      integer :: ival
c
c-----------------------------------------------------------------------
c
      if (iintbuf.le.0) return
c
      ival=0
      call mvbits (intbuf,0,iintbuf,ival,0)
c
      intbuf=0
      iintbuf=0
c
c ****** Empty the character buffer when it becomes full.
c
      if (icbuf.ge.255) call gif_flush_char_buffer (iun)
c
      icbuf=icbuf+1
      cbuf(icbuf:icbuf)=char(ival)
c
      return
      end
c#######################################################################
      subroutine gif_flush_char_buffer (iun)
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
c
c-----------------------------------------------------------------------
c
      if (icbuf.le.0) return
c
      call gif_puti (iun,icbuf)
      call gif_put_string (iun,cbuf(1:icbuf))
      icbuf=0
c
      return
      end
c#######################################################################
      subroutine gif_put_string (iun,s)
c
c-----------------------------------------------------------------------
c
c ****** Write string S.
c
c-----------------------------------------------------------------------
c
      integer :: iun
      character(*) :: s
c
c-----------------------------------------------------------------------
c
      do i=1,len(s)
        call gif_putc (iun,s(i:i))
      enddo
c
      end
c#######################################################################
      subroutine gif_put_i2 (iun,i)
c
c-----------------------------------------------------------------------
c
c ****** Write a two-byte integer, with the least significant
c ****** byte first.
c
c-----------------------------------------------------------------------
c
      integer :: iun
      integer :: i
c
c-----------------------------------------------------------------------
c
      integer :: i1,i2
c
c-----------------------------------------------------------------------
c
      i1=mod(i,256)
      i2=i/256
c
      call gif_puti (iun,i1)
      call gif_puti (iun,i2)
c
      end
c#######################################################################
      subroutine gif_putc (iun,c)
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
      character :: c
c
c-----------------------------------------------------------------------
c
c ****** Empty the file buffer if it is full.
c
      if (ibuf.ge.LENBUF) then
        filerec=filerec+1
        write (iun,rec=filerec) buf
        ibuf=0
      end if
c
c ****** Add character C to the buffer.
c
      ibuf=ibuf+1
      buf(ibuf:ibuf)=c
c
      end
c#######################################################################
      subroutine gif_puti (iun,i)
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
      integer :: i
c
c-----------------------------------------------------------------------
c
c ****** Empty the file buffer if it is full.
c
      if (ibuf.ge.LENBUF) then
        filerec=filerec+1
        write (iun,rec=filerec) buf
        ibuf=0
      end if
c
c ****** Add the character corresponding to integer I to the buffer.
c
      ibuf=ibuf+1
      buf(ibuf:ibuf)=char(i)
c
      end
c#######################################################################
      subroutine gif_flush_file_buffer (iun)
c
c-----------------------------------------------------------------------
c
      use gif_buffers
c
c-----------------------------------------------------------------------
c
      integer :: iun
c
c-----------------------------------------------------------------------
c
c ****** Flush the file buffer if it is not empty.
c
      if (ibuf.gt.0) then
        filerec=filerec+1
        write (iun,rec=filerec) buf(1:ibuf)
        filerec=0
        ibuf=0
      end if
c
      end
c#######################################################################
      subroutine load_rainbow_palette (pal)
c
c-----------------------------------------------------------------------
c
c ****** Load the "Rainbow" palette (a standard Spyglass palette
c ****** ranging from violet to blue to cyan to green to yellow
c ****** to red) into PAL.
c
c-----------------------------------------------------------------------
c
      character :: pal(3,0:255)
c
c-----------------------------------------------------------------------
c
      node1=0
      node2=255/9
      node3=255*3/9
      node4=255*5/9
      node5=255*7/9
      node6=255
c
      do i=node1,node2
        r=.5*(1.-(float(i)-node1)/(node2-node1))
        g=0.
        b=1.
        call set_pal_value (r,g,b,pal(1,i))
      enddo
c
      do i=node2+1,node3
        r=0.
        g=(float(i)-node2-1)/(node3-node2-1)
        b=1.
        call set_pal_value (r,g,b,pal(1,i))
      enddo
c
      do i=node3+1,node4
        r=0.
        g=1.
        b=1.-(float(i)-node3-1.)/(node4-node3-1.)
        call set_pal_value (r,g,b,pal(1,i))
      enddo
c
      do i=node4+1,node5
        r=(float(i)-node4-1.)/(node5-node4-1.)
        g=1.
        b=0.
        call set_pal_value (r,g,b,pal(1,i))
      enddo
c
      do i=node5+1,node6
        r=1.
        g=1.-(float(i)-node5-1.)/(node6-node5-1.)
        b=0.
        call set_pal_value (r,g,b,pal(1,i))
      enddo
c
c ****** Set the low and high values to white and black.
c
      pal(1,0)=char(255)
      pal(2,0)=char(255)
      pal(3,0)=char(255)
c
      pal(1,255)=char(0)
      pal(2,255)=char(0)
      pal(3,255)=char(0)
c
      return
      end
c#######################################################################
      subroutine set_pal_value (r,g,b,pal)
c
c-----------------------------------------------------------------------
c
      real :: r,g,b
      character :: pal(3)
c
c-----------------------------------------------------------------------
c
      integer :: ir,ig,ib
c
c-----------------------------------------------------------------------
c
      ir=r*255
      ir=max(0,ir)
      ir=min(255,ir)
      pal(1)=char(ir)
c
      ig=g*255
      ig=max(0,ig)
      ig=min(255,ig)
      pal(2)=char(ig)
c
      ib=b*255
      ib=max(0,ib)
      ib=min(255,ib)
      pal(3)=char(ib)
c
      return
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
c     igetmem=malloc(%val(4*nwd))
      igetmem=malloc(4*nwd)
c
      return
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line.
c
c-----------------------------------------------------------------------
c
c ****** The syntax for the keyword/argument items can be defined
c ****** by using routine DEFARG.
c
c ****** The usage line is constructed in character string ULBUF.
c ****** Use routine WRULIN to write the usage line, since it may
c ****** contain embedded carriage returns.
c
c ****** On return, IERR=0 indicates that the command line was
c ****** parsed successfully.
c
c ****** IERR=1 indicates that no arguments were present.  This
c ****** is usually used to print the usage line.
c
c ****** IERR=2 indicates that a syntax error occured.
c
c ****** IERR=3 indicates that one or more required arguments
c ****** was not supplied.
c
c ****** When IERR=2 or IERR=3, and error message is put into
c ****** character string ERRMSG.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
c ****** Command line arguments.
c
      integer iargc
      external iargc,getarg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      ierr=0
      errmsg=' '
c
c ****** Construct the usage line in ULBUF.
c
      ix=1
      ulbuf=' '
c
      call getarg (0,arg)
      ls=lenstr(arg)
      call addtobuf (arg(1:ls),ulbuf,ix)
      nc=ix-1
      ncc=min0(nc,5)
c
      i=1
      itried=0
  100 continue
      if (i.le.narg) then
        ix0=ix
        call addtobuf (' ',ulbuf,ix)
        if (info(IXREQ,i).eq.0) then
          call addtobuf ('[',ulbuf,ix)
        end if
        call getstr (info(IXKEY,i),arg)
        ls=lenstr(arg)
        call addtobuf (arg(1:ls),ulbuf,ix)
        if (info(IXNAM,i).gt.0) then
          call addtobuf (' ',ulbuf,ix)
          call getstr (info(IXNAM,i),arg)
          ls=lenstr(arg)
          call addtobuf (arg(1:ls),ulbuf,ix)
        end if
        if (info(IXREQ,i).eq.0) then
          call addtobuf (']',ulbuf,ix)
        end if
        nc=nc+ix-ix0
        if (nc.gt.78.and.itried.eq.0) then
          itried=1
          ix=ix0
          call addtobuf (char(13),ulbuf,ix)
          ulbuf(ix:ix+ncc-1)=' '
          ix=ix+ncc
          nc=ncc
          go to 100
        end if
        itried=0
        i=i+1
        go to 100
      end if
c
c ****** Get the number of command line arguments.
c
      na=iargc()
      if (na.eq.0) then
        ierr=1
        go to 900
      end if
c
      ia=1
  200 continue
c
      ia0=ia
c
c ****** Process arguments with syntax: <kw> <arg>
c
      if (na-ia+1.ge.2) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw> <arg> <arg>
c
      if (na-ia+1.ge.3) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKAA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            ls=ls+1
            arg(ls:ls)=' '
            ia=ia+1
            call getarg (ia,arg(ls+1:))
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw>
c
      if (na-ia+1.ge.1) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPK,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            info(IXARG,iarg)=0
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <arg>
c
      if (na-ia+1.ge.1) then
        iarg=nxtarg(IGRPA)
        if (iarg.gt.0) then
          call getarg (ia,arg)
          ls=lenstr(arg)
          info(IXARG,iarg)=iputstr(arg(1:ls))
          info(IXSET,iarg)=1
          ia=ia+1
          go to 300
        end if
      end if
c
  300 continue
c
c ****** Check that an argument was found.
c
      if (ia.eq.ia0) then
        ierr=2
        go to 900
      end if
c
c ****** Keep processing arguments until done.
c
      if (na-ia+1.gt.0) go to 200
c
c ****** Check that the required arguments were supplied.
c
      do 400 i=1,narg
        if (info(IXREQ,i).ne.0.and.info(IXSET,i).eq.0) then
          ierr=3
          go to 900
        end if
  400 continue
c
      return
c
c ****** Error exit.
c
  900 continue
c
      if (ierr.eq.2) then
        errmsg='### Syntax error.'
      else if (ierr.eq.3) then
        errmsg='### A required argument was not supplied.'
      end if
c
      return
      end
c#######################################################################
      subroutine defarg (igroup,kw,default,argname)
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax for a command line argument item.
c
c-----------------------------------------------------------------------
c
c ****** IGROUP is the syntax group index.
c ****** KW is the keyword.
c ****** DEFAULT is the default value of the argument.
c ****** ARGNAME is the name of the argument (for use in
c ****** constructing the usage line).
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw,default,argname
c
c-----------------------------------------------------------------------
c
c ****** Check the group index.
c
      if (igroup.lt.0.or.igroup.gt.ngroup) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Invalid group index specified.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Check for a null keyword.
c
      if (kw.eq.' ') then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### The keyword is null.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Increment the argument counter.
c
      if (narg.ge.mxarg) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Exceeded number of allowed arguments.'
        write (*,*) 'Maximum number of arguments = ',mxarg
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
      narg=narg+1
c
c ****** Store the group index and keyword.
c
c ****** For group IGRPA (single arguments), the name of the
c ****** argument is stored as the "keyword".
c
      info(IXGRP,narg)=igroup
      ls=lenstr(kw)
      info(IXKEY,narg)=iputstr(kw(1:ls))
c
c ****** Initialize the flag which indicates whether an argument
c ****** has been set.
c
      info(IXSET,narg)=0
c
c ****** If a default argument was supplied, the argument
c ****** does not have to be set.  Use DEFAULT=' ' to
c ****** indicate that an argument is required.
c
c ****** If a default argument has been supplied, store it and
c ****** put its pointer in info(IXARG,narg).  If there is no
c ****** default, set info(IXARG,narg)=0.
c
c ****** Since group IGRPK doesn't have an argument,
c ****** DEFAULT is ignored for this group.
c
      if (igroup.eq.IGRPK) then
        info(IXREQ,narg)=0
        info(IXARG,narg)=0
      else
        if (default.eq.' ') then
          info(IXREQ,narg)=1
          info(IXARG,narg)=0
        else
          info(IXREQ,narg)=0
          ls=lenstr(default)
          info(IXARG,narg)=iputstr(default(1:ls))
        end if
      end if
c
c ****** Store the argument name.  For groups IGRPK (keywords) 
c ****** and IGRPA (single arguments), there is no argument name,
c ****** so ARGNAME is ignored.
c
      if (igroup.eq.IGRPK.or.igroup.eq.IGRPA) then
        info(IXNAM,narg)=0
      else
        ls=lenstr(argname)
        info(IXNAM,narg)=iputstr(argname(1:ls))
      end if
c
      return
      end
c#######################################################################
      subroutine fetcharg (kw,set,arg)
c
c-----------------------------------------------------------------------
c
c ****** Fetch the argument corresponding to keyword KW.
c
c-----------------------------------------------------------------------
c
c ****** If the corresponding argument was set, return SET=.true.,
c ****** and copy the argument to ARG.
c
c ****** If the argument was not set, return SET=.false..
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      logical set
      character*(*) kw,arg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the argument corresponding to keyword KW.
c
      do 100 i=narg,1,-1
        call getstr (info(IXKEY,i),tkw)
        if (tkw.eq.kw) then
          iarg=i
          go to 200
        end if
  100 continue
c
      ls=lenstr(kw)
      write (*,*)
      write (*,*) '### ERROR in FETCHARG:'
      write (*,*) '### Requested keyword could not be matched.'
      write (*,*) 'Keyword = ',kw(1:ls)
      call exit (1)
c
  200 continue
c
      set=info(IXSET,iarg).ne.0
      if (info(IXARG,iarg).gt.0) then
        call getstr (info(IXARG,iarg),arg)
      else
        arg=' '
      end if
c
      return
      end
c#######################################################################
      function matchkw (igroup,kw)
c
c-----------------------------------------------------------------------
c
c ****** Match keyword KW against the list of keywords in
c ****** group IGROUP.
c
c ****** If found, set the function value to the corresponding
c ****** argument number.  Otherwise, set MATCHKW=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
      do 100 i=narg,1,-1
        if (info(IXGRP,i).eq.igroup) then
          call getstr (info(IXKEY,i),tkw)
          if (tkw.eq.kw) then
            matchkw=i
            return
          end if
        end if
  100 continue
c
      matchkw=0
c
      return
      end
c#######################################################################
      function nxtarg (igroup)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the next argument in group IGROUP
c ****** which has not been set.
c
c-----------------------------------------------------------------------
c
c ****** If an empty slot is found, set the function value
c ****** to the corresponding argument number.
c
c ****** Otherwise, set NXTARG=0.
c
c-----------------------------------------------------------------------
c
      use syntax
c
c-----------------------------------------------------------------------
c
      do 100 i=1,narg
        if (info(IXGRP,i).eq.igroup) then
          if (info(IXSET,i).eq.0) then
            nxtarg=i
            return
          end if
        end if
  100 continue
c
      nxtarg=0
c
      return
      end
c#######################################################################
      function iputstr (line)
c
c-----------------------------------------------------------------------
c
c ****** Put the string in LINE into internal storage, and return
c ****** a pointer to it as the function value.
c
c ****** The string can be retrieved by calling routine GETSTR
c ****** with the returned pointer.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
      ls=len(line)
c
c ****** Check that the string is not empty.
c
      if (ls.le.0) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Trying to store a zero-length string.'
        call exit (1)
      end if
c
c ****** Check that a free pointer exists.
c
      if (nstr.ge.mxstr) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for pointers.'
        write (*,*) 'Maximum number of pointers = ',mxstr
        call exit (1)
      end if
c
      nstr=nstr+1
c
c ****** Check that there is enough room to store the string.
c
      ix1=ix+ls-1
      if (ix1.gt.LENBUF) then
        nb=ix1-LENBUF
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for strings.'
        write (*,*) 'Number of additional bytes needed = ',nb
        call exit (1)
      end if
c
      strbuf(ix:ix1)=line
c
c ****** Set IX to point to the next available character.
c
      ix=ix1+1
      indx(nstr)=ix
c
c ****** Return the pointer to the string as the function value.
c
      iputstr=nstr
c
      return
      end
c#######################################################################
      block data bdstr
c
c-----------------------------------------------------------------------
c
      parameter (mxstr=100)
      common /pstr1/ nstr,ix,indx(mxstr)
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data nstr/0/
      data ix/1/
c
      end
c#######################################################################
      subroutine getstr (ip,line)
c
c-----------------------------------------------------------------------
c
c ****** Get the string associated with pointer IP and put it
c ****** into LINE.
c
c ****** This routine retrieves strings stored with routine IPUTSTR.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
c ****** Check that the pointer is valid.
c
      if (ip.lt.0.or.ip.gt.nstr) then
        write (*,*)
        write (*,*) '### ERROR in GETSTR:'
        write (*,*) '### Invalid string pointer.'
        write (*,*) 'Pointer = ',ip
        call exit (1)
      end if
c
c ****** Retrieve the string.
c
      if (ip.eq.1) then
        i0=1
      else
        i0=indx(ip-1)
      end if
      i1=indx(ip)-1
c
      line=strbuf(i0:i1)
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (compress) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          write (*,*)
          write (*,*) '### ERROR in ADDTOBUF:'
          write (*,*) '### Internal buffer overflow.'
          write (*,*) '[This is an internal error.]'
          call exit (1)
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
c#######################################################################
      logical function ifint (alpha,ivalue)
c
c-----------------------------------------------------------------------
c
c ****** If ALPHA represents an integer, return IFINT=.true., and
c ****** put its value into IVALUE.
c
c ****** Otherwise, return IFINT=.false..
c
c-----------------------------------------------------------------------
c
c ****** A valid integer has the format:
c
c          ALPHA = '[A]B[B...B]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      integer ivalue
c
c-----------------------------------------------------------------------
c
      character*5 fmt
c
c-----------------------------------------------------------------------
c
      ifint=.false.
c
      do 100 k=1,len(alpha)
c
        i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
        if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
        if (i.ge.48.and.i.le.57) then
          ifint=.true.
          go to 100
        end if
c
c ****** Failed check: fall through here.
c
        ifint=.false.
c
        return
c
  100 continue
c
c ****** Obtain its numeric value.
c
      fmt='(i  )'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (ifint) read (alpha,fmt) ivalue
c
      return
      end
c#######################################################################
      logical function iffp (alpha,value)
c
c-----------------------------------------------------------------------
c
c ****** Determine if ALPHA represents a floating point number;
c ****** if so, return its value in VALUE.
c
c-----------------------------------------------------------------------
c
c ****** Set IFFP=.TRUE. if ALPHA contains an alphanumeric
c ****** string with the following format:
c
c       ALPHA = '[A][B...B][.][B...B][e[A]B[B...B]]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c ****** The exponent may be denoted by a lower or upper case e.
c ****** The mantissa must have at least one digit, and the
c ****** the exponent, if present, must have between 1 and 3 digits.
c ****** Otherwise, set IFFP=.FALSE.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      logical ifpoint,ifexp
c
c-----------------------------------------------------------------------
c
      character*7 fmt
c
c-----------------------------------------------------------------------
c
      iffp=.false.
      ifpoint=.false.
      ifexp=.false.
      nmant=0
      nexp=0
c
      do 100 k=1,len(alpha)
      i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
      if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
      if (i.ge.48.and.i.le.57) then
c
c ****** Count digits in mantissa and exponent.
c
        if (ifexp) then
          nexp=nexp+1
        else
          nmant=nmant+1
        end if
        go to 100
c
      end if
c
c ****** Check for a decimal point.
c
      if (.not.ifpoint.and.i.eq.46) then
c
c ****** Check that we are in the mantissa.
c
        if (.not.ifexp) then
          ifpoint=.true.
          go to 100
        end if
c
      end if
c
c ****** Check for an exponent.
c
      if (.not.ifexp.and.(i.eq.101.or.i.eq.69)) then
        ifexp=.true.
        ke=k
        go to 100
      end if
c
c ****** Check for an exponent sign.
c
      if (ifexp.and.k.eq.ke+1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Failed check: fall through here.
c
      iffp=.false.
c
      return
c
  100 continue
c
c ****** Final check of validity: check number of digits in
c ****** mantissa and exponent.
c
      if (nmant.ge.1) iffp=.true.
      if (ifexp.and.(nexp.lt.1.or.nexp.gt.3)) iffp=.false.
c
c ****** Obtain its numeric value.
c
      fmt='(f  .0)'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (iffp) read (alpha,fmt) value
c
      return
      end
