#!/bin/csh
#---------------------------------------------------------------------------
# set.sh: Set environment variables to compile and run ENLIL computations.
#
# Usage: set.sh <make | run>
#---------------------------------------------------------------------------

# Checking argument
if ($#argv != 1 || ($1 != 'make' && $1 != 'run')) then
  echo "Usage: set.sh <make | run>"
  exit 1
endif

# Checking system platform
if (`uname` == 'Linux') then
  set compiler = "ifort"
  set system = "linux"
else if (`uname` == 'Darwin') then
  set compiler = "xlf95_r"
  set system = "macosx"
else
  echo "ERROR: Unsupported platform"
  exit 1
endif

# Set directories
setenv ENLIL_DIR  "`pwd`"
setenv ENLIL_BIN  "`pwd`/bin/$system"
setenv NETCDF_DIR "`pwd`/lib/$system"

# Rerun ranlib just in case after copying
pushd lib/$system/lib > /dev/null
foreach lib (*.a)
  ranlib $lib 
end
popd > /dev/null


#--------------------------------------------------------------------------
# Things needed to be set for compilation.
#--------------------------------------------------------------------------
if ($1 == 'make') then

  # Checking fortran compiler
  set fortran = `which $compiler`
  if ($#fortran != 1) then
    echo "ERROR: $compiler is required, but not found"
    exit 1
  endif

  # Set environment variables
  setenv SYSTEM "$system"
  setenv F90 "$fortran"
  if ($compiler == 'ifort') then
    setenv F90FLAGS "-O3 -free"
    setenv CPPFLAGS "-DNDEBUG"
    setenv LDFLAGS "-static"
  else
    setenv F90FLAGS "-O3 -qnosave -qstrict"
    setenv CPPFLAGS "-DNDEBUG"
    setenv LDFLAGS ""
  endif

endif


#--------------------------------------------------------------------------
# Things needed to be set for running execution.
#--------------------------------------------------------------------------
if ($1 == 'run') then

  if ($system == 'macosx') then
    setenv DYLD_LIBRARY_PATH
    setenv DYLD_LIBRARY_PATH "$NETCDF_DIR"
  endif

endif
