!****f* cone2bc/main
! NAME 
!   CLD2BC
! PURPOSE
!   Superimpose plasma cloud on boundary values
! HISTORY
!   1999-07-31 DO NOAA/SEC Boulder
!*********************************************************************
PROGRAM CLD2BC
!=====================================================================
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comcld
  IMPLICIT NONE 
!---------------------------------------------------------------------
  CHARACTER :: lstart*10
  CHARACTER :: ldates(5)*10, ltimes(5)*5
  INTEGER :: iyear, imonth, iday, ihour, imin
  REAL(8) :: mjd_cld, mjd_start
  REAL(8) :: vud, vut, vuv, trot_day, trot_sec
  REAL(8) :: dfast, tfast
  REAL(8) :: zdt, ztimcld
  REAL(8) :: zshift, zx2, zx3, zrad, zzz
  REAL(8) :: z2cld, z3cld, zrcld
  REAL(8) :: timpas, time_cld(22,5)
  REAL(8) :: timcld(5), xcld(5)
  REAL(8) :: dcld(5), tcld(5), vcld(5), radcld(5), x2cld(5), x3cld(5)
  INTEGER :: ncld, ncloud, nrec
  INTEGER :: icld
  INTEGER :: jcloud, jc
  INTEGER :: j, j2, j3, jrec, irec, ireco
!---------------------------------------------------------------------
  NAMELIST/NAMDAT/                                        &
    lstart, ldates, ltimes,                               &
    ldir, lproj, lbnd, lini, lgrd,                        &
    dfast, tfast,                                         &
    x2cld, x3cld, radcld, xcld, vcld, dcld, tcld, ncld, ncloud
!---------------------------------------------------------------------
!L     1.        Clear variables and set parameters
!L       1.1       Characters
  lnum='0123456789'
!L       1.2       Plasma cloud parameters
  ldates(1)='1997-05-12'
  ltimes(1)='14:00'
  x2cld(1)=90d0
  x3cld(1)=180d0
  radcld(1)=25d0
  xcld(1)=1d0
  dcld(1)=6d0
  tcld(1)=1d0
  vcld(1)=650d0
  ncld=0
  ncloud=0
!L       1.3       Units
  vux1=1.496d11
  vux2=0.01745329252d0
  vux3=0.01745329252d0
  vutime=3600d0
  vuv=1d3
  vud=1d6
  vut=1d6
!L       1.4       Solar rotation
  trot_day=27.2753d0
  trot_sec=trot_day*24d0*3600d0
!L       1.5       Output directory and file names
  lfbnd='bnd.nc'
  mfbnd=6
!---------------------------------------------------------------------
!L     2.        Read in the JOB input parameters
!L       2.1       Read in heading
  read(*,'(A)',end=901,err=902) lajob
  write(*,'(a)') '=== CLD2BC: '//lajob
!L       2.2       Read in parameters
  read(*,NAMDAT,end=901,err=902)
!---------------------------------------------------------------------
!L     3.        Process the JOB input parameters
!L       5.1       Constants
  xmass=(1d0+4d0*xalpha)*cmass
!L       5.2       Convert to metric SI units
  dfast=xmass*dfast*vud
  tfast=tfast*vut
!L       3.1       Convert to metric SI units
  dcld(:)=dcld(:)*dfast
  tcld(:)=tcld(:)*tfast
  vcld(:)=vcld(:)*vuv
  x2cld(:)=x2cld(:)*vux2
  x3cld(:)=x3cld(:)*vux3
  radcld(:)=radcld(:)*vux2
!L       3.2       Calendar date
  iyear=0
  do j=1,10
    if(lnum(j:j).eq.lstart(1:1)) iyear=(j-1)*1000
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(2:2)) iyear=iyear+(j-1)*100
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(3:3)) iyear=iyear+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(4:4)) iyear=iyear+j-1
  enddo
  imonth=0
  do j=1,10
    if(lnum(j:j).eq.lstart(6:6)) imonth=imonth+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(7:7)) imonth=imonth+j-1
  enddo
  iday=0
  do j=1,10
    if(lnum(j:j).eq.lstart(9:9)) iday=iday+(j-1)*10
  enddo
  do j=1,10
    if(lnum(j:j).eq.lstart(10:10)) iday=iday+j-1
  enddo
!L       3.3       Reference date
  if(imonth.le.2) then
    iyear=iyear-1
    imonth=imonth+1+12
  else
    imonth=imonth+1
  endif
  mjd_start=int(365.25d0*iyear)+int(30.6001d0*imonth)+iday-679004d0
  mjd_start=mjd_start-int(0.01d0*iyear)+int(0.25d0*int(0.01d0*iyear))
!---------------------------------------------------------------------
!L     4.        Process the date and time
  do jcloud=1,ncloud
!L       4.1       Calendar date
    iyear=0
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(1:1)) iyear=(j-1)*1000
    enddo
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(2:2)) iyear=iyear+(j-1)*100
    enddo
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(3:3)) iyear=iyear+(j-1)*10
    enddo
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(4:4)) iyear=iyear+j-1
    enddo
    imonth=0
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(6:6)) imonth=imonth+(j-1)*10
    enddo
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(7:7)) imonth=imonth+j-1
    enddo
    iday=0
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(9:9)) iday=iday+(j-1)*10
    enddo
    do j=1,10
      if(lnum(j:j).eq.ldates(jcloud)(10:10)) iday=iday+j-1
    enddo
!L       4.2       Time
    ihour=0
    do j=1,10
      if(lnum(j:j).eq.ltimes(jcloud)(1:1)) ihour=ihour+(j-1)*10
    enddo
    do j=1,10
      if(lnum(j:j).eq.ltimes(jcloud)(2:2)) ihour=ihour+j-1
    enddo
    imin=0
    do j=1,10
      if(lnum(j:j).eq.ltimes(jcloud)(4:4)) imin=imin+(j-1)*10
    enddo
    do j=1,10
      if(lnum(j:j).eq.ltimes(jcloud)(5:5)) imin=imin+j-1
    enddo
!L       4.3       Reference date
    if(imonth.le.2) then
      iyear=iyear-1
      imonth=imonth+1+12
    else
      imonth=imonth+1
    endif
    mjd_cld=int(365.25d0*iyear)+int(30.6001d0*imonth)+iday-679004d0
    mjd_cld=mjd_cld-int(0.01d0*iyear)+int(0.25d0*int(0.01d0*iyear))
    mjd_cld=mjd_cld+float(ihour)/24.+float(imin)/1440.
    timcld(jcloud)=86400d0*(mjd_cld-mjd_start)
  enddo
  timcld(1)=0d0
!---------------------------------------------------------------------
!L     5.        Read in boundary values as the NetCDF file
  call RESBND
  x1l=x1bo_rec(1)
  if(refdate.ne.0d0) then
    timebo_rec(:)=timebo_rec(:)+86400d0*(refdate-mjd_start)
    refdate=mjd_start
  endif
!---------------------------------------------------------------------
!L     6.        Time levels for plasma cloud
!L       6.1       For plasma cloud-1
  if(ncld.eq.1) then
    icld=4
    do jcloud=1,ncloud
      timpas=x1l*2d0*radcld(jcloud)/vcld(jcloud)
!     time_cld(1,jcloud)=timcld(jcloud)-0.5d0*timpas-1.800d0
!     time_cld(2,jcloud)=timcld(jcloud)-0.5d0*timpas+1.800d0
!     time_cld(3,jcloud)=timcld(jcloud)+0.5d0*timpas-1.800d0
!     time_cld(4,jcloud)=timcld(jcloud)+0.5d0*timpas+1.800d0
      time_cld(1,jcloud)=timcld(jcloud)
      time_cld(2,jcloud)=timcld(jcloud)+1d0
      time_cld(3,jcloud)=timcld(jcloud)+timpas
      time_cld(4,jcloud)=timcld(jcloud)+timpas+1d0
    enddo
  endif
!L       6.2       For plasma cloud-2
  if(ncld.eq.2) then
    icld=22
    do jcloud=1,ncloud
      timpas=x1l*2d0*radcld(jcloud)/vcld(jcloud)
      if(xcld(jcloud).le.1d0) then
        zdt=xcld(jcloud)*timpas/dble(21)
        do j=1,icld
          time_cld(j,jcloud)=timcld(jcloud)-0.5d0*timpas+dble(j-1)*zdt
        enddo
      else
        zdt=timpas/dble(21)
        do j=1,icld/2
          time_cld(j,jcloud)=timcld(jcloud)-0.5d0*timpas+dble(j-1)*zdt
        enddo
        ztimcld=timcld(jcloud)+(xcld(jcloud)-1d0)*timpas
        do j=icld/2+1,icld
          time_cld(j,jcloud)=ztimcld-0.5d0*timpas+dble(j-1)*zdt
        enddo
      endif
    enddo
  endif
!L       6.3       Write basic information
  do jcloud=1,ncloud
    write(*,'(a7,i2,2x,a6,2(1pe11.3),2(2x,a5,1pe11.3))')           &
     'cloud =',jcloud,                                             &
     'time =',time_cld(1,jcloud)/3600.,time_cld(22,jcloud)/3600.,  &
     'rad =',radcld(jcloud)*180./3.141529,'vel =',vcld(jcloud)/1000.
  enddo
!---------------------------------------------------------------------
!L     7.        Sort time levels
  timebn_rec=timebo_rec
  irec=nbnd
  do jcloud=1,ncloud
    do jc=1,icld
      ireco=1
      irec=1
      timeba_rec=timebn_rec
      do while(ireco.lt.6+jcloud*icld-1)
        ireco=ireco+1
        irec=irec+1
        if((time_cld(jc,jcloud).ge.timeba_rec(ireco-1)).and. &
          (time_cld(jc,jcloud).lt.timeba_rec(ireco))) then
          timebn_rec(irec)=time_cld(jc,jcloud)
          irec=irec+1
        endif
        timebn_rec(irec)=timeba_rec(ireco)
      enddo
    enddo
  enddo
  nrec=irec
!---------------------------------------------------------------------
!L     8.        Time-dependent boundary conditions
!L       8.1       Get coordinates
  x1bo=x1bo_rec(1)
  x2bo(:)=x2bo_rec(:,1)
  x3bo(:)=x3bo_rec(:,1)
  x1boh=x1boh_rec(1)
  x2boh(:)=x2boh_rec(:,1)
  x3boh(:)=x3boh_rec(:,1)
  x1bn=x1bo
  x2bn(:)=x2bo(:)
  x3bn(:)=x3bo(:)
  x1bnh=x1boh
  x2bnh(:)=x2boh(:)
  x3bnh(:)=x3boh(:)
!L       8.2       Transient disturbances
  irec=1
  do jrec=1,nrec
    time=timebn_rec(jrec)
    do while((timebo_rec(irec).le.time).and.(irec.le.6))
      irec=irec+1
    enddo
    if(irec.gt.1) irec=irec-1
    zshift=-6.2831854d0*(time-timebo_rec(irec))/trot_sec
    do while(zshift.gt.6.2831854d0)
      zshift=zshift-6.2831854d0
    enddo
    do while(zshift.lt.0d0)
      zshift=zshift+6.2831854d0
    enddo
!L       8.3       Get original boundary values
    dbo(:,:)=dbo_rec(:,:,irec)
    tbo(:,:)=tbo_rec(:,:,irec)
    v1bo(:,:)=v1bo_rec(:,:,irec)
    b1bo(:,:)=b1bo_rec(:,:,irec)
    b3bo(:,:)=b3bo_rec(:,:,irec)
#ifdef hydcon
    dpbo(:,:)=dpbo_rec(:,:,irec)
#endif
#ifdef mhdcon
    bpbo(:,:)=bpbo_rec(:,:,irec)
#endif
!L       8.4       Interpolate values onto new grid
    call INTERP(dbo,x2bo,x3bo,nbb2,nbb3,dbn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(tbo,x2bo,x3bo,nbb2,nbb3,tbn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(v1bo,x2bo,x3bo,nbb2,nbb3,v1bn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(b1bo,x2bo,x3bo,nbb2,nbb3,b1bn,x2bn,x3bn,nbb2,nbb3,zshift)
    call INTERP(b3bo,x2bo,x3bo,nbb2,nbb3,b3bn,x2bn,x3bn,nbb2,nbb3,zshift)
#ifdef mhdcon
    call INTERP(bpbo,x2bo,x3bo,nbb2,nbb3,bpbn,x2bn,x3bn,nbb2,nbb3,zshift)
#endif
#ifdef hydcon
!   dpbn(:,:)=0d0
    do j3=1,nbb3
      do j2=1,nbb2
        dpbn(j2,j3)=0d0
      enddo
    enddo
#endif
!L       8.5       Plasma cloud parameters
    do jcloud=1,ncloud
      if((timebn_rec(jrec).gt.time_cld(1,jcloud)).and. &
        (timebn_rec(jrec).lt.time_cld(icld,jcloud))) then
        z2cld=x2cld(jcloud)
        z3cld=x3cld(jcloud)
        if(ncld.eq.1) then
          zrcld=radcld(jcloud)
        else
          zzz=2d0*(timcld(jcloud)-timebn_rec(jrec))/ &
            (time_cld(icld,jcloud)-time_cld(1,jcloud))
          zrcld=radcld(jcloud)*sqrt(1d0-abs(zzz))
        endif
        do j3=1,nbb3
          do j2=1,nbb2
            zx2=x2bn(j2)-z2cld
            zx3=x3bn(j3)-z3cld
            zrad=sqrt(zx2**2+zx3**2)
            if(zrad.le.zrcld) then
              if(dbn(j2,j3).lt.dcld(jcloud)) dbn(j2,j3)=dcld(jcloud)
              if(tbn(j2,j3).lt.tcld(jcloud)) tbn(j2,j3)=tcld(jcloud)
              if(v1bn(j2,j3).lt.vcld(jcloud)) v1bn(j2,j3)=vcld(jcloud)
#ifdef hydcon
              dpbn(j2,j3)=dbn(j2,j3)
#endif
            endif
          enddo
        enddo
      endif
    enddo
!L       8.6       Write NetDCF data
    call OUTBND(jrec)
  enddo
!---------------------------------------------------------------------
!L     9.        Stop
  write(*,'(a)') '=== END OF CLD2BC'
STOP
  901 continue
      write (*,'(a)') '*** End of input file'
      goto 903
  902 continue
      write (*,'(a)') '*** Error in input file'
  903 continue
      write (*,'(a)') '*** Error in program CLD2BC'
STOP
!---------------------------------------------------------------------
END PROGRAM CLD2BC


!****f* e0pro/main
! NAME 
!   RESBND
! PURPOSE
!   Resume at time level
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESBND
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comcld
  USE comout
  USE netcdf
  IMPLICIT NONE
  INTEGER :: ib1,ib2,ib3,ib1h,ib2h,ib3h
  INTEGER :: iret,ifile,id
  CHARACTER(len=nf90_max_name) :: iname
!---------------------------------------------------------------------
  INTEGER, DIMENSION(2) :: istart2,  icount2
  DATA  istart2/1,1/,  icount2/nbb2,1/
  INTEGER, DIMENSION(2) :: istart3,  icount3
  DATA  istart3/1,1/,  icount3/nbb3,1/
  INTEGER, DIMENSION(3) :: istart,  icount
  DATA  istart/1,1,1/,  icount/nbb2,nbb3,1/
!---------------------------------------------------------------------
!i     1.        Open the file
  iret=NF90_OPEN(lfbnd,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'boundary',labnd)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'refdate_mjd',refdate)
  if(myproc.eq.0) then
    if(nstep.eq.0) write(*,*) '   reference date MJD =',refdate
  endif
!---------------------------------------------------------------------
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,ib1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,ib2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,ib3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ib1h)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ib2h)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ib3h)
  iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,nbnd)
!---------------------------------------------------------------------
!L     3.        Get time-dependent values
  icount2(2)=nbnd
  icount3(2)=nbnd
  icount(3)=nbnd
!L       3.2       Coordinates
  iret=NF90_GET_VAR(ifile,1,x1bo_rec,(/1/),(/nbnd/))
  iret=NF90_GET_VAR(ifile,2,x2bo_rec,(/1,1/),(/ib2,nbnd/))
  iret=NF90_GET_VAR(ifile,3,x3bo_rec,(/1,1/),(/ib3,nbnd/))
  iret=NF90_GET_VAR(ifile,4,x1boh_rec,(/1/),(/nbnd/))
  iret=NF90_GET_VAR(ifile,5,x2boh_rec,(/1,1/),(/ib2h,nbnd/))
  iret=NF90_GET_VAR(ifile,6,x3boh_rec,(/1,1/),(/ib3h,nbnd/))
!L       3.3       Time
  iret=NF90_GET_VAR(ifile,7,timebo_rec,(/1/),(/nbnd/))
!L       3.4       Variables
  iret=NF90_GET_VAR(ifile,10,dbo_rec,istart,icount)
  iret=NF90_GET_VAR(ifile,11,tbo_rec,istart,icount)
  iret=NF90_GET_VAR(ifile,12,v1bo_rec,istart,icount)
  iret=NF90_GET_VAR(ifile,13,v2bo_rec,istart,icount)
  iret=NF90_GET_VAR(ifile,14,v3bo_rec,istart,icount)
  id=14
#ifdef mhd
    iret=NF90_GET_VAR(ifile,15,b1bo_rec,istart,icount)
    iret=NF90_GET_VAR(ifile,15,b2bo_rec,istart,icount)
    iret=NF90_GET_VAR(ifile,17,b3bo_rec,istart,icount)
    id=17
#endif
#ifdef hydcon
    id=id+1
    iret=NF90_GET_VAR(ifile,id,dpbo_rec,istart,icount)
#endif
#ifdef mhdcon
    id=id+1
    iret=NF90_GET_VAR(ifile,id,bpbo_rec,istart,icount)
#endif
!---------------------------------------------------------------------
!L     4.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     5.        Write
  if(myproc.eq.0) then
    write(*,'(4x,a6,i4,1pe12.4,4i4)') 'resbnd',nbnd,real(timebn), &
      nb1,nb2,nb3
  endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RESBND
