!****f* wsa2bc/main
! NAME 
!   DATE2BC
! PURPOSE
!   Date manipulation
! HISTORY
!   2005-08-31 DO NOAA/SEC Boulder
!*********************************************************************
SUBROUTINE DATE2BC(lcrnum,lstart,mjd_cr,refdate,lrefdate,lreftime)
!=====================================================================
  IMPLICIT NONE 
  CHARACTER :: lcrnum*4, lstart*10, lrefdate*10, lreftime*8
  CHARACTER :: lnum*10
  INTEGER :: idata,   istart
  INTEGER :: icrnum
  INTEGER :: iyear, imonth, iday
  INTEGER :: iyear0, iyear00, iyear000, imonth0, iday0
  INTEGER :: ihour, imin, isec, ihour0, imin0, isec0
  REAL(8) :: mjd_cr, refdate
  INTEGER :: n, i, j, l
!---------------------------------------------------------------------
!L     1.        Corresponding numerical values
  lnum='0123456789'
!L       1.1       Carrington rotation number
  idata=len_trim(lcrnum)
  if(idata.gt.0) then
    call CHANUM(lcrnum,4,icrnum)
  else
    icrnum=0
  endif
!L       1.2       Calendar date
  istart=len_trim(lstart)
  if(istart.gt.0) then
    call CHANUM(lstart(1:4),4,iyear)
    call CHANUM(lstart(6:7),2,imonth)
    call CHANUM(lstart(9:10),2,iday)
  else
    iyear=0
    imonth=0
    iday=0
  endif
!---------------------------------------------------------------------
!L     1.        Modified Julian Date at the beginning of CR
!L       1.1        From known CR number
  if(idata.gt.0) then
    mjd_cr=(dble(icrnum)-1750d0)*27.2753d0+45871.41d0
    refdate=mjd_cr
    l=int(refdate+2468569.5d0)
    n=4*l/146097
    l=l-(146097*n+3)/4
    i=(4000*(l+1))/1461001
    l=l-1461*i/4+31
    j=80*l/2447
    iday=l-2447*j/80
    l=j/11
    imonth=j+2-12*l
    iyear=100*(n-49)+i
  endif
!L       1.1        From known calendar date
  if(istart.gt.0) then
    if(imonth.le.2) then
      iyear=iyear-1
      imonth=imonth+1+12
    else
      imonth=imonth+1
    endif
    refdate=int(365.25d0*iyear)+int(30.6001d0*imonth)+iday-679004d0
    refdate=refdate-int(0.01d0*iyear)+int(0.25d0*int(0.01d0*iyear))
  endif
!---------------------------------------------------------------------
!L     2.        Reference date
!L       2.1        From the start date
  if(istart.gt.0) then
    lrefdate=lstart
    lreftime='00:00:00'
  else
!L       2.2        From the CR beginning date
    iyear000=int(iyear)/1000
    iyear=iyear-1000*iyear000
    iyear00=iyear/100
    iyear=iyear-100*iyear00
    iyear0=iyear/10
    iyear=iyear-10*iyear0
    imonth0=imonth/10
    imonth=imonth-10*imonth0
    iday0=iday/10
    iday=iday-10*iday0
    iyear000=iyear000+1
    iyear00=iyear00+1
    iyear0=iyear0+1
    iyear=iyear+1
    imonth0=imonth0+1
    imonth=imonth+1
    iday0=iday0+1
    iday=iday+1
    lrefdate=lnum(iyear000:iyear000)//lnum(iyear00:iyear00)// &
      lnum(iyear0:iyear0)//lnum(iyear:iyear)//'-'//           &
      lnum(imonth0:imonth0)//lnum(imonth:imonth)//'-'//       &
      lnum(iday0:iday0)//lnum(iday:iday)
    ihour=24*int(refdate-int(refdate))
    imin=1440*int(refdate-int(refdate))-ihour*60
    isec=86400*int(refdate-int(refdate))-ihour*3600-imin*60
    ihour0=ihour/10
    imin0=imin/10
    isec0=isec/10
    ihour=ihour-ihour0*10
    imin=imin-imin0*10
    isec=isec-isec0*10
    ihour0=ihour0+1
    ihour=ihour+1
    imin0=imin0+1
    imin=imin+1
    isec0=isec0+1
    isec=isec+1
    lreftime=lnum(ihour0:ihour0)//lnum(ihour:ihour)//':'//  &
      lnum(imin0:imin0)//lnum(imin:imin)//':'//             &
      lnum(isec0:isec0)//lnum(isec:isec)
  endif
!---------------------------------------------------------------------
!L     9.        Return
RETURN
END SUBROUTINE DATE2BC


!****f* e0pro/main
! NAME 
!   NUMCHA
! PURPOSE
!   Convert numeric value to chartacter string
! HISTORY
!   1981-09-01 DO VUVH Bratislava
!*********************************************************************
SUBROUTINE NUMCHA(KNUM,KLEN,KCHAR)
!---------------------------------------------------------------------
!  KNUM            Numerical value                               L  IN
!  KLEN            Character length                              I  IN
!  KCHAR(KLEN)     Character string                              LA OUT
!---------------------------------------------------------------------
  IMPLICIT NONE
  INTEGER :: knum, klen 
  CHARACTER(len=klen) :: kchar 
  CHARACTER(10) :: lnum 
  INTEGER :: i0, i, inum, j
!---------------------------------------------------------------------
  lnum='0123456789'
  inum=knum
  i0=klen
  do j=1,klen
    i0=i0-1
    i=inum/10**i0
    inum=inum-i*10**i0
    kchar(j:j)=lnum(i+1:i+1)
  enddo
!---------------------------------------------------------------------
RETURN
END SUBROUTINE NUMCHA


!****f* e0pro/main
! NAME 
!   CHANUM
! PURPOSE
!   Convert chartacter string to numeric value
! HISTORY
!   1981-09-01 DO VUVH Bratislava
!*********************************************************************
SUBROUTINE CHANUM(KCHAR,KLEN,KNUM)
!---------------------------------------------------------------------
!  KCHAR(KLEN)     Character string                              LA IN 
!  KLEN            Character length                              I  IN
!  KNUM            Numerical value                               L  OUT
!---------------------------------------------------------------------
  IMPLICIT NONE
  INTEGER :: knum, klen 
  CHARACTER(len=klen) :: kchar 
  CHARACTER(10) :: lnum 
  INTEGER :: i0, ii, j, jj
!---------------------------------------------------------------------
  lnum='0123456789'
  knum=0
  i0=klen
  do j=1,klen
    i0=i0-1
    ii=1
    do jj=1,10
      if(kchar(j:j).eq.lnum(jj:jj)) ii=jj
    enddo
    knum=knum+(ii-1)*10**i0
  enddo
!---------------------------------------------------------------------
RETURN
END SUBROUTINE CHANUM
