!****f* e0mas/inter_comm
! NAME 
!   INTER_COMM
! DESCRIPTION
!   Variables to describe and receive data from MAS using InterComm.
! HISTORY
!   2005-07-06 DO NOAA/SEC
!*********************************************************************
#ifdef ICOMM
MODULE INTER_COMM
!---------------------------------------------------------------------
!L       I.1       InterComm code identifiers
  INTEGER :: mas_id, enlil_id
!L       I.2       InterComm data descriptors
  INTEGER :: ic_1d_1_desc,    ic_1d_1_region,    ic_1d_1_sched
  INTEGER :: ic_1d_2_desc,    ic_1d_2_region,    ic_1d_2_sched
  INTEGER :: ic_1d_ntm1_desc, ic_1d_ntm1_region, ic_1d_ntm1_sched
  INTEGER :: ic_1d_np_desc,   ic_1d_np_region,   ic_1d_np_sched
  INTEGER :: ic_2d_ntp_desc,  ic_2d_ntp_region,  ic_2d_ntp_sched
!L       I.3       MAS constant data
  REAL(8), ALLOCATABLE :: theta_mas(:), phi_mas(:)
  REAL(8) :: r_mas, gamma_mas
  INTEGER :: ntheta_mas, nphi_mas
!L       I.4       MAS time-dependent data
  REAL(8), ALLOCATABLE :: br_mas(:,:), bt_mas(:,:) ,bp_mas(:,:),  &
                          vr_mas(:,:), vt_mas(:,:), vp_mas(:,:),  &
                          d_mas(:,:),  t_mas(:,:)
  REAL(8) :: time_mas
!---------------------------------------------------------------------
END MODULE INTER_COMM
#endif


!****f* e0mas/mascon
! NAME 
!   MASCON
! DESCRIPTION
!   Receive constant values from MAS using InterComm
! HISTORY
!   2005-07-06 DO NOAA/SEC
!*********************************************************************
#ifdef ICOMM
SUBROUTINE MASCON
!---------------------------------------------------------------------
  USE inter_comm
  USE commax
!---------------------------------------------------------------------
  IMPLICIT NONE
  REAL(8) :: rparams(2)
  INTEGER :: blocks_1d(1,2,1), ntp(2), ierr
  REAL(8) :: dummy(2)
  INTEGER :: idummy(2)
!---------------------------------------------------------------------
!L     1.        Initialize the connection between MAS and ENLIL
write(*,*) 'init-1'
  call IC_INIT('ENLIL',1,0,enlil_id)
write(*,*) 'init-0'
write(*,*) 'wait-1'
  call IC_WAIT('MAS',1,mas_id)
write(*,*) 'wait-0'
!---------------------------------------------------------------------
!L     2.        Describe the data distribution for the parameters
!L       2.1       Build the data descriptors
write(*,*) 'bdecomp_desc-1'
! blocks_1d(1,1:2,1)=(/1,1/)
! call IC_CREATE_BDECOMP_DESC(1,blocks_1d,0,1,ic_1d_1_desc)
  blocks_1d(1,1:2,1)=(/1,2/)
  call IC_CREATE_BDECOMP_DESC(1,blocks_1d,0,1,ic_1d_2_desc)
write(*,*) 'bdecomp_desc-0'
!L       2.2       Define the data blocks
write(*,*) 'block_region-1'
! call IC_CREATE_BLOCK_REGION(1,1,1,1,ic_1d_1_region)
  call IC_CREATE_BLOCK_REGION(1,1,2,1,ic_1d_2_region)
write(*,*) 'block_region-0'
!L       2.3       Define the communication schedules
write(*,*) 'compute_schedule-1'
! call IC_COMPUTE_SCHEDULE(enlil_id,mas_id,ic_1d_1_desc,    &
!                          ic_1d_1_region,1,ic_1d_1_sched)
  call IC_COMPUTE_SCHEDULE(enlil_id,mas_id,ic_1d_2_desc,    &
                           ic_1d_2_region,1,ic_1d_2_sched)
write(*,*) 'compute_schedule-0'
!---------------------------------------------------------------------
!L     3.        Receive the parameters
!L       3.1       Receive the mesh dimensions
write(*,*) 'recv-int-1'
  call IC_RECV_INT(mas_id,ic_1d_2_sched,ntp,0,ierr)
write(*,*) 'recv-int-0 - ntp',ntp
  if (ierr.eq.-1) goto 900
  ntheta_mas=ntp(1)
  nphi_mas=ntp(2)
!L       3.2       Receive the real parameters
write(*,*) 'recv-double-1'
  call IC_RECV_DOUBLE(mas_id,ic_1d_2_sched,rparams,0,ierr)
write(*,*) 'recv-double-0 - rparams',rparams
  if (ierr.eq.-1) goto 900
  r_mas=rparams(1)
  gamma_mas=rparams(2)
!---------------------------------------------------------------------
!L     4.        Allocate arrays for receiving the mesh
write(*,*) 'allocate-1'
  ALLOCATE(theta_mas(ntheta_mas))
  ALLOCATE(phi_mas(nphi_mas))
write(*,*) 'allocate-0'
!---------------------------------------------------------------------
!L     5.        Describe the data distribution for the mesh
!L       5.1       Build the data descriptors
write(*,*) 'bdecomp_desc-1'
  blocks_1d(1,1:2,1)=(/1,ntheta_mas/)
  call IC_CREATE_BDECOMP_DESC(1,blocks_1d,0,1,ic_1d_ntm1_desc)
write(*,*) 'bdecomp_desc-0'
write(*,*) 'bdecomp_desc-1'
  blocks_1d(1,1:2,1)=(/1,nphi_mas/)
  call IC_CREATE_BDECOMP_DESC(1,blocks_1d,0,1,ic_1d_np_desc)
write(*,*) 'bdecomp_desc-0'
!L       5.2       Define the data blocks
write(*,*) 'block_region-1'
  call IC_CREATE_BLOCK_REGION(1,1,ntheta_mas,1,ic_1d_ntm1_region)
write(*,*) 'block_region-0'
write(*,*) 'block_region-1'
  call IC_CREATE_BLOCK_REGION(1,1,nphi_mas,1,ic_1d_np_region)
write(*,*) 'block_region-0'
!L       5.3       Define the communication schedules
write(*,*) 'compute_schedule-1'
  call IC_COMPUTE_SCHEDULE(enlil_id,mas_id,ic_1d_ntm1_desc, &
                           ic_1d_ntm1_region,1,ic_1d_ntm1_sched)
write(*,*) 'compute_schedule-0'
write(*,*) 'compute_schedule-1'
  call IC_COMPUTE_SCHEDULE(enlil_id,mas_id,ic_1d_np_desc,   &
                           ic_1d_np_region,1,ic_1d_np_sched)
write(*,*) 'compute_schedule-0'
!---------------------------------------------------------------------
!L     6.        Receive meshes
write(*,*) 'recv-double-1'
  call IC_RECV_DOUBLE(mas_id,ic_1d_ntm1_sched,theta_mas,0,ierr)
write(*,*) 'recv-double-0 - theta',theta_mas
  if (ierr.eq.-1) goto 900
write(*,*) 'recv-double-1'
  call IC_RECV_DOUBLE(mas_id,ic_1d_np_sched,phi_mas,0,ierr)
write(*,*) 'recv-double-0 - phi',phi_mas
  if (ierr.eq.-1) goto 900
!---------------------------------------------------------------------
RETURN
  900 continue
! InterComm error
  write (*,'(a)') '*** Error in subroutine MASCON'
  write (*,*) 'ierr = ',ierr
  call JOBEND
!---------------------------------------------------------------------
RETURN
END SUBROUTINE MASCON
#endif


!****f* e0mas/mastim
! NAME 
!   MASTIM
! DESCRIPTION
!   Receive time-dependent values from MAS using InterComm
! INPUTS
!   kmode = 1,2 for initial, periodic action   
! HISTORY
!   2005-07-06 DO NOAA/SEC
!*********************************************************************
#ifdef ICOMM
SUBROUTINE MASTIM(kmode)
!---------------------------------------------------------------------
  USE inter_comm
  USE commax
  USE combnd
!---------------------------------------------------------------------
  IMPLICIT NONE
  INTEGER :: blocks_2d(1,2,2), lower(2), upper(2), stride(2)
  INTEGER :: kmode, last_data, ierr
  REAL(8) :: dummy(2)
  INTEGER :: idummy(2)
!---------------------------------------------------------------------
!L     1.        Check if it is time to send data
write(*,*) 'MASCON',kmode,last_data
  if (last_data.ne.0) then
    return
  endif 
!---------------------------------------------------------------------
!L     2.        Describe the data distribution for the boundary data
  if (kmode.eq.1) then
!L       2.1       Build the data descriptors
    blocks_2d(1,1:2,1)=(/1,ntheta_mas/)
    blocks_2d(1,1:2,2)=(/1,nphi_mas/)
write(*,*) 'bdecomp_desc-1'
    call IC_CREATE_BDECOMP_DESC(2,blocks_2d,0,1,ic_2d_ntp_desc)
write(*,*) 'bdecomp_desc-0'
!L       2.2       Define the data blocks
    stride=(/1,1/)
    lower=(/1,1/)
    upper=(/ntheta_mas,nphi_mas/)
write(*,*) 'block_region-1'
    call IC_CREATE_BLOCK_REGION(2,lower,upper,stride,ic_2d_ntp_region)
write(*,*) 'block_region-0'
!L       2.3       Define the communication schedules
write(*,*) 'compute_schedule-1'
    call IC_COMPUTE_SCHEDULE(enlil_id,mas_id,ic_2d_ntp_desc,  &
                             ic_2d_ntp_region,1,ic_2d_ntp_sched)
write(*,*) 'compute_schedule-0'
!---------------------------------------------------------------------
!L     3.        Allocate arrays
write(*,*) 'allocate-1'
    ALLOCATE(br_mas(ntheta_mas,nphi_mas))
    ALLOCATE(bt_mas(ntheta_mas,nphi_mas))
    ALLOCATE(bp_mas(ntheta_mas,nphi_mas))
    ALLOCATE(vr_mas(ntheta_mas,nphi_mas))
    ALLOCATE(vt_mas(ntheta_mas,nphi_mas))
    ALLOCATE(vp_mas(ntheta_mas,nphi_mas))
    ALLOCATE(d_mas(ntheta_mas,nphi_mas))
    ALLOCATE(t_mas(ntheta_mas,nphi_mas))
write(*,*) 'allocate-0'
  endif
!---------------------------------------------------------------------
!L     3.        Receive time-dependent data from MAS
!L       3.1       Receive the last data flag
write(*,*) 'recv-int-1'
  call IC_RECV_INT(mas_id,ic_1d_2_sched,idummy,0,ierr)
write(*,*) 'recv-int-0 - idummy',idummy
  last_data=idummy(1)
  if (ierr.eq.-1) goto 900
! No data after last_data=0
!L       3.2       Receive the time
write(*,*) 'recv-double-1'
  call IC_RECV_DOUBLE(mas_id,ic_1d_2_sched,dummy,0,ierr)
write(*,*) 'recv-double-0 - dummy',dummy
  time_mas=dummy(1)
  if (ierr.eq.-1) goto 900
!L       3.3       Receive the 2D arrays
write(*,*) 'recv-double-1'
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,br_mas,0,ierr)
write(*,*) 'recv-double-0 br_mas',br_mas
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,bt_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,bp_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,vr_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,vt_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,vp_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,d_mas,0,ierr)
  if (ierr.eq.-1) goto 900
  call IC_RECV_DOUBLE(mas_id,ic_2d_ntp_sched,t_mas,0,ierr)
  if (ierr.eq.-1) goto 900
!---------------------------------------------------------------------
!L     4.        Interpolate values from MAS grid to ENLIL grid
  call INTERP(d_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              dbo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(t_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              tbo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(vr_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              v1bo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(vt_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              v2bo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(vp_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              v3bo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(br_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              b1bo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(bt_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              b2bo,x2bo,x3bo,nbb2,nbb3,0d0)
  call INTERP(bp_mas,theta_mas,phi_mas,ntheta_mas,nphi_mas, &
              b3bo,x2bo,x3bo,nbb2,nbb3,0d0)
!---------------------------------------------------------------------
RETURN
  900 continue
! InterComm error
  write (*,'(a)') '*** Error in subroutine MASTIM'
  write (*,*) 'ierr = ',ierr
  call JOBEND
RETURN
END SUBROUTINE MASTIM
#endif


!****f* e0ic/interp
! NAME 
!   INTERP
! DESCRIPTION
!   Interpolate boundary values
! HISTORY
!   2004-03-16 DO NOAA/SEC
!*********************************************************************
#ifdef ICOMM
SUBROUTINE INTERP(uold,x2old,x3old,n2old,n3old,unew,x2new,x3new, &
  n2new,n3new,zshift)
  IMPLICIT NONE
  REAL(8) :: uold(n2old,n3old), x2old(n2old), x3old(n3old)
  REAL(8) :: unew(n2new,n3new), x2new(n2new), x3new(n3new)
  REAL(8) :: x2m(n2new), x2p(n2new), x3m(n3new), x3p(n3new)
  REAL(8) :: z2m, z2p, z3m, z3p, zarea, zshift, zz3
  INTEGER :: n2m(n2new), n2p(n2new), n3m(n3new), n3p(n3new)
  INTEGER :: i2m, i2p, i3m, i3p, j2, j3, n2old, n2new, n3old, n3new
!---------------------------------------------------------------------
!L     1.        Set geometry coefficients
!L       1.1       X2 coefficients
  do j2=1,n2new
    i2p=2
    do while((x2new(j2).gt.x2old(i2p)).and.(i2p.lt.n2old))
      i2p=i2p+1
    enddo
    i2m=i2p-1
    n2m(j2)=i2m
    n2p(j2)=i2p
    x2m(j2)=(x2new(j2)-x2old(i2m))
    x2p(j2)=(x2old(i2p)-x2new(j2))
  enddo
!L       1.2       X3 coefficients
  do j3=1,n3new
    i3p=2
    zz3=x3new(j3)+zshift
    if(zz3.gt.6.2831854d0) zz3=zz3-6.2831854d0
    do while((zz3.gt.x3old(i3p)).and.(i3p.lt.n3old))
      i3p=i3p+1
    enddo
    i3m=i3p-1
    n3m(j3)=i3m
    n3p(j3)=i3p
    x3m(j3)=(zz3-x3old(i3m))
    x3p(j3)=(x3old(i3p)-zz3)
  enddo
!---------------------------------------------------------------------
!L     2.        Interpolate boundary values
  do j3=1,n3new
    i3m=n3m(j3)
    i3p=n3p(j3)
    z3m=x3m(j3)
    z3p=x3p(j3)
    do j2=1,n2new
      i2m=n2m(j2)
      i2p=n2p(j2)
      z2m=x2m(j2)
      z2p=x2p(j2)
      zarea=(z2m+z2p)*(z3m+z3p)
      unew(j2,j3)=(z2p*z3p*uold(i2m,i3m)+z2p*z3m*uold(i2m,i3p) &
                  +z2m*z3p*uold(i2p,i3m)+z2m*z3m*uold(i2p,i3p))/zarea
    enddo
  enddo
!---------------------------------------------------------------------
RETURN
END SUBROUTINE INTERP
#endif
  
subroutine aa
return
end
