!****f* e0pro/main
! NAME 
!   MASTER
! PURPOSE
!   The OLYMPUS main control
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE MASTER 
!---------------------------------------------------------------------
  USE comoly
  USE comphy
  USE commet
  IMPLICIT NONE
!---------------------------------------------------------------------
!L     1.        Begin the JOB
!L       1.1       Preset the JOB parameters
  call JOBSET
!L       1.2       Clear variables and arrays
  call JOBCLR
!L       1.2       Read in the JOB parameters
  call JOBPAR
!L       1.3       Set the JOB auxiliary parameters
  call JOBAUX
!L       1.4       Check the JOB parameters
  call JOBCHE
!---------------------------------------------------------------------
!L     2.        Prologue
!L       2.1       Clear variables and arrays
  call RUNCLR
!L       2.2       Set default values
  call RUNPRE
!L       2.3       Define data specific to run
  call RUNPAR
!L       2.4       Check input data
  call RUNCHE
!L       2.5       Set auxiliary values
  call RUNAUX
!---------------------------------------------------------------------
!L     3.        Initialize the run
!L       3.1       Initial conditions
  call RUNINI
!L       3.5       Initial output
  call RUNOUT(1)
!---------------------------------------------------------------------
!L     4.        Main calculation loop
! 400 continue
!L       4.1       Step on the calculation
! call RUNCAL
!L       4.3       Periodic production of output
! call RUNOUT(2)
!L       4.4       Test for completion
! call RUNTST
! if(.not.nlend) goto 400
!---------------------------------------------------------------------
!L     4.        Main calculation loop
  do while(.not.nlend)
!L       4.1       Step on the calculation
    call RUNCAL
!L       4.3       Periodic production of output
    call RUNOUT(2)
!L       4.4       Test for completion
    call RUNTST
  enddo
!---------------------------------------------------------------------
!L     5.        Epilogue
!L       5.1       Final output
  call RUNOUT(3)
!L       5.3       Terminate the JOB
  call JOBEND
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
END SUBROUTINE MASTER


!****f* e0pro/main
! NAME 
!   JOBSET
! PURPOSE
!   Preset the JOB parameters
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBSET
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  IMPLICIT NONE
  INTEGER :: j, i, i0, i00, i000
#ifdef PARAMESH
  INTEGER :: maxprocs, shmem_my_pe, shmem_n_pes
#endif
!---------------------------------------------------------------------
!L     1.        Date and time
  stime=zero
  call RUNTIM
  stime=cptime
  altime=zero
!---------------------------------------------------------------------
!L     2.        Initialize the PARAMESH
#ifdef PARAMESH
#ifdef SGI_SHMEM
  call COMM_START(maxprocs,nprocs,mype)
#endif
  call AMR_INITIALIZE
  mype=shmem_my_pe()
  nprocs=shmem_n_pes()
  if(mype.eq.0) write(*,*) 'Running on ', nprocs, ' processors'
  myproc=mype
  lrefine_min=4
  lrefine_max=8
  lrefine_tim=2
#endif
!---------------------------------------------------------------------
!L     3.        Set characters
  do j=1,120
    lblank(j:j)=' '
  enddo
  lalph='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  lnum='0123456789'
!---------------------------------------------------------------------
!L     4.        Set characters
  i000=myproc/1000
  i00=(myproc-i000*1000)/100
  i0=(myproc-i000*1000-i00*100)/10
  i=myproc-i000*1000-i00*100-i0*10
  i000=i000+1
  i00=i00+1
  i0=i0+1
  i=i+1
  lproc=lnum(i000:i000)//lnum(i00:i00)//lnum(i0:i0)//lnum(i:i)
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
END SUBROUTINE JOBSET


!****f* e0pro/main
! NAME 
!   JOBCLR
! PURPOSE
!   Clear variables and arrays
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBCLR
!---------------------------------------------------------------------
  USE comoly
  IMPLICIT NONE
!---------------------------------------------------------------------
!L     3.        Clear labels
!L       3.1       Principle directory
  ldir=lblank(1:52)
!L       3.1       Project
  lproj=lblank(1:8)
!L       3.1       Job
  lajob=lblank(1:80)
!L       3.1       Case
  lcase=lblank(1:30)
!L       3.1       Code
  lcode=lblank(1:16)
  lnumo=lblank(1:8)
  lprocs=lblank(1:4)
  levels=lblank(1:2)
!L       3.2       Run
  lpar=lblank(1:8)
  lrun=lblank(1:52)
  lapar=lblank(1:80)
!L       3.3       Grid data
  lgrd=lblank(1:12)
  lagrd=lblank(1:80)
  lfgrd=lblank(1:120)
  mfgrd=0
!L       3.3       Initial data
  lin=lblank(1:52)
  lini=lblank(1:8)
  laini=lblank(1:80)
  lfini=lblank(1:120)
  mfini=0
!L       3.4       Boundary data
  lbnd=lblank(1:8)
  labnd=lblank(1:80)
  lfbnd=lblank(1:120)
  mfbnd=0
!L       3.5       Passage data
  lpas=lblank(1:8)
  lapas=lblank(1:80)
  lfpas=lblank(1:120)
  mfpas=0
!L       3.6       Output data
  lfx1s=lblank(1:120)
  lfx2s=lblank(1:120)
  lfx3s=lblank(1:120)
  lfevg=lblank(1:120)
  lfevh=lblank(1:120)
  lfevp=lblank(1:120)
  lfevl=lblank(1:120)
  lfpass=lblank(1:120)
  lftim=lblank(1:120)
  lfval=lblank(1:120)
  mfx1s=0
  mfx2s=0
  mfx3s=0
  mfevo=0
  mfpass=0
  mftim=0
  mfval=0
!L       3.3       Restart data
  lres=lblank(1:52)
  lares=lblank(1:80)
  lfres=lblank(1:120)
  lffin=lblank(1:120)
  mfres=0
  mffin=0
!L       3.3       Reference date
  lrefdate=lblank(1:10)
  lreftime=lblank(1:8)
!---------------------------------------------------------------------
!L     4.        Clear variables
  refdate=0d0
  nlend=.false.
  nlerr=.false.
  nlres=.false.
  nlstop=.false.
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
END SUBROUTINE JOBCLR


!****f* e0pro/main
! NAME 
!   JOBPAR
! PURPOSE
!   Read in the JOB parameters
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBPAR
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  IMPLICIT NONE
#ifdef MPI
  INTEGER :: ie
  include 'mpif.h'
#endif
!---------------------------------------------------------------------
  NAMELIST/NAMJOB/  &
    ALTIME, NSLABS, NLRES, &
#ifdef PARAMESH
    LREFINE_MIN, LREFINE_MAX, LREFINE_TIM, LREFINE_INI, &
#endif
    LDIR, LPROJ, LPROCS, LEVELS, LNUMO, LBND, LPAS, LINI, LRES, LPAR, LRPAR, LGRD
!---------------------------------------------------------------------
!L     1.        Read in the JOB input data
  if(myproc.eq.0) then
!L       1.1       Read in heading
    read(*,'(A)',end=901,err=902) lajob
!L       1.2       Read in parameters
    read(*,NAMJOB,end=901,err=902)
  endif
!---------------------------------------------------------------------
!L     2.        Label the JOB
  if(myproc.eq.0) then
    write(*,'(a)') '=== JOB: '//lajob
    call DAYTIM
  endif
!---------------------------------------------------------------------
!L     3.        Broadcast to all procs
#ifdef MPI
! Wait until all procs come at this point
  call MPI_BARRIER(MPI_COMM_WORLD,ie)
  call MPI_BCAST(altime,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ldir,52,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lproj,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lnumo,6,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lprocs,4,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(levels,2,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lgrd,12,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lbnd,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lpas,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lini,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lres,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lpar,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(lgrd,8,MPI_CHARACTER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nlres,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
! Wait until all procs come at this point
  call MPI_BARRIER(MPI_COMM_WORLD,ie)
#endif
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
  901 continue
      write (*,'(a)') '*** End of input file'
      goto 903
  902 continue
      write (*,'(a)') '*** Error in input file'
  903 continue
      write (*,'(a)') '*** Error in subroutine JOBPAR'
      call JOBEND
RETURN
END SUBROUTINE JOBPAR


!****f* e0pro/main
! NAME 
!   JOBAUX
! PURPOSE
!   Set the JOB auxiliary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBAUX
!---------------------------------------------------------------------
  USE comoly
  IMPLICIT NONE
  INTEGER :: idir, iproj, icase, idin, idout
  INTEGER :: icode, inumo, iprocs, ilevels
  INTEGER :: igrd, iini, ibnd, ipas, ipar, idres, ires
  INTEGER :: irun, j
!---------------------------------------------------------------------
!L     1.        Determine the length of names
!L       1.1       Length of file names
!Principal directory
  idir=0
  do j=1,80
    if(ldir(j:j).ne.' ') idir=idir+1
  enddo
  if(idir.eq.0) then
    ldir='.'
    idir=1
  endif
!Project name
  iproj=0
  do j=1,8
    if(lproj(j:j).ne.' ') iproj=iproj+1
  enddo
!Code names
  inumo=0
  do j=1,8
    if(lnumo(j:j).ne.' ') inumo=inumo+1
  enddo
  iprocs=0
  do j=1,4
    if(lprocs(j:j).ne.' ') iprocs=iprocs+1
  enddo
  ilevels=0
  do j=1,2
    if(levels(j:j).ne.' ') ilevels=ilevels+1
  enddo
!Grid name
  igrd=0
  do j=1,12
    if(lgrd(j:j).ne.' ') igrd=igrd+1
  enddo
!Run names
  iini=0
  ibnd=0
  ipas=0
  ipar=0
  do j=1,8
    if(lini(j:j).ne.' ') iini=iini+1
    if(lbnd(j:j).ne.' ') ibnd=ibnd+1
    if(lpas(j:j).ne.' ') ipas=ipas+1
    if(lpar(j:j).ne.' ') ipar=ipar+1
  enddo
!Resume name
  ires=0
  do j=1,52
    if(lres(j:j).ne.' ') ires=ires+1
  enddo
!L       1.2       Set the case name
  if((iini.eq.0).and.(ibnd.eq.0).and.(ipas.eq.0)) then
  endif
  if((iini.eq.0).and.(ibnd.gt.0).and.(ipas.eq.0)) then
    lcase=lbnd(1:ibnd)//'.'//lgrd(1:igrd)
    icase=ibnd+1+igrd
  endif
  if((iini.eq.0).and.(ibnd.eq.0).and.(ipas.gt.0)) then
    lcase=lpas(1:ipas)//'.'//lgrd(1:igrd)
    icase=ipas+1+igrd
  endif
  if((iini.gt.0).and.(ibnd.eq.0).and.(ipas.eq.0)) then
    lcase=lini(1:iini)//'.'//lgrd(1:igrd)
    icase=iini+1+igrd
  endif
  if((iini.gt.0).and.(ibnd.gt.0).and.(ipas.eq.0)) then
    lcase=lini(1:iini)//'.'//lbnd(1:ibnd)//'.'//lgrd(1:igrd)
    icase=iini+1+ibnd+1+igrd
  endif
  if((iini.gt.0).and.(ibnd.eq.0).and.(ipas.gt.0)) then
    lcase=lini(1:iini)//'.'//lpas(1:ipas)//'.'//lgrd(1:igrd)
    icase=iini+1+ipas+1+igrd
  endif
!L       1.3       Set the code name
  lcode=lprocs(1:iprocs)//'-'//lnumo(1:inumo)//'-'//levels(1:ilevels)
  icode=iprocs+1+inumo+1+ilevels
!L       1.4       Set the run name
  lrun=lcase(1:icase)//'.'//lcode(1:icode)//'.'//lpar(1:ipar)
  irun=icase+1+icode+1+ipar
!---------------------------------------------------------------------
!L     3.        Set the input directory name
  if(iproj.gt.0) then
    ldin=ldir(1:idir)//'/'//lproj(1:iproj)//'/case.'//lcase(1:icase)
    idin=idir+1+iproj+6+icase
  else
    ldin='.'
    idin=1
  endif
!---------------------------------------------------------------------
!L     3.        Set the complete input file names
!L       3.1       Grid data
  lfgrd=ldin(1:idin)//'/grd.nc'
  mfgrd=idin+7
!L       3.1       Initial data
  if(iini.gt.0) then
    lfini=ldin(1:idin)//'/ini.nc'
    mfini=idin+7
  endif
!L       3.1       Boundary data
  if(ibnd.gt.0) then
    lfbnd=ldin(1:idin)//'/bnd.nc'
    mfbnd=idin+7
  endif
!L       3.1       Passage data
  if(ipas.gt.0) then
    lfpas=ldin(1:idin)//'/pas.nc'
    mfpas=idin+7
  endif
!---------------------------------------------------------------------
!L     4.        Set the output direcory name
  if(iproj.gt.0) then
    ldout=ldir(1:idir)//'/'//lproj(1:iproj)//'/run.'//lrun(1:irun)
    idout=idir+1+iproj+5+irun
  else
    ldout='.'
    idout=1
  endif
!---------------------------------------------------------------------
!L     5.        Set the complete output file names
!L       2.1       Distribution at given times
  lfx1s=ldout(1:idout)//'/x1s.nc'
  lfx2s=ldout(1:idout)//'/x2s.nc'
  lfx3s=ldout(1:idout)//'/x3s.nc'
  lfval=ldout(1:idout)//'/val.nc'
  mfx1s=idout+7
  mfx2s=idout+7
  mfx3s=idout+7
  mfval=idout+7
!L       2.2       Evolution at given positions
  lfevg=ldout(1:idout)//'/evg.nc'
  lfevh=ldout(1:idout)//'/evh.nc'
  lfevp=ldout(1:idout)//'/evp.nc'
  lfevl=ldout(1:idout)//'/evl.nc'
  mfevo=idout+7
!L       2.3       Outflow
  lfpass=ldout(1:idout)//'/pas.nc'
  mfpass=idout+7
!L       2.2       Values at time levels
  if(nprocs.gt.1) then
    lftim=ldout(1:idout)//'/'//lproc//'.'
    mftim=idout+6
  else
    lftim=ldout(1:idout)//'/'
    mftim=idout+1
  endif
!---------------------------------------------------------------------
!L     3.        Set the resume directory name
  if(ires.gt.0) then
    if(iproj.gt.0) then
      ldres=ldir(1:idir)//'/'//lproj(1:iproj)//'/run.'//lres(1:ires)
      idres=idir+1+iproj+5+ires
    else
      ldres='../run.'//lres(1:ires)
      idres=7+ires
    endif
  endif
!---------------------------------------------------------------------
!L     5.        Set the complete restart file names
!L       5.1       Initial restart data
  if(ires.gt.0) then
    lfres=ldres(1:idres)//'/res.fin.nc'
    mfres=idres+11
  else
    lfres=ldout(1:idout)//'/res.ini.nc'
    mfres=idout+11
  endif
!L       5.3       Final restart data
  lffin=ldout(1:idout)//'/res.fin.nc'
  mffin=idout+11
!---------------------------------------------------------------------
!L     9.        Terminate
  mfin=0
  mpass=0
  mtim=0
  mval=0
  mevo=0
  mxs=0
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
END SUBROUTINE JOBAUX


!****f* e0pro/main
! NAME 
!   JOBCHE
! PURPOSE
!   Check the JOB parameters
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBCHE
!---------------------------------------------------------------------
  USE comoly
!---------------------------------------------------------------------
  IMPLICIT NONE
  LOGICAL :: ilexis
  INTEGER :: ierr
!---------------------------------------------------------------------
  ilexis=.false.
  ierr=0
!---------------------------------------------------------------------
!L     3.        Check existence of input files
  if(myproc.eq.0) then
    write(*,'(a)') '--- INPUT DATA:'
!L       3.1       Grid data
    if(mfgrd.gt.0) then
      INQUIRE(file=lfgrd,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '    Grid data: '//lfgrd(1:mfgrd)
      else
        write (*,'(a)') '*** File does not exist: '//lfgrd(1:mfgrd)
        ierr=ierr+1
      endif
    else
      write(*,'(a)') '*** No grid data specified'
    endif
!L       3.2       Initial data
    if(mfini.gt.0) then
      INQUIRE(file=lfini,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '    Initial data: '//lfini(1:mfini)
      else
        write(*,'(a)') '*** File does not exist: '//lfini(1:mfini)
        ierr=ierr+1
      endif
    else
!     write(*,'(a)') '    No initial data specified'
    endif
!L       3.2       Resume data
    if(nlres) then
      INQUIRE(file=lfres,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '    Resume data: '//lfres(1:mfres)
      else
        write(*,'(a)') '*** File does not exist: '//lfres(1:mfres)
        ierr=ierr+1
      endif
    else
!     write(*,'(a)') '    No resume data specified'
    endif
!L       3.3       Boundary data
    if(mfbnd.gt.0) then
      INQUIRE(file=lfbnd,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '    Boundary data: '//lfbnd(1:mfbnd)
      else
        write(*,'(a)') '*** File does not exist: '//lfbnd(1:mfbnd)
        ierr=ierr+1
      endif
    endif
!L       3.3       Passage data
    if(mfpas.gt.0) then
      INQUIRE(file=lfpas,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '    Passage data: '//lfpas(1:mfpas)
      else
        write(*,'(a)') '*** File does not exist: '//lfpas(1:mfpas)
        ierr=ierr+1
      endif
    endif
!L       3.4       At least something has to be specified
    if((mfini.eq.0).and.(mfres.eq.0).and.(mfbnd.eq.0).and.(mfpas.eq.0)) then
      write(*,'(a)') '  Either INI, RES, BND, or PAS has to be specified'
      ierr=ierr+1
      if(((mfbnd.gt.1).or.(mfpas.gt.1)).and.(mfgrd.eq.0)) then
        write(*,'(a)') '  For BND or PAS also GRD has to be specified'
        ierr=ierr+1
      endif
    endif
!---------------------------------------------------------------------
!L     4.        Avoid overwritting of output files
!L       4.1       Final data
    if(mffin.gt.0) then
      INQUIRE(file=lffin,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '*** File already exists: '//lffin(1:mffin)
        ierr=ierr+1
      endif
    endif
!L       4.2       Passage data
    if(mfpass.gt.0) then
      INQUIRE(file=lfpass,exist=ilexis)
      if(ilexis) then
        write(*,'(a)') '*** File already exists: '//lfpass(1:mfpass)
        ierr=ierr+1
      endif
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Terminate
  if(ierr.ne.0) then
    write(*,'(a)') '*** Error in subroutine JOBCHE'
    call JOBEND
  endif
RETURN
END SUBROUTINE JOBCHE


!****f* e0pro/main
! NAME 
!   JOBEND
! PURPOSE
!   Terminate the JOB
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE JOBEND
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  IMPLICIT NONE
!---------------------------------------------------------------------
!L     1.        Write a RUN message
!L       1.1       Normal termination
  if(nlend) then
    if(myproc.eq.0) then
      write(*,'(a)') '--- END OF RUN: '//LRUN
    endif
!L       1.2       Annormal termination
  else
    if(myproc.eq.0) then
      write(*,'(a)') '--- ABNORMAL END OF RUN: '//LRUN
    endif
  endif
!---------------------------------------------------------------------
!L     2.        Close the PARAMESH
#ifdef PARAMESH
  call AMR_CLOSE
#endif
!---------------------------------------------------------------------
!L     2.        Write date and time
  call DAYTIM
  call RUNTIM
!---------------------------------------------------------------------
!L     3.        Write output file names
  if(myproc.eq.0) then
    write(*,'(a)') '--- OUTPUT DATA:'
!L       3.1       Final state data
    if(mfin.gt.0) then
      write(*,'(a,i4,a)') '    Final state data      ', mfin, '  :'  &
        //lffin(1:mffin)
    endif
!L       3.2       Boundary passage data
    if(mpass.gt.0) then
      write(*,'(a,i4,a)') '    Boundary passage data ', mpass, '  :'  &
        //lfpass(1:mfpass)
    endif
!L       3.3       Values at time levels
    if(mval.gt.0) then
      write(*,'(a,i4,a)') '    Values at time levels ', mval, '  :'  &
        //lfval(1:mfval)
    endif
!L       3.3       Values at a time level
    if(mtim.gt.0) then
      write(*,'(a,i4,a)') '    Values at a time level', mtim, '  :'  &
        //lftim(1:mftim)
    endif
!L       3.4       Evolution at positions
    if(mevo.gt.0) then
      if(ngeo.gt.0) &
        write(*,'(a,i6,a)') '    Evolution at positions', mevo, '  :'  &
        //lfevg(1:mfevo)
      if(nhel.gt.0) &
        write(*,'(a,i6,a)') '    Evolution at positions', mevo, '  :'  &
        //lfevh(1:mfevo)
      if(nsel.gt.0) &
        write(*,'(a,i6,a)') '    Evolution at positions', mevo, '  :'  &
        //lfevl(1:mfevo)
      if(npla.gt.0) &
        write(*,'(a,i6,a)') '    Evolution at positions', mevo, '  :'  &
        //lfevp(1:mfevo)
    endif
!L       3.5       Values at slices
    if(mxs.gt.0) then
      write(*,'(a,i4,a)') '    Values at X1-slice', mxs, '  :'  &
        //lfx1s(1:mfx1s)
      write(*,'(a,i4,a)') '    Values at X2-slice', mxs, '  :'  &
        //lfx2s(1:mfx2s)
      write(*,'(a,i4,a)') '    Values at X3-slice', mxs, '  :'  &
        //lfx3s(1:mfx3s)
    endif
  endif
!---------------------------------------------------------------------
!L     4.        Write a JOB message
  if(myproc.eq.0) then
!L       4.1       Normal termination
    if(nlend) then
      write(*,'(a)') '=== END OF JOB'
!L       4.2       Annormal termination
    else
      write(*,'(a)') '=== ABNORMAL END OF JOB'
    endif
  endif 
!---------------------------------------------------------------------  
!L     9.        Terminate                                
STOP
END SUBROUTINE JOBEND                         
