!****f* e0pro/main
! NAME 
!   RUNCLR
! PURPOSE
!   Clear variables and arrays                                       
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCLR 
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comglo
  IMPLICIT NONE
!---------------------------------------------------------------------
!L     3.        Clear variables
!L       3.1       Clear
! unk(:,:,:,:,:)=zero
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RUNCLR


!****f* e0pro/main
! NAME 
!   RUNPRE
! PURPOSE
!   Set default variables
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNPRE
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comevo
  USE commet
  USE comout
  IMPLICIT NONE
!---------------------------------------------------------------------
!L     1.        Administration
!L       1.1       Actual dimensions
  nhel=0
  nsel=0
  ngeo=0
  npla=0
  nblk=maxblk
!---------------------------------------------------------------------
!L     3.        Physical problem
!L       3.1       Parameters defining a run
  vutime=1d0
#ifdef cartesian
  vux1=1d0
  vux2=1d0
  vux3=1d0
#endif
#ifdef cylindrical
  vux1=1d0
  vux2=0.017453293d0
  vux3=1d0
#endif
#ifdef spherical
  vux1=1d0
  vux2=0.017453293d0
  vux3=0.017453293d0
#endif
!L       3.2       Parameters defining a run
  nbcmag=0
  xmgrav=1.991d30
  nlxn=.false.
  nlgrav=.true.
!L       3.3       Parameters defining a run
  vrot=0d0
  ntest=0
!---------------------------------------------------------------------
!L     4.        Computational region
!L       4.1       Topographical parameters
!---------------------------------------------------------------------
!L     5.        Boundary conditions
!L       4.1       Boundary conditions
  nbndin=0
  npfrom=0
  nltbc=.false.
!L       4.1       Boundary conditions
  x1hel(:)=0d0
  x2hel(:)=0d0
  x3hel(:)=0d0
  n1hel(:,:)=0
  n2hel(:,:)=0
  n3hel(:,:)=0
  nldiav=.false.
  x1sel(:)=0d0
  n1sel(:,:)=0
!---------------------------------------------------------------------
!L     7.        Local variables
!L       7.1       Perfect gas parameters
  gamcld=5d0/3d0
  xalpha=0d0
!---------------------------------------------------------------------
!L     1.        Method of solution
#ifdef PARAMESH
  ctore=0.1d0
  ctode=0.01d0
#endif
!L       1.2       Numerical control
  akcfl=0.4d0
  akdt=2d0
  akdtmn=0.1d0
  akdtmx=10d0
  dtzero=0d0
  dtmin=0d0
  dtmax=0d0
  nitmax=5
  nltimc=.true.
!L       1.3       Numerical control
  cdifb=1d0
  ndivb=0
!---------------------------------------------------------------------
!L     1.        Output control
  nstart=0
  nstop=0
  tstart=0d0
  tstop=0d0
!L       1.1       Time control
  ndstep=0
  nefrom=0
  neto=0
  nestep=0
  nffrom=0
  nfto=0
  nfstep=0
  ntfrom=0
  ntto=0
  ntstep=0
  nvfrom=0
  nvto=0
  nvstep=0
  nsfrom=0
  nsto=0
  nsstep=0
  npfrom=0
  npto=0
  npstep=0
!L       1.1       Time control
  tdstep=0d0
  tefrom=0d0
  teto=0d0
  testep=0d0
  tffrom=0d0
  tfto=0d0
  tfstep=0d0
  ttfrom=0d0
  ttto=0d0
  ttstep=0d0
  tvfrom=0d0
  tvto=0d0
  tvstep=0d0
  tsfrom=0d0
  tsto=0d0
  tsstep=0d0
  tpfrom=0d0
  tpto=0d0
  tpstep=0d0
!L       1.2       Step control
  x1s=0d0
  x2s=0d0
  x3s=0d0
RETURN
END SUBROUTINE RUNPRE


!****f* e0pro/main
! NAME 
!   RUNPAR
! PURPOSE
!   Define data specific to run
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNPAR
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comout
  USE comevo
  USE combnd
  USE commet
  IMPLICIT NONE
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ie
#endif
!---------------------------------------------------------------------
  NAMELIST /namrun/ &
#ifdef PARAMESH
    CTORE, CTODE,  &
#endif
    VUTIME, VUX1, NTEST, NBCMAG, NLTBC,                                &
    NRECON, ETACON, YJZLIM, ETALIM, YJZANO, ETAANO, NLDIAV, NLXN,      &
    GAMMA, CDIFB, QHEAT, NHEAT,                                        &
    NLXP, NLGRAV, GAMCLD, XMGRAV, XALPHA, TSTART, TSTOP, NSTART, NSTOP,&
    TDSTEP, NDSTEP, TPSTEP, NPSTEP, TEFROM, TETO, TESTEP, NEFROM,      &
    NETO, NESTEP, TFFROM, TFTO, TFSTEP, NFFROM, NFTO, NFSTEP, TVFROM,  &
    TVTO, TVSTEP, NVFROM, NVTO, NVSTEP, TTFROM, TTTO, TTSTEP, NTFROM,  &
    NTTO, NTSTEP, TSFROM, TSTO, TSSTEP, NSFROM, NSTO, NSSTEP,          &
    NVRED, NB1, NBNDIN, NPFROM, VROT, X1S, X2S,                        &
    X3S,  X1HEL, X2HEL, X3HEL, X1SEL,                                  &
    NHEL, NSEL, NGEO, NPLA,                                            &
    AKCFL, AKDT, DTZERO, DTMAX, DTMIN, NSOLVE, NITMAX, NLTIMC, NDIVB
!---------------------------------------------------------------------
!L     1.        Read in parameters
  if(myproc.eq.0) then
    read(*,'(a)') lapar
    read(*, namrun)
  endif
!---------------------------------------------------------------------
!L     2.        Label the run
  if(myproc.eq.0) then
    write(*,'(a)') '--- RUN: '//lapar
!L       2.1       Physical parameters
    write(*,'(2x,2(2x,a8,1pe12.5))') &
      'vutime =',vutime,'vux1   =',vux1
    write(*,'(a)') '--- PHYSICAL PARAMETERS:'
    write(*,'(4x,a8,1pe12.5)') 'vrot   =',vrot
#ifdef gam3d
    write(*,'(4x,a8,1pe12.5)') 'gamcld =',gamcld
#else
    write(*,'(4x,a8,1pe12.5)') 'gamma  =',gamma
#endif
    write(*,'(4x,a8,1pe12.5)') 'xalpha =',xalpha
    write(* ,'(4x,a8,1pe12.5,2x,a8,l12)') &
      'xmgrav =',xmgrav,'nlgrav =',nlgrav
    write(*,'(4x,a8,l12)') 'nlxn   =',nlxn
!   write(*,'(2x,3(2x,a8,1pe12.5))')                                 &
!     'etaano =',etaano,'etacon =',etacon,'etalim =',etalim
!   write(*,'(2x,2(2x,a8,1pe12.5))')                                 &
!     'yjzano =',yjzano,'yjzlim =',yjzlim
!L       2.2       Numerical parameters
    write(*,'(a)') '--- NUMERICAL PARAMETERS:'
    write(*,'(4x,a8,i12)') 'nbndin =',nbndin
    write(*,'(4x,a8,i12)') 'nbcmag =',nbcmag
    write(*,'(2x,2(2x,a8,1pe12.5))') 'akcfl  =',akcfl,'akdt   =',akdt
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'dtzero =',dtzero,'dtmin  =',dtmin,'dtmax  =',dtmax
    write(*,'(4x,a8,i12)') 'ntest  =',ntest
    write(*,'(4x,a8,1pe12.5)') 'cdifb  =',cdifb
    write(*,'(4x,a8,i12)') 'ndivb  =',ndivb
!L       2.3       Output parameters
    write(*,'(a)') '--- OUTPUT PARAMETERS:'
    write(*,'(2x,2(2x,a8,1pe12.5))') &
      'tstart =',tstart,'tstop  =',tstop
    write(*,'(2x,2(2x,a8,i12))') 'nstart =',nstart,'nstop  =',nstop
    write(*,'(4x,a8,1pe12.5)') 'tdstep =',tdstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tefrom =',tefrom,'teto   =',teto, 'testep =',testep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tffrom =',tffrom,'tfto   =',tfto, 'tfstep =',tfstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'ttfrom =',ttfrom,'ttto   =',ttto, 'ttstep =',ttstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tvfrom =',tvfrom,'tvto   =',tvto, 'tvstep =',tvstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'tsfrom =',tsfrom,'tsto   =',tsto, 'tsstep =',tsstep
    write(*,'(4x,a8,i12)') 'ndstep =',ndstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nefrom =',nefrom,'neto   =',neto,'nestep =',nestep
    write(*,'(2x,3(2x,a8,i12))') &
      'nffrom =',nffrom,'nfto   =',nfto,'nfstep =',nfstep
    write(*,'(2x,3(2x,a8,i12))') &
      'ntfrom =',ntfrom,'ntto   =',ntto,'ntstep =',ntstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nvfrom =',nvfrom,'nvto   =',nvto,'nvstep =',nvstep
    write(*,'(2x,3(2x,a8,i12))') &
      'nsfrom =',nsfrom,'nsto   =',nsto,'nsstep =',nsstep
    write(*,'(2x,3(2x,a8,1pe12.5))') &
      'x1s    =',x1s,'x2s    =',x2s,'x3s    =',x3s
    write(*,'(4x,a8,i12)') 'nhel   =',nhel
    write(*,'(4x,a8,i12)') 'nsel   =',nsel
    write(*,'(4x,a8,i12)') 'ngeo   =',ngeo
    write(*,'(4x,a8,i12)') 'npla   =',npla
  endif
#ifdef MPI
!---------------------------------------------------------------------
!L     3.        Wait until all procs come at this point
  call MPI_BARRIER(MPI_COMM_WORLD,ie)
!---------------------------------------------------------------------
!L     4.        Broadcast parameters
  call MPI_BCAST(vutime,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(vux1,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(vrot,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(gamma,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(gamcld,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(xmgrav,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(xalpha,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nlgrav,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nlxn,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(etaano,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(etacon,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(etalim,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(yjzano,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(yjzlim,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nbndin,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ntest,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ndivb,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nltimc,1,MPI_LOGICAL,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(akcfl,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(akdt,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(dtzero,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(dtmin,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(dtmax,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(cdifb,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tstart,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tstop,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tdstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tefrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(teto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(testep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tffrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tfto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tfstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ttfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ttto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ttstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tvfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tvto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tvstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tsfrom,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tsto,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(tsstep,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nstart,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nstop,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ndstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nefrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(neto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nestep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nffrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nfto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nfstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ntfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ntto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ntstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nvfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nvto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nvstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nsfrom,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nsto,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nsstep,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x1s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x2s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x3s,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x1hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x2hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x3hel,maxhel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nhel,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(x1sel,maxsel,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(nsel,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(ngeo,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
  call MPI_BCAST(npla,1,MPI_INTEGER,0,MPI_COMM_WORLD,ie)
!---------------------------------------------------------------------
!L     5.        Wait until all procs come at this point
  call MPI_BARRIER(MPI_COMM_WORLD,ie)
#endif
!---------------------------------------------------------------------
!L     9.        Terminate
RETURN
END SUBROUTINE RUNPAR


!****f* e0pro/main
! NAME 
!   RUNCHE
! PURPOSE
!   Check input data
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCHE
!---------------------------------------------------------------------
  USE comoly
  USE commax
  USE comdim
  USE comphy
  USE comreg
  USE comout
  USE comglo
  USE comevo
  USE combnd
  USE commet
  IMPLICIT NONE
  INTEGER :: ierr
!---------------------------------------------------------------------
  if(myproc.eq.0) then
    ierr=0
!---------------------------------------------------------------------
!L     3.        Physical problem
!L       3.1       Parameters defining a run
!---------------------------------------------------------------------
!L     4.        Computational region
!L       4.1       Topographical parameters
!---------------------------------------------------------------------
!L     7.        Local variables
!---------------------------------------------------------------------
!L     1.        Boundary conditions
!L       1.2       Hydrodynamic variables
!---------------------------------------------------------------------
!L     1.        Method of solution
!L       1.1       Numerical control
    if((akcfl.le.0d0).or.(akcfl.ge.1d0)) then
      write(*,*) '    akcfl (0d0,1d0) =', akcfl
      ierr=ierr+1
    endif
!---------------------------------------------------------------------
!L     1.        Output control
    if((nhel.lt.0).or.(nhel.gt.maxhel)) then
      write(*,*) '    nhel (0,maxhel/ =', nhel
      ierr=ierr+1
    endif
    if((nsel.lt.0).or.(nsel.gt.maxsel)) then
      write(*,*) '    nsel (0,maxsel/ =', nsel
      ierr=ierr+1
    endif
!---------------------------------------------------------------------
!L     1.        Output control
    if((tfstep.eq.zero).and.(nfstep.eq.0)) then
      write(*,'(a)') '*** No RES output at resume time levels'
    endif
    if((tpstep.eq.zero).and.(npstep.eq.0)) then
      write(*,'(a)') '*** No PAS output at passage positions'
    endif
    if((tvstep.eq.zero).and.(nvstep.eq.0)) then
      write(*,'(a)') '*** No VAL output at time levels'
    endif
    if((ttstep.eq.zero).and.(ntstep.eq.0)) then
      write(*,'(a)') '*** No TIM output at time levels'
    endif
    if((tsstep.eq.zero).and.(nsstep.eq.0)) then
      write(*,'(a)') '*** No X1S,X2S,X3S outputs at slices'
    endif
    if((testep.eq.zero).and.(nestep.eq.0)) then
      write(*,'(a)') '*** No EVH,EVL,EVG,EVP,EVF outputs at significant positions'
    endif
    if((tdstep.eq.zero).and.(ndstep.eq.0)) then
      write(*,'(a)') '*** No output of diagnostics'
    endif
!---------------------------------------------------------------------
!L     9.        Terminate
!return if no error in input data
    if(ierr.eq.0)  RETURN
    write(*,*) '*** input data outside limits'
    write(*,*) '    errors', ierr
    write(*,*) '*** error in subroutine 1.6 check'
    call JOBEND
  endif
RETURN
END SUBROUTINE RUNCHE


!****f* e0pro/main
! NAME 
!   RUNAUX
! PURPOSE
!   Set auxiliary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNAUX
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE combnd
  USE comevo
  USE commet
  USE comout
  IMPLICIT NONE
  INTEGER :: jsel, jhel
!---------------------------------------------------------------------
!L     3.        Physical problem
!L       3.1       Parameters defining a run
#ifdef cartesian
  vux2=vux1
  vux3=vux1
#endif
#ifdef cylindrical
  vux3=vux1
#endif
!L       3.4       External forces
  grav=cgrav*xmgrav
!---------------------------------------------------------------------
!L     7.        Local variables
!L       7.1       Perfect gas parameters
  xmass=(1d0+4d0*xalpha)*cmass
  rgas=((2d0+3d0*xalpha)/(1d0+4d0*xalpha))*(ck/cmass)
!---------------------------------------------------------------------
!L     7.        Output parameter
!---------------------------------------------------------------------
!L     8.        Auxiliary values
#ifdef roe
  z13=zone/zthree
  z23=ztwo/zthree
#endif
  if(nlxn) then
    z05=zhalf*xmass
    zgas=ztwo*ck
    ztwom=ztwo/xmass
  else
    z05=zhalf
    zgas=rgas
    ztwom=ztwo
  endif
  zgrav=zero
  if(nlgrav) zgrav=grav
  if(nlxn) zgrav=zgrav*xmass
#ifdef mhd
  zmu=cmu
  zrmu=zone/cmu
  zr2mu=zhalf/cmu
#endif
!---------------------------------------------------------------------
!L     9.        Units
  if(ntest.eq.0) then
    if(myproc.eq.0) then
      write(*,'(a)') '--- UNITS:'
      write(*,'(a,1pe12.4)') '    vutime =',vutime
      write(*,'(a,1pe12.4)') '    vux1   =',vux1
      write(*,'(a,1pe12.4)') '    vux2   =',vux2
      write(*,'(a,1pe12.4)') '    vux3   =',vux3
    endif
    tstart=vutime*tstart
    tstop=vutime*tstop
    tdstep=vutime*tdstep
    tefrom=vutime*tefrom
    teto=vutime*teto
    testep=vutime*testep
    tffrom=vutime*tffrom
    tfto=vutime*tfto
    tfstep=vutime*tfstep
    ttfrom=vutime*ttfrom
    ttto=vutime*ttto
    ttstep=vutime*ttstep
    tvfrom=vutime*tvfrom
    tvto=vutime*tvto
    tvstep=vutime*tvstep
    tsfrom=vutime*tsfrom
    tsto=vutime*tsto
    tsstep=vutime*tsstep
    dtzero=vutime*dtzero
    dtmax=vutime*dtmax
    dtmin=vutime*dtmin
    vrot=vux3*vrot
    x1s=vux1*x1s
    x2s=vux2*x2s
    x3s=vux3*x3s
    do jhel=1,maxhel
      x1hel(jhel)=vux1*x1hel(jhel)
      x2hel(jhel)=vux2*x2hel(jhel)
      x3hel(jhel)=vux3*x3hel(jhel)
    enddo
    do jsel=1,maxsel
      x1sel(jsel)=vux1*x1sel(jsel)
    enddo
  endif 
!---------------------------------------------------------------------  
!L     9.        Terminate                                    
RETURN
END SUBROUTINE RUNAUX


!****f* e0pro/main
! NAME 
!   RUNCAL
! PURPOSE
!   Step on the calculation
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNCAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comtva
  USE comglo
  USE combnd
  USE commet
  USE comevo
  USE comout
!---------------------------------------------------------------------
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
  INTEGER :: j1,j2,j3,lb
  INTEGER :: iopt,iempty,nlayers
#endif
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr
#endif
#ifdef DIAGNO
  call EXPERT(4000,1)
#endif
!---------------------------------------------------------------------
!L     1.        Update time
  nstep=nstep+1
  time=time+dt
!---------------------------------------------------------------------
!L     2.        Numerical grid
#ifdef PARAMESH
!L       2.1       AMR solution
  mype=0
  iopt=1
  iempty=0
  nlayers=nguard
#ifndef ADVANCE_ALL_LEVELS
  call AMR_RESTRICT(mype,iopt,iempty)
#endif
  if(mod(nstep,2).eq.1) then
    if(lrefine_max.gt.lrefine_min) then
!test refinement levels
      call AMR_TEST_REFINEMENT(mype,lrefine_max,lrefine_min)
!refine grid and apply morton reordering to grid blocks if necessary
      call AMR_REFINE_DEREFINE
!     call SHMEM_BARRIER_ALL()
      call GRID
      call GRID1
      call GRID2
      call GRID3
!prolong solution to any new leaf blocks if necessary
      call AMR_PROLONG(mype,iopt,nlayers)
    endif
  endif
#endif
!---------------------------------------------------------------------
!L     3.        Sources and transport processes
  call CALSTP
!---------------------------------------------------------------------
!L     4.        Magnetohydrodynamics
  call CALMHD
!---------------------------------------------------------------------
!L     6.        Examination and diagnostics
!L       6.1       Save values in significant positions
#ifdef MPI
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
#endif
!L       6.1       Save values in significant positions
  if((testep.ne.zero).or.(nestep.ne.0)) then
    if(nxhel.ne.0) call HELVAL
    if(nxgeo.gt.0) call GEOVAL
    if(nxsel.ne.0) call SELVAL
    if(npla.ne.0) call PLAVAL
  endif
!L       6.2       Save diagnostic values
  if(nldiav) call DIAVAL
!L       6.3       Save passage values
  if((tpstep.ne.zero).or.(npstep.ne.0)) call PASVAL
!---------------------------------------------------------------------
!L     7.        New time step value
!Skip if not required
  if(mod(nstep,2).eq.0) then
    if(nltimc) then
      if(akcfl.ne.zero) then
        call TIMSTP
      endif
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4000,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RUNCAL


!****f* e0pro/main
! NAME 
!   RUNOUT
! PURPOSE
!   Control the output                                             
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNOUT(kmode) 
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comevo
  USE comout
  IMPLICIT NONE
  REAL(8) :: ztime
  INTEGER :: kmode, i
#ifdef DIAGNO
  call EXPERT(3000,1)
#endif
!---------------------------------------------------------------------
  if((tstop-tstart).lt.36000d0) then
    ztime=time
  elseif((tstop-tstart).gt.864000d0) then
    ztime=time/86400d0
  else
    ztime=time/3600d0
  endif
!---------------------------------------------------------------------
!L     1.        Initial output
  if(kmode.eq.1) then
!L       1.1       Values at final levels
    call OUTCHE(time,tfout,tffrom,tfto,tfstep, &
               nstep,nfout,nffrom,nfto,nfstep,kmode,mfin,i)
    if(i.eq.1) then
      call OUTFIN
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outfin',mfin,ztime
      endif
    endif
!L       1.2       Values at boundary positions
    call OUTCHE(time,tpout,tpfrom,tpto,tpstep, &
               nstep,npout,npfrom,npto,npstep,kmode,mpass,i)
    if(i.eq.1) then
!     call OUTPAS
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outpas',mpass,ztime
      endif
    endif
!L       1.7       Values at significant positions
    if((tvstep.ne.zero).or.(nvstep.ne.0)) then
      if(mfres.eq.0) then
        call OUTVAL(1)
      endif
      call OUTCHE(time,tvout,tvfrom,tvto,tvstep, &
                 nstep,nvout,nvfrom,nvto,nvstep,kmode,mval,i)
      if(i.eq.1) then
        call OUTVAL(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outval',mval,ztime
        endif
      endif
    endif
!L       1.3       Values at periodic levels
    if((ttstep.ne.zero).or.(ntstep.ne.0)) then
      call OUTCHE(time,ttout,ttfrom,ttto,ttstep, &
                 nstep,ntout,ntfrom,ntto,ntstep,kmode,mtim,i)
      if(i.eq.1) then
        call OUTTIM
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outtim',mtim,ztime
        endif
      endif
    endif
!L       1.5       Values at significant positions
    if((tsstep.ne.zero).or.(nsstep.ne.0)) then
      call OUTX1S(1)
      call OUTX2S(1)
      call OUTX3S(1)
      call OUTCHE(time,tsout,tsfrom,tsto,tsstep, &
                 nstep,nsout,nsfrom,nsto,nsstep,kmode,mxs,i)
      if(i.eq.1) then
        call OUTX1S(2)
        call OUTX2S(2)
        call OUTX3S(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outx*s',mxs,ztime
        endif
      endif
    endif
!L       1.7       Values at significant positions
    if((testep.ne.zero).or.(nestep.ne.0)) then
      if(.not.nlres) then
        if(myproc.eq.0) then
          if(nhel.gt.0) call OUTEVH(1)
          if(nsel.gt.0) call OUTEVL(1)
          if(npla.gt.0) call OUTEVP(1)
        endif
        if(nxgeo.gt.0) call OUTEVG(1)
      endif
      call OUTCHE(time,teout,tefrom,teto,testep, &
                 nstep,neout,nefrom,neto,nestep,kmode,mevo,i)
      if(i.eq.1) then
        if(myproc.eq.0) then
          if(nhel.gt.0) call OUTEVH(2)
          if(nsel.gt.0) call OUTEVL(2)
          if(npla.gt.0) call OUTEVP(2)
        endif
        if(nxgeo.gt.0) call OUTEVG(2)
        if(myproc.eq.0) then
          write(*,'(4x,a6,i6,1pe11.3)') 'outevo',mevo,ztime
        endif
      endif
    endif
!L       1.8       Diagnostics
    if((tdstep.eq.zero).and.(ndstep.eq.0)) then
      if(myproc.eq.0) then
        write(*,'(a)') '*** No output of diagnostics'
      endif
    else
      call OUTDIA(1)
      call OUTDIA(2)
    endif
  endif
!---------------------------------------------------------------------
!L     2.        Periodic output
  if(kmode.eq.2) then
!L       2.1       Values at time levels
    call OUTCHE(time,tfout,tffrom,tfto,tfstep, &
               nstep,nfout,nffrom,nfto,nfstep,kmode,mfin,i)
    if(i.eq.1) then
      call OUTFIN
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outfin', mfin, ztime
      endif
    endif
!L       2.2       Values at time levels
    call OUTCHE(time,tvout,tvfrom,tvto,tvstep, &
               nstep,nvout,nvfrom,nvto,nvstep,kmode,mval,i)
    if(i.eq.1) then
      call OUTVAL(2)
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outval',mval,ztime
      endif
    endif
!L       2.3       Values at significant positions
    call OUTCHE(time,ttout,ttfrom,ttto,ttstep, &
               nstep,ntout,ntfrom,ntto,ntstep,kmode,mtim,i)
    if(i.eq.1) then
      call OUTTIM
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outtim', mtim, ztime
      endif
    endif
!L       2.4       Values at significant positions
    call OUTCHE(time,tsout,tsfrom,tsto,tsstep, &
               nstep,nsout,nsfrom,nsto,nsstep,kmode,mxs,i)
    if(i.eq.1) then
      call OUTX1S(2)
      call OUTX2S(2)
      call OUTX3S(2)
      if(myproc.eq.0) then
        write(*,'(4x,a6,i6,1pe11.3)') 'outx*s', mxs, ztime
      endif
    endif
!L       2.7       Values at significant positions
    call OUTCHE(time,teout,tefrom,teto,testep, &
               nstep,neout,nefrom,neto,nestep,kmode,mevo,i)
    if(i.eq.1) then
      if(myproc.eq.0) then
        if(nhel.gt.0) call OUTEVH(2)
        if(nsel.gt.0) call OUTEVL(2)
        if(npla.gt.0) call OUTEVP(2)
      endif
      if(nxgeo.gt.0) call OUTEVG(2)
    endif
!L       2.8       Diagnostics
    if((tdstep.ne.zero).or.(ndstep.ne.0)) then
!     if((time.ge.tdout).and.(nstep.ge.ndout)) then
        call OUTDIA(2)
        tdout=tdout+tdstep
        ndout=ndout+ndstep
!     endif
    endif
  endif
!---------------------------------------------------------------------
!L     3.        Final output
  if(kmode.eq.3) then
!L       3.1       Diagnostics
    if((tdstep.ne.zero).or.(ndstep.ne.0)) then
      call OUTDIA(3)
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(3000,0)
#endif
RETURN
END SUBROUTINE RUNOUT


!****f* e0pro/main
! NAME 
!   RUNTST
! PURPOSE
!   Test for completion of run
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNTST
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comout
  IMPLICIT NONE
#ifdef DIAGNO
  call EXPERT(4006,1)
#endif
!---------------------------------------------------------------------
!L     1.        Check physical time
  if(tstop.gt.tstart) then
    if(time.ge.tstop) nlend=.true.
    if((nstep.ge.nstop).and.(nstop.gt.0)) nlend=.true.
  else
    if(nstop.gt.0) then
      if(nstep.ge.nstop) nlend=.true.
    else
      if(myproc.eq.0) then
        write(*,'(a)') '*** Not specified termination of the run'
        write(*,'(a)') '*** Error in subprogram RUNTST'
      endif
      nlend=.true.
      nlstop=.true.
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4006,0)
#endif                                                                  
RETURN
END SUBROUTINE RUNTST
