!****f* e0pro/main
! NAME 
!   RUNINI
! PURPOSE
!   Start the calculation
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RUNINI 
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comglo
  USE comevo
  USE commet
  USE comout
  USE comtva
  IMPLICIT NONE
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr, jhel, jsel
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(2000,1)
#endif
!---------------------------------------------------------------------
!L     1.        Initialize variables and parameters
!L       1.1       Set time parameters
  nstep=0
  time=tstart
!L       1.2       Set grid parameters
  nc1l=1+ng1
  nc2l=1+ng2
  nc3l=1+ng3
  nc1r=nc1-ng1
  nc2r=nc2-ng2
  nc3r=nc3-ng3
  nc1lm=nc1l-1
  nc1rp=nc1r+1
#ifdef x2dim
  nc2lm=nc2l-1
  nc2rp=nc2r+1
#else
  nc2lm=nc2l
  nc2rp=nc2r
#endif
#ifdef x3dim
  nc3lm=nc3l-1
  nc3rp=nc3r+1
#else
  nc3lm=nc3l
  nc3rp=nc3r
#endif
#ifdef NGRID
  nc125=nc1/4
  nc175=3*nc1/4+1
  nc1rr=nc1/2
  nc1ll=nc1/2+1
#ifdef x2dim
  nc225=nc2/4
  nc275=3*nc2/4+1
  nc2rr=nc2/2
  nc2ll=nc2/2+1
#else
  nc225=1
  nc275=1
  nc2rr=1
  nc2ll=1
#endif
#ifdef x3dim
  nc325=nc3/4
  nc375=3*nc3/4+1
  nc3rr=nc3/2
  nc3ll=nc3/2+1
#else
  nc325=1
  nc375=1
  nc3rr=1
  nc3ll=1
#endif
#endif
!L       1.3       Set boundary conditions
  if((lbnd.ne.' ').or.(lpas.ne.' ')) then
    nltbc=.true.
  endif
  if(npfrom.eq.0) npfrom=nc1-2*ng1+1
!L       1.4       Tests
  if(ntest.ne.0) then
    rgas=zone
    nlgrav=.false.
    nlxn=.false.
    nlxp=.true.
    z05=zhalf
    zgas=rgas
    ztwom=ztwo
#ifdef mhd
    zmu=zone
    zrmu=zone
    zr2mu=zhalf
#endif
  endif
!---------------------------------------------------------------------
!L     2.        Set the grid
!L       2.1       Read in grid parameters
  call RESGRD
!L       2.4       Set auxiliary grid parameters
#ifdef PARAMESH
  call GRID
#endif
#ifdef NGRID
  call MGRD
  call GRID
#endif
  call GRID1
!#ifdef x2dim
  call GRID2
!#endif
!#ifdef x3dim
  call GRID3
!#endif
!---------------------------------------------------------------------
!L     3.        Resume initial conditions
!L       3.1       Read in initial or restart values
  if(nlres) then
    call RESRES
  else
    if(mfini.gt.0) then
      call RESINI
    endif
  endif
!L       3.2       Gamma dependent constants
#ifndef gam3d
  if(nlxn) then
    gammam=gamma-zone
    zgamma=gamma/xmass
  else
    gammam=gamma-zone
    zgamma=gamma
  endif
#endif
!L       3.3       Read in initial or restart values
  if((mfini.gt.0).or.(nlres)) then
    if(myproc.eq.0) then
      if(nltbc) then
        call BNDTIM(1)
      endif
    endif
!L       3.4       Extrapolate from boundary values
  else
    call BNDTIM(1)
    call BNDINI
  endif
!---------------------------------------------------------------------
!L     4.        Determine initial timestep value
!L       4.1       Global minimum time step
  if(dtzero.eq.zero) then
    if(nlres) then
      dtzero=dt
    else
      call INIDT
#ifdef MPI
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)
      call MPI_ALLREDUCE(dt,dtzero,1,MPI_DOUBLE_PRECISION,      &
        MPI_MIN,MPI_COMM_WORLD,ierr)
      dt=dtzero
#else
      dtzero=dt
#endif
    endif
  endif
!L       4.2       Set limits
  if(dtmin.eq.zero) dtmin=akdtmn*dtzero
  if(dtmax.eq.zero) dtmax=akdtmx*dtzero
!---------------------------------------------------------------------
!L     5.        Continuation
  teout=tstart
  tfout=tstart
  tpout=tstart
  ttout=tstart
  tvout=tstart
  tsout=tstart
  mstep=0
  nsplit=1
!---------------------------------------------------------------------
!L     6.        Save initial values at significant points
!L       6.1       Save values
  call INIEVG
  call INIEVH
  call INIEVL
#ifdef MPI
  call INIPOS
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  call MPI_GATHER(nxhel,1,MPI_INTEGER,nnxhel,1,MPI_INT,0,MPI_COMM_WORLD,ierr)
  call MPI_GATHER(nxsel,1,MPI_INTEGER,nnxsel,1,MPI_INT,0,MPI_COMM_WORLD,ierr)
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  if(myproc.eq.0) then
    nnhel(1)=0
    do jhel=2,nprocs
      nnhel(jhel)=nnhel(jhel-1)+nnxhel(jhel-1)
    enddo
    nnsel(1)=0
    do jsel=2,nprocs
      nnsel(jsel)=nnsel(jsel-1)+nnxsel(jsel-1)
    enddo
  endif
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  call MPI_BCAST(nnxhel,1024,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call MPI_BCAST(nnxsel,1024,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call MPI_BCAST(nnhel,1024,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call MPI_BCAST(nnsel,1024,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
#endif
  if((testep.ne.zero).or.(nestep.ne.0)) then
    if(nxgeo.gt.0) call GEOVAL
    if(nxhel.gt.0) call HELVAL
    if(nxsel.gt.0) call SELVAL
    call PLAVAL
  endif
!L       6.2       Save passage values
  if((tpstep.ne.zero).or.(npstep.ne.0)) call PASVAL
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2000,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RUNINI


!****f* e0pro/main
! NAME 
!   INIDT
! PURPOSE
!   Set the initial timestep
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE INIDT
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comtva
  IMPLICIT NONE
  REAL(8) :: zc1mn, zpthe, zvel, zchar, zcmin
#ifdef x2dim
  REAL(8) :: zc2mn
#endif
#ifdef x3dim
  REAL(8) :: zc3mn
#endif
#ifdef mhd
  REAL(8) :: zpmag
#endif
#ifndef ethe
  REAL(8) :: zukin
#endif
  REAL(8) :: zc1
#ifdef x2dim
  REAL(8) :: zc2
#endif
#ifdef x3dim
  REAL(8) :: zc3
#endif
  INTEGER :: j1, j2, j3, jb
#ifdef DIAGNO
  call EXPERT 2002,1)
#endif
!---------------------------------------------------------------------
!L     1.        Default values
  dt=zone
  zc1mn=1d30
#ifdef x2dim
  zc2mn=1d30
#endif
#ifdef x3dim
  zc3mn=1d30
#endif
!---------------------------------------------------------------------
#ifdef PARAMESH
  do jb=1,lnblocks
  if(nodetype(jb).eq.1) then
#else
  do jb=1,nblk
#endif
!---------------------------------------------------------------------
!L     2.        Find minimum CFL number
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
#ifdef ethe
#ifdef mhd
          zpmag=zr2mu*(unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb)  &
                      +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb)  &
                      +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb))
#ifdef gam3d
          zpthe=(gamma(j1,j2,j3,jb)-zone)*unk(2,j1,j2,j3,jb)
          zchar=sqrt((gamma(j1,j2,j3,jb)*zpthe+ztwom*zpmag)/  &
            unk(1,j1,j2,j3,jb))
#else
          zpthe=(gamma-zone)*unk(2,j1,j2,j3,jb)
          zchar=sqrt((gamma*zpthe+ztwom*zpmag)/unk(1,j1,j2,j3,jb))
#endif
#else
#ifdef gam3d
          zpthe=(gamma(j1,j2,j3,jb)-zone)*unk(2,j1,j2,j3,jb)
          zchar=sqrt((gamma(j1,j2,j3,jb)*zpthe)/unk(1,j1,j2,j3,jb))
#else
          zpthe=(gamma-zone)*unk(2,j1,j2,j3,jb)
          zchar=sqrt((gamma*zpthe)/unk(1,j1,j2,j3,jb))
#endif
#endif
#else
          zukin=zhalf*(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb)  &
                      +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb)  &
                      +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))/ &
                       unk(1,j1,j2,j3,jb)
#ifdef mhd
          zpmag=zr2mu*(unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb)  &
                      +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb)  &
                      +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb))
#ifdef gam3d
          zpthe=(gamma(j1,j2,j3,jb)-zone)*(unk(2,j1,j2,j3,jb)-zukin-zpmag)
          zchar=sqrt((gamma(j1,j2,j3,jb)*zpthe+ztwom*zpmag)/  &
            unk(1,j1,j2,j3,jb))
#else
          zpthe=(gamma-zone)*(unk(2,j1,j2,j3,jb)-zukin-zpmag)
          zchar=sqrt((gamma*zpthe+ztwom*zpmag)/unk(1,j1,j2,j3,jb))
#endif
#else
#ifdef gam3d
          zpthe=(gamma(j1,j2,j3,jb)-zone)*(unk(2,j1,j2,j3,jb)-zukin)
          zchar=sqrt((gamma(j1,j2,j3,jb)*zpthe)/unk(1,j1,j2,j3,jb))
#else
          zpthe=(gamma-zone)*(unk(2,j1,j2,j3,jb)-zukin)
          zchar=sqrt((gamma*zpthe)/unk(1,j1,j2,j3,jb))
#endif
#endif
#endif
!         zvel=sqrt(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb)  &
!               +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb)    &
!               +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))   &
!               /abs(unk(1,j1,j2,j3,jb))
          zvel=sqrt(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc1=dx1(j1,jb)/(zvel+zchar)
#ifdef cartesian
#ifdef x2dim
          zvel=sqrt(unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb))  &
        `       /abs(unk(1,j1,j2,j3,jb))
          zc2=dx2(j2,jb)/(zvel+zchar)
#endif
#ifdef x3dim
          zvel=sqrt(unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc3=dx3(j3,jb)/(zvel+zchar)
#endif
#endif
#ifdef cylindrical
#ifdef x2dim
          zvel=sqrt(unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc2=xc1(j1,jb)*dx2(j2,jb)/(zvel+zchar)
#endif
#ifdef x3dim
          zvel=sqrt(unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc3=dx3(j3,jb)/(zvel+zchar)
#endif
#endif
#ifdef spherical
#ifdef x2dim
          zvel=sqrt(unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc2=xc1(j1,jb)*dx2(j2,jb)/(zvel+zchar)
#endif
#ifdef x3dim
          zvel=sqrt(unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))  &
                /abs(unk(1,j1,j2,j3,jb))
          zc3=xc1(j1,jb)*sint(j2,jb)*dx3(j3,jb)/(zvel+zchar)
#endif
#endif
          zc1mn=min(zc1mn,zc1)
#ifdef x2dim
          zc2mn=min(zc2mn,zc2)
#endif
#ifdef x3dim
          zc3mn=min(zc3mn,zc3)
#endif
        enddo
      enddo
    enddo
!---------------------------------------------------------------------
!L     3.        Output
#ifdef x2dim
#ifdef x3dim
    write(*,'(4x,2i4,2x,a11,3(1pe11.3))') myproc,jb,'INIDT-1,2,3',  &
      abs(akcfl)*zc1mn,abs(akcfl)*zc2mn,abs(akcfl)*zc3mn
#else
    write(*,'(4x,2i4,2x,a9,3(1pe11.3))') myproc,jb,'INIDT-1,2',  &
      abs(akcfl)*zc1mn,abs(akcfl)*zc2mn
#endif
#else
#ifdef x3dim
    write(*,'(4x,2i4,2x,a9,3(1pe11.3))') myproc,jb,'INIDT-1,3',  &
      abs(akcfl)*zc1mn,abs(akcfl)*zc3mn
#else
    write(*,'(4x,2i4,2x,a7,3(1pe11.3))') myproc,jb,'INIDT-1',  &
      abs(akcfl)*zc1mn
#endif
#endif
!---------------------------------------------------------------------
!L     4.        Local minimum time step
#ifdef x2dim
#ifdef x3dim
    zcmin=min(zc1mn,zc2mn,zc3mn)
#else
    zcmin=min(zc1mn,zc2mn)
#endif
#else
#ifdef x3dim
    zcmin=min(zc1mn,zc3mn)
#else
    zcmin=zc1mn
#endif
#endif
    dt=abs(akcfl)*zcmin
    cflmax=akcfl
#ifdef PARAMESH
    dt=zhalf*dt
  endif
#endif
  enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2002,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE INIDT


!****f* e0pro/main
! NAME 
!   BNDINI
! PURPOSE
!   Initial values by extrapolation from boundary values
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDINI
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE combnd
  USE comreg
  USE commet
  USE comout
  USE comglo
  USE comtva
  IMPLICIT NONE
  REAL(8) :: zc1, zc2, zcg, zradbn
#ifdef bctim2c
#ifndef ethe
  REAL(8) :: zc3
#endif
#endif
#ifndef ethe
  REAL(8) :: zukin, zuthe
#ifdef mhd
  REAL(8) :: zc4, zumag
#endif
#endif
  INTEGER :: j1, j2, j3, jb, i1, i2, i3, iv
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(2007,1)
#endif
!---------------------------------------------------------------------
!L     1.        Prologue
#ifdef spherical
#ifdef bctim2c
  zradbn=x1bn(nb1)
#else
  zradbn=x1bn
#endif
#endif
#ifdef PARAMESH
  nblk=lnblocks
#endif
!---------------------------------------------------------------------
!L     2.        Interpolate boundary values
  do jb=1,nblk
    call BNDMAP(jb)
    call BNDINT
!---------------------------------------------------------------------
!L     3.        Extrapolate from single-point boundary conditions
#ifndef bctim2c
    do j3=1,ncc3
      i3=ng3+j3
      do j2=1,ncc2
        i2=ng2+j2
        do j1=1,ncc1
          i1=ng1+j1
!L       3.1       Auxiliary values
#ifdef spherical
          zc1=zradbn/xc1(i1,jb)
          zc2=zc1*zc1
#ifdef etot
#ifdef mhd
          zc4=zc2*zc2
#endif
#endif
#ifdef gam3d
          zcg=zc1**(2d0*(gamma(j1,j2,j3,jb)-1d0))
#else
          zcg=zc1**(2d0*(gamma-1d0))
#endif
#else
          zc1=1d0
          zc2=1d0
#ifdef etot
          zc4=1d0
#endif
          zcg=1d0
#endif
!L       3.2       Density
          unk(1,i1,i2,i3,jb)=zc2*dpv(j2,j3)
!L       3.3       Energy
#ifndef ethe
          zukin=zhalf*dpv(j2,j3)*(v1pv(j2,j3)*v1pv(j2,j3)  &
                                 +v2pv(j2,j3)*v2pv(j2,j3)  &
                                 +v3pv(j2,j3)*v3pv(j2,j3))
#endif
#ifdef etot
#ifdef mhd
          zumag=zr2mu*(b1pv(j2,j3)*b1pv(j2,j3)  &
                      +b1pv(j2,j3)*b1pv(j2,j3)  &
                      +b1pv(j2,j3)*b1pv(j2,j3))
!         zuthe=uupv(j2,j3)-zukin-zumag
          zuthe=zgas*dpv(j2,j3)*tpv(j2,j3)/gammam
#else
          zuthe=zgas*dpv(j2,j3)*tpv(j2,j3)/gammam
#endif
#endif
#ifdef etotmom
          zuthe=uupv(j2,j3)-zukin
#endif
#ifdef etot
#ifdef mhd
          unk(2,i1,i2,i3,jb)=zcg*zuthe+zc2*zukin+zc4*zumag
#else
          unk(2,i1,i2,i3,jb)=zcg*zuthe+zc2*zukin
#endif
#endif
#ifdef etotmom
          unk(2,i1,i2,i3,jb)=zcg*zuthe+zc2*zukin
#endif
#ifdef ethe
          unk(2,i1,i2,i3,jb)=zcg*uupv(j2,j3)
#endif
!L       3.4       Momentum
          unk(3,i1,i2,i3,jb)=zc2*dv1pv(j2,j3)
          unk(4,i1,i2,i3,jb)=zc2*dv2pv(j2,j3)
          unk(5,i1,i2,i3,jb)=zc1*dv3pv(j2,j3)
          iv=5
!L       3.5       Magnetic field
#ifdef mhd
#ifdef staggered
          facevarx(1,i1,i2,i3,jb)=zc2*b1pv(j2,j3)
          facevary(1,i1,i2,i3,jb)=zc2*b2pv(j2,j3)
          facevarz(1,i1,i2,i3,jb)=zc1*b3pv(j2,j3)
#else
          unk(6,i1,i2,i3,jb)=zc2*b1pv(j2,j3)
          unk(7,i1,i2,i3,jb)=zc2*b2pv(j2,j3)
          unk(8,i1,i2,i3,jb)=zc1*b3pv(j2,j3)
          iv=8
#endif
#endif
!L       3.6       Cloud tracer
#ifdef hydcon
          iv=iv+1
          unk(iv,i1,i2,i3,jb)=zc2*dppv(j2,j3)
#endif
!L       3.7       Polarity tracer
#ifdef mhdcon
          iv=iv+1
          unk(iv,i1,i2,i3,jb)=zc2*bppv(j2,j3)
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!L     4.        Extrapolate from two-point boundary conditions
#ifdef bctim2c
    do j3=1,ncc3
      i3=ng3+j3
      do j2=1,ncc2
        i2=ng2+j2
        do j1=1,ncc1
          i1=ng1+j1
!L       4.1       Auxiliary values
#ifdef spherical
          zc1=zradbn/xc1(i1,jb)
          zc2=zc1*zc1
#ifndef ethe
          zc3=zc1**2.1d0
#endif
#ifdef gam3d
          zcg=zc1**(2d0*gamma(j1,j2,j3,jb))
#else
          zcg=zc1**(2d0*gamma)
#endif
#else
          zc1=1d0
          zc2=1d0
          zc3=1d0
#endif
!L       4.2       Density
          unk(1,i1,i2,i3,jb)=zc2*dpv(nb1,j2,j3)
!L       4.3       Energy
#ifdef etot
          unk(2,i1,i2,i3,jb)=zc3*uupv(nb1,j2,j3)
#endif
#ifdef etotmom
          unk(2,i1,i2,i3,jb)=zc1*uupv(nb1,j2,j3)
#endif
#ifdef ethe
          unk(2,i1,i2,i3,jb)=zcg*uupv(nb1,j2,j3)
#endif
!L       4.4       Momentum
          unk(3,i1,i2,i3,jb)=zc2*dv1pv(nb1,j2,j3)
          unk(4,i1,i2,i3,jb)=zc2*dv2pv(nb1,j2,j3)
          unk(5,i1,i2,i3,jb)=zc1*dv3pv(nb1,j2,j3)
          iv=5
!L       4.5       Magnetic field
#ifdef mhd
#ifdef staggered
          facevarx(1,i1,i2,i3,jb)=zc2*b1pv(nb1,j2,j3)
          facevary(1,i1,i2,i3,jb)=zc2*b2pv(nb1,j2,j3)
          facevarz(1,i1,i2,i3,jb)=zc1*b3pv(nb1,j2,j3)
#else
          unk(6,i1,i2,i3,jb)=zc2*b1pv(nb1,j2,j3)
          unk(7,i1,i2,i3,jb)=zc2*b2pv(nb1,j2,j3)
          unk(8,i1,i2,i3,jb)=zc1*b3pv(nb1,j2,j3)
          iv=8
#endif
#endif
!L       4.6       Cloud tracer
#ifdef hydcon
          iv=iv+1
          unk(iv,i1,i2,i3,jb)=zc2*dppv(nb1,j2,j3)
#endif
!L       4.7       Polarity tracer
#ifdef mhdcon
          iv=iv+1
          unk(iv,i1,i2,i3,jb)=zc2*bppv(nb1,j2,j3)
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!L     5.        Div(B) correction
#ifdef mhd
    if(ndivb.eq.1) then
      do j3=1,ncc3
        i3=ng3+j3
        do j2=1,ncc2
          i2=ng2+j2
          do j1=1,ng1
#ifndef bc2tim
            unk(6,j1,i2,i3,jb)=b1pv(j2,j3)
            unk(7,j1,i2,i3,jb)=b2pv(j2,j3)
            unk(8,j1,i2,i3,jb)=b3pv(j2,j3)
#else
            unk(6,j1,i2,i3,jb)=b1pv(j1,j2,j3)
            unk(7,j1,i2,i3,jb)=b2pv(j1,j2,j3)
            unk(8,j1,i2,i3,jb)=b3pv(j1,j2,j3)
#endif
          enddo
        enddo
      enddo
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          do j1=nc1l,nc1r
#ifdef x2dim
              unk(6,j1,j2,j3,jb)= &
                (ax1(j1-2,jb)/ax1(j1,jb))*unk(6,j1-2,j2,j3,jb) &
#ifdef cartesian
                +(vol1(j1-1,jb)/ax1(j1,jb))*rdx2(j2,jb) &
                *(-ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb)    &
#ifdef x3dim
                  +ax2(j2-1,jb)*unk(6,j1,j2-1,j3,jb))
#else
                  +ax2(j2-1,jb)*unk(6,j1,j2-1,j3,jb))
#endif
#endif
#ifdef cylindrical
                +(vol1(j1-1,jb)/ax1(j1,jb))*rx1(j1-1,jb) &
                *rdx2(j2,jb)*(-ax2(j2+1,jb)             &
                *unk(7,j1,j2+1,j3,jb)+ax2(j2-1,jb)      &
#ifdef x3dim
                  *unk(6,j1,j2-1,j3,jb))
#else
                  *unk(6,j1,j2-1,j3,jb))
#endif
#endif
#ifdef spherical
                +(vol1(j1-1,jb)/ax1(j1,jb))*rx1(j1-1,jb) &
                *rsint(j2,jb)*rdx2(j2,jb)                &
                *(-ax2(j2+1,jb)*unk(7,j1,j2+1,j3,jb)     &
#ifdef x3dim
                  +ax2(j2-1,jb)*unk(6,j1,j2-1,j3,jb))
#else
                  +ax2(j2-1,jb)*unk(6,j1,j2-1,j3,jb))
#endif
#endif
#endif
#ifdef x3dim
              unk(6,j1,j2,j3,jb)=unk(6,j1,j2,j3,jb) &
                +(ax1(j1-2,jb)/ax1(j1,jb))*unk(6,j1-2,j2,j3,jb) &
#ifdef cartesian
                +(vol1(j1-1,jb)/ax1(j1,jb))*rdx3(j3,jb)      &
                *(-unk(8,j1,j2,j3+1,jb)+unk(8,j1,j2,j3-1,jb))
#endif
#ifdef cylindrical
                 +(vol1(j1-1,jb)/ax1(j1,jb))*rdx3(j3,jb)      &
                 *(-unk(8,j1,j2,j3+1,jb)+unk(8,j1,j2,j3-1,jb))
#endif
#ifdef spherical
                +(vol1(j1-1,jb)/ax1(j1,jb))*rx1(j1-1,jb)      &
                *rsint(j2,jb)*rdx3(j3,jb)                     &
                *(-unk(8,j1,j2,j3+1,jb)+unk(8,j1,j2,j3-1,jb))
#endif
#endif
          enddo
        enddo
      enddo
    endif
#endif
  enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2007,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE BNDINI
