!****f* e0pro/main
! NAME 
!   MGRD
! PURPOSE
!   Set multi-grid coordinates
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE MGRD
!---------------------------------------------------------------------
  USE comoly
  USE commax
  USE comdim
  USE comreg
  USE comphy
  USE comevo
  USE comtva
  IMPLICIT NONE
#ifdef NGRID
  REAL(8) :: zx1hlf, zx2hlf, zx3hlf
  REAL(8) :: zdx1, zdx2, zdx3
  INTEGER :: jb
#endif
#ifdef DIAGNO
  call EXPERT(1001,1)
#endif
!---------------------------------------------------------------------
#ifdef NGRID
!L     1.        Computational region
  x2geo(1)=90.2d0*0.017453293d0
!L       1.1       Boundary positions
  x1l=0.1d0*vux1
  x1r=1.9d0*vux1
  x2l=0d0*vux2
  x2r=180d0*vux2
  x3l=0d0*vux2
  x3r=360d0*vux2
  zx1hlf=0.5d0*(x1l+x1r)
  zx2hlf=0.5d0*(x2l+x2r)
  zx3hlf=0.5d0*(x3l+x3r)
!L       1.2       Level 1
  xx1l(1)=x1l
  xx1r(1)=x1r
  xx2l(1)=x2l
  xx2r(1)=x2r
  xx3l(1)=x3l
  xx3r(1)=x3r
  nparent(1)=0
  nlevel(1,1)=1
  nlevel(2,1)=1
  nlevel(3,1)=1
  nbc1l(1,1)=-1
  nbc1r(1,1)=-2
  nbc2l(1,1)=-2
  nbc2r(1,1)=-2
  nbc3l(1,1)=-4
  nbc3r(1,1)=-4
!---------------------------------------------------------------------
!L     2.        Meridional refinement
!L       2.1       Level 2
  if(nblk.ge.3) then
    zdx1=(x1r-x1l)/2d0
    zdx2=(x2r-x2l)/4d0
    zdx3=(x3r-x3l)/4d0
    xx1l(2)=x1l
    xx1l(3)=x1l+zdx1
    xx1r(2)=x1l+zdx1
    xx1r(3)=x1l+2d0*zdx1
    xx2l(2:3)=zx2hlf-zdx2
    xx2r(2:3)=zx2hlf+zdx2
    xx3l(2:3)=zx3hlf-zdx3
    xx3r(2:3)=zx3hlf+zdx3
    nparent(2:3)=1
    nlevel(1,2:3)=2
    nlevel(2,2:3)=2
    nlevel(3,2:3)=2
    nbc1l(1,2)=-1
    nbc1l(1,3)=2
    nbc1r(1,2)=3
    nbc1r(1,3)=-2
    nbc2l(1,2:3)=1
    nbc2r(1,2:3)=1
    nbc3l(1,2:3)=1
    nbc3r(1,2:3)=1
    nbc2l(2,2)=-3
    nbc2l(2,3)=3
    nbc2r(2,2)=-4
    nbc2r(2,3)=4
    nbc3l(2,2)=-3
    nbc3l(2,3)=3
    nbc3r(2,2)=-4
    nbc3r(2,3)=4
  endif
!L       2.1       Level 3
  if(nblk.ge.7) then
    zdx1=(x1r-x1l)/4d0
    zdx2=(x2r-x2l)/8d0
    zdx3=(x3r-x3l)/8d0
    xx1l(4)=x1l
    xx1l(5)=x1l+zdx1
    xx1l(6)=x1l+2d0*zdx1
    xx1l(7)=x1l+3d0*zdx1
    xx1r(4)=x1l+zdx1
    xx1r(5)=x1l+2d0*zdx1
    xx1r(6)=x1l+3d0*zdx1
    xx1r(7)=x1l+4d0*zdx1
    xx2l(4:7)=zx2hlf-zdx2
    xx2r(4:7)=zx2hlf+zdx2
    xx3l(4:7)=zx3hlf-zdx3
    xx3r(4:7)=zx3hlf+zdx3
    nparent(4:5)=2
    nparent(6:7)=3
    nlevel(1,4:7)=3
    nlevel(2,4:7)=3
    nlevel(3,4:7)=3
    nbc1l(1,4)=-1
    nbc1l(1,5)=4
    nbc1l(1,6)=5
    nbc1l(1,7)=6
    nbc1r(1,4)=5
    nbc1r(1,5)=6
    nbc1r(1,6)=7
    nbc1r(1,7)=-2
    nbc2l(1,4:5)=2
    nbc2l(1,6:7)=3
    nbc2r(1,4:5)=2
    nbc2r(1,6:7)=3
    nbc3l(1,4:5)=2
    nbc3l(1,6:7)=3
    nbc3r(1,4:5)=2
    nbc3r(1,6:7)=3
    nbc2l(2,4)=-3
    nbc2l(2,5)=3
    nbc2l(2,6)=-3
    nbc2l(2,7)=3
    nbc2r(2,4)=-4
    nbc2r(2,5)=4
    nbc2r(2,6)=-4
    nbc2r(2,7)=4
    nbc3l(2,4)=-3
    nbc3l(2,5)=3
    nbc3l(2,6)=-3
    nbc3l(2,7)=3
    nbc3r(2,4)=-4
    nbc3r(2,5)=4
    nbc3r(2,6)=-4
    nbc3r(2,7)=4
  endif
!L       2.3       Level 4
  if(nblk.ge.15) then
    zdx1=(x1r-x1l)/8d0
    zdx2=(x2r-x2l)/16d0
    zdx3=(x3r-x3l)/16d0
    xx1l(8)=x1l
    xx1l(9)=x1l+zdx1
    xx1l(10)=x1l+2d0*zdx1
    xx1l(11)=x1l+3d0*zdx1
    xx1l(12)=x1l+4d0*zdx1
    xx1l(13)=x1l+5d0*zdx1
    xx1l(14)=x1l+6d0*zdx1
    xx1l(15)=x1l+7d0*zdx1
    xx1r(8)=x1l+zdx1
    xx1r(9)=x1l+2d0*zdx1
    xx1r(10)=x1l+3d0*zdx1
    xx1r(11)=x1l+4d0*zdx1
    xx1r(12)=x1l+5d0*zdx1
    xx1r(13)=x1l+6d0*zdx1
    xx1r(14)=x1l+7d0*zdx1
    xx1r(15)=x1l+8d0*zdx1
    xx2l(8:15)=zx2hlf-zdx2
    xx2r(8:15)=zx2hlf+zdx2
    xx3l(8:15)=zx3hlf-zdx3
    xx3r(8:15)=zx3hlf+zdx3
    nparent(8:9)=4
    nparent(10:11)=5
    nparent(12:13)=6
    nparent(14:15)=7
    nlevel(1,8:15)=4
    nlevel(2,8:15)=4
    nlevel(3,8:15)=4
    nbc1l(1,8)=-1
    nbc1l(1,9)=8
    nbc1l(1,10)=9
    nbc1l(1,11)=10
    nbc1l(1,12)=11
    nbc1l(1,13)=12
    nbc1l(1,14)=13
    nbc1l(1,15)=14
    nbc1r(1,8)=9
    nbc1r(1,9)=10
    nbc1r(1,10)=11
    nbc1r(1,11)=12
    nbc1r(1,12)=13
    nbc1r(1,13)=14
    nbc1r(1,14)=15
    nbc1r(1,15)=-2
    nbc2l(1,8:9)=4
    nbc2l(1,10:11)=5
    nbc2l(1,12:13)=6
    nbc2l(1,14:15)=7
    nbc2r(1,8:9)=4
    nbc2r(1,10:11)=5
    nbc2r(1,12:13)=6
    nbc2r(1,14:15)=7
    nbc3l(1,8:9)=4
    nbc3l(1,10:11)=5
    nbc3l(1,12:13)=6
    nbc3l(1,14:15)=7
    nbc3r(1,8:9)=4
    nbc3r(1,10:11)=5
    nbc3r(1,12:13)=6
    nbc3r(1,14:15)=7
    nbc2l(2,8)=-3
    nbc2l(2,9)=3
    nbc2l(2,10)=-3
    nbc2l(2,11)=3
    nbc2l(2,12)=-3
    nbc2l(2,13)=3
    nbc2l(2,14)=-3
    nbc2l(2,15)=3
    nbc2r(2,8)=-4
    nbc2r(2,9)=4
    nbc2r(2,10)=-4
    nbc2r(2,11)=4
    nbc2r(2,12)=-4
    nbc2r(2,13)=4
    nbc2r(2,14)=-4
    nbc2r(2,15)=4
    nbc3l(2,8)=-3
    nbc3l(2,9)=3
    nbc3l(2,10)=-3
    nbc3l(2,11)=3
    nbc3l(2,12)=-3
    nbc3l(2,13)=3
    nbc3l(2,14)=-3
    nbc3l(2,15)=3
    nbc3r(2,8)=-4
    nbc3r(2,9)=4
    nbc3r(2,10)=-4
    nbc3r(2,11)=4
    nbc3r(2,12)=-4
    nbc3r(2,13)=4
    nbc3r(2,14)=-4
    nbc3r(2,15)=4
  endif
!---------------------------------------------------------------------
!L     3.        Angular refinement
!L       3.1       Level 5
  if(nblk.ge.22) then
    zdx1=(x1r-x1l)/8d0
    zdx2=(x2r-x2l)/32d0
    zdx3=(x3r-x3l)/32d0
    xx1l(16)=x1l+zdx1
    xx1l(17)=x1l+2d0*zdx1
    xx1l(18)=x1l+3d0*zdx1
    xx1l(19)=x1l+4d0*zdx1
    xx1l(20)=x1l+5d0*zdx1
    xx1l(21)=x1l+6d0*zdx1
    xx1l(22)=x1l+7d0*zdx1
    xx1r(16)=x1l+2d0*zdx1
    xx1r(17)=x1l+3d0*zdx1
    xx1r(18)=x1l+4d0*zdx1
    xx1r(19)=x1l+5d0*zdx1
    xx1r(20)=x1l+6d0*zdx1
    xx1r(21)=x1l+7d0*zdx1
    xx1r(22)=x1l+8d0*zdx1
    xx2l(16:22)=zx2hlf-zdx2
    xx2r(16:22)=zx2hlf+zdx2
    xx3l(16:22)=zx3hlf-zdx3
    xx3r(16:22)=zx3hlf+zdx3
    nparent(16)=9
    nparent(17)=10
    nparent(18)=11
    nparent(19)=12
    nparent(20)=13
    nparent(21)=14
    nparent(22)=15
    nlevel(1,16:22)=4
    nlevel(2,16:22)=5
    nlevel(3,16:22)=5
    nbc1l(1,16)=8
    nbc1l(1,17)=16
    nbc1l(1,18)=17
    nbc1l(1,19)=18
    nbc1l(1,20)=19
    nbc1l(1,21)=20
    nbc1l(1,22)=21
    nbc1r(1,16)=17
    nbc1r(1,17)=18
    nbc1r(1,18)=19
    nbc1r(1,19)=20
    nbc1r(1,20)=21
    nbc1r(1,21)=22
    nbc1r(1,22)=-2
    nbc2l(1,16)=9
    nbc2l(1,17)=10
    nbc2l(1,18)=11
    nbc2l(1,19)=12
    nbc2l(1,20)=13
    nbc2l(1,21)=14
    nbc2l(1,22)=15
    nbc2r(1,16)=9
    nbc2r(1,17)=10
    nbc2r(1,18)=11
    nbc2r(1,19)=12
    nbc2r(1,20)=13
    nbc2r(1,21)=14
    nbc2r(1,22)=15
    nbc2l(2,16)=-3
    nbc2l(2,17)=-3
    nbc2l(2,18)=-3
    nbc2l(2,19)=-3
    nbc2l(2,20)=-3
    nbc2l(2,21)=-3
    nbc2l(2,22)=-3
    nbc3l(1,16)=9
    nbc3l(1,17)=10
    nbc3l(1,18)=11
    nbc3l(1,19)=12
    nbc3l(1,20)=13
    nbc3l(1,21)=14
    nbc3l(1,22)=15
    nbc3r(1,16)=9
    nbc3r(1,17)=10
    nbc3r(1,18)=11
    nbc3r(1,19)=12
    nbc3r(1,20)=13
    nbc3r(1,21)=14
    nbc3r(1,22)=15
    nbc2r(2,16)=-4
    nbc2r(2,17)=-4
    nbc2r(2,18)=-4
    nbc2r(2,19)=-4
    nbc2r(2,20)=-4
    nbc2r(2,21)=-4
    nbc2r(2,22)=-4
    nbc3l(2,16)=-3
    nbc3l(2,17)=-3
    nbc3l(2,18)=-3
    nbc3l(2,19)=-3
    nbc3l(2,20)=-3
    nbc3l(2,21)=-3
    nbc3l(2,22)=-3
    nbc3r(2,16)=-4
    nbc3r(2,17)=-4
    nbc3r(2,18)=-4
    nbc3r(2,19)=-4
    nbc3r(2,20)=-4
    nbc3r(2,21)=-4
    nbc3r(2,22)=-4
  endif
!L       3.2       Level 6
  if(nblk.ge.28) then
    zdx1=(x1r-x1l)/8d0
    zdx2=(x2r-x2l)/64d0
    zdx3=(x3r-x3l)/64d0
    xx1l(23)=x1l+2d0*zdx1
    xx1l(24)=x1l+3d0*zdx1
    xx1l(25)=x1l+4d0*zdx1
    xx1l(26)=x1l+5d0*zdx1
    xx1l(27)=x1l+6d0*zdx1
    xx1l(28)=x1l+7d0*zdx1
    xx1r(23)=x1l+3d0*zdx1
    xx1r(24)=x1l+4d0*zdx1
    xx1r(25)=x1l+5d0*zdx1
    xx1r(26)=x1l+6d0*zdx1
    xx1r(27)=x1l+7d0*zdx1
    xx1r(28)=x1l+8d0*zdx1
    xx2l(23:28)=zx2hlf-zdx2
    xx2r(23:28)=zx2hlf+zdx2
    xx3l(23:28)=zx3hlf-zdx3
    xx3r(23:28)=zx3hlf+zdx3
    nparent(23)=17
    nparent(24)=18
    nparent(25)=19
    nparent(26)=20
    nparent(27)=21
    nparent(28)=22
    nlevel(1,23:28)=4
    nlevel(2,23:28)=6
    nlevel(3,23:28)=6
    nbc1l(1,23)=16
    nbc1l(1,24)=23
    nbc1l(1,25)=24
    nbc1l(1,26)=25
    nbc1l(1,27)=26
    nbc1l(1,28)=27
    nbc1r(1,23)=24
    nbc1r(1,24)=25
    nbc1r(1,25)=26
    nbc1r(1,26)=27
    nbc1r(1,27)=28
    nbc1r(1,28)=-2
    nbc2l(1,23)=17
    nbc2l(1,24)=18
    nbc2l(1,25)=19
    nbc2l(1,26)=20
    nbc2l(1,27)=21
    nbc2l(1,28)=22
    nbc2r(1,23)=17
    nbc2r(1,24)=18
    nbc2r(1,25)=19
    nbc2r(1,26)=20
    nbc2r(1,27)=21
    nbc2r(1,28)=22
    nbc3l(1,23)=17
    nbc3l(1,24)=18
    nbc3l(1,25)=19
    nbc3l(1,26)=20
    nbc3l(1,27)=21
    nbc3l(1,28)=22
    nbc3r(1,23)=17
    nbc3r(1,24)=18
    nbc3r(1,25)=19
    nbc3r(1,26)=20
    nbc3r(1,27)=21
    nbc3r(1,28)=22
    nbc2l(2,23)=-3
    nbc2l(2,24)=-3
    nbc2l(2,25)=-3
    nbc2l(2,26)=-3
    nbc2l(2,27)=-3
    nbc2l(2,28)=-3
    nbc2r(2,23)=-4
    nbc2r(2,24)=-4
    nbc2r(2,25)=-4
    nbc2r(2,26)=-4
    nbc2r(2,27)=-4
    nbc2r(2,28)=-4
    nbc3l(2,23)=-3
    nbc3l(2,24)=-3
    nbc3l(2,25)=-3
    nbc3l(2,26)=-3
    nbc3l(2,27)=-3
    nbc3l(2,28)=-3
    nbc3r(2,23)=-4
    nbc3r(2,24)=-4
    nbc3r(2,25)=-4
    nbc3r(2,26)=-4
    nbc3r(2,27)=-4
    nbc3r(2,28)=-4
  endif
!---------------------------------------------------------------------
!L     4.        Geospace refinement
!L       4.1       Level 7
  if(nblk.ge.32) then
    zdx1=(x1r-x1l)/16d0
    zdx2=(x2r-x2l)/128d0
    zdx3=(x3r-x3l)/128d0
    xx1l(29)=zx1hlf-zdx1-zdx1
    xx1l(30)=zx1hlf-zdx1
    xx1l(31)=zx1hlf
    xx1l(32)=zx1hlf+zdx1
    xx1r(29)=zx1hlf-zdx1
    xx1r(30)=zx1hlf
    xx1r(31)=zx1hlf+zdx1
    xx1r(32)=zx1hlf+zdx1+zdx1
    xx2l(29:32)=zx2hlf-zdx2
    xx2r(29:32)=zx2hlf+zdx2
    xx3l(29:32)=zx3hlf-zdx3
    xx3r(29:32)=zx3hlf+zdx3
    nparent(29:30)=24
    nparent(31:32)=25
    nlevel(1,29:32)=5
    nlevel(2,29:32)=7
    nlevel(3,29:32)=7
    nbc1l(1,29)=23
    nbc1l(1,30)=29
    nbc1l(1,31)=30
    nbc1l(1,32)=31
    nbc1r(1,29)=30
    nbc1r(1,30)=31
    nbc1r(1,31)=32
    nbc1r(1,32)=26
    nbc2l(1,29:30)=24
    nbc2l(1,31:32)=25
    nbc2r(1,29:30)=24
    nbc2r(1,31:32)=25
    nbc3l(1,29:30)=24
    nbc3l(1,31:32)=25
    nbc3r(1,29:30)=24
    nbc3r(1,31:32)=25
    nbc2l(2,29)=-3
    nbc2l(2,30)=3
    nbc2l(2,31)=-3
    nbc2l(2,32)=3
    nbc2r(2,29)=-4
    nbc2r(2,30)=4
    nbc2r(2,31)=-4
    nbc2r(2,32)=4
    nbc3l(2,29)=-3
    nbc3l(2,30)=3
    nbc3l(2,31)=-3
    nbc3l(2,32)=3
    nbc3r(2,29)=-4
    nbc3r(2,30)=4
    nbc3r(2,31)=-4
    nbc3r(2,32)=4
  endif
!L       4.2       Level 8
  if(nblk.ge.36) then
    zdx1=(x1r-x1l)/32d0
    zdx2=(x2r-x2l)/256d0
    zdx3=(x3r-x3l)/256d0
    xx1l(33)=zx1hlf-zdx1-zdx1
    xx1l(34)=zx1hlf-zdx1
    xx1l(35)=zx1hlf
    xx1l(36)=zx1hlf+zdx1
    xx1r(33)=zx1hlf-zdx1
    xx1r(34)=zx1hlf
    xx1r(35)=zx1hlf+zdx1
    xx1r(36)=zx1hlf+zdx1+zdx1
    xx2l(33:36)=zx2hlf-zdx2
    xx2r(33:36)=zx2hlf+zdx2
    xx3l(33:36)=zx3hlf-zdx3
    xx3r(33:36)=zx3hlf+zdx3
    nparent(33:34)=30
    nparent(35:36)=31
    nlevel(1,33:36)=6
    nlevel(2,33:36)=8
    nlevel(3,33:36)=8
    nbc1l(1,33)=29
    nbc1l(1,34)=33
    nbc1l(1,35)=34
    nbc1l(1,36)=35
    nbc1r(1,33)=34
    nbc1r(1,34)=35
    nbc1r(1,35)=36
    nbc1r(1,36)=32
    nbc2l(1,33:34)=30
    nbc2l(1,35:36)=31
    nbc2r(1,33:34)=30
    nbc2r(1,35:36)=31
    nbc3l(1,33:34)=30
    nbc3l(1,35:36)=31
    nbc3r(1,33:34)=30
    nbc3r(1,35:36)=31
    nbc2l(2,33)=-3
    nbc2l(2,34)=3
    nbc2l(2,35)=-3
    nbc2l(2,36)=3
    nbc2r(2,33)=-4
    nbc2r(2,34)=4
    nbc2r(2,35)=-4
    nbc2r(2,36)=4
    nbc3l(2,33)=-3
    nbc3l(2,34)=3
    nbc3l(2,35)=-3
    nbc3l(2,36)=3
    nbc3r(2,33)=-4
    nbc3r(2,34)=4
    nbc3r(2,35)=-4
    nbc3r(2,36)=4
  endif
!L       4.3       Level 9
  if(nblk.ge.40) then
    zdx1=(x1r-x1l)/64d0
    zdx2=(x2r-x2l)/512d0
    zdx3=(x3r-x3l)/512d0
    xx1l(37)=zx1hlf-zdx1-zdx1
    xx1l(38)=zx1hlf-zdx1
    xx1l(39)=zx1hlf
    xx1l(40)=zx1hlf+zdx1
    xx1r(37)=zx1hlf-zdx1
    xx1r(38)=zx1hlf
    xx1r(39)=zx1hlf+zdx1
    xx1r(40)=zx1hlf+zdx1+zdx1
    xx2l(37:40)=zx2hlf-zdx2
    xx2r(37:40)=zx2hlf+zdx2
    xx3l(37:40)=zx3hlf-zdx3
    xx3r(37:40)=zx3hlf+zdx3
    nparent(37:38)=34
    nparent(39:40)=35
    nlevel(1,37:40)=7
    nlevel(2,37:40)=9
    nlevel(3,37:40)=9
    nbc1l(1,37)=33
    nbc1l(1,38)=37
    nbc1l(1,39)=38
    nbc1l(1,40)=39
    nbc1r(1,37)=38
    nbc1r(1,38)=39
    nbc1r(1,39)=40
    nbc1r(1,40)=36
    nbc2l(1,37:38)=34
    nbc2l(1,39:40)=35
    nbc2r(1,37:38)=34
    nbc2r(1,39:40)=35
    nbc2l(2,37)=-3
    nbc2l(2,38)=3
    nbc2l(2,39)=-3
    nbc2l(2,40)=3
    nbc2r(2,37)=-4
    nbc2r(2,38)=4
    nbc2r(2,39)=-4
    nbc2r(2,40)=4
    nbc3l(2,37)=-3
    nbc3l(2,38)=3
    nbc3l(2,39)=-3
    nbc3l(2,40)=3
    nbc3r(2,37)=-4
    nbc3r(2,38)=4
    nbc3r(2,39)=-4
    nbc3r(2,40)=4
  endif
do jb=1,nblk
write(*,*) jb,nlevel(1,jb),'L',xx1l(jb)/1.496e11,nbc1l(1:2,jb),'R',xx1r(jb)/1.496e11,nbc1r(1:2,jb)
write(*,*) jb,nlevel(2,jb),'L',xx2l(jb)*57.296,nbc2l(1:2,jb),'R',xx2r(jb)*57.296,nbc2r(1:2,jb)
enddo
!---------------------------------------------------------------------
!L     9.        Return
#endif
#ifdef DIAGNO
  call EXPERT(1001,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE MGRD
