!****f* e0pro/main
! NAME 
!   RESINI
! PURPOSE
!   Resume at time level                                              
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESINI
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  USE paramesh_dimensions
  USE physicaldata
  USE tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comout
  USE comglo
  USE netcdf
  IMPLICIT NONE
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
  INTEGER, DIMENSION(2) :: istart1, icount1
  INTEGER, DIMENSION(2) :: istart2, icount2
  INTEGER, DIMENSION(2) :: istart3, icount3
  INTEGER, DIMENSION(3) :: istartg, icountg
  DATA istart1/1,1/, icount1/ncc1,1/
  DATA istart2/1,1/, icount2/ncc1,1/
  DATA istart3/1,1/, icount3/ncc1,1/
  DATA istartg/1,1,1/, icountg/ncc1,ncc2,1/
  INTEGER :: jj1,jj2,jj3
#endif
  REAL(8) :: zstart,zdt
  INTEGER ::  iret,ifile,istep
  INTEGER :: icc1,icc2,icc3,ihh1,ihh2,ihh3
  INTEGER ::  j1,j2,j3,jb
  INTEGER ::  iblk
  INTEGER, DIMENSION(4) :: istart, icount
  DATA istart/1,1,1,1/, icount/ncc1,ncc2,ncc3,1/
  CHARACTER(len=nf90_max_name) :: iname
#ifdef staggered
  INTEGER, DIMENSION(4) :: istart1h, icount1h
  INTEGER, DIMENSION(4) :: istart2h, icount2h
  INTEGER, DIMENSION(4) :: istart3h, icount3h
  DATA istart1h/1,1,1,1/, icount1h/nhh1,ncc2,ncc3,1/
  DATA istart2h/1,1,1,1/, icount2h/ncc1,nhh2,ncc3,1/
  DATA istart3h/1,1,1,1/, icount3h/ncc1,ncc2,nhh3,1/
#endif
#ifdef DIAGNO
  call EXPERT(2004,1)
#endif
!---------------------------------------------------------------------
!L     1.        Open the file
  iret=NF90_OPEN(lfini,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'initial',laini)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'refdate_mjd',refdate)
  if(myproc.eq.0) write(*,*) '   reference date MJD =',refdate
!---------------------------------------------------------------------
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,icc1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,icc2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,icc3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ihh1)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ihh2)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ihh3)
  iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,iblk)
!---------------------------------------------------------------------
  write(*,*) 'idim',icc1,icc2,icc3,ihh1,ihh2,ihh3,iblk
  write(*,*) 'ndim',ncc1,ncc2,ncc3,nhh1,nhh2,nhh3,nblk
  istart(1)=1+myproc*ncc1
#ifdef staggered
  istart1h(1)=1+myproc*ncc1
  istart2h(1)=1+myproc*ncc1
  istart3h(1)=1+myproc*ncc1
#endif
!---------------------------------------------------------------------
!L     3.        Get values
  iret=NF90_GET_VAR(ifile,7,zstart)
  iret=NF90_GET_VAR(ifile,8,zdt)
  iret=NF90_GET_VAR(ifile,9,istep)
#ifdef PARAMESH
!---------------------------------------------------------------------
!L     3.        Get values
!L       3.1       Cell positions
  istart3(1)=1-ncc3
  do jj3=1,icc3/ncc3
    istart3(1)=istart3(1)+ncc3
    iret=NF90_GET_VAR(ifile,1,istart3,icount3,zzx3)
    istart2(1)=1-ncc2
    do jj2=1,icc2/ncc2
      istart2(1)=istart2(1)+ncc2
      iret=NF90_GET_VAR(ifile,1,istart2,icount2,zzx2)
      istart1(1)=1-ncc1
      do jj1=1,icc1/ncc1
        istart1(1)=istart1(1)+ncc1
        iret=NF90_GET_VAR(ifile,1,istart1,icount1,zzx1)
!L       3.2       Check block correspondence
        do jb=1,lnblocks
          if(nodetype(jb).eq.1) then
            if(zzx1(1).gt.bnd_box(1,1,jb)) then
              if(zzx1(1).lt.bnd_box(2,1,jb)) then
                if(zzx2(1).gt.bnd_box(1,2,jb)) then
                  if(zzx2(1).lt.bnd_box(2,2,jb)) then
#ifdef x3dim
                    if(zzx3(1).gt.bnd_box(1,3,jb)) then
                      if(zzx3(1).lt.bnd_box(2,3,jb)) then
#endif
                        istart(1)=istart1(1)
                        istart(2)=istart2(1)
                        istart(3)=istart3(1)
#ifdef staggered
                        istart1h(1)=istart1(1)
                        istart1h(2)=istart2(1)
                        istart1h(3)=istart3(1)
                        istart2h(1)=istart1(1)
                        istart2h(2)=istart2(1)
                        istart2h(3)=istart3(1)
                        istart3h(1)=istart1(1)
                        istart3h(2)=istart2(1)
                        istart3h(3)=istart3(1)
#endif
!L       3.3       Density
                        iret=NF90_GET_VAR(ifile,10,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
!L       3.4       Temperature
                        iret=NF90_GET_VAR(ifile,11,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(2,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
!L       3.5       X1-velocity
                        iret=NF90_GET_VAR(ifile,12,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(3,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
!L       3.6       X2-velocity
                        iret=NF90_GET_VAR(ifile,13,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(4,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
!L       3.7       X3-velocity
                        iret=NF90_GET_VAR(ifile,14,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(5,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#ifdef mhd
!L       3.5       X1-magnetic field
#ifdef staggered
                        iret=NF90_GET_VAR(ifile,15,istart1h,icount1h,zzxh23)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,nhh1
                              facevarx(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzxh23(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#else
                        iret=NF90_GET_VAR(ifile,15,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(6,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#endif
!L       3.6       X2-magnetic field
#ifdef staggered
                        iret=NF90_GET_VAR(ifile,16,istart2h,icount2h,zzx1h3)
                        do j3=1,ncc3
                          do j2=1,nhh2
                            do j1=1,ncc1
                              facevary(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx1h3(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#else
                        iret=NF90_GET_VAR(ifile,16,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(7,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#endif
!L       3.7       X3-magnetic field
#ifdef staggered
                        iret=NF90_GET_VAR(ifile 17,istart3h,icount3h,zzx12h)
#ifdef x3dim
                        do j3=1,nhh3
#else
                        do j3=1,ncc3
#endif
                          do j2=1,ncc2
                            do j1=1,ncc1
                              facevarz(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx12h(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#else
                        iret=NF90_GET_VAR(ifile,17,istart,icount,zzx123)
                        do j3=1,ncc3
                          do j2=1,ncc2
                            do j1=1,ncc1
                              unk(8,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
                            enddo
                          enddo
                        enddo
#endif
#endif
!L       3.8       End of if and do
                      endif
                    endif
                  endif
                endif
              endif
#ifdef x3dim
            endif
          endif
#endif
        enddo
      enddo
    enddo
  enddo
#else
!---------------------------------------------------------------------
!L     3.        Get values
    jb=1
!L       3.1       Cell positions
    iret=NF90_GET_VAR(ifile,1,zx1)
    iret=NF90_GET_VAR(ifile,2,zx2)
    iret=NF90_GET_VAR(ifile,3,zx3)
    iret=NF90_GET_VAR(ifile,4,zh1)
    iret=NF90_GET_VAR(ifile,5,zh2)
    iret=NF90_GET_VAR(ifile,6,zh3)
    do j1=1,ncc1
      xc1(ng1+j1,jb)=zx1(j1,jb)
    enddo
    do j2=1,ncc2
      xc2(ng2+j2,jb)=zx2(j2,jb)
    enddo
    do j3=1,ncc3
      xc3(ng3+j3,jb)=zx3(j3,jb)
    enddo
    do j1=1,nhh1
      xh1(ng1+j1,jb)=zh1(j1,jb)
    enddo
    do j2=1,nhh2
      xh2(ng2+j2,jb)=zh2(j2,jb)
    enddo
    do j3=1,nhh3
      xh3(ng3+j3,jb)=zh3(j3,jb)
    enddo
!L       3.3       Density
    iret=NF90_GET_VAR(ifile,10,zzx123,start=istart,count=icount)
    do j3=1,ncc3
      do j2=1,ncc2
        do j1=1,ncc1
          unk(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
        enddo
      enddo
    enddo
!L       3.4       Temperature
    iret=NF90_GET_VAR(ifile,11,zzx123,start=istart,count=icount)
    do j3=1,ncc3
      do j2=1,ncc2
        do j1=1,ncc1
          unk(2,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
        enddo
      enddo
    enddo
!L       3.5       X1-velocity
    iret=NF90_GET_VAR(ifile,12,zzx123,start=istart,count=icount)
    do j3=1,ncc3
      do j2=1,ncc2
        do j1=1,ncc1
          unk(3,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
        enddo
      enddo
    enddo
!L       3.6       X2-velocity
    iret=NF90_GET_VAR(ifile,13,zzx123,start=istart,count=icount)
    do j3=1,ncc3
      do j2=1,ncc2
        do j1=1,ncc1
          unk(4,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
        enddo
      enddo
    enddo
!L       3.7       X3-velocity
    iret=NF90_GET_VAR(ifile,14,zzx123,start=istart,count=icount)
    do j3=1,ncc3
      do j2=1,ncc2
        do j1=1,ncc1
          unk(5,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
        enddo
      enddo
    enddo
#ifdef mhd
!L       3.5       X1-magnetic field
#ifdef staggered
        iret=NF90_GET_VAR(ifile,15,zzxh23,start=istart1h,count=icount1h)
        do j3=1,ncc3
          do j2=1,ncc2
            do j1=1,nhh1
              facevarx(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzxh23(j1,j2,j3)
            enddo
          enddo
        enddo
#else
        iret=NF90_GET_VAR(ifile,15,zzx123,start=istart,count=icount)
        do j3=1,ncc3
          do j2=1,ncc2
            do j1=1,ncc1
              unk(6,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
            enddo
          enddo
        enddo
#endif
!L       3.6       X2-magnetic field
#ifdef staggered
        iret=NF90_GET_VAR(ifile,16,zzx1h3,start=istart2h,count=icount2h)
        do j3=1,ncc3
          do j2=1,nhh2
            do j1=1,ncc1
              facevary(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx1h3(j1,j2,j3)
            enddo
          enddo
        enddo
#else
        iret=NF90_GET_VAR(ifile,16,zzx123,start=istart,count=icount)
        do j3=1,ncc3
          do j2=1,ncc2
            do j1=1,ncc1
              unk(7,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
            enddo
          enddo
        enddo
#endif
!L       3.7       X3-magnetic field
#ifdef staggered
        iret=NF90_GET_VAR(ifile,17,zzx12h,start=istart3h,count=icount3h)
#ifdef x3dim
          do j3=1,nhh3
#else
          do j3=1,ncc3
#endif
            do j2=1,ncc2
              do j1=1,ncc1
                facevarz(1,ng1+j1,ng2+j2,ng3+j3,jb)=zzx12h(j1,j2,j3)
              enddo
            enddo
          enddo
#else
          iret=NF90_GET_VAR(ifile,17,zzx123,start=istart,count=icount)
          do j3=1,ncc3
            do j2=1,ncc2
              do j1=1,ncc1
                unk(8,ng1+j1,ng2+j2,ng3+j3,jb)=zzx123(j1,j2,j3)
              enddo
            enddo
          enddo
#endif
#endif
#endif
!---------------------------------------------------------------------
!L     4.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     5.        Write
  if(myproc.eq.0) then
    write(*,'(4x,a6,1pe12.4,3i5)') 'resini',real(zstart),nc1,nc2,nc3
  endif
#ifdef staggered
!---------------------------------------------------------------------
!L     6.        Cell-centered field values
!      do j3=1,ncc3
!        do j2=1,ncc2
!          do j1=1,ncc1
!            b1(j1,j2,j3)=zhalf*(b1hcc(j1+1,j2,j3)+b1hcc(j1,j2,j3))
!            b2(j1,j2,j3)=zhalf*(b2chc(j1,j2+1,j3)+b2chc(j1,j2,j3))
!            b3(j1,j2,j3)=zhalf*(b3cch(j1,j2,j3+1)+b3cch(j1,j2,j3))
!          enddo
!        enddo
!      enddo
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2004,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RESINI


!****f* e0pro/main
! NAME 
!   RESRES
! PURPOSE
!   Resume at time level
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESRES
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  USE paramesh_dimensions
  USE physicaldata
  USE tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comout
  USE comglo
  USE netcdf
  IMPLICIT NONE
  INTEGER :: iret,ifile
  INTEGER :: ic1,ic2,ic3,ih1,ih2,ih3
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
  call EXPERT(2004,1)
#endif
!---------------------------------------------------------------------
!L     1.        Open the file
  iret=NF90_OPEN(lfres,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'resume',lares)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'refdate_mjd',refdate)
  if(myproc.eq.0) write(*,*) '   reference date MJD =',refdate
!---------------------------------------------------------------------
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,ic1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,ic2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,ic3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ih1)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ih2)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ih3)
!---------------------------------------------------------------------
!L     3.        Get constant values
!L       3.1       Coordinates
  iret=NF90_GET_VAR(ifile,1,xc1)
  iret=NF90_GET_VAR(ifile,2,xc2)
  iret=NF90_GET_VAR(ifile,3,xc3)
!L       3.2       Coordinates
  iret=NF90_GET_VAR(ifile,4,xh1)
  iret=NF90_GET_VAR(ifile,5,xh2)
  iret=NF90_GET_VAR(ifile,6,xh3)
!L       3.3       Time
  iret=NF90_GET_VAR(ifile,7,time)
  iret=NF90_GET_VAR(ifile,8,dt)
  iret=NF90_GET_VAR(ifile,9,nstep)
!L       3.4       Variables
  iret=NF90_GET_VAR(ifile,10,unk)
#ifdef mhd
#ifdef staggered
  iret=NF90_GET_VAR(ifile,11,b1hcc)
  iret=NF90_GET_VAR(ifile,12,b2chc)
  iret=NF90_GET_VAR(ifile,13,b3cch)
#endif
#endif
!---------------------------------------------------------------------
!L     4.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     5.        Write
  write(*,'(4x,a6,1pe12.4,3i4)') 'resres',real(time),nc1,nc2,nc3
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2004,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RESRES


!****f* e0pro/main
! NAME 
!   RESBND
! PURPOSE
!   Resume at time level
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESBND
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comout
  USE comglo
  USE netcdf
  IMPLICIT NONE
  REAL(8) :: zdt
  INTEGER :: ib1,ib2,ib3
  INTEGER :: iret,ifile,id,istep
#ifdef PARAMESH
  INTEGER :: ibnd
#endif
  CHARACTER(len=nf90_max_name) :: iname
!---------------------------------------------------------------------
  INTEGER, DIMENSION(3) :: istart,  icount
  DATA  istart/1,1,1/,  icount/nbb2,nbb3,1/
#ifdef DIAGNO
  call EXPERT(2005,1)
#endif
!---------------------------------------------------------------------
  istart(3)=nbnd
!---------------------------------------------------------------------
!L     1.        Open the file
  iret=NF90_OPEN(lfbnd,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'boundary',labnd)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'refdate_mjd',refdate)
  if(myproc.eq.0) then
    if(nstep.eq.0) write(*,*) '   reference date MJD =',refdate
  endif
!---------------------------------------------------------------------
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,nb1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,nb2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,nb3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ib1)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ib2)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ib3)
#ifdef PARAMESH
  iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,ibnd)
#endif
!---------------------------------------------------------------------
!L     3.        Get time-dependent values
!L       3.2       Coordinates
  iret=NF90_GET_VAR(ifile,1,x1bn)
  iret=NF90_GET_VAR(ifile,2,x2bn)
  iret=NF90_GET_VAR(ifile,3,x3bn)
!L       3.3       Time
  iret=NF90_GET_VAR(ifile,7,timebn,start=(/nbnd/))
  iret=NF90_GET_VAR(ifile,8,zdt,start=(/nbnd/))
  iret=NF90_GET_VAR(ifile,9,istep,start=(/nbnd/))
!L       3.4       Variables
  iret=NF90_GET_VAR(ifile,10,dbn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,11,tbn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,12,v1bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,13,v2bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,14,v3bn,start=istart,count=icount)
  id=14
#ifdef mhd
  iret=NF90_GET_VAR(ifile,15,b1bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,16,b2bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,17,b3bn,istart,icount)
  id=17
#endif
#ifdef hydcon
  id=id+1
  iret=NF90_GET_VAR(ifile,id,dpbn,istart,icount)
#endif
#ifdef mhdcon
  id=id+1
  iret=NF90_GET_VAR(ifile,id,bpbn,istart,icount)
#endif
!---------------------------------------------------------------------
!L     4.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     5.        Write
  if(myproc.eq.0) then
    write(*,'(4x,a6,i4,1pe12.4,4i4)') 'resbnd',nbnd,real(timebn), &
      nb1,nb2,nb3
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RESBND


!****f* e0pro/main
! NAME 
!   RESPAS
! PURPOSE
!   Resume at time level
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESPAS
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  USE comout
  USE comglo
  USE netcdf
  IMPLICIT NONE
  REAL(8) :: zdt
  INTEGER :: ib1,ib2,ib3
  INTEGER :: iret,ifile,id,istep
#ifdef PARAMESH
  INTEGER :: ibnd
#endif
  CHARACTER(len=nf90_max_name) :: iname
!---------------------------------------------------------------------
  INTEGER, DIMENSION(4) :: istart,icount
  DATA  istart/1,1,1,1/, icount/nbb1,nbb2,nbb3,1/
#ifdef DIAGNO
  call EXPERT(2005,1)
#endif
!---------------------------------------------------------------------
  istart(4)=nbnd
!---------------------------------------------------------------------
!L     1.        Open the file
  iret=NF90_OPEN(lfbnd,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'passage',lapas)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'refdate_mjd',refdate)
  if(myproc.eq.0) write(*,*) '   reference date MJD =',refdate
!---------------------------------------------------------------------
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,nb1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,nb2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,nb3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ib1)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ib2)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ib3)
#ifdef PARAMESH
  iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,ibnd)
#endif
!---------------------------------------------------------------------
!L     3.        Get time-dependent values
!L       3.2       Coordinates
  iret=NF90_GET_VAR(ifile,1,x1bn)
  iret=NF90_GET_VAR(ifile,2,x2bn)
  iret=NF90_GET_VAR(ifile,3,x3bn)
!L       3.3       Time
  iret=NF90_GET_VAR(ifile,7,timebn,start=(/nbnd/))
  iret=NF90_GET_VAR(ifile,8,zdt,start=(/nbnd/))
  iret=NF90_GET_VAR(ifile,9,istep,start=(/nbnd/))
!L       3.4       Variables
  iret=NF90_GET_VAR(ifile,10,dbn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,11,tbn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,12,v1bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,13,v2bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,14,v3bn,start=istart,count=icount)
  id=14
#ifdef mhd
  iret=NF90_GET_VAR(ifile,15,b1bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,16,b2bn,start=istart,count=icount)
  iret=NF90_GET_VAR(ifile,17,b3bn,start=istart,count=icount)
  id=17
#endif
#ifdef hydcon
  id=id+1
  iret=NF90_GET_VAR(ifile,id,dpbn,start=istart,count=icount)
#endif
#ifdef mhdcon
  id=id+1
  iret=NF90_GET_VAR(ifile,id,bpbn,start=istart,count=icount)
#endif
!---------------------------------------------------------------------
!L     4.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     5.        Write
  if(myproc.eq.0) then
    write(*,'(4x,a6,i4,1pe12.4,4i4)') 'respas',nbnd,real(timebn), &
      nb1,nb2,nb3
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE RESPAS


!****f* e0pro/main
! NAME 
!   RESGRD
! PURPOSE
!   Resume at time level
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE RESGRD
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  USE paramesh_dimensions
  USE physicaldata
  USE tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE combnd
  USE comreg
  USE comout
  USE comglo
  USE netcdf
  IMPLICIT NONE
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
  INTEGER :: j
  INTEGER, DIMENSION(2) ::  istart1,icount1,istart1h,icount1h
  INTEGER, DIMENSION(2) ::  istart2,icount2,istart2h,icount2h
  INTEGER, DIMENSION(2) ::  istart3,icount3,istart3h,icount3h
  DATA istart1/1,1/,icount1/1,1/,istart1h/1,1/,icount1h/1,1/
  DATA istart2/1,1/,icount2/1,1/,istart2h/1,1/,icount2h/1,1/
  DATA istart3/1,1/,icount3/1,1/,istart3h/1,1/,icount3h/1,1/
#else
  INTEGER, DIMENSION(2) :: istart1,icount1,istart1h,icount1h
  DATA istart1/1,1/,icount1/ncc1,1/,istart1h/1,1/,icount1h/nhh1,1/
#endif
  INTEGER :: iret,ifile,iblk
  INTEGER :: icc1,icc2,icc3,ihh1,ihh2,ihh3,j1,j2,j3,jb
  CHARACTER(len=nf90_max_name) :: iname
#ifdef DIAGNO
  call EXPERT(2006,1)
#endif
!---------------------------------------------------------------------
!L     1.        Open the file
  iret=NF90_OPEN(lfgrd,nf90_nowrite,ifile)
  iret=NF90_GET_ATT(ifile,NF90_GLOBAL,'grid',lagrd)
!L     2.        Get array dimensions
  iret=NF90_INQUIRE_DIMENSION(ifile,1,iname,icc1)
  iret=NF90_INQUIRE_DIMENSION(ifile,2,iname,icc2)
  iret=NF90_INQUIRE_DIMENSION(ifile,3,iname,icc3)
  iret=NF90_INQUIRE_DIMENSION(ifile,4,iname,ihh1)
  iret=NF90_INQUIRE_DIMENSION(ifile,5,iname,ihh2)
  iret=NF90_INQUIRE_DIMENSION(ifile,6,iname,ihh3)
  iret=NF90_INQUIRE_DIMENSION(ifile,7,iname,iblk)
!---------------------------------------------------------------------
!L     4.        Set number of blocks
#ifdef PARAMESH
    if(iblk.eq.1) then
      lrefine_ini=1
      do while(icc1.gt.ncc1)
        icc1=icc1/2
        icc2=icc2/2
        icc3=icc3/2
        lrefine_ini=lrefine_ini+1
      enddo
      if(icc2.lt.1) icc2=1
      if(icc3.lt.1) icc3=1
    else
      nblk=iblk
    endif
!L       3.2       Block boundaries
    iret=NF90_GET_VAR(ifile,4,x1l,istart1h,icount1h)
    iret=NF90_GET_VAR(ifile,5,x2l,istart2h,icount2h)
    iret=NF90_GET_VAR(ifile,6,x3l,istart3h,icount3h)
    istart1h(1)=ihh1
    istart2h(1)=ihh2
    istart3h(1)=ihh3
    iret=NF90_GET_VAR(ifile,4,x1r,istart1h,icount1h)
    iret=NF90_GET_VAR(ifile,5,x2r,istart2h,icount2h)
    iret=NF90_GET_VAR(ifile,6,x3r,istart3h,icount3h)
!L       3.2       Block boundaries
! set up a single block covering the whole square domain
    lnblocks=0
    if(mype.eq.0) then
      lnblocks=1
      coord(1,1)=zhalf*(x1l+x1r)
      coord(2,1)=zhalf*(x2l+x2r)
      coord(3,1)=zhalf*(x3l+x3r)
      bnd_box(1,1,1)=x1l
      bnd_box(2,1,1)=x1r
      bnd_box(1,2,1)=x2l
      bnd_box(2,2,1)=x2r
      bnd_box(1,3,1)=x3l
      bnd_box(2,3,1)=x3r
      bsize(1,1)=bnd_box(2,1,1)-bnd_box(1,1,1)
      bsize(2,1)=bnd_box(2,2,1)-bnd_box(1,2,1)
      bsize(3,1)=bnd_box(2,3,1)-bnd_box(1,3,1)
      nodetype(1)=1
      lrefine(1)=1
!L       3.2       X1-boundary conditions
#ifdef bc1l_periodic
        neigh(1,1,1)=1
#endif
#ifdef bc1r_periodic
        neigh(1,2,1)=1
#endif
#ifdef bc1l_free
        neigh(1,1,1)=-21
#endif
#ifdef bc1r_free
        neigh(1,2,1)=-21
#endif
#ifdef bc1l_solid
        neigh(1,1,1)=-22
#endif
#ifdef bc1r_solid
        neigh(1,2,1)=-22
#endif
!L       3.2       X2-boundary conditions
#ifdef bc2l_periodic
        neigh(1,3,1)=1
#endif
#ifdef bc2r_periodic
        neigh(1,4,1)=1
#endif
#ifdef bc2l_free
        neigh(1,3,1)=-21
#endif
#ifdef bc2r_free
        neigh(1,4,1)=-21
#endif
#ifdef bc2l_solid
        neigh(1,3,1)=-22
#endif
#ifdef bc2r_solid
        neigh(1,4,1)=-22
#endif
!L       3.2       X3-boundary conditions
#ifdef bc3l_periodic
        neigh(1,5,1)=1
#endif
#ifdef bc3r_periodic
        neigh(1,6,1)=1
#endif
#ifdef bc3l_free
        neigh(1,5,1)=-21
#endif
#ifdef bc3r_free
        neigh(1,6,1)=-21
#endif
#ifdef bc3l_solid
        neigh(1,5,1)=-22
#endif
#ifdef bc3r_solid
        neigh(1,6,1)=-22
#endif
!L       3.2       X1-boundary conditions
! block is on pe 0
      neigh(2,:,1)=0
      refine(1)=.true.
    endif
!L       3.2       X1-cell positions
! Now cycle over blocks,refining all existing leaf blocks.
    do j=2,lrefine_ini
      refine(1:lnblocks)=.true.
      call AMR_REFINE_DEREFINE
    enddo
    nblk=lnblocks
! Output details of the current grid
!   if(mype.eq.0) write(*,*) 'blk/blk-coords/blk-sizes'
!   do jb=1,lnblocks
!     write(*,*) jb,(coord(j1,jb),j1=1,ndim),(bsize(j2,jb),j2=1,ndim)
!   enddo
!L       3.2       X1-cell positions
    icount1(1)=ncc1
    icount1h(1)=nhh1
    icc1=ihh1-1
    istart1(1)=1-ncc1
    do j=1,icc1/ncc1
      istart1(1)=istart1(1)+ncc1
      istart1h(1)=istart1(1)
      iret=NF90_GET_VAR(ifile,1,zzx1,istart1,icount1)
      do jb=1,lnblocks
        if(nodetype(jb).eq.1) then
          if(zzx1(1).gt.bnd_box(1,1,jb)) then
            if(zzx1(1).lt.bnd_box(2,1,jb)) then
              do j1=1,ncc1
                xc1(ng1+j1,jb)=zzx1(j1)
              enddo
            endif
          endif
        endif
      enddo
    enddo
!L       3.2       X2-cell positions
    icount2(1)=ncc2
    icount2h(1)=nhh2
    icc2=ihh2-1
    istart2(1)=1-ncc2
    do j=1,icc2/ncc2
      istart2(1)=istart2(1)+ncc2
      istart2h(1)=istart2(1)
      iret=NF90_GET_VAR(ifile,2,zzx2,istart2,icount2)
      do jb=1,lnblocks
        if(nodetype(jb).eq.1) then
          if(zzx2(1).gt.bnd_box(1,2,jb)) then
            if(zzx2(1).lt.bnd_box (2,2,jb)) then
              do j2=2,ncc2
                xc2(ng2+j2,jb)=zzx2(j2)
              enddo
            endif
          endif
        endif
      enddo
    enddo
!L       3.2       X3-cell positions
    icount3(1)=ncc3
    icount3h(1)=nhh3
    icc3=ihh3-1
    istart3(1)=1-ncc3
    do j=1,icc3/ncc3
      istart3(1)=istart3(1)+ncc3
      istart3h(1)=istart3(1)
      iret=NF90_GET_VAR(ifile,3,zzx3,istart3,icount3)
#ifdef x3dim
         do jb=1,lnblocks
           if(nodetype(jb).eq.1) then
             if(zzx3(1).gt.bnd_box(1,3,jb)) then
               if(zzx3(1).lt.bnd_box(2,3,jb)) then
                 do j3=1,ncc3
                   xc3(ng3+j3,jb)=zzx3(j3)
                 enddo
               endif
            endif
          endif
        enddo
#else
        do jb=1,lnblocks
          if(nodetype(jb).eq.1) then
            do j3=1,ncc3
              xc3(ng3+j3,jb)=zzx3(j3)
            enddo
          endif
        enddo
#endif
    enddo
!L       3.2       X1-cell positions
    icount1h(1)=nhh1
    icc1=ihh1-1
    istart1(1)=1-ncc1
    do j=1,icc1/ncc1
      istart1(1)=istart1(1)+ncc1
      istart1h(1)=istart1(1)
      iret=NF90_GET_VAR(ifile,4,zzh1,istart1,icount1h)
      do jb=1,lnblocks
        if(nodetype(jb).eq.1) then
          if(zzh1(1).ge.bnd_box(1,1,jb)) then
            if(zzh1(1).le.bnd_box(2,1,jb)) then
              do j1=1,nhh1
                xh1(ng1+j1,jb)=zzh1(j1)
              enddo
            endif
          endif
        endif
      enddo
    enddo
!L       3.2       X2-cell positions
    icount2h(1)=nhh2
    icc2=ihh2-1
    istart2(1)=1-ncc2
    do j=1,icc2/ncc2
      istart2(1)=istart2(1)+ncc2
      istart2h(1)=istart2(1)
      iret=NF90_GET_VAR(ifile,5,zzh2,istart2,icount2h)
      do jb=1,lnblocks
        if(nodetype(jb).eq.1) then
          if(zzh2(1).ge.bnd_box(1,2,jb)) then
            if(zzh2(1).le.bnd_box(2,2,jb)) then
              do j2=2,nhh2
                xh2(ng2+j2,jb)=zzh2(j2)
              enddo
            endif
          endif
        endif
      enddo
    enddo
!L       3.2       X3-cell positions
    icount3h(1)=nhh3
    icc3=ihh3-1
    istart3(1)=1-ncc3
    do j=1,icc3/ncc3
      istart3(1)=istart3(1)+ncc3
      istart3h(1)=istart3(1)
      iret=NF90_GET_VAR(ifile,6,zzh3,istart3,icount3h)
#ifdef x3dim
         do jb=1,lnblocks
           if(nodetype(jb).eq.1) then
             if(zzh3(1).ge.bnd_box(1,3,jb)) then
               if(zzh3(1).le.bnd_box(2,3,jb)) then
                 do j3=1,nhh3
                   xh3(ng3+j3,jb)=zzh3(j3)
                 enddo
               endif
            endif
          endif
        enddo
#else
        do jb=1,lnblocks
          if(nodetype(jb).eq.1) then
            do j3=1,nhh3
              xh3(ng3+j3,jb)=zzh3(j3)
            enddo
          endif
        enddo
#endif
    enddo
#else
!---------------------------------------------------------------------
!L     3.        Get coordinates
    istart1(1)=1+myproc*ncc1
    icount1(1)=ncc1
    istart1h(1)=1+myproc*ncc1
    icount1h(1)=nhh1
!L       3.1       Coordinates
    iret=NF90_GET_VAR(ifile,1,zx1,istart1,icount1)
    do jb=1,nblk
      do j1=1,ncc1
        xc1(ng1+j1,jb)=zx1(j1,jb)
      enddo
    enddo
    iret=NF90_GET_VAR(ifile,2,zx2)
    do jb=1,nblk
      do j2=1,ncc2
        xc2(ng2+j2,jb)=zx2(j2,jb)
      enddo
    enddo
    iret=NF90_GET_VAR(ifile,3,zx3)
    do jb=1,nblk
      do j3=1,ncc3
        xc3(ng3+j3,jb)=zx3(j3,jb)
      enddo
    enddo
!L       3.2       Coordinates
    iret=NF90_GET_VAR(ifile,4,zh1,istart1h,icount1h)
    do jb=1,nblk
      do j1=1,nhh1
        xh1(ng1+j1,jb)=zh1(j1,jb)
      enddo
    enddo
    iret=NF90_GET_VAR(ifile,5,zh2)
    do jb=1,nblk
      do j2=1,nhh2
        xh2(ng2+j2,jb)=zh2(j2,jb)
      enddo
    enddo
    iret=NF90_GET_VAR(ifile,6,zh3)
    do jb=1,nblk
      do j3=1,nhh3
        xh3(ng3+j3,jb)=zh3(j3,jb)
      enddo
    enddo
!L       3.1       Block boundaries
    x1l=zh1(1,1)
    x2l=zh2(1,1)
    x3l=zh3(1,1)
    x1r=zh1(nhh1,1)
    x2r=zh2(nhh2,1)
    x3r=zh3(nhh3,1)
#endif
!---------------------------------------------------------------------
!L     5.        Terminate
!L       5.1       Close the file
  iret=NF90_CLOSE(ifile)
!L       5.2       Write
  if(myproc.eq.0) then
    write(*,'(4x,a6,3i5)') 'resgrd',ncc1,ncc2,ncc3
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2006,0)
#endif
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE RESGRD
