!****f* e0pro/main
! NAME 
!   OUTCHE
! PURPOSE
!   Control the output
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTCHE(ptime,pout,pfrom,pto,pstep, &
  ktime,kout,kfrom,kto,kstep,kmode,knum,konoff)
!---------------------------------------------------------------------
  IMPLICIT NONE
!---------------------------------------------------------------------
  REAL(8) :: ptime, pout, pfrom, pto, pstep
  INTEGER :: ktime, kout, kfrom, kto, kstep, kmode, knum, konoff
!---------------------------------------------------------------------
!L     1.        Initial output
  konoff=0
  if(kmode.eq.1) then
    if(pstep.gt.0d0) then
      if(ptime.eq.pfrom) then
        konoff=1
        knum=1
        pout=pfrom+pstep
      else if(ptime.lt.pfrom) then
        konoff=0
        knum=0
        pout=pfrom
      else
        konoff=0
        knum=int((ptime-pfrom)/pstep)+1
        pout=pfrom+float(knum)*pstep
      endif
    else if(kstep.gt.0) then
      if(ktime.eq.kfrom) then
        konoff=1
        knum=1
        kout=kfrom+kstep
      else if(ktime.lt.kfrom) then
        konoff=0
        knum=0
        kout=kfrom
      else
        konoff=0
        knum=int((ktime-kfrom)/kstep)+1
        kout=kfrom+knum*kstep
      endif
    endif
!---------------------------------------------------------------------
!L     2.        Periodic output
  else
    if(pstep.gt.0d0) then
      if(pfrom.lt.pto) then
        if((ptime.ge.pfrom).and.(ptime.lt.pto+pstep)) then
          if(ptime.ge.pout) konoff=1
        endif
      else
        if((pfrom.eq.0d0).and.(pto.eq.0d0)) then
          if(ptime.ge.pout) konoff=1
        endif
      endif
      if(konoff.eq.1) pout=pout+pstep
    else if(kstep.gt.0) then
      if(kfrom.lt.kto) then
        if((ktime.ge.kfrom).and.(ktime.le.kto)) then
          if(ktime.ge.kout) konoff=1
        endif
      else
        if((kfrom.eq.0).and.(kto.eq.0)) then
          if(ktime.ge.kout) konoff=1
        endif
      endif
      if(konoff.eq.1) kout=kout+kstep
    endif
    if(konoff.eq.1) knum=knum+1
  endif
!---------------------------------------------------------------------
!L     4.        Return
RETURN
END SUBROUTINE OUTCHE


!****f* e0pro/main
! NAME 
!   OUTDIA
! PURPOSE
!   Output of diagnostics
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTDIA(kmode)
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE commet
  IMPLICIT NONE
  REAL(8) :: ztime, zdt
  INTEGER :: kmode
#ifdef DIAGNO
  call EXPERT(3001,1)
#endif
!---------------------------------------------------------------------
!L     1.        Initial output
  if(myproc.eq.0) then
    if(kmode.eq.1) then
      write(*,'(4x,a)') '---------------------------------------------'
      if((tstop-tstart).lt.36000d0) then
        write(*,'(16x,a12,a11,a10)') 'time (s)','dt (s)','cflmax'
      elseif((tstop-tstart).gt.864000d0) then
        write(*,'(16x,a12,a11,a10)') 'time (d)','dt (d)','cflmax'
      else
        write(*,'(16x,a12,a11,a10)') 'time (h)','dt (h)','cflmax'
      endif
      write(*,'(4x,a)') '---------------------------------------------'
    endif
!---------------------------------------------------------------------
!L     2.        Periodic output
    if(kmode.eq.2) then
      if((tstop-tstart).lt.36000d0) then
        ztime=time
        zdt=dt
       elseif((tstop-tstart).gt.864000d0) then
        ztime=time/86400d0
        zdt=dt/86400d0
      else
        ztime=time/3600d0
        zdt=dt/3600d0
      endif
      write(*,'(4x,a6,i6,1pe12.4,1pe11.3,1pe10.2)') 'step  ',  &
        nstep,ztime,zdt,cflmax
    endif
!---------------------------------------------------------------------
!L     3.        Final output
    if(kmode.eq.3) then
      write(*,'(4x,a)') '---------------------------------------------'
      if((tstop-tstart).lt.36000d0) then
        write(*,'(16x,a12,a11,a10)') 'time (s)','dt (s)','cflmax'
      elseif((tstop-tstart).gt.864000d0) then
        write(*,'(16x,a12,a11,a10)') 'time (d)','dt (d)','cflmax'
      else
        write(*,'(16x,a12,a11,a10)') 'time (h)','dt (h)','cflmax'
      endif
      write(*,'(4x,a)') '---------------------------------------------'
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(3001,0)
#endif                                                                  
RETURN
END SUBROUTINE OUTDIA
