!****f* e0pro/main
! NAME 
!   OUTX1S
! PURPOSE
!   Output on X1-slice
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTX1S(kmode) 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(8) :: zuthe,zt
#ifndef ethe
  REAL(8) :: zukin
#endif
#ifdef etot
#ifdef mhd
  REAL(8) :: zumag
#endif
#endif
  INTEGER :: iret,ifile,id
  INTEGER :: kmode
  INTEGER :: j2,j3,jb,iv
  INTEGER,DIMENSION(3)  ::  idims,istart,icount
  DATA  idims/2,3,4/,istart/1,1,1/,icount/ncc2,ncc3,1/
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(3005,1)
#endif
!---------------------------------------------------------------------
  istart(3)=mxs
!---------------------------------------------------------------------
!L     1.        Create the new file
  if(kmode.eq.1) then
    iret=NF90_CREATE(lfx1s,nf90_noclobber,ifile)
!---------------------------------------------------------------------
!L     2.        Set global attribures
    iret=NF90_PUT_ATT(ifile,nf90_global,'type','X1S')
    iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
      'Values on the X2-X3 slice at time levels')
#include <attribut.src>
!---------------------------------------------------------------------
!L     3.        Set array dimensions
    iret=NF90_DEF_DIM(ifile,'n1',1,id)
    iret=NF90_DEF_DIM(ifile,'n2',ncc2,id)
    iret=NF90_DEF_DIM(ifile,'n3',ncc3,id)
    iret=NF90_DEF_DIM(ifile,'ntime',nf90_unlimited,id)
!---------------------------------------------------------------------
!L     4.        Define constant values
    iret=NF90_DEF_VAR(ifile,'X1',nf90_float,1,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
    iret=NF90_PUT_ATT(ifile,id,'units','m')
    iret=NF90_DEF_VAR(ifile,'X2',nf90_float,2,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
    if(lgeom.eq.'cartesian') then
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    endif
    iret=NF90_DEF_VAR(ifile,'X3',nf90_float,3,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
    if(lgeom.eq.'spherical') then
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    endif
!---------------------------------------------------------------------
!L     5.        Define time-dependent values
    iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,4,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
    iret=NF90_PUT_ATT(ifile,id,'units','s')
    iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
    iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
    iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
    iret=NF90_PUT_ATT(ifile,id,'units','K')
    iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
      iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X1-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X2-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X3-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
      iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Density of plasma cloud')
#endif
#ifdef mhdcon
      iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Polarity of magnetic field')
#endif
!leave define mode
    iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!L     6.        Write constant values
    iret=NF90_PUT_VAR(ifile,1,x1s)
    do j2=nc2l,nc2r
      zzx2(j2-ng2)=xc2(j2,1)
    enddo
    iret=NF90_PUT_VAR(ifile,2,zzx2)
    do j3=nc3l,nc3r
      zzx3(j3-ng3)=xc3(j3,1)
    enddo
    iret=NF90_PUT_VAR(ifile,3,zzx3)
  else
!---------------------------------------------------------------------
!L     7.        Open the existing file
    iret=NF90_OPEN(lfx1s,nf90_write,ifile)
!---------------------------------------------------------------------
!L     8.        Write time-dependent values
    jb=1
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mxs,1/))
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        zzx23(j2-ng2,j3-ng3)=unk(1,nx1s,j2,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,5,zzx23,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
#ifdef ethe
          zuthe=unk(2,nx1s,j2,j3,jb)
#else
          zukin=zhalf*(unk(3,nx1s,j2,j3,jb)*unk(3,nx1s,j2,j3,jb)   &
                      +unk(4,nx1s,j2,j3,jb)*unk(4,nx1s,j2,j3,jb)   &
                      +unk(5,nx1s,j2,j3,jb)*unk(5,nx1s,j2,j3,jb))/ &
                      unk(1,nx1s,j2,j3,jb)
#ifdef etot
#ifdef mhd
              zumag=zr2mu*(unk(6,nx1s,j2,j3,jb)*unk(6,nx1s,j2,j3,jb) &
                          +unk(7,nx1s,j2,j3,jb)*unk(7,nx1s,j2,j3,jb) &
                          +unk(8,nx1s,j2,j3,jb)*unk(8,nx1s,j2,j3,jb))
              zuthe=unk(2,nx1s,j2,j3,jb)-zukin-zumag
#else
              zuthe=unk(2,nx1s,j2,j3,jb)-zukin
#endif
#else
            zuthe=unk(2,nx1s,j2,j3,jb)-zukin
#endif
#endif
#ifdef gam3d
          zt=((gamma(nx1s,j2,j3,jb)-zone)/zgas)*zuthe/unk(1,nx1s,j2,j3,jb)
#else
          zt=((gamma-zone)/zgas)*zuthe/unk(1,nx1s,j2,j3,jb)
#endif
        zzx23(j2-ng2,j3-ng3)=zt
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,6,zzx23,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        zzx23(j2-ng2,j3-ng3)=unk(3,nx1s,j2,j3,jb)/unk(1,nx1s,j2,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,7,zzx23,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        zzx23(j2-ng2,j3-ng3)=unk(4,nx1s,j2,j3,jb)/unk(1,nx1s,j2,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,8,zzx23,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j2=nc2l,nc2r
        zzx23(j2-ng2,j3-ng3)=unk(5,nx1s,j2,j3,jb)/unk(1,nx1s,j2,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,9,zzx23,start=istart,count=icount)
    id=9
    iv=5
#ifdef mhd
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          zzx23(j2-ng2,j3-ng3)=unk(6,nx1s,j2,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,10,zzx23,start=istart,count=icount)
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          zzx23(j2-ng2,j3-ng3)=unk(7,nx1s,j2,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,11,zzx23,start=istart,count=icount)
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          zzx23(j2-ng2,j3-ng3)=unk(8,nx1s,j2,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,12,zzx23,start=istart,count=icount)
      id=12
      iv=8
#endif
#ifdef hydcon
      id=id+1
      iv=iv+1
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          zzx23(j2-ng2,j3-ng3)=unk(iv,nx1s,j2,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx23,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iv=iv+1
      do j3=nc3l,nc3r
        do j2=nc2l,nc2r
          zzx23(j2-ng2,j3-ng3)=unk(iv,nx1s,j2,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx23,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!L     9.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     10.       Return
#ifdef DIAGNO
  call EXPERT(3005,1)
#endif
RETURN
END SUBROUTINE OUTX1S


!****f* e0pro/main
! NAME 
!   OUTX2S
! PURPOSE
!   Output on X2-slice
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTX2S(kmode)
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(8) :: zuthe,zt
#ifndef ethe
  REAL(8) :: zukin
#endif
#ifdef etot
#ifdef mhd
  REAL(8) :: zumag
#endif
#endif
  INTEGER :: iret,ifile,id
  INTEGER :: kmode
  INTEGER :: j1,j3,jb,iv
  INTEGER,DIMENSION(3)  ::  idims,istart,icount
  DATA  idims/1,3,4/, istart/1,1,1/, icount/ncc1,ncc3,1/
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(3005,1)
#endif
!---------------------------------------------------------------------
  istart(3)=mxs
!---------------------------------------------------------------------
!L     1.        Create the new file
  if(kmode.eq.1) then
    iret=NF90_CREATE(lfx2s,nf90_noclobber,ifile)
!---------------------------------------------------------------------
!L     2.        Set global attributes
    iret=NF90_PUT_ATT(ifile,nf90_global,'type','X2S')
    iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
      'Values on the X1-X3 slice at time levels')
#include <attribut.src>
!---------------------------------------------------------------------
!L     3.        Set array dimensions
    iret=NF90_DEF_DIM(ifile,'n1',ncc1,id)
    iret=NF90_DEF_DIM(ifile,'n2',1,id)
    iret=NF90_DEF_DIM(ifile,'n3',ncc3,id)
    iret=NF90_DEF_DIM(ifile,'ntime',nf90_unlimited,id)
!---------------------------------------------------------------------
!L     4.        Define constant values
    iret=NF90_DEF_VAR(ifile,'X1',nf90_float,1,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
    iret=NF90_PUT_ATT(ifile,id,'units','m')
    iret=NF90_DEF_VAR(ifile,'X2',nf90_float,2,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
    if(lgeom.eq.'cartesian') then
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    endif
    iret=NF90_DEF_VAR(ifile,'X3',nf90_float,3,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
    if(lgeom.eq.'spherical') then
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    endif
!---------------------------------------------------------------------
!L     4.        Define time-dependent values
    iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,4,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
    iret=NF90_PUT_ATT(ifile,id,'units','s')
    iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
    iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
    iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
    iret=NF90_PUT_ATT(ifile,id,'units','K')
    iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
      iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X1-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                               'X2-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X3-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
      iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Density of plasma cloud')
#endif
#ifdef mhdcon
      iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Polarity of magnetic field')
#endif
!leave define mode
    iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!L     6.        Write constant values
    do j1=nc1l,nc1r
      zzx1(j1-ng1)=xc1(j1,1)
    enddo
    iret=NF90_PUT_VAR(ifile,1,zzx1)
    iret=NF90_PUT_VAR(ifile,2,x2s)
    do j3=nc3l,nc3r
      zzx3(j3-ng3)=xc3(j3,1)
    enddo
    iret=NF90_PUT_VAR(ifile,3,zzx3)
  else
!---------------------------------------------------------------------
!L     7.        Open the existing file
    iret=NF90_OPEN(lfx2s,nf90_write,ifile)
!---------------------------------------------------------------------
!L     8.        Write time-dependent values
    jb=1
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mxs,1/))
    do j3=nc3l,nc3r
      do j1=nc1l,nc1r
        zzx13(j1-ng1,j3-ng3)=unk(1,j1,nx2s,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,5,zzx13,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j1=nc1l,nc1r
#ifdef ethe
          zuthe=unk(2,j1,nx2s,j3,jb)
#else
          zukin=zhalf*(unk(3,j1,nx2s,j3,jb)*unk(3,j1,nx2s,j3,jb)   &
                      +unk(4,j1,nx2s,j3,jb)*unk(4,j1,nx2s,j3,jb)   &
                      +unk(5,j1,nx2s,j3,jb)*unk(5,j1,nx2s,j3,jb))/ &
                      unk(1,j1,nx2s,j3,jb)
#ifdef etot
#ifdef mhd
              zumag=zr2mu*(unk(6,j1,nx2s,j3,jb)*unk(6,j1,nx2s,j3,jb) &
                          +unk(7,j1,nx2s,j3,jb)*unk(7,j1,nx2s,j3,jb) &
                          +unk(8,j1,nx2s,j3,jb)*unk(8,j1,nx2s,j3,jb))
              zuthe=unk(2,j1,nx2s,j3,jb)-zukin-zumag
#else
              zuthe=unk(2,j1,nx2s,j3,jb)-zukin
#endif
#else
            zuthe=unk(2,j1,nx2s,j3,jb)-zukin
#endif
#endif
#ifdef gam3d
          zt=((gamma(j1,nx2s,j3,jb)-zone)/zgas)*zuthe/unk(1,j1,nx2s,j3,jb)
#else
          zt=((gamma-zone)/zgas)*zuthe/unk(1,j1,nx2s,j3,jb)
#endif
        zzx13(j1-ng1,j3-ng3)=zt
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,6,zzx13,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j1=nc1l,nc1r
        zzx13 (j1-ng1,j3-ng3)=unk(3,j1,nx2s,j3,jb)/            &
                                    unk(1,j1,nx2s,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,7,zzx13,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j1=nc1l,nc1r
        zzx13(j1-ng1,j3-ng3)=unk(4,j1,nx2s,j3,jb)/            &
                                    unk(1,j1,nx2s,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,8,zzx13,start=istart,count=icount)
    do j3=nc3l,nc3r
      do j1=nc1l,nc1r
       zzx13(j1-ng1,j3-ng3)=unk(5,j1,nx2s,j3,jb)/            &
                                    unk(1,j1,nx2s,j3,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,9,zzx13,start=istart,count=icount)
    id=9
    iv=5
#ifdef mhd
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
          zzx13(j1-ng1,j3-ng3)=unk(6,j1,nx2s,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,10,zzx13,start=istart,count=icount)
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
          zzx13(j1-ng1,j3-ng3)=unk(7,j1,nx2s,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,11,zzx13,start=istart,count=icount)
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
          zzx13(j1-ng1,j3-ng3)=unk(8,j1,nx2s,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,12,zzx13,start=istart,count=icount)
      id=12
      iv=8
#endif
#ifdef hydcon
      id=id+1
      iv=iv+1
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
          zzx13(j1-ng1,j3-ng3)=unk(iv,j1,nx2s,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx13,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iv=iv+1
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
          zzx13(j1-ng1,j3-ng3)=unk(iv,j1,nx2s,j3,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx13,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!L     9.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     10.       Return
#ifdef DIAGNO
  call EXPERT(3005,1)
#endif
RETURN
END SUBROUTINE OUTX2S


!****f* e0pro/main
! NAME 
!   OUTX3S
! PURPOSE
!   Output on X3-slice
! INPUTS
!   kmode =1,2,3 for initial,periodic,final output
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE OUTX3S(kmode)
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
    use paramesh_dimensions
    use physicaldata
    use tree
#endif
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comout
  USE netcdf
  IMPLICIT NONE
  REAL(8) :: zuthe,zt
#ifndef ethe
  REAL(8) :: zukin
#endif
#ifdef etot
#ifdef mhd
  REAL(8) :: zumag
#endif
#endif
  INTEGER :: iret,ifile,id
  INTEGER :: kmode
  INTEGER :: j1,j2,jb,iv
  INTEGER,DIMENSION(3)  ::  idims,istart,icount
  DATA  idims/1,2,4 /,istart/ 1,1,1 /,icount/ ncc1,ncc2,1 /
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(3006,1)
#endif
!---------------------------------------------------------------------
  istart(3)=mxs
!---------------------------------------------------------------------
!L     1.        Create the new file
  if(kmode.eq.1) then
    iret=NF90_CREATE(lfx3s,nf90_noclobber,ifile)
!---------------------------------------------------------------------
!L     2.        Set global attributes
    iret=NF90_PUT_ATT(ifile,nf90_global,'type','X3S')
    iret=NF90_PUT_ATT(ifile,nf90_global,'title', &
      'Values on the X1-X2 slice at time levels')
#include <attribut.src>
!---------------------------------------------------------------------
!L     3.        Set array dimensions
    iret=NF90_DEF_DIM(ifile,'n1',ncc1,id)
    iret=NF90_DEF_DIM(ifile,'n2',ncc2,id)
    iret=NF90_DEF_DIM(ifile,'n3',1,id)
    iret=NF90_DEF_DIM(ifile,'ntime',nf90_unlimited,id)
!---------------------------------------------------------------------
!L     4.        Define constant values
    iret=NF90_DEF_VAR(ifile,'X1',nf90_float,1,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-mesh positions')
    iret=NF90_PUT_ATT(ifile,id,'units','m')
    iret=NF90_DEF_VAR(ifile,'X2',nf90_float,2,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-mesh positions')
    if(lgeom.eq.'cartesian') then
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    endif
    iret=NF90_DEF_VAR(ifile,'X3',nf90_float,3,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-mesh positions')
    if(lgeom.eq.'spherical') then
      iret=NF90_PUT_ATT(ifile,id,'units','radian')
    else
      iret=NF90_PUT_ATT(ifile,id,'units','m')
    endif
!---------------------------------------------------------------------
!L     5.        Define time-dependent values
    iret=NF90_DEF_VAR(ifile,'TIME',nf90_float,4,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
    iret=NF90_PUT_ATT(ifile,id,'units','s')
    iret=NF90_DEF_VAR(ifile,'D',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
    iret=NF90_PUT_ATT(ifile,id,'units','kg/m3')
    iret=NF90_DEF_VAR(ifile,'T',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
    iret=NF90_PUT_ATT(ifile,id,'units','K')
    iret=NF90_DEF_VAR(ifile,'V1',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V2',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
    iret=NF90_DEF_VAR(ifile,'V3',nf90_float,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
    iret=NF90_PUT_ATT(ifile,id,'units','m/s')
#ifdef mhd
      iret=NF90_DEF_VAR(ifile,'B1',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X1-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_DEF_VAR(ifile,'B2',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X2-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
      iret=NF90_DEF_VAR(ifile,'B3',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'X3-magnetic field')
      iret=NF90_PUT_ATT(ifile,id,'units','T')
#endif
#ifdef hydcon
      iret=NF90_DEF_VAR(ifile,'DP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Density of plasma cloud')
#endif
#ifdef mhdcon
      iret=NF90_DEF_VAR(ifile,'BP',nf90_float,idims,id)
      iret=NF90_PUT_ATT(ifile,id,'long_name',           &
                                'Polarity of magnetic field')
#endif
!leave define mode
    iret=NF90_ENDDEF(ifile)
!---------------------------------------------------------------------
!L     6.        Write constant values
    do j1=nc1l,nc1r
      zzx1(j1-ng1)=xc1(j1,1)
    enddo
    iret=NF90_PUT_VAR(ifile,1,zzx1)
    do j2=nc2l,nc2r
      zzx2(j2-ng2)=xc2(j2,1)
    enddo
    iret=NF90_PUT_VAR(ifile,2,zzx2)
    iret=NF90_PUT_VAR(ifile,3,x3s)
  else
!---------------------------------------------------------------------
!L     7.        Open the existing file
    iret=NF90_OPEN(lfx3s,nf90_write,ifile)
!---------------------------------------------------------------------
!L     8.        Write time-dependent values
    jb=1
    iret=NF90_PUT_VAR(ifile,4,time,start=(/mxs,1/))
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        zzx12(j1-ng1,j2-ng2)=unk(1,j1,j2,nx3s,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,5,zzx12,start=istart,count=icount)
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
#ifdef ethe
          zuthe=unk(2,j1,j2,nx3s,jb)
#else
          zukin=zhalf*(unk(3,j1,j2,nx3s,jb)*unk(3,j1,j2,nx3s,jb)   &
                      +unk(3,j1,j2,nx3s,jb)*unk(3,j1,j2,nx3s,jb)   &
                      +unk(4,j1,j2,nx3s,jb)*unk(4,j1,j2,nx3s,jb))/ &
                      unk(1,j1,j2,nx3s,jb)
#ifdef etot
#ifdef mhd
              zumag=zr2mu*(unk(6,j1,j2,nx3s,jb)*unk(6,j1,j2,nx3s,jb) &
                          +unk(7,j1,j2,nx3s,jb)*unk(7,j1,j2,nx3s,jb) &
                          +unk(8,j1,j2,nx3s,jb)*unk(8,j1,j2,nx3s,jb))
              zuthe=unk(2,j1,j2,nx3s,jb)-zukin-zumag
#else
              zuthe=unk(2,j1,j2,nx3s,jb)-zukin
#endif
#else
            zuthe=unk(2,j1,j2,nx3s,jb)-zukin
#endif
#endif
#ifdef gam3d
          zt=((gamma(j1,j2,nx3s,jb)-zone)/zgas)*zuthe/unk(1,j1,j2,nx3s,jb)
#else
          zt=((gamma-zone)/zgas)*zuthe/unk(1,j1,j2,nx3s,jb)
#endif
        zzx12(j1-ng1,j2-ng2)=zt
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,6,zzx12,start=istart,count=icount)
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        zzx12(j1-ng1,j2-ng2)=unk(3,j1,j2,nx3s,jb)/unk(1,j1,j2,nx3s,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,7,zzx12,start=istart,count=icount)
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        zzx12(j1-ng1,j2-ng2)=unk(4,j1,j2,nx3s,jb)/unk(1,j1,j2,nx3s,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,8,zzx12,start=istart,count=icount)
    do j2=nc2l,nc2r
      do j1=nc1l,nc1r
        zzx12(j1-ng1,j2-ng2)=unk(5,j1,j2,nx3s,jb)/unk(1,j1,j2,nx3s,jb)
      enddo
    enddo
    iret=NF90_PUT_VAR(ifile,9,zzx12,start=istart,count=icount)
    id=9
    iv=5
#ifdef mhd
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          zzx12(j1-ng1,j2-ng2)=unk(6,j1,j2,nx3s,jb)/unk(1,j1,j2,nx3s,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,10,zzx12,start=istart,count=icount)
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          zzx12(j1-ng1,j2-ng2)=unk(7,j1,j2,nx3s,jb)/unk(1,j1,j2,nx3s,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,11,zzx12,start=istart,count=icount)
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          zzx12(j1-ng1,j2-ng2)=unk(8,j1,j2,nx3s,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,12,zzx12,start=istart,count=icount)
      id=12
      iv=8
#endif
#ifdef hydcon
      id=id+1
      iv=iv+1
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          zzx12(j1-ng1,j2-ng2)=unk(iv,j1,j2,nx3s,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx12,start=istart,count=icount)
#endif
#ifdef mhdcon
      id=id+1
      iv=iv+1
      do j2=nc2l,nc2r
        do j1=nc1l,nc1r
          zzx12(j1-ng1,j2-ng2)=unk(iv,j1,j2,nx3s,jb)
        enddo
      enddo
      iret=NF90_PUT_VAR(ifile,id,zzx12,start=istart,count=icount)
#endif
  endif
!---------------------------------------------------------------------
!L     9.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
!L     10.       Return
#ifdef DIAGNO
  call EXPERT(3006,0)
#endif                                                                  
RETURN
END SUBROUTINE OUTX3S
