!****f* e0pro/main
! NAME 
!   CALMHD
! PURPOSE
!   Magnetohydrodynamics
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE CALMHD
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comphy
  USE combnd
  USE commet
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr
#endif
  IMPLICIT NONE
  REAL(8) :: zzdt
  INTEGER :: jb
#ifdef DIAGNO
  call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!L     1.        Auxiliary values
zzdt=dt
if(nstep.eq.1) then
iib1=0
iib2=0
iib3=0
iib4=0
endif
  do jb=1,nblk
if(jb.eq.1) iib1=iib1+1
if(jb.eq.2) iib2=iib2+1
if(jb.eq.4) iib3=iib3+1
if(jb.eq.8) iib4=iib4+1
if(iib1.gt.8) iib1=1
if(iib2.gt.4) iib2=1
if(iib3.gt.2) iib3=1
if(iib4.gt.1) iib4=1
iib=iib1
if(jb.ge.2) iib=iib2
if(jb.ge.4) iib=iib3
if(jb.ge.8) iib=iib4
!if(iib.eq.1) then
if(iib.ge.0) then
dt=8d0*zzdt
if(jb.ge.2) dt=4d0*zzdt
if(jb.ge.4) dt=2d0*zzdt
if(jb.ge.8) dt=zzdt
dt=zzdt
  dth=zhalf*dt
  dtq=zhalf*dth
  if(nlxn) then
    dtmom=dt/xmass
  else
    dtmom=dt
  endif
!DT coef
  dthmom=zhalf*dtmom
#ifdef mhd
    dtmag=dt/zmu
    dtene=dt/zmu
    if(nlxn) then
      dtmag=dtmag/xmass
    endif
    dthmag=zhalf*dtmag
    dthene=zhalf*dtene
#endif
!---------------------------------------------------------------------
!L     2.        Directional splitting - cell centers
#ifdef split
!L       2.1       Save values for div(B) treatment
#ifdef toth
#ifdef MPI
      call BNDMPI
#endif
      if((myproc.eq.0).and.nltbc) then
        call BNDTIM(2)
      endif
      call BND1
      call BND2
      call BND3
#ifdef NGRID
      call BND1NG
      call BND2NG
!     call BND3NG
#endif
      call TOTH1
#endif
!L       2.2       Space splitting 123
    if(nsplit.eq.1) then
      if(nc1.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
          call BNDMPI
#endif
        if((myproc.eq.0).and.nltbc) then
          call BNDTIM(2)
        endif
        call BND1
#ifdef NGRID
        call BND1NG
#endif
        call MHD1(jb)
      endif
      if(nc2.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND2
#ifdef NGRID
        call BND2NG
#endif
        call MHD2
      endif
      if(nc3.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND3
#ifdef NGRID
        call BND3NG
#endif
        call MHD3
      endif
      nsplit=2
!L       2.3       Space splitting 321
    else
      if(nc3.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND3
#ifdef NGRID
        call BND3NG
#endif
        call MHD3
      endif
      if(nc2.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
        call BND2
#ifdef NGRID
        call BND2NG
#endif
        call MHD2
      endif
      if(nc1.gt.1) then
#ifdef PARAMESH
          call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
          call BNDMPI
#endif
        if((myproc.eq.0).and.nltbc) then
          call BNDTIM(2)
        endif
        call BND1
#ifdef NGRID
        call BND1NG
#endif
        call MHD1(jb)
      endif
      nsplit=1
    endif
!L       2.4       Div(B) treatment
#ifdef toth
#ifdef PARAMESH
        call AMR_GUARDCELL(mype,iopt,nlayers)
#else
#ifdef MPI
          call BNDMPI
#endif
          call BND1
#ifdef NGRID
          call BND1NG
#endif
#ifdef x2dim
          call BND2
#ifdef NGRID
          call BND2NG
#endif
#endif
#ifdef x3dim
          call BND3
#ifdef NGRID
          call BND3NG
#endif
#endif
#endif
      call TOTH2
#endif
#endif
endif
enddo
!---------------------------------------------------------------------
!L     3.        Multi-dimensional - cell centers
#ifndef split
#ifndef staggered
!L       3.1       Boundary conditions
#ifdef PARAMESH
    call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
    call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
      call BNDTIM(2)
    endif
    call BND1
    call BND2
    call BND3
#ifdef NGRID
    call BND1NG
    call BND2NG
    call BND3NG
#endif
!L       3.2       Cell-centered grid
    call MHC
#endif
#endif
!---------------------------------------------------------------------
!L     4.        Div(B) treatment
#ifdef powell
!L       4.1       Powell correction
#ifdef PARAMESH
    call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
!L       4.1       Boundary conditions
#ifdef MPI
    call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
      call BNDTIM(2)
    endif
    call BND1
    call BND2
    call BND3
#ifdef NGRID
    call BND1NG
    call BND2NG
    call BND3NG
#endif
    call POWELL
#endif
#ifdef janhunen
!L       4.1       Janhunen correction
#ifdef PARAMESH
    call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
!L       4.1       Boundary conditions
#ifdef MPI
    call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
      call BNDTIM(2)
    endif
    call BND1
    call BND2
    call BND3
#ifdef NGRID
    call BND1NG
    call BND2NG
    call BND3NG
#endif
    call JANHUNEN
#endif
#ifdef difb
!L       4.1       Diffusion
#ifdef PARAMESH
    call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
!L       4.1       Boundary conditions
#ifdef MPI
    call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
      call BNDTIM(2)
    endif
    call BND1
    call BND2
    call BND3
#ifdef NGRID
    call BND1NG
    call BND2NG
    call BND3NG
#endif
    call DIFMAG
#endif
!---------------------------------------------------------------------
!L     5.        Multi-dimensional - staggered grid
#ifndef split
#ifdef staggered
!L       5.1       Boundary conditions
#ifdef PARAMESH
    call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
    call BNDMPI
#endif
    if((myproc.eq.0).and.nltbc) then
      call BNDTIM(2)
    endif
    call BND1
    call BND2
    call BND3
#ifdef NGRID
    call BND1NG
    call BND2NG
    call BND3NG
#endif
!L       5.2       Staggered grid
    call MHS
#endif
#endif
!---------------------------------------------------------------------
!L     6.        Multi-dimensional - vector potential
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE CALMHD                         


!****f* e0pro/main
! NAME 
!   CALSTP
! PURPOSE
!   Sources and transport processes
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE CALSTP
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comphy
  USE combnd
  USE commet
  IMPLICIT NONE
#ifdef DIAGNO
  call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!L     1.        Internal boundaries
#ifdef heat
!L       3.1       Volumetric heating
  call HEAT1
#endif
!---------------------------------------------------------------------
!L     2.        Internal boundaries
#ifdef resis
!L       2.1       Boundary conditions
#ifdef PARAMESH
  call AMR_GUARDCELL(mype,iopt,nlayers)
#endif
#ifdef MPI
  call BNDMPI
#endif
  if((myproc.eq.0).and.nltbc) then
    call BNDTIM(2)
  endif
  call BND1
  call BND2
  call BND3
!L       2.1       Curl of B
  call CURLB
!L       2.2       Anomaleous resisitivity
  call ANORES
!L       2.3       Update MHD variables
  call MHDRES
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE CALSTP                         


!****f* e0pro/main
! NAME 
!   TIMSTP
! PURPOSE
!   Timestep control
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE TIMSTP
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comphy
  USE commet
  IMPLICIT NONE
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr
  REAL(8) :: zcflmx
#endif
  REAL(8) :: zamp,zakdt
  INTEGER :: jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
  INTEGER :: j1,j2,j3,lb
  INTEGER :: iopt,nlayers
#endif
#ifdef DIAGNO
  call EXPERT(4002,1)
#endif
!---------------------------------------------------------------------
!L     1.        Choose the most restrictive value of DTSTEP
  nlimdt=0
!L       1.1       Maximum CFL number
  cflmax=zero
#ifdef PARAMESH
  do jb=1,lnblocks
    if(nodetype(jb).eq.1) then
#else
  do jb=1,nblk
#endif
#ifdef resis
    cflmx(jb)=dt*max(cfl1mx(jb),cfl2mx(jb),cfl3mx(jb),cflres(jb))
#else
    cflmx(jb)=dt*max(cfl1mx(jb),cfl2mx(jb),cfl3mx(jb))
#endif
#ifdef DIAGNO
    write(*,'(4x,2(i4),2x,a4,3(1pe11.3))') myproc,jb,'cfl',  &
      dt*cfl1mx(jb),dt*cfl2mx(jb),dt*cfl3mx(jb)
    write(*,'(4x,2(i4),2x,a4,3(1pe11.3))') myproc,jb,'dt ',  &
      abs(akcfl)/cfl1mx(jb),abs(akcfl)/cfl2mx(jb),abs(akcfl)/cfl3mx(jb)
#endif
    cflmax=max(cflmax,cflmx(jb))
#ifdef PARAMESH
  endif
#endif
  enddo
!L       1.2       Maximum CFL number
#ifdef MPI
    zcflmx=cflmax
!Wait until all procs come at this point
    call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    call MPI_ALLREDUCE(zcflmx,cflmax,1,MPI_DOUBLE_PRECISION,  &
      MPI_MAX,MPI_COMM_WORLD,ierr)
#endif
!---------------------------------------------------------------------
!L     2.        Choose the most restrictive value of DTSTEP
!L       2.1       Amplification factor
  zamp=abs(akcfl)/cflmax
!L       2.2       Limits on DTSTEP amplification
  if(zamp.gt.akdt) then
    zamp=akdt
    nlimdt=1
  else
    zakdt=zone/akdt
    if(zamp.lt.zakdt) then
      zamp=zakdt
      nlimdt=2
    endif
  endif
!L       2.3       Form DTSTEP
  dt=zamp*dt
!L       2.4       Check maximum/minimum limits
  if(dt.gt.dtmax) then
    dt=dtmax
    nlimdt=3
  else
    if(dt.lt.dtmin) then
      dt=dtmin
      nlimdt=4
    endif
  endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4002,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE TIMSTP


!****f* e0pro/main
! NAME 
!   PASVAL
! PURPOSE
!   Save passage valeus in time
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE PASVAL
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comreg
  USE comglo
  USE combnd
  IMPLICIT NONE
  INTEGER :: iplus, jx2, jx3
#ifdef bctim2c
  INTEGER :: jx1
#endif
#ifdef DIAGNO
  call EXPERT(4005,1)
#endif
!---------------------------------------------------------------------
!L     1.        Save values
  iplus=nxfrom-1
#ifdef bctim2c
    do jx1=1,ng1
      x1p(jx1)=xc1(jx1+iplus,1)
    enddo
#else
    x1p=xc1(1+iplus,1)
#endif
  do jx2=1,nc2
    x2p(jx2)=xc2(jx2+iplus,1)
  enddo
  do jx3=1,nc3
    x3p(jx3)=xc3(jx3+iplus,1)
  enddo
!L       1.1       X1-X2-X3 mesh
! do jx1=1,ng1
!   ix1=jx1+iplus
!   do jx2=1,nc2
!     ix2=jx2+ng2
!     do jx3=1,nc3
!       ix3=jx3+ng3
!       unkpv(:,jx1,jx2,jx3,:)=unk(:,ix1,ix2,ix3,:)
!     enddo
!   enddo
! enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE PASVAL


!****f* e4cal/heat1
! NAME 
!   HEAT1
! PURPOSE
!   Volumetric heating
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE HEAT1
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
    use paramesh_dimensions 
    use physicaldata 
    use tree 
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE commet
  USE comadm
  USE comtva
  IMPLICIT NONE
  REAL(8) :: zthe, zrad
#ifndef ethe
  REAL(8) :: zkin
#ifdef mhd
  REAL(8) :: zmag
#endif
#endif
  INTEGER :: j1,j2,j3,jb
#ifdef PARAMESH
#include "amr_shmem.fh"
#include "shmem_reduce.fh"
#endif
#ifdef DIAGNO
  call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!L     1.        Auxiliary values
#ifdef PARAMESH
!loop over leaf grid blocks
  do jb=1,lnblocks
#ifndef ADVANCE_ALL_LEVELS
    if(nodetype(jb).eq.1) then
#endif
#else
  do jb=1,nblk
#endif
!L       2.1       Parameters
    cfl1mx(jb)=zero
!L       2.2       Add the heating to the energy
    do j2=nc2l,nc2r
      do j3=nc3l,nc3r
        do j1=nc1l,nc1r
!L       6.2       Temperature
#ifndef ethe
      zkin=zhalf*(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb) &
                 +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb) &
                 +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))
#endif
#ifdef etot
#ifdef mhd
      zmag=zr2mu*(unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb) &
                 +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb) &
                 +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb))
      zthe=unk(2,j1,j2,j3,jb)-zkin-zmag
#else
      zthe=unk(2,j1,j2,j3,jb)-zkin
#endif
#else
      zthe=unk(2,j1,j2,j3,jb)
#endif
      if(nheat.eq.0) zrad=1d0
      if(nheat.eq.1) zrad=(xc1(1,jb)/xc1(j1,jb))
      if(nheat.eq.2) zrad=(xc1(1,jb)/xc1(j1,jb))**2
      if(nheat.eq.3) zrad=(xc1(1,jb)/xc1(j1,jb))**3
      unk(2,j1,j2,j3,jb)=unk(2,j1,j2,j3,jb)+qheat*zthe*zrad/float(ncc1)
          enddo
        enddo
      enddo
   enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE HEAT1                          
