!****f* e0pro/main
! NAME 
!   HELVAL
! PURPOSE
!   Save evolution at heliospheric points
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE HELVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
  REAL(8) :: zuthe
#ifndef ethe
  REAL(8) :: zukin
#endif
#ifdef etot
#ifdef mhd
  REAL(8) :: zumag
#endif
#endif
  INTEGER :: i1, i2, i3, iv, jb, jhel
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr, ic, ixhel, jproc
  INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
  INTEGER, PARAMETER :: ibuf=maxhel*(3+nnvar)
  REAL(8) :: zrecv(ibuf), zsend(ibuf)
#endif
#ifdef DIAGNO
  call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!L     1.        Save values
  do jb=1,nblk
    do jhel=1,nhel
      if(n1hel(jhel,jb).ne.0) then
      if(n2hel(jhel,jb).ne.0) then
      if(n3hel(jhel,jb).ne.0) then
!L       1.1       X1-X2-X3 mesh
        i1=n1hel(jhel,jb)
        i2=n2hel(jhel,jb)
        i3=n3hel(jhel,jb)
        dhel(jhel)=unk(1,i1,i2,i3,jb)
#ifdef ethe
        zuthe=unk(2,i1,i2,i3,jb)
#else
        zukin=zhalf*(unk(3,i1,i2,i3,jb)*unk(3,i1,i2,i3,jb)  &
                    +unk(4,i1,i2,i3,jb)*unk(4,i1,i2,i3,jb)  &
                    +unk(5,i1,i2,i3,jb)*unk(5,i1,i2,i3,jb)) &
                    /unk(1,i1,i2,i3,jb)
#ifdef etot
#ifdef mhd
        zumag=zr2mu*(unk(6,i1,i2,i3,jb)*unk(6,i1,i2,i3,jb)  &
                    +unk(7,i1,i2,i3,jb)*unk(7,i1,i2,i3,jb)  &
                    +unk(8,i1,i2,i3,jb)*unk(8,i1,i2,i3,jb))
        zuthe=unk(2,i1,i2,i3,jb)-zukin-zumag
#else
        zuthe=unk(2,i1,i2,i3,jb)-zukin
#endif
#else
        zuthe=unk(2,i1,i2,i3,jb)-zukin
#endif
#endif
#ifdef gam3d
        thel(jhel)=((gamma(i1,i2,i3,jb)-zone)/zgas)*zuthe/ &
          unk(1,i1,i2,i3,jb)
#else
        thel(jhel)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,jb)
#endif
        v1hel(jhel)=unk(3,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
        v2hel(jhel)=unk(4,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
        v3hel(jhel)=unk(5,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
#ifdef mhd
        b1hel(jhel)=unk(6,i1,i2,i3,jb)
        b2hel(jhel)=unk(7,i1,i2,i3,jb)
        b3hel(jhel)=unk(8,i1,i2,i3,jb)
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        dphel(jhel)=unk(iv,i1,i2,i3,jb)
#endif
#ifdef mhdcon
        iv=iv+1
        bphel(jhel)=unk(iv,i1,i2,i3,jb)
#endif
#ifdef mhd
        if(nldiav) then
          e1hel(jhel)=-v2hel(jhel)*b3hel(jhel)+v3hel(jhel)*b2hel(jhel)
          e2hel(jhel)=-v3hel(jhel)*b1hel(jhel)+v1hel(jhel)*b3hel(jhel)
          e3hel(jhel)=-v1hel(jhel)*b2hel(jhel)+v2hel(jhel)*b1hel(jhel)
!         xj1hel(jhel)=b1a(i1,i2,i3)
!         xj2hel(jhel)=b2a(i1,i2,i3)
!         xj3hel(jhel)=b3a(i1,i2,i3)
      endif
#endif
      endif
      endif
      endif
    enddo
  enddo
!---------------------------------------------------------------------
!L     8.        Copy to the last processor
#ifdef MPI
  if(nxhel.gt.0) then
    ic=0
    do jhel=1,nxhel
      ihel=nnhel(myproc+1)+jhel
      zsend(ic+01)=x1hel(ihel)
      zsend(ic+02)=x2hel(ihel)
      zsend(ic+03)=x3hel(ihel)
      zsend(ic+04)=dhel(ihel)
      zsend(ic+05)=thel(ihel)
      zsend(ic+06)=v1hel(ihel)
      zsend(ic+07)=v2hel(ihel)
      zsend(ic+08)=v3hel(ihel)
#ifdef mhd
      zsend(ic+9)=b1hel(ihel)
      zsend(ic+10)=b2hel(ihel)
      zsend(ic+11)=b3hel(ihel)
#ifdef hydcon
      zsend(ic+12)=dphel(ihel)
#ifdef mhdcon
      zsend(ic+13)=bphel(ihel)
      ic=ic+13
#else
      ic=ic+12
#endif
#else
#ifdef mhdcon
      zsend(ic+12)=bphel(ihel)
      ic=ic+12
#else
      ic=11
#endif
#endif
#else
#ifdef hydcon
      zsend(ic+9)=dphel(ihel)
      ic=ic+9
#else
      ic=8
#endif
#endif
    enddo
    call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
      510,MPI_COMM_WORLD,ierr)
!   call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                   zrecv,ic,MPI_DOUBLE_PRECISION, &
!                   0,MPI_COMM_WORLD,ierr)
  endif
! call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  if(myproc.eq.0) then
    do jproc=2,nprocs
      if(nnxhel(jproc).gt.0) then
        call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
          510,MPI_COMM_WORLD,status,ierr)
        ic=0
        do jhel=1,nnxhel(jproc)
          ihel=nnhel(jproc)+jhel
          x1hel(ihel)=zrecv(ic+01)
          x2hel(ihel)=zrecv(ic+02)
          x3hel(ihel)=zrecv(ic+03)
          dhel(ihel)=zrecv(ic+04)
          thel(ihel)=zrecv(ic+05)
          v1hel(ihel)=zrecv(ic+06)
          v2hel(ihel)=zrecv(ic+07)
          v3hel(ihel)=zrecv(ic+08)
#ifdef mhd
          b1hel(ihel)=zrecv(ic+9)
          b2hel(ihel)=zrecv(ic+10)
          b3hel(ihel)=zrecv(ic+11)
#ifdef hydcon
          dphel(ihel)=zrecv(ic+12)
#ifdef mhdcon
          bphel(ihel)=zrecv(ic+13)
          ic=ic+13
#else
          ic=ic+12
#endif
#else
#ifdef mhdcon
          bphel(ihel)=zrecv(ic+12)
          ic=ic+12
#else
          ic=11
#endif
#endif
#else
#ifdef hydcon
          dphel(ihel)=zrecv(ic+9)
          ic=ic+9
#else
          ic=8
#endif
#endif
        enddo
      endif
    enddo
  endif
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE HELVAL


!****f* e0pro/main
! NAME 
!   DIAVAL
! PURPOSE
!   Save diagnostic values in time
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE DIAVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  IMPLICIT NONE
  INTEGER :: j1,j2,j3,jb
#ifdef DIAGNO
  call EXPERT(4004,1)
#endif
!---------------------------------------------------------------------
!L     1.        Clear values
  totmas=zero
  enetot=zero
  enethe=zero
  enekin=zero
#ifdef mhd
  enemag=zero
#endif
!---------------------------------------------------------------------
!L     2.        Save values
!L       2.1       X1-X2-X3 mesh
#ifdef PARAMESH
  do jb=1,lnblocks
  if(nodetype(jb).eq.1) then
#else
  do jb=1,nblk
#endif
    do j3=1,nc3
      do j2=1,nc2
        do j1=1,nc1
          totmas=totmas+unk(1,j1,j2,j3,jb)
          enekin=enekin+(unk(3,j1,j2,j3,jb)*unk(3,j1,j2,j3,jb) &
                        +unk(4,j1,j2,j3,jb)*unk(4,j1,j2,j3,jb) &
                        +unk(5,j1,j2,j3,jb)*unk(5,j1,j2,j3,jb))/ &
                        unk(1,j1,j2,j3,jb)
#ifdef mhd
          enemag=enemag+unk(6,j1,j2,j3,jb)*unk(6,j1,j2,j3,jb) &
                       +unk(7,j1,j2,j3,jb)*unk(7,j1,j2,j3,jb) &
                       +unk(8,j1,j2,j3,jb)*unk(8,j1,j2,j3,jb)
#endif
#ifdef ethe
          enethe=enethe+unk(2,j1,j2,j3,jb)
#else
          enetot=enetot+unk(2,j1,j2,j3,jb)
#endif
        enddo
      enddo
    enddo
#ifdef PARAMESH
  endif
#endif
  enddo
!---------------------------------------------------------------------
!L     3.        Adjust
  enekin=zhalf*enekin
  if(nlxn) then
    totmas=totmas*xmass
    enekin=enekin*xmass
  endif
#ifdef mhd
    enemag=zr2mu*enemag
#endif
#ifdef etot
    enethe=enetot-enekin-enemag
#else
    enetot=enethe+enekin+enemag
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4004,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE DIAVAL


!****f* e0pro/main
! NAME 
!   SELVAL
! PURPOSE
!   Save evolution at points along Sun-Earth line
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE SELVAL
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
    REAL(8) :: zukin,zumag,zuthe
#endif
  REAL(8) :: incl, nn, ll, gg, lambda, omega
  REAL(8) :: zdate, bo, bocos, bosin
  REAL(8) :: zmm, zmp, zpm, zpp, zzz
  INTEGER :: i1, i2m, i2p, i3m, i3p, iv, jb, jsel
  INTEGER :: i2sel
#ifdef x2dim
  INTEGER :: j2
#endif
#ifdef MPI
    include 'mpif.h'
    INTEGER :: ierr, ic, ixsel, jproc
    INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
    INTEGER, PARAMETER :: ibuf=maxsel*(3+nnvar)
    REAL(8) zrecv(ibuf), zsend(ibuf)
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!L     1.        Compute the heliographic latitude
  if(refdate.gt.0d0) then
! Modified Julian date
    zdate=refdate+time/86400d0
! Inclination of the rotation axis to the ecliptic
    incl=7.25d0
! Number of days from J2000.0 (starts on 1 January 2000 at 12:00 UT)
    nn=zdate-51544.5d0
! Mean longitude of Sun, corrected for aberration
    ll=280.472d0+0.9856474d0*nn
! Mean anomaly
    gg=357.528d0+0.9856003d0*nn
    gg=gg*cpi/180d0
! Ecliptic longitude
    lambda=ll+1.915d0*sin(gg)+0.020d0*sin(2d0*gg)
! Longitude of the ascending node
    omega=73d0+40d0/60d0+50.25d0*(zdate+3243.75d0)/365.25d0/3600d0
! Heliographic latitude
    Bosin=sin((lambda-omega)*cpi/180d0)*sin(incl*cpi/180d0)
    Bocos=sqrt(1d0-Bosin**2)
    Bo=0.5d0*cpi-atan(Bosin/Bocos)
  else
    Bo=0.5d0*cpi
  endif
!---------------------------------------------------------------------
!L     2.        Positions of Sun-Earth-line points
  do jb=1,nblk
    do jsel=1,nsel
      if(n1sel(jsel,jb).ne.0) then
      if(n3sel(jsel,jb).ne.0) then
#ifdef x2dim
        i2sel=nc2l
        do j2=nc2l+1,nc2r-1
          if(xc2(j2,jb).le.Bo) i2sel=j2
        enddo
#else
        i2sel=1
#endif
!---------------------------------------------------------------------
!L     3.        Save values
        i2m=1
        i3m=1
        x2sel(jsel)=Bo
!L       3.1       X1-X2-X3 mesh
        i1=n1sel(jsel,jb)
        i2m=i2sel
        i3m=n3sel(jsel,jb)
        i2p=i2m
        i3p=i3m
#ifdef x2dim
#ifdef x3dim
        if(i2m.lt.nc2r-1) i2p=i2m+1
        if(i3m.lt.nc3r-1) i3p=i3m+1
        zzz=(xc2(i2p,jb)-xc2(i2m,jb))*(xc3(i3p,jb)-xc3(i3m,jb))
        zmm=(xc2(i2p,jb)-x2sel(jsel))*(xc3(i3p,jb)-x3sel(jsel))/zzz
        zmp=(xc2(i2p,jb)-x2sel(jsel))*(x3sel(jsel)-xc3(i3m,jb))/zzz
        zpm=(x2sel(jsel)-xc2(i2m,jb))*(xc3(i3p,jb)-x3sel(jsel))/zzz
        zpp=(x2sel(jsel)-xc2(i2m,jb))*(x3sel(jsel)-xc3(i3m,jb))/zzz
#else
        if(i2m.lt.nc2r-1) i2p=i2m+1
        zzz=xc2(i2p,jb)-xc2(i2m,jb)
        zmm=0.5d0*(xc2(i2p,jb)-x2sel(jsel))/zzz
        zmp=0.5d0*(xc2(i2p,jb)-x2sel(jsel))/zzz
        zpm=0.5d0*(x2sel(jsel)-xc2(i2m,jb))/zzz
        zpp=0.5d0*(x2sel(jsel)-xc2(i2m,jb))/zzz
#endif
#else
#ifdef x3dim
        if(i3m.lt.nc3r-1) i3p=i3m+1
        zzz=xc3(i3p,jb)-xc3(i3m,jb)
        zmm=0.5d0*(xc3(i3p,jb)-x3sel(jsel))/zzz
        zmp=0.5d0*(x3sel(jsel)-xc3(i3m,jb))/zzz
        zpm=0.5d0*(xc3(i3p,jb)-x3sel(jsel))/zzz
        zpp=0.5d0*(x3sel(jsel)-xc3(i3m,jb))/zzz
#else
        zmm=0.25d0
        zmp=0.25d0
        zpm=0.25d0
        zpp=0.25d0
#endif
#endif
        dsel(jsel)=zmm*unk(1,i1,i2m,i3m,jb)+zmp*unk(1,i1,i2m,i3p,jb) &
                  +zpm*unk(1,i1,i2p,i3m,jb)+zpp*unk(1,i1,i2p,i3p,jb)
        v1sel(jsel)=zmm*(unk(3,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                   +zmp*(unk(3,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                   +zpm*(unk(3,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                   +zpp*(unk(3,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
        v2sel(jsel)=zmm*(unk(4,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                   +zmp*(unk(4,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                   +zpm*(unk(4,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                   +zpp*(unk(4,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
        v3sel(jsel)=zmm*(unk(5,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                   +zmp*(unk(5,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                   +zpm*(unk(5,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                   +zpp*(unk(5,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
#ifdef mhd
        b1sel(jsel)=zmm*unk(6,i1,i2m,i3m,jb)+zmp*unk(6,i1,i2m,i3p,jb) &
                   +zpm*unk(6,i1,i2p,i3m,jb)+zpp*unk(6,i1,i2p,i3p,jb)
        b2sel(jsel)=zmm*unk(7,i1,i2m,i3m,jb)+zmp*unk(7,i1,i2m,i3p,jb) &
                   +zpm*unk(7,i1,i2p,i3m,jb)+zpp*unk(7,i1,i2p,i3p,jb)
        b3sel(jsel)=zmm*unk(8,i1,i2m,i3m,jb)+zmp*unk(8,i1,i2m,i3p,jb) &
                   +zpm*unk(8,i1,i2p,i3m,jb)+zpp*unk(8,i1,i2p,i3p,jb)
#endif
#ifdef etot
#ifdef mhd
        zukin=zhalf*(dsel(jsel)*v1sel(jsel)*v1sel(jsel)  &
                    +dsel(jsel)*v2sel(jsel)*v2sel(jsel)  &
                    +dsel(jsel)*v3sel(jsel)*v3sel(jsel))
        zumag=zr2mu*(b1sel(jsel)*b1sel(jsel)+b2sel(jsel)*b2sel(jsel) &
          +b3sel(jsel)*b3sel(jsel))
        zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
             +zmp*unk(2,i1,i2m,i3p,jb) &
             +zpm*unk(2,i1,i2p,i3m,jb) &
             +zpp*unk(2,i1,i2p,i3p,jb) &
             -zukin-zumag
#ifdef gam3d
        tsel(jsel)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dsel(jsel)
#else
        tsel(jsel)=((gamma-zone)/zgas)*zuthe/dsel(jsel)
#endif
#else
        zukin=zhalf*(dsel(jsel)*v1sel(jsel)*v1sel(jsel)  &
                    +dsel(jsel)*v2sel(jsel)*v2sel(jsel)  &
                    +dsel(jsel)*v3sel(jsel)*v3sel(jsel))
        zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
             +zmp*unk(2,i1,i2m,i3p,jb) &
             +zpm*unk(2,i1,i2p,i3m,jb) &
             +zpp*unk(2,i1,i2p,i3p,jb) &
             -zukin
#ifdef gam3d
        tsel(jsel)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dsel(jsel)
#else
        tsel(jsel)=((gamma-zone)/zgas)*zuthe/dsel(jsel)
#endif
#endif
#else
#ifdef gam3d
        tsel(jsel)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
            +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
            +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
            +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#else
        tsel(jsel)=((gamma-zone)/zgas)* &
            (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
            +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
            +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
            +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#endif
#endif
#ifdef mhd
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        dpsel(jsel)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
                   +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
#ifdef mhdcon
        iv=iv+1
        bpsel(jsel)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
                   +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
      endif
      endif
    enddo
  enddo
!---------------------------------------------------------------------
!L     8.        Copy to the last processor
#ifdef MPI
  if(nxsel.gt.0) then
    ic=0
    do jsel=1,nxsel
      isel=nnsel(myproc+1)+jsel
      zsend(ic+1)=x1sel(isel)
      zsend(ic+2)=x2sel(isel)
      zsend(ic+3)=x3sel(isel)
      zsend(ic+4)=dsel(isel)
      zsend(ic+5)=tsel(isel)
      zsend(ic+6)=v1sel(isel)
      zsend(ic+7)=v2sel(isel)
      zsend(ic+8)=v3sel(isel)
#ifdef mhd
      zsend(ic+9)=b1sel(isel)
      zsend(ic+10)=b2sel(isel)
      zsend(ic+11)=b3sel(isel)
#ifdef hydcon
      zsend(ic+12)=dpsel(isel)
#ifdef mhdcon
      zsend(ic+13)=bpsel(isel)
      ic=ic+13
#else
      ic=ic+12
#endif
#else
#ifdef mhdcon
      zsend(ic+12)=bpsel(isel)
      ic=ic+12
#else
      ic=11
#endif
#endif
#else
#ifdef hydcon
      zsend(ic+9)=dpsel(isel)
      ic=ic+9
#else
      ic=8
#endif
#endif
    enddo
    call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
      520,MPI_COMM_WORLD,ierr)
!   call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                   zrecv,ic,MPI_DOUBLE_PRECISION, &
!                     0,MPI_COMM_WORLD,ierr)
  endif
! call MPI_BARRIER(MPI_COMM_WORLD,ierr)
  if(myproc.eq.0) then
    do jproc=2,nprocs
      if(nnxsel(jproc).gt.0) then
        call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
          520,MPI_COMM_WORLD,status,ierr)
        ic=0
        do jsel=1,nnxsel(jproc)
          isel=nnsel(jproc)+jsel
          x1sel(isel)=zrecv(ic+1)
          x2sel(isel)=zrecv(ic+2)
          x3sel(isel)=zrecv(ic+3)
          dsel(isel)=zrecv(ic+4)
          tsel(isel)=zrecv(ic+5)
          v1sel(isel)=zrecv(ic+6)
          v2sel(isel)=zrecv(ic+7)
          v3sel(isel)=zrecv(ic+8)
#ifdef mhd
          b1sel(isel)=zrecv(ic+9)
          b2sel(isel)=zrecv(ic+10)
          b3sel(isel)=zrecv(ic+11)
#ifdef hydcon
          dpsel(isel)=zrecv(ic+12)
#ifdef mhdcon
          bpsel(isel)=zrecv(ic+13)
          ic=ic+13
#else
          ic=ic+12
#endif
#else
#ifdef mhdcon
          bpsel(isel)=zrecv(ic+12)
          ic=ic+12
#else
          ic=11
#endif
#endif
#else
#ifdef hydcon
          dpsel(isel)=zrecv(ic+9)
          ic=ic+9
#else
          ic=8
#endif
#endif
        enddo
      endif
    enddo
  endif
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE SELVAL


!****f* e0pro/main
! NAME 
!   GEOVAL
! PURPOSE
!   Save evolution at geospace points
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE GEOVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
  REAL(8) :: zukin,zumag,zuthe
#endif
  REAL(8) :: incl, nn, ll, gg, lambda, omega
  REAL(8) :: zdate, bo, bocos, bosin
  REAL(8) :: zmm, zmp, zpm, zpp, zzz
  INTEGER :: i2m,i2p,i3m,i3p
  INTEGER :: js,jb,i1,i2,i3,iv
#ifdef x2dim
  INTEGER :: i2geo, j2
#endif
#ifdef DIAGNO
  call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!L     1.        Compute the heliographic latitude
  if(refdate.gt.0d0) then
! Modified Julian date
    zdate=refdate+time/86400d0
! Inclination of the rotation axis to the ecliptic
    incl=7.25d0
! Number of days from J2000.0 (starts on 1 January 2000 at 12:00 UT)
    nn=zdate-51544.5d0
! Mean longitude of Sun, corrected for aberration
    ll=280.472d0+0.9856474d0*nn
! Mean anomaly
    gg=357.528d0+0.9856003d0*nn
    gg=gg*cpi/180d0
! Ecliptic longitude
    lambda=ll+1.915d0*sin(gg)+0.020d0*sin(2d0*gg)
! Longitude of the ascending node
    omega=73d0+40d0/60d0+50.25d0*(zdate+3243.75d0)/365.25d0/3600d0
! Heliographic latitude
    Bosin=sin((lambda-omega)*cpi/180d0)*sin(incl*cpi/180d0)
    Bocos=sqrt(1d0-Bosin**2)
    Bo=0.5d0*cpi-atan(Bosin/Bocos)
  else
    Bo=0.5d0*cpi
  endif
!---------------------------------------------------------------------
!L     2.        Positions of Geospace points
  do jb=1,nblk
    if(n1geo(1,jb).ne.0) then
    if(n3geo(1,jb).ne.0) then
#ifdef x2dim
      i2geo=nc2l
      do j2=nc2l+1,nc2r-1
        if(xc2(j2,jb).le.Bo) i2geo=j2
      enddo
      x2geo(1)=Bo
      x2geo(2:3)=xc2(i2geo,jb)
      x2geo(4:5)=xc2(i2geo+1,jb)
      n2geo(1,jb)=i2geo
      n2geo(2:3,jb)=i2geo
      n2geo(4:5,jb)=i2geo+1
#else
      x2geo(1)=Bo
      x2geo(2:5)=xc2(1,jb)
      n2geo(1:5,jb)=1
#endif
!---------------------------------------------------------------------
!L     3.        Save values
      i2m=1
      i3m=1
!L       3.1       X1-X2-X3 mesh
      i1=n1geo(1,jb)
      i2m=n2geo(1,jb)
      i3m=n3geo(1,jb)
      i2p=i2m
      i3p=i3m
#ifdef x2dim
#ifdef x3dim
      if(i2m.lt.nc2r-1) i2p=i2m+1
      if(i3m.lt.nc3r-1) i3p=i3m+1
      zzz=(xc2(i2p,jb)-xc2(i2m,jb))*(xc3(i3p,jb)-xc3(i3m,jb))
      zmm=(xc2(i2p,jb)-x2geo(1))*(xc3(i3p,jb)-x3geo(1))/zzz
      zmp=(xc2(i2p,jb)-x2geo(1))*(x3geo(1)-xc3(i3m,jb))/zzz
      zpm=(x2geo(1)-xc2(i2m,jb))*(xc3(i3p,jb)-x3geo(1))/zzz
      zpp=(x2geo(1)-xc2(i2m,jb))*(x3geo(1)-xc3(i3m,jb))/zzz
#else
      if(i2m.lt.nc2r-1) i2p=i2m+1
      zzz=xc2(i2p,jb)-xc2(i2m,jb)
      zmm=0.5d0*(xc2(i2p,jb)-x2geo(1))/zzz
      zmp=0.5d0*(xc2(i2p,jb)-x2geo(1))/zzz
      zpm=0.5d0*(x2geo(1)-xc2(i2p,jb))/zzz
      zpp=0.5d0*(x2geo(1)-xc2(i2p,jb))/zzz
#endif
#else
#ifdef x3dim
      if(i3m.lt.nc3r-1) i3p=i3m+1
      zzz=xc3(i3p,jb)-xc3(i3m,jb)
      zmm=0.5d0*(xc3(i3p,jb)-x3geo(1))/zzz
      zmp=0.5d0*(x3geo(1)-xc3(i3m,jb))/zzz
      zpm=0.5d0*(xc3(i3p,jb)-x3geo(1))/zzz
      zpp=0.5d0*(x3geo(1)-xc3(i3p,jb))/zzz
#else
      zmm=0.25d0
      zmp=0.25d0
      zpm=0.25d0
      zpp=0.25d0
#endif
#endif
      dgeo(1)=zmm*unk(1,i1,i2m,i3m,jb)+zmp*unk(1,i1,i2m,i3p,jb) &
             +zpm*unk(1,i1,i2p,i3m,jb)+zpp*unk(1,i1,i2p,i3p,jb)
      v1geo(1)=zmm*(unk(3,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
              +zmp*(unk(3,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
              +zpm*(unk(3,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
              +zpp*(unk(3,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
      v2geo(1)=zmm*(unk(4,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
              +zmp*(unk(4,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
              +zpm*(unk(4,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
              +zpp*(unk(4,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
      v3geo(1)=zmm*(unk(5,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
              +zmp*(unk(5,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
              +zpm*(unk(5,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
              +zpp*(unk(5,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
#ifdef mhd
      b1geo(1)=zmm*unk(6,i1,i2m,i3m,jb)+zmp*unk(6,i1,i2m,i3p,jb) &
              +zpm*unk(6,i1,i2p,i3m,jb)+zpp*unk(6,i1,i2p,i3p,jb)
      b2geo(1)=zmm*unk(7,i1,i2m,i3m,jb)+zmp*unk(7,i1,i2m,i3p,jb) &
              +zpm*unk(7,i1,i2p,i3m,jb)+zpp*unk(7,i1,i2p,i3p,jb)
      b3geo(1)=zmm*unk(8,i1,i2m,i3m,jb)+zmp*unk(8,i1,i2m,i3p,jb) &
              +zpm*unk(8,i1,i2p,i3m,jb)+zpp*unk(8,i1,i2p,i3p,jb)
#endif
#ifdef etot
#ifdef mhd
      zukin=zhalf*(dgeo(1)*v1geo(1)*v1geo(1)  &
                  +dgeo(1)*v2geo(1)*v2geo(1)  &
                  +dgeo(1)*v3geo(1)*v3geo(1))
      zumag=zr2mu*(b1geo(1)*b1geo(1)+b2geo(1)*b2geo(1)+b3geo(1)*b3geo(1))
      zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
           +zmp*unk(2,i1,i2m,i3p,jb) &
           +zpm*unk(2,i1,i2p,i3m,jb) &
           +zpp*unk(2,i1,i2p,i3p,jb) &
           -zukin-zumag
#ifdef gam3d
      tgeo(1)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dgeo(1)
#else
      tgeo(1)=((gamma-zone)/zgas)*zuthe/dgeo(1)
#endif
#else
      zukin=zhalf*(dgeo(1)*v1geo(1)*v1geo(1)  &
                  +dgeo(1)*v2geo(1)*v2geo(1)  &
                  +dgeo(1)*v3geo(1)*v3geo(1))
      zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
           +zmp*unk(2,i1,i2m,i3p,jb) &
           +zpm*unk(2,i1,i2p,i3m,jb) &
           +zpp*unk(2,i1,i2p,i3p,jb) &
           -zukin
#ifdef gam3d
      tgeo(1)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dgeo(1)
#else
      tgeo(1)=((gamma-zone)/zgas)*zuthe/dgeo(1)
#endif
#endif
#else
#ifdef gam3d
      tgeo(1)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
          +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
          +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
          +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#else
      tgeo(1)=((gamma-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
          +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
          +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
          +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#endif
#endif
#ifdef mhd
      iv=8
#else
      iv=5
#endif
#ifdef hydcon
      iv=iv+1
      dpgeo(1)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
              +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
#ifdef mhdcon
      iv=iv+1
      bpgeo(1)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
              +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
!---------------------------------------------------------------------
!L     4.        Save values
      do js=2,5
!L       4.1       X1-X2-X3 mesh
        i1=n1geo(js,jb)
        i2=n2geo(js,jb)
        i3=n3geo(js,jb)
        dgeo(js)=unk(1,i1,i2,i3,jb)
#ifdef etot
#ifdef mhd
        zukin=zhalf*(unk(3,i1,i2,i3,jb)*unk(3,i1,i2,i3,jb)  &
                    +unk(4,i1,i2,i3,jb)*unk(4,i1,i2,i3,jb)  &
                    +unk(5,i1,i2,i3,jb)*unk(5,i1,i2,i3,jb)) &
                    /unk(1,i1,i2,i3,jb)
        zumag=zr2mu*(unk(6,i1,i2,i3,jb)*unk(6,i1,i2,i3,jb)  &
                    +unk(7,i1,i2,i3,jb)*unk(7,i1,i2,i3,jb)  &
                    +unk(8,i1,i2,i3,jb)*unk(8,i1,i2,i3,jb))
        zuthe=unk(2,i1,i2,i3,jb)-zukin-zumag
#ifdef gam3d
        tgeo(js)=((gamma(i1,i2,i3,jb)-zone)/zgas)*zuthe/ &
          unk(1,i1,i2,i3,jb)
#else
        tgeo(js)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,jb)
#endif
#else
        zukin=zhalf*(unk(3,i1,i2,i3,jb)*unk(3,i1,i2,i3,jb)  &
                    +unk(4,i1,i2,i3,jb)*unk(4,i1,i2,i3,jb)  &
                    +unk(5,i1,i2,i3,jb)*unk(5,i1,i2,i3,jb)) &
                    /unk(1,i1,i2,i3,jb)
        zuthe=unk(2,i1,i2,i3,jb)-zukin
#ifdef gam3d
        tgeo(js)=((gamma(i1,i2,i3,jb)-zone)/zgas)*zuthe/ &
          unk(1,i1,i2,i3,jb)
#else
        tgeo(js)=((gamma-zone)/zgas)*zuthe/unk(1,i1,i2,i3,jb)
#endif
#endif
#else
#ifdef gam3d
        tgeo(js)=((gamma(i1,i2,i3,jb)-zone)/zgas)*unk(2,i1,i2,i3,jb)/ &
          unk(1,i1,i2,i3,jb)
#else
        tgeo(js)=((gamma-zone)/zgas)*unk(2,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
#endif
#endif
        v1geo(js)=unk(3,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
        v2geo(js)=unk(4,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
        v3geo(js)=unk(5,i1,i2,i3,jb)/unk(1,i1,i2,i3,jb)
#ifdef mhd
        b1geo(js)=unk(6,i1,i2,i3,jb)
        b2geo(js)=unk(7,i1,i2,i3,jb)
        b3geo(js)=unk(8,i1,i2,i3,jb)
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        dpgeo(js)=unk(iv,i1,i2,i3,jb)
#endif
#ifdef mhdcon
        iv=iv+1
        bpgeo(js)=unk(iv,i1,i2,i3,jb)
#endif
#ifdef mhd
        if(nldiav) then
          e1geo(js)=-v2geo(js)*b3geo(js)+v3geo(js)*b2geo(js)
          e2geo(js)=-v3geo(js)*b1geo(js)+v1geo(js)*b3geo(js)
          e3geo(js)=-v1geo(js)*b2geo(js)+v2geo(js)*b1geo(js)
!         xj1geo(js)=b1a(i1,i2,i3)
!         xj2geo(js)=b2a(i1,i2,i3)
!         xj3geo(js)=b3a(i1,i2,i3)
        endif
#endif
      enddo
    endif
    endif
  enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE GEOVAL


!****f* e0pro/main
! NAME 
!   PLAVAL
! PURPOSE
!   Save evolution at planetary positions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE PLAVAL
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE comglo
  USE comevo
  USE comadm
  IMPLICIT NONE
#ifndef ethe
  REAL(8) :: zukin,zumag,zuthe
#endif
  REAL(8) :: mjd0, zdate, zx, x3ear
  REAL(8) :: zmm, zmp, zpm, zpp, zzz
  INTEGER :: i1pla, i2pla, i3pla
  INTEGER :: i1, i2m, i2p, i3m, i3p, iv
  INTEGER :: j1, jp, jb
#ifdef x2dim
  INTEGER :: j2
#endif
#ifdef x3dim
  INTEGER :: j3
#endif
#ifdef MPI
  include 'mpif.h'
  INTEGER :: ierr, ic, ipla, jproc
  INTEGER, DIMENSION(MPI_STATUS_SIZE) :: status
  INTEGER, PARAMETER :: ibuf=maxhel*(3+nnvar)
  REAL(8) :: zrecv(ibuf), zsend(ibuf)
#endif
!---------------------------------------------------------------------
!L     1.        Compute the heliographic latitude
!------------------  Initialize common  ----------------------
  REAL(8)  z0(9), z1(9), z2(9), z3(9), z4(9), z5(9), z6(9), z7(9), z8(9)
  DATA Z0/0.071422,0.027962,0.017202,0.009146,0.00145, 0.000584,0.000205,0.000104,0.000069/
  DATA Z1/3.8484,  3.02812, 1.74022, 4.51234, 4.53364, 4.89884, 2.46615, 3.78556, 3.16948/
  DATA Z2/0.388301,0.013195,0.032044,0.175301,0.090478,0.105558,0.088593,0.016965,0.471239/
  DATA Z3/1.34041, 2.28638, 1.78547, 5.85209, 0.23911, 1.61094, 2.96706, 0.773181,3.91303/
  DATA Z4/0.3871,  0.7233,  1.,      1.5237,  5.2028,  9.5385,  19.182,  30.06,   39.44/
  DATA Z5/0.07974, 0.00506, 0.017,   0.141704,0.249374,0.534156,0.901554,0.27054, 9.86/
  DATA Z6/2.73514, 3.85017, 3.33929, 1.04656, 1.76188, 3.1257,  4.49084, 2.33498, 5.23114/
  DATA Z7/0.122173,0.059341,0.,      0.03142, 0.01972, 0.043633,0.01396, 0.031416,0.300197/
  DATA Z8/0.836013,1.33168, 0.,      0.858702,1.74533, 1.977458,1.28805, 2.29162, 1.91812/
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(4003,1)
#endif
!---------------------------------------------------------------------
!L     1.        Compute the heliographic latitude
! jd0=2436935d0           ! Julian Date of 1-1-1960
! Modified Julian date
  mjd0=36934.5d0
! pi2 = 2*!PI			; 2 pi.
! nl = N_elements(list)	  	; number of planets to process.
!-----------------  Days since Epoch  ---------------
! frac=hms(0)/24.+hms(1)/1440.+hms(2)/86400.
! nd=jd-jd0+frac
  zdate=refdate+time/86400d0-mjd0
!-----------------  Save Earth azimuth  -------------
  zx=zdate*z0(3)+z1(3)
  do while(zx.lt.0d0)
    zx=zx+360d0
  enddo
  do while(zx.gt.360d0)
    zx=zx-360d0
  enddo
  zx=(cpi/180d0)*zx
  zx=zx+z2(3)*sin(zx-z3(3))
! zx=zx mod pi2
  x3ear=zx
!-----------------  Loop thru planets  --------------
  do jp=1,npla
    zx=zdate*z0(jp)+z1(jp)
    do while(zx.lt.0d0)
      zx=zx+360d0
    enddo
    do while(zx.gt.360d0)
      zx=zx-360d0
    enddo
    zx=(cpi/180d0)*zx
    zx=zx+z2(jp)*sin(zx-z3(jp))
!   zx=zx mod pi2
! Heliocentric distance (AU)
    x1pla(jp)=1.496d11*(z4(jp)+z5(jp)*sin(zx-z6(jp)))
! Heliocentric latitude (radian)
    x2pla(jp)=0.5d0*cpi-z7(jp)*sin(zx-z8(jp))
! Heliocentric longitude (radian)
    zx=cpi+zx-x3ear
    do while(zx.lt.0d0)
      zx=zx+2d0*cpi
    enddo
    do while(zx.gt.2d0*cpi)
      zx=zx-2d0*cpi
    enddo
    x3pla(jp)=zx
  enddo
!---------------------------------------------------------------------
!L     1.        Positions of Sun-Earth-line points
  do jb=1,nblk
    do jp=1,npla
      if((x1pla(jp).gt.xh1(nc1l,jb)).and. &
         (x1pla(jp).le.xh1(nc1r+1,jb))) then
      if((x2pla(jp).gt.xh2(nc2l,jb)).and. &
         (x2pla(jp).le.xh2(nc2r+1,jb))) then
      if((x3pla(jp).gt.xh3(nc3l,jb)).and. &
         (x3pla(jp).le.xh3(nc3r+1,jb))) then
!---------------------------------------------------------------------
!L     1.        Positions of planets
!L       1.1       Radial position
        i1pla=nc1l
        do j1=nc1l+1,nc1r-1
          if(xc1(j1,jb).le.x1pla(jp)) i1pla=j1
        enddo
!L       1.2       Meridional position
#ifdef x2dim
        i2pla=nc2l
        do j2=nc2l+1,nc2r-1
          if(xc2(j2,jb).le.x2pla(jp)) i2pla=j2
        enddo
#else
        i2pla=1
#endif
!L       1.3       Azimuthal position
#ifdef x3dim
        i3pla=nc3l
        do j3=nc3l+1,nc3r-1
          if(xc3(j3,jb).le.x3pla(jp)) i3pla=j3
        enddo
#else
        i3pla=1
#endif
!---------------------------------------------------------------------
!L     2.        Save values
        i2m=1
        i3m=1
!L       2.1       X1-X2-X3 mesh
        i1=i1pla
        i2m=i2pla
        i3m=i3pla
        i2p=i2m
        i3p=i3m
#ifdef x2dim
#ifdef x3dim
        if(i2m.lt.nc2r-1) then
          i2p=i2m+1
        else
          i2p=nc2l
        endif
        if(i3m.lt.nc3r-1) then
          i3p=i3m+1
        else
          i3p=nc3l
        endif
        zzz=(xc2(i2p,jb)-xc2(i2m,jb))*(xc3(i3p,jb)-xc3(i3m,jb))
        zmm=(xc2(i2p,jb)-x2pla(jp))*(xc3(i3p,jb)-x3pla(jp))/zzz
        zmp=(xc2(i2p,jb)-x2pla(jp))*(x3pla(jp)-xc3(i3m,jb))/zzz
        zpm=(x2pla(jp)-xc2(i2m,jb))*(xc3(i3p,jb)-x3pla(jp))/zzz
        zpp=(x2pla(jp)-xc2(i2m,jb))*(x3pla(jp)-xc3(i3m,jb))/zzz
#else
        if(i2m.lt.nc2r-1) then
          i2p=i2m+1
        else
          i2p=nc2l
        endif
        zzz=xc2(i2p,jb)-xc2(i2m,jb)
        zmm=0.5d0*(xc2(i2p,jb)-x2pla(jp))/zzz
        zmp=0.5d0*(xc2(i2p,jb)-x2pla(jp))/zzz
        zpm=0.5d0*(x2pla(jp)-xc2(i2m,jb))/zzz
        zpp=0.5d0*(x2pla(jp)-xc2(i2m,jb))/zzz
#endif
#else
#ifdef x3dim
        if(i3m.lt.nc3r-1) then
          i3p=i3m+1
        else
          i3p=nc3l
        endif
        zzz=xc3(i3p,jb)-xc3(i3m,jb)
        zmm=0.5d0*(xc3(i3p,jb)-x3pla(jp))/zzz
        zmp=0.5d0*(x3pla(jp)-xc3(i3m,jb))/zzz
        zpm=0.5d0*(xc3(i3p,jb)-x3pla(jp))/zzz
        zpp=0.5d0*(x3pla(jp)-xc3(i3m,jb))/zzz
#else
        zmm=0.25d0
        zmp=0.25d0
        zpm=0.25d0
        zpp=0.25d0
#endif
#endif
        dpla(jp)=zmm*unk(1,i1,i2m,i3m,jb)+zmp*unk(1,i1,i2m,i3p,jb) &
                +zpm*unk(1,i1,i2p,i3m,jb)+zpp*unk(1,i1,i2p,i3p,jb)
        v1pla(jp)=zmm*(unk(3,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                 +zmp*(unk(3,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                 +zpm*(unk(3,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                 +zpp*(unk(3,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
        v2pla(jp)=zmm*(unk(4,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                 +zmp*(unk(4,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                 +zpm*(unk(4,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                 +zpp*(unk(4,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
        v3pla(jp)=zmm*(unk(5,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
                 +zmp*(unk(5,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
                 +zpm*(unk(5,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
                 +zpp*(unk(5,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb))
#ifdef mhd
        b1pla(jp)=zmm*unk(6,i1,i2m,i3m,jb)+zmp*unk(6,i1,i2m,i3p,jb) &
                 +zpm*unk(6,i1,i2p,i3m,jb)+zpp*unk(6,i1,i2p,i3p,jb)
        b2pla(jp)=zmm*unk(7,i1,i2m,i3m,jb)+zmp*unk(7,i1,i2m,i3p,jb) &
                 +zpm*unk(7,i1,i2p,i3m,jb)+zpp*unk(7,i1,i2p,i3p,jb)
        b3pla(jp)=zmm*unk(8,i1,i2m,i3m,jb)+zmp*unk(8,i1,i2m,i3p,jb) &
                 +zpm*unk(8,i1,i2p,i3m,jb)+zpp*unk(8,i1,i2p,i3p,jb)
#endif
#ifdef etot
#ifdef mhd
        zukin=zhalf*(dpla(jp)*v1pla(jp)*v1pla(jp)  &
                    +dpla(jp)*v2pla(jp)*v2pla(jp)  &
                    +dpla(jp)*v3pla(jp)*v3pla(jp))
        zumag=zr2mu*(b1pla(jp)*b1pla(jp)+b2pla(jp)*b2pla(jp)+ &
                     b3pla(jp)*b3pla(jp))
        zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
              +zmp*unk(2,i1,i2m,i3p,jb) &
              +zpm*unk(2,i1,i2p,i3m,jb) &
              +zpp*unk(2,i1,i2p,i3p,jb) &
              -zukin-zumag
#ifdef gam3d
        tpla(jp)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dpla(jp)
#else
        tpla(jp)=((gamma-zone)/zgas)*zuthe/dpla(jp)
#endif
#else
        zukin=zhalf*(dpla(jp)*v1pla(jp)*v1pla(jp)  &
                    +dpla(jp)*v2pla(jp)*v2pla(jp)  &
                    +dpla(jp)*v3pla(jp)*v3pla(jp))
        zuthe=zmm*unk(2,i1,i2m,i3m,jb) &
             +zmp*unk(2,i1,i2m,i3p,jb) &
             +zpm*unk(2,i1,i2p,i3m,jb) &
             +zpp*unk(2,i1,i2p,i3p,jb) &
             -zukin
#ifdef gam3d
        tpla(jp)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)*zuthe/dpla(jp)
#else
        tpla(jp)=((gamma-zone)/zgas)*zuthe/dpla(jp)
#endif
#endif
#else
#ifdef gam3d
        tpla(jp)=((gamma(i1,i2m,i3m,jb)-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
          +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
          +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
          +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#else
        tpla(jp)=((gamma-zone)/zgas)* &
          (zmm*(unk(2,i1,i2m,i3m,jb)/unk(1,i1,i2m,i3m,jb)) &
          +zmp*(unk(2,i1,i2m,i3p,jb)/unk(1,i1,i2m,i3p,jb)) &
          +zpm*(unk(2,i1,i2p,i3m,jb)/unk(1,i1,i2p,i3m,jb)) &
          +zpp*(unk(2,i1,i2p,i3p,jb)/unk(1,i1,i2p,i3p,jb)))
#endif
#endif
#ifdef mhd
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        dppla(jp)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
                 +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
#ifdef mhdcon
        iv=iv+1
        bppla(jp)=zmm*unk(iv,i1,i2m,i3m,jb)+zmp*unk(iv,i1,i2m,i3p,jb) &
                 +zpm*unk(iv,i1,i2p,i3m,jb)+zpp*unk(iv,i1,i2p,i3p,jb)
#endif
!---------------------------------------------------------------------
      endif
      endif
      endif
    enddo
  enddo
!---------------------------------------------------------------------
!L     8.        Copy to the last processor
#ifdef MPI
    if(myproc.gt.0) then
      ic=-1
      do jp=1,npla
        ic=ic+1
        zsend(ic+1)=x1pla(jp)
        zsend(ic+2)=x2pla(jp)
        zsend(ic+3)=x3pla(jp)
        zsend(ic+4)=dpla(jp)
        zsend(ic+5)=tpla(jp)
        zsend(ic+6)=v1pla(jp)
        zsend(ic+7)=v2pla(jp)
        zsend(ic+8)=v3pla(jp)
#ifdef mhd
          zsend(ic+9)=b1pla(ipla)
          zsend(ic+10)=b2pla(ipla)
          zsend(ic+11)=b3pla(ipla)
#ifdef hydcon
            zsend(ic+12)=dppla(ipla)
#ifdef mhdcon
              zsend(ic+13)=bppla(ipla)
              ic=ic+13
#else
              ic=ic+12
#endif
#else
#ifdef mhdcon
              zsend(ic+12)=bppla(ipla)
              ic=ic+12
#else
              ic=ic+11
#endif
#endif
#else
#ifdef hydcon
            zsend(ic+9)=dppla(ipla)
            ic=ic+9
#else
            ic=ic+8
#endif
#endif
      enddo
      call MPI_SEND(zsend,ibuf,MPI_DOUBLE_PRECISION,0,  &
                    520,MPI_COMM_WORLD,ierr)
!     call MPI_GATHER(zsend,ic,MPI_DOUBLE_PRECISION, &
!                     zrecv,ic,MPI_DOUBLE_PRECISION, &
!                     0,MPI_COMM_WORLD,ierr)
!     call MPI_BARRIER(MPI_COMM_WORLD,ierr)
    endif
!---------------------------------------------------------------------
!L     9.        Get values at the first processor
    if(myproc.eq.0) then
      do jproc=2,nprocs
        call MPI_RECV(zrecv,ibuf,MPI_DOUBLE_PRECISION,jproc-1,  &
          520,MPI_COMM_WORLD,status,ierr)
        ic=-1
        do jp=1,npla
          ic=ic+1
          if(zrecv(ic).gt.0d0) then
            x1pla(jp)=zrecv(ic+1)
            x2pla(jp)=zrecv(ic+2)
            x3pla(jp)=zrecv(ic+3)
            dpla(jp)=zrecv(ic+4)
            tpla(jp)=zrecv(ic+5)
            v1pla(jp)=zrecv(ic+6)
            v2pla(jp)=zrecv(ic+7)
            v3pla(jp)=zrecv(ic+8)
#ifdef mhd
              b1pla(jp)=zrecv(ic+9)
              b2pla(jp)=zrecv(ic+10)
              b3pla(jp)=zrecv(ic+11)
#ifdef hydcon
                dppla(jp)=zrecv(ic+12)
#ifdef mhdcon
                  bppla(jp)=zrecv(ic+13)
                  ic=ic+13
#else
                  ic=ic+12
#endif
#else
#ifdef mhdcon
                  bppla(jp)=zrecv(ic+12)
                  ic=ic+12
#else
                  ic=ic+11
#endif
#endif
#else
#ifdef hydcon
                dppla(jp)=zrecv(ic+9)
                ic=ic+9
#else
                ic=ic+8
#endif
#endif
          endif
        enddo
      enddo
    endif
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(4003,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE PLAVAL
