!****f* e0pro/main
! NAME 
!   BNDTIM
! PURPOSE
!   Time-dependent boundary conditions
! INPUTS
!   kmode = 1,2 for initial, periodic action   
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDTIM(kmode)
!---------------------------------------------------------------------
#ifdef PARAMESH
#include "paramesh_preprocessor.fh"
  use paramesh_dimensions
  use physicaldata
  use tree
#endif
  USE comoly
  USE comdim
  USE comreg
  USE comphy
  USE combnd
  USE commet
  IMPLICIT NONE
  INTEGER :: kmode, jb
#ifdef DIAGNO
  call EXPERT(6001,1)
#endif
!---------------------------------------------------------------------
!L     1.        Get boundary values
!L       1.1       Using InterComm
#ifdef ICOMM
  if(kmode.eq.1) call MASCON
  call MASTIM(kmode)
#else
!L       1.2       Using data file
  call BNDRDR(kmode)
#endif
!---------------------------------------------------------------------
!L     2.        Interpolate
  do jb=1,nblk
#ifdef NGRID
  if(nbc1l(1,jb).eq.-1) then
#endif
    call BNDMAP(jb)
    call BNDINT
    call BND0(jb)
#ifdef NGRID
  endif
#endif
  enddo
! if(nbndin.eq.0) call BNDIN0
! if(nbndin.eq.1) call BNDIN1
! if(nbndin.eq.2) call BNDIN2
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(6001,0)
#endif                       
!---------------------------------------------------------------------  
RETURN 
END SUBROUTINE BNDTIM                      


!****f* e0pro/main
! NAME 
!   BND0
! PURPOSE
!   X1-boundary external conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND0(kblk)
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
  use paramesh_dimensions 
  use physicaldata 
  use tree 
#endif                                                                  
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE combnd
  USE comtva 
  IMPLICIT NONE
  INTEGER :: j1,j2,j3, i2, i3, iv, kblk
#ifdef DIAGNO                                                           
  call EXPERT(5001,1)
#endif
!---------------------------------------------------------------------
#ifdef MPI
  if(myproc.eq.0) then
#endif
#ifdef bc1l_specified
!---------------------------------------------------------------------
!L     1.        One interface cell
#ifdef bctim1h
!L       1.1       Specified values (X1-X2-X3 array)
    do j3=1,ncc3
      i3=ng3+j3
      do j2=1,ncc2
        i2=ng2+j2
        unk(1,1,i2,i3,kblk)=2d0*dpv(j2,j3)-unk(1,4,i2,i3,kblk)
        unk(1,2,i2,i3,kblk)=2d0*dpv(j2,j3)-unk(1,3,i2,i3,kblk)
        unk(2,1,i2,i3,kblk)=2d0*uupv(j2,j3)-unk(2,4,i2,i3,kblk)
        unk(2,2,i2,i3,kblk)=2d0*uupv(j2,j3)-unk(2,3,i2,i3,kblk)
        unk(3,1,i2,i3,kblk)=2d0*dv1pv(j2,j3)-unk(3,4,i2,i3,kblk)
        unk(3,2,i2,i3,kblk)=2d0*dv1pv(j2,j3)-unk(3,3,i2,i3,kblk)
        unk(4,1,i2,i3,kblk)=2d0*dv2pv(j2,j3)-unk(4,4,i2,i3,kblk)
        unk(4,2,i2,i3,kblk)=2d0*dv2pv(j2,j3)-unk(4,3,i2,i3,kblk)
        unk(5,1,i2,i3,kblk)=2d0*dv3pv(j2,j3)-unk(5,4,i2,i3,kblk)
        unk(5,2,i2,i3,kblk)=2d0*dv3pv(j2,j3)-unk(5,3,i2,i3,kblk)
#ifdef mhd
        unk(6,1,i2,i3,kblk)=ztwo*b1pv(j2,j3)-unk(6,3,i2,i3,kblk)
        unk(6,2,i2,i3,kblk)=zthree*b1pv(j2,j3)-unk(6,3,i2,i3,kblk)
        unk(7,1,i2,i3,kblk)=ztwo*b2pv(j2,j3)-unk(7,3,i2,i3,kblk)
        unk(7,2,i2,i3,kblk)=zthree*b2pv(j2,j3)-unk(7,3,i2,i3,kblk)
        unk(8,1,i2,i3,kblk)=ztwo*b3pv(j2,j3)-unk(8,3,i2,i3,kblk)
        unk(8,2,i2,i3,kblk)=zthree*b3pv(j2,j3)-unk(8,3,i2,i3,kblk)
        iv=8
#else
        iv=5
#endif
#ifdef hydcon
        iv=iv+1
        unk(iv,1,i2,i3,kblk)=ztwo*dppv(j2,j3)-unk(iv,3,i2,i3,kblk)
        unk(iv,2,i2,i3,kblk)=zthree*dppv(j2,j3)-unk(iv,3,i2,i3,kblk)
#endif
#ifdef mhdcon
        iv=iv+1
        unk(iv,1,i2,i3,kblk)=ztwo*bppv(j2,j3)-unk(iv,3,i2,i3,kblk)
        unk(iv,2,i2,i3,kblk)=zthree*bppv(j2,j3)-unk(iv,3,i2,i3,kblk)
#endif
#ifdef mhd
#ifdef staggered
        b1hcc(j1,ng2+j2,ng3+j3)=b1pv(j2,j3)
        b2chc(j1,ng2+j2,ng3+j3)=b2pv(j2,j3)
        b3cch(j1,ng2+j2,ng3+j3)=b3pv(j2,j3)
#endif
#endif
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!L     2.        One boundary cell
#ifdef bctim1c
!L       1.2       Specified values (X1-X2-X3 array)
    do j3=1,ncc3
      i3=ng3+j3
      do j2=1,ncc2
        i2=ng2+j2
        do j1=1,ng1
          unk(1,j1,i2,i3,kblk)=dpv(j2,j3)
          unk(2,j1,i2,i3,kblk)=uupv(j2,j3)
          unk(3,j1,i2,i3,kblk)=dv1pv(j2,j3)
          unk(4,j1,i2,i3,kblk)=dv2pv(j2,j3)
          unk(5,j1,i2,i3,kblk)=dv3pv(j2,j3)
#ifdef mhd
          unk(6,j1,i2,i3,kblk)=b1pv(j2,j3)
          unk(7,j1,i2,i3,kblk)=b2pv(j2,j3)
          unk(8,j1,i2,i3,kblk)=b3pv(j2,j3)
          iv=8
#else
          iv=5
#endif
#ifdef hydcon
          iv=iv+1
          unk(iv,j1,i2,i3,kblk)=dppv(j2,j3)
#endif
#ifdef mhdcon
          iv=iv+1
          unk(iv,j1,i2,i3,kblk)=bppv(j2,j3)
#endif
#ifdef mhd
#ifdef staggered
          b1hcc(j1,i2,i3)=b1pv(j2,j3)
          b2chc(j1,i2,i3)=b2pv(j2,j3)
          b3cch(j1,i2,i3)=b3pv(j2,j3)
#endif
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
!L     3.        Two boundary cells
#ifdef bctim2c
!L       1.3       Specified values (X1-X2-X3 array)
    do j3=1,ncc3
      i3=ng3+j3
      do j2=1,ncc2
        i2=ng2+j2
        do j1=1,nb1
          unk(1,j1,i2,i3,kblk)=dpv(j1,j2,j3)
          unk(2,j1,i2,i3,kblk)=uupv(j1,j2,j3)
          unk(3,j1,i2,i3,kblk)=dv1pv(j1,j2,j3)
          unk(4,j1,i2,i3,kblk)=dv2pv(j1,j2,j3)
          unk(5,j1,i2,i3,kblk)=dv3pv(j1,j2,j3)
#ifdef mhd
          unk(6,j1,i2,i3,kblk)=b1pv(j1,j2,j3)
          unk(7,j1,i2,i3,kblk)=b2pv(j1,j2,j3)
          unk(8,j1,i2,i3,kblk)=b3pv(j1,j2,j3)
          iv=8
#else
          iv=5
#endif
#ifdef hydcon
          iv=iv+1
          unk(iv,j1,i2,i3,kblk)=dppv(j1,j2,j3)
#endif
#ifdef mhdcon
          iv=iv+1
          unk(iv,j1,i2,i3,kblk)=bppv(j1,j2,j3)
#endif
#ifdef mhd
#ifdef staggered
          b1hcc(j1,i2,i3)=b1pv(j1,j2,j3)
          b2chc(j1,i2,i3)=b2pv(j1,j2,j3)
          b3cch(j1,i2,i3)=b3pv(j1,j2,j3)
#endif
#endif
        enddo
      enddo
    enddo
#endif
!---------------------------------------------------------------------
#endif
#ifdef MPI
  endif
#endif
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5001,0) 
#endif 
RETURN
END SUBROUTINE BND0
