!****f* e0pro/main
! NAME 
!   BND1
! PURPOSE
!   X1-boundary conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND1 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
  use paramesh_dimensions 
  use physicaldata 
  use tree 
#endif                                                                  
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE combnd
  USE comtva 
  IMPLICIT NONE
  INTEGER :: jb
#ifdef DIAGNO                                                           
  call EXPERT(5001,1)
#endif
!---------------------------------------------------------------------
!L     1.        X1-left boundary
  do jb=1,nblk
#ifdef MPI
    if(myproc.ne.0) then
#endif
#ifdef NGRID
    if(nbc1l(1,jb).eq.-2) then
#endif
!L       1.4       Free flow (X1-X2-X3 array)
#ifdef bc1l_free
      unk(:,1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,2,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,3,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,3,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.5       Free flow (X1-X2-X3 array)
#ifdef bc1l_flow
      unk(:,1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,2,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,3,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,3,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.6       Solid wall (X1-X2-X3 array)
#ifdef bc1l_solid
      unk(:,1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      unk(3,1,nc2l:nc2r,nc3l:nc3r,jb)=-unk(3,4,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,2,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      unk(3, 2, nc2l:nc2r, nc3l:nc3r, jb)=-unk(3,3,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,2,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,3,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.7       Periodic system (X1-X2-X3 array)
#ifdef bc1l_periodic
      unk(:,1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,2,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,1,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevarx(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,1,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevary(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,1,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevarz(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,2,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevarx(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,2,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevary(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,2,nc2l:nc2r,nc3l:nc3r,jb)=      &
        facevarz(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
#ifdef MPI
    endif
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
!L     2.        X1-right boundary
  do jb=1,nblk
#ifdef MPI
    if(myproc.eq.nprocs- 1) then
#endif
#ifdef NGRID
    if(nbc1r(1,jb).eq.-2) then
#endif
!L       2.1       Free flow (X1-X2-X3 array)
#ifdef bc1r_free
      unk(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarx(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevary(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarz(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.2       Free flow (X1-X2-X3 array)
#ifdef bc1r_flow
      unk(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarx(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevary(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarz(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.3       Solid wall (X1-X2-X3 array)
#ifdef bc1r_solid
      unk(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      unk(3,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=-unk(3,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      unk(3,nc1,nc2l:nc2r,nc3l:nc3r,jb)=-unk(3,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarx(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevary(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=  &
        facevarz(:,nc1-2,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarx(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevary(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevary(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarz(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=    &
        facevarz(:,nc1-3,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.4       Periodic system (X1-X2-X3 array)
#ifdef bc1r_periodic
      unk(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      unk(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=unk(:,4,nc2l:nc2r,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)= &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)= &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1-1,nc2l:nc2r,nc3l:nc3r,jb)= &
        facevarx(:,3,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=   &
        facevarx(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=   &
        facevarx(:,4,nc2l:nc2r,nc3l:nc3r,jb)
      facevarx(:,nc1,nc2l:nc2r,nc3l:nc3r,jb)=   &
        facevarx(:,4,nc2l:nc2r,nc3l:nc3r,jb)
#endif
#endif
#endif
#ifdef MPI
    endif
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5001,0) 
#endif 
RETURN
END SUBROUTINE BND1
