!****f* e0pro/main
! NAME 
!   BND2
! PURPOSE
!   X2-boundary conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND2 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
  use paramesh_dimensions 
  use physicaldata 
  use tree 
#endif                                                                  
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE comtva 
  IMPLICIT NONE
#ifdef x2dim
  INTEGER :: jb 
#endif
#ifdef DIAGNO                                                           
  call EXPERT(5002,1)
#endif
!---------------------------------------------------------------------
#ifdef x2dim
!L     1.        X2-left boundary
  do jb=1,nblk
#ifdef NGRID
    if(nbc2l(1,jb).eq.-2) then
#endif
!L       1.1       Free flow (X1-X2-X3 array)
#ifdef bc2l_free
      unk(:,:,1,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(:,:,2,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,1,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,1,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,1,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
      facevarx(:,:,2,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,2,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,2,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.2       Free flow (X1-X2-X3 array)
#ifdef bc2l_free0
      unk(:,:,1,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(:,:,2,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(4,:,1,nc3l:nc3r,jb)=zero
      unk(4,:,2,nc3l:nc3r,jb)=zero
#ifdef mhd
#ifdef staggered
      facevarx(:,:,1,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,1,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,1,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
      facevarx(:,:,2,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,2,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,2,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.3       Free flow (X1-X2-X3 array)
#ifdef bc2l_flow
      unk(:,:,1,nc3l:nc3r,jb)=unk(:,:,4,nc3l:nc3r,jb)
      unk(:,:,2,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,1,nc3l:nc3r,jb)=facevarx(:,:,4,nc3l:nc3r,jb)
      facevary(:,:,1,nc3l:nc3r,jb)=facevary(:,:,4,nc3l:nc3r,jb)
      facevarz(:,:,1,nc3l:nc3r,jb)=facevarz(:,:,4,nc3l:nc3r,jb)
      facevarx(:,:,2,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,2,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,2,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.4       Solid wall (X1-X2-X3 array)
#ifdef bc2l_solid
      unk(:,:,1,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(4,:,1,nc3l:nc3r,jb)=-unk(4,:,3,nc3l:nc3r,jb)
      unk(:,:,2,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(4,:,2,nc3l:nc3r,jb)=-unk(4,:,3,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,1,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,1,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,1,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
      facevarx(:,:,2,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,2,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,2,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       1.5       Periodic system (X1-X2-X3 array)
#ifdef bc2l_periodic
      unk(:,:,1,nc3l:nc3r,jb)=unk(:,:,nc2-3,nc3l:nc3r,jb)
      unk(:,:,2,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,1,nc3l:nc3r,jb)=facevarx(:,:,nc2-3,nc3l:nc3r,jb)
      facevary(:,:,1,nc3l:nc3r,jb)=facevary(:,:,nc2-3,nc3l:nc3r,jb)
      facevarz(:,:,1,nc3l:nc3r,jb)=facevarz(:,:,nc2-3,nc3l:nc3r,jb)
#endif
#endif
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
!L     2.        X2-right boundary
  do jb=1,nblk
#ifdef NGRID
    if(nbc2r(1,jb).eq.-2) then
#endif
!L       2.1       Free flow (X1-X2-X3 array)
#ifdef bc2r_free
      unk(:,:,nc2-1,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
      unk(:,:,nc2,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,nc2-1,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2-1,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2-1,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
      facevarx(:,:,nc2,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.2       Free flow (X1-X2-X3 array)
#ifdef bc2r_free0
      unk(:,:,nc2-1,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
      unk(:,:,nc2,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
      unk(4,:, nc2-1,nc3l:nc3r,jb)=zero
      unk(4,:, nc2,nc3l:nc3r,jb)=zero
#ifdef mhd
#ifdef staggered
      facevarx(:,:,nc2-1,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2-1,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2-1,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
      facevarx(:,:,nc2,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.3       Free flow (X1-X2-X3 array)
#ifdef bc2r_flow
      unk(:,:,nc2-1,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
      unk(:,:,nc2,nc3l:nc3r,jb)=unk(:,:,nc2-3,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,nc2-1,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2-1,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2-1,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
      facevarx(:,:,nc2,nc3l:nc3r,jb)=facevarx(:,:,nc2-3,nc3l:nc3r,jb)
      facevary(:,:,nc2,nc3l:nc3r,jb)=facevary(:,:,nc2-3,nc3l:nc3r,jb)
      facevarz(:,:,nc2,nc3l:nc3r,jb)=facevarz(:,:,nc2-3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.4       Solid wall (X1-X2-X3 array)
#ifdef bc2r_solid
      unk(:,:,nc2-1,nc3l:nc3r,jb)=unk(:,:,nc2-2,nc3l:nc3r,jb)
      unk(4,:,nc2-1,nc3l:nc3r,jb)=-unk(4,:,nc2-2,nc3l:nc3r,jb)
      unk(:,:,nc2,nc3l:nc3r,jb)=unk(:,:,nc2-3,nc3l:nc3r,jb)
      unk(4,:,nc2,nc3l:nc3r,jb)=- unk(4,:,nc2-3,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,nc2-1,nc3l:nc3r,jb)=facevarx(:,:,nc2-2,nc3l:nc3r,jb)
      facevary(:,:,nc2-1,nc3l:nc3r,jb)=facevary(:,:,nc2-2,nc3l:nc3r,jb)
      facevarz(:,:,nc2-1,nc3l:nc3r,jb)=facevarz(:,:,nc2-2,nc3l:nc3r,jb)
      facevarx(:,:,nc2,nc3l:nc3r,jb)=facevarx(:,:,nc2-3,nc3l:nc3r,jb)
      facevary(:,:,nc2,nc3l:nc3r,jb)=facevary(:,:,nc2-3,nc3l:nc3r,jb)
      facevarz(:,:,nc2,nc3l:nc3r,jb)=facevarz(:,:,nc2-3,nc3l:nc3r,jb)
#endif
#endif
#endif
!L       2.5       Periodic system (X1-X2-X3 array)
#ifdef bc2r_periodic
      unk(:,:,nc2-1,nc3l:nc3r,jb)=unk(:,:,3,nc3l:nc3r,jb)
      unk(:,:,nc2,nc3l:nc3r,jb)=unk(:,:,4,nc3l:nc3r,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,nc2-1,nc3l:nc3r,jb)=facevarx(:,:,3,nc3l:nc3r,jb)
      facevary(:,:,nc2-1,nc3l:nc3r,jb)=facevary(:,:,3,nc3l:nc3r,jb)
      facevarz(:,:,nc2-1,nc3l:nc3r,jb)=facevarz(:,:,3,nc3l:nc3r,jb)
      facevarx(:,:,nc2,nc3l:nc3r,jb)=facevarx(:,:,4,nc3l:nc3r,jb)
      facevary(:,:,nc2,nc3l:nc3r,jb)=facevary(:,:,4,nc3l:nc3r,jb)
      facevarz(:,:,nc2,nc3l:nc3r,jb)=facevarz(:,:,4,nc3l:nc3r,jb)
#endif
#endif
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
#endif
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5002,0) 
#endif                                                                  
RETURN
END SUBROUTINE BND2
