!****f* e0pro/main
! NAME 
!   BND2NG
! PURPOSE
!   Transfer boundary values between blocks
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND2NG 
!---------------------------------------------------------------------
  USE comoly 
  USE comdim 
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
#ifdef NGRID                                                              
  INTEGER :: i1, ib, j1, jb
#ifdef x2dim
  INTEGER :: i2, j2
#endif
#ifdef x3dim
  INTEGER :: i3, j3
#endif
#endif
#ifdef DIAGNO
  call EXPERT(5007,1)
#endif
#ifdef NGRID
#ifdef x2dim
!---------------------------------------------------------------------
!L     1.        X2-Left boundary
  do jb=1,nblk
    ib=nbc2l(1,jb)
    if(ib.gt.0) then
!L       1.1       From equally-spaced grid
      if(nlevel(2,ib).eq.nlevel(2,jb)) then
        unk(:,:,nc2l-2,:,jb)=unk(:,:,nc2r-1,:,ib)
        unk(:,:,nc2l-1,:,jb)=unk(:,:,nc2r,:,ib)
      endif
!L       1.2       From rougher block
      if(nlevel(2,ib).lt.nlevel(2,jb)) then
        if(nlevel(1,ib).lt.nlevel(1,jb)) then
          if(nbc2l(2,jb).lt.0) then
            i1=nc1l-1
          else
            i1=nc1rr
          endif
          do j1=nc1l,nc1r,2
            i1=i1+1
#ifdef x3dim
            i3=nc325
            do j3=nc3l,nc3r,2
              i3=i3+1
              unk(:,j1  ,nc2l-2,j3  ,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1+1,nc2l-2,j3  ,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1  ,nc2l-1,j3  ,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1+1,nc2l-1,j3  ,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1  ,nc2l-2,j3+1,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1+1,nc2l-2,j3+1,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1  ,nc2l-1,j3+1,jb)=unk(:,i1,nc225,i3,ib)
              unk(:,j1+1,nc2l-1,j3+1,jb)=unk(:,i1,nc225,i3,ib)
            enddo
#else
            unk(:,j1  ,nc2l-2,1,jb)=unk(:,i1,nc225,1,ib)
            unk(:,j1+1,nc2l-2,1,jb)=unk(:,i1,nc225,1,ib)
            unk(:,j1  ,nc2l-1,1,jb)=unk(:,i1,nc225,1,ib)
            unk(:,j1+1,nc2l-1,1,jb)=unk(:,i1,nc225,1,ib)
#endif
          enddo
        else
!L       1.3       From rougher interface
          do j1=nc1l,nc1r
#ifdef x3dim
            i3=nc325
            do j3=nc3l,nc3r,2
              i3=i3+1
              unk(:,j1,nc2l-2,j3  ,jb)=unk(:,j1,nc225,i3,ib)
              unk(:,j1,nc2l-1,j3  ,jb)=unk(:,j1,nc225,i3,ib)
              unk(:,j1,nc2l-2,j3+1,jb)=unk(:,j1,nc225,i3,ib)
              unk(:,j1,nc2l-1,j3+1,jb)=unk(:,j1,nc225,i3,ib)
            enddo
#else
            unk(:,j1,nc2l-2,1,jb)=unk(:,j1,nc225,1,ib)
            unk(:,j1,nc2l-1,1,jb)=unk(:,j1,nc225,1,ib)
#endif
          enddo
        endif
      endif
!L       1.4       Continue
    endif
  enddo
!---------------------------------------------------------------------
!L     2.        X2-Right boundary
  do jb=1,nblk
    ib=nbc2r(1,jb)
    if(ib.gt.0) then
!L       2.1       From equally-spaced grid
      if(nlevel(2,ib).eq.nlevel(2,jb)) then
        unk(:,:,nc2r+1,:,jb)=unk(:,:,nc2l  ,:,ib)
        unk(:,:,nc2r+2,:,jb)=unk(:,:,nc2l+1,:,ib)
      endif
!L       2.2       From rougher block
      if(nlevel(2,ib).lt.nlevel(2,jb)) then
        if(nlevel(1,ib).lt.nlevel(1,jb)) then
          if(nbc2r(2,jb).lt.0) then
            i1=nc1l-1
          else
            i1=nc1rr
          endif
          do j1=nc1l,nc1r,2
            i1=i1+1
#ifdef x3dim
            i3=nc325
            do j3=nc3l,nc3r,2
              i3=i3+1
              unk(:,j1  ,nc2r+1,j3  ,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1+1,nc2r+1,j3  ,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1  ,nc2r+2,j3  ,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1+1,nc2r+2,j3  ,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1  ,nc2r+1,j3+1,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1+1,nc2r+1,j3+1,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1  ,nc2r+2,j3+1,jb)=unk(:,i1,nc275,i3,ib)
              unk(:,j1+1,nc2r+2,j3+1,jb)=unk(:,i1,nc275,i3,ib)
            enddo
#else
            unk(:,j1  ,nc2r+1,1,jb)=unk(:,i1,nc275,1,ib)
            unk(:,j1+1,nc2r+1,1,jb)=unk(:,i1,nc275,1,ib)
            unk(:,j1  ,nc2r+2,1,jb)=unk(:,i1,nc275,1,ib)
            unk(:,j1+1,nc2r+2,1,jb)=unk(:,i1,nc275,1,ib)
#endif
          enddo
        else
!L       2.3       From rougher interface
          do j1=nc1l,nc1r
#ifdef x3dim
            i3=nc325
            do j3=nc3l,nc3r,2
              i3=i3+1
              unk(:,j1,nc2r+1,j3  ,jb)=unk(:,j1,nc275,i3,ib)
              unk(:,j1,nc2r+2,j3  ,jb)=unk(:,j1,nc275,i3,ib)
              unk(:,j1,nc2r+1,j3+1,jb)=unk(:,j1,nc275,i3,ib)
              unk(:,j1,nc2r+2,j3+1,jb)=unk(:,j1,nc275,i3,ib)
            enddo
#else
            unk(:,j1,nc2r+1,1,jb)=unk(:,j1,nc275,1,ib)
            unk(:,j1,nc2r+2,1,jb)=unk(:,j1,nc275,1,ib)
#endif
          enddo
        endif
      endif
!L       2.4       Continue
    endif
  enddo
!---------------------------------------------------------------------
#endif
#endif
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5007,0)
#endif
RETURN
END SUBROUTINE BND2NG
