!****f* e0pro/main
! NAME 
!   BND3
! PURPOSE
!   X3-boundary conditions
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND3 
!---------------------------------------------------------------------
#ifdef PARAMESH                                                         
#include "paramesh_preprocessor.fh"                                     
  use paramesh_dimensions 
  use physicaldata 
  use tree 
#endif        
  USE comoly 
  USE comdim 
  USE commet
  USE comreg
  USE comglo
  USE comtva 
  IMPLICIT NONE
#ifdef x3dim
  INTEGER :: jb 
#ifdef bc3l_aperiodic
  INTEGER :: i2, j1, j2 
#endif
#endif
#ifdef DIAGNO                                                           
  call EXPERT(5003,0)
#endif
!---------------------------------------------------------------------
#ifdef x3dim
!L     1.        X3-left boundary
  do jb=1,nblk
#ifdef NGRID
    if(nbc3l(1,jb).eq.-2) then
#endif
!L       1.1       Free flow (X1-X2-X3 array)
#ifdef bc3l_free
      unk(:,:,:,1,jb)=unk(:,:,:,3,jb)
      unk(:,:,:,2,jb)=unk(:,:,:,3,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,1,jb)=facevarx(:,:,:,3,jb)
      facevary(:,:,:,1,jb)=facevary(:,:,:,3,jb)
      facevarz(:,:,:,1,jb)=facevarz(:,:,:,3,jb)
      facevarx(:,:,:,2,jb)=facevarx(:,:,:,3,jb)
      facevary(:,:,:,2,jb)=facevary(:,:,:,3,jb)
      facevarz(:,:,:,2,jb)=facevarz(:,:,:,3,jb)
#endif
#endif
#endif
!L       1.2       Free flow (X1-X2-X3 array)
#ifdef bc3l_flow
      unk(:,:,:,1,jb)=unk(:,:,:,4,jb)
      unk(:,:,:,2,jb)=unk(:,:,:,3,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,1,jb)=facevarx(:,:,:,4,jb)
      facevary(:,:,:,1,jb)=facevary(:,:,:,4,jb)
      facevarz(:,:,:,1,jb)=facevarz(:,:,:,4,jb)
      facevarx(:,:,:,2,jb)=facevarx(:,:,:,3,jb)
      facevary(:,:,:,2,jb)=facevary(:,:,:,3,jb)
      facevarz(:,:,:,2,jb)=facevarz(:,:,:,3,jb)
#endif
#endif
#endif
!L       1.3       Solid wall (X1-X2-X3 array)
#ifdef bc3l_solid
      unk(:,:,:,1,jb)=unk(:,:,:,4,jb)
      unk(5,:,:,1,jb)=-unk(5,:,:,4,jb)
      unk(:,:,:,2,jb)=unk(:,:,:,3,jb)
      unk(5,:,:,2,jb)=-unk(5,:,:,3,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,1,jb)=facevarx(:,:,:,4,jb)
      facevary(:,:,:,1,jb)=facevary(:,:,:,4,jb)
      facevarz(:,:,:,1,jb)=facevarz(:,:,:,4,jb)
      facevarx(:,:,:,2,jb)=facevarx(:,:,:,3,jb)
      facevary(:,:,:,2,jb)=facevary(:,:,:,3,jb)
      facevarz(:,:,:,2,jb)=facevarz(:,:,:,3,jb)
#endif
#endif
#endif
!L       1.4       Periodic system (X1-X2-X3 array)
#ifdef bc3l_periodic
      unk(:,:,:,1,jb)=unk(:,:,:,nc3-3,jb)
      unk(:,:,:,2,jb)=unk(:,:,:,nc3-2,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,1,jb)=facevarx(:,:,:,nc3-3,jb)
      facevary(:,:,:,1,jb)=facevary(:,:,:,nc3-3,jb)
      facevarz(:,:,:,1,jb)=facevarz(:,:,:,nc3-3,jb)
      facevarx(:,:,:,2,jb)=facevarx(:,:,:,nc3-2,jb)
      facevary(:,:,:,2,jb)=facevary(:,:,:,nc3-2,jb)
      facevarz(:,:,:,2,jb)=facevarz(:,:,:,nc3-2,jb)
#endif
#endif
#endif
!L       1.5       Aperiodic system (X1-X2-X3 array)
#ifdef bc3l_aperiodic
      do j2=nc2l,nc2r
        i2=nc2-j2+1
        do j1=nc1l,nc1r
          unk(:,:,j2,1,jb)=unk(:,:,i2,nc3-3,jb)
          unk(:,:,j2,2,jb)=unk(:,:,i2,nc3-2,jb)
#ifdef mhd
#ifdef staggered
          facevarx(:,:,j2,1,jb)=facevarx(:,:,i2,nc3-3,jb)
          facevary(:,:,j2,1,jb)=facevary(:,:,i2,nc3-3,jb)
          facevarz(:,:,j2,1,jb)=facevarz(:,:,i2,nc3-3,jb)
          facevarx(:,:,j2,2,jb)=facevarx(:,:,i2,nc3-2,jb)
          facevary(:,:,j2,2,jb)=facevary(:,:,i2,nc3-2,jb)
          facevarz(:,:,j2,2,jb)=facevarz(:,:,i2,nc3-2,jb)
#endif
#endif
        enddo
      enddo
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
!L     2.        X3-right boundary
  do jb=1,nblk
#ifdef NGRID
    if(nbc3r(1,jb).eq.-2) then
#endif
!L       2.1       Free flow (X1-X2-X3 array)
#ifdef bc3r_free
      unk(:,:,:,nc3-1,jb)=unk(:,:,:,nc3-2,jb)
      unk(:,:,:,nc3,jb)=unk(:,:,:,nc3-2,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,nc3-1,jb)=facevarx(:,:,:,nc3-2,jb)
      facevary(:,:,:,nc3-1,jb)=facevary(:,:,:,nc3-2,jb)
      facevarz(:,:,:,nc3-1,jb)=facevarz(:,:,:,nc3-2,jb)
      facevarx(:,:,:,nc3,jb)=facevarx(:,:,:,nc3-2,jb)
      facevary(:,:,:,nc3,jb)=facevary(:,:,:,nc3-2,jb)
      facevarz(:,:,:,nc3,jb)=facevarz(:,:,:,nc3-2,jb)
#endif
#endif
#endif
!L       2.2       Free flow (X1-X2-X3 array)
#ifdef bc3r_flow
      unk(:,:,:,nc3-1,jb)=unk(:,:,:,nc3-2,jb)
      unk(:,:,:,nc3,jb)=unk(:,:,:,nc3-3,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,nc3-1,jb)=facevarx(:,:,:,nc3-2,jb)
      facevary(:,:,:,nc3-1,jb)=facevary(:,:,:,nc3-2,jb)
      facevarz(:,:,:,nc3-1,jb)=facevarz(:,:,:,nc3-2,jb)
      facevarx(:,:,:,nc3,jb)=facevarx(:,:,:,nc3-3,jb)
      facevary(:,:,:,nc3,jb)=facevary(:,:,:,nc3-3,jb)
      facevarz(:,:,:,nc3,jb)=facevarz(:,:,:,nc3-3,jb)
#endif
#endif
#endif
!L       2.3       Solid wall (X1-X2-X3 array)
#ifdef bc3r_solid
      unk(:,:,:,nc3-1,jb)=unk(:,:,:,nc3-2,jb)
      unk(5,:,:,nc3-1,jb)=-unk(5,:,:,nc3-2,jb)
      unk(:,:,:,nc3,jb)=unk(:,:,:,nc3-3,jb)
      unk(5,:,:,nc3,jb)=-unk(5,:,:,nc3-3,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,nc3-1,jb)=facevarx(:,:,:,nc3-2,jb)
      facevary(:,:,:,nc3-1,jb)=facevary(:,:,:,nc3-2,jb)
      facevarz(:,:,:,nc3-1,jb)=facevarz(:,:,:,nc3-2,jb)
      facevarx(:,:,:,nc3,jb)=facevarx(:,:,:,nc3-3,jb)
      facevary(:,:,:,nc3,jb)=facevary(:,:,:,nc3-3,jb)
      facevarz(:,:,:,nc3,jb)=facevarz(:,:,:,nc3-3,jb)
#endif
#endif
#endif
!L       2.4       Periodic system (X1-X2-X3 array)
#ifdef bc3r_periodic
      unk(:,:,:,nc3-1,jb)=unk(:,:,:,3,jb)
      unk(:,:,:,nc3,jb)=unk(:,:,:,4,jb)
#ifdef mhd
#ifdef staggered
      facevarx(:,:,:,nc3-1,jb)=facevarx(:,:,:,3,jb)
      facevary(:,:,:,nc3-1,jb)=facevary(:,:,:,3,jb)
      facevarz(:,:,:,nc3-1,jb)=facevarz(:,:,:,3,jb)
      facevarx(:,:,:,nc3,jb)=facevarx(:,:,:,4,jb)
      facevary(:,:,:,nc3,jb)=facevary(:,:,:,4,jb)
      facevarz(:,:,:,nc3,jb)=facevarz(:,:,:,4,jb)
#endif
#endif
#endif
!L       2.5       Aperiodic system (X1-X2-X3 array)
#ifdef bc3r_aperiodic
      do j2=nc2l,nc2r
        i2=nc2-j2+1
        do j1=nc1l,nc1r
          unk(:,:,j2,nc3-1,jb)=unk(:,:,i2,3,jb)
          unk(:,:,j2,nc3,jb)=unk(:,:,i2,4,jb)
#ifdef mhd
#ifdef staggered
          facevarx(:,:,j2,nc3-1,jb)=facevarx(:,:,i2,3,jb)
          facevary(:,:,j2,nc3-1,jb)=facevary(:,:,i2,3,jb)
          facevarz(:,:,j2,nc3-1,jb)=facevarz(:,:,i2,3,jb)
          facevarx(:,:,j2,nc3,jb)=facevarx(:,:,i2,4,jb)
          facevary(:,:,j2,nc3,jb)=facevary(:,:,i2,4,jb)
          facevarz(:,:,j2,nc3,jb)=facevarz(:,:,i2,4,jb)
#endif
#endif
        enddo
      enddo
#endif
#ifdef NGRID
    endif
#endif
  enddo
!---------------------------------------------------------------------
#endif
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5003,0) 
#endif        
RETURN
END SUBROUTINE BND3
