!****f* e0pro/main
! NAME 
!   BND3NG
! PURPOSE
!   Transfer boundary values between blocks
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BND3NG 
!---------------------------------------------------------------------
  USE comoly 
  USE comdim 
  USE comphy
  USE comreg
  USE comglo
  USE combnd
  USE comadm
  IMPLICIT NONE
#ifdef NGRID                                                              
  INTEGER :: i1, ib, j1, jb
#ifdef x2dim
  INTEGER :: i2, j2
#endif
#ifdef x3dim
  INTEGER :: i3, j3
#endif
#endif
#ifdef DIAGNO
  call EXPERT(5007,1)
#endif
#ifdef NGRID
#ifdef x3dim
!---------------------------------------------------------------------
!L     1.        X3-Left boundary
  do jb=1,nblk
    ib=nbc3l(1,jb)
    if(ib.gt.0) then
!L       1.1       From equally-spaced grid
      if(nlevel(3,ib).eq.nlevel(3,jb)) then
        unk(:,:,:,nc3l-2,jb)=unk(:,:,:,nc3r-1,ib)
        unk(:,:,:,nc3l-1,jb)=unk(:,:,:,nc3r,ib)
      endif
!L       1.2       From rougher block
      if(nlevel(3,ib).lt.nlevel(3,jb)) then
        if(nlevel(1,ib).lt.nlevel(1,jb)) then
          if(nbc3l(2,jb).lt.0) then
            i1=nc1l-1
          else
            i1=nc1rr
          endif
          do j1=nc1l,nc1r,2
            i1=i1+1
#ifdef x2dim
            i2=nc225
            do j2=nc2l,nc2r,2
              i2=i2+1
              unk(:,j1  ,j2  ,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1+1,j2  ,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1  ,j2+1,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1+1,j2+1,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1  ,j2  ,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1+1,j2  ,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1  ,j2+1,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1+1,j2+1,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
            enddo
#else
            unk(:,j1  ,1,nc3l-2,jb)=unk(:,i1,1,nc325,ib)
            unk(:,j1+1,1,nc3l-2,jb)=unk(:,i1,1,nc325,ib)
            unk(:,j1  ,1,nc3l-1,jb)=unk(:,i1,1,nc325,ib)
            unk(:,j1+1,1,nc3l-1,jb)=unk(:,i1,1,nc325,ib)
#endif
          enddo
        else
!L       1.3       From rougher interface
          do j1=nc1l,nc1r
#ifdef x2dim
            i2=nc225
            do j2=nc2l,nc2r,2
              i2=i2+1
              unk(:,j1,j2  ,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1,j2+1,nc3l-2,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1,j2  ,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
              unk(:,j1,j2+1,nc3l-1,jb)=unk(:,i1,i2,nc325,ib)
            enddo
#else
            unk(:,j1,1,nc3l-2,jb)=unk(:,i1,1,nc325,ib)
            unk(:,j1,1,nc3l-1,jb)=unk(:,i1,1,nc325,ib)
#endif
          enddo
        endif
      endif
!L       1.4       Continue
    endif
  enddo
!---------------------------------------------------------------------
!L     2.        X3-Right boundary
  do jb=1,nblk
    ib=nbc3r(1,jb)
    if(ib.gt.0) then
!L       2.1       From equally-spaced grid
      if(nlevel(3,ib).eq.nlevel(3,jb)) then
        unk(:,:,:,nc3r+1,jb)=unk(:,:,:,nc3l  ,ib)
        unk(:,:,:,nc3r+2,jb)=unk(:,:,:,nc3l+1,ib)
      endif
!L       2.2       From rougher block
      if(nlevel(3,ib).lt.nlevel(3,jb)) then
        if(nlevel(1,ib).lt.nlevel(1,jb)) then
          if(nbc3r(2,jb).lt.0) then
            i1=nc1l-1
          else
            i1=nc1rr
          endif
          do j1=nc1l,nc1r,2
            i1=i1+1
#ifdef x2dim
            i2=nc225
            do j2=nc2l,nc2r,2
              i2=i2+1
              unk(:,j1  ,j2  ,nc3r+1,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1+1,j2  ,nc3r+1,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1  ,j2+1,nc3r+1,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1+1,j2+1,nc3r+1,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1  ,j2  ,nc3r+2,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1+1,j2  ,nc3r+2,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1  ,j2+1,nc3r+2,jb)=unk(:,i1,i2,nc375,ib)
              unk(:,j1+1,j2+1,nc3r+2,jb)=unk(:,i1,i2,nc375,ib)
            enddo
#else
            unk(:,j1  ,1,nc3r+1,jb)=unk(:,i1,1,nc375,ib)
            unk(:,j1+1,1,nc3r+1,jb)=unk(:,i1,1,nc375,ib)
            unk(:,j1  ,1,nc3r+2,jb)=unk(:,i1,1,nc375,ib)
            unk(:,j1+1,1,nc3r+2,jb)=unk(:,i1,1,nc375,ib)
#endif
          enddo
        else
!L       2.3       From rougher interface
          do j1=nc1l,nc1r
#ifdef x2dim
            i2=nc225
            do j2=nc2l,nc2r,2
              i2=i2+1
              unk(:,j1,j2  ,nc3r+1,jb)=unk(:,j1,i2,nc375,ib)
              unk(:,j1,j2+1,nc3r+1,jb)=unk(:,j1,i2,nc375,ib)
              unk(:,j1,j2  ,nc3r+2,jb)=unk(:,j1,i2,nc375,ib)
              unk(:,j1,j2+1,nc3r+2,jb)=unk(:,j1,i2,nc375,ib)
            enddo
#else
            unk(:,j1,1,nc3r+1,jb)=unk(:,j1,1,nc375,ib)
            unk(:,j1,1,nc3r+2,jb)=unk(:,j1,1,nc375,ib)
#endif
          enddo
        endif
      endif
!L       2.4       Continue
    endif
  enddo
!---------------------------------------------------------------------
#endif
#endif
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(5007,0)
#endif
RETURN
END SUBROUTINE BND3NG
