!****f* e0pro/main
! NAME 
!   BNDGRD
! PURPOSE
!   Azimuthal magnetic field at the inner boundary
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BNDGRD
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  IMPLICIT NONE 
  REAL(8) :: dh1
#ifdef x2dim
  REAL(8) :: dh2, zdh2
#endif
#ifdef x3dim
  REAL(8) :: dh3
#endif
  INTEGER :: j1, j2, j3
!---------------------------------------------------------------------
!L     1.        Grid spacing
  dh1=(x1r-x1l)/dble(ncc1)
#ifdef x2dim
  dh2=(x2r-x2l)/dble(ncc2)
#endif
#ifdef x3dim
  dh3=(x3r-x3l)/dble(ncc3)
#endif
!---------------------------------------------------------------------
!L     2.        Heliospheric cell interfaces
!L       2.1       X1-coordinate
  do j1=1,nhh1
    xhh1(j1)=x1l+dh1*dble(j1-1)
  enddo
!L       2.2       X2-coordinate
#ifdef x2dim
    do j2=1,nhh2
      xhh2(j2)=x2l+dh2*dble(j2-1)
    enddo
#else
    xhh2(1)=x2l
    xhh2(2)=x2r
#endif
!L       2.3       X3-coordinate
#ifdef x3dim
    do j3=1,nhh3
      xhh3(j3)=x3l+dh3*dble(j3-1)
    enddo
#else
    xhh3(1)=x3l
    xhh3(2)=x3r
#endif
!---------------------------------------------------------------------
!L     3.        Heliospheric cell centers
!L       3.1       X1-coordinate
  do j1=1,ncc1
    xcc1(j1)=0.5d0*(xhh1(j1+1)+xhh1(j1))
  enddo
!L       3.2       X2-coordinate
  do j2=1,ncc2
    xcc2(j2)=0.5d0*(xhh2(j2+1)+xhh2(j2))
  enddo
!L       3.3       X3-coordinate
  do j3=1,ncc3
    xcc3(j3)=0.5d0*(xhh3(j3+1)+xhh3(j3))
  enddo
!---------------------------------------------------------------------
!L     4.        Modify if non-uniform grid
#ifdef x2dim
  if(lx2grd.ne.'uniform') then
!L       4.1       Heliospheric non-uniform grid
    if(lx2grd.eq.'sin') then
      do j2=2,nhh2
        xhh2(j2)=xhh2(j2-1)+dh2/sin(xcc2(j2-1))
      enddo
    endif
!L       4.2       Heliospheric non-uniform grid
    if(lx2grd.eq.'sqrtsin') then
      do j2=2,nhh2
        xhh2(j2)=xhh2(j2-1)+dh2/dsqrt(sin(xcc2(j2-1)))
      enddo
    endif
!L       4.3       Re-scale
    zdh2=(x2r-x2l)/(xhh2(nhh2)-xhh2(1))
    do j2=2,ncc2
      xhh2(j2)=x2l+(xhh2(j2)-x2l)*zdh2
    enddo
    xhh2(1)=x2l
    xhh2(nhh2)=x2r
    do j2=1,ncc2
      xcc2(j2)=0.5d0*(xhh2(j2+1)+xhh2(j2))
    enddo
  endif
#endif
!---------------------------------------------------------------------
!L     5.        Boundary grid
!L       5.1       Cell interfaces
#ifdef bctim2c
    do j1=1,nbb1h
      x1bnh(j1)=x1l-dh1*dble(nbb1h-j1)
    enddo
#else
    x1bnh=x1l
#endif
  x2bnh=xhh2
  x3bnh=xhh3
!L       5.2       Cell centers
#ifdef bctim2c
    do j1=1,nbb1
      x1bn(j1)=0.5d0*(x1bnh(j1+1)+x1bnh(j1))
    enddo
#else
    x1bn=x1l
#endif
  x2bn=xcc2
  x3bn=xcc3
!---------------------------------------------------------------------
RETURN
END SUBROUTINE BNDGRD


!****f* interp/main
! NAME 
!   INTERP
! PURPOSE
!   Analytic boundary conditions for 1-D runs
! HISTORY
!   1999-07-31 DO NOAA/SEC Boulder
!*********************************************************************
SUBROUTINE INTERP(uold,x2old,x3old,n2old,n3old,unew,x2new,x3new, &
  n2new,n3new,zshift)
!---------------------------------------------------------------------
  IMPLICIT NONE
  REAL(8) :: uold, x2old, x3old, unew, x2new, x3new
  REAL(8) :: z2m, z2p, z3m, z3p
  REAL(8) :: zshift
  REAL(8) :: zarea, zmm, zmp, zpm, zpp
  INTEGER :: n2old, n2new, n3old, n3new
  dimension  uold(n2old,n3old), x2old(n2old), x3old(n3old)
  dimension  unew(n2new,n3new), x2new(n2new), x3new(n3new)
#ifdef x2dim
  REAL(8) :: x2m, x2p
  INTEGER :: n2m, n2p
  INTEGER :: i2m, i2p
  INTEGER :: j2
  dimension  x2m(n2new), x2p(n2new)
  dimension  n2m(n2new), n2p(n2new)
#endif
#ifdef x3dim
  REAL(8) :: x3m, x3p
  REAL(8) :: zz3
  INTEGER :: n3m, n3p
  INTEGER :: i3m, i3p
  INTEGER :: j3
  dimension  x3m(n3new), x3p(n3new)
  dimension  n3m(n3new), n3p(n3new)
#endif
!---------------------------------------------------------------------
!L     1.        Set geometry coefficients
!L       1.1       X2 coefficients
#ifdef x2dim
  do j2=1,n2new
    i2p=2
    do while((x2new(j2).gt.x2old(i2p)).and.(i2p.lt.n2old))
      i2p=i2p+1
    enddo
    i2m=i2p-1
    n2m(j2)=i2m
    n2p(j2)=i2p
    x2m(j2)=(x2new(j2)-x2old(i2m))
    x2p(j2)=(x2old(i2p)-x2new(j2))
  enddo
#endif
!L       1.2       X3 coefficients
#ifdef x3dim
  do j3=1,n3new
    i3p=2
    zz3=x3new(j3)+zshift
    if(zz3.gt.6.2831854d0) zz3=zz3-6.2831854d0
    do while((zz3.gt.x3old(i3p)).and.(i3p.lt.n3old))
      i3p=i3p+1
    enddo
    i3m=i3p-1
    n3m(j3)=i3m
    n3p(j3)=i3p
    x3m(j3)=(zz3-x3old(i3m))
    x3p(j3)=(x3old(i3p)-zz3)
  enddo
#endif
!---------------------------------------------------------------------
!L     2.        Interpolate boundary values
#ifdef x2dim
#ifdef x3dim
!L       2.1       X2-X3 surface
  do j3=1,n3new
    i3m=n3m(j3)
    i3p=n3p(j3)
    z3m=x3m(j3)
    z3p=x3p(j3)
    do j2=1,n2new
      i2m=n2m(j2)
      i2p=n2p(j2)
      z2m=x2m(j2)
      z2p=x2p(j2)
      zarea=(z2m+z2p)*(z3m+z3p)
      zmm=z2p*z3p/zarea
      zmp=z2p*z3m/zarea
      zpm=z2m*z3p/zarea
      zpp=z2m*z3m/zarea
      unew(j2,j3)=zmm*uold(i2m,i3m)+zmp*uold(i2m,i3p) &
                 +zpm*uold(i2p,i3m)+zpp*uold(i2p,i3p)
    enddo
  enddo
#else
!L       2.2       X2 line
  do j2=1,n2new
    i2m=n2m(j2)
    i2p=n2p(j2)
    z2m=x2m(j2)
    z2p=x2p(j2)
    zarea=z2m+z2p
    zmm=z2p/zarea
    zpp=z2m/zarea
    unew(j2,1)=zmm*uold(i2m,1)+zpp*uold(i2p,1)
  enddo
#endif
#else
#ifdef x3dim
!L       2.3       X3 line
  do j3=1,n3new
    i3m=n3m(j3)
    i3p=n3p(j3)
    z3m=x3m(j3)
    z3p=x3p(j3)
    zarea=z3m+z3p
    zmm=z3p/zarea
    zpp=z3m/zarea
    unew(1,j3)=zmm*uold(1,i3m)+zpp*uold(1,i3p)
  enddo
#else
!L       2.2       Point
  unew(1,1)=uold(1,1)
#endif
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE INTERP


!****f* e0pro/main
! NAME 
!   BCBPHI
! PURPOSE
!   Azimuthal magnetic field at the inner boundary
! INPUTS
!   kmode =0,1,2,3,4 for various approximations
! HISTORY
!   2005-03-12 DO NOAA/SEC
!*********************************************************************
SUBROUTINE BCBPHI(bfast,vfast,kmode)
!---------------------------------------------------------------------
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  IMPLICIT NONE
  REAL(8) :: bfast, vfast
  INTEGER :: kmode
#ifdef mhd
  REAL(8)  :: DR, DT, DP
  REAL(8), DIMENSION(7)  :: X1X
  REAL(8), DIMENSION(7,nbb2,nbb3)  :: B1X, B3X
  REAL(8), DIMENSION(7,nbb2,nbb3)  :: A1X, A2X, AA2X, A3X
  REAL(8) :: zrad, vsint, trot_day, trot_sec, vrot_radsec
  INTEGER :: j1, j2, j3
#ifdef bctim2c
  INTEGER :: i1
#endif
#endif
!---------------------------------------------------------------------
#ifdef DIAGNO
  call EXPERT(2005,1)
#endif
#ifdef mhd
!---------------------------------------------------------------------
!L     1.        Auxiliary values
!L       5.3       Solar rotation
  trot_day=27.2753d0
  trot_sec=trot_day*24d0*3600d0
  vrot_radsec=360d0*vux3/trot_sec
!---------------------------------------------------------------------
!L     1.        Direct values
  if(kmode.eq.0) then
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
#ifdef bctim2c
        do j1=1,nbb1
          b3bn(j1,j2,j3)=-b1bn(j1,j2,j3)*vsint/v1bn(j1,j2,j3)
        enddo
#else
        b3bn(j2,j3)=-b1bn(j2,j3)*vsint/v1bn(j2,j3)
#endif
      enddo
    enddo
  endif
!---------------------------------------------------------------------
!L     2.        Direct values
  if(kmode.eq.1) then
    dr=(x1r-x1l)/dble(ncc1)
    zrad=(x1l-0.5d0*dr)/(x1l+0.5d0*dr)
    zrad=zrad*zrad
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
#ifdef bctim2c
        do j1=1,nbb1
          b3bn(j1,j2,j3)=-b1bn(j1,j2,j3)*vsint/v1bn(j1,j2,j3)
        enddo
#else
        b3bn(j2,j3)=-b1bn(j2,j3)*vsint/v1bn(j2,j3)
#endif
      enddo
    enddo
  endif
!---------------------------------------------------------------------
!L     3.        Direct values
  if(kmode.eq.2) then
    do j2=1,nbb2
      vsint=x1l*vrot_radsec*sin(x2bn(j2))
      do j3=1,nbb3
#ifdef bctim2c
        do j1=1,nbb1
          b3bn(j1,j2,j3)=-bfast*vsint/vfast
          b1bn(j1,j2,j3)=-b3bn(j1,j2,j3)*v1bn(j1,j2,j3)/vsint
        enddo
#else
        b3bn(j2,j3)=-bfast*vsint/vfast
        b1bn(j2,j3)=-b3bn(j2,j3)*v1bn(j2,j3)/vsint
#endif
      enddo
    enddo
  endif
!---------------------------------------------------------------------
!L     4.        Direct values
#ifndef bctim2c
  if(kmode.eq.3) then
!L       4.1       Uncorrected magnetic field at surrounding cells
    dr=(x1r-x1l)/dble(ncc1)
    x1x(1)=x1l-3.5d0*dr
    x1x(2)=x1l-2.5d0*dr
    x1x(3)=x1l-1.5d0*dr
    x1x(4)=x1l-0.5d0*dr
    x1x(5)=x1l+0.5d0*dr
    x1x(6)=x1l+1.5d0*dr
    dt=(x2r-x2l)/dble(ncc2)
    dp=(x3r-x3l)/dble(ncc3)
    do j1=1,6
      zrad=(x1x(4)/x1x(j1))**2
      do j2=1,nbb2
        vsint=x1x(j1)*vrot_radsec*sin(x2bn(j2))
        do j3=1,nbb3
          b1x(j1,j2,j3)=b1bn(j2,j3)*zrad
          b3x(j1,j2,j3)=-b1x(j1,j2,j3)*vsint/v1bn(j2,j3)
          b3bn(j2,j3)=-b1bn(j2,j3)*vsint/v1bn(j2,j3)
        enddo
      enddo
    enddo
!L       4.2       Vector potential at surrounding cells
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=1,6
          a1x(j1,j2,j3)=0d0
          a2x(j1,j2,j3)=0d0
          aa2x(j1,j2,j3)=0d0
          a3x(j1,j2,j3)=0d0
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=2,5
          a2x(j1+1,j2,j3)=(x1x(j1-1)*a2x(j1-1,j2,j3)  &
            +dr*x1x(j1)*b3x(j1,j2,j3))/x1x(j1+1)
        enddo
      enddo
    enddo
    do j1=1,6
      do j2=1,nbb2
        do j3=2,nbb3-1
          aa2x(j1,j2,j3+1)=aa2x(j1,j2,j3-1)  &
            -dp*x1x(j1)*sin(x2bn(j2))*b1x(j1,j2,j3)
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=1,6
          a2x(j1,j2,j3)=a2x(j1,j2,j3)+aa2x(j1,j2,j3)
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=2,nbb2-1
        do j1=1,6
          a1x(j1,j2+1,j3)=a1x(j1,j2-1,j3)  &
            -dt*x1x(j1)*b3x(j1,j2,j3)
          a3x(j1,j2+1,j3)=(sin(x2bn(j2-1))*a3x(j1,j2-1,j3)  &
            +dt*x1x(j1)*sin(x2bn(j2))*b1x(j1,j2,j3))/sin(x2bn(j2+1))
        enddo
      enddo
    enddo
!L       4.3       Magnetic field at the inner boundary
    do j3=2,nbb3-1
      do j2=2,nbb2-1
        b1bn(j2,j3)=                                                      &
          +(0.5d0/(x1x(4)*sin(x2bn(j2))*dt))*                             &
          (sin(x2bn(j2+1))*a3x(4,j2+1,j3)-sin(x2bn(j2-1))*a3x(4,j2-1,j3)) &
          -(0.5d0/(x1x(4)*sin(x2bn(j2))*dp))*                             &
          (a2x(4,j2,j3+1)-a2x(4,j2,j3-1))
        b2bn(j2,j3)=                                            &
          +(0.5d0/(x1x(4)*sin(x2bn(j2))*dp))*                   &
          (a1x(4,j2,j3+1)-a1x(4,j2,j3-1))                       &
          -(0.5d0/(x1x(4)*dr))*                                 &
          (x1x(5)*a3x(5,j2,j3)-x1x(3)*a3x(3,j2,j3))
        b3bn(j2,j3)=                                            &
          +(0.5d0/(x1x(4)*dr))*                                 &
          (x1x(5)*a2x(5,j2,j3)-x1x(3)*a2x(3,j2,j3))             &
          -(0.5d0/(x1x(4)*dt))*                                 &
          (a1x(4,j2+1,j3)-a1x(4,j2-1,j3))
      enddo
    enddo
  endif
#endif
!---------------------------------------------------------------------
!L     5.        Direct values
#ifdef bctim2c
  if(kmode.eq.3) then
!L       4.1       Uncorrected magnetic field at surrounding cells
    dr=(x1r-x1l)/dble(ncc1)
    x1x(1)=x1l-4.5d0*dr
    x1x(2)=x1l-3.5d0*dr
    x1x(3)=x1l-2.5d0*dr
    x1x(4)=x1l-1.5d0*dr
    x1x(5)=x1l-0.5d0*dr
    x1x(6)=x1l+0.5d0*dr
    x1x(7)=x1l+1.5d0*dr
    dt=(x2r-x2l)/dble(ncc2)
    dp=(x3r-x3l)/dble(ncc3)
    do j1=1,7
      zrad=(x1x(5)/x1x(j1))**2
      do j2=1,nbb2
        vsint=x1x(j1)*vrot_radsec*sin(x2bn(j2))
        do j3=1,nbb3
          b1x(j1,j2,j3)=b1bn(nbb1,j2,j3)*zrad
          b3x(j1,j2,j3)=-b1x(j1,j2,j3)*vsint/v1bn(nbb1,j2,j3)
          b3bn(nbb1,j2,j3)=-b1bn(nbb1,j2,j3)*vsint/v1bn(nbb1,j2,j3)
        enddo
      enddo
    enddo
!L       4.2       Vector potential at surrounding cells
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=1,7
          a1x(j1,j2,j3)=0d0
          a2x(j1,j2,j3)=0d0
          aa2x(j1,j2,j3)=0d0
          a3x(j1,j2,j3)=0d0
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=2,6
          a2x(j1+1,j2,j3)=(x1x(j1-1)*a2x(j1-1,j2,j3)  &
            +dr*x1x(j1)*b3x(j1,j2,j3))/x1x(j1+1)
        enddo
      enddo
    enddo
    do j1=1,7
      do j2=1,nbb2
        do j3=2,nbb3-1
          aa2x(j1,j2,j3+1)=aa2x(j1,j2,j3-1)  &
            -dp*x1x(j1)*sin(x2bn(j2))*b1x(j1,j2,j3)
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=1,nbb2
        do j1=1,7
          a2x(j1,j2,j3)=a2x(j1,j2,j3)+aa2x(j1,j2,j3)
        enddo
      enddo
    enddo
    do j3=1,nbb3
      do j2=2,nbb2-1
        do j1=1,7
          a1x(j1,j2+1,j3)=a1x(j1,j2-1,j3)  &
            -dt*x1x(j1)*b3x(j1,j2,j3)
          a3x(j1,j2+1,j3)=(sin(x2bn(j2-1))*a3x(j1,j2-1,j3)  &
            +dt*x1x(j1)*sin(x2bn(j2))*b1x(j1,j2,j3))/sin(x2bn(j2+1))
        enddo
      enddo
    enddo
!L       4.3       Magnetic field at the inner boundary
    do j3=2,nbb3-1
      do j2=2,nbb2-1
        do j1=1,nbb1
          i1=j1+3
          b1bn(j1,j2,j3)=                                                     &
            +(0.5d0/(x1x(i1)*sin(x2bn(j2))*dt))*                              &
            (sin(x2bn(j2+1))*a3x(i1,j2+1,j3)-sin(x2bn(j2-1))*a3x(i1,j2-1,j3)) &
            -(0.5d0/(x1x(i1)*sin(x2bn(j2))*dp))*                              &
            (a2x(i1,j2,j3+1)-a2x(i1,j2,j3-1))
          b2bn(j1,j2,j3)=                                          &
            +(0.5d0/(x1x(i1)*sin(x2bn(j2))*dp))*                   &
            (a1x(i1,j2,j3+1)-a1x(i1,j2,j3-1))                      &
            -(0.5d0/(x1x(i1)*dr))*                                 &
            (x1x(i1+1)*a3x(i1+1,j2,j3)-x1x(i1-1)*a3x(i1-1,j2,j3))
          b3bn(j1,j2,j3)=                                          &
            +(0.5d0/(x1x(i1)*dr))*                                 &
            (x1x(i1+1)*a2x(i1+1,j2,j3)-x1x(i1-1)*a2x(i1-1,j2,j3))  &
            -(0.5d0/(x1x(i1)*dt))*                                 &
            (a1x(i1,j2+1,j3)-a1x(i1,j2-1,j3))
        enddo
      enddo
    enddo
  endif
#endif
!---------------------------------------------------------------------
#endif
!L     9.        Return
#ifdef DIAGNO
  call EXPERT(2005,0)
#endif
!---------------------------------------------------------------------
RETURN
END SUBROUTINE BCBPHI
