!****f* mas2bc/main
! NAME 
!   MAS2BC
! PURPOSE
!   Boundary conditions from MAS results
! HISTORY
!   1999-07-31 DO NOAA/SEC Boulder
!*********************************************************************
PROGRAM MAS2BC
!=====================================================================
  USE comoly
  USE comdim
  USE comphy
  USE comreg
  USE combnd
  IMPLICIT NONE 
!---------------------------------------------------------------------
  CHARACTER :: file_b*80, file_v*80
  CHARACTER :: ldata*16, lstart*10, lcrnum*4
!---------------------------------------------------------------------
  INTEGER :: id
  INTEGER :: j, j2, j3, jrec
  INTEGER :: nbrad, nptot
#ifdef bctim2c
  INTEGER :: j1
#endif
!---------------------------------------------------------------------
  REAL(8) :: mjd_cr
  REAL(8) :: dfast, tfast, vfast, bfast, dvexp, shift
  REAL(8) :: zshift, zptot, zpmag
  REAL(8) :: vud, vut, vuv, vub, trot_day, trot_sec
  INTEGER :: nrec
  INTEGER :: idin, idir, idata
!---------------------------------------------------------------------
  INTEGER, PARAMETER :: ntheta_b=70, nphi_b=65, ntheta_v=91, nphi_v=181
  REAL(8), DIMENSION(ntheta_b) :: theta_b
  REAL(8), DIMENSION(ntheta_v) :: theta_v
  REAL(8), DIMENSION(nphi_b) :: phi_b
  REAL(8), DIMENSION(nphi_v) :: phi_v
  REAL(8), DIMENSION(ntheta_b,nphi_b) :: b
  REAL(8), DIMENSION(ntheta_v,nphi_v) :: v
!---------------------------------------------------------------------
  NAMELIST/NAMDAT/                             &
    ldir,  lproj,  lbnd,   lini,   lgrd,       &
    ldata, lcrnum, lstart, lx2grd,             &
    dfast, tfast,  vfast,  bfast,              &
    dvexp, shift,  xalpha, nbrad,  nptot,      &
    x1l,   x1r,    x2l,    x2r,    x3l,   x3r
!---------------------------------------------------------------------
!L     1.        Clear variables and set parameters
!L       1.1       Clear labels
  do j=1,120
    lblank(j:j)=' '
  enddo
  lalph='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
  lnum='0123456789'
  ldir=lblank(1:80)
  lajob=lblank(1:80)
  lgrd=lblank(1:12)
  lbnd=lblank(1:8)
  ldata=lblank(1:16)
  lcrnum=lblank(1:4)
  lstart=lblank(1:10)
!L       1.2       Default computational region
  x1l=0.14d0
  x1r=1.07d0
  x2l=30d0
  x2r=150d0
  x3l=0d0
  x3r=360d0
  lx2grd='uniform'
!L       1.3       Constants
  xalpha=0d0
!L       1.4       Fast stream paramaters
  dfast=150d0
  tfast=0.6d0
  vfast=650d0
  bfast=170d0
  dvexp=2d0
!L       1.5       Azimuthal shift on source surface (days)
  shift=0d0
!L       1.6       Specification of derived variables
  nbrad=2
  nptot=0
!L       1.7       Units
  vux1=1.496d11
  vux2=0.01745329252d0
  vux3=0.01745329252d0
  vutime=3600d0
  vud=1d6
  vuv=1d3
  vut=1d6
  vub=1d-9
!---------------------------------------------------------------------
!L     2.        Read in the JOB input parameters
!L       2.1       Read in heading
  read(*,'(A)',end=901,err=902) lajob
  write(*,'(a)') '=== MAS2BC: '//lajob
!L       2.2       Read in parameters
  read(*,NAMDAT,end=901,err=902)
!---------------------------------------------------------------------
!L     3.        Process the JOB input parameters
!L       3.3       Length of file names
  idir=len_trim(ldir)
  idata=len_trim(ldata)
!L       3.4       Reference date
  call DATE2BC(lcrnum,lstart,mjd_cr,refdate,lrefdate,lreftime)
!L       3.6       Input directory and file names
  file_b=ldir(1:idir)//'/'//ldata(1:idata)//'/br_helio_scaled.dat'
  file_v=ldir(1:idir)//'/'//ldata(1:idata)//'/v_smooth_0.05_0.025.dat'
!L       3.7       Output directory and file names
  lfgrd='grd.nc'
  mfgrd=6
  lfbnd='bnd.nc'
  mfbnd=6
!---------------------------------------------------------------------
!L     4.        Computational region
!L       4.1       Region boundaries
  x1l=x1l*vux1
  x1r=x1r*vux1
  x2l=x2l*vux2
  x2r=x2r*vux2
  x3l=x3l*vux3
  x3r=x3r*vux3
!L       4.2       Set numerical grid
  call BNDGRD
!L       4.3       Write numerical grid as the NetCDF file
  call OUTGRD
!---------------------------------------------------------------------
!L     5.        Physical parameters
!L       5.1       Constants
  rgas=((2d0+3d0*xalpha)/(1d0+4d0*xalpha))*(ck/cmass)
  xmass=(1d0+4d0*xalpha)*cmass
!L       5.2       Convert to metric SI units
  dfast=xmass*dfast*vud
  tfast=tfast*vut
  vfast=vfast*vuv
  bfast=bfast*vub
!L       5.3       Solar rotation
  trot_day=27.2753d0
  trot_sec=trot_day*24d0*3600d0
!---------------------------------------------------------------------
!L     6.        Read in SAIC source-surface data
!L       6.1       Radial magnetic field
  write(*,'(a)') '--- input boundary file: '//file_b
  open(1,file=file_b,status='old')
  read(1,*) id,j,j,j,theta_b,phi_b,b
  close(1)
!L       6.2       Radial velocity
  write(*,'(a)') '--- input boundary file: '//file_v
  open(1,file=file_v,status='old')
  read(1,*) id,j,j,j,theta_v,phi_v,v
  close(1)
!L       6.3       Convert to metric SI units
  b=b*2.205d-4
  v=v*481.37d3 
!L       6.4       Shift to account solar rotation
  if((shift.gt.0d0).and.(trot_day.gt.0d0)) then
    zshift=2d0*3.1415927d0*shift/trot_day
  else
    zshift=0d0
  endif
!L       6.5       Interpolate values onto new grid
  call INTERP(b,theta_b,phi_b,ntheta_b,nphi_b,b1bn,x2bn,x3bn, &
    nbb2,nbb3,zshift)
  call INTERP(v,theta_v,phi_v,ntheta_v,nphi_v,v1bn,x2bn,x3bn, &
    nbb2,nbb3,zshift)
!---------------------------------------------------------------------
!L     7.        Determine the background boundary conditions
!L       7.1       Set the density
  do j3=1,nbb3
    do j2=1,nbb2
#ifdef bctim2c
      do j1=1,nbb1
        dbn(j1,j2,j3)=dfast*(vfast/v1bn(j1,j2,j3))**dvexp
      enddo
#else
      dbn(j2,j3)=dfast*(vfast/v1bn(j2,j3))**dvexp
#endif
    enddo
  enddo
!L       7.2       Set the temperature
  if(nptot.eq.0) then
    do j3=1,nbb3
      do j2=1,nbb2
#ifdef bctim2c
        do j1=1,nbb1
          tbn(j1,j2,j3)=dfast*tfast/dbn(j1,j2,j3)
        enddo
#else
        tbn(j2,j3)=dfast*tfast/dbn(j2,j3)
#endif
      enddo
    enddo
  else
#ifdef bctim2c
    zptot=rgas*dfast*tfast+b1bn(1,1,1)*b1bn(1,1,1)/(2d0*cmu)
#else
    zptot=rgas*dfast*tfast+b1bn(1,1)*b1bn(1,1)/(2d0*cmu)
#endif
    do j3=1,nbb3
      do j2=1,nbb2
#ifdef bctim2c
        do j1=1,nbb1
          zpmag=b1bn(j1,j2,j3)*b1bn(j1,j2,j3)/(2d0*cmu)
          tbn(j1,j2,j3)=(zptot-zpmag)/(rgas*dbn(j1,j2,j3))
        enddo
#else
        zpmag=b1bn(j2,j3)*b1bn(j2,j3)/(2d0*cmu)
        tbn(j2,j3)=(zptot-zpmag)/(rgas*dbn(j2,j3))
#endif
      enddo
    enddo
  endif
!L       7.3       Set the polarity tracer
#ifdef mhdcon
  do j3=1,nbb3
    do j2=1,nbb2
#ifdef bctim2c
      do j1=1,nbb1
        if(b1bn(j1,j2,j3).lt.0d0) then
          b1bn(j1,j2,j3)=-b1bn(j1,j2,j3)
          bpbn(j1,j2,j3)=-1d0
        else
          bpbn(j1,j2,j3)=1d0
        endif
      enddo
#else
      if(b1bn(j2,j3).lt.0d0) then
        b1bn(j2,j3)=-b1bn(j2,j3)
        bpbn(j2,j3)=-1d0
      else
        bpbn(j2,j3)=1d0
      endif
#endif
    enddo
  enddo
#endif
!L       7.4       Set the azimuthal magnetic field component
  call BCBPHI(bfast,vfast,nbrad)
!---------------------------------------------------------------------
!L     8.        Write boundary values as the NetCDF file
  write(*,'(a)') '--- heliospheric boundary file: '//lfbnd(1:mfbnd)
  nrec=6
  do jrec=1,nrec
!L       8.1       Time of the Carrington rotation maps
    if(jrec.eq.1) time=-2.5d0*trot_sec
    if(jrec.eq.2) time=-1.5d0*trot_sec
    if(jrec.eq.3) time=-0.5d0*trot_sec
    if(jrec.eq.4) time=0.5d0*trot_sec
    if(jrec.eq.5) time=1.5d0*trot_sec
    if(jrec.eq.6) time=2.5d0*trot_sec
!L       8.2       Time with respect to the reference date
    time=time-(refdate-mjd_cr)*86400d0
!L       8.3       Write NetDCF data
    call OUTBND(jrec)
  enddo
!---------------------------------------------------------------------
!L     9.        Stop
  write(*,'(a)') '=== END OF MAS2BC'
STOP
  901 continue
      write (*,'(a)') '*** End of input file'
      goto 903
  902 continue
      write (*,'(a)') '*** Error in input file'
  903 continue
      write (*,'(a)') '*** Error in program MAS2BC'
STOP
!---------------------------------------------------------------------
END PROGRAM MAS2BC
