!****f* out/outbnd
! NAME 
!   OUTBND
! PURPOSE
!   Output of boundary conditions
! HISTORY
!   1999-07-31 DO NOAA/SEC Boulder
!*********************************************************************
SUBROUTINE OUTBND(kstep)
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comreg
  USE comphy
  USE combnd
  USE commet
  USE comout
  USE netcdf
  IMPLICIT NONE
  INTEGER :: kstep
!---------------------------------------------------------------------
#ifdef bctim2c
  INTEGER, DIMENSION(2) :: idims1, istart1, icount1, &
                           idims4, istart4, icount4
  DATA  idims1/1,7/, istart1/1,1/, icount1/nbb1,1/
  DATA  idims4/4,7/, istart4/1,1/, icount4/nbb1h,1/
#else
  INTEGER :: idims1
  INTEGER :: idims4
  DATA  idims1/7/
  DATA  idims4/7/
#endif
  INTEGER, DIMENSION(2) :: idims2, istart2, icount2, &
                           idims3, istart3, icount3, &
                           idims5, istart5, icount5, &
                           idims6, istart6, icount6
  DATA  idims2/2,7/, istart2/1,1/, icount2/ncc2,1/
  DATA  idims3/3,7/, istart3/1,1/, icount3/ncc3,1/
  DATA  idims5/5,7/, istart5/1,1/, icount5/nhh2,1/
  DATA  idims6/6,7/, istart6/1,1/, icount6/nhh3,1/
!---------------------------------------------------------------------
#ifdef bctim2c
  INTEGER, DIMENSION(4) ::  idims, istart, icount
  DATA  idims/1,2,3,7/, istart/1,1,1,1/
  DATA  icount/nbb1,ncc2,ncc3,1/
#else
  INTEGER, DIMENSION(3) ::  idims, istart, icount
  DATA  idims/2,3,7/, istart/1,1,1/
  DATA  icount/ncc2,ncc3,1/
#endif
!---------------------------------------------------------------------
  INTEGER :: id, iret, ifile
!---------------------------------------------------------------------
!L     1.        Define the output file
  if(kstep.eq.1) then
!L       1.1       Create the new file
    iret=NF90_CREATE(lfbnd,nf90_clobber,ifile)
!L       1.2       Set global attributes
    iret=NF90_PUT_ATT(ifile,nf90_global,'type','BND')
    iret=NF90_PUT_ATT(ifile,nf90_global,'title','Boundary values')
#include <attribut.src>
!L       1.3       Set array dimensions
#ifdef bctim2c
      iret=NF90_DEF_DIM(ifile,'n1',nbb1,id)
#else
      iret=NF90_DEF_DIM(ifile,'n1',1,id)
#endif
    iret=NF90_DEF_DIM(ifile,'n2',ncc2,id)
    iret=NF90_DEF_DIM(ifile,'n3',ncc3,id)
#ifdef bctim2c
      iret=NF90_DEF_DIM(ifile,'n1h',nbb1h,id)
#else
      iret=NF90_DEF_DIM(ifile,'n1h',1,id)
#endif
    iret=NF90_DEF_DIM(ifile,'n2h',nhh2,id)
    iret=NF90_DEF_DIM(ifile,'n3h',nhh3,id)
    iret=NF90_DEF_DIM(ifile,'ntime',nf90_unlimited,id)
!L       1.4       Mesh variables
    iret=NF90_DEF_VAR(ifile,'X1',nf90_double,idims1,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-cell positions')
    iret=NF90_DEF_VAR(ifile,'X2',nf90_double,idims2,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-cell positions')
    iret=NF90_DEF_VAR(ifile,'X3',nf90_double,idims3,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-cell positions')
    iret=NF90_DEF_VAR(ifile,'X1H',nf90_double,idims4,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-interface positions')
    iret=NF90_DEF_VAR(ifile,'X2H',nf90_double,idims5,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-interface positions')
    iret=NF90_DEF_VAR(ifile,'X3H',nf90_double,idims6,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-interface positions')
!L       1.5       Time
    iret=NF90_DEF_VAR(ifile,'TIME',nf90_double,7,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
    iret=NF90_DEF_VAR(ifile,'DT',nf90_double,7,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time step')
    iret=NF90_DEF_VAR(ifile,'NSTEP',nf90_int,7,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Numerical step')
!L       1.6       Magnetohydrodynamic variables
    iret=NF90_DEF_VAR(ifile,'D',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Density')
    iret=NF90_DEF_VAR(ifile,'T',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Temperature')
    iret=NF90_DEF_VAR(ifile,'V1',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-velocity')
    iret=NF90_DEF_VAR(ifile,'V2',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-velocity')
    iret=NF90_DEF_VAR(ifile,'V3',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-velocity')
    iret=NF90_DEF_VAR(ifile,'B1',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X1-magnetic field')
    iret=NF90_DEF_VAR(ifile,'B2',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X2-magnetic field')
    iret=NF90_DEF_VAR(ifile,'B3',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','X3-magnetic field')
#ifdef hydcon
    iret=NF90_DEF_VAR(ifile,'DP',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Plasma cloud')
#endif
#ifdef mhdcon
    iret=NF90_DEF_VAR(ifile,'BP',nf90_double,idims,id)
    iret=NF90_PUT_ATT(ifile,id,'long_name','Polarity of magnetic field')
#endif
!L       1.7       Leave define mode
    iret=NF90_ENDDEF(ifile)
!L       1.8       Close the output file
    iret=NF90_CLOSE(ifile)
  endif
!---------------------------------------------------------------------
!L     2.        Output data
!L       2.1       Open the output file
    iret=NF90_OPEN(lfbnd,nf90_write,ifile)
    write(*,*) '   record',kstep,'   day =',time/86400d0
!L       2.2       Mesh variables
    istart2(2)=kstep
    istart3(2)=kstep
#ifdef bctim2c
      istart1(2)=kstep
      iret=NF90_PUT_VAR(ifile,1,x1bn,start=istart1,count=icount1)
#else
      iret=NF90_PUT_VAR(ifile,1,x1bn,start=(/kstep/))
#endif
    iret=NF90_PUT_VAR(ifile,2,x2bn,start=istart2,count=icount2)
    iret=NF90_PUT_VAR(ifile,3,x3bn,start=istart3,count=icount3)
    istart5(2)=kstep
    istart6(2)=kstep
#ifdef bctim2c
      istart4(2)=kstep
      iret=NF90_PUT_VAR(ifile,4,x1bnh,start=istart4,count=icount4)
#else
      iret=NF90_PUT_VAR(ifile,4,x1bnh,start=(/kstep/))
#endif
    iret=NF90_PUT_VAR(ifile,5,x2bnh,start=istart5,count=icount5)
    iret=NF90_PUT_VAR(ifile,6,x3bnh,start=istart6,count=icount6)
!L       2.3       Magnetohydrodynamic variables
#ifdef bctim2c
    istart(4)=kstep
#else
    istart(3)=kstep
#endif
    iret=NF90_PUT_VAR(ifile,7,time,start=(/kstep/))
    iret=NF90_PUT_VAR(ifile,8,0d0,start=(/kstep/))
    iret=NF90_PUT_VAR(ifile,9,kstep,start=(/kstep/))
    iret=NF90_PUT_VAR(ifile,10,dbn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,11,tbn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,12,v1bn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,13,v2bn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,14,v3bn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,15,b1bn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,16,b2bn,start=istart,count=icount)
    iret=NF90_PUT_VAR(ifile,17,b3bn,start=istart,count=icount)
#ifdef hydcon
#ifdef mhdcon
        iret=NF90_PUT_VAR(ifile,18,dpbn,start=istart,count=icount)
        iret=NF90_PUT_VAR(ifile,19,bpbn,start=istart,count=icount)
#else
        iret=NF90_PUT_VAR(ifile,18,dpbn,start=istart,count=icount)
#endif
#else
#ifdef mhdcon
        iret=NF90_PUT_VAR(ifile,18,bpbn,start=istart,count=icount)
#endif
#endif
!L                  2.4      Close the output file
    iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
RETURN
END SUBROUTINE OUTBND


!****f* out/outgrd
! NAME 
!   OUTGRD
! PURPOSE
!   Output of computational grid
! HISTORY
!   1999-07-31 DO NOAA/SEC Boulder
!*********************************************************************
SUBROUTINE OUTGRD
!---------------------------------------------------------------------
  USE comoly
  USE compro
  USE comdim
  USE comphy
  USE comreg
  USE commet
  USE comout
  USE netcdf
!---------------------------------------------------------------------
  IMPLICIT NONE
  INTEGER :: iret,ifile,id
  INTEGER, DIMENSION(2) :: idims1, idims2, idims3, &
                           idims4, idims5, idims6
  DATA  idims1 /1,7/
  DATA  idims2 /2,7/
  DATA  idims3 /3,7/
  DATA  idims4 /4,7/
  DATA  idims5 /5,7/
  DATA  idims6 /6,7/
!---------------------------------------------------------------------
!L     1.        Create or open the file
  nblk=1
!L       1.1       Create the new file
  iret=NF90_CREATE(lfgrd,nf90_clobber,ifile)
  write(*,'(a)') '--- heliospheric grid file: '//lfgrd(1:mfgrd)
!---------------------------------------------------------------------
!L     2.        Set global attributes
  iret=NF90_PUT_ATT(ifile,nf90_global,'type','GRD')
  iret=NF90_PUT_ATT(ifile,nf90_global,'title','Numerical Grid')
#include <attribut.src>
!---------------------------------------------------------------------
!L     3.        Set array dimensions
  iret=NF90_DEF_DIM(ifile,'n1',ncc1,id)
  iret=NF90_DEF_DIM(ifile,'n2',ncc2,id)
  iret=NF90_DEF_DIM(ifile,'n3',ncc3,id)
  iret=NF90_DEF_DIM(ifile,'n1h',nhh1,id)
  iret=NF90_DEF_DIM(ifile,'n2h',nhh2,id)
  iret=NF90_DEF_DIM(ifile,'n3h',nhh3,id)
  iret=NF90_DEF_DIM(ifile,'nblk',nblk,id)
!---------------------------------------------------------------------
!L     4.        Define values
!L       4.1       Mesh structure
  iret=NF90_DEF_VAR(ifile,'X1',nf90_double,idims1,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X1-cell positions')
  iret=NF90_PUT_ATT(ifile,id,'units','m')
  iret=NF90_DEF_VAR(ifile,'X2',nf90_double,idims2,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X2-cell positions')
  iret=NF90_PUT_ATT(ifile,id,'units','radian')
  iret=NF90_DEF_VAR(ifile,'X3',nf90_double,idims3,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X3-cell positions')
  iret=NF90_PUT_ATT(ifile,id,'units','radian')
!L       4.2       Mesh structure
  iret=NF90_DEF_VAR(ifile,'X1H',nf90_double,idims4,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X1-interface positions')
  iret=NF90_PUT_ATT(ifile,id,'units','m')
  iret=NF90_DEF_VAR(ifile,'X2H',nf90_double,idims5,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X2-interface positions')
  iret=NF90_PUT_ATT(ifile,id,'units','radian')
  iret=NF90_DEF_VAR(ifile,'X3H',nf90_double,idims6,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','X3-interface positions')
  iret=NF90_PUT_ATT(ifile,id,'units','radian')
!L       4.3       Time
  iret=NF90_DEF_VAR(ifile,'TIME',nf90_double,0,id)
  iret=NF90_PUT_ATT(ifile,id,'long_name','Physical time')
  iret=NF90_PUT_ATT(ifile,id,'units','s')
!L       4.4       Leave define mode
  iret=NF90_ENDDEF (ifile)
!---------------------------------------------------------------------
!L     5.        Write values
!L       5.1       Mesh structure
  iret=NF90_PUT_VAR(ifile,1,xcc1)
  iret=NF90_PUT_VAR(ifile,2,xcc2)
  iret=NF90_PUT_VAR(ifile,3,xcc3)
!L       5.2       Mesh structure
  iret=NF90_PUT_VAR(ifile,4,xhh1)
  iret=NF90_PUT_VAR(ifile,5,xhh2)
  iret=NF90_PUT_VAR(ifile,6,xhh3)
!L       5.3       Time
  iret=NF90_PUT_VAR(ifile,7,time)
!---------------------------------------------------------------------
!L     6.        Close the file
  iret=NF90_CLOSE(ifile)
!---------------------------------------------------------------------
RETURN 
END SUBROUTINE OUTGRD
