c
c     ##################################################################
c     #                                                                #
c     #      MAS: Magnetohydrodynamic Algorithm outside a Sphere       #
c     #                                                                #
c     #      Authors: Z. Mikic (SAIC, San Diego)                       #
c     #               J. Linker (SAIC, San Diego)                      #
c     #                                                                #
c     #          A code for the study of resistive MHD evolution of    #
c     #      solar coronal magnetic fields in spherical geometry.      #
c     #                                                                #
c     ##################################################################
c
c-----------------------------------------------------------------------
c
c ****** Parameters and common blocks.
c
c-----------------------------------------------------------------------
c
      cliche ident
c
c-----------------------------------------------------------------------
c
c ****** Code name.
c
      character idcode*3,vers*(*),update*(*),source*(*)
      parameter (idcode='MAS')
      parameter (vers='0.66d_ccmc_v01')
      parameter (update='04/30/2003')
      parameter (source='mas47d_ccmc_v01.F')
c
c ****** Machine name and type.
c
      character*(*) machname,machtype
      parameter (machname='<MACHNAME>')
      parameter (machtype='<MACHTYPE>')
c
c ****** Run time and date.
c
      character rtime*20,rdate*20
      common /idcom1/ rtime,rdate
c
c ****** Run ID.
c
      character*16 runid
      common /idcom2/ runid
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche globals
c
c-----------------------------------------------------------------------
c
      parameter (nr=<NR>,nt=<NT>,mp=<MP>)
      parameter (nrm1=nr-1,nrm2=nr-2,nrm3=nr-3)
      parameter (ntm1=nt-1,ntm2=nt-2,ntm3=nt-3)
      parameter (np=2**mp,npd3=np/2+1,npp1=np+1)
      parameter (nrt=nr*nt,nrtp=nr*nt*np)
c
c ****** NC is the number of words needed to hold the real
c ****** or imaginary part of a complex field.
c
      parameter (nc=nr*nt*npd3)
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche comm
c
c-----------------------------------------------------------------------
c
      use globals
c
c ****** Constants.
c
      parameter (pi=3.14159265358979)
c
c ****** Debug switch.
c
      common /idbug/ idebug
c
c ****** File names.
c
      character*72 infile,outfile,rsofile,rsifile
      common /fncom/ infile,outfile,rsofile,rsifile
c
c ****** Options.
c
      character*16 option,eqtype,fldtype
      common /optcom/ option,eqtype,fldtype
c
c ****** Time step related variables.
c
      common /timevars/ ntime,time,ntmax,dtime,dtmin,dtmax,tmax,
     & dtmult,strmax,isitype,cfl,omegaeta,dtimeav
c
c ****** Mesh related variables.
c
      parameter (nmseg=10)
      common /meshvars/ r0,r1,rl,t0,t1,tl,phi0,phi1,phil,
     & r(nrm1),rh(nr),dr(nrm1),drh(nr),dri(nrm1),
     & t(ntm1),th(nt),dt(ntm1),dth(nt),dti(ntm1),
     & st(ntm1),sth(nt),ct(ntm1),cth(nt),
     & phi(npp1),fm(npd3),fm2(npd3),dphi,
     & sp(np),cp(np),
     & drratio(nmseg),rfrac(nmseg),
     & dtratio(nmseg),tfrac(nmseg),
     & nfrmesh,nftmesh
c
c ******* Normalization variables.
c
      common /norm1/ fnorml,fnormt,fnormm,
     &               fn_n,fn_rho,fn_p,fn_t,fn_v,fn_b,
     &               fn_qrad,fn_kappa,fn_rd1,fn_rd2,fn_q0
c
c ******* Helium fraction.
c
      common /hefrac1/ he_frac,he_rho,he_p,he_np
c
c ****** Physical variables.
c
      real neradmin
      logical predflow, si_aggressive
      common /physvars/ g0,ifvdgv,gamma,ifrho,iftemp,ifpc,ifohmic,
     &                  ifhvisc,isetrho,omegarho,ifeqcs,ifradbc,
     &                  fkspitz,ifimplrad,
     &                  radloss,tcond,tbc0,tbc1,tne0,
     &                  tc_r,tc_dr,tc_fac0,tc_fac1,
     &                  alpha_nocoll,tc_nocoll_r,tc_nocoll_dr,
     &                  neradmin,predflow,betapc_p, betapc_v,
     &                  si_aggressive,fac_cflv,betapc_si,ifbheat,
     &                  t0_tc,alpha_tc,dt0_tc,nfilt_kappa,
     &                  gammar(nrm1),gammarh(nr)
c
c ****** Viscosity and resistivity profiles.
c
      parameter (mxnodes=20)
      logical recell_unift
      common /viscom1/ ivistype,visc,recell,
     &                 recell_facr,recell_fact,recell_facp,
     &                 recell_viscmax,recell_unift,
     &                 nrvisp,rvisp(mxnodes),vispr(mxnodes),
     &                 ntvisp,tvisp(mxnodes),vispt(mxnodes),
     &                 visfac(nrm1,ntm1),viscel(nrm1,ntm1)
c
      logical rmcell_unift
      common /etacom1/ ifideal,ietatype,slund,rmcell,
     &                 rmcell_facr,rmcell_fact,rmcell_facp,
     &                 rmcell_etamax,rmcell_unift,
     &                 nretap,retap(mxnodes),etapr(mxnodes),
     &                 ntetap,tetap(mxnodes),etapt(mxnodes),
     &                 etafac(nrm1,ntm1),etacel(nrm1,ntm1)
c
      common /etacom2/ ifeta_dyn,eta_min,eta_max,bsqfloor,fjfrac,
     &                 eta_jmin,eta_dj,eta_jslope
c
c ****** Gamma profile
c
      common /gammaprof/ gamma1, gamma2, drgamma, rgamma
c
c ****** Collisionless heat flux radial profile.
c
      common /nocoll1/ prof_nocoll(nr)
c
c ****** Drive profile.
c
      parameter (mxtnodes=100)
      character*72 bheatfile
      common /drvcom/ tnode(mxtnodes),vnode(mxtnodes),
     &                tnode_ch(mxtnodes),q0_eq_ch(mxtnodes),
     &                q0_pole_ch(mxtnodes),hlen_eq_ch(mxtnodes),
     &                hlen_pole_ch(mxtnodes),q0phys_ch(mxtnodes),
     &                qalfven_ch(mxtnodes),
     &                dthmax,th0,ishearprof,arotate,brotate,crotate,
     &                thprof_th,thprof_dth,thprof(nt),
     &                awthprof_th,awthprof_dth,awthprof(nt),
     &                awthprof2_th,brbyb2(nt,np),bheatfile
c
c ****** Characteristic boundary conditions.
c
      logical parchar,ubzero
      common /charbcs/ gbc0err,parchar,ubzero,nfiltub,nfiltne0
c
c ****** Numerical factors.
c
      logical fdphi
      logical split_visc
      common /numfacts/ simult,upwinda,upwindv,pdma,
     &                  upwindar,upwindat,upwindap,upwindphi,
     &                  fdphi,split_visc
c
c ****** Primary fields.
c
      common /fields/ arr(nr,nt,npd3),ari(nr,nt,npd3),
     &                atr(nr,nt,npd3),ati(nr,nt,npd3),
     &                apr(nr,nt,npd3),api(nr,nt,npd3),
     &                vrr(nr,nt,npd3),vri(nr,nt,npd3),
     &                vtr(nr,nt,npd3),vti(nr,nt,npd3),
     &                vpr(nr,nt,npd3),vpi(nr,nt,npd3),
     &                prr(nr,nt,npd3),pri(nr,nt,npd3),
     &                rhr(nr,nt,npd3),rhi(nr,nt,npd3),
     &                tr(nr,nt,npd3),ti(nr,nt,npd3),
     &                epr(nr,nt,npd3),epi(nr,nt,npd3),
     &                emr(nr,nt,npd3),emi(nr,nt,npd3),
     &                br(nr,nt,np),bt(nr,nt,np),bp(nr,nt,np),
     &                fjr(nr,nt,np),fjt(nr,nt,np),fjp(nr,nt,np),
     &                vr(nr,nt,np),vt(nr,nt,np),vp(nr,nt,np),
     &                rho(nr,nt,np),p(nr,nt,np),temp(nr,nt,np),
     &                ep(nr,nt,np),em(nr,nt,np)
c
c ****** Radial field (artificial).
c
      common /bias/ br00
c
c ****** Boundary conditions.
c
      character*16 bcr0type,bcr1type
      common /bctyp/ bcr0type,bcr1type
c
      common /bcs0/ vrbcr0 (nt,npd3,2),
     &              vrbcr1 (nt,npd3,2),
     &              vtbcr0 (nt,npd3,2),
     &              vtbcr1 (nt,npd3,2),
     &              vpbcr0 (nt,npd3,2),
     &              vpbcr1 (nt,npd3,2),
     &              rhobcr0(nt,npd3,2),
     &              rhobcr1(nt,npd3,2),
     &              pbcr0  (nt,npd3,2),
     &              pbcr1  (nt,npd3,2),
     &              tbcr0  (nt,npd3,2),
     &              tbcr1  (nt,npd3,2),
     &              tbcr0re(nt,np),
     &              tbcr1re(nt,np),
     &              epbcr0 (nt,npd3,2),
     &              epbcr1 (nt,npd3,2),
     &              embcr0 (nt,npd3,2),
     &              embcr1 (nt,npd3,2),
     &              vmag0,
     &              rhobcr00(nt,np),
     &              rhorad(nt,np),
     &              rhogbc0(nt,np),
     &              vrparr0(nt,np),
     &              vtparr0(nt,np),
     &              vpparr0(nt,np),
     &              vrtransr0(nt,np),
     &              vttransr0(nt,np),
     &              vptransr0(nt,np),
     &              vrexbr0(nt,np),
     &              vtexbr0(nt,np),
     &              vpexbr0(nt,np)
c
c ****** Emerging flux.
c
      parameter (mxefnodes=100)
      logical emgflux
      character*72 brfile_ef
      common /eflux1/ emgflux,bmin,
     &                tnode_ef(mxefnodes),
     &                brfile_ef(mxefnodes)
      character*16 vrbcr0ef,vtbcr0ef,vpbcr0ef
      common /eflux2/ vrbcr0ef,
     &                vtbcr0ef,
     &                vpbcr0ef
c
c ****** Electric field at r=R0.
c
      common /e0com/ er0(nt,np),et0(nt,np),ep0(nt,np)
c
c ****** Diagnostics.
c
      parameter (nhq=27)
      parameter (nhistmax=2002)
      parameter (maxmodes=npd3)
      common /diagsprm/ ihistint,thistint,ifhststp,ihist,
     &                  ipltxint,tpltxint,ifplxstp,
     &                  ihst,jhst,khst,
     &                  irsdump,trsdump
      common /thistory/ thist(nhistmax,nhq),
     &                  bsqmhst(nhistmax,maxmodes),
     &                  vsqmhst(nhistmax,maxmodes),
     &                  mmodes(maxmodes)
      parameter (mxplist=30)
      character*8 plotlist
      common /pltvars1/ nplist
      common /pltvars2/ plotlist(mxplist)
c
c ****** Alfven waves.
c
      common /awcom1/ nsawp,nsawm,dtawpav,dtawmav,
     &                ifaw,pw0,rho_aw
     &                
c
c ****** Test wave.
c
      common /tstwv/ nwave,lwave,epswave
c
c ****** 1D Parker solar wind.
c
      common /parkcom1/ np1d
      character*72 onedfile
      common /parkcom2/ onedfile
c
c ****** Plasma equilibrium parameters.
c
      common /eqcom1/ rhor0,pr0,rhofac1,rhofac2
c
c ****** Magnetic field normalization.
c
      common /bfield1/ b0
c
      logical bingauss
      common /bfield2/ bingauss
c
c ****** Specified photospheric field.
c
      character*72 bnfile
      common /bfield3/ bnfile
c
c ****** Dipole magnetic field tilt angle.
c
      common /bfield4/ dipangle
c
c ****** Miscellaneous.
c
      logical ifabort,ifend
      common /misc1/ sifac(nrm1,ntm1),ifabort,ifend,
     & ifrsin,ifrsout,iftest
c
c ****** Dump output data file format.
c
      character*16 dformat
      common /misc2/ dformat
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Number of equations to solve in CG solver.
c
      common /cgeqns/ ncgeq
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      cliche cgcom2
c
c-----------------------------------------------------------------------
c
c ****** CG field solver parameters.
c
      common /cgvals/ ifprec,epscg,epscga,ncgmax,sigma,
     &                ncg,ncghist
      common /cgnrm/ bnrm,rnrm
c
c ****** CG field solver counters.
c
      common /cgcount1/ nsolves_a,nsolves_t,nsolves_si,
     &                  nsolves_visc,nsolves_pot,
     &                  ntotal_a,ntotal_t,ntotal_si,
     &                  ntotal_visc,ntotal_pot
c
      character*8 solvtype
      common /cgcount2/ solvtype
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
      module solver_types
c
c-----------------------------------------------------------------------
c
c ****** Definitions of various solver types.
c
      integer,parameter:: ST_AMATP0  = 1
      integer,parameter:: ST_AMATT   = 2
      integer,parameter:: ST_VMATRT0 = 3
      integer,parameter:: ST_VMATP0  = 4
      integer,parameter:: ST_VMATRTP = 5
      integer,parameter:: ST_A       = 6
      integer,parameter:: ST_T       = 7
c
c ****** Current solver type.
c
      integer:: solve_type
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module matrix_pot
c
c-----------------------------------------------------------------------
c
c ****** Storage for matrix diagonals for the potential field solve.
c
      real,dimension(:,:),allocatable:: a00,ap0,a0p
c
c-----------------------------------------------------------------------
c
c ****** Storage for the preconditioning diagonal.
c
      real,dimension(:,:),allocatable:: d
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module matrix_vmat
c
c-----------------------------------------------------------------------
c
c ****** Storage for matrix diagonals for the semi-implicit
c ****** and viscosity solves.
c
      real,dimension(:,:),allocatable:: arr00,arrp0,arr0p,
     &                                  att00,attp0,att0p,
     &                                  app00,appp0,app0p,
     &                                  art00,artp0,art0m,artpm,
     &                                  arp00,arpp0,
     &                                  atp00,atp0p
c
c-----------------------------------------------------------------------
c
c ****** Storage for the preconditioning diagonals.
c
      real,dimension(:,:),allocatable:: drr,dtt,dpp
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module matrix_a
c
c-----------------------------------------------------------------------
c
c ****** Variables for the implicit resistivity advance.
c
      integer:: m_a,iri_a
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      module rdtxt2d_arrays
c
c-----------------------------------------------------------------------
c
c ****** Arrays for reading in data in routine RDTXT2D.
c
      real,allocatable,target:: f_rdtxt2d(:,:),
     &                          x_rdtxt2d(:),
     &                          y_rdtxt2d(:)
c
c-----------------------------------------------------------------------
c
      end module
c
c-----------------------------------------------------------------------
c
      cliche nmlist
c
c-----------------------------------------------------------------------
c
c ****** Namelist input variables.
c
      namelist /invars/ ntmax,tmax,dtmax,dtmin,rl,
     & g0,ifvdgv,simult,dtmult,strmax,isitype,cfl,omegaeta,
     & mmodes,upwinda,upwindv,pdma,
     & ifideal,ietatype,slund,rmcell,
     & rmcell_facr,rmcell_fact,rmcell_facp,
     & rmcell_etamax,rmcell_unift,
     & nretap,retap,etapr,ntetap,tetap,etapt,
     & ivistype,visc,recell,
     & recell_facr,recell_fact,recell_facp,
     & recell_viscmax,recell_unift,
     & nrvisp,rvisp,vispr,ntvisp,tvisp,vispt,
     & ifrsout,rsifile,
     & option,eqtype,fldtype,ifprec,icgtype,epscg,ncgmax,sigma,
     & gamma,ncghist,nwave,lwave,epswave,epscga,
     & drratio,rfrac,dtratio,tfrac,nfrmesh,nftmesh,
     & ifrho,iftemp,ifpc,predflow,betapc_p,betapc_v,si_aggressive,
     & fac_cflv,betapc_si,
     & ihistint,thistint,ipltxint,tpltxint,
     & onedfile,np1d,dformat,plotlist,b0,bingauss,
     & rhor0,pr0,dthmax,th0,tnode,vnode,rhofac1,rhofac2,
     & ihst,jhst,khst,irsdump,trsdump,ifohmic,ifhvisc,
     & bcr1type,bcr0type,isetrho,omegarho,upwindar,upwindat,
     & upwindap,upwindphi,ifeqcs,dipangle,bnfile,
     & gbc0err,parchar,ubzero,nfiltub,nfiltne0,
     & tcond,radloss,tbc0,tbc1,tne0,
     & tnode_ch,q0_pole_ch,q0_eq_ch,hlen_pole_ch,hlen_eq_ch,
     & q0phys_ch,qalfven_ch,
     & tc_r,tc_dr,tc_fac0,tc_fac1,
     & he_frac,br00,ifaw,rho_aw,
     & ishearprof,arotate,brotate,crotate,
     & alpha_nocoll,tc_nocoll_r,tc_nocoll_dr,
     & neradmin,thprof_th,thprof_dth,awthprof_th,awthprof_dth,
     & emgflux,bmin,tnode_ef,brfile_ef,awthprof2_th,
     & vrbcr0ef,vtbcr0ef,vpbcr0ef,ifradbc,ifbheat,bheatfile,
     & fdphi,ifimplrad,ifeta_dyn,eta_min,eta_max,eta_jmin,eta_dj,
     & eta_jslope,bsqfloor,fjfrac,
     & t0_tc,alpha_tc,dt0_tc,nfilt_kappa,split_visc,
     & gamma1,gamma2,drgamma,rgamma
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
c ****** Utility macros.
c
c-----------------------------------------------------------------------
c
c ****** Averaging macros.
c
#define AVG(A1,A2,A3,A4) A1(A2,A3,A4)
#define AVGR(A1,A2,A3,A4) (.5*(A1(A2,A3,A4)+A1(A2-1,A3,A4)))
#define AVGT(A1,A2,A3,A4) (.5*(A1(A2,A3,A4)+A1(A2,A3-1,A4)))
#define AVGRT(A1,A2,A3,A4) (.25*(A1(A2,A3,A4)+A1(A2-1,A3,A4)+\
A1(A2,A3-1,A4)+A1(A2-1,A3-1,A4)))
c
#define AVG2(A1,A2,A3) A1(A2,A3)
#define AVG2R(A1,A2,A3) (.5*(A1(A2,A3)+A1(A2-1,A3)))
#define AVG2T(A1,A2,A3) (.5*(A1(A2,A3)+A1(A2,A3-1)))
#define AVG2RT(A1,A2,A3) (.25*(A1(A2,A3)+A1(A2-1,A3)+\
A1(A2,A3-1)+A1(A2-1,A3-1)))
c
c ****** Advection macros.
c
#define ADVR(A1,A2,A3,A4,A5) (.5*((1.-A5)*A1(A2,A3,A4)+\
(1.+A5)*A1(A2-1,A3,A4)))
#define ADVT(A1,A2,A3,A4,A5) (.5*((1.-A5)*A1(A2,A3,A4)+\
(1.+A5)*A1(A2,A3-1,A4)))
#define ADVPHI(A1,A2,A3,A4,A5,A6) (.5*((1.-A6)*A1(A2,A3,A5)+\
(1.+A6)*A1(A2,A3,A4)))
c
#define ADVCT(A1,A2,A3) (.5*((1.-A3)*A1+(1.+A3)*A2))
c
c ****** Cross product macros.
c
#define CROSSR(A1,A2) (A1##t*A2##p-A1##p*A2##t)
#define CROSST(A1,A2) (A1##p*A2##r-A1##r*A2##p)
#define CROSSP(A1,A2) (A1##r*A2##t-A1##t*A2##r)
c
c ****** Extrapolation macros.
c
#define EXTRAPR0(A1,A2,A3,A4) (A1(A2,A3,A4)+(A1(A2,A3,A4)\
-A1(A2+1,A3,A4))*dr(A2-1)*dri(A2))
#define EXTRAPR1(A1,A2,A3,A4) (A1(A2,A3,A4)+(A1(A2,A3,A4)\
-A1(A2-1,A3,A4))*dr(A2)*dri(A2-1))
c
c-----------------------------------------------------------------------
c
c#######################################################################
      block data
c
c-----------------------------------------------------------------------
c
c ****** Default values.
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom2
c
c-----------------------------------------------------------------------
c
      data ntime/0/
      data dtimeav/0./
c
      data ntmax/500/
      data tmax/200./
      data g0/.823/
      data dtmax/1./
      data dtmin/.001/
      data dtmult/1./
      data isitype/1/
      data cfl/.4/
      data rl/1./
      data rfrac/nmseg*1./
      data drratio/nmseg*1./
      data tfrac/nmseg*1./
      data dtratio/nmseg*1./
      data nfrmesh/0/
      data nftmesh/0/
      data ifvdgv/1/
      data simult/1.1/
      data strmax/.5/
      data upwinda/0./
      data upwindar/0./
      data upwindat/0./
      data upwindap/0./
      data upwindv/0./
      data pdma/1./
      data upwindphi/1./
c
      data gbc0err/1.e-5/
      data parchar/.false./
      data ubzero/.false./
      data nfiltub/0/
      data nfiltne0/0/
c
      data ifideal/1/
      data omegaeta/.5/
      data ietatype/1/
      data slund/1.e3/
      data rmcell/0./
      data rmcell_facr/1./
      data rmcell_fact/1./
      data rmcell_facp/1./
      data rmcell_etamax/1.e20/
      data rmcell_unift/.false./
      data nretap/1/
      data retap/mxnodes*1./
      data etapr/mxnodes*1./
      data ntetap/1/
      data tetap/mxnodes*0./
      data etapt/mxnodes*1./
c
      data ifeta_dyn/0/
      data eta_min/0./
      data eta_max/0./
      data eta_jmin/30./
      data eta_dj/5./
      data eta_jslope/1./
      data bsqfloor/0./
      data fjfrac/0.5/
c
      data split_visc/.true./
c
      data ivistype/1/
      data visc/0./
      data recell/0./
      data recell_facr/1./
      data recell_fact/1./
      data recell_facp/1./
      data recell_viscmax/1.e20/
      data recell_unift/.false./
      data nrvisp/1/
      data rvisp/mxnodes*1./
      data vispr/mxnodes*1./
      data ntvisp/1/
      data tvisp/mxnodes*0./
      data vispt/mxnodes*1./
c
      data gamma1/0./
      data gamma2/0./
      data rgamma/0./
      data drgamma/0./
      data mmodes/maxmodes*-1/
      data ihistint/1/
      data thistint/0./
      data ihist/0/
      data ipltxint/0/
      data tpltxint/0./
      data ifabort/.false./
      data ifend/.false./
      data ifrsout/1/
      data rsifile/'rsmas'/
c
      data option/'NO_DEFAULT'/
      data eqtype/'NO_DEFAULT'/
      data fldtype/'NO_DEFAULT'/
c
      data ncgmax/500/
      data epscg/1.e-8/
      data epscga/1.e-8/
      data ifprec/2/
      data sigma/1./
      data ncghist/0/
c
      data gamma/1.05/
      data ifrho/1/
      data iftemp/1/
      data ifpc/1/
      data predflow/.false./
      data betapc_v/.5/
      data betapc_p/.5/
      data si_aggressive/.false./
      data betapc_si/1./
      data fac_cflv/1./
      data nwave/1/
      data lwave/1/
      data epswave/1.e-6/
      data onedfile/'NO_DEFAULT'/
      data np1d/701/
      data dformat/'text'/
      data plotlist/mxplist*' '/
c
      data rhor0/1./
      data pr0/.1/
      data rhofac1/.04/
      data rhofac2/.104/
c
      data b0/1./
      data bingauss/.false./
      data bnfile/'NO_DEFAULT'/
      data dipangle/0./
c
      data tnode/mxtnodes*-1./
      data vnode/mxtnodes*0./
      data dthmax/.6981/
      data th0/1.570796327/
      data ishearprof/1/
      data arotate/0./
      data brotate/0./
      data crotate/0./
c
      data ihst/0/
      data jhst/0/
      data khst/0/
      data irsdump/0/
      data trsdump/0./
c
      data ifohmic/0/
      data ifhvisc/0/
c
      data bcr0type/'fixed'/
      data bcr1type/'extrapolation'/
c
      data isetrho/0/
      data omegarho/.1/
c
      data ifeqcs/0/
c
      data tcond/0./
      data radloss/0./
      data tbc0/5.e5/
      data tbc1/0./
      data tne0/1./
      data tc_r/10./
      data tc_dr/5./
      data tc_fac0/1./
      data tc_fac1/1./
      data neradmin/-1.e10/
      data alpha_tc/2.5/
      data t0_tc/2.5e5/
      data dt0_tc/2.e4/
      data nfilt_kappa/0/                                                         
c
      data alpha_nocoll/0./
      data tc_nocoll_r/10./
      data tc_nocoll_dr/5./
c
      data he_frac/0./
c
      data br00/0./
c
      data tnode_ch/mxtnodes*-1./
      data q0_eq_ch/mxtnodes*1./
      data q0_pole_ch/mxtnodes*1./
      data hlen_eq_ch/mxtnodes*.5/
      data hlen_pole_ch/mxtnodes*.5/
      data q0phys_ch/mxtnodes*0./
      data qalfven_ch/mxtnodes*0./
c
      data thprof_th/0.7/
      data thprof_dth/0.08/
c
      data awthprof_th/0.7/
      data awthprof2_th/2.44159265358979323844/
      data awthprof_dth/0.08/
c
      data ifradbc/1/ 
      data ifbheat/0/ 
      data ifimplrad/1/ 
      data bheatfile/'brbyb2.dat'/ 
c
      data nsawp/0/
      data nsawm/0/
      data dtawpav/0./
      data dtawmav/0./
      data ifaw/0/
      data rho_aw/1./
c
      data emgflux/.false./
      data bmin/.05/
      data tnode_ef/mxefnodes*-1./
      data brfile_ef/mxefnodes*'NO_DEFAULT'/
      data vrbcr0ef/'exb'/
      data vtbcr0ef/'zero'/
      data vpbcr0ef/'zero'/
c
      data fdphi/.false./
c
      end
c#######################################################################
      program MAS
c
c-----------------------------------------------------------------------
c
c ****** MAIN PROGRAM.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      tused=second()
      call setup
c
c ****** Initialization.
c
      call start
c
      write (*,*)
      write (*,*) '>running'
c
c ****** Timestep loop.
c
  100 continue
c
      ntime=ntime+1
c
      call setdt
      time=time+dtime
c
c ****** Keep track of the average time step.
c
      dtimeav=(dtimeav*(ntime-1)+dtime)/ntime
c
      call histchek
      call advance
      call diags
      call rsdump
c
c ****** Check if the CPU time limit has been exceeded.
c
      call chktime
c
c ****** Check if finished.
c
c ****** Set logical flag IFEND=.true. when the run is over.
c
      if (ntime.ge.ntmax.or.time.ge.tmax) ifend=.true.
c
c ****** Check if the user has requested to stop the run
c ****** prematurely.
c
      call check_if_stoprun
c
c ****** End the run on a recoverable abort (IFABORT=.true.).
c
      if (ifabort.or.ifend) go to 200
c
c ****** Advance for another time step.
c
      go to 100
c
  200 continue
c
c ****** Write spatial diagnostics, time histories, and exit.
c
      call wrdiags
c
      end
c#######################################################################
      subroutine chktime
c
c-----------------------------------------------------------------------
c
c ****** Terminate gracefully (by setting the abort flag IFABORT)
c ****** if the CPU time remaining for this run is less than TMIN.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      parameter (tmin=60.)
c
c-----------------------------------------------------------------------
c
c ****** Check the time remaining, and set logical flag
c ****** IFABORT=.true. if less than TMIN seconds is left.
c
      call tremain (tleft) 
c
      if (tleft.le.tmin) then
        write (*,*)
        write (*,*) '### WARNING from CHKTIME:'
        write (*,*) '### The CPU time limit has been exceeded.'
        write (*,*) '### The run is being aborted.'
        write (*,*)
        write (*,*) 'NTIME = ',ntime
        write (*,*) 'TIME = ',time
        write (9,*)
        write (9,*) '### WARNING from CHKTIME:'
        write (9,*) '### The CPU time limit has been exceeded.'
        write (9,*) '### The run is being aborted.'
        write (9,*)
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
        ifrsout=1
        ifabort=.true.
      end if
c
      return
      end
c#######################################################################
      subroutine setup
c
c-----------------------------------------------------------------------
c
c ****** Set up the run, input/output, etc.
c
c-----------------------------------------------------------------------
c
      use ident
      use comm
c
c-----------------------------------------------------------------------
c
      external alpha,letter,rdlin
      logical alpha,letter,rdlin
c
c-----------------------------------------------------------------------
c
c ****** Output line buffer.
c
      character*80 line(20)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line
c
      call parse (ierr)
      if (ierr.ne.0) call exit (1)
c
c ****** Create file names based on RUNID.
c
      lrid=lenstr(runid)
      if (lrid.le.0.or.lrid.gt.9) go to 150
      if (.not.letter(runid(1:1))) go to 150
      do 100 i=1,lrid
        if (.not.alpha(runid(i:i))) go to 150
  100 continue
      go to 200
  150 continue
      write (*,*)
      write (*,*) '### ERROR in SETUP:'
      write (*,*) '### Run ID is invalid.'
      write (*,*) 'RUNID = ',runid
      write (9,*)
      write (9,*) '### ERROR in SETUP:'
      write (9,*) '### Run ID is invalid.'
      write (9,*) 'RUNID = ',runid
      call exit (1)
  200 continue
c
      if (infile.eq.' ') infile='i'//runid(1:lrid)
      outfile='o'//runid(1:lrid)
      rsofile='rs'//runid(1:lrid)
c
c ****** Open the necessary files.
c
      call ffopen (8,infile,'r')
      call ffopen (9,outfile,'rw')
c
c ****** Write out identification and parameters.
c
      call date_and_time(rdate,rtime)
c
      write (line,300) 'Code: ',idcode,
     &                 'Version: ',vers,
     &                 'Updated on: ',update,
     &                 'Source file: ',source,
     &                 'Run ID: ',runid,
     & 'Run started on: ',rdate(1:4)//"/"//rdate(5:6)//"/"//rdate(7:8),
     & 'Run started at: ',rtime(1:2)//":"//rtime(3:4)//":"//rtime(5:6),
     &                 'Ran on machine: ',machname,
     &                 'Machine type: ',machtype
  300 format (9(/,a,a))
      call wrlins (line,10)
c
      write (line,310) '### Parameters:',
     &                 'NR = ',nr,
     &                 'NT = ',nt,
     &                 'NP = ',np,
     &                 'NHISTMAX = ',nhistmax
  310 format (/,a,/,4(/,a,i6))
      call wrlins (line,7)
c
c ****** Echo the input file to the TTY and output file.
c
      write (line,330) '### Input file contents:'
  330 format (/,a,/)
      call wrlins (line,3)
c
  400 continue
      if (rdlin(8,line)) then
        call wrlins (line,1)
        go to 400
      end if
c
      write (*,*)
      write (9,*)
c
c ****** Rewind the input file.
c
      rewind (8)
c
      return
      end
c#######################################################################
      subroutine wrlins (line,nlin)
c
c-----------------------------------------------------------------------
c
c ****** Write NLIN lines from array LINE to the TTY and to
c ****** the output file (unit 9).
c
c-----------------------------------------------------------------------
c
      character*(*) line(*)
c
c-----------------------------------------------------------------------
c
      do 100 i=1,nlin
        ll=max0(1,lenstr(line(i)))
        write (*,*) line(i)(1:ll)
        write (9,*) line(i)(1:ll)
  100 continue
c
      return
      end
c#######################################################################
      subroutine abortrun
c
c-----------------------------------------------------------------------
c
c ****** Abort the run.
c
c ****** Set the abort switch, call routine WRDIAGS to write
c ****** diagnostics, and finish up.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      write (*,*)
      write (*,*) '### COMMENT from ABORTRUN:'
      write (*,*) '### The run is being aborted.'
      write (*,*)
      write (*,*) 'NTIME = ',ntime
      write (*,*) 'TIME = ',time
      write (9,*)
      write (9,*) '### COMMENT from ABORTRUN:'
      write (9,*) '### The run is being aborted.'
      write (9,*)
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
c
c ****** Set the abort switch.
c
      ifabort=.true.
c
c ****** Set the switch to write a restart file.
c
      ifrsout=1
c
c ****** Write diagnostics.
c
      call wrdiags
c
      end
c#######################################################################
      subroutine wrdiags
c
c-----------------------------------------------------------------------
c
c ****** Write out final diagnostics and finish up the run.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Write spatial diagnostics (unless just written).
c
      if (ifabort) then
        call xdiags
      else
        if (ifplxstp.eq.0) call xdiags
      end if
c
c ****** Write the time histories.
c
      call histplot
c
c ****** Write out restart file if appropriate.
c
      if (ifrsout.ne.0) call wrrstrt (rsofile)
c
      call finish
c
      end
c#######################################################################
      subroutine finish
c
c-----------------------------------------------------------------------
c
c ****** Finish up the run and close output files.
c
c-----------------------------------------------------------------------
c
      use ident
      use comm
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Output line buffer.
c
      character*80 line(20)
c
c-----------------------------------------------------------------------
c
      call date_and_time(rdate,rtime)
c
      write (line,100) 'Code: ',idcode,
     &                 'Version: ',vers,
     &                 'Updated on: ',update,
     &                 'Source file: ',source,
     &                 'Run ID: ',runid,
     & 'Run ended on: ',rdate(1:4)//"/"//rdate(5:6)//"/"//rdate(7:8),
     & 'Run ended at: ',rtime(1:2)//":"//rtime(3:4)//":"//rtime(5:6),
     &                 'Ran on machine: ',machname,
     &                 'Machine type: ',machtype
  100 format (9(/,a,a))
      call wrlins (line,10)
c
      write (line,110) '### End of run ...',
     &                 'NTIME = ',ntime,
     &                 'TIME = ',time
  110 format (/,a,//,a,i6,/,a,1pe14.7)
      call wrlins (line,5)
c
c ****** Write out the average time step used.
c
      write (9,*)
      write (9,*) '### Time advance information:'
      write (9,*) 'Average time step used = ',dtimeav
c
c ****** Write out the average time step used in the Alfven
c ****** wave advance.
c
      if (ifaw.ne.0) then
        write (9,*)
        write (9,*) '### Alfven wave advance information:'
        write (9,*) 'Average time step for ep advance = ',dtawpav
        write (9,*) 'Average time step for em advance = ',dtawmav
      end if
c
c ****** Write out the field solver convergence information.
c
      write (9,*)
      write (9,*) '-----------------------------------------'
      write (9,*) '### Field solver convergence information:'
      write (9,*) '-----------------------------------------'
c
      navg_si=0
      if (nsolves_si.ne.0) navg_si=ntotal_si/nsolves_si
c
      write (9,*)
      write (9,*) '### Semi-implicit advance:'
      write (9,*) 'Total number of solves = ',nsolves_si
      write (9,*) 'Total number of iterations = ',ntotal_si
      write (9,*) 'Average number of iterations per solve = ',navg_si
c
      navg_visc=0
      if (nsolves_visc.ne.0) navg_visc=ntotal_visc/nsolves_visc
c
      write (9,*)
      write (9,*) '### Implicit viscosity advance:'
      write (9,*) 'Total number of solves = ',nsolves_visc
      write (9,*) 'Total number of iterations = ',ntotal_visc
      write (9,*) 'Average number of iterations per solve = ',navg_visc
c
      navg_a=0
      if (nsolves_a.ne.0) navg_a=ntotal_a/nsolves_a
c
      write (9,*)
      write (9,*) '### Implicit resistivity advance:'
      write (9,*) 'Total number of solves = ',nsolves_a
      write (9,*) 'Total number of iterations = ',ntotal_a
      write (9,*) 'Average number of iterations per solve = ',navg_a
c
      navg_t=0
      if (nsolves_t.ne.0) navg_t=ntotal_t/nsolves_t
c
      write (9,*)
      write (9,*) '### Implicit thermal conductivity advance:'
      write (9,*) 'Total number of solves = ',nsolves_t
      write (9,*) 'Total number of iterations = ',ntotal_t
      write (9,*) 'Average number of iterations per solve = ',navg_t
c
      navg_pot=0
      if (nsolves_pot.ne.0) navg_pot=ntotal_pot/nsolves_pot
c
      write (9,*)
      write (9,*) '### Potential field solution:'
      write (9,*) 'Total number of solves = ',nsolves_pot
      write (9,*) 'Total number of iterations = ',ntotal_pot
      write (9,*) 'Average number of iterations per solve = ',navg_pot
c
      tused=second()
c
      write (line,120) '### CPU time used = ',tused,' seconds.'
  120 format (/,a,f10.2,a,/)
      call wrlins (line,3)
c
c ****** Close the output file.
c
      close (9)
c
c ****** Exit.
c
      if (ifabort) then
        call exit (1)
      else
        call exit (0)
      end if
c
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line arguments.
c
c-----------------------------------------------------------------------
c
c ****** Syntax is:
c
c     <executable> [-debug] <runid> [<infile>]
c
c ****** Return IERR=0 if the command line parameters are
c ****** valid.
c
c-----------------------------------------------------------------------
c
      use ident
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Command-line arguments.
c
c      integer function iargc
      external iargc
c
c-----------------------------------------------------------------------
c
      character*80 strng
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Set the defaults.
c
      idebug=0
      ifrid=0
      ififil=0
      iarg=1
c
      narg=iargc()
c
      if (narg.eq.0) go to 900
c
c ****** Parse argumets.
c
c ****** Get the debug switch.
c ****** Syntax: -debug
c
      if (narg.ge.1) then
        call getarg (iarg,strng)
        if (strng.eq.'-debug') then
          idebug=1
          iarg=iarg+1
          narg=narg-1
        end if
      end if
c
c ****** Get the run ID.
c ****** Syntax: <runid>
c
      if (narg.ge.1) then
        call getarg (iarg,runid)
        ifrid=1
        iarg=iarg+1
        narg=narg-1
      end if
c
c ****** Get the input file name.
c ****** Syntax: <infile>
c
      if (narg.ge.1) then
        call getarg (iarg,infile)
        ififil=1
        iarg=iarg+1
        narg=narg-1
      end if
c
c ****** There should be no further arguments left.
c
      if (narg.ne.0) go to 900
c
c ****** Parsing completed.  Load arguments.
c
c ****** Check that the run ID has been specified.
c
      if (ifrid.eq.0) go to 900
c
      if (ififil.eq.0) infile=' '
c
      return
c
  900 continue
c
c ****** Error exit.
c
      ierr=1
c
      write (*,'(4(/,1x,a,a))') 'Code: ',idcode,
     &                          'Version: ',vers,
     &                          'Updated on: ',update,
     &                          'Source file: ',source
c
      write (*,'(/,1x,a,/,4(/,1x,a,i6))') '### Parameters:',
     &                                    'NR = ',nr,
     &                                    'NT = ',nt,
     &                                    'NP = ',np,
     &                                    'NHISTMAX = ',nhistmax
c
      write (*,*)
      write (*,*) '### ERROR in PARSE:'
      write (*,*) '### Command line syntax error.'
      write (*,*) 'The command line syntax is:'
      write (*,*)
      write (*,*) '<executable> [-debug] <runid> [<infile>]'
c
      return
      end
c#######################################################################
      subroutine start
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom2
      use nmlist
c
c-----------------------------------------------------------------------
c
c ****** Initialize CG solver counters to zero.
c
      data nsolves_a/0/
      data nsolves_t/0/
      data nsolves_si/0/
      data nsolves_visc/0/
      data nsolves_pot/0/
c
      data ntotal_a/0/
      data ntotal_t/0/
      data ntotal_si/0/
      data ntotal_visc/0/
      data ntotal_pot/0/
c
c-----------------------------------------------------------------------
c
c ****** Temporary arrays for complex B and J.
c
      real bcrr(nr,nt,npd3),bcri(nr,nt,npd3),
     &     bctr(nr,nt,npd3),bcti(nr,nt,npd3),
     &     bcpr(nr,nt,npd3),bcpi(nr,nt,npd3)
c
      real jcrr(nr,nt,npd3),jcri(nr,nt,npd3),
     &     jctr(nr,nt,npd3),jcti(nr,nt,npd3),
     &     jcpr(nr,nt,npd3),jcpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Read the namelist parameters and write them out.
c
      write (9,100)
  100 format (//,' ### Parameter values:',/)
      read (8,invars)
      write (9,invars)
      write (9,*)
      write (9,*)
      close (8)
c
c ****** Find the normalization variables.
c
      call normaliz
c
c ****** Determine the number of quantities requested for
c ****** plotting.
c
      do 200 i=1,mxplist
        if (plotlist(i).eq.' ') go to 210
  200 continue
      i=mxplist+1
  210 continue
      nplist=i-1
c
c ****** Check that the specified types of radial boundary
c ****** conditions are valid.
c
      if (.not.(bcr0type.eq.'fixed'.or.
     &          bcr0type.eq.'1dchar')) then
        write (*,*)
        write (*,*) '### ERROR in START:'
        write (*,*) '### Invalid inner radial boundary condition.'
        write (*,*) 'The requested BC type is invalid.'
        write (*,*) 'BCR0TYPE = ',bcr0type
        write (9,*)
        write (9,*) '### ERROR in START:'
        write (9,*) '### Invalid inner radial boundary condition.'
        write (9,*) 'The requested BC type is invalid.'
        write (9,*) 'BCR0TYPE = ',bcr0type
        call exit (1)
      end if
c
      if (.not.(bcr1type.eq.'open'.or.
     &          bcr1type.eq.'extrapolation'.or.
     &          bcr1type.eq.'1dchar'.or.
     &          bcr1type.eq.'fixed')) then
        write (*,*)
        write (*,*) '### ERROR in START:'
        write (*,*) '### Invalid outer radial boundary condition.'
        write (*,*) 'The requested BC type is invalid.'
        write (*,*) 'BCR1TYPE = ',bcr1type
        write (9,*)
        write (9,*) '### ERROR in START:'
        write (9,*) '### Invalid outer radial boundary condition.'
        write (9,*) 'The requested BC type is invalid.'
        write (9,*) 'BCR1TYPE = ',bcr1type
        call exit (1)
      end if
c
      if (recell.gt.0.) then
        write (9,*)
        write (9,*) '### COMMENT from START:'
        write (9,*) '### The local cell-Reynolds number '//
     &              'viscosity is being used.'
        write (9,*) 'Cell-Reynolds number, RECELL = ',recell
      end if
c
      if (rmcell.gt.0.) then
        write (9,*)
        write (9,*) '### COMMENT from START:'
        write (9,*) '### The local cell-magnetic-Reynolds number '//
     &              'resistivity is being used.'
        write (9,*) 'Cell-magnetic-Reynolds number, RMCELL = ',rmcell
      end if
c
c ****** Set the upwind coefficients for A.
c
      if (upwinda.gt.0.) then
        upwindar=upwinda
        upwindat=upwinda
        upwindap=upwinda
      end if
c
      write (9,*)
      write (9,*) '### COMMENT from START:'
      write (9,*) '### Upwind coefficients for the A equation:'
      write (9,*) 'UPWINDAR = ',upwindar
      write (9,*) 'UPWINDAT = ',upwindat
      write (9,*) 'UPWINDAP = ',upwindap
c
c ****** Initialize all the fields to zero.
c
      call bcast (arr,0.,nc)
      call bcast (ari,0.,nc)
      call bcast (atr,0.,nc)
      call bcast (ati,0.,nc)
      call bcast (apr,0.,nc)
      call bcast (api,0.,nc)
      call bcast (vrr,0.,nc)
      call bcast (vri,0.,nc)
      call bcast (vtr,0.,nc)
      call bcast (vti,0.,nc)
      call bcast (vpr,0.,nc)
      call bcast (vpi,0.,nc)
      call bcast (prr,0.,nc)
      call bcast (pri,0.,nc)
      call bcast (rhr,0.,nc)
      call bcast (rhi,0.,nc)
      call bcast (tr ,0.,nc)
      call bcast (ti ,0.,nc)
      call bcast (epr,0.,nc)
      call bcast (epi,0.,nc)
      call bcast (emr,0.,nc)
      call bcast (emi,0.,nc)
c
      call bcast (br  ,0.,nrtp)
      call bcast (bt  ,0.,nrtp)
      call bcast (bp  ,0.,nrtp)
      call bcast (fjr ,0.,nrtp)
      call bcast (fjt ,0.,nrtp)
      call bcast (fjp ,0.,nrtp)
      call bcast (vr  ,0.,nrtp)
      call bcast (vt  ,0.,nrtp)
      call bcast (vp  ,0.,nrtp)
      call bcast (rho ,0.,nrtp)
      call bcast (p   ,0.,nrtp)
      call bcast (temp,0.,nrtp)
      call bcast (ep  ,0.,nrtp)
      call bcast (em  ,0.,nrtp)
c
c ****** If using a restart file, set the restart switch and
c ****** read in the data.
c
      if (rsifile.ne.' ') then
c
        ifrsin=1
        call rdrstrt
c
c ****** Set the density to be used at cells at the lower radial
c ****** boundary at which there is an inflow.
c
        do k=1,np
          do j=1,nt
            rhobcr00(j,k)=rhor0
          enddo
        enddo
c
      else
c
        ifrsin=0
c
      end if
c
      call setmesh
c
      dtime=dtmax
c
      if (ifrsin.eq.0) then
        time=0.
        call initial
      else
        iftest=0
      end if
c
c ****** Initialize the heating array.
c
      if (ifbheat.ne.0) call initbheat(bheatfile)
c
c ****** Initialize the "GETBC0 density" array.
c
      call fft_tp (rhogbc0,rhobcr0(1,1,1),rhobcr0(1,1,2),0)
c
c ****** Load v, rho, p, ep, and em in real space.
c
      call fft (vr ,vrr,vri,0)
      call fft (vt ,vtr,vti,0)
      call fft (vp ,vpr,vpi,0)
      call fft (rho,rhr,rhi,0)
      call fft (p  ,prr,pri,0)
      call fft (ep ,epr,epi,0)
      call fft (em ,emr,emi,0)
c
c ****** Get the temperature.
c
      do k=1,np
        do j=1,nt
          do i=1,nr
            temp(i,j,k)=he_rho*p(i,j,k)/(he_p*rho(i,j,k))
          enddo
        enddo
      enddo
      call fft (temp,tr,ti,1)
c
c ****** Set the resistivity profile.
c
      call loadeta
c
c ****** Set the viscosity profile.
c
      call loadvisc
c
c ****** Load the gamma profile
c
      call loadgamma
c
      call bfroma (arr,ari,atr,ati,apr,api,
     &             bcrr,bcri,bctr,bcti,bcpr,bcpi)
      call jfromb (bcrr,bcri,bctr,bcti,bcpr,bcpi,
     &             jcrr,jcri,jctr,jcti,jcpr,jcpi)
c
c ****** Set the initial time step.
c
      call setdt
c
c ****** Initialize the transverse boundary velocity.
c
      call setvtrans0 (time+.5*dtime)
c
c ****** Set the point for collection of history diagnostics.
c
      if (ihst.le.0.or.ihst.gt.nr) ihst=nr/2
      ihst=max0(ihst,1)
      ihst=min0(ihst,nrm1)
      if (jhst.le.0.or.jhst.gt.nt) jhst=nt/2
      jhst=max0(jhst,1)
      jhst=min0(jhst,ntm1)
      if (khst.le.0.or.khst.gt.np) khst=1
      khst=max0(khst,1)
      khst=min0(khst,np)
c
      write (9,*)
      write (9,*) '### COMMENT from START:'
      write (9,*) 'The point for collection of history diagnostics'
      write (9,*) 'is located at:'
      write (9,*) 'IHST = ',ihst,', R = ',r(ihst)
      write (9,*) 'JHST = ',jhst,', T = ',t(jhst)
      write (9,*) 'KHST = ',khst,', P = ',phi(khst)
c
      call histchek
c
c ****** Get initial mode history information.
c ****** (Note that B## contains the complex B from the call
c ****** to BFROMA.)
c
      call modehst (bcrr,bcri,bctr,bcti,bcpr,bcpi,'b')
      call modehst (vrr,vri,vtr,vti,vpr,vpi,'v')
c
c ****** Collect and write diagnostics.
c
      call diags
c
      return
      end
c#######################################################################
      subroutine normaliz
c
c-----------------------------------------------------------------------
c
c ****** Calculate the normalization factors that convert
c ****** physical units to normalized code units.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Solar radius [cm].
c
      parameter (rsun=6.96e10)
c
c ****** Solar gravity at r=RSUN [cm/s**2].
c
      parameter (g0phys=0.274e5)
c
c ****** Number density corresponding to a normalized number
c ****** density of 1 [/cm**3].
c
      parameter (fn0phys=1.e8)
c
c ****** Proton mass [g].
c
      parameter (fmp=1.6726e-24)
c
c ****** Boltzmann's constant [erg/K].
c
      parameter (boltz=1.3807e-16)
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters related to the Helium fraction, HE_FRAC.
c ****** HE_FRAC is defined by n_alpha=HE_FRAC*n_p,
c ****** where n_alpha is the Helium number density and
c ****** n_p is the proton number density.
c
c ****** HE_RHO is defined by rho(norm)=HE_RHO*n_e(norm),
c ****** where n_e is the electron number density.
c
      he_rho=(1.+4.*he_frac)/(1.+2.*he_frac)
c
c ****** HE_P is defined by p(norm)=HE_P*n_e(norm)*T(norm),
c ****** where p is the total pressure.
c
      he_p=(2.+3.*he_frac)/(1.+2.*he_frac)
c
c ****** HE_NP is defined by n_p(norm)=HE_NP*n_e(norm).
c
      he_np=1./(1.+2.*he_frac)
c
c ****** Define the normalization variables for length, FNORML,
c ****** time, FNORMT, and mass, FNORMM.
c
      fnorml=rsun
      fnormt=sqrt(g0*fnorml/g0phys)
      fnormm=fn0phys*fmp*(fnorml)**3
c
c ****** Define the factors to normalize various physical
c ****** quantities. The physical quantity is expressed in terms
c ****** of the normalized quantity by:
c ****** Q(physical) = FN_Q * Q(normalized).
c
c ****** Number density normalization [/cm**3].
c 
      fn_n=fn0phys
c
c ****** Mass density normalization [g/cm**3].
c 
      fn_rho=fmp*fn0phys
c
c ****** Pressure normalization [erg/cm**3].
c 
      fn_p=fmp*fn0phys*(fnorml/fnormt)**2
c
c ****** Temperature normalization [K].
c 
      fn_t=fmp*fnorml**2/(boltz*fnormt**2)
c
c ****** Velocity normalization [cm/s].
c 
      fn_v=fnorml/fnormt
c
c ****** Magnetic field normalization [Gauss].
c 
      fn_b=sqrt(4.*pi*fn_rho)*fnorml/fnormt
c
c ****** Radiation loss function Q(T) normalization [erg-cm**3/s].
c 
      fn_qrad=fmp*fnorml**2/(fn0phys*fnormt**3)
c
c ****** Thermal conductivity normalization [erg/cm/s/K].
c 
      fn_kappa=boltz*fn0phys*fnorml**2/fnormt
c
c ****** Magnitude of the Spitzer parallel electron thermal
c ****** conductivity, kappa = 9.e-7*T**2.5 [erg/s/cm/K],
c ****** where T is in degrees Kelvin.
c
      fkspitz=9.e-7
c
c ****** Surface heat flux normalization [ergs/cm**2/s].
c
      fn_q0=fnormm/fnormt**3
c
c ****** Radiation balance boundary condition constants.
c 
      fn_rd1=0.5*tcond*fkspitz*fn_t**2/(he_np*fnorml**2*fn0phys**2)
      fn_rd2=2./7.*(fn_q0/fnorml)/(he_np*fn0phys**2)
c
      return
      end
c#######################################################################
      subroutine rdrstrt
c
c ****** Read the restart file.
c
c uk2 change ....
c 
c This has been completely rewritten to write platform-independent 
c HDF Scientific Dataset format files. 
c NOTE: There is absolutely no compatibility with this format 
c and the previous format. To write old-style restart files, 
c this routine must be replaced in its entirety with the previous 
c version. 
c To read old-style restart files, the routine xxxx must also be 
c replaced.
c
c-----------------------------------------------------------------------
c
      integer sd_id, DFACC_READ,dims(2),FAIL,DFNT_CHAR8,
     &          n_values,DFNT_INT32,DFNT_FLOAT64,istat,
     &          sds_id
      integer sfstart, sfcreate, sfwdata, sfendacc, sfend
      integer heprnt,status,sfrdata, sfselect, sfn2index
      integer sds_index,sffattr,sfrcatt,attr_index
      parameter (DFACC_READ=1,FAIL=-1,DFNT_CHAR8=4, 
     &           DFNT_INT32=24,DFNT_FLOAT64=6)
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c ****** Common block to save the initial boundary values
c ****** of vr, rho, and T.
c
      common /rtbc_ts/ vrbcr0re_ts(nt,np),
     &                 rhobcr0re_ts(nt,np),
     &                 tbcr0re_ts(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Open the restart file.
c
      sd_id = sfstart(rsifile, DFACC_READ)
      if (sd_id .eq. FAIL) then
        write (*,*)
        write (*,*) '### ERROR in RDRSTRT:'
        write (*,*) '### Error while opening the restart file.'
        write (9,*)
        write (9,*) '### ERROR in RDRSTRT:'
        write (9,*) '### Error while opening the restart file.'    
        write (*,*) 'File name: ',rsifile
        write (9,*) 'File name: ',rsifile
        call exit (1)
      end if
c
      PRINT*,'******************************'
      PRINT*,' Reading RESTART file '//rsifile

      write (9,*)
      write (9,*) '### COMMENT from RDRSTRT:'
      write (9,*) 'Reading from restart file: ',rsifile
c
c read the file attributes
c
      attr_index = sffattr(sd_id, 'nrout')
      istat = sfrcatt(sd_id, attr_index,nrin)
c
      attr_index = sffattr(sd_id, 'ntout')
      istat = sfrcatt(sd_id, attr_index,ntin)
c
      attr_index = sffattr(sd_id, 'npout')
      istat = sfrcatt(sd_id, attr_index,npin)
c
      attr_index = sffattr(sd_id, 'time')
      istat = sfrcatt(sd_id, attr_index,time)
c
      attr_index = sffattr(sd_id, 'br00')
      istat = sfrcatt(sd_id, attr_index,br00)
c
      if (nrin.ne.nr.or.ntin.ne.nt.or.npin.ne.np) then
        write (*,*)
        write (*,*) '### ERROR in RDRSTRT:'
        write (*,*) '### Restart file inconsistency.'
        write (*,*) 'The restart file parameters and the code'
        write (*,*) 'parameters do not agree.'
        write (*,*)
        write (*,*) 'Code values:'
        write (*,*) 'NR = ',nr  ,' NT = ',nt  ,' NP = ',np
        write (*,*)
        write (*,*) 'File values:'
        write (*,*) 'NR = ',nrin,' NT = ',ntin,' NP = ',npin
        write (9,*)
        write (9,*) '### ERROR in RDRSTRT:'
        write (9,*) '### Restart file inconsistency.'
        write (9,*) 'The restart file parameters and the code'
        write (9,*) 'parameters do not agree.'
        write (9,*)
        write (9,*) 'Code values:'
        write (9,*) 'NR = ',nr  ,' NT = ',nt  ,' NP = ',np
        write (9,*)
        write (9,*) 'File values:'
        write (9,*) 'NR = ',nrin,' NT = ',ntin,' NP = ',npin
        call exit (1)
      end if
c
c read the various data sets
c
      CALL READ_SDS_1D(sd_id,'r',nrm1,r)
      CALL READ_SDS_1D(sd_id,'t',ntm1,t)
      CALL READ_SDS_2D(sd_id,'etafac',nrm1,ntm1,etafac)
      CALL READ_SDS_2D(sd_id,'visfac',nrm1,ntm1,visfac)
      CALL READ_SDS_3D(sd_id,'arr',nr,nt,npd3,arr)
      CALL READ_SDS_3D(sd_id,'ari',nr,nt,npd3,ari)
      CALL READ_SDS_3D(sd_id,'atr',nr,nt,npd3,atr)
      CALL READ_SDS_3D(sd_id,'ati',nr,nt,npd3,ati)
      CALL READ_SDS_3D(sd_id,'apr',nr,nt,npd3,apr)
      CALL READ_SDS_3D(sd_id,'api',nr,nt,npd3,api)
      CALL READ_SDS_3D(sd_id,'vrr',nr,nt,npd3,vrr)
      CALL READ_SDS_3D(sd_id,'vri',nr,nt,npd3,vri)
      CALL READ_SDS_3D(sd_id,'vtr',nr,nt,npd3,vtr)
      CALL READ_SDS_3D(sd_id,'vti',nr,nt,npd3,vti)
      CALL READ_SDS_3D(sd_id,'vpr',nr,nt,npd3,vpr)
      CALL READ_SDS_3D(sd_id,'vpi',nr,nt,npd3,vpi)
      CALL READ_SDS_3D(sd_id,'rhr',nr,nt,npd3,rhr)
      CALL READ_SDS_3D(sd_id,'rhi',nr,nt,npd3,rhi)
      CALL READ_SDS_3D(sd_id,'prr',nr,nt,npd3,prr)
      CALL READ_SDS_3D(sd_id,'pri',nr,nt,npd3,pri)
      CALL READ_SDS_3D(sd_id,'vrbcr0',nt,npd3,2,vrbcr0)
      CALL READ_SDS_3D(sd_id,'vrbcr1',nt,npd3,2,vrbcr1)
      CALL READ_SDS_3D(sd_id,'vtbcr0',nt,npd3,2,vtbcr0)
      CALL READ_SDS_3D(sd_id,'vtbcr1',nt,npd3,2,vtbcr1)
      CALL READ_SDS_3D(sd_id,'vpbcr0',nt,npd3,2,vpbcr0)
      CALL READ_SDS_3D(sd_id,'vpbcr1',nt,npd3,2,vpbcr1)
      CALL READ_SDS_3D(sd_id,'rhobcr0',nt,npd3,2,rhobcr0)
      CALL READ_SDS_3D(sd_id,'rhobcr1',nt,npd3,2,rhobcr1)
      CALL READ_SDS_3D(sd_id,'pbcr0',nt,npd3,2,pbcr0)
      CALL READ_SDS_3D(sd_id,'pbcr1',nt,npd3,2,pbcr1)
      CALL READ_SDS_3D(sd_id,'epr',nr,nt,npd3,epr)
      CALL READ_SDS_3D(sd_id,'epi',nr,nt,npd3,epi)
      CALL READ_SDS_3D(sd_id,'emr',nr,nt,npd3,emr)
      CALL READ_SDS_3D(sd_id,'emi',nr,nt,npd3,emi)
c
c To restart the IP solutions, we also need to dump the 
c boundary gas variables
c 
      if (eqtype.eq.'interplanetary') then
          CALL READ_SDS_2D(sd_id,'vrbcr0re_ts',nt,np,vrbcr0re_ts)
          CALL READ_SDS_2D(sd_id,'rhobcr0re_ts',nt,np,rhobcr0re_ts)
          CALL READ_SDS_2D(sd_id,'tbcr0re_ts',nt,np,tbcr0re_ts)
      endif
c
c Terminate access to the SD interface and close the file.
      istat = sfend(sd_id)
      if (istat .eq. FAIL) then
        write (*,*)
        write (*,*) '### ERROR in RDRSTRT:'
        write (*,*) '### Error while closing the restart file.'
        write (*,*) 'File name: ',rsifile
        write (9,*)
        write (9,*) '### ERROR in RDRSTRT:'
        write (9,*) '### Error while closing the restart file.'
        write (9,*) 'File name: ',rsifile
       call exit (1)
      else
         print*,'     Success!'
         print*,'     ... file closed'
         PRINT*,'******************************'

         print*
      end if

c
      write (9,*) 'TIME = ',time
      write (9,*) 'NTIME = ',ntime
c

      RETURN
      END
c#######################################################################
      subroutine loadeta
c
c-----------------------------------------------------------------------
c
c ****** Load the resistivity profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*48 errmsg
c
c-----------------------------------------------------------------------
c
      if (ifideal.ne.0) then
c
c ****** Ideal MHD run.
c
        call bcast (etafac,0.,nrm1*ntm1)
c
        write (9,*)
        write (9,*) '### COMMENT from LOADETA:'
        write (9,*) '### This is an ideal MHD run.'
c
        return
c
      end if
c
      if (ietatype.eq.1) then
c
c ****** Uniform resistivity equal to 1./SLUND.
c
        call bcast (etafac,1./slund,nrm1*ntm1)
c
        write (9,*)
        write (9,*) '### COMMENT from LOADETA:'
        write (9,*) '### Uniform resistivity profile used.'
        write (9,*) 'IETATYPE = ',ietatype
        write (9,*) 'SLUND = ',slund
c
      else if (ietatype.eq.2) then
c
c ****** Custom input resistivity profile.
c
c ****** Interpolate bi-linearly using the profile defined by
c ****** the radial profile specified by NRETAP, RETAP, and ETAPR,
c ****** and the theta profile specified by NTETAP, TETAP, and ETAPT.
c ****** The overall profile is the product of the radial and theta
c ****** profiles, and is multiplied by 1./SLUND.
c
c ****** Check the specified nodes.
c
        dummy=flint(0.,nretap,retap,etapr,1,ierr)
        if (ierr.ne.0) then
          errmsg='Invalid radial nodes specified by NRETAP and RETAP.'
          go to 900
        end if
        dummy=flint(0.,ntetap,tetap,etapt,1,ierr)
        if (ierr.ne.0) then
          errmsg='Invalid theta nodes specified by NTETAP and TETAP.'
          go to 900
        end if
c
c ****** Do the interpolation.
c
        do 110 j=1,ntm1
          tfac=flint(t(j),ntetap,tetap,etapt,0,ierr)
          do 100 i=1,nrm1
            rfac=flint(r(i),nretap,retap,etapr,0,ierr)
            etafac(i,j)=rfac*tfac/slund
  100     continue
  110   continue
c
        write (9,*)
        write (9,*) '### COMMENT from LOADETA:'
        write (9,*) '### Custom input resistivity profile used.'
        write (9,*) 'IETATYPE = ',ietatype
        write (9,*) 'SLUND = ',slund
        write (9,*) 'NRETAP = ',nretap
        write (9,*) 'RETAP = '
        write (9,*) (retap(i),i=1,nretap)
        write (9,*) 'ETAPR = '
        write (9,*) (etapr(i),i=1,nretap)
        write (9,*) 'NTETAP = ',ntetap
        write (9,*) 'TETAP = '
        write (9,*) (tetap(i),i=1,ntetap)
        write (9,*) 'ETAPT = '
        write (9,*) (etapt(i),i=1,ntetap)
c
c
      else
c
        write (*,*)
        write (*,*) '### ERROR in LOADETA:'
        write (*,*) '### Invalid resistivity profile type.'
        write (*,*) 'IETATYPE = ',ietatype
        write (9,*)
        write (9,*) '### ERROR in LOADETA:'
        write (9,*) '### Invalid resistivity profile type.'
        write (9,*) 'IETATYPE = ',ietatype
        call exit (1)
c
      end if
c
      return
c
  900 continue
c
c ****** Error in node specification.
c
      write (*,*)
      write (*,*) '### ERROR in LOADETA:'
      write (*,*) '### Invalid resistivity profile specification.'
      write (*,*) errmsg
      write (*,*) 'IETATYPE = ',ietatype
      write (9,*)
      write (9,*) '### ERROR in LOADETA:'
      write (9,*) '### Invalid resistivity profile specification.'
      write (9,*) errmsg
      write (9,*) 'IETATYPE = ',ietatype
      call exit (1)
c
      end
c#######################################################################
      subroutine loadvisc
c
c-----------------------------------------------------------------------
c
c ****** Load the viscosity profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*48 errmsg
c
c-----------------------------------------------------------------------
c
      if (ivistype.eq.1) then
c
c ****** Uniform viscosity equal to VISC.
c
        call bcast (visfac,visc,nrm1*ntm1)
c
        write (9,*)
        write (9,*) '### COMMENT from LOADVISC:'
        write (9,*) '### Uniform viscosity profile used.'
        write (9,*) 'IVISTYPE = ',ivistype
        write (9,*) 'VISC = ',visc
c
      else if (ivistype.eq.2) then
c
c ****** Custom input viscosity profile.
c
c ****** Interpolate bi-linearly using the profile defined by
c ****** the radial profile specified by NRVISP, RVISP, and VISPR,
c ****** and the theta profile specified by NTVISP, TVISP, and VISPT.
c ****** The overall profile is the product of the radial and theta
c ****** profiles, and is multiplied by VISC.
c
c ****** Check the specified nodes.
c
        dummy=flint(0.,nrvisp,rvisp,vispr,1,ierr)
        if (ierr.ne.0) then
          errmsg='Invalid radial nodes specified by NRVISP and RVISP.'
          go to 900
        end if
        dummy=flint(0.,ntvisp,tvisp,vispt,1,ierr)
        if (ierr.ne.0) then
          errmsg='Invalid theta nodes specified by NTVISP and TVISP.'
          go to 900
        end if
c
c ****** Do the interpolation.
c
        do 110 j=1,ntm1
          tfac=flint(t(j),ntvisp,tvisp,vispt,0,ierr)
          do 100 i=1,nrm1
            rfac=flint(r(i),nrvisp,rvisp,vispr,0,ierr)
            visfac(i,j)=rfac*tfac*visc
  100     continue
  110   continue
c
        write (9,*)
        write (9,*) '### COMMENT from LOADVISC:'
        write (9,*) '### Custom input viscosity profile used.'
        write (9,*) 'IVISTYPE = ',ivistype
        write (9,*) 'VISC = ',visc
        write (9,*) 'NRVISP = ',nrvisp
        write (9,*) 'RVISP = '
        write (9,*) (rvisp(i),i=1,nrvisp)
        write (9,*) 'VISPR = '
        write (9,*) (vispr(i),i=1,nrvisp)
        write (9,*) 'NTVISP = ',ntvisp
        write (9,*) 'TVISP = '
        write (9,*) (tvisp(i),i=1,ntvisp)
        write (9,*) 'VISPT = '
        write (9,*) (vispt(i),i=1,ntvisp)
c
      else
c
        write (*,*)
        write (*,*) '### ERROR in LOADVISC:'
        write (*,*) '### Invalid viscosity profile type.'
        write (*,*) 'IVISTYPE = ',ivistype
        write (9,*)
        write (9,*) '### ERROR in LOADVISC:'
        write (9,*) '### Invalid viscosity profile type.'
        write (9,*) 'IVISTYPE = ',ivistype
        call exit (1)
c
      end if
c
      return
c
  900 continue
c
c ****** Error in node specification.
c
      write (*,*)
      write (*,*) '### ERROR in LOADVISC:'
      write (*,*) '### Invalid viscosity profile specification.'
      write (*,*) errmsg
      write (*,*) 'IVISTYPE = ',ivistype
      write (9,*)
      write (9,*) '### ERROR in LOADVISC:'
      write (9,*) '### Invalid viscosity profile specification.'
      write (9,*) errmsg
      write (9,*) 'IVISTYPE = ',ivistype
      call exit (1)
c
      end
c#######################################################################
      subroutine cellvisc
c
c-----------------------------------------------------------------------
c
c ****** Load a local viscosity to give a specified
c ****** cell-Reynolds number, RECELL.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      common /vcinfo/ vcmin,vcmax
c
      save ifirst
      data ifirst/1/
c
c-----------------------------------------------------------------------
c
c ****** If RECELL.le.0., the local viscosity is set to zero.
c
      if (recell.le.0.) then
        if (ifirst.ne.0) then
          call bcast (viscel,0.,nrm1*ntm1)
          vcmin=0.
          vcmax=0.
          ifirst=0
        end if
        return
      end if
c
c ****** Compute the viscosity to give the cell-Reynolds
c ****** number RECELL.
c
      do i=1,nrm1
        do j=1,ntm1
          vcmx=0.
          do k=1,np
            avvr=AVGT (vr,i  ,j+1,k)
            avvt=AVGR (vt,i+1,j  ,k)
            avvp=AVGRT(vp,i+1,j+1,k)
            vdr=abs(avvr*dr(i))*recell_facr
            vdt=abs(avvt*r(i)*dt(j))*recell_fact
            vdp=abs(avvp*r(i)*st(j)*dphi)*recell_facp
            vc=(vdr+vdt+vdp)/recell
            vcmx=max(vcmx,vc)
          enddo
          viscel(i,j)=min(vcmx,recell_viscmax)
        enddo
      enddo
c
c ****** If RECELL_UNIFT=.true., then make the cell viscosity
c ****** uniform in theta, equal to the largest viscosity at
c ****** each radial mesh point.
c
      if (recell_unift) then
        do i=1,nrm1
          vcmx=viscel(i,1)
          do j=1,ntm1
            vcmx=max(vcmx,viscel(i,j))
          enddo
          do j=1,ntm1
            viscel(i,j)=vcmx
          enddo
        enddo
      end if
c
c ****** Find the minimum amd maximum cell viscosity
c ****** as a diagnostic.
c
      vcmax=0.
      vcmin=1.e20
      do i=1,nrm1
        do j=1,ntm1
          vcmin=min(vcmin,viscel(i,j))
          vcmax=max(vcmax,viscel(i,j))
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine celleta
c
c-----------------------------------------------------------------------
c
c ****** Load a local resistivity according to the value of the
c ****** current density.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      common /ecinfo/ ecmin,ecmax
      real sint(ntm1)
c
      save ifirst
      data ifirst/1/
c
c-----------------------------------------------------------------------
c
c ****** If IFETA_DYN.le.0, the local resistivity is set to zero.
c
      if (ifeta_dyn.le.0) then
        if (ifirst.ne.0) then
          call bcast (etacel,0.,nrm1*ntm1)
          ecmin=0.
          ecmax=0.
          ifirst=0
        end if
        return
      end if
c
c ****** Compute the resistivity according to the value of
c ****** the current.
c
      if (ifeta_dyn.eq.1) then
c
c ****** Flat-top profile.
c
        do i=1,nrm1
          do j=1,ntm1
            fjsqmax=0.
            do k=1,np
              avjr=AVGT (fjr,i  ,j+1,k)
              avjt=AVGR (fjt,i+1,j  ,k)
              avjp=AVGRT(fjp,i+1,j+1,k)
              fjsq=(avjr**2+avjt**2+avjp**2)
              fjsqmax=max(fjsq,fjsqmax)
            enddo
            fjval=sqrt(fjsqmax)
            etacel(i,j)=eta_min+(eta_max-eta_min)*
     &                  .5*(1.+tanh((fjval-eta_jmin)/eta_dj))
          enddo
        enddo
c
      else if (ifeta_dyn.eq.2) then
c
c ****** Profile with eta proportional to J (beyond a threshold).
c
        do i=1,nrm1
          do j=1,ntm1
            fjsqmax=0.
            do k=1,np
              avjr=AVGT (fjr,i  ,j+1,k)
              avjt=AVGR (fjt,i+1,j  ,k)
              avjp=AVGRT(fjp,i+1,j+1,k)
              fjsq=(avjr**2+avjt**2+avjp**2)
              fjsqmax=max(fjsq,fjsqmax)
            enddo
            fjval=sqrt(fjsqmax)
            etacel(i,j)=eta_min+(eta_max-eta_min)*
     &                  (fjval/eta_jmin)*eta_jslope*
     &                  .5*(1.+tanh((fjval-eta_jmin)/eta_dj))
          enddo
        enddo
c
      else
c
c ****** Compute resistivity based on maximum current for 
c ****** the mesh and B in (r,theta).  Floor value for bsqmax
c ****** is allowed for null points or weak fields.
c
        do i=1,nrm1
          sint(1)=dt(1)
          sint(ntm1)=dt(ntm1)
          do j=2,ntm2
            sint(j)=st(j)
          enddo
          do j=1,ntm1
            fjsqmax=0.
            bsqmax=max(1e-20,bsqfloor)
            delmin = min(dr(i),r(i)*dt(j),r(i)*sint(j)*dphi)
            do k=1,np
              avbr=AVGT (br,i+1,j  ,k)
              avbt=AVGR (bt,i  ,j+1,k)
              avbp=AVG (bp,i,j,k)
              bsq=(avbr**2+avbt**2+avbp**2)
              bsqmax=max(bsq,bsqmax)
c
              avjr=AVGT (fjr,i  ,j+1,k)
              avjt=AVGR (fjt,i+1,j  ,k)
              avjp=AVGRT(fjp,i+1,j+1,k)
              fjsq=(avjr**2+avjt**2+avjp**2)
              fjsqmax=max(fjsq,fjsqmax)
            enddo
            bmag=sqrt(bsqmax)
            fjval=sqrt(fjsqmax)
            fjthresh=fjfrac*(2*bmag/delmin)
            etacel(i,j)=eta_min+(eta_max-eta_min)*
     &                  (fjval/fjthresh)*eta_jslope*
     &                  .5*( 1.+tanh( (fjval-fjthresh)
     &                  /(eta_dj*fjthresh) ) )
          enddo
        enddo
c
      end if
c
c ****** Find the minimum amd maximum cell resistivity
c ****** as a diagnostic.
c
      ecmax=0.
      ecmin=1.e20
      do i=1,nrm1
        do j=1,ntm1
          ecmin=min(ecmin,etacel(i,j))
          ecmax=max(ecmax,etacel(i,j))
        enddo
      enddo
c
      return
      end
c#######################################################################
      function flint (x,n,xn,fn,icheck,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a function linearly.
c
c-----------------------------------------------------------------------
c
c ****** The funcion is defined at N nodes, with values given by
c ****** FN(N) at positions XN(N).  The function value returned is
c ****** the linear interpolant at X.
c
c ****** Note that if X.lt.XN(1), the function value returned
c ****** is FN(1), and if X.gt.XN(N), the function value returned
c ****** is FN(N).
c
c ****** Call once with ICHECK.ne.0 to check that the values
c ****** in XN(N) are monotonically increasing.  In this mode
c ****** the array XN(N) is checked, and X and FN(N) are not
c ****** accessed.  If the check is passed, IERR=0 is returned.
c ****** Otherwise, IERR=1 is returned.
c
c-----------------------------------------------------------------------
c
      real xn(n),fn(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
      flint=0.
c
c ****** If ICHECK.ne.0, check the abscissa table.
c
      if (icheck.ne.0) then
        if (n.le.0) then
          write (*,*)
          write (*,*) '### ERROR in FLINT:'
          write (*,*) '### Bad dimension of abscissa table.'
          write (*,*) 'N = ',n
          write (9,*)
          write (9,*) '### ERROR in FLINT:'
          write (9,*) '### Bad dimension of abscissa table.'
          write (9,*) 'N = ',n
          ierr=1
          return
        end if
        do 100 i=1,n-1
          if (xn(i+1).le.xn(i)) then
            write (*,*)
            write (*,*) '### ERROR in FLINT:'
            write (*,*) '### Bad data in abscissa table.'
            write (*,*) 'N = ',n
            write (*,*) 'XN = '
            write (*,*) (xn(j),j=1,n)
            write (9,*)
            write (9,*) '### ERROR in FLINT:'
            write (9,*) '### Bad data in abscissa table.'
            write (9,*) 'N = ',n
            write (9,*) 'XN = '
            write (9,*) (xn(j),j=1,n)
            ierr=1
            return
          end if
  100   continue
        return
      end if
c
c ****** Get the interpolated value.
c
      if (x.le.xn(1)) then
        flint=fn(1)
      else if (x.gt.xn(n)) then
        flint=fn(n)
      else
        do 200 i=1,n-1
          if (x.ge.xn(i).and.x.lt.xn(i+1)) go to 300
  200   continue
  300   continue
        x1=xn(i)
        x2=xn(i+1)
        alpha=(x-x1)/(x2-x1)
        flint=fn(i)*(1.-alpha)+fn(i+1)*alpha
      end if
c
      return
      end
c#######################################################################
      subroutine setmesh
c
c-----------------------------------------------------------------------
c
c ****** Set up the mesh.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (ifrsin.eq.0) then
        r0=1.
        r1=r0+rl
      else
        r0=r(1)
        r1=r(nrm1)
        rl=r1-r0
      end if
c
      t0=0.
      t1=pi
      tl=t1-t0
c
      phi0=0.
      phi1=2.*pi
      phil=phi1-phi0
c
c ****** Set up the radial mesh in array R.
c
      if (ifrsin.eq.0) then
        call genmesh ('r',nrm1,r0,r1,nmseg,rfrac,drratio,nfrmesh,r)
      end if
c
      do i=2,nrm1
        rh(i)=.5*(r(i)+r(i-1))
        drh(i)=r(i)-r(i-1)
      enddo
      rh(1)=rh(2)-drh(2)
      rh(nr)=rh(nrm1)+drh(nrm1)
      do i=1,nrm1
        dr(i)=rh(i+1)-rh(i)
        dri(i)=1./dr(i)
      enddo
c
c ****** Set up the theta mesh in array T.
c
      if (ifrsin.eq.0) then
        call genmesh ('t',ntm1,t0,t1,nmseg,tfrac,dtratio,nftmesh,t)
      end if
c
      do j=2,ntm1
        th(j)=.5*(t(j)+t(j-1))
        dth(j)=t(j)-t(j-1)
      enddo
      th(1)=th(2)-dth(2)
      th(nt)=th(ntm1)+dth(ntm1)
      do j=1,ntm1
        dt(j)=th(j+1)-th(j)
        dti(j)=1./dt(j)
      enddo
      do j=1,ntm1
        st(j)=sin(t(j))
        ct(j)=cos(t(j))
      enddo
      do j=1,nt
        sth(j)=sin(th(j))
        cth(j)=cos(th(j))
      enddo
c
      dphi=phil/np
      phi(1)=phi0
      do k=2,np
        phi(k)=phi0+(k-1)*dphi
      enddo
      phi(npp1)=phi1
c
      do k=1,np
        sp(k)=sin(phi(k))
        cp(k)=cos(phi(k))
      enddo
c
c ****** Set up arrays FM and FM2 for use in taking
c ****** derivatives in phi.
c
      if (fdphi) then
c
c ****** Mimic finite differences in phi.
c
        do k=1,npd3
          m=k-1
          fm(k)=sin(m*dphi)/dphi
          fm2(k)=(sin(.5*m*dphi)/(.5*dphi))**2
        enddo
c
      else
c
c ****** Use Fourier derivatives in phi.
c
        do k=1,npd3
          m=k-1
          fm(k)=m
          fm2(k)=m**2
        enddo
c
      end if
c
      return
      end
c#######################################################################
      subroutine genmesh (label,nc,c0,c1,nseg,frac,dratio,nfilt,c)
c
c-----------------------------------------------------------------------
c
c ****** Generate a 1-dimensional mesh.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          LABEL   : [character*(*)]
c                    Name for the mesh coordinate.
c
c          NC      : [integer]
c                    Number of mesh points to load.
c
c          C0      : [real]
c                    The starting location for the coordinate.
c
c          C1      : [real]
c                    The ending location for the coordinate.
c
c          NSEG    : [integer]
c                    Maximum number of mesh segments.
c                    The mesh spacing in each segment varies
c                    exponentially with a uniform amplification
c                    factor.  The actual number of mesh segments
c                    used is NSEG or less.  It is obtained from the
c                    information in array FRAC.
c
c          FRAC    : [real array, dimension NSEG]
c                    The normalized positions of the mesh segment
c                    boundaries (as a fraction of the size of the
c                    domain).  The last value of FRAC specified
c                    must equal (or exceed) 1.
c
c          DRATIO  : [real array, dimension NSEG]
c                    The ratio of the mesh spacing at the end of a
c                    segment to that at the beginning.
c
c          NFILT   : [integer]
c                    The number of times to filter the mesh point
c                    distribution array.  Set NFILT=0 if filtering
c                    is not desired.
c                    Filtering can help to make the derivative
c                    of the mesh spacing continuous.
c
c ****** Output arguments:
c
c          C       : [real array, dimension NC]
c                    The locations of the mesh points.
c
c-----------------------------------------------------------------------
c
c ****** Note that the largest number of segments specified
c ****** must not exceed the internal parameter MXSEG.
c
c-----------------------------------------------------------------------
c
c ****** The arrays DRATIO and FRAC define the mesh as follows.
c
c ****** For example, suppose that a mesh with three segments is
c ****** desired.  Suppose the domain size is c=[0:2].
c ****** In the first segment (with c between 0 and .5) the mesh
c ****** spacing is decreasing with c, such that DC at c=.5 is half
c ****** DC at c=0.  From c=.5 to c=1, the mesh is uniform.  From c=1
c ****** to c=2, the mesh spacing is increasing with c such that DC at
c ****** c=2 is 10 times DC at c=1.  This mesh would be specified by:
c ******
c ******     FRAC=.25,.5,1.
c ******     DRATIO=.5,1.,10.
c
c-----------------------------------------------------------------------
c
      character*(*) label
      real c(nc),frac(nseg),dratio(nseg)
c
c-----------------------------------------------------------------------
c
c ****** Storage for the coordinate transformation.
c
      parameter (mxseg=100)
      real val(mxseg+1),fval(mxseg+1),chi(mxseg+1),rat(mxseg),
     &     bval(mxseg)
c
c-----------------------------------------------------------------------
c
c ****** Find the number of segments specified.
c
      do 100 i=1,nseg
        if (frac(i).ge.1.) go to 200
  100 continue
c
      write (*,*)
      write (*,*) '### ERROR in GENMESH:'
      write (*,*) '### Invalid mesh specification.'
      write (*,*) 'Mesh coordinate: ',label
      write (*,*) 'Last value of FRAC must equal (or exceed) 1.'
      write (9,*)
      write (9,*) '### ERROR in GENMESH:'
      write (9,*) '### Invalid mesh specification.'
      write (9,*) 'Mesh coordinate: ',label
      write (9,*) 'Last value of FRAC must equal (or exceed) 1.'
      call exit (1)
c
  200 continue
      n=i
      frac(n)=1.
c
c ****** Check that the specified values of FRAC are monotonically
c ****** increasing.
c
      do 300 i=2,n
        if (frac(i).lt.frac(i-1)) then
          write (*,*)
          write (*,*) '### ERROR in GENMESH:'
          write (*,*) '### Invalid mesh specification.'
          write (*,*) 'Mesh coordinate: ',label
          write (*,*) 'Values in FRAC must increase monotonically.'
          write (*,*) 'FRAC = ',(frac(j),j=1,n)
          write (9,*)
          write (9,*) '### ERROR in GENMESH:'
          write (9,*) '### Invalid mesh specification.'
          write (9,*) 'Mesh coordinate: ',label
          write (9,*) 'Values in FRAC must increase monotonically.'
          write (9,*) 'FRAC = ',(frac(j),j=1,n)
          call exit (1)
        end if
  300 continue
c
c ****** Check the number of segments.
c
      if (n.gt.mxseg) then
        write (*,*)
        write (*,*) '### ERROR in GENMESH:'
        write (*,*) '### Too many segments specified.'
        write (*,*) 'Mesh coordinate: ',label
        write (*,*) 'Number of segments = ',n
        write (*,*) 'MXSEG              = ',mxseg
        write (9,*)
        write (9,*) '### ERROR in GENMESH:'
        write (9,*) '### Too many segments specified.'
        write (9,*) 'Mesh coordinate: ',label
        write (9,*) 'Number of segments = ',n
        write (9,*) 'MXSEG              = ',mxseg
        call exit (1)
      end if
c
c ****** Set up the limits of the segments.
c
      val(1)=c0
      do 400 i=2,n+1
        val(i)=c0+(c1-c0)*frac(i-1)
  400 continue
c
c ****** Find proportional (chi) limits of segments.
c
      sum=0.
      fac=1.
      do 500 i=1,n
        if (abs(dratio(i)-1.).lt.1.e-6) then
          rat(i)=1.
        else
          rat(i)=alog(dratio(i))/(dratio(i)-1.)
        end if
        sum=sum+rat(i)*(val(i+1)-val(i))/fac
        fac=fac*dratio(i)
  500 continue
c
      chi(1)=0.
      fval(1)=sum
      do 600 i=1,n
        dchi=rat(i)*(val(i+1)-val(i))/fval(i)
        chi(i+1)=chi(i)+dchi
        fval(i+1)=fval(i)*dratio(i)
        bval(i)=(fval(i+1)-fval(i))/(val(i+1)-val(i))
  600 continue
c
c ****** Set up the mesh in array C.
c
      dch=1./(nc-1.)
      do 700 j=2,nc-1
        ch=(j-1)*dch
        i=intchi(ch,chi,n)
        if (abs(bval(i)).lt.1.e-6) then
          c(j)=val(i)+fval(i)*(ch-chi(i))
        else
          c(j)=val(i)+(fval(i)/bval(i))*(exp(bval(i)*(ch-chi(i)))-1.)
        end if
  700 continue
      c(1)=c0
      c(nc)=c1
c
c ****** Filter the mesh if requested.
c
      if (nfilt.gt.0) then
        do i=1,nfilt
          call filter (nc,c)
        enddo
      end if
c
c ****** Write out the mesh information.
c
      write (9,*)
      write (9,*) '### COMMENT from GENMESH:'
      write (9,*) '### Mesh information for coordinate ',label,':'
      write (9,*) 'Number of mesh segments = ',n
c
      write (9,'(/,1x,2a,/)') 'Segment     chi         ',label
      write (9,'(5x,f12.4,f11.4)') chi(1),val(1)
      do 800 i=2,n+1
        write (9,'(1x,i4,f12.4,f11.4)') i-1,chi(i),val(i)
  800 continue
c
      write (9,'(/,1x,a,/)') 'Mesh point location:'
      write (9,'(1x,9a,/)') '   i       ',label,'          d',label,
     &                      '     d',label,'(i)/d',label,'(i-1)'
c
      write (9,910) 1,c(1)
      write (9,910) 2,c(2),c(2)-c(1)
      do 900 j=3,nc
        dc=c(j)-c(j-1)
        ratio=dc/(c(j-1)-c(j-2))
        write (9,910) j,c(j),dc,ratio
  900 continue
  910 format (1x,i4,f11.5,f11.5,f13.5)
c
      write (9,*)
c
      return
      end
c#######################################################################
      function intchi (ch,chi,n)
c
c-----------------------------------------------------------------------
c
      real chi(n+1)
c
c-----------------------------------------------------------------------
c
c ****** Find CHI segment to which CH belongs.
c
      do 100 i=1,n
      if (ch.ge.chi(i).and.ch.le.chi(i+1)) go to 200
  100 continue
c
      write (*,*)
      write (*,*) '### ERROR in INTCHI:'
      write (*,*) '### Error in finding chi segment.'
      write (*,*) '[This is an internal error.]'
      write (9,*)
      write (9,*) '### ERROR in INTCHI:'
      write (9,*) '### Error in finding chi segment.'
      write (9,*) '[This is an internal error.]'
      call exit (1)
c
  200 continue
c
      intchi=i
c
      return
      end
c#######################################################################
      subroutine filter (n,f)
c
c-----------------------------------------------------------------------
c
c ****** Apply a "(1,2,1,)/4" digital filter to F.
c
c-----------------------------------------------------------------------
c
      parameter (u=1.)
c
c-----------------------------------------------------------------------
c
      call filt1d (1,n,u,f)
c
      return
      end
c#######################################################################
      subroutine filt1d (ldim,n,u,f)
c
c-----------------------------------------------------------------------
c
c ****** Utility routine to apply a 1D digital filter.
c
c-----------------------------------------------------------------------
c
      real f(ldim,*)
c
c-----------------------------------------------------------------------
c
      xnorm=1./(1.+u)**2
c
c ****** Save the boundary points.
c
      f0=f(1,1)
      f1=f(1,n)
c
c ****** Forward pass.
c
      do i=1,n-1
        f(1,i)=f(1,i)+u*f(1,i+1)
      enddo
c
c ****** Backward pass.
c
      do i=n,2,-1
        f(1,i)=f(1,i)+u*f(1,i-1)
      enddo
c
c ****** Normalization.
c
      do i=2,n
        f(1,i)=f(1,i)*xnorm
      enddo
c
c ****** Reset the boundary points.
c
      f(1,1)=f0
      f(1,n)=f1
c
      return
      end
c#######################################################################
      subroutine initial
c
c-----------------------------------------------------------------------
c
c ****** Set up the initial equilibrium according to OPTION.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      logical eqload,fldload
c
c-----------------------------------------------------------------------
c
      iftest=0
      eqload=.false.
      fldload=.false.
c
c ****** Load the initial state based on the switch OPTION.
c
      if (option.eq.'wave1') then
        iftest=1
        call wave1
      else if (option.eq.'wave2') then
        iftest=1
        call wave2
      else if (option.eq.'streamer') then
        eqload=.true.
        fldload=.true.
      else if (option.eq.'dipole') then
        eqload=.true.
        fldload=.true.
      else
        write (*,*)
        write (*,*) '### ERROR in INITIAL:'
        write (*,*) '### Invalid option specified.'
        write (*,*) 'OPTION = ',option
        write (9,*)
        write (9,*) '### ERROR in INITIAL:'
        write (9,*) '### Invalid option specified.'
        write (9,*) 'OPTION = ',option
        call exit (1)
      end if
c 
c ****** Load the initial equilibrium.
c
      if (eqload) then
        if (eqtype.eq.'hydrostatic') then
          call loadhseq
        else if (eqtype.eq.'zero-beta') then
          call loadzb
        else if (eqtype.eq.'parker') then
          call load1d (onedfile)
        else
          write (*,*)
          write (*,*) '### ERROR in INITIAL:'
          write (*,*) '### Invalid equilibrium type specified.'
          write (*,*) 'EQTYPE = ',eqtype
          write (9,*)
          write (9,*) '### ERROR in INITIAL:'
          write (9,*) '### Invalid equilibrium type specified.'
          write (9,*) 'EQTYPE = ',eqtype
          call exit (1)
        end if
      end if
c 
c ****** Load the initial magnetic field.
c
      if (fldload) then
        if (fldtype.eq.'dipole') then
          call loaddip
        else if (fldtype.eq.'potential') then
          call potfld (bnfile)
        else
          write (*,*)
          write (*,*) '### ERROR in INITIAL:'
          write (*,*) '### Invalid magnetic field type specified.'
          write (*,*) 'FLDTYPE = ',fldtype
          write (9,*)
          write (9,*) '### ERROR in INITIAL:'
          write (9,*) '### Invalid magnetic field type specified.'
          write (9,*) 'FLDTYPE = ',fldtype
          call exit (1)
        end if
      end if
c
      return
      end
c#######################################################################
       subroutine load1d (fname)
c
c-----------------------------------------------------------------------
c
c ****** Load in a 1D solution for the gas variables.
c
c-----------------------------------------------------------------------
c
c ****** Values for vr, rho, and p are read from file FNAME.
c ****** All other variables are set to zero.
c
c ****** NP1D is the number of radial points in the file.
c
c ****** Typical examples are a Parker wind solution or a hydrostatic
c ****** equilibrium.
c
c-----------------------------------------------------------------------
c
c ****** Note that the main real arrays for v, rho, and p are set
c ****** by this routine.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) fname
c
c-----------------------------------------------------------------------
c
      parameter (nptsmx=2501)
      real rt(nptsmx),rhot(nptsmx),vrt(nptsmx),
     &     pt(nptsmx),pwt(nptsmx)
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (np1d.gt.nptsmx) then
        write (*,*)
        write (*,*) '### ERROR in LOAD1D:'
        write (*,*) '### Could not read the initial state.'
        write (*,*) 'The number of points is too large.'
        write (*,*) 'NP1D   = ',np1d
        write (*,*) 'NPTSMX = ',nptsmx
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in LOAD1D:'
        write (9,*) '### Could not read the initial state.'
        write (9,*) 'The number of points is too large.'
        write (9,*) 'NP1D   = ',np1d
        write (9,*) 'NPTSMX = ',nptsmx
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** Read the initial state.
c
      call ffopen (20,fname,'r')
c
c ****** Skip the header.
c
      read (20,*,err=900,end=900)
c
      do i=1,np1d
        read (20,*,err=900,end=900) rt(i),vrt(i),rhot(i),pt(i),pwt(i)
      enddo
c
      close (20)
c
c ****** Note that the input file contains the (normalized)
c ****** electron number density.
c ****** Convert the electron density to mass density RHO.
c
      do i=1,np1d
        rhot(i)=he_rho*rhot(i)
      enddo
c
c ****** Re-scale p and rho to give a base density of
c ****** RHOR0.
c
      call interp (rt,np1d,r0,ir,alpha)
      irp1=min0(ir+1,np1d)
      rho00=(1.-alpha)*rhot(ir)+alpha*rhot(irp1)
c
      fac=rhor0/rho00
c
      do i=1,np1d
        rhot(i)=fac*rhot(i)
        pt(i)=fac*pt(i)
      enddo
c
c ****** Interpolate to the code mesh.
c
      do i=1,nrm1
        call interp (rt,np1d,r(i),ir,alpha)
        irp1=min0(ir+1,np1d)
        do k=1,np
          do j=1,nt
            vr(i,j,k)=(1.-alpha)*vrt(ir)+alpha*vrt(irp1)
          enddo
        enddo
      enddo
c
      do i=2,nrm1
        call interp (rt,np1d,rh(i),ir,alpha)
        irp1=min0(ir+1,np1d)
        do k=1,np
          do j=2,ntm1
            rho(i,j,k)=(1.-alpha)*rhot(ir)+alpha*rhot(irp1)
            p  (i,j,k)=(1.-alpha)*pt  (ir)+alpha*pt  (irp1)
            ep (i,j,k)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
            em (i,j,k)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
          enddo
        enddo
      enddo
c
c ****** Set boundary values.
c
      call bcast (rhobcr0,0.,2*nt*npd3)
      call bcast (pbcr0  ,0.,2*nt*npd3)
      call bcast (epbcr0 ,0.,2*nt*npd3)
      call bcast (embcr0 ,0.,2*nt*npd3)
      call bcast (vrbcr0 ,0.,2*nt*npd3)
      call bcast (vtbcr0 ,0.,2*nt*npd3)
      call bcast (vpbcr0 ,0.,2*nt*npd3)
c
      call bcast (rhobcr1,0.,2*nt*npd3)
      call bcast (pbcr1  ,0.,2*nt*npd3)
      call bcast (epbcr1 ,0.,2*nt*npd3)
      call bcast (embcr1 ,0.,2*nt*npd3)
      call bcast (vrbcr1 ,0.,2*nt*npd3)
      call bcast (vtbcr1 ,0.,2*nt*npd3)
      call bcast (vpbcr1 ,0.,2*nt*npd3)
c
      call interp (rt,np1d,r(1),ir,alpha)
      irp1=min0(ir+1,np1d)
      do j=1,nt
        rhobcr0(j,1,1)=(1.-alpha)*rhot(ir)+alpha*rhot(irp1)
        pbcr0  (j,1,1)=(1.-alpha)*pt  (ir)+alpha*pt  (irp1)
        epbcr0 (j,1,1)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
        embcr0 (j,1,1)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
      enddo
c
      do k=1,np
        do j=1,nt
          rhobcr00(j,k)=rhobcr0(j,1,1)
        enddo
      enddo
c
      call interp (rt,np1d,r(nrm1),ir,alpha)
      irp1=min0(ir+1,np1d)
      do j=1,nt
        rhobcr1(j,1,1)=(1.-alpha)*rhot(ir)+alpha*rhot(irp1)
        pbcr1  (j,1,1)=(1.-alpha)*pt  (ir)+alpha*pt  (irp1)
        epbcr1 (j,1,1)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
        embcr1 (j,1,1)=(1.-alpha)*pwt (ir)+alpha*pwt (irp1)
      enddo
c
c ****** Put the fields into the main complex arrays.
c
      call fft (vr ,vrr,vri,1)
      call fft (rho,rhr,rhi,1)
      call fft (p  ,prr,pri,1)
      call fft (ep ,epr,epi,1)
      call fft (em ,emr,emi,1)
c
      call setbcrho (rhr,rhi)
      call setbcp (prr,pri)
      call setbcep (epr,epi)
      call setbcem (emr,emi)
c
      write (9,*)
      write (9,*) '### COMMENT from LOAD1D:'
      write (9,*) '### A 1D equilibrium was read in.'
      write (9,*) 'File name: ',fname(1:lf)
      write (9,*)
      write (9,*) 'Base density, RHOR0 = ',rhor0
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in LOAD1D:'
      write (*,*) '### Error while reading data.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) 'Line number: ',i
      write (9,*)
      write (9,*) '### ERROR in LOAD1D:'
      write (9,*) '### Error while reading data.'
      write (9,*) 'File name: ',fname(1:lf)
      write (9,*) 'Line number: ',i
      call exit (1)
c
      end
c#######################################################################
      subroutine interp (x,n,xv,i,alpha)
c
c-----------------------------------------------------------------------
c
c ****** Get interpolation factor ALPHA and table index I.
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I and ALPHA to get the interpolated value.
c
c-----------------------------------------------------------------------
c
      real x(n)
c
c-----------------------------------------------------------------------
c
      do 100 i=1,n-1
        if (xv.ge.x(i).and.xv.le.x(i+1)) then
          alpha=(xv-x(i))/(x(i+1)-x(i))
          go to 200
        end if
  100 continue
c
c ****** Value not found --- signal error and stop.
c
      write (*,*)
      write (*,*) '### ERROR in INTERP:'
      write (*,*) '### Value not found in table.'
      write (*,*) 'Value requested = ',xv
      write (*,*) 'Min table value = ',x(1)
      write (*,*) 'Max table value = ',x(n)
      write (9,*)
      write (9,*) '### ERROR in INTERP:'
      write (9,*) '### Value not found in table.'
      write (9,*) 'Value requested = ',xv
      write (9,*) 'Min table value = ',x(1)
      write (9,*) 'Max table value = ',x(n)
      call exit (1)
c
  200 continue
c
      return
      end
c#######################################################################
       subroutine loadhseq
c
c-----------------------------------------------------------------------
c
c ****** Load in a 1D hydrostatic equilibrium.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Load a hydrostatic equilibrium.
c
      cc=pr0/rhor0**gamma
      fac=1./(gamma-1.)
c
      do 120 k=1,np
      do 110 j=1,nt
      do 100 i=2,nrm1
        rho(i,j,k)=(((gamma-1.)/(cc*gamma))*g0*(1./rh(i)-1.)
     &             +rhor0**(gamma-1.))**fac
        p  (i,j,k)=cc*rho(i,j,k)**gamma
  100 continue
  110 continue
  120 continue
c
c ****** Compute the values at the outer radial boundary.
c
      rhor1=(((gamma-1.)/(cc*gamma))*g0*(1./r(nrm1)-1.)
     &       +rhor0**(gamma-1.))**fac
      pr1=cc*rhor1**gamma
c
c ****** Set boundary values.
c
      call bcast (rhobcr0,0.,2*nt*npd3)
      call bcast (pbcr0  ,0.,2*nt*npd3)
      call bcast (epbcr0 ,0.,2*nt*npd3)
      call bcast (embcr0 ,0.,2*nt*npd3)
      call bcast (vrbcr0 ,0.,2*nt*npd3)
      call bcast (vtbcr0 ,0.,2*nt*npd3)
      call bcast (vpbcr0 ,0.,2*nt*npd3)
c
      call bcast (rhobcr1,0.,2*nt*npd3)
      call bcast (pbcr1  ,0.,2*nt*npd3)
      call bcast (epbcr1 ,0.,2*nt*npd3)
      call bcast (embcr1 ,0.,2*nt*npd3)
      call bcast (vrbcr1 ,0.,2*nt*npd3)
      call bcast (vtbcr1 ,0.,2*nt*npd3)
      call bcast (vpbcr1 ,0.,2*nt*npd3)
c
      do j=1,nt
        rhobcr0(j,1,1)=rhor0
        pbcr0  (j,1,1)=pr0
        rhobcr1(j,1,1)=rhor1
        pbcr1  (j,1,1)=pr1
      enddo
c
      do k=1,np
        do j=1,nt
          rhobcr00(j,k)=rhor0
        enddo
      enddo
c
c ****** Put the fields into the main complex arrays.
c
      call fft (rho,rhr,rhi,1)
      call fft (p  ,prr,pri,1)
      call fft (ep ,epr,epi,1)
      call fft (em ,emr,emi,1)
c
      call setbcrho (rhr,rhi)
      call setbcp (prr,pri)
      call setbcep (epr,epi)
      call setbcem (emr,emi)
c
      write (9,*)
      write (9,*) '### COMMENT from LOADHSEQ:'
      write (9,*) '### A hydrostatic equilibrium was loaded.'
      write (9,*) 'GAMMA  = ',gamma
      write (9,*) 'RHOR0 = ',rhor0
      write (9,*) 'PR0   = ',pr0
      write (9,*) 'G0     = ',g0
c
      return
      end
c#######################################################################
      subroutine loadzb
c
c-----------------------------------------------------------------------
c
c ****** Load in a zero-beta equilibrium.
c
c ****** This routine loads an equilibrium with zero pressure,
c ****** zero gravity, and a specified density profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Load a special density profile.
c
      do k=1,np
        do i=1,nr
          do j=1,nt
            rho(i,j,k)=rhor0*(r0/rh(i))**4
          enddo
        enddo
      enddo
c
      rhor1=rhor0*(r0/r1)**4
c
ccc      do k=1,np
ccc        do j=1,nt
ccc          do i=2,nrm1
ccc            ell=rhofac1+rhofac2*rh(i)
ccc            rho(i,j,k)=rhor0*exp(-(rh(i)-1.)/ell)
ccc          enddo
ccc        enddo
ccc      enddo
c
c ****** Compute the values at the outer radial boundary.
c
ccc      ell=rhofac1+rhofac2*r(nrm1)
ccc      rhor1=rhor0*exp(-(r(nrm1)-1.)/ell)
c
c ****** Set boundary values.
c
      call bcast (rhobcr0,0.,2*nt*npd3)
      call bcast (pbcr0  ,0.,2*nt*npd3)
      call bcast (epbcr0 ,0.,2*nt*npd3)
      call bcast (embcr0 ,0.,2*nt*npd3)
      call bcast (vrbcr0 ,0.,2*nt*npd3)
      call bcast (vtbcr0 ,0.,2*nt*npd3)
      call bcast (vpbcr0 ,0.,2*nt*npd3)
c
      call bcast (rhobcr1,0.,2*nt*npd3)
      call bcast (pbcr1  ,0.,2*nt*npd3)
      call bcast (epbcr1 ,0.,2*nt*npd3)
      call bcast (embcr1 ,0.,2*nt*npd3)
      call bcast (vrbcr1 ,0.,2*nt*npd3)
      call bcast (vtbcr1 ,0.,2*nt*npd3)
      call bcast (vpbcr1 ,0.,2*nt*npd3)
c
      do j=1,nt
        rhobcr0(j,1,1)=rhor0
        rhobcr1(j,1,1)=rhor1
      enddo
c
      do k=1,np
        do j=1,nt
          rhobcr00(j,k)=rhor0
        enddo
      enddo
c
      g0=0.
c
c ****** Put the density into the main complex arrays.
c
      call fft (rho,rhr,rhi,1)
c
      call setbcrho (rhr,rhi)
c
      write (9,*)
      write (9,*) '### COMMENT from LOADZB:'
      write (9,*) '### A zero beta equilibrium was loaded.'
      write (9,*) 'RHOR0 = ',rhor0
c
      return
      end
c#######################################################################
       subroutine loaddip
c
c-----------------------------------------------------------------------
c
c ****** Load in the vector potential for a dipole field.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real atheta(nr,nt,np),aphi(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Compute A in real space.
c ****** Note that this sets boundary points next to the poles.
c
c ****** The variable DIPANGLE is the angle (in degrees) by
c ****** which the dipole is tilted from the z-axis in the x-z
c ****** plane.
c
      sd=sin(dipangle*pi/180.)
      cd=cos(dipangle*pi/180.)
c
      call bcast (atheta,0.,nrtp)
      call bcast (aphi  ,0.,nrtp)
c
      do k=1,np
        do j=1,ntm1
          do i=1,nr
            atheta(i,j,k)=-b0*sin(phi(k))*sd/rh(i)**2
         enddo
        enddo
      enddo
c
      do k=1,np
        do j=1,nt
          do i=1,nr
            aphi(i,j,k)=b0*(sth(j)*cd-cth(j)*cos(phi(k))*sd)/rh(i)**2
         enddo
        enddo
      enddo
c
c ****** Put the field into the main complex arrays.
c
      call fft (atheta,atr,ati,1)
      call fft (aphi  ,apr,api,1)
c
      write (9,*)
      write (9,*) '### COMMENT from LOADDIP:'
      write (9,*) '### A dipole field was loaded.'
      write (9,*) 'B0 = ',b0
      write (9,*) 'DIPANGLE = ',dipangle
c
      return
      end
c#######################################################################
      subroutine potfld (fname)
c
c-----------------------------------------------------------------------
c
c ****** Load a potential field corresponding to a given flux
c ****** distribution at r=R0.
c
c ****** The flux distribution is read in from file FNAME.
c
c-----------------------------------------------------------------------
c
      use solver_types
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom2
c
c-----------------------------------------------------------------------
c
      character*(*) fname
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions.
c
      common /potbcap/ apcr0(nt),apvr0(nt),
     &                 apcr1(nt),apvr1(nt),
     &                 apct0(nr),apvt0(nr),
     &                 apct1(nr),apvt1(nr)
c
      common /potbcat/ atcr0(ntm1),atvr0(ntm1),
     &                 atcr1(ntm1),atvr1(ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for the photospheric magnetic field,
c ****** boundary conditions at r=R0 on A, scratch arrays for
c ****** the tridiagonal solve, and scratch arrays for the
c ****** CG solve.
c
      real br0re(ntm1,np)
c
      real br0r(ntm1,npd3),br0i(ntm1,npd3),
     &     ap0r(nt),at0r(ntm1,npd3),at0i(ntm1,npd3),
     &     aa(ntm3),bb(ntm3),cc(ntm3),dd(ntm3)
c
      real xcgp(nr,nt),rhsp(nr,nt)
      real xcgt(nr,ntm1),rhst(nr,ntm1)
c
c-----------------------------------------------------------------------
c
      character*32 brofile
c
c-----------------------------------------------------------------------
c
c ****** Initialize the magnetic field to zero.
c
      call bcast (arr,0.,nc)
      call bcast (ari,0.,nc)
      call bcast (atr,0.,nc)
      call bcast (ati,0.,nc)
      call bcast (apr,0.,nc)
      call bcast (api,0.,nc)
c
c ****** Read the specified normal magnetic field at r=R0
c ****** from file FNAME into BR0RE.
c
      call readbr (fname,br0re)
c
c ****** Write the interpolated photospheric Br field
c ****** as a diagnostic.
c
      brofile='br_photo'
      call wr2d (brofile,ntm1,ntm1,np,br0re,t,phi)
c
      lf=lenstr(brofile)
c
      write (9,*)
      write (9,*) '### COMMENT from POTFLD:'
      write (9,*) 'The (interpolated) photospheric Br field'//
     &            ' has been written'
      write (9,*) 'to file: ',brofile(1:lf)
c
c ****** Get the Fourier transform of the photospheric
c ****** magnetic field.
c
      call fft_tp1 (ntm1,br0re,br0r,br0i,1)
c
c ****** Get the boundary conditions on the appropriate
c ****** components of the vector potential.
c
c ****** Ap for the m = 0 mode.
c
      ap0r(1)=-.25*r0*dt(1)*br0r(1,1)
      ap0r(2)=-ap0r(1)
      do j=2,ntm2
        ap0r(j+1)=(ap0r(j)*sth(j)+br0r(j,1)*r0*st(j)*dt(j))/sth(j+1)
      enddo
      ap0r(nt)=-ap0r(ntm1)
c
c ****** At for the m > 0 modes.
c
      do j=1,ntm1
        at0r(j,1)=0.
        at0i(j,1)=0.
      enddo
c
c ****** Solve for the boundary values of At at r=R0 for each
c ****** m-mode using a tridiagonal solve.
c
      do k=2,npd3
c
        m=k-1
c
c ****** Real part of At.
c
        do j=2,ntm2
          aa(j-1)=fm2(k)+st(j)*(sth(j+1)/dth(j+1)+sth(j)/dth(j))/dt(j)
          bb(j-1)=-st(j+1)*sth(j+1)/(dt(j)*dth(j+1))
          cc(j-1)=-st(j-1)*sth(j  )/(dt(j)*dth(j  ))
          dd(j-1)=-fm(k)*r0*st(j)*br0i(j,k)
        enddo
c
        call trid (ntm3,cc,aa,bb,dd)
c
        do j=2,ntm2
          at0r(j,k)=dd(j-1)
        enddo
c
        if (m.eq.1) then
          at0r(   1,k)=st(   2)*at0r(   2,k)/dth(   2)
          at0r(ntm1,k)=st(ntm2)*at0r(ntm2,k)/dth(ntm1)
        else
          at0r(   1,k)=0.
          at0r(ntm1,k)=0.
        end if
c
c ****** Imaginary part of At.
c
        do j=2,ntm2
          aa(j-1)=fm2(k)+st(j)*(sth(j+1)/dth(j+1)+sth(j)/dth(j))/dt(j)
          bb(j-1)=-st(j+1)*sth(j+1)/(dt(j)*dth(j+1))
          cc(j-1)=-st(j-1)*sth(j  )/(dt(j)*dth(j  ))
          dd(j-1)= fm(k)*r0*st(j)*br0r(j,k)
        enddo
c
        call trid (ntm3,cc,aa,bb,dd)
c
        do j=2,ntm2
          at0i(j,k)=dd(j-1)
        enddo
c
        if (m.eq.1) then
          at0i(   1,k)=st(   2)*at0i(   2,k)/dth(   2)
          at0i(ntm1,k)=st(ntm2)*at0i(ntm2,k)/dth(ntm1)
        else
          at0i(   1,k)=0.
          at0i(ntm1,k)=0.
        end if
c
      enddo
c
c ****** Solve the potential field equations, one m-mode at a time.
c
c ****** Set the preconditioning switch to the most
c ****** effective preconditioning.
c
c ****** Disabled: ZM, 2/10/99.
c
ccc      ifprech=ifprec
ccc      ifprec=2
c
      if (idebug.gt.0) then
        write (9,*)
        write (9,*) '### COMMENT from POTFLD:'
        write (9,*) '### CG solution of potential field equations:'
      end if
c
      solvtype='pot'
c
c ****** First, do the m = 0 mode.
c
      m=0
c
c ****** Set the boundary condition arrays for the solver.
c
      do i=1,nr
        apct0(i)=1.
        apvt0(i)=0.
        apct1(i)=1.
        apvt1(i)=0.
      enddo
      do j=2,ntm1
        apcr0(j)=rh(1)/rh(2)
        apvr0(j)=2.*ap0r(j)*rh(1)/r0
        apcr1(j)=1.
        apvr1(j)=0.
      enddo
c
c ****** Set the RHS to zero, and use a zero guess.
c
      xcgp(:,:)=0.
      rhsp(:,:)=0.
c
c ****** Solve the equation using CG.
c
c ****** Allocate memory for the matrix diagonals (Ap solve).
c
      allocate (a00(nr,nt))
      allocate (ap0(nr,nt))
      allocate (a0p(nr,nt))
c
c ****** Allocate memory for the preconditioning diagonal (Ap solve).
c
      allocate (d(nr,nt))
c
      solve_type=ST_AMATP0
c
      call amatp0 (rhsp)
      call solve (xcgp,rhsp,ierr)
c
c ****** Check for convergence.
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in POTFLD:'
        write (*,*) '### The field solver did not converge.'
        write (*,*) 'The error occurred while solving the'//
     &              ' m = 0 equation for Ap.'
        write (9,*)
        write (9,*) '### ERROR in POTFLD:'
        write (9,*) '### The field solver did not converge.'
        write (9,*) 'The error occurred while solving the'//
     &              ' m = 0 equation for Ap.'
        call exit (1)
      end if
c
      if (idebug.gt.0) then
        write (9,600) '### APR:',m,ncg,bnrm,rnrm
  600   format (/,1x,a,' M=',i3,' N=',i5,' |B|=',1pe8.2,
     &          ' |R|=',1pe8.2)
      end if
c
c ****** De-allocate memory for the matrix diagonals (Ap solve).
c
      deallocate (a00)
      deallocate (ap0)
      deallocate (a0p)
c
c ****** De-allocate memory for the preconditioning diagonal (Ap solve).
c
      deallocate (d)
c
c ****** Put the solution in the main A arrays.
c
      apr(2:nrm1,2:ntm1,1)=xcgp(2:nrm1,2:ntm1)
c
c ****** Set the boundary conditions.
c
      do j=2,ntm1
        apr( 1,j,1)=apvr0(j)-apcr0(j)*apr(   2,j,1)
        apr(nr,j,1)=apvr1(j)-apcr1(j)*apr(nrm1,j,1)
      enddo
      do i=1,nr
        apr(i, 1,1)=apvt0(i)-apct0(i)*apr(i,   2,1)
        apr(i,nt,1)=apvt1(i)-apct1(i)*apr(i,ntm1,1)
      enddo
c
c ****** Next, do the m > 0 modes.
c
c ****** Allocate memory for the matrix diagonals (At solve).
c
      allocate (a00(nr,ntm1))
      allocate (ap0(nr,ntm1))
      allocate (a0p(nr,ntm1))
c
c ****** Allocate memory for the preconditioning diagonal (At solve).
c
      allocate (d(nr,ntm1))
c
ccc      do k=2,npd3
      do k=2,npd3-1
c
        m=k-1
c
c ****** Real part.
c
c ****** Set the boundary condition arrays for the solver.
c ****** (Note that there is no coupling of At to boundary
c ****** values at the poles.)
c
        do j=2,ntm2
          atcr0(j)=rh(1)/rh(2)
          atvr0(j)=2.*at0r(j,k)*rh(1)/r0
          atcr1(j)=1.
          atvr1(j)=0.
        enddo
c
c ****** Set the RHS to zero, and use a zero guess.
c
        xcgt(:,:)=0.
        rhst(:,:)=0.
c
c ****** Solve the equation using CG.
c
        solve_type=ST_AMATT
c
        call amatt (m,rhst)
        call solve (xcgt,rhst,ierr)
c
c ****** Check for convergence.
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in POTFLD:'
          write (*,*) '### The field solver did not converge.'
          write (*,*) 'The error occurred while solving an'//
     &                ' m > 0 equation for real At.'
          write (*,*) 'M = ',m
          write (9,*)
          write (9,*) '### ERROR in POTFLD:'
          write (9,*) '### The field solver did not converge.'
          write (9,*) 'The error occurred while solving an'//
     &                ' m > 0 equation for real At.'
          write (9,*) 'M = ',m
          call exit (1)
        end if
c
        if (idebug.gt.0) then
          write (9,600) '### ATR:',m,ncg,bnrm,rnrm
        end if
c
c ****** Put the solution in the main A arrays.
c
        atr(2:nrm1,2:ntm2,k)=xcgt(2:nrm1,2:ntm2)
c
c ****** Set the boundary conditions.
c
        do j=2,ntm2
          atr( 1,j,k)=atvr0(j)-atcr0(j)*atr(   2,j,k)
          atr(nr,j,k)=atvr1(j)-atcr1(j)*atr(nrm1,j,k)
        enddo
        if (m.eq.1) then
          do i=1,nr
            atr(i,   1,k)=st(   2)*atr(i,   2,k)/dth(   2)
            atr(i,ntm1,k)=st(ntm2)*atr(i,ntm2,k)/dth(ntm1)
          enddo
        else
          do i=1,nr
            atr(i,   1,k)=0.
            atr(i,ntm1,k)=0.
          enddo
        end if
c
c ****** Set the dummy points to zero.
c
        do i=1,nr
          atr(i,nt,k)=0.
        enddo
c
c ****** Imaginary part.
c
c ****** Set the boundary condition arrays for the solver.
c ****** (Note that there is no coupling of At to boundary
c ****** values at the poles.)
c
        do j=2,ntm2
          atcr0(j)=rh(1)/rh(2)
          atvr0(j)=2.*at0i(j,k)*rh(1)/r0
          atcr1(j)=1.
          atvr1(j)=0.
        enddo
c
c ****** Set the RHS to zero, and use a zero guess.
c
        xcgt(:,:)=0.
        rhst(:,:)=0.
c
c ****** Solve the equation using CG.
c
        solve_type=ST_AMATT
c
        call amatt (m,rhst)
        call solve (xcgt,rhst,ierr)
c
c ****** Check for convergence.
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in POTFLD:'
          write (*,*) '### The field solver did not converge.'
          write (*,*) 'The error occurred while solving an'//
     &                ' m > 0 equation for imaginary At.'
          write (*,*) 'M = ',m
          write (9,*)
          write (9,*) '### ERROR in POTFLD:'
          write (9,*) '### The field solver did not converge.'
          write (9,*) 'The error occurred while solving an'//
     &                ' m > 0 equation for imaginary At.'
          write (9,*) 'M = ',m
          call exit (1)
        end if
c
        if (idebug.gt.0) then
          write (9,600) '### ATI:',m,ncg,bnrm,rnrm
        end if
c
c ****** Put the solution in the main A arrays.
c
        ati(2:nrm1,2:ntm2,k)=xcgt(2:nrm1,2:ntm2)
c
c ****** Set the boundary conditions.
c
        do j=2,ntm2
          ati( 1,j,k)=atvr0(j)-atcr0(j)*ati(   2,j,k)
          ati(nr,j,k)=atvr1(j)-atcr1(j)*ati(nrm1,j,k)
        enddo
        if (m.eq.1) then
          do i=1,nr
            ati(i,   1,k)=st(   2)*ati(i,   2,k)/dth(   2)
            ati(i,ntm1,k)=st(ntm2)*ati(i,ntm2,k)/dth(ntm1)
          enddo
        else
          do i=1,nr
            ati(i,   1,k)=0.
            ati(i,ntm1,k)=0.
          enddo
        end if
c
c ****** Set the dummy points to zero.
c
        do i=1,nr
          ati(i,nt,k)=0.
        enddo
c
      enddo
c
c ****** De-allocate memory for the matrix diagonals (At solve).
c
      deallocate (a00)
      deallocate (ap0)
      deallocate (a0p)
c
c ****** De-allocate memory for the preconditioning diagonal (At solve).
c
      deallocate (d)
c
c ****** Reset the preconditioning switch.
c
c ****** Disabled: ZM, 2/10/99.
c
ccc      ifprec=ifprech
c
c ****** Set Ap for m > 0 modes (from div A = 0).
c
      do k=2,npd3
c
        m=k-1
c
        if (m.eq.np/2) then
          do j=2,ntm1
            do i=1,nr
              apr(i,j,k)=0.
              api(i,j,k)=0.
            enddo
          enddo
        else
          do j=2,ntm1
            do i=1,nr
              apr(i,j,k)=-(st(j)*ati(i,j,k)-st(j-1)*ati(i,j-1,k))
     &                    /(fm(k)*dth(j))
              api(i,j,k)= (st(j)*atr(i,j,k)-st(j-1)*atr(i,j-1,k))
     &                    /(fm(k)*dth(j))
            enddo
          enddo
        end if
c
        if (m.eq.1) then
          do i=1,nr
            apr(i, 1,k)= apr(i,   2,k)
            apr(i,nt,k)= apr(i,ntm1,k)
            api(i, 1,k)= api(i,   2,k)
            api(i,nt,k)= api(i,ntm1,k)
          enddo
        else if (m.eq.2) then
          do i=1,nr
            apr(i, 1,k)=-apr(i,   2,k)
            apr(i,nt,k)=-apr(i,ntm1,k)
            api(i, 1,k)=-api(i,   2,k)
            api(i,nt,k)=-api(i,ntm1,k)
          enddo
        else
          do i=1,nr
            apr(i, 1,k)= apr(i,   2,k)
            apr(i,nt,k)= apr(i,ntm1,k)
            api(i, 1,k)= api(i,   2,k)
            api(i,nt,k)= api(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
      subroutine readbr (fname,br0re)
c
c-----------------------------------------------------------------------
c
c ****** Read in the radial magnetic field at the photosphere
c ****** and interpolate it into array BR0RE.
c
c ****** FNAME is the name of the file to read.
c
c-----------------------------------------------------------------------
c
      use rdtxt2d_arrays
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real br0re(ntm1,np)
c
c-----------------------------------------------------------------------
c
c ****** Br array and its scales.
c
      real,pointer:: bn(:,:),tbn(:),pbn(:)
c
c-----------------------------------------------------------------------
c
c ****** Read in the normal field.
c
c ****** The routine RDTXT2D communicates the arrays read in through
c ****** module RDTXT2D_ARRAYS.
c
      lf=lenstr(fname)
c
      write (9,*)
      write (9,*) '### COMMENT from READBR:'
      write (9,*) '### Reading Br file: ',fname(1:lf)
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
c
      call rdtxt2d (fname,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in READBR:'
        write (*,*) '### The flux file has the wrong format.'
        write (*,*) 'IERR (from RDTXT2D) = ',ierr
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in READBR:'
        write (9,*) '### The flux file has the wrong format.'
        write (9,*) 'IERR (from RDTXT2D) = ',ierr
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** Set the Br arrays to point to the data read in by RDTXT2D.
c
      bn=>f_rdtxt2d
      tbn=>x_rdtxt2d
      pbn=>y_rdtxt2d
c
c ****** Get the dimensions of the Br array read in.
c
      ntbn=size(tbn)
      npbn=size(pbn)
c
c ****** Interpolate the field to the code mesh (into array BR0RE).
c
      call intrp2d (ntbn,npbn,tbn,pbn,bn,ntm1,np,t,phi,br0re,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in READBR:'
        write (*,*) '### The scales in the Br file are invalid.'
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in READBR:'
        write (9,*) '### The scales in the Br file are invalid.'
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** De-allocate the memory for the RDTXT2D arrays.
c
      deallocate (f_rdtxt2d)
      deallocate (x_rdtxt2d)
      deallocate (y_rdtxt2d)
c
c ****** Find the maximum absolute value of the normal field.
c
      b0orig=b0
c
      bnmax=0.
      do k=1,np
        do j=1,ntm1
          bnmax=max(abs(br0re(j,k)),bnmax)
        enddo
      enddo
c
      if (bnmax.eq.0.) then
        write (*,*)
        write (*,*) '### ERROR in READBR:'
        write (*,*) '### Anomaly in the Br data.'
        write (*,*) 'Br is identically zero.'
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in READBR:'
        write (9,*) '### Anomaly in the Br data.'
        write (9,*) 'Br is identically zero.'
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** Normalize the field.
c
c ****** When BINGAUSS=.true., then it is assumed that the
c ****** input magnetic field is being specified in Gauss,
c ****** and the value of B0 is ignored.
c
c ****** Otherwise, normalize the magnetic field so that the maximum
c ****** normal field magnitude is B0.
c ****** When B0=0., leave the field unscaled.
c ****** BSCALE is the scaling factor for the magnetic field; i.e.,
c ****** the original field is BSCALE times the normalized field.
c
      if (b0orig.eq.0.) b0=bnmax
c
      if (bingauss) then
        bscale=fn_b
        b0=bnmax/bscale
      else
        bscale=bnmax/b0
      end if
c
c ****** Write the normalization factor.
c
      write (9,*)
      write (9,*) '### COMMENT from READBR:'
      write (9,*) '### Normalization factor for the normal magnetic'//
     &            ' field:'
      write (9,*)
      write (9,*) 'Maximum field magnitude before normalization  = ',
     &            bnmax
      write (9,*) 'Maximum field magnitude after normalization   = ',
     &            b0
      write (9,*) 'Scaling factor for the magnetic field, BSCALE = ',
     &            bscale
      write (9,*) '[Original field = BSCALE * normalized field.]'
c
      do k=1,np
        do j=1,ntm1
          br0re(j,k)=br0re(j,k)/bscale
        enddo
      enddo
c
c ****** Reset the value of B0.
c
      b0=b0orig
c
c ****** Calculate the total flux.
c
      area=0.
      fluxm=0.
      fluxp=0.
      do j=1,ntm1
        if (j.eq.1) then
          da=.25*r0*sth(2)*dt(1)*dphi
        else if (j.eq.ntm1) then
          da=.25*r0*sth(ntm1)*dt(ntm1)*dphi
        else
          da=r0*st(j)*dt(j)*dphi
        end if
        do k=1,np
          if (br0re(j,k).gt.0.) then
            fluxp=fluxp+br0re(j,k)*da
          else
            fluxm=fluxm+br0re(j,k)*da
          end if
          area=area+da
        enddo
      enddo
c
      write (9,*)
      write (9,*) '### COMMENT from READBR:'
      write (9,*) '### Computed flux balance:'
      write (9,*)
      write (9,*) 'Positive flux = ',fluxp
      write (9,*) 'Negative flux = ',fluxm
c
c ****** Fix the magnetic field so that the total flux is zero.
c
      br00err=(fluxp+fluxm)/area
c
      do k=1,np
        do j=1,ntm1
          br0re(j,k)=br0re(j,k)-br00err
        enddo
      enddo
c
      write (9,*)
      write (9,*) '### COMMENT from READBR:'
      write (9,*) '### Flux balance correction:'
      write (9,*)
      write (9,*) 'BR00 (monopole Br field magnitude) = ',br00err
c
      return
      end
c#######################################################################
      subroutine rdtxt2d (fname,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a 2D field (with scale arrays) from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory for the arrays
c ****** and returns the data and scales through module
c ****** RDTXT2D_ARRAYS.
c
c-----------------------------------------------------------------------
c
      use rdtxt2d_arrays
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c ****** Output arguments:
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c ****** Entities in module RDTXT2D_ARRAYS:
c
c          F_RDTXT2D: [real 2-dimensional array]
c                     This array is allocated by this routine, and
c                     holds the 2D data read in from the file.
c
c          X_RDTXT2D: [real array]
c                     This array is allocated by this routine, and
c                     holds the x-scale of the data array.
c
c          Y_RDTXT2D: [real array]
c                     This array is allocated by this routine, and
c                     holds the y-scale of the data array.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      integer ierr
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (20,fname,'r')
c
c ****** Get the number of dimensions.
c
      read (20,*) ndim
c
      if (ndim.ne.2) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT2D:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) '### The file should contain a 2D dataset.'
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in RDTXT2D:'
        write (9,*) '### Invalid number of dimensions in file.'
        write (9,*) '### The file should contain a 2D dataset.'
        write (9,*) 'NDIM = ',ndim
        write (9,*) 'File name: ',fname(1:lf)
        ierr=1
        go to 900
      end if
c
c ****** Read the dimensions.
c
      read (20,*) nx
      read (20,*) ny
c
c ****** Check that the scales are present.
c
      read (20,*) ifscale
c
      if (ifscale.eq.0) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT2D:'
        write (*,*) '### The file does not contain scales.'
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in RDTXT2D:'
        write (9,*) '### The file does not contain scales.'
        write (9,*) 'File name: ',fname(1:lf)
        ierr=2
        go to 900
      end if
c
c ****** Allocate the required memory.
c
      allocate (f_rdtxt2d(nx,ny))
      allocate (x_rdtxt2d(nx))
      allocate (y_rdtxt2d(ny))
c
c ****** Read the scales.
c
      call rddata (20,nx,x_rdtxt2d,ierr)
      if (ierr.ne.0) go to 900
c
      call rddata (20,ny,y_rdtxt2d,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Read the array.
c
      call rddata (20,nx*ny,f_rdtxt2d,ierr)
      if (ierr.ne.0) go to 900
c
      close (20)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT2D:'
      write (*,*) '### Could not read the required data.'
      write (*,*) 'File name: ',fname(1:lf)
      write (9,*)
      write (9,*) '### ERROR in RDTXT2D:'
      write (9,*) '### Could not read the required data.'
      write (9,*) 'File name: ',fname(1:lf)
      ierr=3
c
      return
      end
c#######################################################################
      subroutine rddata (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine intrp2d (nxi,nyi,xi,yi,fi,nx,ny,x,y,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a 2D field from array FI(NXI,NYI), defined
c ****** on the mesh XI(NXI) x YI(NYI), into the array F(NX,NY),
c ****** defined on the mesh X(NX) x Y(NY).
c
c ****** Note that if a data point is outside the bounds of
c ****** the XI x YI mesh, IERR=2 is returned.
c
c-----------------------------------------------------------------------
c
      real xi(nxi),yi(nyi),fi(nxi,nyi)
      real x(nx),y(ny),f(nx,ny)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Check that the scales XI and YI are monotonic.
c
      dummy=flint(0.,nxi,xi,dum,1,ierr)
      if (ierr.ne.0) go to 900
c
      dummy=flint(0.,nyi,yi,dum,1,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Interpolate the data.
c
      do j=1,ny
        yv=y(j)
        if (yv.lt.yi(1).or.yv.gt.yi(nyi)) then
          go to 910
        end if
        call interp (yi,nyi,yv,jj,ay)
        do i=1,nx
          xv=x(i)
          if (xv.lt.xi(1).or.xv.gt.xi(nxi)) then
            go to 910
          end if
          call interp (xi,nxi,xv,ii,ax)
          f(i,j)= (1.-ax)*((1.-ay)*fi(ii  ,jj  )+ay*fi(ii  ,jj+1))
     &           +    ax *((1.-ay)*fi(ii+1,jj  )+ay*fi(ii+1,jj+1))
        enddo
      enddo
c
      return
c
c ****** Error exit: invalid scales.
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in INTRP2D:'
      write (*,*) '### Scales are not monotonically increasing.'
      write (9,*)
      write (9,*) '### ERROR in INTRP2D:'
      write (9,*) '### Scales are not monotonically increasing.'
      ierr=1
c
      return
c
c ****** Error exit: data outside range of scales.
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in INTRP2D:'
      write (*,*) '### An interpolation was attempted outside the'//
     &            ' range of the defined scales.'
            write(*,*) 'xv,xi(1),xi(nxi): ',xv,xi(1),xi(nxi)
            write(*,*) 'yv,yi(1),yi(nyi): ',yv,yi(1),yi(nyi)
      write (9,*)
      write (9,*) '### ERROR in INTRP2D:'
      write (9,*) '### An interpolation was attempted outside the'//
     &            ' range of the defined scales.'
      ierr=2
c
      return
      end
c#######################################################################
      subroutine intrp2d_internal (nxi,nyi,xi,yi,fi,nx,ny,x,y,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate a 2D field from array FI(NXI,NYI), defined
c ****** on the mesh XI(NXI) x YI(NYI), into the array F(NX,NY),
c ****** defined on the mesh X(NX) x Y(NY) but only for internal
c ****** points in x (2:NXM1,1:NY).
c
c ****** Note that if a data point is outside the bounds of
c ****** the XI x YI mesh, IERR=2 is returned.
c
c-----------------------------------------------------------------------
c
      real xi(nxi),yi(nyi),fi(nxi,nyi)
      real x(nx),y(ny),f(nx,ny)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Check that the scales XI and YI are monotonic.
c
      dummy=flint(0.,nxi,xi,dum,1,ierr)
      if (ierr.ne.0) go to 900
c
      dummy=flint(0.,nyi,yi,dum,1,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Interpolate the data.
c
      do j=1,ny
        yv=y(j)
        if (yv.lt.yi(1).or.yv.gt.yi(nyi)) then
          go to 910
        end if
        call interp (yi,nyi,yv,jj,ay)
        do i=2,nx-1
          xv=x(i)
          if (xv.lt.xi(1).or.xv.gt.xi(nxi)) then
            go to 910
          end if
          call interp (xi,nxi,xv,ii,ax)
          f(i,j)= (1.-ax)*((1.-ay)*fi(ii  ,jj  )+ay*fi(ii  ,jj+1))
     &           +    ax *((1.-ay)*fi(ii+1,jj  )+ay*fi(ii+1,jj+1))
        enddo
      enddo
c
      return
c
c ****** Error exit: invalid scales.
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in INTRP2D_INTERNAL:'
      write (*,*) '### Scales are not monotonically increasing.'
      write (9,*)
      write (9,*) '### ERROR in INTRP2D_INTERNAL:'
      write (9,*) '### Scales are not monotonically increasing.'
      ierr=1
c
      return
c
c ****** Error exit: data outside range of scales.
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in INTRP2D_INTERNAL:'
      write (*,*) '### An interpolation was attempted outside the'//
     &            ' range of the defined scales.'
      write (9,*)
      write (9,*) '### ERROR in INTRP2D_INTERNAL:'
      write (9,*) '### An interpolation was attempted outside the'//
     &            ' range of the defined scales.'
      ierr=2
c
      return
      end
c#######################################################################
      subroutine trid (n,c,a,b,d)
c
c-----------------------------------------------------------------------
c
c ****** Solve the tridiagonal system of equations:
c
c         C(i)*X(i-1) + A(i)*X(i) + B(i)*X(i+1) = D(i)
c
c        for i=2,...,N-1, with
c
c           A(1)*X(1) + B(1)*X(2) = D(1)
c
c        and
c
c           C(N)*X(N-1) + A(N)*X(N) = D(N)
c
c ****** Note that C(1) and B(N) are not referenced.
c
c ****** The array A is overwritten during the call.
c
c ****** D is overwritten with the solution.
c
c-----------------------------------------------------------------------
c
      dimension c(n),a(n),b(n),d(n)
c
c-----------------------------------------------------------------------
c
c ****** Forward elimination.
c
      d(1)=d(1)/a(1)
      a(1)=b(1)/a(1)
      do i=2,n
        ace=1./(a(i)-c(i)*a(i-1))
        if (i.ne.n) a(i)=ace*b(i)
        d(i)=ace*(d(i)-c(i)*d(i-1))
      enddo
c
c ****** Backward substitution.
c
      do i=n-1,1,-1
        d(i)=d(i)-a(i)*d(i+1)
      enddo
c
      return
      end
c#######################################################################
      subroutine amatp0 (rhs)
c
c-----------------------------------------------------------------------
c
c ****** Load the matrix for the m=0 potential field solve for Ap.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Right hand side.
c
      real rhs(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions.
c
      common /potbcap/ apcr0(nt),apvr0(nt),
     &                 apcr1(nt),apvr1(nt),
     &                 apct0(nr),apvt0(nr),
     &                 apct1(nr),apvt1(nr)
c
c-----------------------------------------------------------------------
c
c ****** Initialize matrix diagonals to zero.
c
      a00(:,:)=0.
      ap0(:,:)=0.
      a0p(:,:)=0.
c
c ****** Load the internal points.
c
      do j=2,ntm1
        if (j.eq.2) then
          ratpp=sth(j+1)/st(j  )
          ratpm=sth(j  )/st(j  )
          ratmp=2.
          ratmm=2.
        else if (j.eq.ntm1) then
          ratpp=2.
          ratpm=2.
          ratmp=sth(j  )/st(j-1)
          ratmm=sth(j-1)/st(j-1)
        else
          ratpp=sth(j+1)/st(j  )
          ratpm=sth(j  )/st(j  )
          ratmp=sth(j  )/st(j-1)
          ratmm=sth(j-1)/st(j-1)
        end if
        do i=2,nrm1
          dv=rh(i)**2*drh(i)*dth(j)*sth(j)
          a00(i,j)= sth(j)*dth(j)*rh(i)**2*(dri(i)+dri(i-1))
     &             +drh(i)*sth(j)*(ratpm*dti(j)+ratmp*dti(j-1))
          ap0(i,j)=-sth(j)*dth(j)*rh(i)*rh(i+1)*dri(i)
          a0p(i,j)=-drh(i)*sth(j)*ratpp*dti(j)
          rhs(i,j)=rhs(i,j)*dv
        enddo
      enddo
c
c ****** Eliminate the boundary points.
c
      do j=2,ntm1
        am0=-sth(j)*dth(j)*rh(1)*rh(2)*dri(1)
        a00(   2,j)=a00(   2,j)-am0        *apcr0(j)
        rhs(   2,j)=rhs(   2,j)-am0        *apvr0(j)
        a00(nrm1,j)=a00(nrm1,j)-ap0(nrm1,j)*apcr1(j)
        rhs(nrm1,j)=rhs(nrm1,j)-ap0(nrm1,j)*apvr1(j)
        ap0(nrm1,j)=0.
      enddo
      do i=2,nrm1
        ratmm=2.
        a0m=-drh(i)*sth(2)*ratmm*dti(1)
        a00(i,   2)=a00(i,   2)-a0m        *apct0(i)
        rhs(i,   2)=rhs(i,   2)-a0m        *apvt0(i)
        a00(i,ntm1)=a00(i,ntm1)-a0p(i,ntm1)*apct1(i)
        rhs(i,ntm1)=rhs(i,ntm1)-a0p(i,ntm1)*apvt1(i)
        a0p(i,ntm1)=0.
      enddo
c
c ****** Set the number of equations to be solved.
c
      ncgeq=nr*nt
c
      return
      end
c#######################################################################
      subroutine amatt (m,rhs)
c
c-----------------------------------------------------------------------
c
c ****** Load the matrix for the m > 0 potential field solve for At.
c
c-----------------------------------------------------------------------
c
      use matrix_pot

c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Right hand side.
c
      real rhs(nr,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions.
c
      common /potbcat/ atcr0(ntm1),atvr0(ntm1),
     &                 atcr1(ntm1),atvr1(ntm1)
c
c-----------------------------------------------------------------------
c
      k=m+1
c
c ****** Initialize matrix diagonals to zero.
c
      a00(:,:)=0.
      ap0(:,:)=0.
      a0p(:,:)=0.
c
c ****** Load the internal points.
c
      do j=2,ntm2
        do i=2,nrm1
          dv=rh(i)**2*drh(i)*dt(j)*st(j)**3
          a00(i,j)= st(j)**3*dt(j)*rh(i)**2*(dri(i)+dri(i-1))
     &             +drh(i)*st(j)**2*( sth(j+1)/dth(j+1)
     &                               +sth(j  )/dth(j  ))
ccc     &             +fm2(k)*drh(i)*dt(j)*st(j)
     &             +fm(k)**2*drh(i)*dt(j)*st(j)
          ap0(i,j)=-st(j)**3*dt(j)*rh(i)*rh(i+1)*dri(i)
          a0p(i,j)=-drh(i)*st(j)*st(j+1)*sth(j+1)/dth(j+1)
          rhs(i,j)=rhs(i,j)*dv
        enddo
      enddo
c
c ****** Eliminate the boundary points.
c
      do j=2,ntm2
        am0=-st(j)**3*dt(j)*rh(1)*rh(2)*dri(1)
        a00(   2,j)=a00(   2,j)-am0        *atcr0(j)
        rhs(   2,j)=rhs(   2,j)-am0        *atvr0(j)
        a00(nrm1,j)=a00(nrm1,j)-ap0(nrm1,j)*atcr1(j)
        rhs(nrm1,j)=rhs(nrm1,j)-ap0(nrm1,j)*atvr1(j)
        ap0(nrm1,j)=0.
      enddo
c
c ****** Note that there is no coupling to theta boundary points.
c
      do i=2,nrm1
        a0p(i,ntm2)=0.
      enddo
c
c ****** Set the number of equations to be solved.
c
      ncgeq=nr*ntm1
c
      return
      end
c#######################################################################
      subroutine setdt
c
c-----------------------------------------------------------------------
c
c ****** Set the time step.
c
c-----------------------------------------------------------------------
c
c ****** If the semi-implicit term is being used (ISITYPE.ne.0),
c ****** the time step is set to the smaller of DTMAX and
c ****** the advective time step limit.
c
c ****** Otherwise (i.e., fully explicit), the time step is
c ****** set to the smaller of DTMAX and the fraction
c ****** CFL of the "CFL" limit.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real cflv(nrm1,ntm1,np)
c
c-----------------------------------------------------------------------
c
c ****** Information about maximum speeds and time step limits.
c
      common /dtinfo/ vmax,vamax,csmax,dtcfl,dtadv,betamax
c
c-----------------------------------------------------------------------
c
c ****** Maximum percentage increase in the time step
c ****** allowed per time step.
c
      parameter (epsdt=.05)
c
c-----------------------------------------------------------------------
c
c ****** Set the local (cell-Reynolds number) viscosity.
c
      call cellvisc
c
c ****** Set the local (cell-magnetic-Reynolds number) resistivity.
c
      call celleta
c
      if (isitype.ne.0) go to 200
c
c-----------------------------------------------------------------------
c ****** Fully explicit algorithm.
c-----------------------------------------------------------------------
c
c ****** Find the cfl time step limit.
c
      vamax=0.
      csmax=0.
      vmax=0.
      dtcfl=1.e20
      do k=1,np
      do j=1,ntm1
      do i=1,nrm1
        avrho=AVGRT(rho,i+1,j+1,k)
        avvr =AVGT (vr ,i  ,j+1,k)
        avvt =AVGR (vt ,i+1,j  ,k)
        avvp =AVGRT(vp ,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        avtemp=AVGRT(temp,i+1,j+1,k)
        bsq=avbr**2+avbt**2+avbp**2
        vsq=avvr**2+avvt**2+avvp**2
        cssq=gammar(i)*he_p*avtemp/he_rho
        vasq=bsq/avrho
        ctotsq=cssq+vasq
        cs=sqrt(cssq)
        va=sqrt(vasq)
        ctot=sqrt(ctotsq)
        vmag=sqrt(vsq)
        stsq=.5*(sth(j)**2+sth(j+1)**2)
        fksq=1./dr(i)**2+1./(r(i)*dt(j))**2+fm2(npd3)/(stsq*r(i)**2)
        fkmag=sqrt(fksq)
        dtcfl=min(dtcfl,1./(fkmag*max(vmag+ctot,1.e-20)))
        vamax=max(vamax,va)
        csmax=max(csmax,cs)
        vmax=max(vmax,vmag)
      enddo
      enddo
      enddo
c
c ****** Set the time step to the smaller of DTMAX or CFL times
c ****** the "CFL" limit.
c ****** The time step is not allowed to increase by
c ****** more than a fraction EPSDT of its present value.
c
      dtime=min(cfl*dtcfl,dtmax,(1.+epsdt)*dtime)
c
      betamax=2.*dtime/dtcfl
c
c ****** Set the semi-implicit coefficient to zero.
c
      dtadv=0.
      call bcast (sifac,0.,nrm1*ntm1)
c
      go to 500
c
  200 continue
c
c-----------------------------------------------------------------------
c ****** Semi-implicit algorithm.
c-----------------------------------------------------------------------
c
c ****** First, find the advective limit.
c
      vmax=0.
      dtadv=1.e20
      do k=1,np
      do j=1,ntm1
      do i=1,nrm1
        avvr =AVGT (vr ,i  ,j+1,k)
        avvt =AVGR (vt ,i+1,j  ,k)
        avvp =AVGRT(vp ,i+1,j+1,k)
        vsq=avvr**2+avvt**2+avvp**2
        vmag=sqrt(vsq)
        fkr2=1./dr(i)**2
        fkt2=1./(r(i)*dt(j))**2
        stsq=.5*(sth(j)**2+sth(j+1)**2)
        fkp2=fm2(npd3)/(stsq*r(i)**2)
        fkdotv=sqrt(fkr2*avvr**2+fkt2*avvt**2+fkp2*avvp**2)
        fkdotv=max(fkdotv,1.e-20)
        dtadv=min(dtadv,1./fkdotv)
        vmax=max(vmax,vmag)
        cflv(i,j,k)=fkdotv
      enddo
      enddo
      enddo
c
c-----------------------------------------------------------------------
c
c
c ****** Find time step limit set by the radiation law 
c
      dt_temp=1.e20
      if (radloss.ne.0.) then
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              fne=rho(i,j,k)/he_rho
              radterm=(gammarh(i)-1.)*radloss*(fne*he_np/he_p)
     &                *qrad(fn_t*temp(i,j,k))/fn_qrad
              dqdtemp=ifimplrad*radloss*(gammarh(i)-1.)*
     &        (fne*he_np/he_p)*fn_t*d_qrad(fn_t*temp(i,j,k))/fn_qrad
              dummy=max(1.e-20,radterm-dqdtemp*temp(i,j,k))
              dt_temp=min(dt_temp,temp(i,j,k)/dummy)
            enddo
          enddo
        enddo
      dt_temp=.9*dt_temp
      end if
c
c ****** Set the time step to the smaller of DTMAX or STRMAX times
c ****** the advective limit or DT_TEMP.
c
      dtime=min(strmax*dtadv,dtmax,dt_temp)

      cflv=cflv*dtime
c
c ****** Compute the semi-implicit coefficient based on
c ****** this time step
c
      vasqmx=0.
      cssqmx=0.
      btsqmx=0.
      sifac=0.
      sicof=0.
      do k=1,np
      do j=1,ntm1
      do i=1,nrm1
        avrho=AVGRT(rho,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        avtemp=AVGRT(temp,i+1,j+1,k)
        bsq=avbr**2+avbt**2+avbp**2
        cssq=gammar(i)*he_p*avtemp/he_rho
        vasq=bsq/avrho
        ctotsq=cssq+vasq
        stsq=.5*(sth(j)**2+sth(j+1)**2)
        fksq=4./dr(i)**2+4./(r(i)*dt(j))**2+fm2(npd3)/(stsq*r(i)**2)
        betasq=dtime**2*ctotsq*fksq
        sifac(i,j)=max(sifac(i,j),(.25*betasq/
     &  (1.-fac_cflv*cflv(i,j,k))**2-1.)/(dtime*fksq)) 
        sicof=max(sifac(i,j),sicof)
        vasqmx=max(vasqmx,vasq)
        cssqmx=max(cssqmx,cssq)
        btsqmx=max(btsqmx,betasq)
      enddo
      enddo
      enddo
c
      vamax=sqrt(vasqmx)
      csmax=sqrt(cssqmx)
      betamax=sqrt(btsqmx)
c
      dtcfl=0.
c
c
      if (si_aggressive) then
        sifac=sifac*simult
      else
        sifac=sicof*simult
      endif
c
  500 continue
c
c ****** Check that DTIME exceeds DTMIN.
c ****** If it does not, print a message and set the
c ****** abort flag, IFABORT.
c
      if (dtime.le.dtmin) then
        write (*,*)
        write (*,*) '### WARNING from SETDT:'
        write (*,*) '### Time step is smaller than DTMIN.'
        write (*,*) 'DTIME = ',dtime
        write (*,*) 'DTMIN = ',dtmin
        write (*,*) 'NTIME = ',ntime
        write (*,*) 'TIME = ',time
        write (9,*)
        write (9,*) '### WARNING from SETDT:'
        write (9,*) '### Time step is smaller than DTMIN.'
        write (9,*) 'DTIME = ',dtime
        write (9,*) 'DTMIN = ',dtmin
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
        ifrsout=1
        ifabort=.true.
      end if
c
      if (idebug.gt.0) then
        write (9,*)
        write (9,*) '### COMMENT from SETDT:'
        write (9,*) 'NTIME   = ',ntime
        write (9,*) 'TIME    = ',time
        write (9,*) 'DTIME   = ',dtime
        write (9,*) 'DTCFL   = ',dtcfl
        write (9,*) 'DTADV   = ',dtadv
        write (9,*) 'BETAMAX = ',betamax
        write (9,*) 'VMAX    = ',vmax
        write (9,*) 'VAMAX   = ',vamax
        write (9,*) 'CSMAX   = ',csmax
      end if
c
      return
      end
c#######################################################################
      subroutine advance
c
c-----------------------------------------------------------------------
c
c ****** Advance all quantities by one time step.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      save ircnt
      data ircnt/0/
c
c-----------------------------------------------------------------------
c
c ****** Find the emerging flux.
c
      call newflux
c
c ****** Advance the vector potential.
c
      call adva
c
c ****** Advance the temperature.
c ****** If IFTEMP=0, the temperature is not advanced.
c
      if (iftemp.ne.0) then
c
        call advtemp
c
c ****** Check that the temperature is non-negative.
c
        call mnmxpres (temp,tempmin,tempmax,7.e-4)
c
        if (tempmin.lt.0.) then
          write (*,*)
          write (*,*) '### ERROR in ADVANCE:'
          write (*,*) '### The temperature has become negative.'
          write (*,*)
          write (*,*) 'Minimum value = ',tempmin
          write (*,*) 'Maximum value = ',tempmax
          write (*,*)
          write (*,*) 'NTIME = ',ntime
          write (*,*) 'TIME = ',time
          write (9,*)
          write (9,*) '### ERROR in ADVANCE:'
          write (9,*) '### The temperature has become negative.'
          write (9,*)
          write (9,*) 'Minimum value = ',tempmin
          write (9,*) 'Maximum value = ',tempmax
          write (9,*)
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call abortrun
        end if
c
      end if
c
c ****** Advance the density.
c ****** If IFRHO=0, the density is not advanced.
c
      if (ifrho.ne.0) then
c
        call advrho
c
c ****** Check that the density is positive.
c
        call mnmxpres (rho,rhomin,rhomax,-100000.)
c
        if (rhomin.le.0.) then
          write (*,*)
          write (*,*) '### ERROR in ADVANCE:'
          write (*,*) '### The density has become non-positive.'
          write (*,*)
          write (*,*) 'Minimum value = ',rhomin
          write (*,*) 'Maximum value = ',rhomax
          write (*,*)
          write (*,*) 'NTIME = ',ntime
          write (*,*) 'TIME = ',time
          write (9,*)
          write (9,*) '### ERROR in ADVANCE:'
          write (9,*) '### The density has become non-positive.'
          write (9,*)
          write (9,*) 'Minimum value = ',rhomin
          write (9,*) 'Maximum value = ',rhomax
          write (9,*)
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call abortrun
        end if
c
      end if
c
c ****** Set the density to give a uniform Alfven speed if
c ****** requested.  This is done every ISETRHO time steps.
c
      ircnt=ircnt+1
      if (isetrho.gt.0) then
        if (ircnt.eq.isetrho) then
          call setrho
          ircnt=0
        end if
      end if
c
c ****** Set the pressure.
c
      call setp
c
c ****** Check that the pressure is non-negative.
c
c     call mnmxpres (p,pmin,pmax,-1.) 
c
      if (pmin.lt.0.) then
        write (*,*)
        write (*,*) '### ERROR in ADVANCE:'
        write (*,*) '### The pressure has become negative.'
        write (*,*)
        write (*,*) 'Minimum value = ',pmin
        write (*,*) 'Maximum value = ',pmax
        write (*,*)
        write (*,*) 'NTIME = ',ntime
        write (*,*) 'TIME = ',time
        write (9,*)
        write (9,*) '### ERROR in ADVANCE:'
        write (9,*) '### The pressure has become negative.'
        write (9,*)
        write (9,*) 'Minimum value = ',pmin
        write (9,*) 'Maximum value = ',pmax
        write (9,*)
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
        call abortrun
      end if
c
c ****** Advance the Alfven wave pressure.
c ****** If IFAW=0, the Alfven wave pressure is not advanced.
c
      if (ifaw.ne.0) then
c
        call advpw
c
      end if
c
c ****** Set up the boundary condition arrays for the
c ****** gas variables.
c
      call setbcg (time+.5*dtime)
c
c ****** Advance v.
c
      call advv
c
      return
      end
c#######################################################################
      subroutine mnmx (f,check0,fmin,fmax)
c
c-----------------------------------------------------------------------
c
c ****** Find the minimum and maximum of an array defined on
c ****** the RH and TH mesh.
c
c ****** When CHECK0=.true., the points at r=RH(1) and r=RH(NR)
c ****** are checked.  Otherwise, they are not checked.
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real f(nr,nt,np)
      logical check0
c
c-----------------------------------------------------------------------
c
      if (check0) then
        irmin=1
        irmax=nr
      else
        irmin=2
        irmax=nrm1
      end if
c
      fmin=f(irmin,1,1)
      fmax=fmin
      do k=1,np
        do j=1,nt
          do i=irmin,irmax
            fmin=min(fmin,f(i,j,k))
            fmax=max(fmax,f(i,j,k))
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine mnmxpres (f,fmin,fmax,floor)
c
c-----------------------------------------------------------------------
c
c ****** Find the minimum and maximum of an array.
c ****** If floor > 0., set f values less than the floor
c ****** equal to the floor.
c
c-----------------------------------------------------------------------
c
      use comm
      real f(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      fmin=f(1,1,1)
      fmax=f(1,1,1)
      do k=1,np
        do j=1,ntm1
          do i=1,nrm1
            avf =AVGRT(f   ,i+1,j+1,k)
            avf =AVGRT(f   ,i+1,j+1,k)
            fmin=min(fmin,avf)
            fmax=max(fmax,avf)
          enddo
        enddo
      enddo
      if(floor .gt. 0.) then
      do k=1,np
        do j=1,ntm1
          do i=1,nrm1
            f(i,j,k)=max(floor,f(i,j,k))
          enddo
        enddo
      enddo
      end if
c
      return
      end
c#######################################################################
      subroutine newflux
c
c-----------------------------------------------------------------------
c
c ****** Find the electric field corresponding to the newly emerged
c ****** flux at the r=R0 boundary for an emerging flux run.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Emerging flux tangential electric field at r=R0.
c
      common /efluxcom/ et0ef(nt,npd3,2),ep0ef(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real brefnew(ntm1,np),dbr(ntm1,np)
      real dbrr(ntm1,npd3),dbri(ntm1,npd3)
      real chi(ntm1,npd3,2)
c
c-----------------------------------------------------------------------
c
c ****** Arrays for the tridiagonal solve.
c
      real aa(ntm1),bb(ntm1),cc(ntm1),dd(ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Return immediately if this is not an emerging flux run.
c
      if (.not.emgflux) then
        call bcast (et0ef,0.,2*nt*npd3)
        call bcast (ep0ef,0.,2*nt*npd3)
        return
      end if
c
c ****** Get the new boundary flux.
c
      call eflux (time,brefnew)
c
c ****** Find the tangential electric field required to
c ****** emerge this flux.  This requires the solution of
c ****** an elliptic equation.
c
      do k=1,np
        do j=1,ntm1
          brav=.5*(br(1,j,k)+br(2,j,k))
          dbr(j,k)=brefnew(j,k)-brav
        enddo
      enddo
c
      call fft_tp1 (ntm1,dbr,dbrr,dbri,1)
c
      do k=1,npd3
c
        m=k-1
c
c ****** Set up the tridiagonal matrix coefficients for the
c ****** chi solve for the internal points.
c
c ****** First, do the real part.
c
        do j=2,ntm2
          aa(j)= sth(j+1)/dth(j+1)
     &          +sth(j  )/dth(j  )
     &          +fm2(k)*dt(j)/st(j)
          bb(j)=-sth(j+1)/dth(j+1)
          cc(j)=-sth(j  )/dth(j  )
          dd(j)=r0**2*st(j)*dt(j)*dbrr(j,k)
        enddo
c
c ****** Boundary conditions at the poles.
c
        if (m.eq.0) then
c
          aa(1)=1.
          bb(1)=-1.
          dd(1)=.25*r0**2*dth(2)**2*dbrr(1,1)
c
          aa(ntm1)=1.
          cc(ntm1)=0.
          dd(ntm1)=0.
c
        else
c
          aa(1)=1.
          bb(1)=0.
          dd(1)=0.
c
          aa(ntm1)=1.
          cc(ntm1)=0.
          dd(ntm1)=0.
c
        end if
c
        call trid (ntm1,cc,aa,bb,dd)
c
        do j=1,ntm1
          chi(j,k,1)=dd(j)
        enddo
c
c ****** Now, solve for the imaginary part.
c
        if (m.eq.0) then
c
          do j=1,ntm1
            chi(j,k,2)=0.
          enddo
c
        else
c
          do j=2,ntm2
            aa(j)= sth(j+1)/dth(j+1)
     &            +sth(j  )/dth(j  )
     &            +fm2(k)*dt(j)/st(j)
            bb(j)=-sth(j+1)/dth(j+1)
            cc(j)=-sth(j  )/dth(j  )
            dd(j)=r0**2*st(j)*dt(j)*dbri(j,k)
          enddo
c
c ****** Boundary conditions at the poles.
c
          aa(1)=1.
          bb(1)=0.
          dd(1)=0.
c
          aa(ntm1)=1.
          cc(ntm1)=0.
          dd(ntm1)=0.
c
          call trid (ntm1,cc,aa,bb,dd)
c
          do j=1,ntm1
            chi(j,k,2)=dd(j)
          enddo
c
        end if
c
      enddo
c
c ****** Find the tangential electric field corresponding to the
c ****** emerged flux.
c
      do k=1,npd3
        do j=2,ntm2
          et0ef(j,k,1)= (fm(k)*chi(j,k,2)/(r0*st(j)))/dtime
          et0ef(j,k,2)=-(fm(k)*chi(j,k,1)/(r0*st(j)))/dtime
        enddo
      enddo
c
      do k=1,npd3
        do j=2,ntm1
          ep0ef(j,k,1)=((chi(j,k,1)-chi(j-1,k,1))/(r0*dth(j)))/dtime
          ep0ef(j,k,2)=((chi(j,k,2)-chi(j-1,k,2))/(r0*dth(j)))/dtime
        enddo
      enddo
c
c ****** Set boundary conditions at the poles.
c
      do k=1,npd3
        m=k-1
        if (m.eq.0) then
c
          ep0ef( 1,k,1)=-ep0ef(   2,k,1)
          ep0ef( 1,k,2)=-ep0ef(   2,k,2)
          ep0ef(nt,k,1)=-ep0ef(ntm1,k,1)
          ep0ef(nt,k,2)=-ep0ef(ntm1,k,2)
c
          et0ef(   1,k,1)=0.
          et0ef(   1,k,2)=0.
          et0ef(ntm1,k,1)=0.
          et0ef(ntm1,k,2)=0.
c
        else if (m.eq.1) then
c
          ep0ef( 1,k,1)= ep0ef(   2,k,1)
          ep0ef( 1,k,2)= ep0ef(   2,k,2)
          ep0ef(nt,k,1)= ep0ef(ntm1,k,1)
          ep0ef(nt,k,2)= ep0ef(ntm1,k,2)
c
          et0ef(   1,k,1)= ep0ef(   2,k,2)
          et0ef(   1,k,2)=-ep0ef(   2,k,1)
          et0ef(ntm1,k,1)=-ep0ef(ntm1,k,2)
          et0ef(ntm1,k,2)= ep0ef(ntm1,k,1)
c
        else if (m.eq.2) then
c
          ep0ef( 1,k,1)=-ep0ef(   2,k,1)
          ep0ef( 1,k,2)=-ep0ef(   2,k,2)
          ep0ef(nt,k,1)=-ep0ef(ntm1,k,1)
          ep0ef(nt,k,2)=-ep0ef(ntm1,k,2)
c
          et0ef(   1,k,1)=0.
          et0ef(   1,k,2)=0.
          et0ef(ntm1,k,1)=0.
          et0ef(ntm1,k,2)=0.
c
        else
c
          ep0ef( 1,k,1)= ep0ef(   2,k,1)
          ep0ef( 1,k,2)= ep0ef(   2,k,2)
          ep0ef(nt,k,1)= ep0ef(ntm1,k,1)
          ep0ef(nt,k,2)= ep0ef(ntm1,k,2)
c
          et0ef(   1,k,1)=0.
          et0ef(   1,k,2)=0.
          et0ef(ntm1,k,1)=0.
          et0ef(ntm1,k,2)=0.
c
        end if
      enddo
c
c ****** Set the dummy points to zero.
c
      do k=1,npd3
        et0ef(nt,k,1)=0.
        et0ef(nt,k,2)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine eflux (targ,brnew)
c
c-----------------------------------------------------------------------
c
c ****** Set the emerged flux at the boundary at the present time.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real brnew(ntm1,np)
c
c-----------------------------------------------------------------------
c
c ****** Array to help in the flux interpolation.
c
      save helper_ef
      real helper_ef(mxefnodes)
c
c ****** Variables for flux interpolation.
c
      save ifile0_contents,ifile1_contents
      data ifile0_contents/0/
      data ifile1_contents/0/
c
      save br_ef0,br_ef1
      real br_ef0(ntm1,np),br_ef1(ntm1,np)
c
      logical first
      save first,nodes
      data first/.true./
c
      logical exists
c
c-----------------------------------------------------------------------
c
c ****** On the first time in, check TNODE_EF.
c
      if (first) then
c
c ****** Find the number of nodes specified.
c
        do i=1,mxefnodes
          if (tnode_ef(i).lt.0.) go to 100
        enddo
  100   nodes=i-1
c
c ****** Check the time nodes.
c
        dummy=flint(0.,nodes,tnode_ef,dum,1,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in EFLUX:'
          write (*,*) '### Inconsistent input data in array TNODE_EF.'
          write (9,*)
          write (9,*) '### ERROR in EFLUX:'
          write (9,*) '### Inconsistent input data in array TNODE_EF.'
          call exit (1)
        end if
c
c ****** Check that the specified flux files exist.
c
        do i=1,nodes
          lf=lenstr(brfile_ef(i))
          inquire (file=brfile_ef(i),exist=exists)
          if (.not.exists) then
            write (*,*)
            write (*,*) '### ERROR in EFLUX:'
            write (*,*) '### You have specified a flux file'//
     &                  ' that does not exist.'
            write (*,*) 'File name: ',brfile_ef(i)(1:lf)
            write (9,*)
            write (9,*) '### ERROR in EFLUX:'
            write (9,*) '### You have specified a flux file'//
     &                  ' that does not exist.'
            write (9,*) 'File name: ',brfile_ef(i)(1:lf)
            call exit (1)
          end if
        enddo
c
c ****** Set up the flux interpolation helper array.
c
        do i=1,nodes
          helper_ef(i)=i
        enddo
c
        first=.false.
c
      end if
c
c ****** Interpolate to time TARG.
c
      if (targ.le.tnode_ef(1)) then
        ifile0=1
        ifile1=1
        frac=0.
      else if (targ.ge.tnode_ef(nodes)) then
        ifile0=nodes
        ifile1=nodes
        frac=0.
      else
        file_no=flint(targ,nodes,tnode_ef,helper_ef,0,ierr)
        ifile0=file_no
        ifile1=min(ifile0+1,nodes)
        frac=file_no-ifile0
      end if
c
c ****** Set up the flux in the flux arrays BR_EF0 and BR_EF1
c ****** bounding this interval.
c
c ****** First, check to see if the flux file has changed.
c ****** If it has, check to see if this file has already
c ****** been read in previously; if it has, transfer that
c ****** flux, otherwise read in the new file.
c
      if (ifile0.ne.ifile0_contents) then
        if (ifile0.eq.ifile1_contents) then
          write (9,*)
          write (9,*) '### COMMENT from EFLUX:'
          write (9,*) '### Transferring flux from FILE1 to FILE0.'
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call scopy (ntm1*np,br_ef1,1,br_ef0,1)
        else
          lf=lenstr(brfile_ef(ifile0))
          write (9,*)
          write (9,*) '### COMMENT from EFLUX:'
          write (9,*) '### Reading a new flux file.'
          write (9,*) 'File name: ',brfile_ef(ifile0)(1:lf)
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call readbr (brfile_ef(ifile0),br_ef0)
        end if
        ifile0_contents=ifile0
      end if
c
      if (ifile1.ne.ifile1_contents) then
        if (ifile1.eq.ifile0_contents) then
          write (9,*)
          write (9,*) '### COMMENT from EFLUX:'
          write (9,*) '### Transferring flux from FILE0 to FILE1.'
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call scopy (ntm1*np,br_ef0,1,br_ef1,1)
        else
          lf=lenstr(brfile_ef(ifile1))
          write (9,*)
          write (9,*) '### COMMENT from EFLUX:'
          write (9,*) '### Reading a new flux file.'
          write (9,*) 'File name: ',brfile_ef(ifile1)(1:lf)
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
          call readbr (brfile_ef(ifile1),br_ef1)
        end if
        ifile1_contents=ifile1
      end if
c
c ****** Interpolate the flux to time TARG.
c
      do k=1,np
        do j=1,ntm1
          brnew(j,k)=(1.-frac)*br_ef0(j,k)+frac*br_ef1(j,k)
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine adva
c
c-----------------------------------------------------------------------
c
c ****** Advance the vector potential.
c
c-----------------------------------------------------------------------
c
      use solver_types
      use matrix_a
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real agrr(nr,nt,npd3),agri(nr,nt,npd3),
     &     agtr(nr,nt,npd3),agti(nr,nt,npd3),
     &     agpr(nr,nt,npd3),agpi(nr,nt,npd3)
c
      real aprr(nr,nt,npd3),apri(nr,nt,npd3),
     &     aptr(nr,nt,npd3),apti(nr,nt,npd3),
     &     appr(nr,nt,npd3),appi(nr,nt,npd3)
c
      real bcrr(nr,nt,npd3),bcri(nr,nt,npd3),
     &     bctr(nr,nt,npd3),bcti(nr,nt,npd3),
     &     bcpr(nr,nt,npd3),bcpi(nr,nt,npd3)
c
      real jcrr(nr,nt,npd3),jcri(nr,nt,npd3),
     &     jctr(nr,nt,npd3),jcti(nr,nt,npd3),
     &     jcpr(nr,nt,npd3),jcpi(nr,nt,npd3)
c
      real vxbrr(nr,nt,npd3),vxbri(nr,nt,npd3),
     &     vxbtr(nr,nt,npd3),vxbti(nr,nt,npd3),
     &     vxbpr(nr,nt,npd3),vxbpi(nr,nt,npd3)
c     real arer(nr,nt,np),aret(nr,nt,np),arep(nr,nt,np)
c
      real acr(nr,nt),act(nr,nt),acp(nr,nt),
     &     abr(nr,nt),abt(nr,nt),abp(nr,nt)
c
      real er0r(nt,npd3),er0i(nt,npd3),
     &     et0r(nt,npd3),et0i(nt,npd3),
     &     ep0r(nt,npd3),ep0i(nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** CG work arrays
c
      real xcg(nr,nt,3),rhs(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      character*72 errmsg
c
c-----------------------------------------------------------------------
c
c ****** Zero out the AP, AG, and AB arrays.
c
      call bcast (aprr,0.,nc)
      call bcast (apri,0.,nc)
      call bcast (aptr,0.,nc)
      call bcast (apti,0.,nc)
      call bcast (appr,0.,nc)
      call bcast (appi,0.,nc)
c
      call bcast (agrr,0.,nc)
      call bcast (agri,0.,nc)
      call bcast (agtr,0.,nc)
      call bcast (agti,0.,nc)
      call bcast (agpr,0.,nc)
      call bcast (agpi,0.,nc)
c
      call bcast (abr,0.,nrt)
      call bcast (abt,0.,nrt)
      call bcast (abp,0.,nrt)
c
c ****** Store the vector potential at the r=R0 boundary
c ****** to compute the electric field.
c
      do k=1,npd3
        do j=1,nt
          avarr=arr(1,j,k)
          avari=ari(1,j,k)
          er0r(j,k)=avarr
          er0i(j,k)=avari
        enddo
        do j=1,ntm1
          avatr=.5*(atr(1,j,k)+atr(2,j,k))
          avati=.5*(ati(1,j,k)+ati(2,j,k))
          et0r(j,k)=avatr
          et0i(j,k)=avati
        enddo
        do j=1,nt
          avapr=.5*(apr(1,j,k)+apr(2,j,k))
          avapi=.5*(api(1,j,k)+api(2,j,k))
          ep0r(j,k)=avapr
          ep0i(j,k)=avapi
        enddo
      enddo
c
      if (ifpc.ne.0) then
c
c ****** Predictor step.
c
c       call fft (arer,arr,ari,0)
c       call fft (aret,atr,ati,0)
c       call fft (arep,apr,api,0)
c
c ****** Advective part
c
c       call vdgv (arer,aret,arep,
c    &           arr,ari,atr,ati,apr,api,
c    &           agrr,agri,agtr,agti,agpr,agpi)
        call vcrossb (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)

c       if (predflow) then
c         flowfac=1.
c       else
c         flowfac=0.
c       endif
c
c ****** Eliminate (if flowfac=0) or set the exact (if flowfac=1)
c ******  advective term in the predictor
cc
        do k=1,npd3
          do j=2,ntm1
            do i=1,nrm1
              vxbrr(i,j,k)=betapc_p*vxbrr(i,j,k) 
c    &        +(betapc_p-flowfac)*agrr(i,j,k)
              vxbri(i,j,k)=betapc_p*vxbri(i,j,k)
c    &        +(betapc_p-flowfac)*agri(i,j,k)
            enddo
          enddo
          do j=2,ntm2
            do i=2,nrm1
              vxbtr(i,j,k)=betapc_p*vxbtr(i,j,k)
c    &        (betapc_p-flowfac)*agtr(i,j,k)
              vxbti(i,j,k)=betapc_p*vxbti(i,j,k)
c    &        (betapc_p-flowfac)*agti(i,j,k)
            enddo
          enddo
          do j=2,ntm1
            do i=2,nrm1
              vxbpr(i,j,k)=betapc_p*vxbpr(i,j,k)
c    &        (betapc_p-flowfac)*agpr(i,j,k)
              vxbpi(i,j,k)=betapc_p*vxbpi(i,j,k)
c    &        (betapc_p-flowfac)*agpi(i,j,k)
            enddo
          enddo
        enddo
cc
c ****** Get the predicted A.
c
        call aguess (aprr,apri,aptr,apti,appr,appi,
     &               vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c ****** Set boundary conditions.
c
        call setbca (aprr,apri,aptr,apti,appr,appi)
      else
        aprr=arr
        apri=ari
        aptr=atr
        apti=ati
        appr=apr
        appi=api
      end if
c
c ****** Get the predicted B.
c
      call bfroma (aprr,apri,aptr,apti,appr,appi,
     &               bcrr,bcri,bctr,bcti,bcpr,bcpi)
c
c ****** Corrector step.
c
      call vcrossb (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c
c ****** Set the tangential electric field.
c
      call setebc (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c ****** Advance the boundary values of A.
c
      call getabc
c
c ****** Prepare the right-hand-side, and get the initial guess
c ****** for A for the solution of the implicit resistivity
c ****** equations.
c ****** Use A advanced by the explicit part of the solution.
c ****** This ought to make the CG solver converge in
c ****** zero steps when the resistivity is small (and,
c ****** of course, for ideal MHD).
c ****** The guess is stored in AG##.
c ****** The right-hand-side overwrites the VXB## array
c ****** with the rhs.
c
      if (omegaeta.ne.1.) then
        call fft (fjr,jcrr,jcri,1)
        call fft (fjt,jctr,jcti,1)
        call fft (fjp,jcpr,jcpi,1)
      else
        call bcast (jcrr,0.,nc)
        call bcast (jcri,0.,nc)
        call bcast (jctr,0.,nc)
        call bcast (jcti,0.,nc)
        call bcast (jcpr,0.,nc)
        call bcast (jcpi,0.,nc)
      end if
c
      do k=1,npd3
c
        do j=2,ntm1
          do i=1,nrm1
            eta=AVG2T (etafac,i,j)+AVG2T (etacel,i,j)
            sig=1./max(eta,1.e-20)
            dvr=r(i)**2*dr(i)*dth(j)*sth(j)
            agrr(i,j,k)=arr(i,j,k)+dtime*( vxbrr(i,j,k)
     &                                    -eta*jcrr(i,j,k))
            agri(i,j,k)=ari(i,j,k)+dtime*( vxbri(i,j,k)
     &                                    -eta*jcri(i,j,k))
            vxbrr(i,j,k)=dvr*(sig*arr(i,j,k)+dtime*(sig*vxbrr(i,j,k)
     &                   -(1.-omegaeta)*jcrr(i,j,k)))
            vxbri(i,j,k)=dvr*(sig*ari(i,j,k)+dtime*(sig*vxbri(i,j,k)
     &                   -(1.-omegaeta)*jcri(i,j,k)))
          enddo
        enddo
c
        do j=2,ntm2
          do i=2,nrm1
            eta=AVG2R (etafac,i,j)+AVG2R (etacel,i,j)
            sig=1./max(eta,1.e-20)
            dvt=rh(i)**2*drh(i)*dt(j)*st(j)
            agtr(i,j,k)=atr(i,j,k)+dtime*( vxbtr(i,j,k)
     &                                    -eta*jctr(i,j,k))
            agti(i,j,k)=ati(i,j,k)+dtime*( vxbti(i,j,k)
     &                                    -eta*jcti(i,j,k))
            vxbtr(i,j,k)=dvt*(sig*atr(i,j,k)+dtime*(sig*vxbtr(i,j,k)
     &                   -(1.-omegaeta)*jctr(i,j,k)))
            vxbti(i,j,k)=dvt*(sig*ati(i,j,k)+dtime*(sig*vxbti(i,j,k)
     &                   -(1.-omegaeta)*jcti(i,j,k)))
          enddo
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            eta=AVG2RT(etafac,i,j)+AVG2RT(etacel,i,j)
            sig=1./max(eta,1.e-20)
            dvp=rh(i)**2*drh(i)*dth(j)*sth(j)
            agpr(i,j,k)=apr(i,j,k)+dtime*( vxbpr(i,j,k)
     &                                    -eta*jcpr(i,j,k))
            agpi(i,j,k)=api(i,j,k)+dtime*( vxbpi(i,j,k)
     &                                    -eta*jcpi(i,j,k))
            vxbpr(i,j,k)=dvp*(sig*apr(i,j,k)+dtime*(sig*vxbpr(i,j,k)
     &                   -(1.-omegaeta)*jcpr(i,j,k)))
            vxbpi(i,j,k)=dvp*(sig*api(i,j,k)+dtime*(sig*vxbpi(i,j,k)
     &                   -(1.-omegaeta)*jcpi(i,j,k)))
          enddo
        enddo
c
      enddo
c
c ****** Advance the equations for the vector potential A,
c ****** one azimuthal mode at a time.
c
      if (idebug.gt.0) then
        write (9,*)
        write (9,*) '### COMMENT from ADVA:'
        write (9,*) '### CG solution of implicit resistivity'//
     &              ' equations:'
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
      end if
c
      solvtype='a'
      epscgsave=epscg
      epscg=epscga
c
c ****** Set the number of equations to solve in the CG solver.
c
      ncgeq=3*nr*nt
c
c ****** Set the solver type.
c
      solve_type=ST_A
c
      do k=1,npd3
c
        m=k-1
c
        if (m.eq.0) then
c
c ****** M = 0 mode: advance ARR, ATR, and APR.
c
          iri=1
c
          call bcast (acr,0.,nrt)
          call bcast (act,0.,nrt)
          call bcast (acp,0.,nrt)
          call sbca (acr,act,acp,m,iri,1.)
          call curlsq (abr,abt,abp,acr,act,acp,m,iri)
c
c ****** Load the guess to the vector potential A and
c ****** the right hand side.
c
          call loadcga (agrr (1,1,k),agtr (1,1,k),agpr (1,1,k),
     &                  vxbrr(1,1,k),vxbtr(1,1,k),vxbpr(1,1,k),
     &                  abr,abt,abp,xcg,rhs)
c
c ****** Zero the boundary values of A and the rhs.
c
          call zbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3))
          call zbca (rhs(1,1,1),rhs(1,1,2),rhs(1,1,3))
c
c ***** Solve the implicit resistivity equations using the CG solver.
c
          m_a=m
          iri_a=iri
c
          call solve (xcg,rhs,ierr)
c
          if (ierr.ne.0) then
            errmsg='implicit resistivity (m = 0) equations.'
            go to 1000
          end if
c
          if (idebug.gt.0) then
            write (9,900) '### ARR,ATR,APR:',m,ncg,bnrm,rnrm
  900       format (/,1x,a,' M=',i3,' N=',i5,' |B|=',1pe8.2,
     &              ' |R|=',1pe8.2)
          end if
c
c ****** Set the boundary values of A.
c
          call sbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3),m,iri,1.)
c
c ****** Copy the updated A components into the main A arrays.
c
          call scopy (nrt,xcg(1,1,1),1,arr(1,1,k),1)
          call scopy (nrt,xcg(1,1,2),1,atr(1,1,k),1)
          call scopy (nrt,xcg(1,1,3),1,apr(1,1,k),1)
c
        else
c
c ****** M > 0 modes: first, advance ARR, ATR, and API.
c
          iri=1
c
          call bcast (acr,0.,nrt)
          call bcast (act,0.,nrt)
          call bcast (acp,0.,nrt)
          call sbca (acr,act,acp,m,iri,1.)
          call curlsq (abr,abt,abp,acr,act,acp,m,iri)
c
c ****** Load the guess to the vector potential A and
c ****** the right hand side.
c
          call loadcga (agrr (1,1,k),agtr (1,1,k),agpi (1,1,k),
     &                  vxbrr(1,1,k),vxbtr(1,1,k),vxbpi(1,1,k),
     &                  abr,abt,abp,xcg,rhs)
c
c ****** Zero the boundary values of A and the rhs.
c
          call zbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3))
          call zbca (rhs(1,1,1),rhs(1,1,2),rhs(1,1,3))
c
c ***** Solve the implicit resistivity equations using the CG solver.
c
          m_a=m
          iri_a=iri
c
          call solve (xcg,rhs,ierr)
c
          if (ierr.ne.0) then
            errmsg='implicit resistivity (m > 0) equations.'
            go to 1000
          end if
c
          if (idebug.gt.0) then
            write (9,900) '### ARR,ATR,API:',m,ncg,bnrm,rnrm
          end if
c
c ****** Set the boundary values of A.
c
          call sbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3),m,iri,1.)
c
c ****** Copy the updated A components into the main A arrays.
c
          call scopy (nrt,xcg(1,1,1),1,arr(1,1,k),1)
          call scopy (nrt,xcg(1,1,2),1,atr(1,1,k),1)
          call scopy (nrt,xcg(1,1,3),1,api(1,1,k),1)
c
c ****** M > 0 modes: next, advance ARI, ATI, and APR.
c
          iri=2
c
          call bcast (acr,0.,nrt)
          call bcast (act,0.,nrt)
          call bcast (acp,0.,nrt)
          call sbca (acr,act,acp,m,iri,1.)
          call curlsq (abr,abt,abp,acr,act,acp,m,iri)
c
c ****** Load the guess to the vector potential A and
c ****** the right hand side.
c
          call loadcga (agri (1,1,k),agti (1,1,k),agpr (1,1,k),
     &                  vxbri(1,1,k),vxbti(1,1,k),vxbpr(1,1,k),
     &                  abr,abt,abp,xcg,rhs)
c
c ****** Zero the boundary values of A and the rhs.
c
          call zbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3))
          call zbca (rhs(1,1,1),rhs(1,1,2),rhs(1,1,3))
c
c ***** Solve the implicit resistivity equations using the CG solver.
c
          m_a=m
          iri_a=iri
c
          call solve (xcg,rhs,ierr)
c
          if (ierr.ne.0) then
            errmsg='implicit resistivity (m > 0) equations.'
            go to 1000
          end if
c
          if (idebug.gt.0) then
            write (9,900) '### ARI,ATI,APR:',m,ncg,bnrm,rnrm
          end if
c
c ****** Set the boundary values of A.
c
          call sbca (xcg(1,1,1),xcg(1,1,2),xcg(1,1,3),m,iri,1.)
c
c ****** Copy the updated A components into the main A arrays.
c
          call scopy (nrt,xcg(1,1,1),1,ari(1,1,k),1)
          call scopy (nrt,xcg(1,1,2),1,ati(1,1,k),1)
          call scopy (nrt,xcg(1,1,3),1,apr(1,1,k),1)
c
        end if
c
      enddo
c
c ****** Reset epscg
c
      epscg=epscgsave
c
c ****** Get B and J.
c
      call bfroma (arr,ari,atr,ati,apr,api,
     &             bcrr,bcri,bctr,bcti,bcpr,bcpi)
      call jfromb (bcrr,bcri,bctr,bcti,bcpr,bcpi,
     &             jcrr,jcri,jctr,jcti,jcpr,jcpi)
c
c ****** Get time history information from complex B.
c
      call modehst (bcrr,bcri,bctr,bcti,bcpr,bcpi,'b')
c
c ****** Compute the electric field at the r=R0 boundary.
c ****** Note that the electric field arrays hold
c ****** the old value of the vector potential at the boundary.
c
      do k=1,npd3
        do j=1,nt
          avarr=arr(1,j,k)
          avari=ari(1,j,k)
          er0r(j,k)=-(avarr-er0r(j,k))/dtime
          er0i(j,k)=-(avari-er0i(j,k))/dtime
        enddo
        do j=1,ntm1
          avatr=.5*(atr(1,j,k)+atr(2,j,k))
          avati=.5*(ati(1,j,k)+ati(2,j,k))
          et0r(j,k)=-(avatr-et0r(j,k))/dtime
          et0i(j,k)=-(avati-et0i(j,k))/dtime
        enddo
        do j=1,nt
          avapr=.5*(apr(1,j,k)+apr(2,j,k))
          avapi=.5*(api(1,j,k)+api(2,j,k))
          ep0r(j,k)=-(avapr-ep0r(j,k))/dtime
          ep0i(j,k)=-(avapi-ep0i(j,k))/dtime
        enddo
      enddo
c
c ****** Set the dummy points to zero.
c
      do k=1,npd3
        et0r(nt,k)=0.
        et0i(nt,k)=0.
      enddo
c
c ****** Store the electric field at r=R0 in real space in
c ****** commons.
c
      call fft_tp (er0,er0r,er0i,0)
      call fft_tp (et0,et0r,et0i,0)
      call fft_tp (ep0,ep0r,ep0i,0)
c
      return
c
 1000 continue
c
c ****** Error exit: field solver did not converge.
c
      write (*,*)
      write (*,*) '### ERROR in ADVA:'
      write (*,*) '### The field solver did not converge.'
      write (*,*) 'The error occurred while solving the'
      write (*,*) errmsg
      write (*,*) 'Azimuthal mode M = ',m
      write (*,*) 'Real/imaginary part switch IRI = ',iri
      write (*,*) 'NTIME = ',ntime
      write (*,*) 'TIME = ',time
      write (9,*)
      write (9,*) '### ERROR in ADVA:'
      write (9,*) '### The field solver did not converge.'
      write (9,*) 'The error occurred while solving the'
      write (9,*) errmsg
      write (9,*) 'Azimuthal mode M = ',m
      write (9,*) 'Real/imaginary part switch IRI = ',iri
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
      call abortrun
c
      return
      end
c#######################################################################
      subroutine vcrossb (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c-----------------------------------------------------------------------
c
c ****** Compute (v x B).
c
c-----------------------------------------------------------------------
c
c ****** On return, complex (v x B) is returned in array VXB##.
c
c ****** This routine uses v at time n+1/2 from the main v
c ****** arrays, and B from the main B arrays.
c
c ****** The r-boundary points are loaded with the value
c ****** of v x B on the boundary.
c ****** (The theta-boundary points of v x B are not used).
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vxbrr(nr,nt,npd3),vxbri(nr,nt,npd3),
     &     vxbtr(nr,nt,npd3),vxbti(nr,nt,npd3),
     &     vxbpr(nr,nt,npd3),vxbpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Scratch array for (v x B) in real space. 
c
      real vxbr(nr,nt,np),vxbt(nr,nt,np),vxbp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Zero out boundary values of VXB#.
c
      call zb1 (vxbr,np)
      call zb2 (vxbt,np)
      call zb3 (vxbp,np)
c
      do k=1,np
c
c ****** Take care of periodic wrap-around.
c
        if (k.eq.1) then
          km1=np
        else
          km1=k-1
        end if
c
        if (k.eq.np) then
          kp1=1
        else
          kp1=k+1
        end if
c
c ****** R component.
c
        do j=2,ntm1
          do i=1,nrm1
            avvt=AVGRT(vt,i+1,j  ,k)
            avvp=AVGR (vp,i+1,j  ,k)
            cadvt=sign(upwindat,avvt)
            cadvp=sign(upwindap,avvp)
            if (cadvp.ge.0.) then
              avbt=bt(i,j,k)+.5*cadvp*(bt(i,j,km1)-bt(i,j,k))
            else
              avbt=bt(i,j,k)-.5*cadvp*(bt(i,j,kp1)-bt(i,j,k))
            end if
            avbp=ADVT (bp,i  ,j  ,k,cadvt)
            vxbr(i,j,k)=CROSSR(avv,avb)
          enddo
        enddo
c
c ****** Theta component.
c
        do j=2,ntm2
          do i=2,nrm1
            avvr=AVGRT(vr,i  ,j+1,k)
            avvp=AVGT (vp,i  ,j+1,k)
            cadvr=sign(upwindar,avvr)
            cadvp=sign(upwindap,avvp)
            if (cadvp.ge.0.) then
              avbr=br(i,j,k)+.5*cadvp*(br(i,j,km1)-br(i,j,k))
            else
              avbr=br(i,j,k)-.5*cadvp*(br(i,j,kp1)-br(i,j,k))
            end if
            avbp=ADVR (bp,i  ,j  ,k,cadvr)
            vxbt(i,j,k)=CROSST(avv,avb)
          enddo
        enddo
c
c ****** Phi component
c
        do j=2,ntm1
          do i=2,nrm1
            avvr=AVGR (vr,i  ,j  ,k)
            avvt=AVGT (vt,i  ,j  ,k)
            cadvr=sign(upwindar,avvr)
            cadvt=sign(upwindat,avvt)
            avbr=ADVT (br,i  ,j  ,k,cadvt)
            avbt=ADVR (bt,i  ,j  ,k,cadvr)
            vxbp(i,j,k)=CROSSP(avv,avb)
          enddo
        enddo
c
      enddo
c
c ****** Get v x B on the boundaries.
c
c ****** Theta component.
c
      do k=1,np
        do j=2,ntm2
          avvr=AVGT (vr,   1,j+1,k)
          avvp=AVGRT(vp,   2,j+1,k)
          avbr=AVGR (br,   2,j  ,k)
          avbp=AVG  (bp,   1,j  ,k)
          vxbt( 1,j,k)=CROSST(avv,avb)
          avvr=AVGT (vr,nrm1,j+1,k)
          avvp=AVGRT(vp,nr  ,j+1,k)
          avbr=AVGR (br,nr  ,j  ,k)
          avbp=AVG  (bp,nrm1,j  ,k)
          vxbt(nr,j,k)=CROSST(avv,avb)
        enddo
      enddo
c
c ****** Phi component.
c
      do k=1,np
        do j=2,ntm1
          avvr=AVG  (vr,   1,j  ,k)
          avvt=AVGRT(vt,   2,j  ,k)
          avbr=AVGRT(br,   2,j  ,k)
          avbt=AVG  (bt,   1,j  ,k)
          vxbp( 1,j,k)=CROSSP(avv,avb)
          avvr=AVG  (vr,nrm1,j  ,k)
          avvt=AVGRT(vt,nr  ,j  ,k)
          avbr=AVGRT(br,nr  ,j  ,k)
          avbt=AVG  (bt,nrm1,j  ,k)
          vxbp(nr,j,k)=CROSSP(avv,avb)
        enddo
      enddo
c
      call fft (vxbr,vxbrr,vxbri,1)
      call fft (vxbt,vxbtr,vxbti,1)
      call fft (vxbp,vxbpr,vxbpi,1)
c
      return
      end
c#######################################################################
      subroutine setebc (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary values of the tangential electric field.
c
c-----------------------------------------------------------------------
c
c ****** On input, VXB## has complex (v x B).
c
c ****** On return, the boundary values of the electric field
c ****** are placed in the boundary arrays in common /bcs0e/.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vxbrr(nr,nt,npd3),vxbri(nr,nt,npd3),
     &     vxbtr(nr,nt,npd3),vxbti(nr,nt,npd3),
     &     vxbpr(nr,nt,npd3),vxbpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Radial boundary values of the tangential electric field.
c
      common /bcs0e/ ethbcr0(nt,npd3,2),ethbcr1(nt,npd3,2),
     &               ephbcr0(nt,npd3,2),ephbcr1(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
      real ethbcr0re(nt,np),ephbcr0re(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Lower radial boundary.
c
      do k=1,np
        do j=2,ntm2
          avvr=.5*(vrtransr0(j,k)+vrtransr0(j+1,k))
          avvp=.5*(vptransr0(j,k)+vptransr0(j+1,k))
          avbr=AVGR (br,2,j,k)
          avbp=AVG  (bp,1,j,k)
          ethbcr0re(j,k)=-(avvp*avbr-avvr*avbp)
        enddo
        ethbcr0re(   1,k)=0.
        ethbcr0re(ntm1,k)=0.
        ethbcr0re(nt  ,k)=0.
      enddo
c
      do k=1,np
        do j=2,ntm1
          avvr=vrtransr0(j,k)
          avvt=.5*(vttransr0(j,k)+vttransr0(j-1,k))
          avbr=AVGRT(br,2,j,k)
          avbt=AVG  (bt,1,j,k)
          ephbcr0re(j,k)=-(avvr*avbt-avvt*avbr)
        enddo
        ephbcr0re(   1,k)=0.
        ephbcr0re(nt  ,k)=0.
      enddo
c
      call fft_tp (ethbcr0re,ethbcr0(1,1,1),ethbcr0(1,1,2),1)
      call fft_tp (ephbcr0re,ephbcr0(1,1,1),ephbcr0(1,1,2),1)
c
c ****** Upper radial boundary.
c
      do k=1,npd3
        do j=2,ntm2
          ethbcr1(j,k,1)=-vxbtr(nr,j,k)
          ethbcr1(j,k,2)=-vxbti(nr,j,k)
        enddo
      enddo
c
      do k=1,npd3
        do j=2,ntm1
          ephbcr1(j,k,1)=-vxbpr(nr,j,k)
          ephbcr1(j,k,2)=-vxbpi(nr,j,k)
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine aguess (aprr,apri,aptr,apti,appr,appi,
     &                   vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c-----------------------------------------------------------------------
c
c ****** Predict the vector potential by advancing with (v x B).
c
c-----------------------------------------------------------------------
c
c ****** On input, VXB## has complex (v x B).
c
c ****** On return, AP## has the predicted (complex) vector potential.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real aprr(nr,nt,npd3),apri(nr,nt,npd3),
     &     aptr(nr,nt,npd3),apti(nr,nt,npd3),
     &     appr(nr,nt,npd3),appi(nr,nt,npd3)
c
      real vxbrr(nr,nt,npd3),vxbri(nr,nt,npd3),
     &     vxbtr(nr,nt,npd3),vxbti(nr,nt,npd3),
     &     vxbpr(nr,nt,npd3),vxbpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Set the tangential electric field.
c
      call setebc (vxbrr,vxbri,vxbtr,vxbti,vxbpr,vxbpi)
c
c ****** Advance the radial boundary values.
c
      call getabc
c
c ****** Advance the internal values.
c
      do k=1,npd3
c
        do j=2,ntm1
          do i=1,nrm1
            aprr(i,j,k)=arr(i,j,k)+dtime*vxbrr(i,j,k)
            apri(i,j,k)=ari(i,j,k)+dtime*vxbri(i,j,k)
          enddo
        enddo
c
        do j=2,ntm2
          do i=2,nrm1
            aptr(i,j,k)=atr(i,j,k)+dtime*vxbtr(i,j,k)
            apti(i,j,k)=ati(i,j,k)+dtime*vxbti(i,j,k)
          enddo
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            appr(i,j,k)=apr(i,j,k)+dtime*vxbpr(i,j,k)
            appi(i,j,k)=api(i,j,k)+dtime*vxbpi(i,j,k)
          enddo
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine getabc
c
c-----------------------------------------------------------------------
c
c ****** Get the boundary values of the tangential vector potential
c ****** by advancing with the boundary electric field.
c
c-----------------------------------------------------------------------
c
c ****** On return, the boundary values of the vector potential are
c ****** updated in the boundary arrays in common /bcs0a/.
c
c-----------------------------------------------------------------------
c
c ****** The boundary conditions are of the form:
c
c        ATBC1R0(*) AT(b) + ATBC2R0(*) * AT(bn) = ATBVR0(*)
c        ATBC1R1(*) AT(b) + ATBC2R1(*) * AT(bn) = ATBVR1(*)
c
c        APBC1R0(*) AP(b) + APBC2R0(*) * AP(bn) = APBVR0(*)
c        APBC1R1(*) AP(b) + APBC2R1(*) * AP(bn) = APBVR1(*)
c
c ****** where AT(b) and AP(b) are boundary points, and AP(bn)
c ****** and AP(bn) are their normal internal neighbors.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Radial boundary values of the tangential electric field.
c
      common /bcs0e/ ethbcr0(nt,npd3,2),ethbcr1(nt,npd3,2),
     &               ephbcr0(nt,npd3,2),ephbcr1(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on A at r=R0 and r=R1.
c
      common /bcs0a/ atbc1r0(nt,npd3,2),atbc2r0(nt,npd3,2),
     &               atbvr0(nt,npd3,2),
     &               atbc1r1(nt,npd3,2),atbc2r1(nt,npd3,2),
     &               atbvr1(nt,npd3,2),
     &               apbc1r0(nt,npd3,2),apbc2r0(nt,npd3,2),
     &               apbvr0(nt,npd3,2),
     &               apbc1r1(nt,npd3,2),apbc2r1(nt,npd3,2),
     &               apbvr1(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
c ****** Emerging flux tangential electric field at r=R0.
c
      common /efluxcom/ et0ef(nt,npd3,2),ep0ef(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
c ****** Advance the radial boundary values.
c
      do k=1,npd3
c
        do j=2,ntm2
          atbc1r0(j,k,1)=.5*r0/rh(1)
          atbc1r0(j,k,2)=.5*r0/rh(1)
          atbc2r0(j,k,1)=.5*r0/rh(2)
          atbc2r0(j,k,2)=.5*r0/rh(2)
          avatr=.5*r0*(atr(1,j,k)/rh(1)+atr(2,j,k)/rh(2))
          avati=.5*r0*(ati(1,j,k)/rh(1)+ati(2,j,k)/rh(2))
          atbvr0(j,k,1)=avatr-dtime*(ethbcr0(j,k,1)+et0ef(j,k,1))
          atbvr0(j,k,2)=avati-dtime*(ethbcr0(j,k,2)+et0ef(j,k,2))
          atbc1r1(j,k,1)=.5
          atbc1r1(j,k,2)=.5
          atbc2r1(j,k,1)=.5
          atbc2r1(j,k,2)=.5
          avatr=AVGR (atr,nr,j,k)
          avati=AVGR (ati,nr,j,k)
          atbvr1(j,k,1)=avatr-dtime*ethbcr1(j,k,1)
          atbvr1(j,k,2)=avati-dtime*ethbcr1(j,k,2)
        enddo
c
        do j=2,ntm1
          apbc1r0(j,k,1)=.5*r0/rh(1)
          apbc1r0(j,k,2)=.5*r0/rh(1)
          apbc2r0(j,k,1)=.5*r0/rh(2)
          apbc2r0(j,k,2)=.5*r0/rh(2)
          avapr=.5*r0*(apr(1,j,k)/rh(1)+apr(2,j,k)/rh(2))
          avapi=.5*r0*(api(1,j,k)/rh(1)+api(2,j,k)/rh(2))
          apbvr0(j,k,1)=avapr-dtime*(ephbcr0(j,k,1)+ep0ef(j,k,1))
          apbvr0(j,k,2)=avapi-dtime*(ephbcr0(j,k,2)+ep0ef(j,k,2))
          apbc1r1(j,k,1)=.5
          apbc1r1(j,k,2)=.5
          apbc2r1(j,k,1)=.5
          apbc2r1(j,k,2)=.5
          avapr=AVGR (apr,nr,j,k)
          avapi=AVGR (api,nr,j,k)
          apbvr1(j,k,1)=avapr-dtime*ephbcr1(j,k,1)
          apbvr1(j,k,2)=avapi-dtime*ephbcr1(j,k,2)
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine setbca (aprr,apri,aptr,apti,appr,appi)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary conditions on the vector potential in AP##.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real aprr(nr,nt,npd3),apri(nr,nt,npd3),
     &     aptr(nr,nt,npd3),apti(nr,nt,npd3),
     &     appr(nr,nt,npd3),appi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Set the BCs on the m=0 mode.
c
      m=0
c
      call sbca (aprr(1,1,1),aptr(1,1,1),appr(1,1,1),m,1,1.)
c
c ****** Set the BCs on the m > 0 modes.
c
      do k=2,npd3
c
        m=k-1
c
        call sbca (aprr(1,1,k),aptr(1,1,k),appi(1,1,k),m,1,1.)
        call sbca (apri(1,1,k),apti(1,1,k),appr(1,1,k),m,2,1.)
c
      enddo
c
      return
      end
c#######################################################################
      subroutine sbca (acr,act,acp,m,iri,vmask)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary conditions on the appropriate components
c ****** of A for azimuthal mode M, as expressed by switch IRI,
c ****** as follows.
c
c ****** When m = 0, set BCs on ARR, ATR, and APR.
c ****** When m > 0, set BCs on ARR, ATR, and API when IRI=1,
c ****** and on ARI, ATI, and APR when IRI=2.
c
c ****** VMASK multiplies the boundary value.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real acr(nr,nt),act(nr,nt),acp(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on A at r=R0 and r=R1.
c
      common /bcs0a/ atbc1r0(nt,npd3,2),atbc2r0(nt,npd3,2),
     &               atbvr0(nt,npd3,2),
     &               atbc1r1(nt,npd3,2),atbc2r1(nt,npd3,2),
     &               atbvr1(nt,npd3,2),
     &               apbc1r0(nt,npd3,2),apbc2r0(nt,npd3,2),
     &               apbvr0(nt,npd3,2),
     &               apbc1r1(nt,npd3,2),apbc2r1(nt,npd3,2),
     &               apbvr1(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
      k=m+1
c
c ****** Set the real/imaginary part switches.
c
      if (m.eq.0) then
        irit=1
        irip=1
      else
        if (iri.eq.1) then
          irit=1
          irip=2
          ipm=1
        else
          irit=2
          irip=1
          ipm=-1
        end if
      end if
c
c ****** Radial boundaries.
c
      do j=2,ntm2
        act( 1,j)=( vmask*atbvr0(j,k,irit)
     &             -atbc2r0(j,k,irit)*act(   2,j))
     &            /atbc1r0(j,k,irit)
        act(nr,j)=( vmask*atbvr1(j,k,irit)
     &             -atbc2r1(j,k,irit)*act(nrm1,j))
     &            /atbc1r1(j,k,irit)
      enddo
c
      do j=2,ntm1
        acp( 1,j)=( vmask*apbvr0(j,k,irip)
     &             -apbc2r0(j,k,irip)*acp(   2,j))
     &            /apbc1r0(j,k,irip)
        acp(nr,j)=( vmask*apbvr1(j,k,irip)
     &             -apbc2r1(j,k,irip)*acp(nrm1,j))
     &            /apbc1r1(j,k,irip)
      enddo
c
c ****** Theta boundaries.
c
      if (m.eq.0) then
c
        do i=1,nrm1
          acr(i,   1)=acr(i,   2)
          acr(i,  nt)=acr(i,ntm1)
        enddo
        do i=1,nr
          act(i,   1)=0.
          act(i,ntm1)=0.
        enddo
        do i=1,nr
          acp(i,   1)=-acp(i,   2)
          acp(i,  nt)=-acp(i,ntm1)
        enddo
c
      else if (m.eq.1) then
c
        do i=1,nrm1
          acr(i,   1)=-acr(i,   2)
          acr(i,  nt)=-acr(i,ntm1)
        enddo
        do i=1,nr
          act(i,   1)= ipm*acp(i,   2)
          act(i,ntm1)=-ipm*acp(i,ntm1)
        enddo
        do i=1,nr
          acp(i,   1)=acp(i,   2)
          acp(i,  nt)=acp(i,ntm1)
        enddo
c
      else if (m.eq.2) then
c
        do i=1,nrm1
          acr(i,   1)=acr(i,   2)
          acr(i,  nt)=acr(i,ntm1)
        enddo
        do i=1,nr
          act(i,   1)=0.
          act(i,ntm1)=0.
        enddo
        do i=1,nr
          acp(i,   1)=-acp(i,   2)
          acp(i,  nt)=-acp(i,ntm1)
        enddo
c
      else
c
        do i=1,nrm1
          acr(i,   1)=acr(i,   2)
          acr(i,  nt)=acr(i,ntm1)
        enddo
        do i=1,nr
          act(i,   1)=0.
          act(i,ntm1)=0.
        enddo
        do i=1,nr
          acp(i,   1)=acp(i,   2)
          acp(i,  nt)=acp(i,ntm1)
        enddo
c
      end if
c
c ****** Zero out the dummy points.
c
      do j=1,nt
        acr(nr,j)=0.
      enddo
c
      do i=1,nr
        act(i,nt)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine zbca (acr,act,acp)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary points of A to zero.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real acr(nr,nt),act(nr,nt),acp(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary points.
c
      do i=1,nrm1
        acr(i, 1)=0.
        acr(i,nt)=0.
      enddo
c
      do j=2,ntm2
        act( 1,j)=0.
        act(nr,j)=0.
      enddo
      do i=1,nr
        act(i,   1)=0.
        act(i,ntm1)=0.
      enddo
c
      do j=2,ntm1
        acp( 1,j)=0.
        acp(nr,j)=0.
      enddo
      do i=1,nr
        acp(i, 1)=0.
        acp(i,nt)=0.
      enddo
c
c ****** Dummy points.
c
      do j=1,nt
        acr(nr,j)=0.
      enddo
c
      do i=1,nr
        act(i,nt)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine loadcga (acr,act,acp,scr,sct,scp,
     &                    abr,abt,abp,xcg,rhs)

c-----------------------------------------------------------------------
c
c ****** Load the initial guess for the vector potential into XCG,
c ****** and set the right-hand-side into RHS.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real acr(nr,nt),act(nr,nt),acp(nr,nt)
      real scr(nr,nt),sct(nr,nt),scp(nr,nt)
      real abr(nr,nt),abt(nr,nt),abp(nr,nt)
      real xcg(nr,nt,3),rhs(nr,nt,3)
c
c-----------------------------------------------------------------------
c
c ****** Set the initial guess.
c
      do j=1,nt
        do i=1,nr
          xcg(i,j,1)=acr(i,j)
          xcg(i,j,2)=act(i,j)
          xcg(i,j,3)=acp(i,j)
        enddo
      enddo
c
c ****** Set the rhs.
c
      do j=1,nt
        do i=1,nr
          rhs(i,j,1)=scr(i,j)-abr(i,j)
          rhs(i,j,2)=sct(i,j)-abt(i,j)
          rhs(i,j,3)=scp(i,j)-abp(i,j)
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine bfroma (acrr,acri,actr,acti,acpr,acpi,
     &                   bcrr,bcri,bctr,bcti,bcpr,bcpi)
c
c-----------------------------------------------------------------------
c
c ****** Get the magnetic field from the vector potential.
c
c-----------------------------------------------------------------------
c
c ****** On input the array AC## contains complex A.
c
c ****** Upon return, BC## contains complex B.
c
c ****** The (real) magnetic field is updated in the main B arrays.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real acrr(nr,nt,npd3),acri(nr,nt,npd3),
     &     actr(nr,nt,npd3),acti(nr,nt,npd3),
     &     acpr(nr,nt,npd3),acpi(nr,nt,npd3)
c
      real bcrr(nr,nt,npd3),bcri(nr,nt,npd3),
     &     bctr(nr,nt,npd3),bcti(nr,nt,npd3),
     &     bcpr(nr,nt,npd3),bcpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Zero out unused boundary values.
c
      call zb2 (bcrr,npd3)
      call zb2 (bcri,npd3)
      call zb1 (bctr,npd3)
      call zb1 (bcti,npd3)
      call zb4 (bcpr,npd3)
      call zb4 (bcpi,npd3)
c
      do k=1,npd3
c
        m=k-1
c
        do j=2,ntm2
          do i=1,nr
            bcrr(i,j,k)=(( sth(j+1)*acpr(i,j+1,k)
     &                    -sth(j  )*acpr(i,j  ,k))/dt(j)
     &                   +fm(k)*acti(i,j,k))/(rh(i)*st(j))
            bcri(i,j,k)=( ( sth(j+1)*acpi(i,j+1,k)
     &                     -sth(j  )*acpi(i,j  ,k))/dt(j)
     &                   -fm(k)*actr(i,j,k))/(rh(i)*st(j))
          enddo
        enddo
c
        do j=2,ntm1
          do i=1,nrm1
            bctr(i,j,k)=(-fm(k)*acri(i,j,k)/sth(j)
     &                  -( rh(i+1)*acpr(i+1,j,k)
     &                    -rh(i  )*acpr(i  ,j,k))/dr(i))/r(i)
            bcti(i,j,k)=( fm(k)*acrr(i,j,k)/sth(j)
     &                  -( rh(i+1)*acpi(i+1,j,k)
     &                    -rh(i  )*acpi(i  ,j,k))/dr(i))/r(i)
          enddo
        enddo
c
        do j=2,ntm2
          do i=1,nrm1
            bcpr(i,j,k)=(( rh(i+1)*actr(i+1,j,k)
     &                    -rh(i  )*actr(i  ,j,k))/dr(i)
     &                   -(acrr(i,j+1,k)-acrr(i,j,k))/dt(j))/r(i)
            bcpi(i,j,k)=(( rh(i+1)*acti(i+1,j,k)
     &                    -rh(i  )*acti(i  ,j,k))/dr(i)
     &                   -(acri(i,j+1,k)-acri(i,j,k))/dt(j))/r(i)
          enddo
        enddo
c
c ****** Set boundary points at theta = 0, pi.
c
        if (m.eq.0) then
          do i=1,nr
            bcrr(i,   1,k)=2.*(acpr(i,   2,k)-acpr(i,   1,k))
     &                     /(rh(i)*dt(   1))
            bcrr(i,ntm1,k)=2.*(acpr(i,  nt,k)-acpr(i,ntm1,k))
     &                     /(rh(i)*dt(ntm1))
            bcri(i,   1,k)=2.*(acpi(i,   2,k)-acpi(i,   1,k))
     &                     /(rh(i)*dt(   1))
            bcri(i,ntm1,k)=2.*(acpi(i,  nt,k)-acpi(i,ntm1,k))
     &                     /(rh(i)*dt(ntm1))
          enddo
          do i=1,nrm1
            bctr(i,   1,k)=-bctr(i,   2,k)
            bctr(i,  nt,k)=-bctr(i,ntm1,k)
            bcti(i,   1,k)=-bcti(i,   2,k)
            bcti(i,  nt,k)=-bcti(i,ntm1,k)
          enddo
          do i=1,nrm1
            bcpr(i,   1,k)=0.
            bcpr(i,ntm1,k)=0.
            bcpi(i,   1,k)=0.
            bcpi(i,ntm1,k)=0.
          enddo
        else if (m.eq.1) then
          do i=1,nr
            bcrr(i,   1,k)=0.
            bcrr(i,ntm1,k)=0.
            bcri(i,   1,k)=0.
            bcri(i,ntm1,k)=0.
          enddo
          do i=1,nrm1
            bctr(i,   1,k)=bctr(i,   2,k)
            bctr(i,  nt,k)=bctr(i,ntm1,k)
            bcti(i,   1,k)=bcti(i,   2,k)
            bcti(i,  nt,k)=bcti(i,ntm1,k)
          enddo
          do i=1,nrm1
            bcpr(i,   1,k)=-bcti(i,   2,k)
            bcpr(i,ntm1,k)= bcti(i,ntm1,k)
            bcpi(i,   1,k)= bctr(i,   2,k)
            bcpi(i,ntm1,k)=-bctr(i,ntm1,k)
          enddo
        else if (m.eq.2) then
          do i=1,nr
            bcrr(i,   1,k)=0.
            bcrr(i,  nt,k)=0.
            bcri(i,   1,k)=0.
            bcri(i,  nt,k)=0.
          enddo
          do i=1,nrm1
            bctr(i,   1,k)=-bctr(i,   2,k)
            bctr(i,  nt,k)=-bctr(i,ntm1,k)
            bcti(i,   1,k)=-bcti(i,   2,k)
            bcti(i,  nt,k)=-bcti(i,ntm1,k)
          enddo
          do i=1,nrm1
            bcpr(i,   1,k)=0.
            bcpr(i,ntm1,k)=0.
            bcpi(i,   1,k)=0.
            bcpi(i,ntm1,k)=0.
          enddo
        else
          do i=1,nr
            bcrr(i,   1,k)=0.
            bcrr(i,  nt,k)=0.
            bcri(i,   1,k)=0.
            bcri(i,  nt,k)=0.
          enddo
          do i=1,nrm1
            bctr(i,   1,k)=bctr(i,   2,k)
            bctr(i,  nt,k)=bctr(i,ntm1,k)
            bcti(i,   1,k)=bcti(i,   2,k)
            bcti(i,  nt,k)=bcti(i,ntm1,k)
          enddo
          do i=1,nrm1
            bcpr(i,   1,k)=0.
            bcpr(i,ntm1,k)=0.
            bcpi(i,   1,k)=0.
            bcpi(i,ntm1,k)=0.
          enddo
        end if
c
      enddo
c
c ****** Add in the (artificial monopole) radial field.
c
      if (br00.ne.0.) then
        do j=1,ntm1
          do i=1,nr
            bcrr(i,j,1)=bcrr(i,j,1)+br00*(r0/rh(i))**2
          enddo
        enddo
      end if
c
      call fft (br,bcrr,bcri,0)
      call fft (bt,bctr,bcti,0)
      call fft (bp,bcpr,bcpi,0)
c
      return
      end
c#######################################################################
      subroutine jfromb (bcrr,bcri,bctr,bcti,bcpr,bcpi,
     &                   jcrr,jcri,jctr,jcti,jcpr,jcpi)
c
c-----------------------------------------------------------------------
c
c ****** Get the current density from the magnetic field.
c
c-----------------------------------------------------------------------
c
c ****** Obtain J (real) from B (complex).
c
c ****** On input the array BC## is assumed to contain complex B.
c
c ****** Upon return, the real current density is placed in
c ****** the main FJR, FJT, FJP arrays, while JC## contains complex J.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real bcrr(nr,nt,npd3),bcri(nr,nt,npd3),
     &     bctr(nr,nt,npd3),bcti(nr,nt,npd3),
     &     bcpr(nr,nt,npd3),bcpi(nr,nt,npd3)
c
      real jcrr(nr,nt,npd3),jcri(nr,nt,npd3),
     &     jctr(nr,nt,npd3),jcti(nr,nt,npd3),
     &     jcpr(nr,nt,npd3),jcpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Zero out unused boundary values.
c
      call zb1 (jcrr,npd3)
      call zb1 (jcri,npd3)
      call zb2 (jctr,npd3)
      call zb2 (jcti,npd3)
c
      do k=1,npd3
c
        m=k-1
c
        do j=2,ntm1
          do i=1,nrm1
            jcrr(i,j,k)=(( st(j  )*bcpr(i,j  ,k)
     &                    -st(j-1)*bcpr(i,j-1,k))/dth(j)
     &                   +fm(k)*bcti(i,j,k))/(r(i)*sth(j))
            jcri(i,j,k)=(( st(j  )*bcpi(i,j  ,k)
     &                    -st(j-1)*bcpi(i,j-1,k))/dth(j)
     &                   -fm(k)*bctr(i,j,k))/(r(i)*sth(j))
          enddo
        enddo
c
        do j=2,ntm2
          do i=2,nrm1
            jctr(i,j,k)=(-fm(k)*bcri(i,j,k)/st(j)
     &                  -( r(i  )*bcpr(i  ,j,k)
     &                    -r(i-1)*bcpr(i-1,j,k))/drh(i))/rh(i)
            jcti(i,j,k)=( fm(k)*bcrr(i,j,k)/st(j)
     &                  -( r(i  )*bcpi(i  ,j,k)
     &                    -r(i-1)*bcpi(i-1,j,k))/drh(i))/rh(i)
          enddo
          jctr( 1,j,k)=EXTRAPR0(jctr,   2,j,k)
          jcti( 1,j,k)=EXTRAPR0(jcti,   2,j,k)
          jctr(nr,j,k)=EXTRAPR1(jctr,nrm1,j,k)
          jcti(nr,j,k)=EXTRAPR1(jcti,nrm1,j,k)
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            jcpr(i,j,k)=(( r(i  )*bctr(i  ,j,k)
     &                    -r(i-1)*bctr(i-1,j,k))/drh(i)
     &                   -(bcrr(i,j,k)-bcrr(i,j-1,k))/dth(j))/rh(i)
            jcpi(i,j,k)=(( r(i  )*bcti(i  ,j,k)
     &                    -r(i-1)*bcti(i-1,j,k))/drh(i)
     &                   -(bcri(i,j,k)-bcri(i,j-1,k))/dth(j))/rh(i)
          enddo
          jcpr( 1,j,k)=EXTRAPR0(jcpr,   2,j,k)
          jcpi( 1,j,k)=EXTRAPR0(jcpi,   2,j,k)
          jcpr(nr,j,k)=EXTRAPR1(jcpr,nrm1,j,k)
          jcpi(nr,j,k)=EXTRAPR1(jcpi,nrm1,j,k)
        enddo
c
c ****** Set boundary points at theta = 0, pi.
c
        if (m.eq.0) then
          do i=1,nrm1
            jcrr(i,   1,k)=jcrr(i,   2,k)
            jcrr(i,  nt,k)=jcrr(i,ntm1,k)
            jcri(i,   1,k)=jcri(i,   2,k)
            jcri(i,  nt,k)=jcri(i,ntm1,k)
          enddo
          do i=1,nr
            jctr(i,   1,k)=0.
            jctr(i,ntm1,k)=0.
            jcti(i,   1,k)=0.
            jcti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            jcpr(i,   1,k)=-jcpr(i,   2,k)
            jcpr(i,  nt,k)=-jcpr(i,ntm1,k)
            jcpi(i,   1,k)=-jcpi(i,   2,k)
            jcpi(i,  nt,k)=-jcpi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nrm1
            jcrr(i,   1,k)=-jcrr(i,   2,k)
            jcrr(i,  nt,k)=-jcrr(i,ntm1,k)
            jcri(i,   1,k)=-jcri(i,   2,k)
            jcri(i,  nt,k)=-jcri(i,ntm1,k)
          enddo
          do i=1,nr
            jctr(i,   1,k)= jcpi(i,   2,k)
            jctr(i,ntm1,k)=-jcpi(i,ntm1,k)
            jcti(i,   1,k)=-jcpr(i,   2,k)
            jcti(i,ntm1,k)= jcpr(i,ntm1,k)
          enddo
          do i=1,nr
            jcpr(i,   1,k)=jcpr(i,   2,k)
            jcpr(i,  nt,k)=jcpr(i,ntm1,k)
            jcpi(i,   1,k)=jcpi(i,   2,k)
            jcpi(i,  nt,k)=jcpi(i,ntm1,k)
          enddo
        else if (m.eq.2) then
          do i=1,nrm1
            jcrr(i,   1,k)=jcrr(i,   2,k)
            jcrr(i,  nt,k)=jcrr(i,ntm1,k)
            jcri(i,   1,k)=jcri(i,   2,k)
            jcri(i,  nt,k)=jcri(i,ntm1,k)
          enddo
          do i=1,nr
            jctr(i,   1,k)=0.
            jctr(i,ntm1,k)=0.
            jcti(i,   1,k)=0.
            jcti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            jcpr(i,   1,k)=-jcpr(i,   2,k)
            jcpr(i,  nt,k)=-jcpr(i,ntm1,k)
            jcpi(i,   1,k)=-jcpi(i,   2,k)
            jcpi(i,  nt,k)=-jcpi(i,ntm1,k)
          enddo
        else
          do i=1,nrm1
            jcrr(i,   1,k)=jcrr(i,   2,k)
            jcrr(i,  nt,k)=jcrr(i,ntm1,k)
            jcri(i,   1,k)=jcri(i,   2,k)
            jcri(i,  nt,k)=jcri(i,ntm1,k)
          enddo
          do i=1,nr
            jctr(i,   1,k)=0.
            jctr(i,ntm1,k)=0.
            jcti(i,   1,k)=0.
            jcti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            jcpr(i,   1,k)=jcpr(i,   2,k)
            jcpr(i,  nt,k)=jcpr(i,ntm1,k)
            jcpi(i,   1,k)=jcpi(i,   2,k)
            jcpi(i,  nt,k)=jcpi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      call fft (fjr,jcrr,jcri,0)
      call fft (fjt,jctr,jcti,0)
      call fft (fjp,jcpr,jcpi,0)
c
      return
      end
c#######################################################################
      subroutine advtemp
c
c-----------------------------------------------------------------------
c
c ****** Advance the temperature using the energy equation.
c
c-----------------------------------------------------------------------
c
      use solver_types
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real deriv(nr,nt,np),tre(nr,nt,np),t2re(nr,nt,np),
     &     derivr(nr,nt,npd3),derivi(nr,nt,npd3),
     &     tpr(nr,nt,npd3),tpi(nr,nt,npd3),
     &     tp2r(nr,nt,npd3),tp2i(nr,nt,npd3),
     &     pr0re(nt,np),rhor0re(nt,np),
     &     pr1re(nt,np),rhor1re(nt,np),
     &     vparr(nr,nt,np),vpart(nr,nt,np),
     &     vparp(nr,nt,np)
c
      real tc(nr,nt,np),tb(nr,nt,np)
c
      real scr(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Parallel thermal conductivity arrays.
c
      common /fkparcom/ fkrr(nr,nt,np),
     &                  fktt(nr,nt,np),
     &                  fkpp(nr,nt,np),
     &                  fkrt(nr,nt,np),
     &                  fkrp(nr,nt,np),
     &                  fktp(nr,nt,np), dqdt(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Saved electron density from the transition-region/
c ****** chromospheric radiation balance condition.
c
      real ne0,nerad
      common /radbc1/ ne0(nt,np),nerad(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** CG work arrays
c
      real xcg(nr,nt,np),rhs(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      logical first
      save first
      data first/.true./
c
c-----------------------------------------------------------------------
c
      logical firstch
      save firstch,nodes_ch
      data firstch/.true./
c
c-----------------------------------------------------------------------
c
      logical firstnc
      save firstnc
      data firstnc/.true./
c
c-----------------------------------------------------------------------
c
c ****** Initialize quantities.
c ****** This may be unnecessary.  Needs to be checked!
c
      deriv=0.
      derivr=0.
      derivi=0.
c
c ****** These initializations are necessary.
c
      tpr=tr
      tpi=ti
      tre=temp
c
c ****** Initialize RHORAD.
c ****** (This is used when the thermal conductivity is zero.)
c
      do k=1,np
        do j=1,nt
          rhorad(j,k)=rhobcr00(j,k)
        enddo
      enddo
c
c ****** Set the outer radial boundary values of p.
c
      if (bcr1type.eq.'open') then
        do k=1,np
          do j=1,nt
            vr0=max(vr(nrm1,j,k),0.)
            fac=vr0*dtime/dr(nrm1)
            pr1re(j,k)=(1.-fac)*p(nr,j,k)+fac*p(nrm1,j,k)
          enddo
        enddo
        call fft_tp (pr1re,pbcr1(1,1,1),pbcr1(1,1,2),1)
      end if
c
c ****** Set the boundary conditions on T at r=R1 from those on
c ****** p and rho.
c
      call fft_tp (pr1re  ,pbcr1  (1,1,1),pbcr1  (1,1,2),0)
      call fft_tp (rhor1re,rhobcr1(1,1,1),rhobcr1(1,1,2),0)
c
      do k=1,np
        tbcr1re( 1,k)=0.
        do j=2,ntm1
          if (tbc1.ne.0.) then
            tbcr1re(j,k)=tbc1/fn_t
          else
            tbcr1re(j,k)=he_rho*pr1re(j,k)/(he_p*rhor1re(j,k))
          end if
        enddo
        tbcr1re(nt,k)=0.
      enddo
c
      call fft_tp (tbcr1re,tbcr1  (1,1,1),tbcr1  (1,1,2),1)
c
c ****** Set a fixed temperature at r=R0.
c
      do k=1,np
        do j=1,nt
          tbcr0re(j,k)=tbc0/fn_t
        enddo
      enddo
c
      call fft_tp (tbcr0re,tbcr0  (1,1,1),tbcr0  (1,1,2),1)

      if (tcond.ne.0..and.alpha_nocoll.ne.0.) then
c
c ****** Get the parallel velocity.
c
        call getvpar (vparr,vpart,vparp)
c
c ****** On the first time in, set the profile
c ****** for the collisionless heat flux.
c
        if (firstnc) then
          do i=1,nr
            s=.5*(1.+tanh((rh(i)-tc_nocoll_r)/tc_nocoll_dr))
            prof_nocoll(i)=s
          enddo
          firstnc=.false.
        end if
c
        do k=1,np
          do j=1,nt
            do i=1,nr
              fne=rho(i,j,k)/he_rho
              t2re(i,j,k)=fne*temp(i,j,k)*prof_nocoll(i)
            enddo
          enddo
        enddo
c
        call fft (t2re,tp2r,tp2i,1)
c
      else
c
        t2re=0.
        tp2r=0.
        tp2i=0.
c
      end if
c
c ****** Predictor step.
c
      if (ifpc.ne.0) then
        if (predflow) then
c
c ****** Flow in the predictor
c
          call divqv (tr,ti,vr,vt,vp,temp,derivr,derivi,deriv)
          do k=1,npd3
            do j=2,ntm1
              do i=2,nrm1
                tpr(i,j,k)=tr(i,j,k)-derivr(i,j,k)*dtime
                tpi(i,j,k)=ti(i,j,k)-derivi(i,j,k)*dtime
              enddo
            enddo
          enddo
c
c ****** Add in the collisionless heat flux if
c ****** requested (ALPHA_NOCOLL.ne.0.).
c
          if (tcond.ne.0..and.alpha_nocoll.ne.0.) then
c
            call divqv (tp2r,tp2i,vparr,vpart,vparp,t2re,
     &      derivr,derivi,deriv)
            do k=1,np
              do j=2,ntm1
                do i=2,nrm1
                  fne=rho(i,j,k)/he_rho
                  deriv(i,j,k)=
     &            alpha_nocoll*deriv(i,j,k)/(fne*he_p)
                enddo
              enddo
            enddo
            call fft (deriv,derivr,derivi,1)
            do k=1,npd3
              do j=2,ntm1
                do i=2,nrm1
                  tpr(i,j,k)=tpr(i,j,k)-
     &            derivr(i,j,k)*dtime
                  tpi(i,j,k)=tpi(i,j,k)-
     &            derivi(i,j,k)*dtime
                enddo
              enddo
            enddo
          endif
c
          wavefac=1.
c
        else
c
          wavefac=0.
c
        endif
c
c ****** Wave terms in the predictor
c
        call qdivv (temp,vr,vt,vp,derivr,derivi,deriv)
        call fft (deriv,derivr,derivi,1)

        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              tpr(i,j,k)=tpr(i,j,k)-derivr(i,j,k)*dtime*(betapc_p
     &                   *(gammarh(i)-1.)-wavefac)
              tpi(i,j,k)=tpi(i,j,k)-derivi(i,j,k)*dtime*(betapc_p
     &                   *(gammarh(i)-1.)-wavefac)
            enddo
          enddo
        enddo
c
        if (tcond.ne.0..and.alpha_nocoll.ne.0.) then
          call qdivv (t2re,vparr,vpart,vparp,derivr,derivi,deriv)
          do k=1,np
            do j=2,ntm1
              do i=2,nrm1
                fne=rho(i,j,k)/he_rho
                deriv(i,j,k)=
     &          alpha_nocoll*deriv(i,j,k)/(fne*he_p)
              enddo
            enddo
          enddo
          call fft (deriv,derivr,derivi,1)
          do k=1,npd3
            do j=2,ntm1
              do i=2,nrm1
                tpr(i,j,k)=tpr(i,j,k)-(betapc_p-wavefac)*
     &          derivr(i,j,k)*dtime
                tpi(i,j,k)=tpi(i,j,k)-(betapc_p-wavefac)*
     &          derivi(i,j,k)*dtime
              enddo
            enddo
          enddo
        endif
c
        call setbct (tpr,tpi)
        call fft (tre,tpr,tpi,0)
c
      endif
c
c ****** Corrector step.
c
      call divqv (tpr,tpi,vr,vt,vp,scr,derivr,derivi,deriv)

      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            tr(i,j,k)=tr(i,j,k)-derivr(i,j,k)*dtime
            ti(i,j,k)=ti(i,j,k)-derivi(i,j,k)*dtime
          enddo
        enddo
      enddo

      if (tcond.ne.0..and.alpha_nocoll.ne.0.) then
c
        do k=1,np
          do j=1,nt
            do i=1,nr
              fne=rho(i,j,k)/he_rho
              tre(i,j,k)=fne*tre(i,j,k)*prof_nocoll(i)
            enddo
          enddo
        enddo
c
        call fft (tre,tpr,tpi,1)
c
        call divqv (tpr,tpi,vparr,vpart,vparp,scr,
     &  derivr,derivi,deriv)

        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              fne=rho(i,j,k)/he_rho
              deriv(i,j,k)=
     &        alpha_nocoll*deriv(i,j,k)/(fne*he_p)
            enddo
          enddo
        enddo

        call fft (deriv,derivr,derivi,1)

        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              tr(i,j,k)=tr(i,j,k)-
     &        derivr(i,j,k)*dtime
              ti(i,j,k)=ti(i,j,k)-
     &        derivi(i,j,k)*dtime
            enddo
          enddo
        enddo
c
      endif
c
c ****** Get the (2-gamma)*T*div(v) part.
c
      if (upwindphi.ge.0.) then
        do k=1,np
          if (k.eq.1) then
            km1=np
          else
            km1=k-1
          end if
          if (k.eq.np) then
            kp1=1
          else
            kp1=k+1
          end if
          do j=2,ntm1
            do i=2,nrm1
              deriv(i,j,k)= ( r(i  )**2*vr(i  ,j,k)
     &                       -r(i-1)**2*vr(i-1,j,k)
     &                      )/(rh(i)**2*drh(i))
     &                     +( st(j  )*vt(i,j  ,k)
     &                       -st(j-1)*vt(i,j-1,k)
     &                      )/(rh(i)*sth(j)*dth(j))
     &                     +( vp(i,j,kp1)
     &                       -vp(i,j,km1)
     &                      )/(2.*dphi*rh(i)*sth(j))
            enddo
          enddo
        enddo
      else
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              derivr(i,j,k)= ( r(i  )**2*vrr(i  ,j,k)
     &                      -r(i-1)**2*vrr(i-1,j,k)
     &                     )/(rh(i)**2*drh(i))
     &                    +( st(j  )*vtr(i,j  ,k)
     &                      -st(j-1)*vtr(i,j-1,k)
     &                     )/(rh(i)*sth(j)*dth(j))
     &                    -fm(k)*vpi(i,j,k)/(rh(i)*sth(j))
              derivi(i,j,k)= ( r(i  )**2*vri(i  ,j,k)
     &                      -r(i-1)**2*vri(i-1,j,k)
     &                     )/(rh(i)**2*drh(i))
     &                    +( st(j  )*vti(i,j  ,k)
     &                      -st(j-1)*vti(i,j-1,k)
     &                     )/(rh(i)*sth(j)*dth(j))
     &                    +fm(k)*vpr(i,j,k)/(rh(i)*sth(j))
            enddo
          enddo
        enddo
c
        call zb3 (derivr,npd3)
        call zb3 (derivi,npd3)
c
        call fft (deriv,derivr,derivi,0)
      end if
c
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            deriv(i,j,k)=(2.-gammarh(i))*temp(i,j,k)*deriv(i,j,k)
          enddo
        enddo
      enddo

c
c
c ****** Add in the ohmic heating term.
c
      if (ifohmic.ne.0) then
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              avjr=AVGR (fjr,i,j,k)
              avjt=AVGT (fjt,i,j,k)
              avjp=AVG  (fjp,i,j,k)
              fjsq=avjr**2+avjt**2+avjp**2
              eta=AVG2RT(etafac,i,j)+AVG2RT(etacel,i,j)
              fne=rho(i,j,k)/he_rho
              deriv(i,j,k)=deriv(i,j,k)+
     &        (gammarh(i)-1.)*eta*fjsq/(he_p*fne)
            enddo
          enddo
        enddo
      end if
c
c ****** Add in the viscous heating term if requested.
c ****** ###### Note that this assumes that m=0 only. ######
c
      if (ifhvisc.ne.0) then
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              rnu   = AVG2RT(visfac,i,j) + AVG2RT(viscel,i,j)
              avvr  = AVGR (vr ,i  ,j  ,k)
              avvt  = AVGT (vt ,i  ,j  ,k)
              avvp  = AVG  (vp ,i  ,j  ,k)
              avvrp = AVGRT(vr ,i  ,j+1,k)
              avvrm = AVGRT(vr ,i  ,j  ,k)
              avvtp = AVGRT(vt ,i+1,j  ,k)
              avvtm = AVGRT(vt ,i  ,j  ,k)
              dvrdr = (vr(i,j,k) - vr(i-1,j  ,k) )/drh(i)
              dvtdt = (vt(i,j,k) - vt(i,  j-1,k) )/dth(j)
              dvrdt = (avvrp - avvrm)/dth(j)
              dvtdr = (avvtp - avvtm)/drh(i)
              dvpdr = (vp(i+1,j,k) - vp(i-1,j,k))/( dr(i) + dr(i-1) )
              dvpdt = (vp(i,j+1,k) - vp(i,j-1,k))/( dt(j) + dt(j-1) )
              deriv(i,j,k)=deriv(i,j,k)
     &                 +(gammarh(i)-1.)*rnu*(he_rho/he_p)*
     &                  (dvrdr**2+2.*dvtdr*(dvrdt-avvt)/rh(i)-
     &                  2.*avvp*dvpdr/rh(i) +
     &                  (dvtdt/rh(i)+avvr/rh(i))**2-
     &                  2.*cth(j)*avvp*dvpdt/( rh(i)**2*sth(j)) +
     &                  (avvr/rh(i)+cth(j)*avvt/(rh(i)*sth(j)))**2)
            enddo
          enddo
        enddo
      end if
c
c ****** Add in the coronal heating term.
c
c ****** On the first time in, check TNODE_CH, and load the
c ****** latitudinal coronal heating profile into array THPROF.
c
      if (firstch) then
c
c ****** Find the number of nodes specified.
c
        do i=1,mxtnodes
          if (tnode_ch(i).lt.0.) go to 101
        enddo
  101   nodes_ch=i-1
c
c ****** Check the time nodes.
c
        dummy=flint(0.,nodes_ch,tnode_ch,hlen_eq_ch,1,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ADVTEMP:'
          write (*,*) '### Inconsistent input data in array TNODE_CH.'
          write (9,*)
          write (9,*) '### ERROR in ADVTEMP:'
          write (9,*) '### Inconsistent input data in array TNODE_CH.'
          call exit (1)
        end if
c
c ****** Calculate the latitudinal coronal heating profile.
c
        call load_thprof
        call load_awthprof
c
        firstch=.false.
c
      end if
c
c ****** Interpolate the coronal heating function parameters
c ****** to the present time.
c
      hlen_eq=flint(time,nodes_ch,tnode_ch,hlen_eq_ch,0,ierr)
      hlen_pole=flint(time,nodes_ch,tnode_ch,hlen_pole_ch,0,ierr)
      q0_eq=flint(time,nodes_ch,tnode_ch,q0_eq_ch,0,ierr)
      q0_pole=flint(time,nodes_ch,tnode_ch,q0_pole_ch,0,ierr)
      q0phys=flint(time,nodes_ch,tnode_ch,q0phys_ch,0,ierr)
      pw0=flint(time,nodes_ch,tnode_ch,qalfven_ch,0,ierr)
      q0=q0phys/fn_q0
c
      if (ifbheat.eq.1) then
c
        call geth0_bheat (q0_pole,q0_eq,hlen_pole,hlen_eq,q0,h0)
c
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              hlen=hlen_pole+(hlen_eq-hlen_pole)*(1.-brbyb2(j,k))
              faclat=q0_pole+(q0_eq-q0_pole)*(1.-brbyb2(j,k))
              facq=1./(hlen*(1.+2.*hlen*(1.+hlen)))
              h=h0*exp(-(rh(i)-r0)/hlen)*faclat*facq
              fne=rho(i,j,k)/he_rho
              deriv(i,j,k)=deriv(i,j,k)+(gammarh(i)-1.)*h/(he_p*fne)
            enddo
          enddo
        enddo
c
      else
c      
        call geth0 (q0_pole,q0_eq,hlen_pole,hlen_eq,q0,h0)
c
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              hlen=hlen_pole+(hlen_eq-hlen_pole)*thprof(j)
              faclat=q0_pole+(q0_eq-q0_pole)*thprof(j)
              facq=1./(hlen*(1.+2.*hlen*(1.+hlen)))
              h=h0*exp(-(rh(i)-r0)/hlen)*faclat*facq
              fne=rho(i,j,k)/he_rho
              deriv(i,j,k)=deriv(i,j,k)+(gammarh(i)-1.)*h/(he_p*fne)
            enddo
          enddo
        enddo
      end if
c
c ****** Add in the radiation loss term.
c
      if (radloss.ne.0.) then
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              fne=rho(i,j,k)/he_rho
              radterm=radloss*(fne*he_np/he_p)
     &                *qrad(fn_t*temp(i,j,k))/fn_qrad
              dqdt(i,j,k)=ifimplrad*radloss*(gammarh(i)-1.)*dtime*
     &        (fne*he_np/he_p)*fn_t*d_qrad(fn_t*temp(i,j,k))/fn_qrad
              tre(i,j,k)=temp(i,j,k)
              deriv(i,j,k)=deriv(i,j,k)-(gammarh(i)-1.)*radterm
            enddo
          enddo
        enddo
      end if
c
c ****** Advance the temperature by the source terms.
c
      call fft (deriv,derivr,derivi,1)
c
      do  k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            tr(i,j,k)=tr(i,j,k)+derivr(i,j,k)*dtime
            ti(i,j,k)=ti(i,j,k)+derivi(i,j,k)*dtime
          enddo
        enddo
      enddo
c
c ****** Set the boundary conditions.
c
      call setbct (tr,ti)
c
c ****** Advance the thermal conduction term using a
c ****** fully implicit formulation.
c
      if (tcond.eq.0.) then
c
c ****** Set the temperature in real space.
c
        call fft (temp,tr,ti,0)
c
        return
c
      else
c
c ****** Get the coefficients of the parallel thermal
c ****** conductivity tensor.
c
c ****** Note that this uses T at the old time step to
c ****** evaluate kappa (since TEMP has not been updated
c ****** yet).
c
        call getkappa (temp)
c
c ****** Set the (partially time-advanced) temperature
c ***** in real space.
c
        call fft (temp,tr,ti,0)
c
      end if
c
c ****** Advance the equation for the parallel thermal conductivity.
c
      if (idebug.gt.0) then
        write (9,*)
        write (9,*) '### COMMENT from ADVTEMP:'
        write (9,*) '### CG solution of implicit parallel'//
     &              ' thermal conductivity equation:'
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
      end if
c
      call bcast (tc,0.,nrtp)
      call sbct (tc,1.)
      call div_kappa_grad (tb,tc)
c
c ****** The CG solution of the implicit parallel thermal
c ****** conductivity equations is done in real space.
c
c ****** Set the RHS and the guess.
c
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            dv=rh(i)**2*sth(j)*drh(i)*dth(j)
            fne=rho(i,j,k)/he_rho
            rhs(i,j,k)=fne*(temp(i,j,k)+
     &      dqdt(i,j,k)*tre(i,j,k))*dv-tb(i,j,k)
            xcg(i,j,k)=temp(i,j,k)
          enddo
        enddo
      enddo
c
c ****** Zero the boundary values of T and the rhs.
c
      call zbct (xcg)
      call zbct (rhs)
c
      solvtype='t'
c
c ****** Set the number of equations to solve in the CG solver.
c
      ncgeq=nr*nt*np
c
c ****** Set the solver type.
c
      solve_type=ST_T
c
c ***** Solve the implicit equations using the CG solver.
c
      call solve (xcg,rhs,ierr)
c
c ****** Check for convergence.
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ADVTEMP:'
        write (*,*) '### The field solver did not converge.'
        write (*,*) '### Could not solve the thermal conduction'//
     &              ' equation.'
        write (*,*) 'NTIME = ',ntime
        write (*,*) 'TIME = ',time
        write (9,*)
        write (9,*) '### ERROR in ADVTEMP:'
        write (9,*) '### The field solver did not converge.'
        write (9,*) '### Could not solve the thermal conduction'//
     &              ' equation.'
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
        call exit (1)
      end if
c
      if (idebug.gt.0) then
        write (9,900) '### TEMP:',ncg,bnrm,rnrm
  900   format (/,1x,a,' N=',i5,' |B|=',1pe8.2,' |R|=',1pe8.2)
      end if
c
c ****** Set the boundary values of the temperature.
c
      call sbct (xcg,1.)
c
c ****** Copy the updated T into the main T array.
c
      call scopy (nrtp,xcg,1,temp,1)
c
c ****** Set the temperature in complex space.
c
      call fft (temp,tr,ti,1)
c
c ****** Return here if we are not using radiation
c ****** boundary conditions.
c
      if (ifradbc.ne.1) return
c
c ****** Get the radiation integral in the chromosphere-
c ****** transition region.
c
      rint=radint(tbc0)
c
c ****** Find the coronal density at the base of the corona,
c ****** NERAD, from the radiative balance condition.
c
c
c ****** Poloidal derivative of the temperature
c
      do k=1,npd3
        do j=1,nt
          do i=1,nr
            derivr(i,j,k)= -fm(k)*ti(i,j,k)
            derivi(i,j,k)=  fm(k)*tr(i,j,k)
          enddo
        enddo
      enddo

      call fft (deriv,derivr,derivi,0)
      rint=radint(tbc0)
c
      do k=1,np
        do j=2,ntm1
          brav=(.25*(br(2,j,k)+br(1,j,k)+br(2,j-1,k)+br(1,j-1,k)))
          btav=bt(1,j,k)
          bpav=(.5*(bp(1,j,k)+bp(1,j-1,k)))
          bmod=sqrt(brav**2+btav**2+bpav**2)

          dtds=(brav*(temp(2,j,k)-temp(1,j,k))/dr(1)+
     &    0.25*btav*((temp(1,j+1,k)+temp(2,j+1,k)-
     &                temp(1,j  ,k)-temp(2,j  ,k))/dt(j)+
     &               (temp(1,j  ,k)+temp(2,j  ,k)-
     &                temp(1,j-1,k)-temp(2,j-1,k))/dt(j-1))/r(1)
     &    +0.5*bpav*(deriv(2,j,k)+deriv(1,j,k))/r(1)/sth(j))/bmod
          hlen=hlen_pole+(hlen_eq-hlen_pole)*thprof(j)
          faclat=q0_pole+(q0_eq-q0_pole)*thprof(j)
          facq=1./(hlen*(1.+2.*hlen*(1.+hlen)))
          h=h0*faclat*facq

          nerad(j,k)=sqrt((fn_rd1*dtds**2*(tbc0)**3
     &    +fn_rd2*h*(tbc0)**1.5)/rint)

        enddo
      enddo
c
c ****** Filter the radiation-balance boundary density.
c
      call filt_tp (nerad,'r',nfiltne0)
c
c ****** Advance the base density NE0 towards the required
c ****** value, NERAD, with a time constant TNE0.
c
      if (first) then
        call fft_tp (rhor0re,rhobcr0(1,1,1),rhobcr0(1,1,2),0)
        do k=1,np
          do j=2,ntm1
            ne0(j,k)=rhor0re(j,k)/he_rho
          enddo
        enddo
        first=.false.
      end if
c
      do k=1,np
        do j=2,ntm1
          ne0(j,k)=(ne0(j,k)+nerad(j,k)*dtime/tne0)/(1.+dtime/tne0)
        enddo
      enddo
c
c ****** Set the base density in array RHORAD.
c
      do k=1,np
        rhorad( 1,k)=0.
        do j=2,ntm1
          rhorad(j,k)=he_rho*ne0(j,k)
        enddo
        rhorad(nt,k)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine geth0 (q0_pole,q0_eq,hlen_pole,hlen_eq,q0,h0)
c
c-----------------------------------------------------------------------
c
c ****** Find the coefficient of the volumetric coronal heating
c ****** term H0 to give the required total surface heat flux Q0.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (q0.eq.0.) then
        h0=0.
        return
      end if
c
c ****** Find the total heating integrated over the volume.
c
      sum=0.
      do j=2,ntm1
        do i=2,nrm1
          dv=rh(i)**2*sth(j)*drh(i)*dth(j)
          hlen=hlen_pole+(hlen_eq-hlen_pole)*thprof(j)
          faclat=q0_pole+(q0_eq-q0_pole)*thprof(j)
          facq=1./(hlen*(1.+2.*hlen*(1.+hlen)))
          h=exp(-(rh(i)-r0)/hlen)*faclat*facq
          sum=sum+h*dv
        enddo
      enddo
c
      if (sum.le.0.) then
        write (*,*)
        write (*,*) '### ERROR in GETH0:'
        write (*,*) '### Anomaly in the coronal heating term'//
     &              ' specification.'
        write (*,*) 'Volume-integrated heating .le. 0.'
        write (9,*)
        write (9,*) '### ERROR in GETH0:'
        write (9,*) '### Anomaly in the coronal heating term'//
     &              ' specification.'
        write (9,*) 'Volume-integrated heating .le. 0.'
        call exit (1)
      end if
c
      h0=2.*r0**2*q0/sum
c
      return
      end
c#######################################################################
      subroutine geth0_bheat (q0_pole,q0_eq,hlen_pole,hlen_eq,q0,h0)
c
c-----------------------------------------------------------------------
c
c ****** Find the coefficient of the volumetric coronal heating
c ****** term H0 to give the required total surface heat flux Q0.
c
c ****** This version is for the profile given by BRBYB2.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (q0.eq.0.) then
        h0=0.
        return
      end if
c
c ****** Find the total heating integrated over the volume.
c
      sum=0.
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            dv=rh(i)**2*sth(j)*drh(i)*dth(j)*dphi
            hlen=hlen_pole+(hlen_eq-hlen_pole)*(1.-brbyb2(j,k))
            faclat=q0_pole+(q0_eq-q0_pole)*(1.-brbyb2(j,k))
            facq=1./(hlen*(1.+2.*hlen*(1.+hlen)))
            h=exp(-(rh(i)-r0)/hlen)*faclat*facq
            sum=sum+h*dv
          enddo
        enddo
      enddo
c
      if (sum.le.0.) then
        write (*,*)
        write (*,*) '### ERROR in GETH0_BHEAT:'
        write (*,*) '### Anomaly in the coronal heating term'//
     &              ' specification.'
        write (*,*) 'Volume-integrated heating .le. 0.'
        write (9,*)
        write (9,*) '### ERROR in GETH0_BHEAT:'
        write (9,*) '### Anomaly in the coronal heating term'//
     &              ' specification.'
        write (9,*) 'Volume-integrated heating .le. 0.'
        call exit (1)
      end if
c
      h0=4.*pi*r0**2*q0/sum
c
      return
      end
c#######################################################################
ccc   function qrad (tempk)
      function qrad_rosner (tempk)
c
c-----------------------------------------------------------------------
c
c ****** Rosner radiative loss function in erg-cm**3/s.
c
c ****** TEMPK is the temperature in degrees Kelvin.
c
c-----------------------------------------------------------------------
c
      t=tempk
c
      if (tempk.lt.2.00e4) then
        qalpha=6.15
        qchi=1.408e-30
        t=t*1.e-3
      else if (tempk.ge.2.00e4.and.tempk.lt.3.76e4) then
        qalpha=0.
        qchi=1.412e-22
      else if (tempk.ge.3.76e4.and.tempk.lt.7.94e4) then
        qalpha=2.
        qchi=9.988e-32
      else if (tempk.ge.7.94e4.and.tempk.lt.2.51e5) then
        qalpha=0.
        qchi=6.297e-22
      else if (tempk.ge.2.51e5.and.tempk.lt.5.91e5) then
        qalpha=-2.
        qchi=3.967e-11
      else if (tempk.ge.5.91e5.and.tempk.lt.2.08e6) then
        qalpha=0.
        qchi=1.136e-22
      else if (tempk.ge.2.08e6) then
        qalpha=-0.67
        qchi=1.943e-18
      end if
c
      qrad=qchi*t**qalpha
      qrad_rosner=qrad
c
      return
      end

c#######################################################################
ccc      function d_qrad_mok (tempk)
      function d_qrad (tempk)
c
c-----------------------------------------------------------------------
c
c ****** Derivative of Athay radiative loss function in erg-cm**3/s/K.
c
c ****** TEMPK is the temperature in degrees Kelvin.
c
c-----------------------------------------------------------------------
c
      parameter (tbase=2.e4)
c
c-----------------------------------------------------------------------
c
      t10=log10(tempk)
      dt10=1./log(10.)/tempk
c
      qrad_mok=1.e-22*(0.4*exp(-30.0*(t10-4.60)**2)
     &                +4.0*exp(-20.0*(t10-4.90)**2)
     &                +4.5*exp(-16.0*(t10-5.35)**2)
     &                +2.0*exp(- 4.0*(t10-6.10)**2))
      d_qrad_mok=1.e-22*2.*dt10*
     & (0.4*exp(-30.0*(t10-4.60)**2)*(-30.0)*(t10-4.60)
     & +4.0*exp(-20.0*(t10-4.90)**2)*(-20.0)*(t10-4.90)
     & +4.5*exp(-16.0*(t10-5.35)**2)*(-16.0)*(t10-5.35)
     & +2.0*exp(- 4.0*(t10-6.10)**2)*(- 4.0)*(t10-6.10))
c
c ****** Reduce the radiation loss to zero smoothly for
c ****** temperatures below TBASE.
c
      xx=(tempk/tbase)**4-1.
      xx=1./max(xx,1.e-30)
      dxx=xx*xx*4.0*(tempk)**3/(tbase)**4
c
      d_qrad=exp(-xx)*(dxx*qrad_mok+d_qrad_mok)
c
      return
      end
c#######################################################################
ccc      function qrad_mok (tempk)
      function qrad (tempk)
c
c-----------------------------------------------------------------------
c
c ****** Athay radiative loss function in erg-cm**3/s.
c
c ****** TEMPK is the temperature in degrees Kelvin.
c
c-----------------------------------------------------------------------
c
c ****** This routine was obtained from Yung Mok.
c
c-----------------------------------------------------------------------
c
      parameter (tbase=2.e4)
c
c-----------------------------------------------------------------------
c
      t10=log10(tempk)
c
      qrad_mok=1.e-22*(0.4*exp(-30.0*(t10-4.60)**2)
     &                +4.0*exp(-20.0*(t10-4.90)**2)
     &                +4.5*exp(-16.0*(t10-5.35)**2)
     &                +2.0*exp(- 4.0*(t10-6.10)**2))
c
c ****** Reduce the radiation loss to zero smoothly for
c ****** temperatures below TBASE.
c
      xx=(tempk/tbase)**4-1.
      xx=1./max(xx,1.e-30)
      qrad_mok=qrad_mok*exp(-xx)
c
      qrad=qrad_mok
c
      return
      end
c#######################################################################
      function radint_rosner (tempk)
c
c-----------------------------------------------------------------------
c
c ****** Get the radiation integral int(Q(T)*T**0.5)dT from the
c ****** bottom of the chromosphere (assumed to be at T = 6000K)
c ****** through the transition region up to a temperature
c ****** TEMPK (in degrees Kelvin) by piecewise integration of
c ****** the Rosner et al. (1978) radiation loss curve.
c
c ****** The integral is returned in units of [erg-cm**3-K**1.5/s].
c
c-----------------------------------------------------------------------
c
      t=tempk
c
      if (tempk.le.6.00e3) then
        write (*,*)
        write (*,*) '### ERROR in RADINT:'
        write (*,*) '### Illegal temperature specified.'
        write (*,*) 'T = ',tempk
        write (9,*)
        write (9,*) '### ERROR in RADINT:'
        write (9,*) '### Illegal temperature specified.'
        write (9,*) 'T = ',tempk
        call exit (1)
      else if (tempk.gt.6.00e3.and.tempk.lt.2.00e4) then
        t=t*1.e-3
        radint=          5.821e-27*(t**(+7.65)-8.971e+05)
      else if (tempk.ge.2.00e4.and.tempk.lt.3.76e4) then
        radint=5.222e-17+9.413e-23*(t**(+1.50)-2.828e+06)
      else if (tempk.ge.3.76e4.and.tempk.lt.7.94e4) then
        radint=4.723e-16+2.854e-32*(t**(+3.50)-1.031e+16)
      else if (tempk.ge.7.94e4.and.tempk.lt.2.51e5) then
        radint=4.203e-15+4.198e-22*(t**(+1.50)-2.237e+07)
      else if (tempk.ge.2.51e5.and.tempk.lt.5.91e5) then
        radint=4.760e-14-7.934e-11*(t**(-0.50)-1.996e-03)
      else if (tempk.ge.5.91e5.and.tempk.lt.2.08e6) then
        radint=1.028e-13+7.573e-23*(t**(+1.50)-4.543e+08)
      else if (tempk.ge.2.08e6) then
        radint=2.956e-13+2.341e-18*(t**(+0.83)-1.754e+05)
      end if
c
      radint_rosner=radint

      return
      end
c#######################################################################
      function radint (tempk)
c
c-----------------------------------------------------------------------
c
c ****** Get the radiation integral int(Q(T)*T**0.5)dT from the
c ****** fora Athay's law.  
c
c ****** The integral is returned in units of [erg-cm**3-K**1.5/s].
c
c-----------------------------------------------------------------------
c
      parameter (n_temp=20)
      real temp_array(n_temp),radint_array(n_temp)
      data temp_array /100000.,200000.,300000.,400000.,500000.,
     & 600000.,700000.,800000., 900000.,1.e6,1.1e6,1.2e6,1.3e6,
     & 1.4e6,1.5e6,1.6e6,1.7e6,1.8e6,1.9e6,2e6/
      data radint_array /  .0554986e-13,
     & .2073482e-13,  .4322839e-13,  .6130756e-13,  .7444938e-13,
     & .8629608e-13,  .9910115e-13,  1.1372597e-13,  1.3032514e-13,
     & 1.4875648e-13, 1.6876998e-13,  1.9008915e-13,  2.1244536e-13,
     & 2.3559178e-13,  2.5930832e-13,  2.8340229e-13,  3.0770726e-13,
     & 3.3208089e-13,  3.5640262e-13,  3.8057114e-13 /
      t=tempk
c
      if (tempk.le.temp_array(1).or.tempk.ge.temp_array(n_temp)) then
        write (*,*)
        write (*,*) '### ERROR in RADINT:'
        write (*,*) '### Illegal temperature specified.'
        write (*,*) 'T = ',tempk
        write (9,*)
        write (9,*) '### ERROR in RADINT:'
        write (9,*) '### Illegal temperature specified.'
        write (9,*) 'T = ',tempk
        call exit (1)
      end if
      call interp (temp_array,n_temp,tempk,ir,alpha)
      irp1=min0(ir+1,n_temp)
      radint=(1.-alpha)*radint_array(ir)+alpha*radint_array(irp1)

c
      return
      end
c#######################################################################
      subroutine getvpar (vparr,vpart,vparp)
c
c-----------------------------------------------------------------------
c
c ****** Get the velocity parallel to B.
c
c ****** This is a temporary version where we actually just return
c ****** the velocity.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vparr(nr,nt,np),vpart(nr,nt,np),vparp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Calculate B**2.
c
      do k=1,np
c
        do j=1,nt
          do i=1,nrm1
            vparr(i,j,k)=vr(i,j,k)
          enddo
        enddo
c
        do j=1,ntm1
          do i=1,nr
            vpart(i,j,k)=vt(i,j,k)
          enddo
        enddo
c
        do j=1,nt
          do i=1,nr
            vparp(i,j,k)=vp(i,j,k)
          enddo
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine getvpar_old (vparr,vpart,vparp)
c
c-----------------------------------------------------------------------
c
c ****** Get the velocity parallel to B.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vparr(nr,nt,np),vpart(nr,nt,np),vparp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for B**2.
c
      real brsq(nr,nt,np),btsq(nr,nt,np),bpsq(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Calculate B**2.
c
      do k=1,np
c
        do j=1,ntm1
          do i=1,nr
            brsq(i,j,k)=br(i,j,k)**2
          enddo
        enddo
c
        do j=1,nt
          do i=1,nrm1
            btsq(i,j,k)=bt(i,j,k)**2
          enddo
        enddo
c
        do j=1,ntm1
          do i=1,nrm1
            bpsq(i,j,k)=bp(i,j,k)**2
          enddo
        enddo
c
      enddo
c
c ****** Get the parallel velocity.
c
      do k=1,np
c
        do j=2,ntm1
          do i=1,nrm1
            avvr=AVG  (vr,i  ,j  ,k)
            avvt=AVGRT(vt,i+1,j  ,k)
            avvp=AVGR (vp,i+1,j  ,k)
            avbr=AVGRT(br,i+1,j  ,k)
            avbt=AVG  (bt,i  ,j  ,k)
            avbp=AVGT (bp,i  ,j  ,k)
            avbrsq=AVGRT(brsq,i+1,j  ,k)
            avbtsq=AVG  (btsq,i  ,j  ,k)
            avbpsq=AVGT (bpsq,i  ,j  ,k)
            bsq=avbrsq+avbtsq+avbpsq
            if (bsq.eq.0.) then
              vparr(i,j,k)=avvr
            else
              vparr(i,j,k)=( avvr*avbrsq
     &                      +(avvt*avbt+avvp*avbp)*avbr)/bsq
            end if
          enddo
        enddo
c
        do j=1,ntm1
          do i=2,nrm1
            avvr=AVGRT(vr,i  ,j+1,k)
            avvt=AVG  (vt,i  ,j  ,k)
            avvp=AVGT (vp,i  ,j+1,k)
            avbr=AVG  (br,i  ,j  ,k)
            avbt=AVGRT(bt,i  ,j+1,k)
            avbp=AVGR (bp,i  ,j  ,k)
            avbrsq=AVG  (brsq,i  ,j  ,k)
            avbtsq=AVGRT(btsq,i  ,j+1,k)
            avbpsq=AVGR (bpsq,i  ,j  ,k)
            bsq=avbrsq+avbtsq+avbpsq
            if (bsq.eq.0.) then
              vpart(i,j,k)=0.
            else
              vpart(i,j,k)=( avvt*avbtsq
     &                      +(avvr*avbr+avvp*avbp)*avbt)/bsq
            end if
          enddo
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            avvr=AVGR (vr,i  ,j  ,k)
            avvt=AVGT (vt,i  ,j  ,k)
            avvp=AVG  (vp,i  ,j  ,k)
            avbr=AVGT (br,i  ,j  ,k)
            avbt=AVGR (bt,i  ,j  ,k)
            avbp=AVGRT(bp,i  ,j  ,k)
            avbrsq=AVGT (brsq,i  ,j  ,k)
            avbtsq=AVGR (btsq,i  ,j  ,k)
            avbpsq=AVGRT(bpsq,i  ,j  ,k)
            bsq=avbrsq+avbtsq+avbpsq
            if (bsq.eq.0.) then
              vparp(i,j,k)=0.
            else
              vparp(i,j,k)=( avvp*avbpsq
     &                      +(avvr*avbr+avvt*avbt)*avbp)/bsq
            end if
          enddo
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine setp
c
c-----------------------------------------------------------------------
c
c ****** Set the pressure from the density and temperature.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real pr0re(nt,np),pr1re(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Set the pressure.
c
      do k=1,np
        do j=1,nt
          do i=1,nr
            p(i,j,k)=he_p*rho(i,j,k)*temp(i,j,k)/he_rho
          enddo
        enddo
      enddo
c
c ****** Set the pressure in the complex arrays.
c
      call fft (p,prr,pri,1)
c
c ****** Set the pressure boundary conditions.
c
      do k=1,np
        do j=1,nt
          pr0re(j,k)=.5*(p( 1,j,k)+p(   2,j,k))
          pr1re(j,k)=.5*(p(nr,j,k)+p(nrm1,j,k))
        enddo
      enddo
c
      call fft_tp (pr0re,pbcr0(1,1,1),pbcr0(1,1,2),1)
      call fft_tp (pr1re,pbcr1(1,1,1),pbcr1(1,1,2),1)
c
      return
      end
c#######################################################################
      subroutine setbcp (ppr,ppi)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on pressure.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real ppr(nr,nt,npd3),ppi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          ppr( 1,j,k)=2.*pbcr0(j,k,1)-ppr(   2,j,k)
          ppi( 1,j,k)=2.*pbcr0(j,k,2)-ppi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        if (bcr1type.eq.'open') then
          do j=2,ntm1
            ppr(nr,j,k)=pbcr1(j,k,1)
            ppi(nr,j,k)=pbcr1(j,k,2)
          enddo
        else if (bcr1type.eq.'extrapolation') then
          do j=2,ntm1
            ppr(nr,j,k)=EXTRAPR1(ppr,nrm1,j,k)
            ppi(nr,j,k)=EXTRAPR1(ppi,nrm1,j,k)
          enddo
        else if (bcr1type.eq.'fixed') then
          do j=2,ntm1
            ppr(nr,j,k)=2.*pbcr1(j,k,1)-ppr(nrm1,j,k)
            ppi(nr,j,k)=2.*pbcr1(j,k,2)-ppi(nrm1,j,k)
          enddo
        else if (bcr1type.eq.'1dchar') then
          do j=2,ntm1
            ppr(nr,j,k)=2.*pbcr1(j,k,1)-ppr(nrm1,j,k)
            ppi(nr,j,k)=2.*pbcr1(j,k,2)-ppi(nrm1,j,k)
          enddo
        end if
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nr
            ppr(i, 1,k)=ppr(i,   2,k)
            ppr(i,nt,k)=ppr(i,ntm1,k)
            ppi(i, 1,k)=ppi(i,   2,k)
            ppi(i,nt,k)=ppi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nr
            ppr(i, 1,k)=-ppr(i,   2,k)
            ppr(i,nt,k)=-ppr(i,ntm1,k)
            ppi(i, 1,k)=-ppi(i,   2,k)
            ppi(i,nt,k)=-ppi(i,ntm1,k)
          enddo
        else
          do i=1,nr
            ppr(i, 1,k)=-ppr(i,   2,k)
            ppr(i,nt,k)=-ppr(i,ntm1,k)
            ppi(i, 1,k)=-ppi(i,   2,k)
            ppi(i,nt,k)=-ppi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
      subroutine setbct (tpr,tpi)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on temperature.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Set the BCs on T.
c
      call sbct_complex (tpr,tpi,1.)
c
      return
      end
c#######################################################################
      subroutine sbct (tp,vmask)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on temperature (in real space).
c
c ****** VMASK multiplies the boundary value.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real tp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Inner radial boundary.
c
      do k=1,np
        do j=2,ntm1
          tp( 1,j,k)=2.*vmask*tbcr0re(j,k)-tp(   2,j,k)
        enddo
      enddo
c
c ****** Outer radial boundary.
c
      if (bcr1type.eq.'open') then
        do k=1,np
          do j=2,ntm1
            tp(nr,j,k)=vmask*tbcr1re(j,k)
          enddo
        enddo
      else if (bcr1type.eq.'extrapolation') then
        do k=1,np
          do j=2,ntm1
            tp(nr,j,k)=EXTRAPR1(tp,nrm1,j,k)
          enddo
        enddo
      else if (bcr1type.eq.'fixed') then
        do k=1,np
          do j=2,ntm1
            tp(nr,j,k)=2.*vmask*tbcr1re(j,k)-tp(nrm1,j,k)
          enddo
        enddo
      else if (bcr1type.eq.'1dchar') then
        do k=1,np
          do j=2,ntm1
            tp(nr,j,k)=2.*vmask*tbcr1re(j,k)-tp(nrm1,j,k)
          enddo
        enddo
      end if
c
c ****** Set the theta boundaries.
c
      do i=1,nr
c
c ****** Theta boundary at theta=0.
c
c ****** Get the m=0 component.
c
        tav=0.
        do k=1,np
          tav=tav+tp(i,2,k)
        enddo
        tav=tav/np
c
c ****** Set the boundary condition.
c
        do k=1,np
          tp(i,1,k)=2.*tav-tp(i,2,k)
        enddo
c
c ****** Theta boundary at theta=pi.
c
c ****** Get the m=0 component.
c
        tav=0.
        do k=1,np
          tav=tav+tp(i,ntm1,k)
        enddo
        tav=tav/np
c
c ****** Set the boundary condition.
c
        do k=1,np
          tp(i,nt,k)=2.*tav-tp(i,ntm1,k)
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine sbct_complex (tpr,tpi,vmask)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on temperature.
c
c ****** VMASK multiplies the boundary value.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real tpr(nr,nt,npd3),tpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          tpr( 1,j,k)=2.*vmask*tbcr0(j,k,1)-tpr(   2,j,k)
          tpi( 1,j,k)=2.*vmask*tbcr0(j,k,2)-tpi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        if (bcr1type.eq.'open') then
          do j=2,ntm1
            tpr(nr,j,k)=vmask*tbcr1(j,k,1)
            tpi(nr,j,k)=vmask*tbcr1(j,k,2)
          enddo
        else if (bcr1type.eq.'extrapolation') then
          do j=2,ntm1
            tpr(nr,j,k)=EXTRAPR1(tpr,nrm1,j,k)
            tpi(nr,j,k)=EXTRAPR1(tpi,nrm1,j,k)
          enddo
        else if (bcr1type.eq.'fixed') then
          do j=2,ntm1
            tpr(nr,j,k)=2.*vmask*tbcr1(j,k,1)-tpr(nrm1,j,k)
            tpi(nr,j,k)=2.*vmask*tbcr1(j,k,2)-tpi(nrm1,j,k)
          enddo
        else if (bcr1type.eq.'1dchar') then
          do j=2,ntm1
            tpr(nr,j,k)=2.*vmask*tbcr1(j,k,1)-tpr(nrm1,j,k)
            tpi(nr,j,k)=2.*vmask*tbcr1(j,k,2)-tpi(nrm1,j,k)
          enddo
        end if
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nr
            tpr(i, 1,k)=tpr(i,   2,k)
            tpr(i,nt,k)=tpr(i,ntm1,k)
            tpi(i, 1,k)=tpi(i,   2,k)
            tpi(i,nt,k)=tpi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nr
            tpr(i, 1,k)=-tpr(i,   2,k)
            tpr(i,nt,k)=-tpr(i,ntm1,k)
            tpi(i, 1,k)=-tpi(i,   2,k)
            tpi(i,nt,k)=-tpi(i,ntm1,k)
          enddo
        else
          do i=1,nr
            tpr(i, 1,k)=-tpr(i,   2,k)
            tpr(i,nt,k)=-tpr(i,ntm1,k)
            tpi(i, 1,k)=-tpi(i,   2,k)
            tpi(i,nt,k)=-tpi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
      subroutine zbct (tc)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary points of T to zero.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real tc(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Boundary points.
c
      do k=1,np
        do i=1,nr
          tc(i, 1,k)=0.
          tc(i,nt,k)=0.
        enddo
        do j=2,ntm1
          tc( 1,j,k)=0.
          tc(nr,j,k)=0.
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine getkappa (tc)
c
c-----------------------------------------------------------------------
c
c ****** Get the coefficients of the parallel thermal conductivity
c ****** tensor in [erg/s/cm/K].
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real tc(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Parallel thermal conductivity arrays.
c
      common /fkparcom/ fkrr(nr,nt,np),
     &                  fktt(nr,nt,np),
     &                  fkpp(nr,nt,np),
     &                  fkrt(nr,nt,np),
     &                  fkrp(nr,nt,np),
     &                  fktp(nr,nt,np), dqdt(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Set kappa to the Spitzer value, multiplied by TCOND.
c
      fkpar0=tcond*fkspitz
c
c ****** Internal points.
c
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            brav=AVGT (br,i,j,k)
            btav=AVGR (bt,i,j,k)
            bpav=AVGRT(bp,i,j,k)
            bsq=brav**2+btav**2+bpav**2
            if (bsq.eq.0.) then
              frr=1.
              ftt=1.
              fpp=1.
              frt=0.
              frp=0.
              ftp=0.
            else
              frr=brav*brav/bsq
              ftt=btav*btav/bsq
              fpp=bpav*bpav/bsq
              frt=brav*btav/bsq
              frp=brav*bpav/bsq
              ftp=btav*bpav/bsq
            end if
            tav=AVG  (tc,i,j,k)
            s=.5*(1.+tanh((fn_t*tav-t0_tc)/dt0_tc))
            fkpar=s*fkpar0*(fn_t*tav)**2.5+
     &      (1.-s)*fkpar0*(fn_t*tav)**alpha_tc*(t0_tc)**(2.5-alpha_tc)          
            fkrr(i,j,k)=frr*fkpar
            fktt(i,j,k)=ftt*fkpar
            fkpp(i,j,k)=fpp*fkpar
            fkrt(i,j,k)=frt*fkpar
            fkrp(i,j,k)=frp*fkpar
            fktp(i,j,k)=ftp*fkpar
          enddo
        enddo
      enddo
c
c ****** Boundary points at r=R0.
c
      do k=1,np
        do j=2,ntm1
          brav=AVGRT(br,   2,j  ,k)
          btav=AVG  (bt,   1,j  ,k)
          bpav=AVGT (bp,   1,j  ,k)
          bsq=brav**2+btav**2+bpav**2
          if (bsq.eq.0.) then
            frr=1.
            ftt=1.
            fpp=1.
            frt=0.
            frp=0.
            ftp=0.
          else
            frr=brav*brav/bsq
            ftt=btav*btav/bsq
            fpp=bpav*bpav/bsq
            frt=brav*btav/bsq
            frp=brav*bpav/bsq
            ftp=btav*bpav/bsq
          end if
          tav=AVGR (tc,   2,j  ,k)
          s=.5*(1.+tanh((fn_t*tav-t0_tc)/dt0_tc))
          fkpar=s*fkpar0*(fn_t*tav)**2.5+
     &    (1.-s)*fkpar0*(fn_t*tav)**alpha_tc*(t0_tc)**(2.5-alpha_tc)            
          fkrr( 1,j,k)=2.*frr*fkpar-fkrr(   2,j,k)
          fktt( 1,j,k)=2.*ftt*fkpar-fktt(   2,j,k)
          fkpp( 1,j,k)=2.*fpp*fkpar-fkpp(   2,j,k)
          fkrt( 1,j,k)=2.*frt*fkpar-fkrt(   2,j,k)
          fkrp( 1,j,k)=2.*frp*fkpar-fkrp(   2,j,k)
          fktp( 1,j,k)=2.*ftp*fkpar-fktp(   2,j,k)
        enddo
      enddo
c
c ****** Boundary points at r=R1.
c
      do k=1,np
        do j=2,ntm1
          brav=AVGRT(br,  nr,j  ,k)
          btav=AVG  (bt,nrm1,j  ,k)
          bpav=AVGT (bp,nrm1,j  ,k)
          bsq=brav**2+btav**2+bpav**2
          if (bsq.eq.0.) then
            frr=1.
            ftt=1.
            fpp=1.
            frt=0.
            frp=0.
            ftp=0.
          else
            frr=brav*brav/bsq
            ftt=btav*btav/bsq
            fpp=bpav*bpav/bsq
            frt=brav*btav/bsq
            frp=brav*bpav/bsq
            ftp=btav*bpav/bsq
          end if
          tav=AVGR (tc,  nr,j  ,k)
          s=.5*(1.+tanh((fn_t*tav-t0_tc)/dt0_tc))
          fkpar=s*fkpar0*(fn_t*tav)**2.5+
     &    (1.-s)*fkpar0*(fn_t*tav)**alpha_tc*(t0_tc)**(2.5-alpha_tc)            
          fkrr(nr,j,k)=2.*frr*fkpar-fkrr(nrm1,j,k)
          fktt(nr,j,k)=2.*ftt*fkpar-fktt(nrm1,j,k)
          fkpp(nr,j,k)=2.*fpp*fkpar-fkpp(nrm1,j,k)
          fkrt(nr,j,k)=2.*frt*fkpar-fkrt(nrm1,j,k)
          fkrp(nr,j,k)=2.*frp*fkpar-fkrp(nrm1,j,k)
          fktp(nr,j,k)=2.*ftp*fkpar-fktp(nrm1,j,k)
        enddo
      enddo
c
c ****** Boundary points at theta=0.
c
      do k=1,np
        do i=2,nrm1
          brav=AVG  (br,i  ,   1,k)
          btav=AVGRT(bt,i  ,   2,k)
          bpav=AVGR (bp,i  ,   1,k)
          bsq=brav**2+btav**2+bpav**2
          if (bsq.eq.0.) then
            frr=1.
            ftt=1.
            fpp=1.
            frt=0.
            frp=0.
            ftp=0.
          else
            frr=brav*brav/bsq
            ftt=btav*btav/bsq
            fpp=bpav*bpav/bsq
            frt=brav*btav/bsq
            frp=brav*bpav/bsq
            ftp=btav*bpav/bsq
          end if
          tav=AVGT (tc,i, 2,k)
          s=.5*(1.+tanh((fn_t*tav-t0_tc)/dt0_tc))
          fkpar=s*fkpar0*(fn_t*tav)**2.5+
     &    (1.-s)*fkpar0*(fn_t*tav)**alpha_tc*(t0_tc)**(2.5-alpha_tc)            
          fkrr(i, 1,k)=2.*frr*fkpar-fkrr(i,   2,k)
          fktt(i, 1,k)=2.*ftt*fkpar-fktt(i,   2,k)
          fkpp(i, 1,k)=2.*fpp*fkpar-fkpp(i,   2,k)
          fkrt(i, 1,k)=2.*frt*fkpar-fkrt(i,   2,k)
          fkrp(i, 1,k)=2.*frp*fkpar-fkrp(i,   2,k)
          fktp(i, 1,k)=2.*ftp*fkpar-fktp(i,   2,k)
        enddo
      enddo
c
c ****** Boundary points at theta=pi.
c
      do k=1,np
        do i=2,nrm1
          brav=AVG  (br,i  ,ntm1,k)
          btav=AVGRT(bt,i  ,nt  ,k)
          bpav=AVGR (bp,i  ,ntm1,k)
          bsq=brav**2+btav**2+bpav**2
          if (bsq.eq.0.) then
            frr=1.
            ftt=1.
            fpp=1.
            frt=0.
            frp=0.
            ftp=0.
          else
            frr=brav*brav/bsq
            ftt=btav*btav/bsq
            fpp=bpav*bpav/bsq
            frt=brav*btav/bsq
            frp=brav*bpav/bsq
            ftp=btav*bpav/bsq
          end if
          tav=AVGT (tc,i,nt,k)
          s=.5*(1.+tanh((fn_t*tav-t0_tc)/dt0_tc))
          fkpar=s*fkpar0*(fn_t*tav)**2.5+
     &    (1.-s)*fkpar0*(fn_t*tav)**alpha_tc*(t0_tc)**(2.5-alpha_tc)            
          fkrr(i,nt,k)=2.*frr*fkpar-fkrr(i,ntm1,k)
          fktt(i,nt,k)=2.*ftt*fkpar-fktt(i,ntm1,k)
          fkpp(i,nt,k)=2.*fpp*fkpar-fkpp(i,ntm1,k)
          fkrt(i,nt,k)=2.*frt*fkpar-fkrt(i,ntm1,k)
          fkrp(i,nt,k)=2.*frp*fkpar-fkrp(i,ntm1,k)
          fktp(i,nt,k)=2.*ftp*fkpar-fktp(i,ntm1,k)
        enddo
      enddo
c
c ****** "Corner points".
c
c ****** Note that the corner points are never used in the
c ****** calculation.  Set them (arbitrarily) to equal
c ****** their neighbors.
c
      do k=1,np
c
        fkrr( 1, 1,k)=fkrr( 1,   2,k)
        fkrr(nr, 1,k)=fkrr(nr,   2,k)
        fkrr( 1,nt,k)=fkrr( 1,ntm1,k)
        fkrr(nr,nt,k)=fkrr(nr,ntm1,k)
c
        fktt( 1, 1,k)=fktt( 1,   2,k)
        fktt(nr, 1,k)=fktt(nr,   2,k)
        fktt( 1,nt,k)=fktt( 1,ntm1,k)
        fktt(nr,nt,k)=fktt(nr,ntm1,k)
c
        fkpp( 1, 1,k)=fkpp( 1,   2,k)
        fkpp(nr, 1,k)=fkpp(nr,   2,k)
        fkpp( 1,nt,k)=fkpp( 1,ntm1,k)
        fkpp(nr,nt,k)=fkpp(nr,ntm1,k)
c
        fkrt( 1, 1,k)=fkrt( 1,   2,k)
        fkrt(nr, 1,k)=fkrt(nr,   2,k)
        fkrt( 1,nt,k)=fkrt( 1,ntm1,k)
        fkrt(nr,nt,k)=fkrt(nr,ntm1,k)
c
        fkrp( 1, 1,k)=fkrp( 1,   2,k)
        fkrp(nr, 1,k)=fkrp(nr,   2,k)
        fkrp( 1,nt,k)=fkrp( 1,ntm1,k)
        fkrp(nr,nt,k)=fkrp(nr,ntm1,k)
c
        fktp( 1, 1,k)=fktp( 1,   2,k)
        fktp(nr, 1,k)=fktp(nr,   2,k)
        fktp( 1,nt,k)=fktp( 1,ntm1,k)
        fktp(nr,nt,k)=fktp(nr,ntm1,k)
c
      enddo
c
c ****** Filter the thermal conductivity.
c
      call filt_vec (fkrr,'r',nfilt_kappa)
      call filt_vec (fktt,'r',nfilt_kappa)
      call filt_vec (fkpp,'r',nfilt_kappa)
      call filt_vec (fkrt,'t',nfilt_kappa)
      call filt_vec (fkrp,'p',nfilt_kappa)
      call filt_vec (fktp,'r',nfilt_kappa)
c
c ****** Apply the modification (which may be used to reduce 
c ****** kappa in the collisionless regime).
c
c ****** Kappa is modified by multiplying by a function that
c ****** goes from the value TC_FAC0 to TC_FAC1 at a radius
c ****** TC_R, with a width given by TC_DR.
c
      do i=1,nr
        s=.5*(1.+tanh((rh(i)-tc_r)/tc_dr))
        fac=tc_fac0+(tc_fac1-tc_fac0)*s
        do k=1,np
          do j=1,nt
            fkrr(i,j,k)=fkrr(i,j,k)*fac
            fktt(i,j,k)=fktt(i,j,k)*fac
            fkpp(i,j,k)=fkpp(i,j,k)*fac
            fkrt(i,j,k)=fkrt(i,j,k)*fac
            fkrp(i,j,k)=fkrp(i,j,k)*fac
            fktp(i,j,k)=fktp(i,j,k)*fac
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine filt_vec (f,comp,nfilt)
c
c-----------------------------------------------------------------------
c
c ****** Filter the vector component in the array F.
c ****** COMP specifies the component of the vector.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real f(nr,nt,np)
      character*(*) comp
c
c-----------------------------------------------------------------------
c
      real, dimension(nr,nt,npd3) :: fr,fi
c
c-----------------------------------------------------------------------
c
      if (nfilt.le.0) return
c
c ****** Get the FFT of the array F.
c
      call fft (f,fr,fi,1)
c
c ****** Apply the filter NFILT times.
c
      do ifilt=1,nfilt
c
        do k=1,npd3
c
          m=k-1
c
c ****** Filter in theta.
c
          do i=1,nr
            call filter_v (nr,nt,fr(i,1,k))
            call filter_v (nr,nt,fi(i,1,k))
          enddo
c
c ****** Set boundary conditions at the poles.
c
          if (m.eq.0) then
            if (comp.eq.'r') then
              do i=1,nr
                fr(i, 1,k)=fr(i,   2,k)
                fi(i, 1,k)=fi(i,   2,k)
                fr(i,nt,k)=fr(i,ntm1,k)
                fi(i,nt,k)=fi(i,ntm1,k)
              enddo
            else if (comp.eq.'t') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'p') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            end if
          else if (m.eq.1) then
            if (comp.eq.'r') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'t') then
              do i=1,nr
                fr(i, 1,k)=fr(i,   2,k)
                fi(i, 1,k)=fi(i,   2,k)
                fr(i,nt,k)=fr(i,ntm1,k)
                fi(i,nt,k)=fi(i,ntm1,k)
              enddo
            else if (comp.eq.'p') then
              do i=1,nr
                fr(i, 1,k)=fr(i,   2,k)
                fi(i, 1,k)=fi(i,   2,k)
                fr(i,nt,k)=fr(i,ntm1,k)
                fi(i,nt,k)=fi(i,ntm1,k)
              enddo
            end if
          else if (m.eq.2) then
            if (comp.eq.'r') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'t') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'p') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            end if
          else
            if (comp.eq.'r') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'t') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            else if (comp.eq.'p') then
              do i=1,nr
                fr(i, 1,k)=-fr(i,   2,k)
                fi(i, 1,k)=-fi(i,   2,k)
                fr(i,nt,k)=-fr(i,ntm1,k)
                fi(i,nt,k)=-fi(i,ntm1,k)
              enddo
            end if
          end if
c
c ****** Filter in r.
c
          do j=1,nt
            call filter_v (1,nr,fr(1,j,k))
            call filter_v (1,nr,fi(1,j,k))
          enddo
c
c ****** Filter in phi.
c
          flt=cos(.5*dphi*m)**2
          do i=1,nr
            do j=1,nt
              fr(i,j,k)=fr(i,j,k)*flt
              fi(i,j,k)=fi(i,j,k)*flt
            enddo
          enddo
c
        enddo
c
      enddo
c
c ****** Transform back to real space.
c
      call fft (f,fr,fi,0)
c
      return
      end
c#######################################################################
      subroutine filter_v (ldim,n,f)
c
c-----------------------------------------------------------------------
c
c ****** Apply a "(1,2,1,)/4" digital filter to F.
c
c-----------------------------------------------------------------------
c
      parameter (u=1.)
c
c-----------------------------------------------------------------------
c
      integer :: ldim,n
      real, dimension(ldim,n) :: f
c
c-----------------------------------------------------------------------
c
      call filt1d (ldim,n,u,f)
c
      return
      end
c#######################################################################
      subroutine divqv (qr,qi,vvr,vvt,vvp,qre,divr,divi,divre)
c
c-----------------------------------------------------------------------
c
c ****** Call the selected routine to get the divergence of (q*v).
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (upwindphi.lt.0.) then
c
c ****** Use centered differencing in phi (spectral).
c
        call divqv_cphi (qr,qi,vvr,vvt,vvp,qre,divr,divi,divre)
c
      else
c
c ****** Use upwind differencing in phi.
c
        call divqv_uphi (qr,qi,vvr,vvt,vvp,qre,divr,divi,divre)
c
      end if
c
      return
      end
c#######################################################################
      subroutine divqv_cphi (qr,qi,vvr,vvt,vvp,qre,divr,divi,divre)
c
c-----------------------------------------------------------------------
c
c ****** Get the divergence of (q*v), where q is a scalar.
c
c ****** This version uses centered dervatives in phi
c ****** (i.e., spectral phi derivatives).
c
c-----------------------------------------------------------------------
c
c ****** On input, QR and QI contain the real and imaginary
c ****** parts of Q, and VVR, VVT, and VVP contain the advection
c ****** velocity in real space.
c
c ****** Upon return, DIVR and DIVI contain complex div(q*v), and
c ****** DIVRE contains real div(q*v).
c
c ****** QRE is used as a scratch array.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real qr(nr,nt,npd3),qi(nr,nt,npd3)
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
      real qre(nr,nt,np)
      real divr(nr,nt,npd3),divi(nr,nt,npd3)
      real divre(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Upwind coefficients.
c
      real uwr(nr,nt),uwt(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Get q in real space.
c
      call fft (qre,qr,qi,0)
c
c ****** Get q*v-phi in complex space in arrays DIVR and DIVI.
c
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            divre(i,j,k)=qre(i,j,k)*vvp(i,j,k)
          enddo
        enddo
      enddo
c
      call zb3 (divre,np)
c
      call fft (divre,divr,divi,1)
c
c ****** Get phi derivative of q*v-phi in real space
c ****** in array DIVRE.
c
      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            hold=divr(i,j,k)
            divr(i,j,k)=-fm(k)*divi(i,j,k)/(rh(i)*sth(j))
            divi(i,j,k)= fm(k)*hold       /(rh(i)*sth(j))
          enddo
        enddo
      enddo
c
      call fft (divre,divr,divi,0)
c
c ****** Compute div(q*v) in real space in DIVRE.
c
      do k=1,np
c
c ****** Get the upwind coefficients.
c
        do j=2,ntm1
          call uwpdm (nrm1, 1,qre(1,j,k),vvr(1,j,k),upwindv,uwr(1,j))
        enddo
        do i=2,nrm1
          call uwpdm (ntm1,nr,qre(i,1,k),vvt(i,1,k),upwindv,uwt(i,1))
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            cadvrp=sign(uwr(i  ,j  ),vvr(i  ,j  ,k))
            cadvrm=sign(uwr(i-1,j  ),vvr(i-1,j  ,k))
            cadvtp=sign(uwt(i  ,j  ),vvt(i  ,j  ,k))
            cadvtm=sign(uwt(i  ,j-1),vvt(i  ,j-1,k))
            divre(i,j,k)=divre(i,j,k)
     &       +( r(i  )**2*vvr(i  ,j,k)*ADVR(qre,i+1,j,k,cadvrp)
     &         -r(i-1)**2*vvr(i-1,j,k)*ADVR(qre,i  ,j,k,cadvrm)
     &        )/(rh(i)**2*drh(i))
     &       +( st(j  )*vvt(i,j  ,k)*ADVT(qre,i,j+1,k,cadvtp)
     &         -st(j-1)*vvt(i,j-1,k)*ADVT(qre,i,j  ,k,cadvtm)
     &        )/(rh(i)*sth(j)*dth(j))
          enddo
        enddo
c
      enddo
c
c ****** Transform div(q*v) to complex space in arrays DIVR and DIVI.
c
      call fft (divre,divr,divi,1)
c
      return
      end
c#######################################################################
      subroutine divqv_uphi (qr,qi,vvr,vvt,vvp,qre,divr,divi,divre)
c
c-----------------------------------------------------------------------
c
c ****** Get the divergence of (q*v), where q is a scalar.
c
c ****** This version uses upwind dervatives in phi (in real space),
c ****** according to the value of UPWINDPHI.
c
c-----------------------------------------------------------------------
c
c ****** On input, QR and QI contain the real and imaginary
c ****** parts of Q, and VVR, VVT, and VVP contain the advection
c ****** velocity in real space.
c
c ****** Upon return, DIVR and DIVI contain complex div(q*v), and
c ****** DIVRE contains real div(q*v).
c
c ****** QRE is used as a scratch array.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real qr(nr,nt,npd3),qi(nr,nt,npd3)
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
      real qre(nr,nt,np)
      real divr(nr,nt,npd3),divi(nr,nt,npd3)
      real divre(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Upwind coefficients.
c
      real uwr(nr,nt),uwt(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Get q in real space.
c
      call fft (qre,qr,qi,0)
c
c ****** Compute div(q*v) in real space in DIVRE.
c
      call zb3 (divre,np)
c
      do k=1,np
c
c ****** Take care of periodic wrap-around.
c
        if (k.eq.1) then
          km1=np
        else
          km1=k-1
        end if
c
        if (k.eq.np) then
          kp1=1
        else
          kp1=k+1
        end if
c
c ****** Get the upwind coefficients.
c
        do j=2,ntm1
          call uwpdm (nrm1, 1,qre(1,j,k),vvr(1,j,k),upwindv,uwr(1,j))
        enddo
        do i=2,nrm1
          call uwpdm (ntm1,nr,qre(i,1,k),vvt(i,1,k),upwindv,uwt(i,1))
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            cadvrp=sign(uwr(i  ,j  ),vvr(i  ,j  ,k))
            cadvrm=sign(uwr(i-1,j  ),vvr(i-1,j  ,k))
            cadvtp=sign(uwt(i  ,j  ),vvt(i  ,j  ,k))
            cadvtm=sign(uwt(i  ,j-1),vvt(i  ,j-1,k))
            avvpp=.5*(vvp(i,j,k)+vvp(i,j,kp1))
            avvpm=.5*(vvp(i,j,k)+vvp(i,j,km1))
            cadvpp=sign(upwindphi,avvpp)
            cadvpm=sign(upwindphi,avvpm)
            divre(i,j,k)=
     &        ( r(i  )**2*vvr(i  ,j,k)*ADVR(qre,i+1,j,k,cadvrp)
     &         -r(i-1)**2*vvr(i-1,j,k)*ADVR(qre,i  ,j,k,cadvrm)
     &        )/(rh(i)**2*drh(i))
     &       +( st(j  )*vvt(i,j  ,k)*ADVT(qre,i,j+1,k,cadvtp)
     &         -st(j-1)*vvt(i,j-1,k)*ADVT(qre,i,j  ,k,cadvtm)
     &        )/(rh(i)*sth(j)*dth(j))
     &       +( avvpp*ADVPHI(qre,i,j,k  ,kp1,cadvpp)
     &         -avvpm*ADVPHI(qre,i,j,km1,k  ,cadvpm)
     &        )/(dphi*rh(i)*sth(j))
          enddo
        enddo
c
      enddo
c
c ****** Transform div(q*v) to complex space in arrays DIVR and DIVI.
c
      call fft (divre,divr,divi,1)
c
      return
      end
c#######################################################################
      subroutine qdivv (qre,vvr,vvt,vvp,divr,divi,divre)
c
c-----------------------------------------------------------------------
c
c ****** Get the divergence of (v)
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
      real divr(nr,nt,npd3),divi(nr,nt,npd3)
      real divre(nr,nt,np),qre(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      if (upwindphi.ge.0.) then
        do k=1,np
          if (k.eq.1) then
            km1=np
          else
            km1=k-1
          end if
          if (k.eq.np) then
            kp1=1
          else
            kp1=k+1
          end if
          do j=2,ntm1
            do i=2,nrm1
              divre(i,j,k)= qre(i,j,k)*(( r(i  )**2*vvr(i  ,j,k)
     &                       -r(i-1)**2*vvr(i-1,j,k)
     &                      )/(rh(i)**2*drh(i))
     &                     +( st(j  )*vvt(i,j  ,k)
     &                       -st(j-1)*vvt(i,j-1,k)
     &                      )/(rh(i)*sth(j)*dth(j))
     &                     +( vvp(i,j,kp1)
     &                       -vvp(i,j,km1)
     &                      )/(2.*dphi*rh(i)*sth(j)))
            enddo
          enddo
        enddo
      else
c       do k=1,npd3
c         do j=2,ntm1
c           do i=2,nrm1
c             derivr(i,j,k)= ( r(i  )**2*vrr(i  ,j,k)
c    &                      -r(i-1)**2*vrr(i-1,j,k)
c    &                     )/(rh(i)**2*drh(i))
c    &                    +( st(j  )*vtr(i,j  ,k)
c    &                      -st(j-1)*vtr(i,j-1,k)
c    &                     )/(rh(i)*sth(j)*dth(j))
c    &                    -fm(k)*vpi(i,j,k)/(rh(i)*sth(j))
c             derivi(i,j,k)= ( r(i  )**2*vri(i  ,j,k)
c    &                      -r(i-1)**2*vri(i-1,j,k)
c    &                     )/(rh(i)**2*drh(i))
c    &                    +( st(j  )*vti(i,j  ,k)
c    &                      -st(j-1)*vti(i,j-1,k)
c    &                     )/(rh(i)*sth(j)*dth(j))
c    &                    +fm(k)*vpr(i,j,k)/(rh(i)*sth(j))
c           enddo
c         enddo
c       enddo
c
c       call zb3 (derivr,npd3)
c       call zb3 (derivi,npd3)
c
c       call fft (deriv,derivr,derivi,0)
        write (*,*)
        write (*,*) '### ERROR in QDIVV:'
        write (*,*) '### option UPWINDPHI must be >=0'
        write (*,*)
        write (*,*) 'UPWINDPHI = ',upwindphi
        write (*,*)
        write (9,*)
        write (9,*) '### ERROR in QDIVV:'
        write (9,*) '### option UPWINDPHI must be >=0'
        write (9,*)
        write (9,*) 'UPWINDPHI = ',upwindphi
        write (9,*)
        call abortrun
      end if
      return
      end
c#######################################################################
      subroutine advrho
c
c-----------------------------------------------------------------------
c
c ****** Advance the density using the mass continuity equation.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real divre(nr,nt,np),rhore(nr,nt,np),
     &     divr(nr,nt,npd3),divi(nr,nt,npd3),
     &     rhor(nr,nt,npd3),rhoi(nr,nt,npd3),
     &     rbcr1re(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Initialize quantities.
c ****** This may be unnecessary.  Needs to be checked!
c
      divre=0.
      rhore=0.
      divr=0.
      divi=0.
      rhor=rhr
      rhoi=rhi
c
c ****** Set the outer radial boundary values of rho.
c
      if (bcr1type.eq.'open') then
        do k=1,np
          do j=1,nt
            vr0=max(vr(nrm1,j,k),0.)
            fac=vr0*dtime/dr(nrm1)
            rbcr1re(j,k)=(1.-fac)*rho(nr,j,k)+fac*rho(nrm1,j,k)
          enddo
        enddo
        call fft_tp (rbcr1re,rhobcr1(1,1,1),rhobcr1(1,1,2),1)
      end if
c
c ****** Set the inner radial boundary values of rho.
c ****** This sets the values of the density in cells with
c ****** an inward-directed flow.
c
      do k=1,np
        rhogbc0( 1,k)=0.
        do j=2,ntm1
          rhogbc0(j,k)=rhorad(j,k)
ccc          if (rhogbc0(j,k).eq.0.) then
ccc            rhogbc0(j,k)=rhorad(j,k)
ccc          end if
        enddo
        rhogbc0(nt,k)=0.
      enddo
c
      call fft_tp (rhogbc0,rhobcr0(1,1,1),rhobcr0(1,1,2),1)
c
c ****** Predictor step.
c
      if (ifpc.ne.0) then
        if (predflow) then

          call divqv (rhr,rhi,vr,vt,vp,rhore,divr,divi,divre)
c
c
          do k=1,npd3
            do j=2,ntm1
              do i=2,nrm1
                rhor(i,j,k)=rhr(i,j,k)-divr(i,j,k)*dtime
                rhoi(i,j,k)=rhi(i,j,k)-divi(i,j,k)*dtime
              enddo
            enddo
          enddo
          wavefac=1.
        else
          wavefac=0.
        endif
        call qdivv (rho,vr,vt,vp,divr,divi,divre)
        call fft (divre,divr,divi,1)

        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              rhor(i,j,k)=rhor(i,j,k)-divr(i,j,k)*dtime*(betapc_p
     &                   -wavefac)
              rhoi(i,j,k)=rhoi(i,j,k)-divi(i,j,k)*dtime*(betapc_p
     &                   -wavefac)
            enddo
          enddo
        enddo
c
c ****** Set boundary conditions.
c
        call setbcrho (rhor,rhoi)
      
      end if
c
c ****** Corrector step.
c
      call divqv (rhor,rhoi,vr,vt,vp,rhore,divr,divi,divre)
c
c
      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            rhr(i,j,k)=rhr(i,j,k)-divr(i,j,k)*dtime
            rhi(i,j,k)=rhi(i,j,k)-divi(i,j,k)*dtime
          enddo
        enddo
      enddo
c
c ****** Set boundary conditions.
c
      call setbcrho (rhr,rhi)
c
c ****** Set the density in real space.
c
      call fft (rho,rhr,rhi,0)
c
      return
      end
c#######################################################################
      subroutine setbcrho (rhor,rhoi)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on density.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real rhor(nr,nt,npd3),rhoi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          rhor( 1,j,k)=2.*rhobcr0(j,k,1)-rhor(   2,j,k)
          rhoi( 1,j,k)=2.*rhobcr0(j,k,2)-rhoi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        if (bcr1type.eq.'open') then
          do j=2,ntm1
            rhor(nr,j,k)=rhobcr1(j,k,1)
            rhoi(nr,j,k)=rhobcr1(j,k,2)
          enddo
        else if (bcr1type.eq.'extrapolation') then
          do j=2,ntm1
            rhor(nr,j,k)=EXTRAPR1(rhor,nrm1,j,k)
            rhoi(nr,j,k)=EXTRAPR1(rhoi,nrm1,j,k)
          enddo
        else if (bcr1type.eq.'fixed') then
          do j=2,ntm1
            rhor(nr,j,k)=2.*rhobcr1(j,k,1)-rhor(nrm1,j,k)
            rhoi(nr,j,k)=2.*rhobcr1(j,k,2)-rhoi(nrm1,j,k)
          enddo
        else if (bcr1type.eq.'1dchar') then
          do j=2,ntm1
            rhor(nr,j,k)=2.*rhobcr1(j,k,1)-rhor(nrm1,j,k)
            rhoi(nr,j,k)=2.*rhobcr1(j,k,2)-rhoi(nrm1,j,k)
          enddo
        end if
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nr
            rhor(i, 1,k)=rhor(i,   2,k)
            rhor(i,nt,k)=rhor(i,ntm1,k)
            rhoi(i, 1,k)=rhoi(i,   2,k)
            rhoi(i,nt,k)=rhoi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nr
            rhor(i, 1,k)=-rhor(i,   2,k)
            rhor(i,nt,k)=-rhor(i,ntm1,k)
            rhoi(i, 1,k)=-rhoi(i,   2,k)
            rhoi(i,nt,k)=-rhoi(i,ntm1,k)
          enddo
        else
          do i=1,nr
            rhor(i, 1,k)=-rhor(i,   2,k)
            rhor(i,nt,k)=-rhor(i,ntm1,k)
            rhoi(i, 1,k)=-rhoi(i,   2,k)
            rhoi(i,nt,k)=-rhoi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
       subroutine setrho
c
c-----------------------------------------------------------------------
c
c ****** Set the density to give uniform Alfven speed.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      do k=1,np
        do i=2,nrm1
          do j=2,ntm1
            avbrsq=(.5*(br(i,j,k)**2+br(i,j-1,k)**2))
            avbtsq=(.5*(bt(i,j,k)**2+bt(i-1,j,k)**2))
            avbpsq=(.25*(bp(i,j,k)**2+bp(i-1,j,k)**2
     &             +bp(i,j-1,k)**2+bp(i-1,j-1,k)**2))
            bsq=avbrsq+avbtsq+avbpsq
            rho(i,j,k)=omegarho*bsq+(1.-omegarho)*rho(i,j,k)
          enddo
        enddo
        do i=1,nr
          rho(i, 1,k)=rho(i,   2,k)
          rho(i,nt,k)=rho(i,ntm1,k)
        enddo
        do j=2,ntm1
          rho( 1,j,k)=rho(   2,j,k)
          rho(nr,j,k)=rho(nrm1,j,k)
        enddo
      enddo
c
c ****** Put the density into the main complex arrays.
c
      call fft (rho,rhr,rhi,1)
c
      return
      end
c#######################################################################
      subroutine setbcg (targ)
c
c-----------------------------------------------------------------------
c
c ****** Set up the boundary condition arrays for the gas variables
c ****** as a function of time.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vrbcr0re(nt,np),vtbcr0re(nt,np),vpbcr0re(nt,np)
      real vrbcr1re(nt,np),vtbcr1re(nt,np),vpbcr1re(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** For a test case, set VMAG0=1.
c
      if (iftest.ne.0) then
        vmag0=1.
        return
      end if
c
c ***** Zero out the velocities.
c
      call bcast (vrbcr1,0.,2*nt*npd3)
      call bcast (vtbcr1,0.,2*nt*npd3)
      call bcast (vpbcr1,0.,2*nt*npd3)
c
      call bcast (vrparr0,0.,nt*np)
      call bcast (vtparr0,0.,nt*np)
      call bcast (vpparr0,0.,nt*np)
c
c ****** Compute the lower boundary with the user specified
c ****** boundary conditions.
c
      if (bcr0type.eq.'1dchar') then
        call getbc0
      end if
c
c ****** Compute the upper boundary with the user specified
c ****** boundary conditions.
c
      if (bcr1type.eq.'1dchar') then
        call getbc1
      else if (bcr1type.eq.'open') then
c
c ****** For 'open' outer boundary, advect vr, vt, vp using vr.
c
        do k=1,np
          do j=1,nt
            avvr=AVGR (vr,nrm1,j  ,k)
            vr0=max(avvr,0.)
            fac=vr0*dtime/drh(nrm1)
            vrbcr1re(j,k)=(1.-fac)*vr(nrm1,j,k)+fac*vr(nrm2,j,k)
          enddo
c
          do j=1,ntm1
            avvr=AVGT (vr,nrm1,j+1,k)
            vr0=max(avvr,0.)
            fac=vr0*dtime/dr(nrm1)
            vtbcr1re(j,k)=(1.-fac)*vt(nr  ,j,k)+fac*vt(nrm1,j,k)
          enddo
          vtbcr1re(nt,k)=0.
c
          do j=1,nt
            vr0=max(vr(nrm1,j,k),0.)
            fac=vr0*dtime/dr(nrm1)
            vpbcr1re(j,k)=(1.-fac)*vp(nr  ,j,k)+fac*vp(nrm1,j,k)
          enddo
        enddo
        call fft_tp (vrbcr1re,vrbcr1(1,1,1),vrbcr1(1,1,2),1)
        call fft_tp (vtbcr1re,vtbcr1(1,1,1),vtbcr1(1,1,2),1)
        call fft_tp (vpbcr1re,vpbcr1(1,1,1),vpbcr1(1,1,2),1)
      end if
c
c ****** Set the shear (transverse) velocity at the lower
c ****** radial boundary.
c
      call setvtrans0 (targ)
c
c ****** Calculate the "E x B" velocity at r=R0 for
c ****** an emerging flux run.
c
      call getexb0
c
c ****** Set the velocity boundary arrays on the lower
c ****** radial boundary.
c
      if (vrbcr0ef.eq.'exb') then
        vrmsk=1.
      else
        vrmsk=0.
      end if
c
      if (vtbcr0ef.eq.'exb') then
        vtmsk=1.
      else
        vtmsk=0.
      end if
c
      if (vpbcr0ef.eq.'exb') then
        vpmsk=1.
      else
        vpmsk=0.
      end if
c
      do k=1,np
        do j=1,nt
          vrbcr0re(j,k)=vrparr0(j,k)+vrtransr0(j,k)+vrmsk*vrexbr0(j,k)
        enddo
        do j=1,ntm1
          vtbcr0re(j,k)=vtparr0(j,k)+vttransr0(j,k)+vtmsk*vtexbr0(j,k)
        enddo
        vtbcr0re(nt,k)=0.
        do j=1,nt
          vpbcr0re(j,k)=vpparr0(j,k)+vptransr0(j,k)+vpmsk*vpexbr0(j,k)
        enddo
      enddo
c
      call fft_tp (vrbcr0re,vrbcr0(1,1,1),vrbcr0(1,1,2),1)
      call fft_tp (vtbcr0re,vtbcr0(1,1,1),vtbcr0(1,1,2),1)
      call fft_tp (vpbcr0re,vpbcr0(1,1,1),vpbcr0(1,1,2),1)
c
      return
      end
c#######################################################################
      subroutine setvtrans0 (targ)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary condition arrays for the "transverse"
c ****** velocity at the lower boundary.
c
c-----------------------------------------------------------------------
c
c ****** The velocity at the lower radial boundary
c ****** is set according to the magnitude (interpolated in time)
c ****** from the piecewise-linear function defined by
c ****** TNODE and VNODE.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      save ifirst,nodes
      data ifirst/1/
c
c-----------------------------------------------------------------------
c
c ***** Zero out the transverse velocity.
c
      call bcast (vrtransr0,0.,nt*np)
      call bcast (vttransr0,0.,nt*np)
      call bcast (vptransr0,0.,nt*np)
c
c ****** For a test case, set VMAG0=1.
c
      if (iftest.ne.0) then
        vmag0=1.
        return
      end if
c
c ****** On the first time in, check TNODE.
c
      if (ifirst.ne.0) then
c
c ****** Find the number of nodes specified.
c
        do 100 i=1,mxtnodes
          if (tnode(i).lt.0.) go to 150
  100   continue
  150   nodes=i-1
c
c ****** Check the time nodes.
c
        dummy=flint(0.,nodes,tnode,vnode,1,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in SETVTRANS:'
          write (*,*) '### Inconsistent input data in array TNODE.'
          write (9,*)
          write (9,*) '### ERROR in SETVTRANS:'
          write (9,*) '### Inconsistent input data in array TNODE.'
          call exit (1)
        end if
c
        ifirst=0
c
      end if
c
c ****** Interpolate to time TARG.
c
      vmag0=flint(targ,nodes,tnode,vnode,0,ierr)
c
c ****** Add in the applied shear.
c
      if (ishearprof.eq.1) then
        npow=4
        width=.5*dthmax
        do k=1,np
          do j=1,nt
            thp=(th(j)-.5*pi)/width
            vptransr0(j,k)=vmag0*thp*exp((1.-thp**npow)/npow)
          enddo
        enddo
      else if (ishearprof.eq.2) then
        do k=1,np
          do j=1,nt
            thp=-(th(j)-.5*pi)
            omega=arotate-brotate*sin(thp)**2
     &                   -crotate*sin(thp)**4
            vptransr0(j,k)=vmag0*omega*sin(th(j))
          enddo
        enddo
      else if (ishearprof.eq.3) then
        npow=4
        width=.5*dthmax
        do k=1,np
          do j=1,nt
            thp=(th(j)-th0)/width
            vptransr0(j,k)=vmag0*thp*exp((1.-thp**npow)/npow)
          enddo
        enddo
      else
        write (*,*)
        write (*,*) '### ERROR in SETVTRANS:'
        write (*,*) '### Invalid shear profile type.'
        write (*,*) 'ISHEARPROF = ',ishearprof
        write (9,*)
        write (9,*) '### ERROR in SETVTRANS:'
        write (9,*) '### Invalid shear profile type.'
        write (9,*) 'ISHEARPROF = ',ishearprof
        call exit (1)
      end if
c
      return
      end
c#######################################################################
      subroutine getexb0
c
c-----------------------------------------------------------------------
c
c ****** Get the "(E x B)/B**2" velocity at the lower boundary.
c ****** This is used as a boundary condition in emerging flux runs.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ***** Zero out the velocity.
c
      call bcast (vrexbr0,0.,nt*np)
      call bcast (vtexbr0,0.,nt*np)
      call bcast (vpexbr0,0.,nt*np)
c
      if (.not.emgflux) return
c
      bminsq=bmin**2
c
      do k=1,np
        do j=2,ntm1
          avbr=AVGRT(br,2,j  ,k)
          avbt=AVG  (bt,1,j  ,k)
          avbp=AVGT (bp,1,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          fac=(1.-exp(-4.*bsq/bminsq))
          bsq=max(bminsq,bsq)
          avet=.5*(et0(j,k)+et0(j-1,k))
          avep=ep0(j,k)
          vrexbr0(j,k)=(avet*avbp-avep*avbt)*fac/bsq
        enddo
      enddo
c
      do k=1,np
        do j=1,ntm1
          avbr=AVGR (br,2,j  ,k)
          avbt=AVGT (bt,1,j+1,k)
          avbp=AVG  (bp,1,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          fac=(1.-exp(-4.*bsq/bminsq))
          bsq=max(bminsq,bsq)
          aver=.5*(er0(j,k)+er0(j+1,k))
          avep=.5*(ep0(j,k)+ep0(j+1,k))
          vtexbr0(j,k)=(avep*avbr-aver*avbp)*fac/bsq
        enddo
      enddo
c
      do k=1,np
        do j=2,ntm1
          avbr=AVGRT(br,2,j  ,k)
          avbt=AVG  (bt,1,j  ,k)
          avbp=AVGT (bp,1,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          fac=(1.-exp(-4.*bsq/bminsq))
          bsq=max(bminsq,bsq)
          aver=er0(j,k)
          avet=.5*(et0(j,k)+et0(j-1,k))
          vpexbr0(j,k)=(aver*avbt-avet*avbr)*fac/bsq
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine getbc0
c
c-----------------------------------------------------------------------
c
c ****** Call the selected routine to solve the characteristic
c ****** equations at the lower radial boundary.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (parchar) then
c
c ****** Solve characterictics parallel to B.
c
        call getbc0b
c
      else
c
c ****** Solve "radial gas characterictics".
c
        call getbc0g
c
      end if
c
      return
      end
c#######################################################################
      subroutine getbc0g
c
c-----------------------------------------------------------------------
c
c ****** Get the boundary values at r=R0 using the characteristics.
c ****** This version solves the "radial gas characteristics".
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Maximum number of iterations in the characteristic solve.
c
      parameter (nitmax=100)
c
c-----------------------------------------------------------------------
c
c ****** Parallel flow velocity and expansion factor
c ****** at the boundary.
c
      common /gbc0com/ ub(nt,np),h1(nt,np)
c
c-----------------------------------------------------------------------
c
      real vtparr0h(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Zero out the expansion factor.  This quantity is not used
c ****** in this formulation, and is only used for diagnostic
c ****** purposes.
c
      call bcast (h1,0.,nt*np)
c
      do k=1,np
        do j=2,ntm1
c
c ****** Get the direction of B.
c
          brav=AVGRT(br,2,j,k)
          btav=AVG  (bt,1,j,k)
          bpav=AVGT (bp,1,j,k)
          bmag=sqrt(brav**2+btav**2+bpav**2)
          if (bmag.eq.0.) then
            brhat=1.
            bthat=0.
            bphat=0.
          else
            brhat=brav/bmag
            bthat=btav/bmag
            bphat=bpav/bmag
          end if
c
c ****** Boundary condition on p (specified).
c
          pb=.5*(p(1,j,k)+p(2,j,k))
c
c ****** Initialize the variables.
c
          ub(j,k)=vr(1,j,k)
          us=vr(1,j,k)
          ps=.5*(p(1,j,k)+p(2,j,k))
          rhos=.5*(rho(1,j,k)+rho(2,j,k))
          cs=sqrt(gammar(1)*ps/rhos)
c
          niter=0
c
  100     continue
c
          niter=niter+1
c
          dchi=-(us-cs)*dtime
c
          if (dchi.lt.0..or.dchi.gt.r(nrm1)) then
            write (*,*)
            write (*,*) '### ERROR in GETBC0G:'
            write (*,*) '### Anomaly in characteristic BCs:'
            write (*,*) 'DCHI is invalid.'
            write (*,*)
            write (*,*) 'NTIME  = ',ntime
            write (*,*) 'TIME  = ',time
            write (*,*) 'DCHI = ',dchi
            write (*,*) 'J = ',j
            write (*,*) 'K = ',k
            write (9,*)
            write (9,*) '### ERROR in GETBC0G:'
            write (9,*) '### Anomaly in characteristic BCs:'
            write (9,*) 'DCHI is invalid.'
            write (9,*)
            write (9,*) 'NTIME  = ',ntime
            write (9,*) 'TIME  = ',time
            write (9,*) 'DCHI = ',dchi
            write (9,*) 'J = ',j
            write (9,*) 'K = ',k
            call abortrun
          end if
c
          rval=r(1)+dchi
          call interp (r,nrm1,rval,ir,alpha)
          irp1=min0(ir+1,nrm1)
          uss=(1.-alpha)*vr(ir,j,k)+alpha*vr(irp1,j,k)
          us=.5*(uss+ub(j,k))
c
          rval=r(1)+.5*dchi
          call interp (rh,nr,rval,ir,alpha)
          irp1=min0(ir+1,nr)
          ps=(1.-alpha)*p(ir,j,k)+alpha*p(irp1,j,k)
          rhos=(1.-alpha)*rho(ir,j,k)+alpha*rho(irp1,j,k)
          cs=sqrt(gammar(1)*ps/rhos)
          rs=rval
c
          dp=2.*(pb-ps)
          du=dp/(rhos*cs)-g0*dtime/rs**2+2.*us*cs*dtime/rs
c
          ubold=ub(j,k)
          ub(j,k)=us+.5*du
c
          err=abs(ub(j,k)-ubold)/cs
c
          if (niter.gt.nitmax) then
            write (*,*)
            write (*,*) '### ERROR in GETBC0G:'
            write (*,*) '### Anomaly in characteristic BCs:'
            write (*,*) 'Iteration for UB did not converge.'
            write (*,*)
            write (*,*) 'NTIME  = ',ntime
            write (*,*) 'TIME  = ',time
            write (*,*) 'UB = ',ub(j,k)
            write (*,*) 'CS = ',cs
            write (*,*) 'ERR = ',err
            write (*,*) 'J = ',j
            write (*,*) 'K = ',k
            write (9,*)
            write (9,*) '### ERROR in GETBC0G:'
            write (9,*) '### Anomaly in characteristic BCs:'
            write (9,*) 'Iteration for UB did not converge.'
            write (9,*)
            write (9,*) 'NTIME  = ',ntime
            write (9,*) 'TIME  = ',time
            write (9,*) 'UB = ',ub(j,k)
            write (9,*) 'CS = ',cs
            write (9,*) 'ERR = ',err
            write (9,*) 'J = ',j
            write (9,*) 'K = ',k
            call abortrun
          end if
c
          if (idebug.gt.0) then
            if (j.eq.jhst.and.k.eq.khst) then
              if (niter.eq.1) then
                write (9,*)
                write (9,*) '### COMMENT from GETBC0G:'
                write (9,*) 'TIME = ',time,' J = ',j,' K = ',k
              end if
              write (9,*) 'NITER = ',niter,' ERR = ',err
              write (9,*) 'RS = ',rs,' DCHI = ',dchi
              write (9,*) 'UB = ',ub(j,k),' US = ',us,' CS = ',cs
              write (9,*) 'PS = ',ps,' RHOS = ',rhos
              write (9,*) 'PB = ',pb,' P_TERM = ',dp/(rhos*cs)
              write (9,*) 'V_TERM = ',  -fac*dudr,"G_TERM = ",
     &         -g0*dtime/rs**2
              write (9,*) 'U_TERM = ',  2.*us*cs*dtime/rs ,
     &        "DU = ",du

            end if
          end if
c
          if (err.gt.gbc0err) go to 100
c
c ****** Check to see if the flow is directed out of the domain
c ****** (ub.lt.0).  If so, specify p and determine rho from the
c ****** compatibility condition along the u-characteristic.
c ****** The rho values are stored in array RHOGBC0.
c
          if (ub(j,k).lt.0.) then
c
c ****** When UBZERO=.true., set the flow to zero if
c ****** it is negative.
c
            if (ubzero) then
              ub(j,k)=0.
              rhogbc0(j,k)=0.
            else
              drhodr=(rho(2,j,k)-rho(1,j,k))/dr(1)
              dpdr=(p(2,j,k)-p(1,j,k))/dr(1)
              ps=.5*(p(1,j,k)+p(2,j,k))
              rhos=.5*(rho(1,j,k)+rho(2,j,k))
              cssq=gammar(1)*ps/rhos
              dp=ub(j,k)*dtime*dpdr
              drho=dp/cssq
              rhob=rhos+drho-ub(j,k)*dtime*drhodr
              rhogbc0(j,k)=rhob
            end if
          else
            rhogbc0(j,k)=0.
          end if
c
c ****** Set UB to be the velocity parallel to B directed
c ****** into the domain.
c
          dir=sign(1.,brhat)
          ub(j,k)=ub(j,k)*brhat*dir
c
        enddo
      enddo
c
c ****** Compute the parallel velocity.
c
      do k=1,np
        do j=2,ntm1
          brav=AVGRT(br,2,j,k)
          btav=AVG  (bt,1,j,k)
          bpav=AVGT (bp,1,j,k)
          bmag=sqrt(brav**2+btav**2+bpav**2)
          if (bmag.eq.0.) then
            brhat=1.
            bthat=0.
            bphat=0.
          else
            brhat=brav/bmag
            bthat=btav/bmag
            bphat=bpav/bmag
          end if
          dir=sign(1.,brhat)
          vrparr0 (j,k)=ub(j,k)*brhat*dir
          vtparr0h(j,k)=ub(j,k)*bthat*dir
          vpparr0 (j,k)=ub(j,k)*bphat*dir
        enddo
      enddo
c
c ****** Filter the boundary velocity.
c
      call filt_tp (vrparr0 ,'r',nfiltub)
      call filt_tp (vtparr0h,'t',nfiltub)
      call filt_tp (vpparr0 ,'p',nfiltub)
c
c ***** Set v-theta on the appropriate mesh.
c
      do k=1,np
        do j=2,ntm2
          vtparr0(j,k)=.5*(vtparr0h(j,k)+vtparr0h(j+1,k))
        enddo
      enddo
c
c ****** Set the boundary and dummy points to zero.
c
      do k=1,np
c
        ub(   1,k)=0.
        ub(nt  ,k)=0.
c
        vrparr0(   1,k)=0.
        vrparr0(nt  ,k)=0.
c
        vtparr0(   1,k)=0.
        vtparr0(ntm1,k)=0.
        vtparr0(nt  ,k)=0.
c
        vpparr0(   1,k)=0.
        vpparr0(nt  ,k)=0.
c
        rhogbc0(   1,k)=0.
        rhogbc0(nt  ,k)=0.
c
      enddo
c
      return
      end
c#######################################################################
      subroutine getbc0b
c
c-----------------------------------------------------------------------
c
c ****** Get the boundary values at r=R0 using the characteristics.
c ****** This version solves the characteristics along B.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Maximum number of iterations in the characteristic solve.
c
      parameter (nitmax=100)
c
c ****** Criterion to determine if the magnetic field is
c ****** tangential to the boundary.
c
      parameter (epsbn=.01)
c
c-----------------------------------------------------------------------
c
c ****** Parallel flow velocity and expansion factor
c ****** at the boundary.
c
      common /gbc0com/ ub(nt,np),h1(nt,np)
c
c-----------------------------------------------------------------------
c
      real vtparr0h(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Solve the characteristic BCs for all theta and phi points.
c
      do k=1,np
c
        do j=2,ntm1
c
          xv0=r0*sth(j)*cp(k)
          yv0=r0*sth(j)*sp(k)
          zv0=r0*cth(j)
c
c ****** Boundary condition on p (specified).
c
          p0=.5*(p(1,j,k)+p(2,j,k))
c
c ****** Get the direction of B.
c
          brav=AVGRT(br,2,j,k)
          btav=AVG  (bt,1,j,k)
          bpav=AVGT (bp,1,j,k)
          bmag=sqrt(brav**2+btav**2+bpav**2)
          if (bmag.eq.0.) then
            brhat=1.
            bthat=0.
            bphat=0.
          else
            brhat=brav/bmag
            bthat=btav/bmag
            bphat=bpav/bmag
          end if
c
c ****** The s coordinate, which specifies the distance along
c ****** a magnetic field line, is defined to increase into the
c ****** domain.
c
c ****** Switch the sign of B so that s increases into the domain.
c
          dir=sign(1.,brhat)
c
          brhat=brhat*dir
          bthat=bthat*dir
          bphat=bphat*dir
c
          bxhat=brhat*sth(j)*cp(k)+bthat*cth(j)*cp(k)-bphat*sp(k)
          byhat=brhat*sth(j)*sp(k)+bthat*cth(j)*sp(k)+bphat*cp(k)
          bzhat=brhat*cth(j)      -bthat*sth(j)
c
c ****** Initialize the iteration.
c
          niter=0
          dchi=0.
          go to 200
c
c ****** Next iteration step.
c
  100     continue
c
          niter=niter+1
c
c ****** Drop the characteristic into the domain.
c
          dchi=-(us-cs)*dtime
c
          if (dchi.le.0.) then
            write (*,*)
            write (*,*) '### ERROR in GETBC0B:'
            write (*,*) '### Anomaly in characteristic BCs:'
            write (*,*) 'DCHI is invalid.'
            write (*,*)
            write (*,*) 'NTIME  = ',ntime
            write (*,*) 'TIME  = ',time
            write (*,*) 'DCHI = ',dchi
            write (*,*) 'J = ',j
            write (*,*) 'K = ',k
            write (9,*)
            write (9,*) '### ERROR in GETBC0B:'
            write (9,*) '### Anomaly in characteristic BCs:'
            write (9,*) 'DCHI is invalid.'
            write (9,*)
            write (9,*) 'NTIME  = ',ntime
            write (9,*) 'TIME  = ',time
            write (9,*) 'DCHI = ',dchi
            write (9,*) 'J = ',j
            write (9,*) 'K = ',k
            call abortrun
          end if
c
  200     continue
c
c ****** Define UB to be the inward-directed component of the
c ****** flow velocity along B at the boundary.  Thus, UB > 0
c ****** implies inflow.
c
c ****** Interpolate quantities at the midpoint of the characteristic.
c
          xv1=xv0+.5*dchi*bxhat
          yv1=yv0+.5*dchi*byhat
          zv1=zv0+.5*dchi*bzhat
c
          call c2s (xv1,yv1,zv1,rv,tv,pv)
          rv=max(r0,rv)
          rv=min(r1,rv)
c
          call interp (r ,nrm1,rv,ir ,ar )
          call interp (rh,nr  ,rv,irh,arh)
          call interp (t ,ntm1,tv,it ,at )
          call interp (th,nt  ,tv,ith,ath)
          call phifac (np,phi,pv,kk,kp1,ap)
c
          vrs= (1.-ar )*( (1.-ath)*( (1.-ap)*vr(ir   ,ith  ,kk )
     &                              +    ap *vr(ir   ,ith  ,kp1))
     &                   +    ath *( (1.-ap)*vr(ir   ,ith+1,kk )
     &                              +    ap *vr(ir   ,ith+1,kp1)))
     &        +    ar  *( (1.-ath)*( (1.-ap)*vr(ir +1,ith  ,kk )
     &                              +    ap *vr(ir +1,ith  ,kp1))
     &                  +     ath *( (1.-ap)*vr(ir +1,ith+1,kk )
     &                              +    ap *vr(ir +1,ith+1,kp1)))
c
          vts= (1.-arh)*( (1.-at )*( (1.-ap)*vt(irh  ,it   ,kk )
     &                              +    ap *vt(irh  ,it   ,kp1))
     &                   +    at  *( (1.-ap)*vt(irh  ,it +1,kk )
     &                              +    ap *vt(irh  ,it +1,kp1)))
     &        +    arh *( (1.-at )*( (1.-ap)*vt(irh+1,it   ,kk )
     &                              +    ap *vt(irh+1,it   ,kp1))
     &                  +     at  *( (1.-ap)*vt(irh+1,it +1,kk )
     &                              +    ap *vt(irh+1,it +1,kp1)))
c
          vps= (1.-arh)*( (1.-ath)*( (1.-ap)*vp(irh  ,ith  ,kk )
     &                              +    ap *vp(irh  ,ith  ,kp1))
     &                   +    ath *( (1.-ap)*vp(irh  ,ith+1,kk )
     &                              +    ap *vp(irh  ,ith+1,kp1)))
     &        +    arh *( (1.-ath)*( (1.-ap)*vp(irh+1,ith  ,kk )
     &                              +    ap *vp(irh+1,ith  ,kp1))
     &                  +     ath *( (1.-ap)*vp(irh+1,ith+1,kk )
     &                              +    ap *vp(irh+1,ith+1,kp1)))
c
          ps= (1.-arh)*( (1.-ath)*( (1.-ap)*p(irh  ,ith  ,kk )
     &                             +    ap *p(irh  ,ith  ,kp1))
     &                  +    ath *( (1.-ap)*p(irh  ,ith+1,kk )
     &                             +    ap *p(irh  ,ith+1,kp1)))
     &       +    arh *( (1.-ath)*( (1.-ap)*p(irh+1,ith  ,kk )
     &                             +    ap *p(irh+1,ith  ,kp1))
     &                 +     ath *( (1.-ap)*p(irh+1,ith+1,kk )
     &                             +    ap *p(irh+1,ith+1,kp1)))
c
          rhos= (1.-arh)*( (1.-ath)*( (1.-ap)*rho(irh  ,ith  ,kk )
     &                               +    ap *rho(irh  ,ith  ,kp1))
     &                    +    ath *( (1.-ap)*rho(irh  ,ith+1,kk )
     &                               +    ap *rho(irh  ,ith+1,kp1)))
     &         +    arh *( (1.-ath)*( (1.-ap)*rho(irh+1,ith  ,kk )
     &                               +    ap *rho(irh+1,ith  ,kp1))
     &                   +     ath *( (1.-ap)*rho(irh+1,ith+1,kk )
     &                               +    ap *rho(irh+1,ith+1,kp1)))
c
          us=vrs*brhat+vts*bthat+vps*bphat
          cs=sqrt(gammar(1)*ps/rhos)
          rs=rv
c
c ****** When on the initialization step, set UB in preparation
c ****** for the first iteration.
c
          if (niter.eq.0) then
            ub(j,k)=us
            go to 100
          end if
c
c ****** Get the area expansion factor.
c
          brs= (1.-arh)*( (1.-at )*( (1.-ap)*br(irh  ,it   ,kk )
     &                              +    ap *br(irh  ,it   ,kp1))
     &                   +    at  *( (1.-ap)*br(irh  ,it +1,kk )
     &                              +    ap *br(irh  ,it +1,kp1)))
     &        +    arh *( (1.-at )*( (1.-ap)*br(irh+1,it   ,kk )
     &                              +    ap *br(irh+1,it   ,kp1))
     &                  +     at  *( (1.-ap)*br(irh+1,it +1,kk )
     &                              +    ap *br(irh+1,it +1,kp1)))
c
          bts= (1.-ar )*( (1.-ath)*( (1.-ap)*bt(ir   ,ith  ,kk )
     &                              +    ap *bt(ir   ,ith  ,kp1))
     &                   +    ath *( (1.-ap)*bt(ir   ,ith+1,kk )
     &                              +    ap *bt(ir   ,ith+1,kp1)))
     &        +    ar  *( (1.-ath)*( (1.-ap)*bt(ir +1,ith  ,kk )
     &                              +    ap *bt(ir +1,ith  ,kp1))
     &                  +     ath *( (1.-ap)*bt(ir +1,ith+1,kk )
     &                              +    ap *bt(ir +1,ith+1,kp1)))
c
          bps= (1.-ar )*( (1.-at )*( (1.-ap)*bp(ir   ,it   ,kk )
     &                              +    ap *bp(ir   ,it   ,kp1))
     &                   +    at  *( (1.-ap)*bp(ir   ,it +1,kk )
     &                              +    ap *bp(ir   ,it +1,kp1)))
     &        +    ar  *( (1.-at )*( (1.-ap)*bp(ir +1,it   ,kk )
     &                              +    ap *bp(ir +1,it   ,kp1))
     &                  +     at  *( (1.-ap)*bp(ir +1,it +1,kk )
     &                              +    ap *bp(ir +1,it +1,kp1)))
c
          bmags=sqrt(brs**2+bts**2+bps**2)
c
          if (bmag.eq.0.) then
            h1(j,k)=2./rs
          else
            h1(j,k)=-(bmags-bmag)/(bmag*.5*dchi)
          end if
c
c ****** Update the boundary velocity.
c
          dp=2.*(p0-ps)
          du=dp/(rhos*cs)-g0*dtime*brhat/rs**2
     &       +us*cs*dtime*h1(j,k)
c
          ubold=ub(j,k)
          ub(j,k)=us+.5*du
c
          err=abs(ub(j,k)-ubold)/cs
c
          if (idebug.gt.0) then
            if (j.eq.jhst.and.k.eq.khst) then
              if (niter.eq.1) then
                write (9,*)
                write (9,*) '### COMMENT from GETBC0B:'
                write (9,*) 'TIME = ',time
                write (9,*) 'NTIME = ',ntime
                write (9,*) 'J = ',j,' K = ',k
                write (9,*) 'BMAG = ',bmag
                write (9,*) 'BRHAT = ',brhat
                write (9,*) 'BTHAT = ',bthat
                write (9,*) 'BPHAT = ',bphat
              end if
              write (9,*)
              write (9,*) 'NITER = ',niter,' ERR = ',err
              write (9,*) 'DCHI = ',dchi,' RS = ',rs
              write (9,*) 'RV = ',rv,' TV = ',tv,' PV = ',pv
              write (9,*) 'UB = ',ub(j,k),' US = ',us
              write (9,*) 'PS = ',ps,' RHOS = ',rhos,' CS = ',cs
              write (9,*) 'H1 = ',h1(j,k),' BMAGS = ',bmags
            end if
          end if
c
c ****** Check for convergence.
c
          if (err.lt.gbc0err) go to 300
c
c ****** Check for divergence.
c
          if (niter.gt.nitmax) then
            write (*,*)
            write (*,*) '### ERROR in GETBC0B:'
            write (*,*) '### Anomaly in characteristic BCs:'
            write (*,*) 'The iteration for UB did not converge.'
            write (*,*)
            write (*,*) 'NTIME  = ',ntime
            write (*,*) 'TIME  = ',time
            write (*,*) 'UB = ',ub(j,k)
            write (*,*) 'CS = ',cs
            write (*,*) 'ERR = ',err
            write (*,*) 'J = ',j
            write (*,*) 'K = ',k
            write (9,*)
            write (9,*) '### ERROR in GETBC0B:'
            write (9,*) '### Anomaly in characteristic BCs:'
            write (9,*) 'The iteration for UB did not converge.'
            write (9,*)
            write (9,*) 'NTIME  = ',ntime
            write (9,*) 'TIME  = ',time
            write (9,*) 'UB = ',ub(j,k)
            write (9,*) 'CS = ',cs
            write (9,*) 'ERR = ',err
            write (9,*) 'J = ',j
            write (9,*) 'K = ',k
            call abortrun
          end if
c
c ****** Iterate for another step.
c
          go to 100
c
c ****** Iteration converged.
c
  300     continue
c
c ****** Check to see if the flow is directed out of the domain
c ****** (UB.lt.0).  If so, specify p and determine rho from the
c ****** compatibility condition along the u-characteristic.
c ****** The rho values are stored in array RHOGBC0.
c
          if (ub(j,k).lt.0.) then
c
c ****** When UBZERO=.true., set the flow to zero if
c ****** it is negative.
c
            if (ubzero) then
              ub(j,k)=0.
              rhogbc0(j,k)=0.
              go to 400
            end if
c
            ds=-ub(j,k)*dtime
c
            xv1=xv0+ds*bxhat
            yv1=yv0+ds*byhat
            zv1=zv0+ds*bzhat
c
            call c2s (xv1,yv1,zv1,rv,tv,pv)
            rv=max(r0,rv)
            rv=min(r1,rv)
c
            call interp (rh,nr  ,rv,irh,arh)
            call interp (th,nt  ,tv,ith,ath)
            call phifac (np,phi,pv,kk,kp1,ap)
c
            ps= (1.-arh)*( (1.-ath)*( (1.-ap)*p(irh  ,ith  ,kk )
     &                               +    ap *p(irh  ,ith  ,kp1))
     &                    +    ath *( (1.-ap)*p(irh  ,ith+1,kk )
     &                               +    ap *p(irh  ,ith+1,kp1)))
     &         +    arh *( (1.-ath)*( (1.-ap)*p(irh+1,ith  ,kk )
     &                               +    ap *p(irh+1,ith  ,kp1))
     &                   +     ath *( (1.-ap)*p(irh+1,ith+1,kk )
     &                               +    ap *p(irh+1,ith+1,kp1)))
c
            rhos= (1.-arh)*( (1.-ath)*( (1.-ap)*rho(irh  ,ith  ,kk )
     &                                 +    ap *rho(irh  ,ith  ,kp1))
     &                      +    ath *( (1.-ap)*rho(irh  ,ith+1,kk )
     &                                 +    ap *rho(irh  ,ith+1,kp1)))
     &           +    arh *( (1.-ath)*( (1.-ap)*rho(irh+1,ith  ,kk )
     &                                 +    ap *rho(irh+1,ith  ,kp1))
     &                     +     ath *( (1.-ap)*rho(irh+1,ith+1,kk )
     &                                 +    ap *rho(irh+1,ith+1,kp1)))
c
            dp=(p0-ps)
            cssq=gammar(1)*ps/rhos
            drho=dp/cssq
            rhob=rhos+drho
            rhogbc0(j,k)=rhob
c
  400       continue
c
          else
c
            rhogbc0(j,k)=0.
c
          end if
c
c ****** Check if the field is tangential to the boundary.
c ****** If it is, set the parallel flow to zero.
c
          if (brhat.lt.epsbn) then
            ub(j,k)=ub(j,k)*brhat/epsbn
          end if
c
        enddo
      enddo
c
c ****** Compute the parallel velocity.
c
      do k=1,np
        do j=2,ntm1
          brav=AVGRT(br,2,j,k)
          btav=AVG  (bt,1,j,k)
          bpav=AVGT (bp,1,j,k)
          bmag=sqrt(brav**2+btav**2+bpav**2)
          if (bmag.eq.0.) then
            brhat=1.
            bthat=0.
            bphat=0.
          else
            brhat=brav/bmag
            bthat=btav/bmag
            bphat=bpav/bmag
          end if
          dir=sign(1.,brhat)
          vrparr0 (j,k)=ub(j,k)*brhat*dir
          vtparr0h(j,k)=ub(j,k)*bthat*dir
          vpparr0 (j,k)=ub(j,k)*bphat*dir
        enddo
      enddo
c
c ****** Filter the boundary velocity.
c
      call filt_tp (vrparr0 ,'r',nfiltub)
      call filt_tp (vtparr0h,'t',nfiltub)
      call filt_tp (vpparr0 ,'p',nfiltub)
c
c ***** Set v-theta on the appropriate mesh.
c
      do k=1,np
        do j=2,ntm2
          vtparr0(j,k)=.5*(vtparr0h(j,k)+vtparr0h(j+1,k))
        enddo
      enddo
c
c ****** Set the boundary and dummy points to zero.
c
      do k=1,np
c
        ub(   1,k)=0.
        ub(nt  ,k)=0.
c
        vrparr0(   1,k)=0.
        vrparr0(nt  ,k)=0.
c
        vtparr0(   1,k)=0.
        vtparr0(ntm1,k)=0.
        vtparr0(nt  ,k)=0.
c
        vpparr0(   1,k)=0.
        vpparr0(nt  ,k)=0.
c
        rhogbc0(   1,k)=0.
        rhogbc0(nt  ,k)=0.
c
      enddo
c
      return
      end
c#######################################################################
      subroutine c2s (x,y,z,r,t,p)
c
c-----------------------------------------------------------------------
c
c ****** Convert from Cartesian coordinates (X,Y,Z)
c ****** to spherical coordinates (R,T,P).
c
c ****** This routine returns T and P in radians, in the
c ****** following range:
c
c          0. .le. t .le. pi
c          0. .le. p .lt. 2.*pi
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pid2=.5*pi)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
      r=sqrt(x**2+y**2+z**2)
c
      if (r.eq.0.) then
        t=0.
      else
        t=acos(z/r)
      end if
c
      if (x.eq.0.) then
        if (y.ge.0.) then
          p=pid2
        else
          p=-pid2
        end if
      else
        p=atan2(y,x)
      end if
      if (p.lt.0.) p=p+pi2
c
      return
      end
c#######################################################################
      subroutine phifac (np,p,pv,k,kp1,ap)
c
c-----------------------------------------------------------------------
c
c ****** Get the interpolation factor at the phi value given by PV.
c
c-----------------------------------------------------------------------
c
      real p(np)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
c ****** Take care of the 2D case.
c
      if (np.eq.1) then
        k=1
        kp1=1
        ap=0.
        return
      end if
c
c ****** Take care of the periodic wrap-around.
c
      pvv=amod(pv,pi2)
      if (pvv.lt.0.) pvv=pvv+pi2
c
c ****** Get the phi interpolation factor.
c
      if (pvv.ge.p(np)) then
        k=np
        kp1=1
        ap=(pvv-p(np))/(p(np)-p(np-1))
      else
        call interp (p,np,pvv,k,ap)
        if (k.eq.np) then
          kp1=1
        else
          kp1=k+1
        end if
      end if
c
      return
      end
c#######################################################################
      subroutine filt_tp (f,comp,nfilt)
c
c-----------------------------------------------------------------------
c
c ****** Filter the vector component in the theta-phi array F.
c ****** COMP specifies the component of the vector.
c
c ****** This routine assumes that the array staggers the poles
c ****** in theta.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real f(nt,np)
      character*(*) comp
c
c-----------------------------------------------------------------------
c
      real fr(nt,npd3),fi(nt,npd3)
c
c-----------------------------------------------------------------------
c
      if (nfilt.le.0) return
c
c ****** Set the dummy values to zero.
c
      do k=1,np
        f( 1,k)=0.
        f(nt,k)=0.
      enddo
c
c ****** Get the FFT of the array F.
c
      call fft_tp (f,fr,fi,1)
c
c ****** Apply the filter NFILT times.
c
      do ifilt=1,nfilt
c
        do k=1,npd3
c
          m=k-1
c
c ****** Set boundary conditions at the poles.
c
          if (m.eq.0.or.m.eq.2) then
            if (comp.eq.'r') then
              pm= 1.
            else
              pm=-1.
            end if
          else if (m.eq.1) then
            if (comp.eq.'r') then
              pm=-1.
            else
              pm= 1.
            end if
          else
            if (comp.eq.'t') then
              pm=-1.
            else
              pm= 1.
            end if
          end if
c
          fr( 1,k)=pm*fr(   2,k)
          fi( 1,k)=pm*fi(   2,k)
          fr(nt,k)=pm*fr(ntm1,k)
          fi(nt,k)=pm*fi(ntm1,k)
c
c ****** Filter in theta.
c
          call filter (nt,fr(1,k))
          call filter (nt,fi(1,k))
c
c ****** Filter in phi.
c
          flt=cos(.5*dphi*m)**2
          do j=1,nt
            fr(j,k)=fr(j,k)*flt
            fi(j,k)=fi(j,k)*flt
          enddo
c
        enddo
c
      enddo
c
c ****** Transform back to real space.
c
      call fft_tp (f,fr,fi,0)
c
      return
      end
c#######################################################################
      subroutine getbc1
c
c-----------------------------------------------------------------------
c
c ****** Get the boundary values at r=R1 using the characteristics.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vrbcr1re(nt,np),vtbcr1re(nt,np),
     &     vpbcr1re(nt,np),rbcr1re(nt,np),
     &     pbcr1re(nt,np)
c
c-----------------------------------------------------------------------
c
      do k=1,np
        do j=2,ntm1
          rs=r(nrm1)
          us=vr(nrm1,j,k)
          ps=.5*( p(nr,j,k)+p(nrm1,j,k))
          rhos=.5*( rho(nr,j,k)+rho(nrm1,j,k))
          cs=sqrt(gammar(nrm1)*ps/rhos)
c
c ****** Get the values of the variables at the boundary.
c
          dudr=(vr(nrm1,j,k)-vr(nrm2,j,k))/drh(nrm1)
          drhodr=(rho(nr,j,k)-rho(nrm1,j,k))/dr(nrm1)
          dpdr=(p(nr,j,k)-p(nrm1,j,k))/dr(nrm1)
c
          if (us.gt.cs) then
c
c ****** Supersonic outflow boundary.
c
            pb=ps-dtime*(us*dpdr+rhos*cs**2*(dudr+2.*us/rs))
            ub=us-dtime*(dpdr/rhos+dudr*us+g0/rs**2)
            rhob=rhos-dtime*drhodr*us+(pb-ps+dtime*dpdr*us)/cs**2
c
          else if (us.gt.0.) then
c
c ****** Subsonic outflow boundary.
c
            pb=ps-dtime*(.5*(dpdr+rhos*cs*dudr)*(us+cs)
     &            +rhos*cs*(.5*g0/rs+us*cs)/rs)
            ub=us+(pb-ps)/(rhos*cs)
            rhob=rhos-dtime*drhodr*us+(pb-ps+dtime*dpdr*us)/cs**2
c
          else if (us.gt.-cs) then
c
c ****** Subsonic inflow boundary.
c
            pb=ps-dtime*(.5*(dpdr+rhos*cs*dudr)*(us+cs)
     &            +rhos*cs*(.5*g0/rs+us*cs)/rs)
            ub=us+(pb-ps)/(rhos*cs)
            rhob=rhos+(pb-ps)/cs**2
c
          else
c
c ****** Supersonic inflow: stop with an error.
c
            write (*,*)
            write (*,*) '### ERROR in GETBC1:'
            write (*,*) '### Anomaly in characteristic BCs:'
            write (*,*) 'Supersonic inflow at the upper boundary.'
            write (*,*)
            write (*,*) 'NTIME = ',ntime
            write (*,*) 'TIME = ',time
            write (*,*) 'US = ',us
            write (*,*) 'CS = ',cs
            write (*,*) 'J = ',j
            write (*,*) 'K = ',k
            write (9,*)
            write (9,*) '### ERROR in GETBC1:'
            write (9,*) '### Anomaly in characteristic BCs:'
            write (9,*) 'Supersonic inflow at the upper boundary.'
            write (9,*)
            write (9,*) 'NTIME = ',ntime
            write (9,*) 'TIME = ',time
            write (9,*) 'US = ',us
            write (9,*) 'CS = ',cs
            write (9,*) 'J = ',j
            write (9,*) 'K = ',k
            call abortrun
c
          end if
c
          if (idebug.gt.0) then
            if (j.eq.jhst.and.k.eq.khst) then
              write (9,*)
              write (9,*) '### COMMENT from GETBC1:'
              write (9,*) 'TIME = ',time,' NTIME = ',ntime,
     &                    ' J = ',j,' K = ',k
              write (9,*) 'UB = ',ub,' PB = ',pb,' RHOB = ',rhob
            end if
          end if
c
          vrbcr1re(j,k)=ub
          rbcr1re (j,k)=rhob
          pbcr1re (j,k)=pb
c
        enddo
      enddo
c
c ****** Set the boundary points to zero.
c
      do k=1,np
c
        vrbcr1re( 1,k)=0.
        vrbcr1re(nt,k)=0.
c
        pbcr1re ( 1,k)=0.
        pbcr1re (nt,k)=0.
c
        rbcr1re ( 1,k)=0.
        rbcr1re (nt,k)=0.
c
      enddo
c
c ****** Use (old) vr to advect vt and vp.
c
      do k=1,np
        do j=2,ntm2
          avvr=AVGT (vr,nrm1,j+1,k)
          vr0=max(avvr,0.)
          fac=vr0*dtime/dr(nrm1)
          vtbcr1re(j,k)=(1.-fac)*vt(nr  ,j,k)+fac*vt(nrm1,j,k)
        enddo
        do j=2,ntm1
          avvr=AVG  (vr,nrm1,j  ,k)
          vr0=max(avvr,0.)
          fac=vr0*dtime/dr(nrm1)
          vpbcr1re(j,k)=(1.-fac)*vp(nr  ,j,k)+fac*vp(nrm1,j,k)
        enddo
      enddo
c
c ****** Set the boundary and dummy points to zero.
c
      do k=1,np
c
        vtbcr1re(   1,k)=0.
        vtbcr1re(ntm1,k)=0.
        vtbcr1re(nt  ,k)=0.
c
        vpbcr1re(   1,k)=0.
        vpbcr1re(nt  ,k)=0.
c
      enddo
c
c ****** Transform the boundary values to complex space.
c
      call fft_tp (vrbcr1re,vrbcr1 (1,1,1),vrbcr1 (1,1,2),1)
      call fft_tp (vtbcr1re,vtbcr1 (1,1,1),vtbcr1 (1,1,2),1)
      call fft_tp (vpbcr1re,vpbcr1 (1,1,1),vpbcr1 (1,1,2),1)
      call fft_tp (pbcr1re ,pbcr1  (1,1,1),pbcr1  (1,1,2),1)
      call fft_tp (rbcr1re ,rhobcr1(1,1,1),rhobcr1(1,1,2),1)
c
      return
      end
c#######################################################################
      subroutine advv
c
c-----------------------------------------------------------------------
c
c ****** Advance the velocity.
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real dpcr(nr,nt,npd3),dpci(nr,nt,npd3),
     &     dpre(nr,nt,np),dpwre(nr,nt,np),
     &     vrbcr1re(nt,np),vtbcr1re(nt,np),vpbcr1re(nt,np)
c
      real vprr(nr,nt,npd3),vpri(nr,nt,npd3),
     &     vptr(nr,nt,npd3),vpti(nr,nt,npd3),
     &     vppr(nr,nt,npd3),vppi(nr,nt,npd3)
c
      real derivrr(nr,nt,npd3),derivri(nr,nt,npd3),
     &     derivtr(nr,nt,npd3),derivti(nr,nt,npd3),
     &     derivpr(nr,nt,npd3),derivpi(nr,nt,npd3)
c
      real jxbr(nr,nt,np),jxbt(nr,nt,np),jxbp(nr,nt,np)
c
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
c
      real srr(nr,nt,npd3),sri(nr,nt,npd3),
     &     str(nr,nt,npd3),sti(nr,nt,npd3),
     &     spr(nr,nt,npd3),spi(nr,nt,npd3)
c
      real vgrr(nr,nt,npd3),vgri(nr,nt,npd3),
     &     vgtr(nr,nt,npd3),vgti(nr,nt,npd3),
     &     vgpr(nr,nt,npd3),vgpi(nr,nt,npd3)

      real xcg(nrm2,ntm2,3),rhs(nrm2,ntm2,3)
c
      real coef(nrm1,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Initialize quantities.
c ****** This may be unnecessary.  Needs to be checked!
c
      derivrr=0.
      derivri=0.
      derivtr=0.
      derivti=0.
      derivpr=0.
      derivpi=0.
      vgrr=0.
      vgri=0.
      vgtr=0.
      vgti=0.
      vgpr=0.
      vgpi=0.
      vprr=vrr
      vpri=vri
      vptr=vtr
      vpti=vti
      vppr=vpr
      vppi=vpi
      vvr=vr
      vvt=vt
      vvp=vp
c
c ****** Set the outer radial boundary values of v.
c
      if (bcr1type.eq.'open') then
        do k=1,np
          do j=1,nt
            avvr=AVGR (vr,nrm1,j  ,k)
            vr0=max(avvr,0.)
            fac=vr0*dtime/drh(nrm1)
            vrbcr1re(j,k)=(1.-fac)*vr(nrm1,j,k)+fac*vr(nrm2,j,k)
          enddo
          do j=1,ntm1
            avvr=AVGT (vr,nrm1,j+1,k)
            vr0=max(avvr,0.)
            fac=vr0*dtime/dr(nrm1)
            vtbcr1re(j,k)=(1.-fac)*vt(nr  ,j,k)+fac*vt(nrm1,j,k)
          enddo
          vtbcr1re(nt,k)=0.
          do j=1,nt
            vr0=max(vr(nrm1,j,k),0.)
            fac=vr0*dtime/dr(nrm1)
            vpbcr1re(j,k)=(1.-fac)*vp(nr  ,j,k)+fac*vp(nrm1,j,k)
          enddo
        enddo
        call fft_tp (vrbcr1re,vrbcr1(1,1,1),vrbcr1(1,1,2),1)
        call fft_tp (vtbcr1re,vtbcr1(1,1,1),vtbcr1(1,1,2),1)
        call fft_tp (vpbcr1re,vpbcr1(1,1,1),vpbcr1(1,1,2),1)
      end if
c
c ****** Prepare the source term.
c
c ****** Get the J x B force.
c
      call zb1 (jxbr,np)
      call zb2 (jxbt,np)
      call zb3 (jxbp,np)
c
      do k=1,np
        do j=2,ntm1
        do i=2,nrm2
          avbt=AVG  (bt ,i  ,j  ,k)
          avbp=AVGT (bp ,i  ,j  ,k)
          avjt=AVGRT(fjt,i+1,j  ,k)
          avjp=AVGR (fjp,i+1,j  ,k)
          jxbr(i,j,k)=CROSSR(avj,avb)
        enddo
        enddo
        do j=2,ntm2
        do i=2,nrm1
          avbr=AVG  (br ,i  ,j  ,k)
          avbp=AVGR (bp ,i  ,j  ,k)
          avjr=AVGRT(fjr,i  ,j+1,k)
          avjp=AVGT (fjp,i  ,j+1,k)
          jxbt(i,j,k)=CROSST(avj,avb)
        enddo
        enddo
c
c ****** Check if we are doing the equatorial current sheet.
c
        if (ifeqcs.ne.0) then
          ntd2=nt/2
          if (nt.ne.2*ntd2) then
            do j=ntd2,ntd2+1
            do i=2,nrm1
              avbp=AVGR (bp ,i  ,j  ,k)
              avjr=AVGRT(fjr,i  ,j+1,k)
              brsqp=br(i,j+1,k)**2
              brsqm=br(i,j-1,k)**2
              rbtp=.5*r(i  )*(bt(i  ,j,k)+bt(i  ,j+1,k))
              rbtm=.5*r(i-1)*(bt(i-1,j,k)+bt(i-1,j+1,k))
              jxbt(i,j,k)= br(i,j,k)*(rbtp-rbtm)/(rh(i)*drh(i))
     &                    -.5*(brsqp-brsqm)/(rh(i)*(dth(j)+dth(j+1)))
     &                    -avjr*avbp
            enddo
            enddo
          end if
        end if
c
        do j=2,ntm1
        do i=2,nrm1
          avbr=AVGT (br ,i  ,j  ,k)
          avbt=AVGR (bt ,i  ,j  ,k)
          avjr=AVGR (fjr,i  ,j  ,k)
          avjt=AVGT (fjt,i  ,j  ,k)
          jxbp(i,j,k)=CROSSP(avj,avb)
        enddo
        enddo
      enddo
c
c ****** Get the phi-derivative of p in DPRE.
c ****** Use the arrays DPCR, DPCI as scratch.
c
      do k=1,npd3
      do j=2,ntm1
      do i=2,nrm1
        dpcr(i,j,k)=-fm(k)*pri(i,j,k)/(rh(i)*sth(j))
        dpci(i,j,k)= fm(k)*prr(i,j,k)/(rh(i)*sth(j))
      enddo
      enddo
      enddo
c
      call zb3 (dpcr,npd3)
      call zb3 (dpci,npd3)
c
      call fft (dpre,dpcr,dpci,0)
c
c ****** Get the phi-derivative of the Alfven wave pressure in DPWRE.
c ****** Use the arrays DPCR, DPCI as scratch.
c
      do k=1,npd3
      do j=2,ntm1
      do i=2,nrm1
        dpcr(i,j,k)=-fm(k)*.5*(epi(i,j,k)+emi(i,j,k))/(rh(i)*sth(j))
        dpci(i,j,k)= fm(k)*.5*(epr(i,j,k)+emr(i,j,k))/(rh(i)*sth(j))
      enddo
      enddo
      enddo
c
      call zb3 (dpcr,npd3)
      call zb3 (dpci,npd3)
c
      call fft (dpwre,dpcr,dpci,0)
c
c ****** Add the pressure and gravity forces to J x B,
c ****** and divide by rho.
c
      do k=1,np
        do j=2,ntm1
        do i=2,nrm2
          avrho=AVGR (rho,i+1,j  ,k)
          jxbr(i,j,k)=( jxbr(i,j,k)
     &                 -(p(i+1,j,k)-p(i,j,k))/dr(i)
     &                 -.5*( (ep(i+1,j,k)+em(i+1,j,k))
     &                      -(ep(i  ,j,k)+em(i  ,j,k)))/dr(i)
     &                )/avrho-g0/r(i)**2
        enddo
        enddo
        do j=2,ntm2
        do i=2,nrm1
          avrho=AVGT (rho,i  ,j+1,k)
          jxbt(i,j,k)=( jxbt(i,j,k)
     &                 -(p(i,j+1,k)-p(i,j,k))/(rh(i)*dt(j))
     &                 -.5*( (ep(i,j+1,k)+em(i,j+1,k))
     &                      -(ep(i,j  ,k)+em(i,j  ,k)))/(rh(i)*dt(j))
     &                )/avrho
        enddo
        enddo
        do j=2,ntm1
        do i=2,nrm1
          avrho=AVG  (rho,i  ,j  ,k)
          jxbp(i,j,k)=(jxbp(i,j,k)-dpre(i,j,k)-dpwre(i,j,k))/avrho
        enddo
        enddo
      enddo
c
c ****** Obtain the transform of this term in array S##.
c
      call fft (jxbr,srr,sri,1)
      call fft (jxbt,str,sti,1)
      call fft (jxbp,spr,spi,1)
c
      if (ifpc.ne.0) then
c
c ****** Predictor step.
c
        if (predflow) then
c
c ****** Flow in the predictor
c
          call vdgv (vr,vt,vp,
     &           vrr,vri,vtr,vti,vpr,vpi,
     &           derivrr,derivri,derivtr,derivti,derivpr,derivpi)

        else
          derivrr=0.
          derivri=0.
          derivtr=0.
          derivti=0.
          derivpr=0.
          derivpi=0.
        endif
c
c ****** SI term in the predictor
c
        if(betapc_si.eq.1.) then
          iopt=1
          coef=sifac
        else
          iopt=0
          coef=0.
        endif
        call delsq (vgrr,vgri,vgtr,vgti,vgpr,vgpi,coef)
c
c ****** Right hand side
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm2
              r00=AVGR (rhr,i+1,j  ,1)
              derivrr(i,j,k)=vrr(i,j,k)*r00+r00*dtime*(-derivrr(i,j,k)+
     &        srr(i,j,k)*betapc_v) -vgrr(i,j,k)*dtime
              derivri(i,j,k)=vri(i,j,k)*r00+r00*dtime*(-derivri(i,j,k)+
     &        sri(i,j,k)*betapc_v) -vgri(i,j,k)*dtime
            enddo
          enddo
          do j=2,ntm2
            do i=2,nrm1
              r00=AVGT (rhr,i  ,j+1,1)
              derivtr(i,j,k)=r00*vtr(i,j,k)+r00*dtime*(-derivtr(i,j,k)+
     &        str(i,j,k)*betapc_v) -vgtr(i,j,k)*dtime
              derivti(i,j,k)=r00*vti(i,j,k)+r00*dtime*(-derivti(i,j,k)+
     &        sti(i,j,k)*betapc_v) -vgti(i,j,k)*dtime
            enddo
          enddo
          do j=2,ntm1
            do i=2,nrm1
              r00=AVG  (rhr,i  ,j  ,1)
              derivpr(i,j,k)=r00*vpr(i,j,k)+r00*dtime*(-derivpr(i,j,k)+
     &        spr(i,j,k)*betapc_v) -vgpr(i,j,k)*dtime
              derivpi(i,j,k)=r00*vpi(i,j,k)+r00*dtime*(-derivpi(i,j,k)+
     &        spi(i,j,k)*betapc_v) -vgpi(i,j,k)*dtime
            enddo
          enddo
        enddo
c
c ****** SI solve in the predictor
c
c ****** Semi-implicit term.
c
        if (idebug.gt.0) then
          write (9,*)
          write (9,*) '### COMMENT from ADVV:'
          write (9,*) '### CG solution of semi-implicit equations:'
          write (9,*) '### [Predictor step.]'
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
        end if
c
        solvtype='si'
c
        call invdelsq (derivrr,derivri,derivtr,derivti,derivpr,derivpi,
     &   vprr,vpri,vptr,vpti,vppr,vppi,iopt,
     &   vprr,vpri,vptr,vpti,vppr,vppi)
c
c ****** Boundary conditions
c
        call setbcv (vprr,vpri,vptr,vpti,vppr,vppi)
c
c ****** Get the real predicted velocity.
c
        call fft (vvr,vprr,vpri,0)
        call fft (vvt,vptr,vpti,0)
        call fft (vvp,vppr,vppi,0)
      endif 
c
c ****** Corrector step.
c
      call vdgv (vvr,vvt,vvp,
     &           vprr,vpri,vptr,vpti,vppr,vppi,
     &         derivrr,derivri,derivtr,derivti,derivpr,derivpi)
      if (ifpc.eq.0) then
c
c ****** If no predictor is used
c ****** Set the guess for the velocity (for the CG solution
c ****** of the semi-implicit equations) equal to the explicit
c ****** part of the solution.  This ought to make the CG
c ****** solver converge in zero steps when the semi-implicit
c ****** term is zero (i.e., for an explicit case).
c ****** The guess is computed in the VG## array.
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm2
              vprr(i,j,k)=vrr(i,j,k)+dtime*(srr(i,j,k)-derivrr(i,j,k))
              vpri(i,j,k)=vri(i,j,k)+dtime*(sri(i,j,k)-derivri(i,j,k))
            enddo
          enddo
          do j=2,ntm2
            do i=2,nrm1
              vptr(i,j,k)=vtr(i,j,k)+dtime*(str(i,j,k)-derivtr(i,j,k))
              vpti(i,j,k)=vti(i,j,k)+dtime*(sti(i,j,k)-derivti(i,j,k))
            enddo
          enddo
          do j=2,ntm1
            do i=2,nrm1
              vppr(i,j,k)=vpr(i,j,k)+dtime*(spr(i,j,k)-derivpr(i,j,k))
              vppi(i,j,k)=vpi(i,j,k)+dtime*(spi(i,j,k)-derivpi(i,j,k))
            enddo
          enddo
        enddo
      endif
c
c ****** Check if the viscosity is being split or not.
c
      if (split_visc) then
        iopt=1
      else
        iopt=3
      end if
c
      coef=sifac
c
c ****** Right hand side
c
      call delsq (vgrr,vgri,vgtr,vgti,vgpr,vgpi,coef)
      
      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm2
            r00=AVGR (rhr,i+1,j  ,1)
            derivrr(i,j,k)=vrr(i,j,k)*r00+r00*dtime*(-derivrr(i,j,k)+
     &      srr(i,j,k)) -vgrr(i,j,k)*dtime
            derivri(i,j,k)=vri(i,j,k)*r00+r00*dtime*(-derivri(i,j,k)+
     &      sri(i,j,k)) -vgri(i,j,k)*dtime
          enddo
        enddo
        do j=2,ntm2
          do i=2,nrm1
            r00=AVGT (rhr,i  ,j+1,1)
            derivtr(i,j,k)=r00*vtr(i,j,k)+r00*dtime*(-derivtr(i,j,k)+
     &      str(i,j,k)) -vgtr(i,j,k)*dtime
            derivti(i,j,k)=r00*vti(i,j,k)+r00*dtime*(-derivti(i,j,k)+
     &      sti(i,j,k)) -vgti(i,j,k)*dtime
          enddo
        enddo
        do j=2,ntm1
          do i=2,nrm1
            r00=AVG  (rhr,i  ,j  ,1)
            derivpr(i,j,k)=r00*vpr(i,j,k)+r00*dtime*(-derivpr(i,j,k)+
     &      spr(i,j,k)) -vgpr(i,j,k)*dtime
            derivpi(i,j,k)=r00*vpi(i,j,k)+r00*dtime*(-derivpi(i,j,k)+
     &      spi(i,j,k)) -vgpi(i,j,k)*dtime
          enddo
        enddo
      enddo
c
c ****** SI advance
c
      if (idebug.gt.0) then
        write (9,*)
        write (9,*) '### COMMENT from ADVV:'
        write (9,*) '### CG solution of semi-implicit equations:'
        write (9,*) '### [Corrector step.]'
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
      end if
c
      solvtype='si'
c
      call invdelsq (derivrr,derivri,derivtr,derivti,derivpr,derivpi,
     &   vprr,vpri,vptr,vpti,vppr,vppi,iopt,
     &   vrr,vri,vtr,vti,vpr,vpi)
c
      if (split_visc) then
c
c ****** Do the implicit viscosity advance.
c
c ****** Load the rhs.
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm2
              r00=AVGR (rhr,i+1,j  ,1)
              srr(i,j,k)=r00*vrr(i,j,k)
              sri(i,j,k)=r00*vri(i,j,k)
            enddo
          enddo
          do j=2,ntm2
            do i=2,nrm1
              r00=AVGT (rhr,i  ,j+1,1)
              str(i,j,k)=r00*vtr(i,j,k)
              sti(i,j,k)=r00*vti(i,j,k)
            enddo
          enddo
          do j=2,ntm1
            do i=2,nrm1
              r00=AVG  (rhr,i  ,j  ,1)
              spr(i,j,k)=r00*vpr(i,j,k)
              spi(i,j,k)=r00*vpi(i,j,k)
            enddo
          enddo
        enddo
c
c ****** Set the guess for the velocity equal to the explicit
c ****** part of the solution.  This ought to make the CG
c ****** solver converge rapidly when the viscosity
c ****** is small.
c ****** The guess is computed in the VG## array.
c
        do j=1,ntm1
          do i=1,nrm1
            coef(i,j)=visfac(i,j)+viscel(i,j)
          enddo
        enddo
        call delsq (vgrr,vgri,vgtr,vgti,vgpr,vgpi,coef)
c
        do k=1,npd3
c
        do j=2,ntm1
        do i=2,nrm2
          r00=AVGR (rhr,i+1,j  ,1)
          vgrr(i,j,k)=vrr(i,j,k)+dtime*vgrr(i,j,k)/r00
          vgri(i,j,k)=vri(i,j,k)+dtime*vgri(i,j,k)/r00
        enddo
        enddo
c
        do j=2,ntm2
        do i=2,nrm1
          r00=AVGT (rhr,i  ,j+1,1)
          vgtr(i,j,k)=vtr(i,j,k)+dtime*vgtr(i,j,k)/r00
          vgti(i,j,k)=vti(i,j,k)+dtime*vgti(i,j,k)/r00
        enddo
        enddo
c
        do j=2,ntm1
        do i=2,nrm1
          r00=AVG  (rhr,i  ,j  ,1)
          vgpr(i,j,k)=vpr(i,j,k)+dtime*vgpr(i,j,k)/r00
          vgpi(i,j,k)=vpi(i,j,k)+dtime*vgpi(i,j,k)/r00
        enddo
        enddo
c
      enddo
c
c ****** Solve the implicit equations.
c
        if (idebug.gt.0) then
          write (9,*)
          write (9,*) '### COMMENT from ADVV:'
          write (9,*) '### CG solution of implicit viscosity:'
          write (9,*) 'NTIME = ',ntime
          write (9,*) 'TIME = ',time
        end if
c
        solvtype='visc'
c
        call invdelsq (srr,sri,str,sti,spr,spi,
     &     vgrr,vgri,vgtr,vgti,vgpr,vgpi,2,
     &     vrr,vri,vtr,vti,vpr,vpi)
c
      end if
c
c ****** Get mode history information for v.
c
      call modehst (vrr,vri,vtr,vti,vpr,vpi,'v')
c
c ****** Load v in real space.
c
      call fft (vr,vrr,vri,0)
      call fft (vt,vtr,vti,0)
      call fft (vp,vpr,vpi,0)
c
      return
      end
c#######################################################################
      subroutine delsq (srr,sri,str,sti,spr,spi,source)
c
c-----------------------------------------------------------------------
c
c ****** Get del-squared of v, with a coefficient term equal to
c ****** SOURCE.  The result is returned in the (complex)
c ****** vector S.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*72 errmsg

      real srr(nr,nt,npd3),sri(nr,nt,npd3),
     &     str(nr,nt,npd3),sti(nr,nt,npd3),
     &     spr(nr,nt,npd3),spi(nr,nt,npd3)
      real source(nrm1,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Add in the explicit parts to obtain the source term.
c
      do k=1,npd3
c
      emm=fm(k)
      emm2=fm2(k)
c
      do j=2,ntm1
      do i=2,nrm2
        s00=AVGR (rhr,i+1,j  ,1)*AVG2T (source,i  ,j  )
        sp0=AVG  (rhr,i+1,j  ,1)*AVG2RT(source,i+1,j  )
        sm0=AVG  (rhr,i  ,j  ,1)*AVG2RT(source,i  ,j  )
        s0p=AVGRT(rhr,i+1,j+1,1)*AVG2  (source,i  ,j  )
        s0m=AVGRT(rhr,i+1,j  ,1)*AVG2  (source,i  ,j-1)
        spp=AVGT (rhr,i+1,j+1,1)*AVG2R (source,i+1,j  )
        smp=AVGT (rhr,i  ,j+1,1)*AVG2R (source,i  ,j  )
        spm=AVGT (rhr,i+1,j  ,1)*AVG2R (source,i+1,j-1)
        smm=AVGT (rhr,i  ,j  ,1)*AVG2R (source,i  ,j-1)
        r00=AVGR (rhr,i+1,j  ,1)
        dsqvrr=
     &   ( sp0*rh(i+1)**2*(vrr(i+1,j,k)-vrr(i  ,j,k))/drh(i+1)
     &    -sm0*rh(i  )**2*(vrr(i  ,j,k)-vrr(i-1,j,k))/drh(i  )
     &   )/(r(i)**2*dr(i))
     &  +( s0p*st(j  )*(vrr(i,j+1,k)-vrr(i,j  ,k))/dt(j  )
     &    -s0m*st(j-1)*(vrr(i,j  ,k)-vrr(i,j-1,k))/dt(j-1)
     &   )/(r(i)**2*dth(j)*sth(j))
     &  -2.*s00*vrr(i,j,k)/r(i)**2
     &  +.5*( (1./rh(i+1)-rh(i+1)/r(i)**2)
     &        *( st(j  )*(s00+spp)*vtr(i+1,j  ,k)
     &          -st(j-1)*(s00+spm)*vtr(i+1,j-1,k))
     &       -(1./rh(i  )-rh(i  )/r(i)**2)
     &        *( st(j  )*(s00+smp)*vtr(i  ,j  ,k)
     &          -st(j-1)*(s00+smm)*vtr(i  ,j-1,k))
     &      )/(sth(j)*dth(j)*dr(i))
     &  -emm2*s00*vrr(i,j,k)/(r(i)*sth(j))**2
     &  -.5*emm*( ( (s00+sp0)*vpi(i+1,j,k)/rh(i+1)
     &             -(s00+sm0)*vpi(i  ,j,k)/rh(i  ))
     &           -( (s00+sp0)*vpi(i+1,j,k)*rh(i+1)
     &             -(s00+sm0)*vpi(i  ,j,k)*rh(i  ))/r(i)**2
     &          )/(sth(j)*dr(i))
        dsqvri=
     &   ( sp0*rh(i+1)**2*(vri(i+1,j,k)-vri(i  ,j,k))/drh(i+1)
     &    -sm0*rh(i  )**2*(vri(i  ,j,k)-vri(i-1,j,k))/drh(i  )
     &   )/(r(i)**2*dr(i))
     &  +( s0p*st(j  )*(vri(i,j+1,k)-vri(i,j  ,k))/dt(j  )
     &    -s0m*st(j-1)*(vri(i,j  ,k)-vri(i,j-1,k))/dt(j-1)
     &   )/(r(i)**2*dth(j)*sth(j))
     &  -2.*s00*vri(i,j,k)/r(i)**2
     &  +.5*( (1./rh(i+1)-rh(i+1)/r(i)**2)
     &        *( st(j  )*(s00+spp)*vti(i+1,j  ,k)
     &          -st(j-1)*(s00+spm)*vti(i+1,j-1,k))
     &       -(1./rh(i  )-rh(i  )/r(i)**2)
     &        *( st(j  )*(s00+smp)*vti(i  ,j  ,k)
     &          -st(j-1)*(s00+smm)*vti(i  ,j-1,k))
     &      )/(sth(j)*dth(j)*dr(i))
     &  -emm2*s00*vri(i,j,k)/(r(i)*sth(j))**2
     &  +.5*emm*( ( (s00+sp0)*vpr(i+1,j,k)/rh(i+1)
     &             -(s00+sm0)*vpr(i  ,j,k)/rh(i  ))
     &           -( (s00+sp0)*vpr(i+1,j,k)*rh(i+1)
     &             -(s00+sm0)*vpr(i  ,j,k)*rh(i  ))/r(i)**2
     &          )/(sth(j)*dr(i))
        srr(i,j,k)=dsqvrr
        sri(i,j,k)=dsqvri
      enddo
      enddo
c
      do j=2,ntm2
      do i=2,nrm1
        s00=AVGT (rhr,i  ,j+1,1)*AVG2R (source,i  ,j  )
        sp0=AVGRT(rhr,i+1,j+1,1)*AVG2  (source,i  ,j  )
        sm0=AVGRT(rhr,i  ,j+1,1)*AVG2  (source,i-1,j  )
        s0p=AVG  (rhr,i  ,j+1,1)*AVG2RT(source,i  ,j+1)
        s0m=AVG  (rhr,i  ,j  ,1)*AVG2RT(source,i  ,j  )
        spp=AVGR (rhr,i+1,j+1,1)*AVG2T (source,i  ,j+1)
        smp=AVGR (rhr,i  ,j+1,1)*AVG2T (source,i-1,j+1)
        spm=AVGR (rhr,i+1,j  ,1)*AVG2T (source,i  ,j  )
        smm=AVGR (rhr,i  ,j  ,1)*AVG2T (source,i-1,j  )
        r00=AVGT (rhr,i  ,j+1,1)
        dsqvtr=
     &   ( sp0*r(i  )**2*(vtr(i+1,j,k)-vtr(i  ,j,k))/dr(i  )
     &    -sm0*r(i-1)**2*(vtr(i  ,j,k)-vtr(i-1,j,k))/dr(i-1)
     &   )/(rh(i)**2*drh(i))
     &  +( s0p*sth(j+1)*(vtr(i,j+1,k)-vtr(i,j  ,k))/dth(j+1)
     &    -s0m*sth(j  )*(vtr(i,j  ,k)-vtr(i,j-1,k))/dth(j  )
     &   )/(rh(i)**2*dt(j)*st(j))
     &  -s00*vtr(i,j,k)/(rh(i)**2*st(j)**2)
     &  -.5*( (rh(i)-r(i  )**2/rh(i))
     &        *( (s00+spp)*vrr(i  ,j+1,k)
     &          -(s00+spm)*vrr(i  ,j  ,k))
     &       -(rh(i)-r(i-1)**2/rh(i))
     &        *( (s00+smp)*vrr(i-1,j+1,k)
     &          -(s00+smm)*vrr(i-1,j  ,k))
     &      )/(rh(i)**2*dt(j)*drh(i))
     &  -emm2*s00*vtr(i,j,k)/(rh(i)*st(j))**2
     &  -.5*emm*( ( (s00+s0p)*vpi(i,j+1,k)/sth(j+1)
     &             -(s00+s0m)*vpi(i,j  ,k)/sth(j  ))
     &           -( (s00+s0p)*vpi(i,j+1,k)*sth(j+1)
     &             -(s00+s0m)*vpi(i,j  ,k)*sth(j  ))/st(j)**2
     &          )/(rh(i)**2*dt(j))
        dsqvti=
     &   ( sp0*r(i  )**2*(vti(i+1,j,k)-vti(i  ,j,k))/dr(i  )
     &    -sm0*r(i-1)**2*(vti(i  ,j,k)-vti(i-1,j,k))/dr(i-1)
     &   )/(rh(i)**2*drh(i))
     &  +( s0p*sth(j+1)*(vti(i,j+1,k)-vti(i,j  ,k))/dth(j+1)
     &    -s0m*sth(j  )*(vti(i,j  ,k)-vti(i,j-1,k))/dth(j  )
     &   )/(rh(i)**2*dt(j)*st(j))
     &  -s00*vti(i,j,k)/(rh(i)**2*st(j)**2)
     &  -.5*( (rh(i)-r(i  )**2/rh(i))
     &        *( (s00+spp)*vri(i  ,j+1,k)
     &          -(s00+spm)*vri(i  ,j  ,k))
     &       -(rh(i)-r(i-1)**2/rh(i))
     &        *( (s00+smp)*vri(i-1,j+1,k)
     &          -(s00+smm)*vri(i-1,j  ,k))
     &      )/(rh(i)**2*dt(j)*drh(i))
     &  -emm2*s00*vti(i,j,k)/(rh(i)*st(j))**2
     &  +.5*emm*( ( (s00+s0p)*vpr(i,j+1,k)/sth(j+1)
     &             -(s00+s0m)*vpr(i,j  ,k)/sth(j  ))
     &           -( (s00+s0p)*vpr(i,j+1,k)*sth(j+1)
     &             -(s00+s0m)*vpr(i,j  ,k)*sth(j  ))/st(j)**2
     &          )/(rh(i)**2*dt(j))
        str(i,j,k)=dsqvtr
        sti(i,j,k)=dsqvti
      enddo
      enddo
c
      do j=2,ntm1
      if (j.eq.2) then
        dstj  =1./st(j  )
        dstjm1=5./dt(1)
      else if (j.eq.ntm1) then
        dstj  =5./dt(ntm1)
        dstjm1=1./st(j-1)
      else
        dstj  =1./st(j  )
        dstjm1=1./st(j-1)
      end if
      do i=2,nrm1
        s00=AVG  (rhr,i  ,j  ,1)*AVG2RT(source,i  ,j  )
        sp0=AVGR (rhr,i+1,j  ,1)*AVG2T (source,i  ,j  )
        sm0=AVGR (rhr,i  ,j  ,1)*AVG2T (source,i-1,j  )
        s0p=AVGT (rhr,i  ,j+1,1)*AVG2R (source,i  ,j  )
        s0m=AVGT (rhr,i  ,j  ,1)*AVG2R (source,i  ,j-1)
        r00=AVG  (rhr,i  ,j  ,1)
        dsqvpr=
     &   ( sp0*r(i  )**2*(vpr(i+1,j,k)-vpr(i  ,j,k))/dr(i  )
     &    -sm0*r(i-1)**2*(vpr(i  ,j,k)-vpr(i-1,j,k))/dr(i-1)
     &   )/(rh(i)**2*drh(i))
     &  +( s0p*st(j  )*(vpr(i,j+1,k)-vpr(i,j  ,k))/dt(j  )
     &    -s0m*st(j-1)*(vpr(i,j  ,k)-vpr(i,j-1,k))/dt(j-1)
     &   )/(rh(i)**2*dth(j)*sth(j))
     &  -(1.+emm2)*s00*vpr(i,j,k)/(rh(i)*sth(j))**2
     &  -.5*emm*( ( (s00+sp0)*vri(i  ,j,k)*r(i  )**2
     &             -(s00+sm0)*vri(i-1,j,k)*r(i-1)**2)/rh(i)**2
     &           -( (s00+sp0)*vri(i  ,j,k)
     &             -(s00+sm0)*vri(i-1,j,k))
     &          )/(rh(i)*sth(j)*drh(i))
     &  +.5*emm*( ( (s00+s0p)*vti(i,j  ,k)*dstj
     &             -(s00+s0m)*vti(i,j-1,k)*dstjm1)
     &           -( (s00+s0p)*vti(i,j  ,k)*st(j  )
     &             -(s00+s0m)*vti(i,j-1,k)*st(j-1))/sth(j)**2
     &          )/(rh(i)**2*dth(j))
        dsqvpi=
     &   ( sp0*r(i  )**2*(vpi(i+1,j,k)-vpi(i  ,j,k))/dr(i  )
     &    -sm0*r(i-1)**2*(vpi(i  ,j,k)-vpi(i-1,j,k))/dr(i-1)
     &   )/(rh(i)**2*drh(i))
     &  +( s0p*st(j  )*(vpi(i,j+1,k)-vpi(i,j  ,k))/dt(j  )
     &    -s0m*st(j-1)*(vpi(i,j  ,k)-vpi(i,j-1,k))/dt(j-1)
     &   )/(rh(i)**2*dth(j)*sth(j))
     &  -(1.+emm2)*s00*vpi(i,j,k)/(rh(i)*sth(j))**2
     &  +.5*emm*( ( (s00+sp0)*vrr(i  ,j,k)*r(i  )**2
     &             -(s00+sm0)*vrr(i-1,j,k)*r(i-1)**2)/rh(i)**2
     &           -( (s00+sp0)*vrr(i  ,j,k)
     &             -(s00+sm0)*vrr(i-1,j,k))
     &          )/(rh(i)*sth(j)*drh(i))
     &  -.5*emm*( ( (s00+s0p)*vtr(i,j  ,k)*dstj
     &             -(s00+s0m)*vtr(i,j-1,k)*dstjm1)
     &           -( (s00+s0p)*vtr(i,j  ,k)*st(j  )
     &             -(s00+s0m)*vtr(i,j-1,k)*st(j-1))/sth(j)**2
     &          )/(rh(i)**2*dth(j))
        spr(i,j,k)=dsqvpr
        spi(i,j,k)=dsqvpi
      enddo
      enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine invdelsq (srr,sri,str,sti,spr,spi,vgrr,vgri,vgtr,vgti,
     &     vgpr,vgpi,iopt,vorr,vori, votr,voti, vopr,vopi)
c
c-----------------------------------------------------------------------
c
c ****** Semi-implicit advance.
c
c-----------------------------------------------------------------------
c
      use solver_types
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real vgrr(nr,nt,npd3),vgri(nr,nt,npd3),
     &     vgtr(nr,nt,npd3),vgti(nr,nt,npd3),
     &     vgpr(nr,nt,npd3),vgpi(nr,nt,npd3)
      real vorr(nr,nt,npd3),vori(nr,nt,npd3),
     &     votr(nr,nt,npd3),voti(nr,nt,npd3),
     &     vopr(nr,nt,npd3),vopi(nr,nt,npd3)
      real srr(nr,nt,npd3),sri(nr,nt,npd3),
     &     str(nr,nt,npd3),sti(nr,nt,npd3),
     &     spr(nr,nt,npd3),spi(nr,nt,npd3)

      real xcg(nr,nt,3),rhs(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      character*32 eqmsg
      character*72 errmsg
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for the matrix diagonals.
c
      allocate (arr00(nr,nt))
      allocate (arrp0(nr,nt))
      allocate (arr0p(nr,nt))
      allocate (att00(nr,nt))
      allocate (attp0(nr,nt))
      allocate (att0p(nr,nt))
      allocate (app00(nr,nt))
      allocate (appp0(nr,nt))
      allocate (app0p(nr,nt))
      allocate (art00(nr,nt))
      allocate (artp0(nr,nt))
      allocate (art0m(nr,nt))
      allocate (artpm(nr,nt))
      allocate (arp00(nr,nt))
      allocate (arpp0(nr,nt))
      allocate (atp00(nr,nt))
      allocate (atp0p(nr,nt))
c
c ****** Allocate memory for the preconditioning diagonals.
c
      allocate (drr(nr,nt))
      allocate (dtt(nr,nt))
      allocate (dpp(nr,nt))
c
      if (solvtype.eq.'si') then
        eqmsg='semi-implicit'
      else
        eqmsg='implicit viscosity'
      end if
c
      le=lenstr(eqmsg)
c
c ****** Solve for the velocity components.
c
      do 700 k=1,npd3
c
        m=k-1
c
c ****** Load the CG boundary condition arrays.
c
        if (m.eq.0) then
c
          call loadvbc (m,1,1,1)
c
c ****** Solve the 2 coupled equations for (vr,vt).
c
          solve_type=ST_VMATRT0
c
          call putvrt (srr,str,rhs)
          call vmatrt0 (rhs,iopt)
          call putvrt (vgrr,vgtr,xcg)
          call solve (xcg,rhs,ierr)
          if (ierr.ne.0) then
            errmsg=eqmsg(1:le)//' (VRR,VTR) equations.'
            go to 1000
          end if
          call setvrt (xcg,vorr,votr)
c
          if (idebug.gt.0) then
            write (9,600) '### VRR,VTR    :',m,ncg,bnrm,rnrm
  600       format (/,1x,a,' M=',i3,' N=',i5,' |B|=',1pe8.2,
     &              ' |R|=',1pe8.2)
          end if
c
c ****** Solve the equation for vp.
c
          solve_type=ST_VMATP0
c
          call putvp (spr,rhs)
          call vmatp0 (rhs,iopt)
          call putvp (vgpr,xcg)
          call solve (xcg,rhs,ierr)
          if (ierr.ne.0) then
            errmsg=eqmsg(1:le)//' VPR equation.'
            go to 1000
          end if
          call setvp (xcg,vopr)
c
          if (idebug.gt.0)
     &      write (9,600) '### VPR        :',m,ncg,bnrm,rnrm
c
        else
c
c ****** Solve the 3 coupled equations for (vr,vt,vp).
c
c ****** First do (VRR,VTR,VPI).
c
          solve_type=ST_VMATRTP
c
          call loadvbc (m,1,1,2)
c
          call putvrtp (srr(1,1,k),str(1,1,k),spi(1,1,k),rhs)
          call vmatrtp (rhs,m,1,iopt)
          call putvrtp (vgrr(1,1,k),vgtr(1,1,k),vgpi(1,1,k),xcg)
          call solve (xcg,rhs,ierr)
          if (ierr.ne.0) then
            errmsg=eqmsg(1:le)//' (VRR,VTR,VPI) equations.'
            go to 1000
          end if
          call setvrtp (m,1,xcg,vorr(1,1,k),votr(1,1,k),vopi(1,1,k))
c
          if (idebug.gt.0)
     &      write (9,600) '### VRR,VTR,VPI:',m,ncg,bnrm,rnrm
c
c ****** Next do (VRI,VTI,VPR).
c
          solve_type=ST_VMATRTP
c
          call loadvbc (m,2,2,1)
c
          call putvrtp (sri(1,1,k),sti(1,1,k),spr(1,1,k),rhs)
          call vmatrtp (rhs,m,2,iopt)
          call putvrtp (vgri(1,1,k),vgti(1,1,k),vgpr(1,1,k),xcg)
          call solve (xcg,rhs,ierr)
          if (ierr.ne.0) then
            errmsg=eqmsg(1:le)//' (VRI,VTI,VPR) equations.'
            go to 1000
          end if
          call setvrtp (m,2,xcg,vori(1,1,k),voti(1,1,k),vopr(1,1,k))
c
          if (idebug.gt.0)
     &      write (9,600) '### VRI,VTI,VPR:',m,ncg,bnrm,rnrm
c
        end if
c
  700 continue
c
c ****** De-allocate memory for the matrix diagonals.
c
      deallocate (arr00)
      deallocate (arrp0)
      deallocate (arr0p)
      deallocate (att00)
      deallocate (attp0)
      deallocate (att0p)
      deallocate (app00)
      deallocate (appp0)
      deallocate (app0p)
      deallocate (art00)
      deallocate (artp0)
      deallocate (art0m)
      deallocate (artpm)
      deallocate (arp00)
      deallocate (arpp0)
      deallocate (atp00)
      deallocate (atp0p)
c
c ****** De-allocate memory for the preconditioning diagonals.
c
      deallocate (drr)
      deallocate (dtt)
      deallocate (dpp)
c
      return
c
 1000 continue
c
c ****** Error: field solver did not converge.
c
      write (*,*)
      write (*,*) '### ERROR in INVDELSQ:'
      write (*,*) '### The field solver did not converge.'
      write (*,*) 'The error occurred while solving the'
      write (*,*) errmsg
      write (*,*) 'Azimuthal mode M = ',m
      write (*,*) 'NTIME = ',ntime
      write (*,*) 'TIME = ',time
      write (9,*)
      write (9,*) '### ERROR in INVDELSQ:'
      write (9,*) '### The field solver did not converge.'
      write (9,*) 'The error occurred while solving the'
      write (9,*) errmsg
      write (9,*) 'Azimuthal mode M = ',m
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
      call abortrun
c
      return
      end
c#######################################################################
      subroutine getvdgq (qr,qi,vvr,vvt,vvp,qre,vdgqr,vdgqi,vdgqre)
c
c-----------------------------------------------------------------------
c
c ****** Call the selected routine to get v-dot-grad-q.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (upwindphi.lt.0.) then
c
c ****** Use centered differencing in phi (spectral).
c
        call vdgq_cphi (tr,ti,vvr,vvt,vvp,qre,vdgqr,vdgqi,vdgqre)
      else
c
c ****** Use upwind differencing in phi.
c
c
        call vdgq_uphi (tr,ti,vvr,vvt,vvp,qre,vdgqr,vdgqi,vdgqre)
      end if
c
      return
      end
c#######################################################################
      subroutine vdgq_cphi (qr,qi,vvr,vvt,vvp,qre,vdgqr,vdgqi,vdgqre)
c
c-----------------------------------------------------------------------
c
c ****** Compute v-dot-grad-q.
c
c ****** This version uses centered dervatives in phi
c ****** (i.e., spectral phi derivatives).
c
c-----------------------------------------------------------------------
c
c ****** Note that the "v" is at time n+1/2, while the scalar in
c ****** "grad-q" is the passed argument.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real qr(nr,nt,npd3),qi(nr,nt,npd3)
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
      real qre(nr,nt,np)
      real vdgqr(nr,nt,npd3),vdgqi(nr,nt,npd3)
      real vdgqre(nr,nt,np)

c
c-----------------------------------------------------------------------

      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            vdgqr(i,j,k)=-fm(k)*qr(i,j,k)/(rh(i)*sth(j))
            vdgqi(i,j,k)= fm(k)*qi(i,j,k)/(rh(i)*sth(j))
          enddo
        enddo
      enddo
c
c
c ****** get 1/sintheta dq/dphi  in real space 
c
      call fft (vdgqre,vdgqr,vdgqi,0)

      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            avvr =AVGR (vr ,i  ,j  ,k)
            avvt =AVGT (vt ,i  ,j  ,k)
            cadvr=sign(upwindv,avvr)
            cadvt=sign(upwindv,avvt)
            dqdrp=(qre(i+1,j,k)-qre(i  ,j,k))/dr(i)
            dqdrm=(qre(i  ,j,k)-qre(i-1,j,k))/dr(i-1)
            dqdtp=(qre(i,j+1,k)-qre(i,j  ,k))/(rh(i)*dt(j  ))
            dqdtm=(qre(i,j  ,k)-qre(i,j-1,k))/(rh(i)*dt(j-1))
            vdgqre(i,j,k)= vp(i,j,k)*vdgqre(i,j,k)+ 
     &                   avvr*ADVCT(dqdrp,dqdrm,cadvr)
     &                  +avvt*ADVCT(dqdtp,dqdtm,cadvt)
          enddo
        enddo
      enddo

      call zb3 (vdgqre,np)
      
c
      return
      end
c#######################################################################
      subroutine vdgq_uphi (qr,qi,vvr,vvt,vvp,qre,vdgqr,vdgqi,vdgqre)
c
c-----------------------------------------------------------------------
c
c ****** Compute v-dot-grad-q.
c
c ****** This version uses upwind dervatives in phi (in real space),
c ****** according to the value of UPWINDPHI.
c
c-----------------------------------------------------------------------
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real qr(nr,nt,npd3),qi(nr,nt,npd3)
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
      real qre(nr,nt,np)
      real vdgqr(nr,nt,npd3),vdgqi(nr,nt,npd3)
      real vdgqre(nr,nt,np)
c
c
c-----------------------------------------------------------------------

      do k=1,np
c
c ****** Take care of periodic wrap-around.
c
        if (k.eq.1) then
          km1=np
        else
          km1=k-1
        end if
c
        if (k.eq.np) then
          kp1=1
        else
          kp1=k+1
        end if
        do j=2,ntm1
          do i=2,nrm1
            avvr =AVGR (vr ,i  ,j  ,k)
            avvt =AVGT (vt ,i  ,j  ,k)
            avvp =AVG  (vp ,i  ,j  ,k)
            cadvr=sign(upwindv,avvr)
            cadvt=sign(upwindv,avvt)
            cadvp=sign(upwindphi,avvp)
            dqdrp=(qre(i+1,j,k)-qre(i  ,j,k))/dr(i)
            dqdrm=(qre(i  ,j,k)-qre(i-1,j,k))/dr(i-1)
            dqdtp=(qre(i,j+1,k)-qre(i,j  ,k))/(rh(i)*dt(j  ))
            dqdtm=(qre(i,j  ,k)-qre(i,j-1,k))/(rh(i)*dt(j-1))
            dqdpp=(qre(i,j,kp1)-qre(i,j,k  ))/(dphi*rh(i)*sth(j))
            dqdpm=(qre(i,j,k  )-qre(i,j,km1))/(dphi*rh(i)*sth(j))
            vdgqre(i,j,k)= avvr*ADVCT(dqdrp,dqdrm,cadvr)
     &                 +avvt*ADVCT(dqdtp,dqdtm,cadvt)
     &                 +avvp*ADVCT(dqdpp,dqdpm,cadvp)
          enddo
        enddo
c
      enddo
c
      call zb3 (vdgqre,np)
c
c
      return
      end
c#######################################################################
      subroutine vdgv (vvr,vvt,vvp,
     &                 vcrr,vcri,vctr,vcti,vcpr,vcpi,
     &                 vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
c-----------------------------------------------------------------------
c
c ****** Call the selected routine to get v-dot-grad-v.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (upwindphi.lt.0.) then
c
c ****** Use centered differencing in phi (spectral).
c
        call vdgv_cphi (vvr,vvt,vvp,
     &                  vcrr,vcri,vctr,vcti,vcpr,vcpi,
     &                  vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
      else
c
c ****** Use upwind differencing in phi.
c
        call vdgv_uphi (vvr,vvt,vvp,
     &                  vcrr,vcri,vctr,vcti,vcpr,vcpi,
     &                  vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
      end if
c
      return
      end
c#######################################################################
      subroutine vdgv_cphi (vvr,vvt,vvp,
     &                      vcrr,vcri,vctr,vcti,vcpr,vcpi,
     &                      vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
c-----------------------------------------------------------------------
c
c ****** Compute v-dot-grad-v.
c
c ****** This version uses centered dervatives in phi
c ****** (i.e., spectral phi derivatives).
c
c-----------------------------------------------------------------------
c
c ****** Note that the "v" is at time n+1/2, while the one in
c ****** "grad-v" is the passed argument.
c
c ****** On input, VV# contains the real velocity and
c ****** VC## contains the complex velocity.
c ****** It is this v that is used in the "grad-v".
c ****** The v at time n+1/2 is obtained from the main
c ****** real and complex v arrays.
c
c ****** Upon return, VDGV## contains complex v-dot-grad-v.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
c
      real vcrr(nr,nt,npd3),vcri(nr,nt,npd3),
     &     vctr(nr,nt,npd3),vcti(nr,nt,npd3),
     &     vcpr(nr,nt,npd3),vcpi(nr,nt,npd3)
c
      real vdgvrr(nr,nt,npd3),vdgvri(nr,nt,npd3),
     &     vdgvtr(nr,nt,npd3),vdgvti(nr,nt,npd3),
     &     vdgvpr(nr,nt,npd3),vdgvpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real vdgvr(nr,nt,np),vdgvt(nr,nt,np),vdgvp(nr,nt,np)
      real dpcr(nr,nt,npd3),dpci(nr,nt,npd3)
      real dpre(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** If IFVDGV=0, set v-dot-grad-v to zero.
c
      if (ifvdgv.eq.0) then
        call bcast (vdgvrr,0.,nc)
        call bcast (vdgvri,0.,nc)
        call bcast (vdgvtr,0.,nc)
        call bcast (vdgvti,0.,nc)
        call bcast (vdgvpr,0.,nc)
        call bcast (vdgvpi,0.,nc)
        return
      end if
c
c ****** First, put the real-space contribution into VDGV#.
c
      do 400 k=1,np
c
c ****** R component.
c
        do 150 j=2,ntm1
        do 140 i=2,nrm2
          avvr =AVG  (vr ,i  ,j  ,k)
          avvt =AVGRT(vt ,i+1,j  ,k)
          avvp =AVGR (vp ,i+1,j  ,k)
          avvvt=AVGRT(vvt,i+1,j  ,k)
          avvvp=AVGR (vvp,i+1,j  ,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          dvrdrp=(vvr(i+1,j,k)-vvr(i  ,j,k))/drh(i+1)
          dvrdrm=(vvr(i  ,j,k)-vvr(i-1,j,k))/drh(i  )
          dvrdtp=(vvr(i,j+1,k)-vvr(i,j  ,k))/(r(i)*dt(j  ))
          dvrdtm=(vvr(i,j  ,k)-vvr(i,j-1,k))/(r(i)*dt(j-1))
          vdgvr(i,j,k)= avvr*ADVCT(dvrdrp,dvrdrm,cadvr)
     &                 +avvt*ADVCT(dvrdtp,dvrdtm,cadvt)
     &                 -(avvt*avvvt+avvp*avvvp)/r(i)
  140   continue
  150   continue
c
c ****** Theta component.
c
        do 250 j=2,ntm2
        do 240 i=2,nrm1
          avvr =AVGRT(vr ,i  ,j+1,k)
          avvt =AVG  (vt ,i  ,j  ,k)
          avvp =AVGT (vp ,i  ,j+1,k)
          avvvr=AVGRT(vvr,i  ,j+1,k)
          avvvp=AVGT (vvp,i  ,j+1,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          dvtdrp=(vvt(i+1,j,k)-vvt(i  ,j,k))/dr(i  )
          dvtdrm=(vvt(i  ,j,k)-vvt(i-1,j,k))/dr(i-1)
          dvtdtp=(vvt(i,j+1,k)-vvt(i,j  ,k))/(rh(i)*dth(j+1))
          dvtdtm=(vvt(i,j  ,k)-vvt(i,j-1,k))/(rh(i)*dth(j  ))
          vdgvt(i,j,k)= avvr*ADVCT(dvtdrp,dvtdrm,cadvr)
     &                 +avvt*ADVCT(dvtdtp,dvtdtm,cadvt)
     &                 +(avvt*avvvr-ct(j)*avvp*avvvp/st(j))/rh(i)
  240   continue
  250   continue
c
c ****** Phi component.
c
        do 350 j=2,ntm1
        do 340 i=2,nrm1
          avvr =AVGR (vr ,i  ,j  ,k)
          avvt =AVGT (vt ,i  ,j  ,k)
          avvp =AVG  (vp ,i  ,j  ,k)
          avvvr=AVGR (vvr,i  ,j  ,k)
          avvvt=AVGT (vvt,i  ,j  ,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          dvpdrp=(vvp(i+1,j,k)-vvp(i  ,j,k))/dr(i  )
          dvpdrm=(vvp(i  ,j,k)-vvp(i-1,j,k))/dr(i-1)
          dvpdtp=(vvp(i,j+1,k)-vvp(i,j  ,k))/(rh(i)*dt(j  ))
          dvpdtm=(vvp(i,j  ,k)-vvp(i,j-1,k))/(rh(i)*dt(j-1))
          vdgvp(i,j,k)= avvr*ADVCT(dvpdrp,dvpdrm,cadvr)
     &                 +avvt*ADVCT(dvpdtp,dvpdtm,cadvt)
     &                 +avvp*(avvvr+cth(j)*avvvt/sth(j))/rh(i)
  340   continue
  350   continue
c
  400 continue
c
c ****** Next, add the phi derivatives.
c
c ****** R component.
c
      do 520 k=1,npd3
      do 510 j=2,ntm1
      do 500 i=2,nrm2
        dpcr(i,j,k)=-fm(k)*vcri(i,j,k)/(r(i)*sth(j))
        dpci(i,j,k)= fm(k)*vcrr(i,j,k)/(r(i)*sth(j))
  500 continue
  510 continue
  520 continue
c
      call zb1 (dpcr,npd3)
      call zb1 (dpci,npd3)
c
      call fft (dpre,dpcr,dpci,0)
c
      do 550 k=1,np
      do 540 j=2,ntm1
      do 530 i=2,nrm2
        avvp=AVGR (vp,i+1,j  ,k)
        vdgvr(i,j,k)=vdgvr(i,j,k)+avvp*dpre(i,j,k)
  530 continue
  540 continue
  550 continue
c
c ****** Theta component.
c
      do 620 k=1,npd3
      do 610 j=2,ntm2
      do 600 i=2,nrm1
        dpcr(i,j,k)=-fm(k)*vcti(i,j,k)/(rh(i)*st(j))
        dpci(i,j,k)= fm(k)*vctr(i,j,k)/(rh(i)*st(j))
  600 continue
  610 continue
  620 continue
c
      call zb2 (dpcr,npd3)
      call zb2 (dpci,npd3)
c
      call fft (dpre,dpcr,dpci,0)
c
      do 650 k=1,np
      do 640 j=2,ntm2
      do 630 i=2,nrm1
        avvp=AVGT (vp,i  ,j+1,k)
        vdgvt(i,j,k)=vdgvt(i,j,k)+avvp*dpre(i,j,k)
  630 continue
  640 continue
  650 continue
c
c ****** Phi component.
c
      do 720 k=1,npd3
      do 710 j=2,ntm1
      do 700 i=2,nrm1
        dpcr(i,j,k)=-fm(k)*vcpi(i,j,k)/(rh(i)*sth(j))
        dpci(i,j,k)= fm(k)*vcpr(i,j,k)/(rh(i)*sth(j))
  700 continue
  710 continue
  720 continue
c
      call zb3 (dpcr,npd3)
      call zb3 (dpci,npd3)
c
      call fft (dpre,dpcr,dpci,0)
c
      do 750 k=1,np
      do 740 j=2,ntm1
      do 730 i=2,nrm1
        avvp=AVG  (vp,i  ,j  ,k)
        vdgvp(i,j,k)=vdgvp(i,j,k)+avvp*dpre(i,j,k)
  730 continue
  740 continue
  750 continue
c
      call zb1 (vdgvr,np)
      call zb2 (vdgvt,np)
      call zb3 (vdgvp,np)
c
c ****** Obtain the complex transform in VDGV##.
c
      call fft (vdgvr,vdgvrr,vdgvri,1)
      call fft (vdgvt,vdgvtr,vdgvti,1)
      call fft (vdgvp,vdgvpr,vdgvpi,1)
c
      return
      end
c#######################################################################
      subroutine vdgv_uphi (vvr,vvt,vvp,
     &                      vcrr,vcri,vctr,vcti,vcpr,vcpi,
     &                      vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
c-----------------------------------------------------------------------
c
c ****** Compute v-dot-grad-v.
c
c ****** This version uses upwind dervatives in phi (in real space),
c ****** according to the value of UPWINDPHI.
c
c-----------------------------------------------------------------------
c
c ****** Note that the "v" is at time n+1/2, while the one in
c ****** "grad-v" is the passed argument.
c
c ****** On input, VV# contains the real velocity and
c ****** VC## contains the complex velocity.
c ****** It is this v that is used in the "grad-v".
c ****** The v at time n+1/2 is obtained from the main
c ****** real and complex v arrays.
c
c ****** Upon return, VDGV## contains complex v-dot-grad-v.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vvr(nr,nt,np),vvt(nr,nt,np),vvp(nr,nt,np)
c
      real vcrr(nr,nt,npd3),vcri(nr,nt,npd3),
     &     vctr(nr,nt,npd3),vcti(nr,nt,npd3),
     &     vcpr(nr,nt,npd3),vcpi(nr,nt,npd3)
c
      real vdgvrr(nr,nt,npd3),vdgvri(nr,nt,npd3),
     &     vdgvtr(nr,nt,npd3),vdgvti(nr,nt,npd3),
     &     vdgvpr(nr,nt,npd3),vdgvpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** Scratch arrays.
c
      real vdgvr(nr,nt,np),vdgvt(nr,nt,np),vdgvp(nr,nt,np)
      real dpcr(nr,nt,npd3),dpci(nr,nt,npd3)
      real dpre(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** If IFVDGV=0, set v-dot-grad-v to zero.
c
      if (ifvdgv.eq.0) then
        call bcast (vdgvrr,0.,nc)
        call bcast (vdgvri,0.,nc)
        call bcast (vdgvtr,0.,nc)
        call bcast (vdgvti,0.,nc)
        call bcast (vdgvpr,0.,nc)
        call bcast (vdgvpi,0.,nc)
        return
      end if
c
c ****** Put v-dot-grad-v into VDGV#.
c
      do k=1,np
c
c ****** Take care of periodic wrap-around.
c
        if (k.eq.1) then
          km1=np
        else
          km1=k-1
        end if
c
        if (k.eq.np) then
          kp1=1
        else
          kp1=k+1
        end if
c
c ****** R component.
c
        do j=2,ntm1
        do i=2,nrm2
          avvr =AVG  (vr ,i  ,j  ,k)
          avvt =AVGRT(vt ,i+1,j  ,k)
          avvp =AVGR (vp ,i+1,j  ,k)
          avvvt=AVGRT(vvt,i+1,j  ,k)
          avvvp=AVGR (vvp,i+1,j  ,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          cadvp=sign(upwindphi,avvp)
          dvrdrp=(vvr(i+1,j,k)-vvr(i  ,j,k))/drh(i+1)
          dvrdrm=(vvr(i  ,j,k)-vvr(i-1,j,k))/drh(i  )
          dvrdtp=(vvr(i,j+1,k)-vvr(i,j  ,k))/(r(i)*dt(j  ))
          dvrdtm=(vvr(i,j  ,k)-vvr(i,j-1,k))/(r(i)*dt(j-1))
          dvrdpp=(vvr(i,j,kp1)-vvr(i,j,k  ))/(dphi*r(i)*sth(j))
          dvrdpm=(vvr(i,j,k  )-vvr(i,j,km1))/(dphi*r(i)*sth(j))
          vdgvr(i,j,k)= avvr*ADVCT(dvrdrp,dvrdrm,cadvr)
     &                 +avvt*ADVCT(dvrdtp,dvrdtm,cadvt)
     &                 +avvp*ADVCT(dvrdpp,dvrdpm,cadvp)
     &                 -(avvt*avvvt+avvp*avvvp)/r(i)
        enddo
        enddo
c
c ****** Theta component.
c
        do j=2,ntm2
        do i=2,nrm1
          avvr =AVGRT(vr ,i  ,j+1,k)
          avvt =AVG  (vt ,i  ,j  ,k)
          avvp =AVGT (vp ,i  ,j+1,k)
          avvvr=AVGRT(vvr,i  ,j+1,k)
          avvvp=AVGT (vvp,i  ,j+1,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          cadvp=sign(upwindphi,avvp)
          dvtdrp=(vvt(i+1,j,k)-vvt(i  ,j,k))/dr(i  )
          dvtdrm=(vvt(i  ,j,k)-vvt(i-1,j,k))/dr(i-1)
          dvtdtp=(vvt(i,j+1,k)-vvt(i,j  ,k))/(rh(i)*dth(j+1))
          dvtdtm=(vvt(i,j  ,k)-vvt(i,j-1,k))/(rh(i)*dth(j  ))
          dvtdpp=(vvt(i,j,kp1)-vvt(i,j,k  ))/(dphi*rh(i)*st(j))
          dvtdpm=(vvt(i,j,k  )-vvt(i,j,km1))/(dphi*rh(i)*st(j))
          vdgvt(i,j,k)= avvr*ADVCT(dvtdrp,dvtdrm,cadvr)
     &                 +avvt*ADVCT(dvtdtp,dvtdtm,cadvt)
     &                 +avvp*ADVCT(dvtdpp,dvtdpm,cadvp)
     &                 +(avvt*avvvr-ct(j)*avvp*avvvp/st(j))/rh(i)
        enddo
        enddo
c
c ****** Phi component.
c
        do j=2,ntm1
        do i=2,nrm1
          avvr =AVGR (vr ,i  ,j  ,k)
          avvt =AVGT (vt ,i  ,j  ,k)
          avvp =AVG  (vp ,i  ,j  ,k)
          avvvr=AVGR (vvr,i  ,j  ,k)
          avvvt=AVGT (vvt,i  ,j  ,k)
          cadvr=sign(upwindv,avvr)
          cadvt=sign(upwindv,avvt)
          cadvp=sign(upwindphi,avvp)
          dvpdrp=(vvp(i+1,j,k)-vvp(i  ,j,k))/dr(i  )
          dvpdrm=(vvp(i  ,j,k)-vvp(i-1,j,k))/dr(i-1)
          dvpdtp=(vvp(i,j+1,k)-vvp(i,j  ,k))/(rh(i)*dt(j  ))
          dvpdtm=(vvp(i,j  ,k)-vvp(i,j-1,k))/(rh(i)*dt(j-1))
          dvpdpp=(vvp(i,j,kp1)-vvp(i,j,k  ))/(dphi*rh(i)*sth(j))
          dvpdpm=(vvp(i,j,k  )-vvp(i,j,km1))/(dphi*rh(i)*sth(j))
          vdgvp(i,j,k)= avvr*ADVCT(dvpdrp,dvpdrm,cadvr)
     &                 +avvt*ADVCT(dvpdtp,dvpdtm,cadvt)
     &                 +avvp*ADVCT(dvpdpp,dvpdpm,cadvp)
     &                 +avvp*(avvvr+cth(j)*avvvt/sth(j))/rh(i)
        enddo
        enddo
c
      enddo
c
      call zb1 (vdgvr,np)
      call zb2 (vdgvt,np)
      call zb3 (vdgvp,np)
c
c ****** Obtain the complex transform in VDGV##.
c
      call fft (vdgvr,vdgvrr,vdgvri,1)
      call fft (vdgvt,vdgvtr,vdgvti,1)
      call fft (vdgvp,vdgvpr,vdgvpi,1)
c
      return
      end
c#######################################################################
      subroutine vguess (vprr,vpri,vptr,vpti,vppr,vppi,
     &                   vdgvrr,vdgvri,vdgvtr,vdgvti,vdgvpr,vdgvpi)
c
c-----------------------------------------------------------------------
c
c ****** Predict the velocity by advancing with v-dot-grad-v.
c
c-----------------------------------------------------------------------
c
c ****** On input, VDGV## has complex v-dot-grad-v.
c
c ****** On return, VP## has the predicted (complex) velocity.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vprr(nr,nt,npd3),vpri(nr,nt,npd3),
     &     vptr(nr,nt,npd3),vpti(nr,nt,npd3),
     &     vppr(nr,nt,npd3),vppi(nr,nt,npd3)
c
      real vdgvrr(nr,nt,npd3),vdgvri(nr,nt,npd3),
     &     vdgvtr(nr,nt,npd3),vdgvti(nr,nt,npd3),
     &     vdgvpr(nr,nt,npd3),vdgvpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
c ****** If IFVDGV=0, v-dot-grad-v is assumed to be zero, so that
c ****** the guess is just the velocity at the previous time step.
c
      if (ifvdgv.eq.0) then
        call scopy (nc,vrr,1,vprr,1)
        call scopy (nc,vri,1,vpri,1)
        call scopy (nc,vtr,1,vptr,1)
        call scopy (nc,vti,1,vpti,1)
        call scopy (nc,vpr,1,vppr,1)
        call scopy (nc,vpi,1,vppi,1)
        return
      end if
c
c ****** Get the complex predicted v.
c
      do k=1,npd3
c
        do j=2,ntm1
          do i=2,nrm2
            vprr(i,j,k)=vrr(i,j,k)-dtime*vdgvrr(i,j,k)
            vpri(i,j,k)=vri(i,j,k)-dtime*vdgvri(i,j,k)
          enddo
        enddo
c
        do j=2,ntm2
          do i=2,nrm1
            vptr(i,j,k)=vtr(i,j,k)-dtime*vdgvtr(i,j,k)
            vpti(i,j,k)=vti(i,j,k)-dtime*vdgvti(i,j,k)
          enddo
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            vppr(i,j,k)=vpr(i,j,k)-dtime*vdgvpr(i,j,k)
            vppi(i,j,k)=vpi(i,j,k)-dtime*vdgvpi(i,j,k)
          enddo
        enddo
c
      enddo
c
      call zb1 (vprr,npd3)
      call zb1 (vpri,npd3)
      call zb2 (vptr,npd3)
      call zb2 (vpti,npd3)
      call zb3 (vppr,npd3)
      call zb3 (vppi,npd3)
c
      return
      end
c#######################################################################
      subroutine setbcv (vprr,vpri,vptr,vpti,vppr,vppi)
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary conditions on v.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vprr(nr,nt,npd3),vpri(nr,nt,npd3),
     &     vptr(nr,nt,npd3),vpti(nr,nt,npd3),
     &     vppr(nr,nt,npd3),vppi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          vprr(   1,j,k)=vrbcr0(j,k,1)
          vpri(   1,j,k)=vrbcr0(j,k,2)
        enddo
        do j=2,ntm2
          vptr(   1,j,k)=2.*vtbcr0(j,k,1)-vptr(   2,j,k)
          vpti(   1,j,k)=2.*vtbcr0(j,k,2)-vpti(   2,j,k)
        enddo
        do j=2,ntm1
          vppr(   1,j,k)=2.*vpbcr0(j,k,1)-vppr(   2,j,k)
          vppi(   1,j,k)=2.*vpbcr0(j,k,2)-vppi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        if (bcr1type.eq.'open') then
          do j=2,ntm1
            vprr(nrm1,j,k)=vrbcr1(j,k,1)
            vpri(nrm1,j,k)=vrbcr1(j,k,2)
          enddo
          do j=2,ntm2
            vptr(  nr,j,k)=vtbcr1(j,k,1)
            vpti(  nr,j,k)=vtbcr1(j,k,2)
          enddo
          do j=2,ntm1
            vppr(  nr,j,k)=vpbcr1(j,k,1)
            vppi(  nr,j,k)=vpbcr1(j,k,2)
          enddo
        else if (bcr1type.eq.'fixed') then
          do j=2,ntm1
            vprr(nrm1,j,k)=vrbcr1(j,k,1)
            vpri(nrm1,j,k)=vrbcr1(j,k,2)
          enddo
          do j=2,ntm2
            vptr(  nr,j,k)=2.*vtbcr1(j,k,1)-vptr(nrm1,j,k)
            vpti(  nr,j,k)=2.*vtbcr1(j,k,2)-vpti(nrm1,j,k)
          enddo
          do j=2,ntm1
            vppr(  nr,j,k)=2.*vpbcr1(j,k,1)-vppr(nrm1,j,k)
            vppi(  nr,j,k)=2.*vpbcr1(j,k,2)-vppi(nrm1,j,k)
          enddo
        else if (bcr1type.eq.'1dchar') then
          do j=2,ntm1
            vprr(nrm1,j,k)=vrbcr1(j,k,1)
            vpri(nrm1,j,k)=vrbcr1(j,k,2)
          enddo
          do j=2,ntm2
            vptr(  nr,j,k)=vtbcr1(j,k,1)
            vpti(  nr,j,k)=vtbcr1(j,k,2)
          enddo
          do j=2,ntm1
            vppr(  nr,j,k)=vpbcr1(j,k,1)
            vppi(  nr,j,k)=vpbcr1(j,k,2)
          enddo
        end if
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nrm1
            vprr(i,   1,k)=vprr(i,   2,k)
            vpri(i,   1,k)=vpri(i,   2,k)
            vprr(i,  nt,k)=vprr(i,ntm1,k)
            vpri(i,  nt,k)=vpri(i,ntm1,k)
          enddo
          do i=1,nr
            vptr(i,   1,k)=0.
            vpti(i,   1,k)=0.
            vptr(i,ntm1,k)=0.
            vpti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            vppr(i,   1,k)=-vppr(i,   2,k)
            vppi(i,   1,k)=-vppi(i,   2,k)
            vppr(i,  nt,k)=-vppr(i,ntm1,k)
            vppi(i,  nt,k)=-vppi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nrm1
            vprr(i,   1,k)=-vprr(i,   2,k)
            vpri(i,   1,k)=-vpri(i,   2,k)
            vprr(i,  nt,k)=-vprr(i,ntm1,k)
            vpri(i,  nt,k)=-vpri(i,ntm1,k)
          enddo
          do i=1,nr
            vptr(i,   1,k)= vppi(i,   2,k)
            vpti(i,   1,k)=-vppr(i,   2,k)
            vptr(i,ntm1,k)=-vppi(i,ntm1,k)
            vpti(i,ntm1,k)= vppr(i,ntm1,k)
          enddo
          do i=1,nr
            vppr(i,   1,k)=vppr(i,   2,k)
            vppi(i,   1,k)=vppi(i,   2,k)
            vppr(i,  nt,k)=vppr(i,ntm1,k)
            vppi(i,  nt,k)=vppi(i,ntm1,k)
          enddo
        else if (m.eq.2) then
          do i=1,nrm1
            vprr(i,   1,k)=vprr(i,   2,k)
            vpri(i,   1,k)=vpri(i,   2,k)
            vprr(i,  nt,k)=vprr(i,ntm1,k)
            vpri(i,  nt,k)=vpri(i,ntm1,k)
          enddo
          do i=1,nr
            vptr(i,   1,k)=0.
            vpti(i,   1,k)=0.
            vptr(i,ntm1,k)=0.
            vpti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            vppr(i,   1,k)=-vppr(i,   2,k)
            vppi(i,   1,k)=-vppi(i,   2,k)
            vppr(i,  nt,k)=-vppr(i,ntm1,k)
            vppi(i,  nt,k)=-vppi(i,ntm1,k)
          enddo
        else
          do i=1,nrm1
            vprr(i,   1,k)=vprr(i,   2,k)
            vpri(i,   1,k)=vpri(i,   2,k)
            vprr(i,  nt,k)=vprr(i,ntm1,k)
            vpri(i,  nt,k)=vpri(i,ntm1,k)
          enddo
          do i=1,nr
            vptr(i,   1,k)=0.
            vpti(i,   1,k)=0.
            vptr(i,ntm1,k)=0.
            vpti(i,ntm1,k)=0.
          enddo
          do i=1,nr
            vppr(i,   1,k)=vppr(i,   2,k)
            vppi(i,   1,k)=vppi(i,   2,k)
            vppr(i,  nt,k)=vppr(i,ntm1,k)
            vppi(i,  nt,k)=vppi(i,ntm1,k)
          enddo
        end if
c
c ****** Zero out the dummy points.
c
        do j=1,nt
          vprr(nr,j,k)=0.
          vpri(nr,j,k)=0.
        enddo
c
        do i=1,nr
          vptr(i,nt,k)=0.
          vpti(i,nt,k)=0.
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine loadvbc (m,irir,irit,irip)
c
c-----------------------------------------------------------------------
c
c ****** Load the boundary condition arrays for the velocity
c ****** for azimuthal mode M.
c
c-----------------------------------------------------------------------
c
c ****** For variables which straddle a boundary, the boundary
c ****** conditions are of the form:
c
c           V(b) + VC##(*) * V(bn) = VV##(*)
c
c ****** where V(b) is the boundary point, and V(bn) is its
c ****** normal internal neighbor.
c
c ****** For variables which fall on a boundary, the boundary
c ****** conditions are of the form:
c
c           V(b) = VV##(*)
c
c ****** IRIR should be set to 1 if the boundary
c ****** conditions are for the real part of vr, and to 2
c ****** for the imaginary part.  Similarly, IRIT and IRIP
c ****** control the vt and vp components.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
      k=m+1
c
c ****** Boundary at r = R0.
c
      do j=2,ntm1
        vrvr0(j)=vrbcr0(j,k,irir)
      enddo
      do j=2,ntm2
        vtcr0(j)=1.
        vtvr0(j)=2.*vtbcr0(j,k,irit)
      enddo
      do j=2,ntm1
        vpcr0(j)=1.
        vpvr0(j)=2.*vpbcr0(j,k,irip)
      enddo
c
c ****** Boundary at r = R1.
c
      if (bcr1type.eq.'open') then
        do j=2,ntm1
          vrvr1(j)=vrbcr1(j,k,irir)
        enddo
        do j=2,ntm2
          vtcr1(j)=0.
          vtvr1(j)=vtbcr1(j,k,irit)
        enddo
        do j=2,ntm1
          vpcr1(j)=0.
          vpvr1(j)=vpbcr1(j,k,irip)
        enddo
      else if (bcr1type.eq.'fixed') then
        do j=2,ntm1
          vrvr1(j)=vrbcr1(j,k,irir)
        enddo
        do j=2,ntm2
          vtcr1(j)=1.
          vtvr1(j)=2.*vtbcr1(j,k,irit)
        enddo
        do j=2,ntm1
          vpcr1(j)=1.
          vpvr1(j)=2.*vpbcr1(j,k,irip)
        enddo
      else if (bcr1type.eq.'1dchar') then
        do j=2,ntm1
          vrvr1(j)=vrbcr1(j,k,irir)
        enddo
        do j=2,ntm2
          vtcr1(j)=0.
          vtvr1(j)=vtbcr1(j,k,irit)
        enddo
        do j=2,ntm1
          vpcr1(j)=0.
          vpvr1(j)=vpbcr1(j,k,irip)
        enddo
      end if
c
c ****** Boundaries at theta = 0, pi.
c
      if (m.eq.0) then
        do i=1,nrm1
          vrct0(i)=-1.
          vrvt0(i)=0.
          vrct1(i)=-1.
          vrvt1(i)=0.
        enddo
      else if (m.eq.1) then
        do i=1,nrm1
          vrct0(i)=1.
          vrvt0(i)=0.
          vrct1(i)=1.
          vrvt1(i)=0.
        enddo
      else
        do i=1,nrm1
          vrct0(i)=-1.
          vrvt0(i)=0.
          vrct1(i)=-1.
          vrvt1(i)=0.
        enddo
      end if
c
      if (m.eq.0) then
        do i=1,nr
          vtvt0(i)=0.
          vtvt1(i)=0.
        enddo
      else if (m.eq.1) then
c
c ****** Not used!  (The m=1 BC couples vt and vp and is
c ****** implemented in the matrix-loading code directly.)
c
      else
        do i=1,nr
          vtvt0(i)=0.
          vtvt1(i)=0.
        enddo
      end if
c
      if (m.eq.0) then
        do i=1,nr
          vpct0(i)=1.
          vpvt0(i)=0.
          vpct1(i)=1.
          vpvt1(i)=0.
        enddo
      else if (m.eq.1) then
        do i=1,nr
          vpct0(i)=-1.
          vpvt0(i)=0.
          vpct1(i)=-1.
          vpvt1(i)=0.
        enddo
      else if (m.eq.2) then
        do i=1,nr
          vpct0(i)=1.
          vpvt0(i)=0.
          vpct1(i)=1.
          vpvt1(i)=0.
        enddo
      else
        do i=1,nr
          vpct0(i)=-1.
          vpvt0(i)=0.
          vpct1(i)=-1.
          vpvt1(i)=0.
        enddo
      end if
c
      return
      end
c#######################################################################
      subroutine vmatrt0 (rhs,iopt)
c
c-----------------------------------------------------------------------
c
c ****** Load the matrix for the m=0 mode for the coupled
c ****** (vr,vt) equations.
c
c-----------------------------------------------------------------------
c
c ****** Variable IOPT controls which matrix is loaded.
c ****** When IOPT=0, coeff=0. 
c ****** When IOPT=1, load the matrix for the semi-implicit term.
c ****** When IOPT=2, load the matrix for the viscous term.
c ****** When IOPT=3, load the matrix for both terms.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Right hand side.
c
      real rhs(nr,nt,2)
c
c-----------------------------------------------------------------------
c
c ****** Temporary array.
c
      real coef(nrm1,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
c ****** Initialize matrix diagonals to zero.
c
      arr00(:,:)=0.
      arrp0(:,:)=0.
      arr0p(:,:)=0.
      att00(:,:)=0.
      attp0(:,:)=0.
      att0p(:,:)=0.
      art00(:,:)=0.
      artp0(:,:)=0.
      art0m(:,:)=0.
      artpm(:,:)=0.
c
c ****** Load the coefficient of the div-grad operator.
c
      if (iopt.eq.1) then
        isim=1
        ivsm=0
      else if (iopt.eq.2) then
        isim=0
        ivsm=1
      else if (iopt.eq.3) then
        isim=1
        ivsm=1
      else
        isim=0
        ivsm=0
      end if
c
      do j=1,ntm1
        do i=1,nrm1
          coef(i,j)=dtime*( isim*sifac(i,j)
     &                     +ivsm*(visfac(i,j)+viscel(i,j)))
        enddo
      enddo
c
c ****** Load the internal points for the vr equation.
c
      do j=2,ntm1
      do i=2,nrm2
        s00=AVGR (rhr,i+1,j  ,1)*AVG2T (coef,i  ,j  )
        sp0=AVG  (rhr,i+1,j  ,1)*AVG2RT(coef,i+1,j  )
        sm0=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        s0p=AVGRT(rhr,i+1,j+1,1)*AVG2  (coef,i  ,j  )
        s0m=AVGRT(rhr,i+1,j  ,1)*AVG2  (coef,i  ,j-1)
        spp=AVGT (rhr,i+1,j+1,1)*AVG2R (coef,i+1,j  )
        smp=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        spm=AVGT (rhr,i+1,j  ,1)*AVG2R (coef,i+1,j-1)
        smm=AVGT (rhr,i  ,j  ,1)*AVG2R (coef,i  ,j-1)
        r00=AVGR (rhr,i+1,j  ,1)
        dvr=r(i)**2*dr(i)*dth(j)*sth(j)
        arr00(i,j)=r00*dvr+
     &   ( sth(j)*dth(j)*( sp0*rh(i+1)**2/drh(i+1)
     &                    +sm0*rh(i  )**2/drh(i  ))
     &    +dr(i)*( s0p*st(j  )/dt(j  )
     &            +s0m*st(j-1)/dt(j-1))
     &    +2.*s00*dr(i)*sth(j)*dth(j))
        arrp0(i,j)=-sp0*sth(j)*dth(j)*rh(i+1)**2/drh(i+1)
        arr0p(i,j)=-s0p*dr(i)*st(j)/dt(j)
        art00(i,j)= .5*(s00+smp)*st(j  )*(r(i)**2/rh(i  )-rh(i  ))
        artp0(i,j)=-.5*(s00+spp)*st(j  )*(r(i)**2/rh(i+1)-rh(i+1))
        art0m(i,j)=-.5*(s00+smm)*st(j-1)*(r(i)**2/rh(i  )-rh(i  ))
        artpm(i,j)= .5*(s00+spm)*st(j-1)*(r(i)**2/rh(i+1)-rh(i+1))
        rhs(i,j,1)=rhs(i,j,1)*dvr
      enddo
      enddo
c
c ****** Load the internal points for the vt equation.
c
      do j=2,ntm2
      do i=2,nrm1
        s00=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        sp0=AVGRT(rhr,i+1,j+1,1)*AVG2  (coef,i  ,j  )
        sm0=AVGRT(rhr,i  ,j+1,1)*AVG2  (coef,i-1,j  )
        s0p=AVG  (rhr,i  ,j+1,1)*AVG2RT(coef,i  ,j+1)
        s0m=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        r00=AVGT (rhr,i  ,j+1,1)
        dvt=rh(i)**2*drh(i)*dt(j)*st(j)
        att00(i,j)=r00*dvt+
     &   ( st(j)*dt(j)*( sp0*r(i  )**2/dr(i  )
     &                  +sm0*r(i-1)**2/dr(i-1))
     &    +drh(i)*( s0p*sth(j+1)/dth(j+1)
     &             +s0m*sth(j  )/dth(j  ))
     &    +s00*drh(i)*dt(j)/st(j))
        attp0(i,j)=-sp0*st(j)*dt(j)*r(i)**2/dr(i)
        att0p(i,j)=-s0p*drh(i)*sth(j+1)/dth(j+1)
        rhs(i,j,2)=rhs(i,j,2)*dvt
      enddo
      enddo
c
c ****** Eliminate the boundary points.
c
c ****** (vr points in vr equation).
c
      do j=2,ntm1
        sp0=AVG  (rhr,  2,j  ,1)*AVG2RT(coef,  2,j  )
        arrm0=-sp0*sth(j)*dth(j)*rh(2)**2/drh(2)
        rhs(   2,j,1)=rhs(   2,j,1)-arrm0        *vrvr0(j)
        rhs(nrm2,j,1)=rhs(nrm2,j,1)-arrp0(nrm2,j)*vrvr1(j)
        arrp0(nrm2,j)=0.
      enddo
      do i=2,nrm2
        s0p=AVGRT(rhr,i+1,  2,1)*AVG2  (coef,i  ,  1)
        arr0m=-s0p*dr(i)*st(1)/dt(1)
        arr00(i,   2)=arr00(i,   2)-arr0m        *vrct0(i)
        rhs(i,   2,1)=rhs(i,   2,1)-arr0m        *vrvt0(i)
        arr00(i,ntm1)=arr00(i,ntm1)-arr0p(i,ntm1)*vrct1(i)
        rhs(i,ntm1,1)=rhs(i,ntm1,1)-arr0p(i,ntm1)*vrvt1(i)
        arr0p(i,ntm1)=0.
      enddo
c
c ****** (vt points in vt equation).
c
      do j=2,ntm2
        sp0=AVGRT(rhr,  2,j+1,1)*AVG2  (coef,  1,j  )
        attm0=-sp0*st(j)*dt(j)*r(1)**2/dr(1)
        att00(   2,j)=att00(   2,j)-attm0        *vtcr0(j)
        rhs(   2,j,2)=rhs(   2,j,2)-attm0        *vtvr0(j)
        att00(nrm1,j)=att00(nrm1,j)-attp0(nrm1,j)*vtcr1(j)
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-attp0(nrm1,j)*vtvr1(j)
        attp0(nrm1,j)=0.
      enddo
      do i=2,nrm1
        s0p=AVG  (rhr,i  ,  2,1)*AVG2RT(coef,i  ,  2)
        att0m=-s0p*drh(i)*sth(2)/dth(2)
        rhs(i,   2,2)=rhs(i,   2,2)-att0m        *vtvt0(i)
        rhs(i,ntm2,2)=rhs(i,ntm2,2)-att0p(i,ntm2)*vtvt0(i)
        att0p(i,ntm2)=0.
      enddo
c
c ****** (vt points in vr equation).
c
      do i=2,nrm2
        rhs(i,   2,1)=rhs(i,   2,1)-art0m(i,   2)*vtvt0(i  )
        art0m(i,   2)=0.
        rhs(i,ntm1,1)=rhs(i,ntm1,1)-art00(i,ntm1)*vtvt1(i  )
        art00(i,ntm1)=0.
        rhs(i,   2,1)=rhs(i,   2,1)-artpm(i,   2)*vtvt0(i+1)
        artpm(i,   2)=0.
        rhs(i,ntm1,1)=rhs(i,ntm1,1)-artp0(i,ntm1)*vtvt1(i+1)
        artp0(i,ntm1)=0.
      enddo
c
c ****** (vr points in vt equation).
c
      do j=2,ntm2
        s00=AVGR (rhr,   2,j  ,1)*AVG2T (coef,   1,j  )
        spp=AVGT (rhr,   2,j+1,1)*AVG2R (coef,   2,j  )
        atrm0=-.5*(s00+spp)*st(j)*(r(   1)**2/rh(   2)-rh(   2))
        rhs(   2,j,2)=rhs(   2,j,2)-atrm0        *vrvr0(j  )
        s00=AVGR (rhr,   2,j+1,1)*AVG2T (coef,   1,j+1)
        spm=AVGT (rhr,   2,j+1,1)*AVG2R (coef,   2,j  )
        atrmp= .5*(s00+spm)*st(j)*(r(   1)**2/rh(   2)-rh(   2))
        rhs(   2,j,2)=rhs(   2,j,2)-atrmp        *vrvr0(j+1)
        s00=AVGR (rhr,nr  ,j  ,1)*AVG2T (coef,nrm1,j  )
        smp=AVGT (rhr,nrm1,j+1,1)*AVG2R (coef,nrm1,j  )
        atr00= .5*(s00+smp)*st(j)*(r(nrm1)**2/rh(nrm1)-rh(nrm1))
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-atr00        *vrvr1(j  )
        s00=AVGR (rhr,nr  ,j+1,1)*AVG2T (coef,nrm1,j+1)
        smm=AVGT (rhr,nrm1,j+1,1)*AVG2R (coef,nrm1,j  )
        atr0p=-.5*(s00+smm)*st(j)*(r(nrm1)**2/rh(nrm1)-rh(nrm1))
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-atr0p        *vrvr1(j+1)
      enddo
c
c ****** Set the dummy points.
c
      do j=2,ntm1
        arr00(nrm1,j)=1.
        arrp0(nrm1,j)=0.
        arr0p(nrm1,j)=0.
        rhs(nrm1,j,1)=0.
        art00(nrm1,j)=0.
        artp0(nrm1,j)=0.
        art0m(nrm1,j)=0.
        artpm(nrm1,j)=0.
      enddo
c
      do i=2,nrm1
        att00(i,ntm1)=1.
        attp0(i,ntm1)=0.
        att0p(i,ntm1)=0.
        rhs(i,ntm1,2)=0.
      enddo
c
c ****** Set the number of equations to be solved.
c
      ncgeq=2*nr*nt
c
      return
      end
c#######################################################################
      subroutine vmatp0 (rhs,iopt)
c
c-----------------------------------------------------------------------
c
c ****** Load the matrix for the m=0 mode for the vp equation.
c
c-----------------------------------------------------------------------
c
c ****** Variable IOPT controls which matrix is loaded.
c ****** When IOPT=1, load the matrix for the semi-implicit term.
c ****** When IOPT=2, load the matrix for the viscous term.
c ****** When IOPT=3, load the matrix for both terms.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Right hand side.
c
      real rhs(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Temporary array.
c
      real coef(nrm1,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
c ****** Initialize matrix diagonals to zero.
c
      app00(:,:)=0.
      appp0(:,:)=0.
      app0p(:,:)=0.
c
c ****** Load the coefficient of the div-grad operator.
c
      if (iopt.eq.1) then
        isim=1
        ivsm=0
      else if (iopt.eq.2) then
        isim=0
        ivsm=1
      else if (iopt.eq.3) then
        isim=1
        ivsm=1
      else
        isim=0
        ivsm=0
      end if
c
      do j=1,ntm1
        do i=1,nrm1
          coef(i,j)=dtime*( isim*sifac(i,j)
     &                     +ivsm*(visfac(i,j)+viscel(i,j)))
        enddo
      enddo
c
c ****** Load the internal points for the vp equation.
c
      do j=2,ntm1
      do i=2,nrm1
        s00=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        sp0=AVGR (rhr,i+1,j  ,1)*AVG2T (coef,i  ,j  )
        sm0=AVGR (rhr,i  ,j  ,1)*AVG2T (coef,i-1,j  )
        s0p=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        s0m=AVGT (rhr,i  ,j  ,1)*AVG2R (coef,i  ,j-1)
        r00=AVG  (rhr,i  ,j  ,1)
        dvp=rh(i)**2*drh(i)*dth(j)*sth(j)
        app00(i,j)=r00*dvp+
     &   ( sth(j)*dth(j)*( sp0*r(i  )**2/dr(i  )
     &                    +sm0*r(i-1)**2/dr(i-1))
     &    +drh(i)*( s0p*st(j  )/dt(j  )
     &             +s0m*st(j-1)/dt(j-1))
     &    +s00*drh(i)*dth(j)/sth(j))
        appp0(i,j)=-sp0*sth(j)*dth(j)*r(i)**2/dr(i)
        app0p(i,j)=-s0p*drh(i)*st(j)/dt(j)
        rhs(i,j)=rhs(i,j)*dvp
      enddo
      enddo
c
c ****** Eliminate the boundary points.
c
      do j=2,ntm1
        sp0=AVGR (rhr,  2,j  ,1)*AVG2T (coef,  1,j  )
        appm0=-sp0*sth(j)*dth(j)*r(1)**2/dr(1)
        app00(   2,j)=app00(   2,j)-appm0        *vpcr0(j)
        rhs  (   2,j)=rhs  (   2,j)-appm0        *vpvr0(j)
        app00(nrm1,j)=app00(nrm1,j)-appp0(nrm1,j)*vpcr1(j)
        rhs  (nrm1,j)=rhs  (nrm1,j)-appp0(nrm1,j)*vpvr1(j)
        appp0(nrm1,j)=0.
      enddo
      do i=2,nrm1
        s0p=AVGT (rhr,i  ,  2,1)*AVG2R (coef,i  ,  1)
        app0m=-s0p*drh(i)*st(1)/dt(1)
        app00(i,   2)=app00(i,   2)-app0m        *vpct0(i)
        rhs  (i,   2)=rhs  (i,   2)-app0m        *vpvt0(i)
        app00(i,ntm1)=app00(i,ntm1)-app0p(i,ntm1)*vpct1(i)
        rhs  (i,ntm1)=rhs  (i,ntm1)-app0p(i,ntm1)*vpvt1(i)
        app0p(i,ntm1)=0.
      enddo
c
c ****** Set the number of equations to be solved.
c
      ncgeq=nr*nt
c
      return
      end
c#######################################################################
      subroutine vmatrtp (rhs,m,iri,iopt)
c
c-----------------------------------------------------------------------
c
c ****** Load the matrix for m.gt.0 modes for the
c ****** coupled (vr,vt,vp) equations.
c
c-----------------------------------------------------------------------
c
c ****** IRI determines which set of equations is solved.
c ****** When IRI=1, solve for (VRR,VTR,VPI);
c ****** when IRI=2, solve for (VRI,VTI,VPR).
c
c ****** Variable IOPT controls which matrix is loaded.
c ****** When IOPT=1, load the matrix for the semi-implicit term.
c ****** When IOPT=2, load the matrix for the viscous term.
c ****** When IOPT=3, load the matrix for both terms.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use comm
      use cgcom1
c
c-----------------------------------------------------------------------
c
c ****** Right hand side.
c
      real rhs(nr,nt,3)
c
c-----------------------------------------------------------------------
c
c ****** Temporary array.
c
      real coef(nrm1,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
      k=m+1
c
c ****** Initialize matrix diagonals to zero.
c
      arr00(:,:)=0.
      arrp0(:,:)=0.
      arr0p(:,:)=0.
      att00(:,:)=0.
      attp0(:,:)=0.
      att0p(:,:)=0.
      app00(:,:)=0.
      appp0(:,:)=0.
      app0p(:,:)=0.
      art00(:,:)=0.
      artp0(:,:)=0.
      art0m(:,:)=0.
      artpm(:,:)=0.
      arp00(:,:)=0.
      arpp0(:,:)=0.
      atp00(:,:)=0.
      atp0p(:,:)=0.
c
c ****** Set the value of i*m.
c
      if (iri.eq.1) then
        emm=-fm(k)
      else
        emm= fm(k)
      end if
c
      emm2=fm2(k)
c
c ****** Load the coefficient of the div-grad operator.
c
      if (iopt.eq.1) then
        isim=1
        ivsm=0
      else if (iopt.eq.2) then
        isim=0
        ivsm=1
      else if (iopt.eq.3) then
        isim=1
        ivsm=1
      else
        isim=0
        ivsm=0
      end if
c
      do j=1,ntm1
        do i=1,nrm1
          coef(i,j)=dtime*( isim*sifac(i,j)
     &                     +ivsm*(visfac(i,j)+viscel(i,j)))
        enddo
      enddo
c
c ****** Load the internal points for the vr equation.
c
      do j=2,ntm1
      do i=2,nrm2
        s00=AVGR (rhr,i+1,j  ,1)*AVG2T (coef,i  ,j  )
        sp0=AVG  (rhr,i+1,j  ,1)*AVG2RT(coef,i+1,j  )
        sm0=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        s0p=AVGRT(rhr,i+1,j+1,1)*AVG2  (coef,i  ,j  )
        s0m=AVGRT(rhr,i+1,j  ,1)*AVG2  (coef,i  ,j-1)
        spp=AVGT (rhr,i+1,j+1,1)*AVG2R (coef,i+1,j  )
        smp=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        spm=AVGT (rhr,i+1,j  ,1)*AVG2R (coef,i+1,j-1)
        smm=AVGT (rhr,i  ,j  ,1)*AVG2R (coef,i  ,j-1)
        r00=AVGR (rhr,i+1,j  ,1)
        dvr=r(i)**2*dr(i)*dth(j)*sth(j)
        arr00(i,j)=r00*dvr+
     &   ( sth(j)*dth(j)*( sp0*rh(i+1)**2/drh(i+1)
     &                    +sm0*rh(i  )**2/drh(i  ))
     &    +dr(i)*( s0p*st(j  )/dt(j  )
     &            +s0m*st(j-1)/dt(j-1))
     &    +s00*dr(i)*dth(j)*(2.*sth(j)+emm2/sth(j)))
        arrp0(i,j)=-sp0*sth(j)*dth(j)*rh(i+1)**2/drh(i+1)
        arr0p(i,j)=-s0p*dr(i)*st(j)/dt(j)
        art00(i,j)= .5*(s00+smp)*st(j  )*(r(i)**2/rh(i  )-rh(i  ))
        artp0(i,j)=-.5*(s00+spp)*st(j  )*(r(i)**2/rh(i+1)-rh(i+1))
        art0m(i,j)=-.5*(s00+smm)*st(j-1)*(r(i)**2/rh(i  )-rh(i  ))
        artpm(i,j)= .5*(s00+spm)*st(j-1)*(r(i)**2/rh(i+1)-rh(i+1))
        arp00(i,j)= .5*(s00+sm0)*emm*dth(j)*( r(i)**2/rh(i  )
     &                                           -rh(i  ))
        arpp0(i,j)=-.5*(s00+sp0)*emm*dth(j)*( r(i)**2/rh(i+1)
     &                                           -rh(i+1))
        rhs(i,j,1)=rhs(i,j,1)*dvr
      enddo
      enddo
c
c ****** Load the internal points for the vt equation.
c
      do j=2,ntm2
      do i=2,nrm1
        s00=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        sp0=AVGRT(rhr,i+1,j+1,1)*AVG2  (coef,i  ,j  )
        sm0=AVGRT(rhr,i  ,j+1,1)*AVG2  (coef,i-1,j  )
        s0p=AVG  (rhr,i  ,j+1,1)*AVG2RT(coef,i  ,j+1)
        s0m=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        r00=AVGT (rhr,i  ,j+1,1)
        dvt=rh(i)**2*drh(i)*dt(j)*st(j)
        att00(i,j)=r00*dvt+
     &   ( st(j)*dt(j)*( sp0*r(i  )**2/dr(i  )
     &                  +sm0*r(i-1)**2/dr(i-1))
     &    +drh(i)*( s0p*sth(j+1)/dth(j+1)
     &             +s0m*sth(j  )/dth(j  ))
     &    +(1.+emm2)*s00*drh(i)*dt(j)/st(j))
        attp0(i,j)=-sp0*st(j)*dt(j)*r(i)**2/dr(i)
        att0p(i,j)=-s0p*drh(i)*sth(j+1)/dth(j+1)
        atp00(i,j)= .5*(s00+s0m)*emm*drh(i)
     &                    *(st(j)/sth(j  )-sth(j  )/st(j))
        atp0p(i,j)=-.5*(s00+s0p)*emm*drh(i)
     &                    *(st(j)/sth(j+1)-sth(j+1)/st(j))
        rhs(i,j,2)=rhs(i,j,2)*dvt
      enddo
      enddo
c
c ****** Load the internal points for the vp equation.
c
      do j=2,ntm1
      do i=2,nrm1
        s00=AVG  (rhr,i  ,j  ,1)*AVG2RT(coef,i  ,j  )
        sp0=AVGR (rhr,i+1,j  ,1)*AVG2T (coef,i  ,j  )
        sm0=AVGR (rhr,i  ,j  ,1)*AVG2T (coef,i-1,j  )
        s0p=AVGT (rhr,i  ,j+1,1)*AVG2R (coef,i  ,j  )
        s0m=AVGT (rhr,i  ,j  ,1)*AVG2R (coef,i  ,j-1)
        r00=AVG  (rhr,i  ,j  ,1)
        dvp=rh(i)**2*drh(i)*dth(j)*sth(j)
        app00(i,j)=r00*dvp+
     &   ( sth(j)*dth(j)*( sp0*r(i  )**2/dr(i  )
     &                    +sm0*r(i-1)**2/dr(i-1))
     &    +drh(i)*( s0p*st(j  )/dt(j  )
     &             +s0m*st(j-1)/dt(j-1))
     &    +(1.+emm2)*s00*drh(i)*dth(j)/sth(j))
        appp0(i,j)=-sp0*sth(j)*dth(j)*r(i)**2/dr(i)
        app0p(i,j)=-s0p*drh(i)*st(j)/dt(j)
        rhs(i,j,3)=rhs(i,j,3)*dvp
      enddo
      enddo
c
c ****** Eliminate the boundary points.
c
c ****** (vr points in vr equation).
c
      do j=2,ntm1
        sp0=AVG  (rhr,  2,j  ,1)*AVG2RT(coef,  2,j  )
        arrm0=-sp0*sth(j)*dth(j)*rh(2)**2/drh(2)
        rhs(   2,j,1)=rhs(   2,j,1)-arrm0        *vrvr0(j)
        rhs(nrm2,j,1)=rhs(nrm2,j,1)-arrp0(nrm2,j)*vrvr1(j)
        arrp0(nrm2,j)=0.
      enddo
      do i=2,nrm2
        s0p=AVGRT(rhr,i+1,  2,1)*AVG2  (coef,i  ,  1)
        arr0m=-s0p*dr(i)*st(1)/dt(1)
        arr00(i,   2)=arr00(i,   2)-arr0m        *vrct0(i)
        rhs(i,   2,1)=rhs(i,   2,1)-arr0m        *vrvt0(i)
        arr00(i,ntm1)=arr00(i,ntm1)-arr0p(i,ntm1)*vrct1(i)
        rhs(i,ntm1,1)=rhs(i,ntm1,1)-arr0p(i,ntm1)*vrvt1(i)
        arr0p(i,ntm1)=0.
      enddo
c
c ****** (vt points in vt equation).
c
      do j=2,ntm2
        sp0=AVGRT(rhr,  2,j+1,1)*AVG2  (coef,  1,j  )
        attm0=-sp0*st(j)*dt(j)*r(1)**2/dr(1)
        att00(   2,j)=att00(   2,j)-attm0        *vtcr0(j)
        rhs(   2,j,2)=rhs(   2,j,2)-attm0        *vtvr0(j)
        att00(nrm1,j)=att00(nrm1,j)-attp0(nrm1,j)*vtcr1(j)
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-attp0(nrm1,j)*vtvr1(j)
        attp0(nrm1,j)=0.
      enddo
c
c ****** Note that the theta = 0, pi boundary values
c ****** contribute only for the m=1 mode.
c
      if (m.eq.1) then
        do i=2,nrm1
          s0p=AVG  (rhr,i  ,  2,1)*AVG2RT(coef,i  ,  2)
          att0m=-s0p*drh(i)*sth(2)/dth(2)
          att00(i,   2)=att00(i,   2)-att0m
          att00(i,ntm2)=att00(i,ntm2)-att0p(i,ntm2)
          att0p(i,ntm2)=0.
        enddo
      else
        do i=2,nrm1
          s0p=AVG  (rhr,i  ,  2,1)*AVG2RT(coef,i  ,  2)
          att0m=-s0p*drh(i)*sth(2)/dth(2)
          rhs(i,   2,2)=rhs(i,   2,2)-att0m        *vtvt0(i)
          rhs(i,ntm2,2)=rhs(i,ntm2,2)-att0p(i,ntm2)*vtvt0(i)
          att0p(i,ntm2)=0.
        enddo
      end if
c
c ****** (vp points in vp equation).
c
      do j=2,ntm1
        sp0=AVGR (rhr,  2,j  ,1)*AVG2T (coef,  1,j  )
        appm0=-sp0*sth(j)*dth(j)*r(1)**2/dr(1)
        app00(   2,j)=app00(   2,j)-appm0        *vpcr0(j)
        rhs(   2,j,3)=rhs(   2,j,3)-appm0        *vpvr0(j)
        app00(nrm1,j)=app00(nrm1,j)-appp0(nrm1,j)*vpcr1(j)
        rhs(nrm1,j,3)=rhs(nrm1,j,3)-appp0(nrm1,j)*vpvr1(j)
        appp0(nrm1,j)=0.
      enddo
      do i=2,nrm1
        s0p=AVGT (rhr,i  ,  2,1)*AVG2R (coef,i  ,  1)
        app0m=-s0p*drh(i)*st(1)/dt(1)
        app00(i,   2)=app00(i,   2)-app0m        *vpct0(i)
        rhs(i,   2,3)=rhs(i,   2,3)-app0m        *vpvt0(i)
        app00(i,ntm1)=app00(i,ntm1)-app0p(i,ntm1)*vpct1(i)
        rhs(i,ntm1,3)=rhs(i,ntm1,3)-app0p(i,ntm1)*vpvt1(i)
        app0p(i,ntm1)=0.
      enddo
c
c ****** (vt points in vr equation).
c
c ****** Note that the (unknown) vt boundary points for m=1
c ****** do not contribute, since the coupling coefficients
c ****** (ART0M and ARTPM at theta = 0 and ART00 and ARTP0
c ****** at theta = pi) are zero.
c
      if (m.ne.1) then
        do i=2,nrm2
          rhs(i,   2,1)=rhs(i,   2,1)-art0m(i,   2)*vtvt0(i  )
          art0m(i,   2)=0.
          rhs(i,ntm1,1)=rhs(i,ntm1,1)-art00(i,ntm1)*vtvt1(i  )
          art00(i,ntm1)=0.
          rhs(i,   2,1)=rhs(i,   2,1)-artpm(i,   2)*vtvt0(i+1)
          artpm(i,   2)=0.
          rhs(i,ntm1,1)=rhs(i,ntm1,1)-artp0(i,ntm1)*vtvt1(i+1)
          artp0(i,ntm1)=0.
        enddo
      end if
c
c ****** (vr points in vt equation).
c
      do j=2,ntm2
        s00=AVGR (rhr,   2,j  ,1)*AVG2T (coef,   1,j  )
        spp=AVGT (rhr,   2,j+1,1)*AVG2R (coef,   2,j  )
        atrm0=-.5*(s00+spp)*st(j)*(r(   1)**2/rh(   2)-rh(   2))
        rhs(   2,j,2)=rhs(   2,j,2)-atrm0        *vrvr0(j  )
        s00=AVGR (rhr,   2,j+1,1)*AVG2T (coef,   1,j+1)
        spm=AVGT (rhr,   2,j+1,1)*AVG2R (coef,   2,j  )
        atrmp= .5*(s00+spm)*st(j)*(r(   1)**2/rh(   2)-rh(   2))
        rhs(   2,j,2)=rhs(   2,j,2)-atrmp        *vrvr0(j+1)
        s00=AVGR (rhr,nr  ,j  ,1)*AVG2T (coef,nrm1,j  )
        smp=AVGT (rhr,nrm1,j+1,1)*AVG2R (coef,nrm1,j  )
        atr00= .5*(s00+smp)*st(j)*(r(nrm1)**2/rh(nrm1)-rh(nrm1))
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-atr00        *vrvr1(j  )
        s00=AVGR (rhr,nr  ,j+1,1)*AVG2T (coef,nrm1,j+1)
        smm=AVGT (rhr,nrm1,j+1,1)*AVG2R (coef,nrm1,j  )
        atr0p=-.5*(s00+smm)*st(j)*(r(nrm1)**2/rh(nrm1)-rh(nrm1))
        rhs(nrm1,j,2)=rhs(nrm1,j,2)-atr0p        *vrvr1(j+1)
      enddo
c
c ****** (vr points in vp equation).
c
      do j=2,ntm1
        s00=AVGR (rhr,   2,j  ,1)*AVG2T (coef,   1,j  )
        sp0=AVG  (rhr,   2,j  ,1)*AVG2RT(coef,   2,j  )
        aprm0=-.5*(s00+sp0)*emm*dth(j)*(r(   1)**2/rh(   2)-rh(   2))
        rhs(   2,j,3)=rhs(   2,j,3)-aprm0        *vrvr0(j  )
        s00=AVGR (rhr,  nr,j  ,1)*AVG2T (coef,nrm1,j  )
        sm0=AVG  (rhr,nrm1,j  ,1)*AVG2RT(coef,nrm1,j  )
        apr00= .5*(s00+sm0)*emm*dth(j)*(r(nrm1)**2/rh(nrm1)-rh(nrm1))
        rhs(nrm1,j,3)=rhs(nrm1,j,3)-apr00        *vrvr1(j  )
      enddo
c
c ****** (vt points in vp equation).
c ****** (Note that these terms only contribute for m=1.)
c
      if (m.eq.1) then
        do i=2,nrm1
          s00=AVGT (rhr,i  ,   2,1)*AVG2R (coef,i  ,   1)
          s0p=AVG  (rhr,i  ,   2,1)*AVG2RT(coef,i  ,   2)
          apt0m=-.5*(s00+s0p)*emm*drh(i)*(-2.5)
          rhs(i,   2,3)=rhs(i,   2,3)-apt0m        *vtvt0(i  )
          s00=AVGT (rhr,i  ,nt  ,1)*AVG2R (coef,i  ,ntm1)
          s0m=AVG  (rhr,i  ,ntm1,1)*AVG2RT(coef,i  ,ntm1)
          apt00= .5*(s00+s0m)*emm*drh(i)*(-2.5)
          rhs(i,ntm1,3)=rhs(i,ntm1,3)-apt00        *vtvt1(i  )
        enddo
      end if
c
c ****** Set the dummy points.
c
      do j=2,ntm1
        arr00(nrm1,j)=1.
        arrp0(nrm1,j)=0.
        arr0p(nrm1,j)=0.
        rhs(nrm1,j,1)=0.
        art00(nrm1,j)=0.
        artp0(nrm1,j)=0.
        art0m(nrm1,j)=0.
        artpm(nrm1,j)=0.
        arp00(nrm1,j)=0.
        arpp0(nrm1,j)=0.
      enddo
c
      do i=2,nrm1
        att00(i,ntm1)=1.
        attp0(i,ntm1)=0.
        att0p(i,ntm1)=0.
        atp00(i,ntm1)=0.
        atp0p(i,ntm1)=0.
        rhs(i,ntm1,2)=0.
      enddo
c
c ****** Set the number of equations to be solved.
c
      ncgeq=3*nr*nt
c
      return
      end
c#######################################################################
      subroutine putvrt (vvr,vvt,xcg)
c
c-----------------------------------------------------------------------
c
c ****** Put the vr and vt components of the velocity from arrays
c ****** VVR and VVT into the CG array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vvr(nr,nt),vvt(nr,nt),xcg(nr,nt,2)
c
c-----------------------------------------------------------------------
c
c ****** Load the internal points of XCG with VVR and VVT.
c
      xcg(2:nrm2,2:ntm1,1)=vvr(2:nrm2,2:ntm1)
      xcg(2:nrm1,2:ntm2,2)=vvt(2:nrm1,2:ntm2)
c
c ****** Set the boundary points to zero.
c
      xcg( :, 1,:)=0.
      xcg( :,nt,:)=0.
      xcg( 1, :,:)=0.
      xcg(nr, :,:)=0.
c
      xcg(nrm1,:,1)=0.
      xcg(:,ntm1,2)=0.
c
      return
      end
c#######################################################################
      subroutine putvp (vvp,xcg)
c
c-----------------------------------------------------------------------
c
c ****** Put the vp component of the velocity in array
c ****** VVP into the CG array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vvp(nr,nt),xcg(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Load the internal points of XCG with VVP.
c
      xcg(2:nrm1,2:ntm1)=vvp(2:nrm1,2:ntm1)
c
c ****** Set the boundary points to zero.
c
      xcg( :, 1)=0.
      xcg( :,nt)=0.
      xcg( 1, :)=0.
      xcg(nr, :)=0.
c
      return
      end
c#######################################################################
      subroutine putvrtp (vvr,vvt,vvp,xcg)
c
c-----------------------------------------------------------------------
c
c ****** Put the vr, vt, and vp components of the velocity from
c ****** arrays VVR, VVT, and VVP into the CG array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real xcg(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      call putvrt (vvr,vvt,xcg)
      call putvp (vvp,xcg(1,1,3))
c
      return
      end
c#######################################################################
      subroutine setvrt (xcg,vvr,vvt)
c
c-----------------------------------------------------------------------
c
c ****** Set the vr and vt components of the velocity in arrays
c ****** VVR and VVT from the CG solution in array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real xcg(nr,nt,2),vvr(nr,nt),vvt(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      vvr(2:nrm2,2:ntm1)=xcg(2:nrm2,2:ntm1,1)
      vvt(2:nrm1,2:ntm2)=xcg(2:nrm1,2:ntm2,2)
c
c ****** Set the boundary points.
c
      do j=2,ntm1
        vvr(   1,j)=vrvr0(j)
        vvr(nrm1,j)=vrvr1(j)
      enddo
      do i=1,nrm1
        vvr(i,   1)=vrvt0(i)-vvr(i,   2)*vrct0(i)
        vvr(i,  nt)=vrvt1(i)-vvr(i,ntm1)*vrct1(i)
      enddo
c
      do j=2,ntm2
        vvt( 1,j)=vtvr0(j)-vvt(   2,j)*vtcr0(j)
        vvt(nr,j)=vtvr1(j)-vvt(nrm1,j)*vtcr1(j)
      enddo
      do i=1,nr
        vvt(i,   1)=vtvt0(i)
        vvt(i,ntm1)=vtvt1(i)
      enddo
c
c ****** Set the dummy points to zero.
c
      do j=1,nt
        vvr(nr,j)=0.
      enddo
c
      do i=1,nr
        vvt(i,nt)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine setvp (xcg,vvp)
c
c-----------------------------------------------------------------------
c
c ****** Set the vp component of the velocity in array
c ****** VVP from the CG solution in array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real xcg(nr,nt),vvp(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      vvp(2:nrm1,2:ntm1)=xcg(2:nrm1,2:ntm1)
c
c ****** Set the boundary points.
c
      do j=2,ntm1
        vvp( 1,j)=vpvr0(j)-vvp(   2,j)*vpcr0(j)
        vvp(nr,j)=vpvr1(j)-vvp(nrm1,j)*vpcr1(j)
      enddo
      do i=1,nr
        vvp(i, 1)=vpvt0(i)-vvp(i,   2)*vpct0(i)
        vvp(i,nt)=vpvt1(i)-vvp(i,ntm1)*vpct1(i)
      enddo
c
      return
      end
c#######################################################################
      subroutine setvrtp (m,iri,xcg,vvr,vvt,vvp)
c
c-----------------------------------------------------------------------
c
c ****** Set the vr, vt, and vp components of the velocity in arrays
c ****** VVR, VVT, and VVP from the CG solution in array XCG.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real xcg(nr,nt,3),vvr(nr,nt),vvt(nr,nt),vvp(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Boundary conditions on v.
c
      common /vbccom/             vrvr0(nt  ),
     &                            vrvr1(nt  ),
     &                vtcr0(ntm1),vtvr0(ntm1),
     &                vtcr1(ntm1),vtvr1(ntm1),
     &                vpcr0(nt  ),vpvr0(nt  ),
     &                vpcr1(nt  ),vpvr1(nt  ),
     &                vrct0(nrm1),vrvt0(nrm1),
     &                vrct1(nrm1),vrvt1(nrm1),
     &                            vtvt0(nr  ),
     &                            vtvt1(nr  ),
     &                vpct0(nr  ),vpvt0(nr  ),
     &                vpct1(nr  ),vpvt1(nr  )
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      vvr(2:nrm2,2:ntm1)=xcg(2:nrm2,2:ntm1,1)
      vvt(2:nrm1,2:ntm2)=xcg(2:nrm1,2:ntm2,2)
      vvp(2:nrm1,2:ntm1)=xcg(2:nrm1,2:ntm1,3)
c
c ****** Set the boundary points.
c
      do j=2,ntm1
        vvr(   1,j)=vrvr0(j)
        vvr(nrm1,j)=vrvr1(j)
      enddo
      do i=1,nrm1
        vvr(i,   1)=vrvt0(i)-vvr(i,   2)*vrct0(i)
        vvr(i,  nt)=vrvt1(i)-vvr(i,ntm1)*vrct1(i)
      enddo
c
      do j=2,ntm1
        vvp( 1,j)=vpvr0(j)-vvp(   2,j)*vpcr0(j)
        vvp(nr,j)=vpvr1(j)-vvp(nrm1,j)*vpcr1(j)
      enddo
      do i=1,nr
        vvp(i, 1)=vpvt0(i)-vvp(i,   2)*vpct0(i)
        vvp(i,nt)=vpvt1(i)-vvp(i,ntm1)*vpct1(i)
      enddo
c
      do j=2,ntm2
        vvt( 1,j)=vtvr0(j)-vvt(   2,j)*vtcr0(j)
        vvt(nr,j)=vtvr1(j)-vvt(nrm1,j)*vtcr1(j)
      enddo
c
c ****** Note that the BC on vt at theta = 0, pi for the m=1 mode
c ****** is special.
c
      if (m.eq.1) then
c
        if (iri.eq.1) then
c
c ****** Real part of vt and imaginary part of vp.
c
          do i=1,nr
            vvt(i,   1)= vvp(i,   2)
            vvt(i,ntm1)=-vvp(i,ntm1)
          enddo
c
        else
c
c ****** Imaginary part of vt and real part of vp.
c
          do i=1,nr
            vvt(i,   1)=-vvp(i,   2)
            vvt(i,ntm1)= vvp(i,ntm1)
          enddo
c
        end if
c
      else
c
        do i=1,nr
          vvt(i,   1)=vtvt0(i)
          vvt(i,ntm1)=vtvt1(i)
        enddo
c
      end if
c
c ****** Set the dummy points.
c
      do j=1,nt
        vvr(nr,j)=0.
      enddo
c
      do i=1,nr
        vvt(i,nt)=0.
      enddo
c
      return
      end
c#######################################################################
      subroutine advpw
c
c-----------------------------------------------------------------------
c
c ****** Advance the Alfven wave pressure.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real va(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the Alfven speed.
c
      call getva (va)
c
c ****** Advance the Alfven wave energy density ep.
c
      call advep (va)
c
c ****** Advance the Alfven wave energy density em.
c
      call advem (va)
c
      return
      end
c#######################################################################
      subroutine advep (va)
c
c-----------------------------------------------------------------------
c
c ****** Advance the Alfven wave energy density ep
c ****** (with v_A parallel to B).
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real va(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      real vawr(nr,nt,np),vawt(nr,nt,np),vawp(nr,nt,np),
     &     divre(nr,nt,np),divvre(nr,nt,np),
     &     divr(nr,nt,npd3),divi(nr,nt,npd3),
     &     eppr(nr,nt,npd3),eppi(nr,nt,npd3),
     &     ere(nr,nt,np),
     &     fkdotvmx(nr),rhopaw(nr)
c
c-----------------------------------------------------------------------
c
c ****** Get the wave advection velocity.
c
      pm=1.
c
      call getvaw (pm,va,vawr,vawt,vawp)
c
c ****** Determine the boundary conditions on the Alfven waves
c ****** at the radial boundaries.
c
      call getbcep (vawr)
c
c ****** Store div(v) in real space in DIVVRE.
c
      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            divr(i,j,k)= ( r(i  )**2*vrr(i  ,j,k)
     &                    -r(i-1)**2*vrr(i-1,j,k)
     &                   )/(rh(i)**2*drh(i))
     &                  +( st(j  )*vtr(i,j  ,k)
     &                    -st(j-1)*vtr(i,j-1,k)
     &                   )/(rh(i)*sth(j)*dth(j))
     &                  -fm(k)*vpi(i,j,k)/(rh(i)*sth(j))
            divi(i,j,k)= ( r(i  )**2*vri(i  ,j,k)
     &                    -r(i-1)**2*vri(i-1,j,k)
     &                   )/(rh(i)**2*drh(i))
     &                  +( st(j  )*vti(i,j  ,k)
     &                    -st(j-1)*vti(i,j-1,k)
     &                   )/(rh(i)*sth(j)*dth(j))
     &                  +fm(k)*vpr(i,j,k)/(rh(i)*sth(j))
          enddo
        enddo
      enddo
c
      call zb3 (divr,npd3)
      call zb3 (divi,npd3)
c
      call fft (divvre,divr,divi,0)
c
c ****** Find the maximum time step allowed for an explicit
c ****** advance.
c
      do i=2,nrm1
        fkdotvmx(i)=1.e-20
        do k=1,np
          do j=2,ntm1
            avvr=AVGR (vawr,i,j,k)
            avvt=AVGT (vawt,i,j,k)
            avvp=AVG  (vawp,i,j,k)
            fkr2=1./drh(i)**2
            fkt2=1./(rh(i)*dth(j))**2
            fkp2=fm2(npd3)/(sth(j)*rh(i))**2
            fkdotv=sqrt(fkr2*avvr**2+fkt2*avvt**2+fkp2*avvp**2)
            fkdotvmx(i)=max(fkdotvmx(i),fkdotv)
          enddo
        enddo
      enddo
c
      dtaw=dtime
      do i=2,nrm1
        dtaw=min(dtaw,strmax*rho_aw/fkdotvmx(i))
      enddo
c
      nsteps=.99+dtime/dtaw
c
      dtaw=dtime/nsteps
c
c ****** Keep track of the average time step used.
c
      nsawp=nsawp+1
      dtawpav=(dtawpav*(nsawp-1)+dtaw)/nsawp
c
c ****** Set the (artificial) radial "density profile".
c
      do i=2,nrm1
        dtawr=strmax/fkdotvmx(i)
        rhopaw(i)=max(dtaw/dtawr,1.)
      enddo
c
c ****** Advance the Alfven wave energy density.
c
      do it=1,nsteps
c
c ****** Predictor step.
c
        call divqv (epr,epi,vawr,vawt,vawp,ere,divr,divi,divre)
c
        if (ifpc.ne.0) then
c
          do k=1,npd3
            do j=2,ntm1
              do i=2,nrm1
                eppr(i,j,k)=epr(i,j,k)-divr(i,j,k)*dtaw/rhopaw(i)
                eppi(i,j,k)=epi(i,j,k)-divi(i,j,k)*dtaw/rhopaw(i)
              enddo
            enddo
          enddo
c
c ****** Set boundary conditions.
c
          call setbcep (eppr,eppi)
c
c ****** Corrector step.
c
          call divqv (eppr,eppi,vawr,vawt,vawp,ere,divr,divi,divre)
c
        end if
c
c ****** Advance ep by the advection amount.
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              epr(i,j,k)=epr(i,j,k)-divr(i,j,k)*dtaw/rhopaw(i)
              epi(i,j,k)=epi(i,j,k)-divi(i,j,k)*dtaw/rhopaw(i)
            enddo
          enddo
        enddo
c
c ****** Add in the -.5*ep*div(v) part.
c
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              divre(i,j,k)=ep(i,j,k)*divvre(i,j,k)
            enddo
          enddo
        enddo
c
        call fft (divre,divr,divi,1)
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              epr(i,j,k)=epr(i,j,k)-.5*divr(i,j,k)*dtaw/rhopaw(i)
              epi(i,j,k)=epi(i,j,k)-.5*divi(i,j,k)*dtaw/rhopaw(i)
            enddo
          enddo
        enddo
c
c ****** Set boundary conditions.
c
        call setbcep (epr,epi)
c
c ****** Get the wave energy density in real space.
c
        call fft (ep,epr,epi,0)
c
      enddo
c
      return
      end
c#######################################################################
      subroutine advem (va)
c
c-----------------------------------------------------------------------
c
c ****** Advance the Alfven wave energy density em
c ****** (with v_A anti-parallel to B).
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real va(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      real vawr(nr,nt,np),vawt(nr,nt,np),vawp(nr,nt,np),
     &     divre(nr,nt,np),divvre(nr,nt,np),
     &     divr(nr,nt,npd3),divi(nr,nt,npd3),
     &     empr(nr,nt,npd3),empi(nr,nt,npd3),
     &     ere(nr,nt,np),
     &     fkdotvmx(nr),rhopaw(nr)
c
c-----------------------------------------------------------------------
c
c ****** Get the wave advection velocity.
c
      pm=-1.
c
      call getvaw (pm,va,vawr,vawt,vawp)
c
c ****** Determine the boundary conditions on the Alfven waves
c ****** at the radial boundaries.
c
      call getbcem (vawr)
c
c ****** Store div(v) in real space in DIVVRE.
c
      do k=1,npd3
        do j=2,ntm1
          do i=2,nrm1
            divr(i,j,k)= ( r(i  )**2*vrr(i  ,j,k)
     &                    -r(i-1)**2*vrr(i-1,j,k)
     &                   )/(rh(i)**2*drh(i))
     &                  +( st(j  )*vtr(i,j  ,k)
     &                    -st(j-1)*vtr(i,j-1,k)
     &                   )/(rh(i)*sth(j)*dth(j))
     &                  -fm(k)*vpi(i,j,k)/(rh(i)*sth(j))
            divi(i,j,k)= ( r(i  )**2*vri(i  ,j,k)
     &                    -r(i-1)**2*vri(i-1,j,k)
     &                   )/(rh(i)**2*drh(i))
     &                  +( st(j  )*vti(i,j  ,k)
     &                    -st(j-1)*vti(i,j-1,k)
     &                   )/(rh(i)*sth(j)*dth(j))
     &                  +fm(k)*vpr(i,j,k)/(rh(i)*sth(j))
          enddo
        enddo
      enddo
c
      call zb3 (divr,npd3)
      call zb3 (divi,npd3)
c
      call fft (divvre,divr,divi,0)
c
c ****** Find the maximum time step allowed for an explicit
c ****** advance.
c
      do i=2,nrm1
        fkdotvmx(i)=1.e-20
        do k=1,np
          do j=2,ntm1
            avvr=AVGR (vawr,i,j,k)
            avvt=AVGT (vawt,i,j,k)
            avvp=AVG  (vawp,i,j,k)
            fkr2=1./drh(i)**2
            fkt2=1./(rh(i)*dth(j))**2
            fkp2=fm2(npd3)/(sth(j)*rh(i))**2
            fkdotv=sqrt(fkr2*avvr**2+fkt2*avvt**2+fkp2*avvp**2)
            fkdotvmx(i)=max(fkdotvmx(i),fkdotv)
          enddo
        enddo
      enddo
c
      dtaw=dtime
      do i=2,nrm1
        dtaw=min(dtaw,strmax*rho_aw/fkdotvmx(i))
      enddo
c
      nsteps=.99+dtime/dtaw
c
      dtaw=dtime/nsteps
c
c ****** Keep track of the average time step used.
c
      nsawm=nsawm+1
      dtawmav=(dtawmav*(nsawm-1)+dtaw)/nsawm
c
c ****** Set the (artificial) radial "density profile".
c
      do i=2,nrm1
        dtawr=strmax/fkdotvmx(i)
        rhopaw(i)=max(dtaw/dtawr,1.)
      enddo
c
c ****** Advance the Alfven wave energy density.
c
      do it=1,nsteps
c
c ****** Predictor step.
c
        call divqv (emr,emi,vawr,vawt,vawp,ere,divr,divi,divre)
c
        if (ifpc.ne.0) then
c
          do k=1,npd3
            do j=2,ntm1
              do i=2,nrm1
                empr(i,j,k)=emr(i,j,k)-divr(i,j,k)*dtaw/rhopaw(i)
                empi(i,j,k)=emi(i,j,k)-divi(i,j,k)*dtaw/rhopaw(i)
              enddo
            enddo
          enddo
c
c ****** Set boundary conditions.
c
          call setbcem (empr,empi)
c
c ****** Corrector step.
c
          call divqv (empr,empi,vawr,vawt,vawp,ere,divr,divi,divre)
c
        end if
c
c ****** Advance em by the advection amount.
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              emr(i,j,k)=emr(i,j,k)-divr(i,j,k)*dtaw/rhopaw(i)
              emi(i,j,k)=emi(i,j,k)-divi(i,j,k)*dtaw/rhopaw(i)
            enddo
          enddo
        enddo
c
c ****** Add in the -.5*em*div(v) part.
c
        do k=1,np
          do j=2,ntm1
            do i=2,nrm1
              divre(i,j,k)=em(i,j,k)*divvre(i,j,k)
            enddo
          enddo
        enddo
c
        call fft (divre,divr,divi,1)
c
        do k=1,npd3
          do j=2,ntm1
            do i=2,nrm1
              emr(i,j,k)=emr(i,j,k)-.5*divr(i,j,k)*dtaw/rhopaw(i)
              emi(i,j,k)=emi(i,j,k)-.5*divi(i,j,k)*dtaw/rhopaw(i)
            enddo
          enddo
        enddo
c
c ****** Set boundary conditions.
c
        call setbcem (emr,emi)
c
c ****** Get the wave energy density in real space.
c
        call fft (em,emr,emi,0)
c
      enddo
c
      return
      end
c#######################################################################
      subroutine getva (va)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the Alfven speed.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real va(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      do k=1,np
        do j=2,ntm1
          do i=2,nrm1
            avrho=AVG  (rho,i,j,k)
            avbr =AVGT (br ,i,j,k)
            avbt =AVGR (bt ,i,j,k)
            avbp =AVGRT(bp ,i,j,k)
            bsq=avbr**2+avbt**2+avbp**2
            va(i,j,k)=sqrt(bsq/avrho)
          enddo
        enddo
        do j=2,ntm1
          i=1
          avrho=AVGR (rho,i+1,j  ,k)
          avbr =AVGRT(br ,i+1,j  ,k)
          avbt =AVG  (bt ,i  ,j  ,k)
          avbp =AVGT (bp ,i  ,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          vab=sqrt(bsq/avrho)
          va( 1,j,k)=2.*vab-va(   2,j,k)
          i=nrm1
          avrho=AVGR (rho,i+1,j  ,k)
          avbr =AVGRT(br ,i+1,j  ,k)
          avbt =AVG  (bt ,i  ,j  ,k)
          avbp =AVGT (bp ,i  ,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          vab=sqrt(bsq/avrho)
          va(nr,j,k)=2.*vab-va(nrm1,j,k)
        enddo
        do i=2,nrm1
          j=1
          avrho=AVGT (rho,i  ,j+1,k)
          avbr =AVG  (br ,i  ,j  ,k)
          avbt =AVGRT(bt ,i  ,j+1,k)
          avbp =AVGR (bp ,i  ,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          vab=sqrt(bsq/avrho)
          va(i, 1,k)=2.*vab-va(i,   2,k)
          j=ntm1
          avrho=AVGT (rho,i  ,j+1,k)
          avbr =AVG  (br ,i  ,j  ,k)
          avbt =AVGRT(bt ,i  ,j+1,k)
          avbp =AVGR (bp ,i  ,j  ,k)
          bsq=avbr**2+avbt**2+avbp**2
          vab=sqrt(bsq/avrho)
          va(i,nt,k)=2.*vab-va(i,ntm1,k)
        enddo
        i=1
        j=1
        avrho=AVGRT(rho,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        bsq=avbr**2+avbt**2+avbp**2
        vab=sqrt(bsq/avrho)
        va( 1, 1,k)=4.*vab-va(   1,   2,k)
     &                    -va(   2,   1,k)
     &                    -va(   2,   2,k)
        i=nrm1
        j=1
        avrho=AVGRT(rho,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        bsq=avbr**2+avbt**2+avbp**2
        vab=sqrt(bsq/avrho)
        va(nr, 1,k)=4.*vab-va(nr  ,   2,k)
     &                    -va(nrm1,   1,k)
     &                    -va(nrm1,   2,k)
        i=1
        j=ntm1
        avrho=AVGRT(rho,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        bsq=avbr**2+avbt**2+avbp**2
        vab=sqrt(bsq/avrho)
        va( 1,nt,k)=4.*vab-va(   1,ntm1,k)
     &                    -va(   2,nt  ,k)
     &                    -va(   2,ntm1,k)
        i=nrm1
        j=ntm1
        avrho=AVGRT(rho,i+1,j+1,k)
        avbr =AVGR (br ,i+1,j  ,k)
        avbt =AVGT (bt ,i  ,j+1,k)
        avbp =AVG  (bp ,i  ,j  ,k)
        bsq=avbr**2+avbt**2+avbp**2
        vab=sqrt(bsq/avrho)
        va(nr,nt,k)=4.*vab-va(nr  ,ntm1,k)
     &                    -va(nrm1,nt  ,k)
     &                    -va(nrm1,ntm1,k)
      enddo
c
      return
      end
c#######################################################################
      subroutine getvaw (pm,va,vawr,vawt,vawp)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the Alfven wave advection velocity.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real va(nr,nt,np)
      real vawr(nr,nt,np),vawt(nr,nt,np),vawp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      do k=1,np
c
        do j=2,ntm1
          do i=1,nrm1
            avva=AVGR (va,i+1,j  ,k)
            avbr=AVGRT(br,i+1,j  ,k)
            avbt=AVG  (bt,i  ,j  ,k)
            avbp=AVGT (bp,i  ,j  ,k)
            bsq=avbr**2+avbt**2+avbp**2
            bmag=sqrt(bsq)
            if (bmag.eq.0.) then
              brhat=1.
            else
              brhat=avbr/bmag
            end if
            vawr(i,j,k)=vr(i,j,k)+pm*brhat*avva
          enddo
        enddo
c
        do j=1,ntm1
          do i=2,nrm1
            avva=AVGT (va,i  ,j+1,k)
            avbr=AVG  (br,i  ,j  ,k)
            avbt=AVGRT(bt,i  ,j+1,k)
            avbp=AVGR (bp,i  ,j  ,k)
            bsq=avbr**2+avbt**2+avbp**2
            bmag=sqrt(bsq)
            if (bmag.eq.0.) then
              bthat=0.
            else
              bthat=avbt/bmag
            end if
            vawt(i,j,k)=vt(i,j,k)+pm*bthat*avva
          enddo
        enddo
c
        do j=2,ntm1
          do i=2,nrm1
            avva=AVG  (va,i  ,j  ,k)
            avbr=AVGT (br,i  ,j  ,k)
            avbt=AVGR (bt,i  ,j  ,k)
            avbp=AVGRT(bp,i  ,j  ,k)
            bsq=avbr**2+avbt**2+avbp**2
            bmag=sqrt(bsq)
            if (bmag.eq.0.) then
              bphat=0.
            else
              bphat=avbp/bmag
            end if
            vawp(i,j,k)=vp(i,j,k)+pm*bphat*avva
          enddo
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine getbcep (vawr)
c
c-----------------------------------------------------------------------
c
c ****** Set the Alfven wave energy density ep at the radial
c ****** boundaries.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vawr(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      real epbcr0re(nt,np),epbcr1re(nt,np)
c
c-----------------------------------------------------------------------
c
      do k=1,np
        do j=1,nt
          if (vawr(1,j,k).ge.0.) then
            epbcr0re(j,k)=2.*pw0*awthprof(j)
          else
            epbcr0re(j,k)=EXTRAPR0(ep,   2,j,k)

          end if
          if (vawr(nrm1,j,k).le.0.) then
            epbcr1re(j,k)=0.
          else
            epbcr1re(j,k)=EXTRAPR1(ep,nrm1,j,k)
          end if
        enddo
      enddo
c
      call fft_tp (epbcr0re,epbcr0(1,1,1),epbcr0(1,1,2),1)
      call fft_tp (epbcr1re,epbcr1(1,1,1),epbcr1(1,1,2),1)
c
      return
      end
c#######################################################################
      subroutine getbcem (vawr)
c
c-----------------------------------------------------------------------
c
c ****** Set the Alfven wave energy density em at the radial
c ****** boundaries.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real vawr(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      real embcr0re(nt,np),embcr1re(nt,np)
c
c-----------------------------------------------------------------------
c
      do k=1,np
        do j=1,nt
          if (vawr(1,j,k).ge.0.) then
            embcr0re(j,k)=2.*pw0*awthprof(j)
          else
            embcr0re(j,k)=EXTRAPR0(em,   2,j,k)

          end if
          if (vawr(nrm1,j,k).le.0.) then
            embcr1re(j,k)=0.
          else
            embcr1re(j,k)=EXTRAPR1(em,nrm1,j,k)
          end if
        enddo
      enddo
c
      call fft_tp (embcr0re,embcr0(1,1,1),embcr0(1,1,2),1)
      call fft_tp (embcr1re,embcr1(1,1,1),embcr1(1,1,2),1)
c
      return
      end
c#######################################################################
      subroutine setbcep (eppr,eppi)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on the Alfven wave energy density ep.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real eppr(nr,nt,npd3),eppi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          eppr( 1,j,k)=2.*epbcr0(j,k,1)-eppr(   2,j,k)
          eppi( 1,j,k)=2.*epbcr0(j,k,1)-eppi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        do j=2,ntm1
          eppr(nr,j,k)=2.*epbcr1(j,k,1)-eppr(nrm1,j,k)
          eppi(nr,j,k)=2.*epbcr1(j,k,2)-eppi(nrm1,j,k)
        enddo
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nr
            eppr(i, 1,k)=eppr(i,   2,k)
            eppr(i,nt,k)=eppr(i,ntm1,k)
            eppi(i, 1,k)=eppi(i,   2,k)
            eppi(i,nt,k)=eppi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nr
            eppr(i, 1,k)=-eppr(i,   2,k)
            eppr(i,nt,k)=-eppr(i,ntm1,k)
            eppi(i, 1,k)=-eppi(i,   2,k)
            eppi(i,nt,k)=-eppi(i,ntm1,k)
          enddo
        else
          do i=1,nr
            eppr(i, 1,k)=-eppr(i,   2,k)
            eppr(i,nt,k)=-eppr(i,ntm1,k)
            eppi(i, 1,k)=-eppi(i,   2,k)
            eppi(i,nt,k)=-eppi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
      subroutine setbcem (empr,empi)
c
c-----------------------------------------------------------------------
c
c ****** Set boundary conditions on the Alfven wave energy density em.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real empr(nr,nt,npd3),empi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      do k=1,npd3
c
        m=k-1
c
c ****** Inner radial boundary.
c
        do j=2,ntm1
          empr( 1,j,k)=2.*embcr0(j,k,1)-empr(   2,j,k)
          empi( 1,j,k)=2.*embcr0(j,k,1)-empi(   2,j,k)
        enddo
c
c ****** Outer radial boundary.
c
        do j=2,ntm1
          empr(nr,j,k)=2.*embcr1(j,k,1)-empr(nrm1,j,k)
          empi(nr,j,k)=2.*embcr1(j,k,2)-empi(nrm1,j,k)
        enddo
c
c ****** Theta boundaries.
c
        if (m.eq.0) then
          do i=1,nr
            empr(i, 1,k)=empr(i,   2,k)
            empr(i,nt,k)=empr(i,ntm1,k)
            empi(i, 1,k)=empi(i,   2,k)
            empi(i,nt,k)=empi(i,ntm1,k)
          enddo
        else if (m.eq.1) then
          do i=1,nr
            empr(i, 1,k)=-empr(i,   2,k)
            empr(i,nt,k)=-empr(i,ntm1,k)
            empi(i, 1,k)=-empi(i,   2,k)
            empi(i,nt,k)=-empi(i,ntm1,k)
          enddo
        else
          do i=1,nr
            empr(i, 1,k)=-empr(i,   2,k)
            empr(i,nt,k)=-empr(i,ntm1,k)
            empi(i, 1,k)=-empi(i,   2,k)
            empi(i,nt,k)=-empi(i,ntm1,k)
          enddo
        end if
c
      enddo
c
      return
      end
c#######################################################################
      subroutine wave1
c
c-----------------------------------------------------------------------
c
c ****** Initialize an Alfven test wave.
c
c-----------------------------------------------------------------------
c
c ****** This routine initializes an m=0 torsional Alfven wave
c ****** with perturbed v-theta and b-theta.
c
c-----------------------------------------------------------------------
c
c ****** The solution is given by:
c
c           A-phi = EPSWAVE*[A*j_sub_l(k*r)+B*y_sub_l(k*r)]
c                          *P_sub_l_sup_1(theta)
c                          *cos(omega*t)
c
c ****** where j_sub_l and y_sub_l are spherical Bessel functions,
c ****** and P_sub_l_sup_1 is the associated Legendre function,
c ****** and k = omega/va.
c
c ****** The theta mode-number l is given by LWAVE.
c ****** The radial mode-number is given by NWAVE.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      common /ftawcom/ ltaw,x0taw,x1taw
c
c-----------------------------------------------------------------------
c
      g0=0.
      br00=1.
      rho00=1.
      va00=br00/sqrt(rho00)
c
      if (nwave.lt.1.or.lwave.lt.1) then
        write (*,*)
        write (*,*) '### ERROR in routine WAVE1:'
        write (*,*) '### NWAVE or LWAVE is invalid.'
        write (*,*) 'NWAVE = ',nwave
        write (*,*) 'LWAVE = ',lwave
        write (9,*)
        write (9,*) '### ERROR in routine WAVE1:'
        write (9,*) '### NWAVE or LWAVE is invalid.'
        write (9,*) 'NWAVE = ',nwave
        write (9,*) 'LWAVE = ',lwave
        call exit (1)
      end if
c
      do 110 j=1,nt
        do 100 i=1,nr
          rhr(i,j,1)=rho00*(r0/rh(i))**4
  100   continue
  110 continue
c
c ****** Compute the analytic wave frequency.
c
      ltaw=lwave
      x0taw=r0/va00
      x1taw=r1/va00
c
c ****** Use a conservative guess for the starting frequency
c ****** for the first root.
c
      omg0=pi*va00/rl
      omega0=.1*omg0
      domega=.01*omg0
      omegamx=10*nwave*omg0
c
c ****** Find the NWAVE-th root.
c
      do 150 i=1,nwave
        call nxtroot (omega0,domega,omega,omegamx,ierr)
        if (ierr.eq.0) then
          write (9,*)
          write (9,*) '### Comment from WAVE1: '
          write (9,*) '### Found a root for the Alfven wave.'
          write (9,*) 'Radial mode number N = ',i
          write (9,*) 'OMEGA = ',omega
        else
          write (*,*)
          write (*,*) '### ERROR in WAVE1: '
          write (*,*) '### Could not find the requested root.'
          write (*,*) 'NWAVE = ',nwave
          write (*,*) 'Number of roots found so far = ',i-1
          write (9,*)
          write (9,*) '### ERROR in WAVE1: '
          write (9,*) '### Could not find the requested root.'
          write (9,*) 'NWAVE = ',nwave
          write (9,*) 'Number of roots found so far = ',i-1
          call exit (1)
        end if
  150 continue
c
      write (9,*)
      write (9,*) '### Comment from WAVE1: '
      write (9,*) '### The normal mode frequency has been found.'
      write (9,*) 'LWAVE = ',lwave
      write (9,*) 'NWAVE = ',nwave
      write (9,*) 'OMEGA = ',omega
c
c ****** Get the coefficients of the j_sub_l and y_sub_l functions.
c
      fj0=sbj(lwave,omega*r0/va00)
      fy0=sby(lwave,omega*r0/va00)
c
      if (abs(fj0).gt.abs(fy0)) then
        coefj=-fy0/fj0
        coefy=1.
      else
        coefj=1.
        coefy=-fj0/fy0
      end if
c
c ****** Initialize the fields.
c ****** Note that v is defined a half-timestep ahead
c ****** of A, B and J.
c
      sdt=sin(.5*omega*dtime)
      do 210 j=1,nt
        fnt=pl1(lwave,th(j))
        do 200 i=1,nr
          arg=omega*rh(i)/va00
          fnr=coefj*sbj(lwave,arg)+coefy*sby(lwave,arg)
          apr(i,j,1)=epswave*fnr*fnt
  200   continue
  210 continue
      do 310 j=1,ntm1
        fnt=pl1(lwave,t(j))
        do 300 i=1,nr
          arg=omega*rh(i)/va00
          fnr=coefj*sbj(lwave,arg)+coefy*sby(lwave,arg)
          vtr(i,j,1)=epswave*omega*rh(i)**2*fnr*fnt*sdt/(r0**2*br00)
  300   continue
  310 continue
c
c ****** Set the B.C.s on v.
c
      bcr1type='fixed'
c
      call bcast (vrbcr0,0.,2*nt*npd3)
      call bcast (vtbcr0,0.,2*nt*npd3)
      call bcast (vpbcr0,0.,2*nt*npd3)
      call bcast (vrbcr1,0.,2*nt*npd3)
      call bcast (vtbcr1,0.,2*nt*npd3)
      call bcast (vpbcr1,0.,2*nt*npd3)
c
      vmag0=1.
c
c ****** Set the B.C.s on rho and p.
c
      call bcast (rhobcr0,0.,2*nt*npd3)
      call bcast (rhobcr1,0.,2*nt*npd3)
      call bcast (pbcr0  ,0.,2*nt*npd3)
      call bcast (pbcr1  ,0.,2*nt*npd3)
c
      do 400 j=2,ntm1
        rhobcr0(j,1,1)=rho00
        rhobcr1(j,1,1)=rho00*(r0/r1)**4
  400 continue
c
      return
      end
c#######################################################################
      subroutine wave2
c
c-----------------------------------------------------------------------
c
c ****** Initialize an Alfven test wave.
c
c-----------------------------------------------------------------------
c
c ****** This routine initializes an m=0 torsional Alfven wave
c ****** with perturbed v-phi and b-phi.
c
c-----------------------------------------------------------------------
c
c ****** The solution is given by:
c
c           A-theta = EPSWAVE*[sin(k*(r-R0))/r]*f(theta)
c                            *cos(omega*t)
c
c ****** where f(theta) is an arbitrary function of theta
c ****** (subject to f(0) = f(pi) = 0), and k = omega/va.
c
c ****** Currently, f = P_sub_l_sup_1 is used.
c
c ****** The theta mode-number l is given by LWAVE.
c ****** The radial mode-number is given by NWAVE.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      g0=0.
      br00=1.
      rho00=1.
      va00=br00/sqrt(rho00)
c
      if (nwave.lt.1.or.lwave.lt.1) then
        write (*,*)
        write (*,*) '### ERROR in routine WAVE2:'
        write (*,*) '### NWAVE or LWAVE is invalid.'
        write (*,*) 'NWAVE = ',nwave
        write (*,*) 'LWAVE = ',lwave
        write (9,*)
        write (9,*) '### ERROR in routine WAVE2:'
        write (9,*) '### NWAVE or LWAVE is invalid.'
        write (9,*) 'NWAVE = ',nwave
        write (9,*) 'LWAVE = ',lwave
        call exit (1)
      end if
c
      do 110 j=1,nt
        do 100 i=1,nr
          rhr(i,j,1)=rho00*(r0/rh(i))**4
  100   continue
  110 continue
c
c ****** Compute the analytic wave frequency.
c
      omega=nwave*pi*va00/rl
c
      write (9,*)
      write (9,*) '### Comment from WAVE2: '
      write (9,*) '### Normal mode frequency:'
      write (9,*) 'LWAVE = ',lwave
      write (9,*) 'NWAVE = ',nwave
      write (9,*) 'OMEGA = ',omega
c
c ****** Initialize the fields.
c ****** Note that v is defined a half-timestep ahead
c ****** of A, B and J.
c
      sdt=sin(.5*omega*dtime)
      do 210 j=1,ntm1
        fnt=pl1(lwave,t(j))
        do 200 i=1,nr
          fnr=sin(omega*(rh(i)-r0)/va00)/rh(i)
          atr(i,j,1)=epswave*fnr*fnt
  200   continue
  210 continue
      do 310 j=1,nt
        fnt=pl1(lwave,th(j))
        do 300 i=1,nr
          fnr=sin(omega*(rh(i)-r0)/va00)
          vpr(i,j,1)=-epswave*omega*rh(i)*fnr*fnt*sdt/(r0**2*br00)
  300   continue
  310 continue
c
c ****** Set the B.C.s on v.
c
      bcr1type='fixed'
c
      call bcast (vrbcr0,0.,2*nt*npd3)
      call bcast (vtbcr0,0.,2*nt*npd3)
      call bcast (vpbcr0,0.,2*nt*npd3)
      call bcast (vrbcr1,0.,2*nt*npd3)
      call bcast (vtbcr1,0.,2*nt*npd3)
      call bcast (vpbcr1,0.,2*nt*npd3)
c
      vmag0=1.
c
c ****** Set the B.C.s on rho and p.
c
      call bcast (rhobcr0,0.,2*nt*npd3)
      call bcast (rhobcr1,0.,2*nt*npd3)
      call bcast (pbcr0  ,0.,2*nt*npd3)
      call bcast (pbcr1  ,0.,2*nt*npd3)
c
      do 400 j=2,ntm1
        rhobcr0(j,1,1)=rho00
        rhobcr1(j,1,1)=rho00*(r0/r1)**4
  400 continue
c
      return
      end
c#######################################################################
      subroutine nxtroot (omega0,domega,omega,omegamx,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Find the next root of the test wave dispersion relation
c ****** OMEGA which is greater than or equal to OMEGA0.
c
c-----------------------------------------------------------------------
c
c ****** Upon successful completion (IERR=0), the root is
c ****** returned in OMEGA.  OMEGA0 is reset to the frequency
c ****** at which the search for more roots can be resumed
c ****** by calling NXTROOT again.
c
c ****** OMEGAMX is the maximum omega for the root search.
c
c ****** FTAW is the function of omega which needs to be zeroed.
c
c-----------------------------------------------------------------------
c
      external ftaw
c
c-----------------------------------------------------------------------
c
c ****** March with increasing omega until a root is located.
c
      ierr=1
c
      omega=omega0
      ffm=ftaw(omega)
c
  100 continue
      w0=omega
      omega=omega+domega
      if (omega.gt.omegamx) return
      w1=omega
      ffp=ftaw(omega)
c
      if (ffp*ffm.le.0.) then
c
c ****** Find the root.
c
        omega=rfsolv(w0,w1,ftaw,ierr)
        if (ierr.ne.0) return
        omega0=w1
        return
      end if
      ffm=ffp
      go to 100
c
      end
c#######################################################################
      function ftaw (omega)
c
c-----------------------------------------------------------------------
c
c ****** Dispersion relation for a spherical torsional Alfven wave
c ****** with m=0 and finite v-theta.
c
c-----------------------------------------------------------------------
c
      common /ftawcom/ l,x0,x1
c
c-----------------------------------------------------------------------
c
      w0=omega*x0
      w1=omega*x1
      fj0=sbj(l,w0)
      fy0=sby(l,w0)
      fj1=sbj(l,w1)
      fy1=sby(l,w1)
c
      ftaw=fj0*fy1-fj1*fy0
c
      return
      end
c#######################################################################
      function rfsolv (x0,x1,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Solve the equation F(X) = 0. using regula falsi.
c
c-----------------------------------------------------------------------
c
c ****** The function F should be designed to be called
c ****** with one real argument, with a solution FUNC(X)=0.
c
c ****** X0 and X1 are the limits of the interval which
c ****** brackets a root.
c
c ****** The fractional error in X is required to be smaller
c ****** than EPS.
c
c ****** NMAX is the maximum number of iterations.
c
c-----------------------------------------------------------------------
c
      external f
c
c-----------------------------------------------------------------------
c
      parameter (nmax=100)
      parameter (eps=1.e-9)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      w0=x0
      w1=x1
c
      w2old=w0
      f0=f(w0)
      f1=f(w1)
      n=0
c
  100 continue
c
      n=n+1
c
      if (f0.eq.0.) then
        w2=w0
        f2=f0
        go to 200
      end if
c
      if (f1.eq.0.) then
        w2=w1
        f2=f1
        go to 200
      end if
c
      if (f0*f1.gt.0.) then
        ierr=1
        return
      end if
c
      w2=(f1*w0-f0*w1)/(f1-f0)
      f2=f(w2)
c
c ****** Check if converged.
c
      err=(w2-w2old)
      if (w2.ne.0.) err=err/w2
c
      if (abs(err).le.eps) go to 200
c
      if (n.gt.nmax) then
        ierr=1
        rfsolv=w2
        return
      end if
c
      w2old=w2
      if (f2*f0.lt.0.) then
        f1=f2
        w1=w2
      else
        f0=f2
        w0=w2
      end if
c
      go to 100
c
  200 continue
c
c ****** Converged.
c
      rfsolv=w2
c
      return
      end
c#######################################################################
      function sbj (l,x)
c
c-----------------------------------------------------------------------
c
c ****** Evaluate the spherical Bessel function of the first kind,
c ****** j_sub_l(x).
c
c-----------------------------------------------------------------------
c
c ****** The accuracy of this routine for small x is not
c ****** very good for a 32-bit machine, but is adequate on a
c ****** 64-bit machine.
c ****** The small-argument expansion should really be used.
c
c-----------------------------------------------------------------------
c
      if (l.lt.0) then
        write (*,*)
        write (*,*) '### ERROR in SBJ:'
        write (*,*) '### Invalid order specified.'
        write (*,*) '[Spherical Bessel function j_sub_l.]'
        write (*,*) 'L = ',l
        write (9,*)
        write (9,*) '### ERROR in SBJ:'
        write (9,*) '### Invalid order specified.'
        write (9,*) '[Spherical Bessel function j_sub_l.]'
        write (9,*) 'L = ',l
        call exit (1)
      end if
c
      if (x.eq.0.) then
        if (l.eq.0) then
          sbj=1.
        else
          sbj=0.
        end if
        return
      end if
c
      sx=sin(x)
      cx=cos(x)
c
      f0=1./x
      f1=1./x**2
      g0=0.
      g1=1./x
c
c ****** Use the recurrence for l.gt.1.
c
      if (l.eq.0) then
        sbj=f0*sx
      else if (l.eq.1) then
        sbj=f1*sx-g1*cx
      else
        flm1=f0
        fl=f1
        glm1=g0
        gl=g1
        do 100 i=1,l-1
          flp1=(2*i+1.)*fl/x-flm1
          glp1=(2*i+1.)*gl/x-glm1
          flm1=fl
          fl=flp1
          glm1=gl
          gl=glp1
  100   continue
        sbj=flp1*sx-glp1*cx
      end if
c
      return
      end
c#######################################################################
      function sby (l,x)
c
c-----------------------------------------------------------------------
c
c ****** Evaluate the spherical Bessel function of the second kind,
c ****** y_sub_l(x).
c
c-----------------------------------------------------------------------
c
      if (l.lt.0) then
        write (*,*)
        write (*,*) '### ERROR in SBY:'
        write (*,*) '### Invalid order specified.'
        write (*,*) '[Spherical Bessel function y_sub_l.]'
        write (*,*) 'L = ',l
        write (9,*)
        write (9,*) '### ERROR in SBY:'
        write (9,*) '### Invalid order specified.'
        write (9,*) '[Spherical Bessel function y_sub_l.]'
        write (9,*) 'L = ',l
        call exit (1)
      end if
c
      if (x.eq.0.) then
        write (*,*)
        write (*,*) '### ERROR in SBY:'
        write (*,*) '### Invalid argument specified.'
        write (*,*) '[Spherical Bessel function y_sub_l.]'
        write (*,*) 'X = ',x
        write (9,*)
        write (9,*) '### ERROR in SBY:'
        write (9,*) '### Invalid argument specified.'
        write (9,*) '[Spherical Bessel function y_sub_l.]'
        write (9,*) 'X = ',x
        call exit (1)
      end if
c
      sx=sin(x)
      cx=cos(x)
c
      f0=1./x
      f1=1./x**2
      g0=0.
      g1=1./x
c
c ****** Use the recurrence for l.gt.1.
c
      if (l.eq.0) then
        sby=-f0*cx
      else if (l.eq.1) then
        sby=-g1*sx-f1*cx
      else
        flm1=f0
        fl=f1
        glm1=g0
        gl=g1
        do 100 i=1,l-1
          flp1=(2*i+1.)*fl/x-flm1
          glp1=(2*i+1.)*gl/x-glm1
          flm1=fl
          fl=flp1
          glm1=gl
          gl=glp1
  100   continue
        sby=-glp1*sx-flp1*cx
      end if
c
      return
      end
c#######################################################################
      function pl1 (l,theta)
c
c-----------------------------------------------------------------------
c
c ****** Evaluate the associated Legendre function
c ****** P_sub_l_sup_1(cos(theta)).
c
c-----------------------------------------------------------------------
c
      if (l.lt.1) then
        write (*,*)
        write (*,*) '### ERROR in PL1:'
        write (*,*) '### Invalid degree specified.'
        write (*,*) '[Associated Legendre function (m=1).]'
        write (*,*) 'L = ',l
        write (9,*)
        write (9,*) '### ERROR in PL1:'
        write (9,*) '### Invalid degree specified.'
        write (9,*) '[Associated Legendre function (m=1).]'
        write (9,*) 'L = ',l
        call exit (1)
      end if
c
c ****** Use the recurrence relation for L.ge.3.
c
      x=cos(theta)
      st=sin(theta)
c
      p11=st
      p21=3.*x*st
c
      if (l.eq.1) then
        pl1=p11
      else if (l.eq.2) then
        pl1=p21
      else
        plm1=p11
        pl=p21
        do 100 i=2,l-1
          plp1=((2*i+1.)*x*pl-(i+1.)*plm1)/float(i)
          plm1=pl
          pl=plp1
  100   continue
        pl1=plp1
      end if
c
      return
      end
c#######################################################################
      subroutine fft (re,compr,compi,mode)
c
c-----------------------------------------------------------------------
c
c ****** Take the fast Fourier transform.
c
c ****** Call with MODE=1 for a forward transform, and
c ****** MODE.ne.1 for an inverse transform.
c
c-----------------------------------------------------------------------
c
c ****** CRAY-2 version.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      common /workfft/ workr(65*(np+1)),worki(65*(np+1))
c
c-----------------------------------------------------------------------
c
c ****** For a spherically axisymmetric case, just copy the data.
c
      if (np.eq.1) then
        if (mode.eq.1) then
          call scopy (nrt,re,1,compr,1)
          call bcast (compi,0.,nrt)
        else
          call scopy (nrt,compr,1,re,1)
        end if
      else
        call fft2d (re,compr,compi,workr,worki,nrt,mp,mode)
      end if
c
      return
      end
c#######################################################################
      subroutine fft_tp (re,compr,compi,mode)
c
c-----------------------------------------------------------------------
c
c ****** Take the fast Fourier transform of a (theta,phi) array.
c
c ****** Call with MODE=1 for a forward transform, and
c ****** MODE.ne.1 for an inverse transform.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      call fft_tp1 (nt,re,compr,compi,mode)
c
      return
      end
c#######################################################################
      subroutine fft_tp1 (n,re,compr,compi,mode)
c
c-----------------------------------------------------------------------
c
c ****** Take the fast Fourier transform of a (theta,phi) array
c ****** with N theta points.
c
c ****** Call with MODE=1 for a forward transform, and
c ****** MODE.ne.1 for an inverse transform.
c
c-----------------------------------------------------------------------
c
c ****** CRAY-2 version.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      common /workfft/ workr(65*(np+1)),worki(65*(np+1))
c
c-----------------------------------------------------------------------
c
c ****** For a spherically axisymmetric case, just copy the data.
c
      if (np.eq.1) then
        if (mode.eq.1) then
          call scopy (n,re,1,compr,1)
          call bcast (compi,0.,n)
        else
          call scopy (n,compr,1,re,1)
        end if
      else
        call fft2d (re,compr,compi,workr,worki,n,mp,mode)
      end if
c
      return
      end
c#######################################################################
      subroutine fft2d (re,compr,compi,workr,worki,nx,my,iflag)
c
c-----------------------------------------------------------------------
c
c ****** Do a forward or inverse FFT.
c
c ****** This version does a full transform (without de-aliasing).
c
c-----------------------------------------------------------------------
c
c ****** This routine uses the subroutine VCFTX to do the actual
c ****** FFT.  (This can call the binary VCFT routine or an
c ****** equivalent FORTRAN routine.)
c
c-----------------------------------------------------------------------
c
      real re(nx,*),compr(nx,*),compi(nx,*)
      real workr(65,*),worki(65,*)
c
c-----------------------------------------------------------------------
c
c     Performs the FFT of a two-dimensional array over
c     the second dimension, for all points in the first
c     dimension.  The operations over this first dimension are
c     vectorized.
c
c     The definition of the arguments is as follows:
c
c       IFLAG:   An integer flag which determines the direction of
c                the FFT.  Fourier analysis is performed when IFLAG=1,
c                and Fourier synthesis (i.e., back to real space) is
c                performed when IFLAG.ne.1.
c
c       NX:      The number of points in the non-FFT direction
c                (over which the vectorization is performed).
c
c       MY:      The power-of-two which gives the number of points
c                in the FFT direction.  Thus the number of points in
c                the FFT direction is NY=2**MY.  Note that MY needs
c                to be between 2 and 10, inclusive.
c
c       RE:      Real array dimensioned NX by NY which contains the
c                data in real space. This array is used as input
c                when IFLAG=1, and is output when IFLAG.ne.1.
c
c       COMPR:   Real part of complex array dimensioned NX by NY/2+1
c                which contains the non-dealiased Fourier modes.
c                This array is used as input when IFLAG.ne.1, and is
c                output when IFLAG=1.
c
c       COMPI:   Imaginary part of complex array; dimensioned
c                like COMPR.
c
c       WORKR:   Work array of length 65*NY words.
c
c       WORKI:   Work array of length 65*NY words.
c
c     Note that RE is not overwritten in the call with IFLAG=1,
c     and similarly, COMP is not overwritten in the call with
c     IFLAG.ne.1.
c
c     The operation of this routine can be summarized by:
c
c                           NY
c       COMP(I,M) = 1/NY * SUM RE(I,J)*EXP[-2*pi*i*(J-1)*(M-1)/NY]
c                          J=1
c
c                         for I=1,2,...,NX and M=1,2,...,NY/2+1
c                         when IFLAG=1, and
c
c                  NY
c       RE(I,J) = SUM COMP(I,M)*EXP[2*pi*i*(J-1)*(M-1)/NY]
c                 M=1
c
c                         for I=1,2,...,NX and J=1,2,...,NY
c                         when IFLAG.ne.1, where the elements COMP(I,M)
c                         for M=NY/2+2,...,NY are not stored in array
c                         COMP, but obey COMP(I,M)=conj(COMP(I,NY-M+2)).
c
c-----------------------------------------------------------------------
c
c ****** IMPORTANT: when IFLAG.ne.1, array COMP should have zero
c ****** imaginary parts for M=1 and M=NY/2+1 for all I, on input,
c ****** as is physically required.
c
c-----------------------------------------------------------------------
c
      ny=2**my
      nyd2=ny/2+1
c
      if (iflag.eq.1) then
c
c ****** FORWARD TRANSFORM.
c
        do i=1,nx,128
c
          n=min0(128,nx-i+1)
          n2=n/2
          n2p=n-n2
c
c ****** Pack two x-lines at a time.
c
          do j=1,ny
            do k=1,n2p
              workr(k,j)=re(i-1+k,j)
              worki(k,j)=re(i+n-k,j)
            enddo
          enddo
c
          call vcftx (my,1,workr,worki,65,1,n2p)
c
c ****** Unpack the two x-lines.
c
CDIR$ IVDEP
          do k=1,n2p
            compr(i-1+k,1)=workr(k,1)
            compi(i-1+k,1)=0.
            compr(i+n-k,1)=worki(k,1)
            compi(i+n-k,1)=0.
          enddo
c
          do m=2,nyd2
CDIR$ IVDEP
            do k=1,n2p
              compr(i-1+k,m)= .5*(workr(k,m)+workr(k,ny+2-m))
              compi(i-1+k,m)= .5*(worki(k,m)-worki(k,ny+2-m))
              compr(i+n-k,m)= .5*(worki(k,m)+worki(k,ny+2-m))
              compi(i+n-k,m)=-.5*(workr(k,m)-workr(k,ny+2-m))
            enddo
          enddo
c
        enddo
c
      else
c
c ****** INVERSE TRANSFORM.
c
        do i=1,nx,128
c
          n=min0(128,nx-i+1)
          n2=n/2
          n2p=n-n2
c
c ****** Pack two x-lines at a time.
c
c ****** Assume imaginary parts are zero for m=0 and m=NY/2.
c
          do k=1,n2p
            workr(k,   1)=compr(i-1+k,   1)
            worki(k,   1)=compr(i+n-k,   1)
            workr(k,nyd2)=compr(i-1+k,nyd2)
            worki(k,nyd2)=compr(i+n-k,nyd2)
          enddo
c
c ****** Positive m.
c
          do m=2,nyd2-1
CDIR$ IVDEP
            do k=1,n2p
              workr(k,m)=compr(i-1+k,m)-compi(i+n-k,m)
              worki(k,m)=compi(i-1+k,m)+compr(i+n-k,m)
            enddo
          enddo
c
c ****** Negative m.
c
          do m=nyd2+1,ny
CDIR$ IVDEP
            do k=1,n2p
              workr(k,m)= compr(i-1+k,ny+2-m)+compi(i+n-k,ny+2-m)
              worki(k,m)=-compi(i-1+k,ny+2-m)+compr(i+n-k,ny+2-m)
            enddo
          enddo
c
          call vcftx (my,0,workr,worki,65,1,n2p)
c
          do j=1,ny
CDIR$ IVDEP
            do k=1,n2p
              re(i-1+k,j)=workr(k,j)
              re(i+n-k,j)=worki(k,j)
            enddo
          enddo
c
        enddo
c
      end if
c
      return
      end
c#######################################################################
      subroutine vcftx (m,mode,ar,ai,incf,incv,nv)
c
c     Call the CAL VCFT for M.ge.4, and the FORTRAN version for
c     other M's.
c
cFORTRAN      if (m.ge.4) then
cFORTRAN        call vcft (m,mode,ar,ai,incf,incv,nv)
cFORTRAN      else
        call vcftf (m,mode,ar,ai,incf,incv,nv)
cFORTRAN      end if
c
      return
      end
c#######################################################################
      subroutine vcftf (m,mode,ar,ai,incf,incv,nv)
      dimension ar(incf,*),ai(incf,*)
c
c     FORTRAN Cray-2 VCFT look-alike.
c
      n=2**m
      isign=1-2*mode
c
      call cpftv (ar,ai,n,incf,nv,incv,isign)
c
c     Normalization.
c
      if (mode.eq.1) then
        xnrm=1./n
        do 200 if=1,n
cdir$ ivdep
        do 100 iv=1,1+(nv-1)*incv,incv
        ar(iv,if)=ar(iv,if)*xnrm
        ai(iv,if)=ai(iv,if)*xnrm
  100   continue
  200   continue
      end if
c
      return
      end
c#######################################################################
      subroutine cpftv (r,i,n,incf,nv,incv,isign)
      parameter (log2nx=15)
      real r(incf,*),i(incf,*)
      integer span,rc
      double precision qt,qq
      real sines(log2nx),i0,i1
      save sines
c
c     This subroutine performs an FFT in one direction of a
c     two-dimensional array, for each row in the non-FFT direction.
c     The loops are vectorized in this perpendicular direction.
c     This routine is a modified version of Langdon's CPFT
c     FFT routine (which is based on Singleton's FFT algorithm).
c     This routine is unnormalized, in the sense that if it is
c     called twice in succession (with the same array), first with
c     ISIGN=1 and then with ISIGN=-1, the array is multiplied by N.
c
c     The meaning of the arguments is as follows:
c     R     = real array containing the real parts of the complex
c             data to be Fourier-transformed,
c     I     = real array containing the imaginary parts of the complex
c             data to be Fourier-transformed,
c     N     = length of the FFT (must be a power of 2),
c     INCF  = increment between successive elements of R (and I)
c             in the FFT direction when it is regarded
c             as a real one-dimensional array,
c     NV    = number of rows over which to perform the FFT
c             (i.e., in the vector direction),
c     INCV  = increment between successive elements of R (and I)
c             in the perpendicular direction when it is regarded
c             as a real one-dimensional array,
c     ISIGN = sign to be used in the exponential of the FFT
c             (use +1 or -1).
c
c#### Modified by Z. Mikic, SAIC, La Jolla, August 18, 1985.
c
      if(sines(1).eq.1.) go to 1
      sines(1)=1.
      qt=1.
      qt=datan(qt)
      do 2 is=2,log2nx
      qq=dsin(qt)
      sines(is)=qq
    2 qt=qt*.5
    1 continue
c
      if(n.eq.1) return
      nvl=1+(nv-1)*incv
c*ZM* For 2-d arrays R and I, set INC=1.
      inc=1
      sgn=isign
      ninc=n*inc
      span=ninc
      it=n/2
      do 1000 is=1,log2nx
c... (2000=recur)
      if(it.eq.1) go to 2000
 1000 it=it/2
c
c  if truncated rather than rounded arithmetic is used,
c  singleton's magnitude correcton should be applied to cos and sin.
 1500 t=sin+(s*cos-c*sin)
      cos=cos-(c*cos+s*sin)
      sin=t
c... (3000=repl)
 3000 k1=k0+span
c*ZM* Vector loops introduced all end with 01.
cdir$ ivdep
      do 101 iv=1,nvl,incv
      r0=r(iv,1+k0)
      r1=r(iv,1+k1)
      i0=i(iv,1+k0)
      i1=i(iv,1+k1)
      r(iv,1+k0)=r0+r1
      i(iv,1+k0)=i0+i1
      r0=r0-r1
      i0=i0-i1
      r(iv,1+k1)=cos*r0-sin*i0
      i(iv,1+k1)=sin*r0+cos*i0
  101 continue
      k0=k1+span
      if(k0.lt.ninc) go to 3000
      k1=k0-ninc
      cos=-cos
      k0=span-k1
      if(k1.lt.k0) go to 3000
      k0=k0+inc
      k1=span-k0
      if(k0.lt.k1) go to 1500
 2000 continue
      span=span/2
      k0=0
c... (4000=zero)
 4000 k1=k0+span
cdir$ ivdep
      do 201 iv=1,nvl,incv
      r0=r(iv,1+k0)
      r1=r(iv,1+k1)
      i0=i(iv,1+k0)
      i1=i(iv,1+k1)
      r(iv,1+k0)=r0+r1
      i(iv,1+k0)=i0+i1
      r(iv,1+k1)=r0-r1
      i(iv,1+k1)=i0-i1
  201 continue
      k0=k1+span
      if(k0.lt.ninc) go to 4000
      if(span.eq.inc) go to 5000
      k0=span/2
 4500 k1=k0+span
cdir$ ivdep
      do 301 iv=1,nvl,incv
      r0=r(iv,1+k0)
      r1=r(iv,1+k1)
      i0=i(iv,1+k0)
      i1=i(iv,1+k1)
      r(iv,1+k0)=r0+r1
      i(iv,1+k0)=i0+i1
      r(iv,1+k1)=(i1-i0)*sgn
      i(iv,1+k1)=(r0-r1)*sgn
  301 continue
      k0=k1+span
      if(k0.lt.ninc) go to 4500
      k1=inc+inc
      if(span.eq.k1) go to 2000
      c=2.*sines(is)**2
      is=is-1
      sin=sign(sines(is),sgn)
      s=sin
      cos=1.-c
      k0=inc
      go to 3000
c
 5000 n1=ninc-inc
      n2=ninc/2
      ij=0
      ji=0
      rc=0
      if(n2.eq.inc) return
      go to 5020
c... (5010=even)
 5010 ij=n1-ij
      ji=n1-ji
cdir$ ivdep
      do 401 iv=1,nvl,incv
      t=r(iv,1+ij)
      r(iv,1+ij)=r(iv,1+ji)
      r(iv,1+ji)=t
      t=i(iv,1+ij)
      i(iv,1+ij)=i(iv,1+ji)
      i(iv,1+ji)=t
  401 continue
      if(ij.gt.n2) go to 5010
c... (5020=odd)
 5020 ij=ij+inc
      ji=ji+n2
cdir$ ivdep
      do 501 iv=1,nvl,incv
      t=r(iv,1+ij)
      r(iv,1+ij)=r(iv,1+ji)
      r(iv,1+ji)=t
      t=i(iv,1+ij)
      i(iv,1+ij)=i(iv,1+ji)
      i(iv,1+ji)=t
  501 continue
      it=n2
c... (6000=incrv)
 6000 it=it/2
      rc=rc-it
      if(rc.ge.0) go to 6000
      rc=rc+2*it
      ji=rc
      ij=ij+inc
      if(ij.le.ji) go to 5010
      if(ij.lt.n2) go to 5020
      return
      end
c#######################################################################
      subroutine solve (x,rhs,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Solve the implicit equations iteratively.
c
c-----------------------------------------------------------------------
c
c ****** Return IERR=0 if the iteration converges; otherwise,
c ****** IERR is set to a nonzero value.
c
c ****** X is the initial guess at the solution.
c ****** RHS is the right-hand side.
c
c-----------------------------------------------------------------------
c
      use cgcom2
c
c-----------------------------------------------------------------------
c
c ****** Find the preconditioning matrix.
c
      call precond
c
c ****** Solve the equations using the CG method.
c
      call cgsolv (x,rhs,ierr)
c
c ****** Check for convergence.
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in SOLVE:'
        write (*,*) '### Field solver did not converge.'
        write (*,*) 'IERR = ',ierr
      end if
c
c ****** Update the cumulative iteration counters.
c
      if (solvtype.eq.'si') then
        nsolves_si=nsolves_si+1
        ntotal_si=ntotal_si+ncg
      else if (solvtype.eq.'visc') then
        nsolves_visc=nsolves_visc+1
        ntotal_visc=ntotal_visc+ncg
      else if (solvtype.eq.'pot') then
        nsolves_pot=nsolves_pot+1
        ntotal_pot=ntotal_pot+ncg
      else if (solvtype.eq.'a') then
        nsolves_a=nsolves_a+1
        ntotal_a=ntotal_a+ncg
      else if (solvtype.eq.'t') then
        nsolves_t=nsolves_t+1
        ntotal_t=ntotal_t+ncg
      end if
c
      return
      end
c#######################################################################
      subroutine precond
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine selects the appropriate routine,
c ****** depending on which equation is being solved.
c
c-----------------------------------------------------------------------
c
      use solver_types
c
c-----------------------------------------------------------------------
c
      select case (solve_type)
      case (ST_AMATP0)
        call precond_amatp0
      case (ST_AMATT)
        call precond_amatt
      case (ST_VMATRT0)
        call precond_vmatrt0
      case (ST_VMATP0)
        call precond_vmatp0
      case (ST_VMATRTP)
        call precond_vmatrtp
      case (ST_A)
      case (ST_T)
      case default
        write (9,*)
        write (9,*) '### ERROR in PRECOND:'
        write (9,*) '### Invalid SOLVE_TYPE.'
        write (9,*) '### SOLVE_TYPE = ',solve_type
        write (9,*) '### [This is an internal error.]'
        write (*,*)
        write (*,*) '### ERROR in PRECOND:'
        write (*,*) '### Invalid SOLVE_TYPE.'
        write (*,*) '### SOLVE_TYPE = ',solve_type
        write (*,*) '### [This is an internal error.]'
        call exit (1)
      end select
c
      return
      end
c#######################################################################
      subroutine precond_amatp0
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for Ap in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
c ****** When IFPREC=0, do not perform any preconditioning.
c ****** When IFPREC=1, perform diagonal scaling preconditioning.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) then
c
c ****** No preconditioning.
c
        return
c
      else if (ifprec.eq.1) then
c
c-----------------------------------------------------------------------
c ****** Diagonal scaling (IFPREC=1).
c-----------------------------------------------------------------------
c
        d(2:nrm1,2:ntm1)=1./a00(2:nrm1,2:ntm1)
c
      else
c
c ****** Error: Invalid choice for IFPREC.
c
        write (*,*)
        write (*,*) '### ERROR in PRECOND_AMATP0:'
        write (*,*) '### Invalid value for IFPREC.'
        write (*,*) 'IFPREC = ',ifprec
        write (9,*)
        write (9,*) '### ERROR in PRECOND_AMATP0:'
        write (9,*) '### Invalid value for IFPREC.'
        write (9,*) 'IFPREC = ',ifprec
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine precond_amatt
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for At in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
c ****** When IFPREC=0, do not perform any preconditioning.
c ****** When IFPREC=1, perform diagonal scaling preconditioning.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) then
c
c ****** No preconditioning.
c
        return
c
      else if (ifprec.eq.1) then
c
c-----------------------------------------------------------------------
c ****** Diagonal scaling (IFPREC=1).
c-----------------------------------------------------------------------
c
        d(2:nrm1,2:ntm2)=1./a00(2:nrm1,2:ntm2)
c
      else
c
c ****** Error: Invalid choice for IFPREC.
c
        write (*,*)
        write (*,*) '### ERROR in PRECOND_AMATT:'
        write (*,*) '### Invalid value for IFPREC.'
        write (*,*) 'IFPREC = ',ifprec
        write (9,*)
        write (9,*) '### ERROR in PRECOND_AMATT:'
        write (9,*) '### Invalid value for IFPREC.'
        write (9,*) 'IFPREC = ',ifprec
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine precond_vmatrt0
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equations for (vr,vt) in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
c ****** When IFPREC=0, do not perform any preconditioning.
c ****** When IFPREC=1, perform diagonal scaling preconditioning.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) then
c
c ****** No preconditioning.
c
        return
c
      else if (ifprec.eq.1) then
c
c-----------------------------------------------------------------------
c ****** Diagonal scaling (IFPREC=1).
c-----------------------------------------------------------------------
c
        drr(2:nrm1,2:ntm1)=1./arr00(2:nrm1,2:ntm1)
        dtt(2:nrm1,2:ntm1)=1./att00(2:nrm1,2:ntm1)
c
      else
c
c ****** Error: Invalid choice for IFPREC.
c
        write (*,*)
        write (*,*) '### ERROR in PRECOND_VMATRT0:'
        write (*,*) '### Invalid value for IFPREC.'
        write (*,*) 'IFPREC = ',ifprec
        write (9,*)
        write (9,*) '### ERROR in PRECOND_VMATRT0:'
        write (9,*) '### Invalid value for IFPREC.'
        write (9,*) 'IFPREC = ',ifprec
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine precond_vmatp0
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for vp in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
c ****** When IFPREC=0, do not perform any preconditioning.
c ****** When IFPREC=1, perform diagonal scaling preconditioning.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) then
c
c ****** No preconditioning.
c
        return
c
      else if (ifprec.eq.1) then
c
c-----------------------------------------------------------------------
c ****** Diagonal scaling (IFPREC=1).
c-----------------------------------------------------------------------
c
        dpp(2:nrm1,2:ntm1)=1./app00(2:nrm1,2:ntm1)
c
      else
c
c ****** Error: Invalid choice for IFPREC.
c
        write (*,*)
        write (*,*) '### ERROR in PRECOND_VMATP0:'
        write (*,*) '### Invalid value for IFPREC.'
        write (*,*) 'IFPREC = ',ifprec
        write (9,*)
        write (9,*) '### ERROR in PRECOND_VMATP0:'
        write (9,*) '### Invalid value for IFPREC.'
        write (9,*) 'IFPREC = ',ifprec
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine precond_vmatrtp
c
c-----------------------------------------------------------------------
c
c ****** Compute the preconditioning matrix diagonal.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for (vr,vt,vp)
c ****** in the semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
c ****** When IFPREC=0, do not perform any preconditioning.
c ****** When IFPREC=1, perform diagonal scaling preconditioning.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) then
c
c ****** No preconditioning.
c
        return
c
      else if (ifprec.eq.1) then
c
c-----------------------------------------------------------------------
c ****** Diagonal scaling (IFPREC=1).
c-----------------------------------------------------------------------
c
        drr(2:nrm1,2:ntm1)=1./arr00(2:nrm1,2:ntm1)
        dtt(2:nrm1,2:ntm1)=1./att00(2:nrm1,2:ntm1)
        dpp(2:nrm1,2:ntm1)=1./app00(2:nrm1,2:ntm1)
c
      else
c
c ****** Error: Invalid choice for IFPREC.
c
        write (*,*)
        write (*,*) '### ERROR in PRECOND_VMATRTP:'
        write (*,*) '### Invalid value for IFPREC.'
        write (*,*) 'IFPREC = ',ifprec
        write (9,*)
        write (9,*) '### ERROR in PRECOND_VMATRTP:'
        write (9,*) '### Invalid value for IFPREC.'
        write (9,*) 'IFPREC = ',ifprec
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine cgsolv (x,r,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Solve the linear system:
c
c            A * x = b
c
c ****** using the classical Conjugate Gradient method for symmetric
c ****** and positive-definite matrices.
c
c-----------------------------------------------------------------------
c
c ****** On input, X contains a guess at the solution, and
c ****** R contains the right-hand side, b.
c
c ****** On exit, X contains an estimate to the solution, and
c ****** R contains the residual (b-Ax).
c
c ****** IERR=0 indicates that the solution converged to the
c ****** requested accuracy.  Other values indicate that the
c ****** iteration did not converge for the given maximum number
c ****** of iterations.
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom1
      use cgcom2
c
c-----------------------------------------------------------------------
c
      real x(ncgeq),r(ncgeq)
c
c-----------------------------------------------------------------------
c
c ****** Scratch space for CG iteration vectors.
c
      real p(ncgeq),ap(ncgeq)
c
c-----------------------------------------------------------------------
c
      ncg=0
c
c ****** Get the norm of the RHS.
c
      bnrm=sqrt(dot_product(r,r))
c
c ****** If the RHS is zero, return with a zero solution.
c
      if (bnrm.eq.0.) then
        x(:)=0.
        rnrm=0.
        ierr=0
        return
      end if
c
c-----------------------------------------------------------------------
c ****** Initialization.
c-----------------------------------------------------------------------
c
      call ax (ap,x)
      r(:)=r(:)-ap(:)
c
c ****** Find the initial error norm.
c
      call ernorm (ncgeq,r,ierr)
      if (ierr.ge.0) return
c
      p(:)=r(:)
      call linv (p)
      call uinv (p)
      top=dot_product(r,p)
c
c-----------------------------------------------------------------------
c ****** Main iteration loop.
c-----------------------------------------------------------------------
c
      do

        ncg=ncg+1
c
        call ax (ap,p)
        bot=dot_product(p,ap)
        alphai=top/bot
        x(:)=x(:)+alphai*p(:)
        r(:)=r(:)-alphai*ap(:)
c
c ****** Check for convergence.
c
        call ernorm (ncgeq,r,ierr)
        if (ierr.ge.0) exit
c
        ap(:)=r(:)
        call linv (ap)
        call uinv (ap)
        bot=top
        top=dot_product(r,ap)
        betai=top/bot
        p(:)=betai*p(:)+ap(:)
c
      enddo
c
      return
      end
c#######################################################################
      subroutine ernorm (nq,r,ierr)
c
c-----------------------------------------------------------------------
c
c ****** This subroutine checks if the iterative solver has
c ****** converged or if the maximum allowed number of iterations,
c ****** NCGMAX, has been exceeded.
c
c-----------------------------------------------------------------------
c
c ****** Convergence is deemed to have occurred when:
c ******
c ******     ||R||/||B|| .lt. EPSCG
c ******
c ****** where ||R|| is the norm of the (preconditioned)
c ****** residual, ||B|| is the norm of the (preconditioned)
c ****** RHS, and EPSCG is the specified convergence criterion.
c
c ****** Set IERR=0 if the error is below the error criterion
c ****** (i.e., the solution has converged).
c ****** Set IERR=-1 if the error does not yet meet the error
c ****** criterion and the number of iterations is less than NCGMAX.
c ****** Set IERR=1 if the maximum number of iterations has
c ****** been exceeded without convergence.
c
c-----------------------------------------------------------------------
c
      use cgcom2
c
c-----------------------------------------------------------------------
c
      real r(nq)
c
c-----------------------------------------------------------------------
c
      ierr=-1
c
      rnrm=sqrt(dot_product(r,r))
      epsn=rnrm/bnrm
c
      if (ncghist.gt.0) then
        if (mod(ncg,ncghist).eq.0) then
          if (ncg.eq.0) then
            write (9,*)
            write (9,*) '### Comment from ERNORM:'
            write (9,*) '### Convergence information:'
          end if
          write (9,100) ncg,epsn
  100     format (1x,'N = ',i6,' EPSN = ',1pe10.3)
        end if
      end if
c
c ****** Check for convergence.
c
      if (epsn.lt.epscg) then
        if (ncghist.gt.0) then
          write (9,*)
          write (9,*) '### Comment from ERNORM:'
          write (9,*) '### The CG solver has converged.'
          write (9,100) ncg,epsn
        end if
        ierr=0
      else if (ncg.ge.ncgmax) then
        write (*,*)
        write (*,*) '### ERROR in ERNORM:'
        write (*,*) '### Exceeded maximum number of iterations.'
        write (*,*) 'NCGMAX = ',ncgmax
        write (*,*) 'EPSN = ',epsn
        write (9,*)
        write (9,*) '### ERROR in ERNORM:'
        write (9,*) '### Exceeded maximum number of iterations.'
        write (9,*) 'NCGMAX = ',ncgmax
        write (9,*) 'EPSN = ',epsn
        ierr=1
      end if
c
      return
      end
c#######################################################################
      subroutine ax (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine selects the appropriate matrix,
c ****** depending on which equation is being solved.
c
c-----------------------------------------------------------------------
c
      use solver_types
c
c-----------------------------------------------------------------------
c
      select case (solve_type)
      case (ST_AMATP0)
        call ax_amatp0 (y,x)
      case (ST_AMATT)
        call ax_amatt (y,x)
      case (ST_VMATRT0)
        call ax_vmatrt0 (y,x)
      case (ST_VMATP0)
        call ax_vmatp0 (y,x)
      case (ST_VMATRTP)
        call ax_vmatrtp (y,x)
      case (ST_A)
        call ax_a (y,x)
      case (ST_T)
        call ax_t (y,x)
      case default
        write (9,*)
        write (9,*) '### ERROR in AX:'
        write (9,*) '### Invalid SOLVE_TYPE.'
        write (9,*) '### SOLVE_TYPE = ',solve_type
        write (9,*) '### [This is an internal error.]'
        write (*,*)
        write (*,*) '### ERROR in AX:'
        write (*,*) '### Invalid SOLVE_TYPE.'
        write (*,*) '### SOLVE_TYPE = ',solve_type
        write (*,*) '### [This is an internal error.]'
        call exit (1)
      end select
c
      return
      end
c#######################################################################
      subroutine ax_amatp0 (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for Ap in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt),y(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      y(2:nrm1,2:ntm1)= a00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1)
     &                 +ap0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1)
     &                 +ap0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1)
     &                 +a0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  )
     &                 +a0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2)
c
c ****** Set the boundary points to zero.
c
      y(:, 1)=0.
      y(:,nt)=0.
      y( 1,:)=0.
      y(nr,:)=0.
c
      return
      end
c#######################################################################
      subroutine ax_amatt (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for At in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,ntm1),y(nr,ntm1)
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      y(2:nrm1,2:ntm2)= a00(2:nrm1,2:ntm2)*x(2:nrm1,2:ntm2)
     &                 +ap0(2:nrm1,2:ntm2)*x(3:nr  ,2:ntm2)
     &                 +ap0(1:nrm2,2:ntm2)*x(1:nrm2,2:ntm2)
     &                 +a0p(2:nrm1,2:ntm2)*x(2:nrm1,3:ntm1)
     &                 +a0p(2:nrm1,1:ntm3)*x(2:nrm1,1:ntm3)
c
c ****** Set the boundary points to zero.
c
      y(:,   1)=0.
      y(:,ntm1)=0.
      y( 1,:)=0.
      y(nr,:)=0.
c
      return
      end
c#######################################################################
      subroutine ax_vmatrt0 (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equations for (vr,vt) in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,2),y(nr,nt,2)
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      y(2:nrm1,2:ntm1,1)= arr00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
     &                   +arrp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,1)
     &                   +arrp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,1)
     &                   +arr0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,1)
     &                   +arr0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,1)
     &                   +art0m(2:nrm1,2:ntm1)*x(2:nrm1,1:ntm2,2)
     &                   +artpm(2:nrm1,2:ntm1)*x(3:nr  ,1:ntm2,2)
     &                   +art00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
     &                   +artp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,2)
c
      y(2:nrm1,2:ntm1,2)= att00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
     &                   +attp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,2)
     &                   +attp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,2)
     &                   +att0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,2)
     &                   +att0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,2)
     &                   +art0m(2:nrm1,3:nt  )*x(2:nrm1,3:nt  ,1)
     &                   +artpm(1:nrm2,3:nt  )*x(1:nrm2,3:nt  ,1)
     &                   +art00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
     &                   +artp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,1)
c
c ****** Set the boundary points to zero.
c
      y(:, 1,:)=0.
      y(:,nt,:)=0.
      y( 1,:,:)=0.
      y(nr,:,:)=0.
c
      return
      end
c#######################################################################
      subroutine ax_vmatp0 (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for vp in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt),y(nr,nt)
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      y(2:nrm1,2:ntm1)= app00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1)
     &                 +appp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1)
     &                 +appp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1)
     &                 +app0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  )
     &                 +app0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2)
c
c ****** Set the boundary points to zero.
c
      y(:, 1)=0.
      y(:,nt)=0.
      y( 1,:)=0.
      y(nr,:)=0.
c
      return
      end
c#######################################################################
      subroutine ax_vmatrtp (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for (vr,vt,vp)
c ****** in the semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,3),y(nr,nt,3)
c
c-----------------------------------------------------------------------
c
c ****** Set the internal points.
c
      y(2:nrm1,2:ntm1,1)= arr00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
     &                   +arrp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,1)
     &                   +arrp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,1)
     &                   +arr0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,1)
     &                   +arr0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,1)
     &                   +art0m(2:nrm1,2:ntm1)*x(2:nrm1,1:ntm2,2)
     &                   +artpm(2:nrm1,2:ntm1)*x(3:nr  ,1:ntm2,2)
     &                   +art00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
     &                   +artp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,2)
     &                   +arp00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,3)
     &                   +arpp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,3)
c
      y(2:nrm1,2:ntm1,2)= att00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
     &                   +attp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,2)
     &                   +attp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,2)
     &                   +att0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,2)
     &                   +att0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,2)
     &                   +art0m(2:nrm1,3:nt  )*x(2:nrm1,3:nt  ,1)
     &                   +artpm(1:nrm2,3:nt  )*x(1:nrm2,3:nt  ,1)
     &                   +art00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
     &                   +artp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,1)
     &                   +atp00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,3)
     &                   +atp0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,3)
c
      y(2:nrm1,2:ntm1,3)= app00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,3)
     &                   +appp0(2:nrm1,2:ntm1)*x(3:nr  ,2:ntm1,3)
     &                   +appp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,3)
     &                   +app0p(2:nrm1,2:ntm1)*x(2:nrm1,3:nt  ,3)
     &                   +app0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,3)
     &                   +arp00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
     &                   +arpp0(1:nrm2,2:ntm1)*x(1:nrm2,2:ntm1,1)
     &                   +atp00(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
     &                   +atp0p(2:nrm1,1:ntm2)*x(2:nrm1,1:ntm2,2)
c
c ****** Set the boundary points to zero.
c
      y(:, 1,:)=0.
      y(:,nt,:)=0.
      y( 1,:,:)=0.
      y(nr,:,:)=0.
c
      return
      end
c#######################################################################
      subroutine ax_a (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the implicit resistivity advance.
c
c-----------------------------------------------------------------------
c
      use matrix_a
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real y(nr,nt,3),x(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      m=m_a
      iri=iri_a
c
c ****** Set the boundary values of x.
c
      call sbca (x(1,1,1),x(1,1,2),x(1,1,3),m,iri,0.)
c
c ****** Do the internal points.
c
      call curlsq (y(1,1,1),y(1,1,2),y(1,1,3),
     &             x(1,1,1),x(1,1,2),x(1,1,3),m,iri)
c
c ****** Zero the boundary values of x.
c
      call zbca (x(1,1,1),x(1,1,2),x(1,1,3))
c
c ****** Zero the boundary values of y.
c
      call zbca (y(1,1,1),y(1,1,2),y(1,1,3))
c
      return
      end
c#######################################################################
      subroutine curlsq (yar,yat,yap,xar,xat,xap,m,iri)
c
c-----------------------------------------------------------------------
c
c ****** Set y = dV * (sigma*x + omegaeta*dtime*curl-curl-x) at the
c ****** internal points.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real xar(nr,nt),xat(nr,nt),xap(nr,nt)
      real yar(nr,nt),yat(nr,nt),yap(nr,nt)
c
c-----------------------------------------------------------------------
c
      k=m+1
c
      if (iri.eq.1) then
        emm=-fm(k)
      else
        emm= fm(k)
      end if
c
      fac=omegaeta*dtime
c
      do j=2,ntm1
      do i=1,nrm1
        eta=AVG2T (etafac,i,j)+AVG2T (etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvr=r(i)**2*dr(i)*dth(j)*sth(j)
        yar(i,j)=dvr*sig*xar(i,j)+fac*(
     &            st(j  )*(rh(i+1)*xat(i+1,j  )-rh(i)*xat(i,j  ))
     &           -st(j-1)*(rh(i+1)*xat(i+1,j-1)-rh(i)*xat(i,j-1))
     &           -dr(i)*( st(j  )*dti(j  )*(xar(i,j+1)-xar(i,j  ))
     &                   -st(j-1)*dti(j-1)*(xar(i,j  )-xar(i,j-1)))
     &           +emm*dth(j)*(rh(i+1)*xap(i+1,j)-rh(i)*xap(i,j))
     &           +fm2(k)*dr(i)*dth(j)*xar(i,j)/sth(j))
      enddo
      enddo
c
      do j=2,ntm2
      do i=2,nrm1
        eta=AVG2R (etafac,i,j)+AVG2R (etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvt=rh(i)**2*drh(i)*dt(j)*st(j)
        yat(i,j)=dvt*sig*xat(i,j)+fac*(
     &           -rh(i)*st(j)*dt(j)*(
     &             dri(i  )*(rh(i+1)*xat(i+1,j)-rh(i  )*xat(i  ,j))
     &            -dri(i-1)*(rh(i  )*xat(i  ,j)-rh(i-1)*xat(i-1,j)))
     &           +rh(i)*st(j)*( (xar(i  ,j+1)-xar(i  ,j))
     &                         -(xar(i-1,j+1)-xar(i-1,j)))
     &           +emm*(drh(i)/st(j))*( sth(j+1)*xap(i,j+1)
     &                               -sth(j  )*xap(i,j  ))
     &           +fm2(k)*drh(i)*dt(j)*xat(i,j)/st(j))
      enddo
      enddo
c
      do j=2,ntm1
c
c ****** Special treatment of Jphi at the points near the poles.
c
      if (j.eq.2) then
        ratpp=sth(j+1)/st(j  )
        ratpm=sth(j  )/st(j  )
        facp=1./st(j  )
        facm=0.
        if (m.eq.0) then
          ratmp=2.
          ratmm=2.
        else
          ratmp=0.
          ratmm=0.
        end if
      else if (j.eq.ntm1) then
        ratmp=sth(j  )/st(j-1)
        ratmm=sth(j-1)/st(j-1)
        facp=0.
        facm=1./st(j-1)
        if (m.eq.0) then
          ratpp=2.
          ratpm=2.
        else
          ratpp=0.
          ratpm=0.
        end if
      else
        ratpp=sth(j+1)/st(j  )
        ratpm=sth(j  )/st(j  )
        ratmp=sth(j  )/st(j-1)
        ratmm=sth(j-1)/st(j-1)
        facp=1./st(j  )
        facm=1./st(j-1)
      end if
c
      do i=2,nrm1
        eta=AVG2RT(etafac,i,j)+AVG2RT(etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvp=rh(i)**2*drh(i)*dth(j)*sth(j)
        yap(i,j)=dvp*sig*xap(i,j)+fac*(
     &           -drh(i)*sth(j)*(
     &             dti(j  )*(ratpp*xap(i,j+1)-ratpm*xap(i,j  ))
     &            -dti(j-1)*(ratmp*xap(i,j  )-ratmm*xap(i,j-1)))
     &           -rh(i)*sth(j)*dth(j)*(
     &             dri(i  )*(rh(i+1)*xap(i+1,j)-rh(i  )*xap(i  ,j))
     &            -dri(i-1)*(rh(i  )*xap(i  ,j)-rh(i-1)*xap(i-1,j)))
     &           -emm*rh(i)*dth(j)*(xar(i,j)-xar(i-1,j))
     &           -emm*drh(i)*sth(j)*(facp*xat(i,j)-facm*xat(i,j-1)))
      enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine ax_t (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = A * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the implicit parallel thermal
c ****** conductivity advance.
c
c-----------------------------------------------------------------------
c
c ****** Set the boundary values of x.
c
      call sbct (x,0.)
c
c ****** Do the internal points.
c
      call div_kappa_grad (y,x)
c
c ****** Zero the boundary values of x.
c
      call zbct (x)
c
c ****** Zero the boundary values of y.
c
      call zbct (y)
c
      return
      end
c#######################################################################
      subroutine div_kappa_grad (y,x)
c
c-----------------------------------------------------------------------
c
c ****** Set y = dV * [n_e*x + (dtime*(gamma-1)/const) *
c ****** div-kappa_par-grad-x] at the internal points.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real y(nr,nt,np),x(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Parallel thermal conductivity arrays.
c
      common /fkparcom/ fkrr(nr,nt,np),
     &                  fktt(nr,nt,np),
     &                  fkpp(nr,nt,np),
     &                  fkrt(nr,nt,np),
     &                  fkrp(nr,nt,np),
     &                  fktp(nr,nt,np), dqdt(nr,nt,np)
c
c-----------------------------------------------------------------------
c ****** NOTE!!!!
c ****** In theory we should use gamma dependent on r. In practice
c ****** when we use thermal conduction gamma is constant. Using
c ****** a variable r makes the operator not self-adjoint.

      fac=(gammar(1)-1.)*dtime/(he_p*fn_kappa)
c
      do k=1,np
        kp1=mod(k+1,np)
        if (kp1.le.0) kp1=kp1+np
        km1=mod(k-1,np)
        if (km1.le.0) km1=km1+np
        do j=2,ntm1
          do i=2,nrm1
            dgrr=sth(j)*dth(j)*
     &            ( r(i  )**2*.5*(fkrr(i+1,j,k)+fkrr(i  ,j,k))
     &              *(x(i+1,j,k)-x(i  ,j,k))/dr(i  )
     &             -r(i-1)**2*.5*(fkrr(i  ,j,k)+fkrr(i-1,j,k))
     &              *(x(i  ,j,k)-x(i-1,j,k))/dr(i-1)
     &            )
            dgtt=drh(i)*
     &            ( st(j  )*.5*(fktt(i,j+1,k)+fktt(i,j  ,k))
     &              *(x(i,j+1,k)-x(i,j  ,k))/dt(j  )
     &             -st(j-1)*.5*(fktt(i,j  ,k)+fktt(i,j-1,k))
     &              *(x(i,j  ,k)-x(i,j-1,k))/dt(j-1)
     &            )
            dgpp=drh(i)*dth(j)*
     &            ( .5*(fkpp(i,j,kp1)+fkpp(i,j,k  ))
     &              *(x(i,j,kp1)-x(i,j,k  ))/dphi
     &             -.5*(fkpp(i,j,k  )+fkpp(i,j,km1))
     &              *(x(i,j,k  )-x(i,j,km1))/dphi
     &            )/(dphi*sth(j))
            dgrt=.25*sth(j)*
     &            ( rh(i+1)*fkrt(i+1,j,k)
     &              *(x(i+1,j+1,k)-x(i+1,j-1,k))
     &             -rh(i-1)*fkrt(i-1,j,k)
     &              *(x(i-1,j+1,k)-x(i-1,j-1,k))
     &            )
            dgtr=.25*rh(i)*
     &            ( sth(j+1)*fkrt(i,j+1,k)
     &              *(x(i+1,j+1,k)-x(i-1,j+1,k))
     &             -sth(j-1)*fkrt(i,j-1,k)
     &              *(x(i+1,j-1,k)-x(i-1,j-1,k))
     &            )
            dgrp=.25*dth(j)*
     &            ( rh(i+1)*fkrp(i+1,j,k)
     &              *(x(i+1,j,kp1)-x(i+1,j,km1))/dphi
     &             -rh(i-1)*fkrp(i-1,j,k)
     &              *(x(i-1,j,kp1)-x(i-1,j,km1))/dphi
     &            )
            dgpr=.25*rh(i)*dth(j)*
     &            ( fkrp(i,j,kp1)
     &              *(x(i+1,j,kp1)-x(i-1,j,kp1))
     &             -fkrp(i,j,km1)
     &              *(x(i+1,j,km1)-x(i-1,j,km1))
     &            )/dphi
            dgtp=.25*drh(i)*
     &            ( fktp(i,j+1,k)
     &              *(x(i,j+1,kp1)-x(i,j+1,km1))/dphi
     &             -fktp(i,j-1,k)
     &              *(x(i,j-1,kp1)-x(i,j-1,km1))/dphi
     &            )
            dgpt=.25*drh(i)*
     &            ( fktp(i,j,kp1)
     &              *(x(i,j+1,kp1)-x(i,j-1,kp1))
     &             -fktp(i,j,km1)
     &              *(x(i,j+1,km1)-x(i,j-1,km1))
     &            )/dphi
            fne=rho(i,j,k)/he_rho
            dv=rh(i)**2*drh(i)*dth(j)*sth(j)
            y(i,j,k)=dv*fne*(1.+dqdt(i,j,k))*x(i,j,k)
     &      -fac*( dgrr+dgtt+dgpp+dgrt+dgtr
     &                                    +dgrp+dgpr+dgtp+dgpt)
                    
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine linv (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x := D * L(inv) * x.
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0.or.ifprec.eq.1) return
c
      return
      end
c#######################################################################
      subroutine uinv (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x := U(inv) * x.
c
c-----------------------------------------------------------------------
c
      use globals
      use cgcom2
c
c-----------------------------------------------------------------------
c
      if (ifprec.eq.0) return
c
      if (ifprec.eq.1) then
        call dtimes (x)
        return
      end if
c
      return
      end
c#######################################################################
      subroutine dtimes (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine selects the appropriate diagonal,
c ****** depending on which equation is being solved.
c
c-----------------------------------------------------------------------
c
      use solver_types
c
c-----------------------------------------------------------------------
c
      select case (solve_type)
      case (ST_AMATP0)
        call dtimes_amatp0 (x)
      case (ST_AMATT)
        call dtimes_amatt (x)
      case (ST_VMATRT0)
        call dtimes_vmatrt0 (x)
      case (ST_VMATP0)
        call dtimes_vmatp0 (x)
      case (ST_VMATRTP)
        call dtimes_vmatrtp (x)
      case (ST_A)
        call dtimes_a (x)
      case (ST_T)
        call dtimes_t (x)
      case default
        write (9,*)
        write (9,*) '### ERROR in DTIMES:'
        write (9,*) '### Invalid SOLVE_TYPE.'
        write (9,*) '### SOLVE_TYPE = ',solve_type
        write (9,*) '### [This is an internal error.]'
        write (*,*)
        write (*,*) '### ERROR in DTIMES:'
        write (*,*) '### Invalid SOLVE_TYPE.'
        write (*,*) '### SOLVE_TYPE = ',solve_type
        write (*,*) '### [This is an internal error.]'
        call exit (1)
      end select
c
      return
      end
c#######################################################################
      subroutine dtimes_amatp0 (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for Ap in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt)
c
c-----------------------------------------------------------------------
c
      x(2:nrm1,2:ntm1)=d(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1)
c
      return
      end
c#######################################################################
      subroutine dtimes_amatt (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for At in the
c ****** potential field solve.
c
c-----------------------------------------------------------------------
c
      use matrix_pot
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,ntm1)
c
c-----------------------------------------------------------------------
c
      x(2:nrm1,2:ntm2)=d(2:nrm1,2:ntm2)*x(2:nrm1,2:ntm2)
c
      return
      end
c#######################################################################
      subroutine dtimes_vmatrt0 (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equations for (vr,vt) in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,2)
c
c-----------------------------------------------------------------------
c
      x(2:nrm1,2:ntm1,1)=drr(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
      x(2:nrm1,2:ntm1,2)=dtt(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
c
      return
      end
c#######################################################################
      subroutine dtimes_vmatp0 (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m=0 equation for vp in the
c ****** semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt)
c
c-----------------------------------------------------------------------
c
      x(2:nrm1,2:ntm1)=dpp(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1)
c
      return
      end
c#######################################################################
      subroutine dtimes_vmatrtp (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x = D * x.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the m.ne.0 equations for (vr,vt,vp)
c ****** in the semi-implicit and viscosity solves.
c
c-----------------------------------------------------------------------
c
      use matrix_vmat
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      x(2:nrm1,2:ntm1,1)=drr(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,1)
      x(2:nrm1,2:ntm1,2)=dtt(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,2)
      x(2:nrm1,2:ntm1,3)=dpp(2:nrm1,2:ntm1)*x(2:nrm1,2:ntm1,3)
c
      return
      end
c#######################################################################
      subroutine dtimes_a (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x := x / diag(A), where diag(A) is the matrix main
c ****** diagonal, at the internal points.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the implicit resistivity advance.
c
c-----------------------------------------------------------------------
c
      use matrix_a
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,3)
c
c-----------------------------------------------------------------------
c
      m=m_a
c
      k=m+1
c
      fac=omegaeta*dtime
c
      do j=2,ntm1
      do i=1,nrm1
        eta=AVG2T (etafac,i,j)+AVG2T (etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvr=r(i)**2*dr(i)*dth(j)*sth(j)
        x(i,j,1)=x(i,j,1)/(dvr*sig+fac*(
     &            dr(i)*(st(j  )*dti(j  )+st(j-1)*dti(j-1))
     &           +fm2(k)*dr(i)*dth(j)/sth(j)))
      enddo
      enddo
c
      do j=2,ntm2
      do i=2,nrm1
        eta=AVG2R (etafac,i,j)+AVG2R (etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvt=rh(i)**2*drh(i)*dt(j)*st(j)
        x(i,j,2)=x(i,j,2)/(dvt*sig+fac*(
     &            rh(i)*st(j)*dt(j)*(
     &             dri(i  )*rh(i  )+dri(i-1)*rh(i  ))
     &           +fm2(k)*drh(i)*dt(j)/st(j)))
      enddo
      enddo
c
      do j=2,ntm1
c
c ****** Special treatment of Jphi at the points near the poles.
c
      if (j.eq.2) then
        ratpm=sth(j  )/st(j  )
        if (m.eq.0) then
          ratmp=2.
        else
          ratmp=0.
        end if
      else if (j.eq.ntm1) then
        ratmp=sth(j  )/st(j-1)
        if (m.eq.0) then
          ratpm=2.
        else
          ratpm=0.
        end if
      else
        ratpm=sth(j  )/st(j  )
        ratmp=sth(j  )/st(j-1)
      end if
c
      do i=2,nrm1
        eta=AVG2RT(etafac,i,j)+AVG2RT(etacel,i,j)
        sig=1./max(eta,1.e-20)
        dvp=rh(i)**2*drh(i)*dth(j)*sth(j)
        x(i,j,3)=x(i,j,3)/(dvp*sig+fac*(
     &            drh(i)*sth(j)*(
     &             dti(j  )*ratpm+dti(j-1)*ratmp)
     &           +rh(i)*sth(j)*dth(j)*(
     &             dri(i  )*rh(i  )+dri(i-1)*rh(i  ))))
      enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine dtimes_t (x)
c
c-----------------------------------------------------------------------
c
c ****** Set x := x / diag(A), where diag(A) is the matrix main
c ****** diagonal, at the internal points.
c
c-----------------------------------------------------------------------
c
c ****** This routine is for the implicit parallel thermal
c ****** conduction advance.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real x(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Parallel thermal conductivity arrays.
c
      common /fkparcom/ fkrr(nr,nt,np),
     &                  fktt(nr,nt,np),
     &                  fkpp(nr,nt,np),
     &                  fkrt(nr,nt,np),
     &                  fkrp(nr,nt,np),
     &                  fktp(nr,nt,np), dqdt(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      fac=(gammar(1)-1.)*dtime/(he_p*fn_kappa)
c
      do k=1,np
        kp1=mod(k+1,np)
        if (kp1.le.0) kp1=kp1+np
        km1=mod(k-1,np)
        if (km1.le.0) km1=km1+np
        do j=2,ntm1
          do i=2,nrm1
            dgrr=sth(j)*dth(j)*
     &            ( r(i  )**2*.5*(fkrr(i+1,j,k)+fkrr(i  ,j,k))/dr(i  )
     &             +r(i-1)**2*.5*(fkrr(i  ,j,k)+fkrr(i-1,j,k))/dr(i-1)
     &            )
            dgtt=drh(i)*
     &            ( st(j  )*.5*(fktt(i,j+1,k)+fktt(i,j  ,k))/dt(j  )
     &             +st(j-1)*.5*(fktt(i,j  ,k)+fktt(i,j-1,k))/dt(j-1)
     &            )
            dgpp=drh(i)*dth(j)*
     &            ( .5*(fkpp(i,j,kp1)+fkpp(i,j,k  ))/dphi
     &             +.5*(fkpp(i,j,k  )+fkpp(i,j,km1))/dphi
     &            )/(dphi*sth(j))
            fne=rho(i,j,k)/he_rho
            dv=rh(i)**2*drh(i)*dth(j)*sth(j)
            x(i,j,k)=x(i,j,k)/(dv*fne*(1.+dqdt(i,j,k))+
     &      fac*(dgrr+dgtt+dgpp))
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine histchek
c
c-----------------------------------------------------------------------
c
c ****** Set switch for collection of time histories.
c ****** Also, set switch for plotting of spatial diagnostics.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Collect time-histories at intervals of IHISTINT timesteps,
c ****** or every THISTINT Alfven times.
c
c ****** Disable collection at timestep intervals if IHISTINT.le.0.
c
      if (ihistint.le.0) then
        ifhststp=0
      else
        ifhststp=mod(ntime,ihistint)
        if (ifhststp.eq.0) then
          ifhststp=1
        else
          ifhststp=0
        end if
      end if
c
c ****** Disable collection at time intervals if THISTINT.le.0.
c
      if (ihistint.le.0) then
        if (thistint.le.0) then
          ifhststp=0
        else
          trem=amod(time,thistint)
          if (trem.lt.dtime) then
            ifhststp=1
          else
            ifhststp=0
          end if
        end if
      end if
c
c ****** Time history diagnostics are collected whenever IFHSTSTP.ne.0.
c
c ****** Check for an overflowing history buffer.
c
      if (ihist.eq.nhistmax) then
        call histplot
        ihist=0
      end if
c
c ****** Increment the time-history index.
c
      if (ifhststp.ne.0) then
        ihist=ihist+1
      end if
c
c ****** Set the switch for spatial diagnostics.
c
c ****** Plot x-diagnostics at intervals of IPLTXINT time steps,
c ****** and every TPLTXINT resistive times.
c
c ****** Disable plotting at time step intervals if IPLTXINT.le.0.
c
      if (ipltxint.le.0) then
        ifplxstp=0
      else
        ifplxstp=mod(ntime,ipltxint)
        if (ifplxstp.eq.0) then
          ifplxstp=1
        else
          ifplxstp=0
        end if
      end if
c
c ****** Disable plotting at time intervals if TPLTXINT.le.0.
c
      if (ifplxstp.eq.0) then
        if (tpltxint.le.0.) then
          ifplxstp=0
        else
          trem=amod(time,tpltxint)
          if (trem.lt.dtime) then
            ifplxstp=1
          else
            ifplxstp=0
          end if
        end if
      end if
c
      return
      end
c#######################################################################
      subroutine diags
c
c-----------------------------------------------------------------------
c
c ****** Collect time diagnostics.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Collect time history diagnostics if on a diagnostic interval.
c
      if (ifhststp.ne.0) call tdiagcol
c
c ****** Plot spatial diagnostics if on diagnostic interval.
c
      if (ifplxstp.ne.0) call xdiags
c
      return
      end
c#######################################################################
      subroutine xdiags
c
c-----------------------------------------------------------------------
c
c ****** Write out spatial diagnostics.
c
c-----------------------------------------------------------------------
c
c ****** Do a plot dump.
c
      call pdump
c
      return
      end
c#######################################################################
      subroutine rsdump
c
c-----------------------------------------------------------------------
c
c ****** Write a restart file if on a dump interval.
c
c-----------------------------------------------------------------------
c
      use ident
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Restart file name and sequence number.
c
      character*72 rstname
      save iseq
      data iseq/0/
c
c-----------------------------------------------------------------------
c
c ****** Check if this is a restart file dump interval.
c
c ****** Disable restart file dump at time step intervals
c ****** if IRSDUMP.le.0.
c
      if (irsdump.le.0) then
        ifrsstp=0
      else
        ifrsstp=mod(ntime,irsdump)
        if (ifrsstp.eq.0) then
          ifrsstp=1
        else
          ifrsstp=0
        end if
      end if
c
c ****** Disable restart file dump at time intervals
c ****** if TRSDUMP.le.0.
c
      if (ifrsstp.eq.0) then
        if (trsdump.le.0.) then
          ifrsstp=0
        else
          trem=amod(time,trsdump)
          if (trem.lt.dtime) then
            ifrsstp=1
          else
            ifrsstp=0
          end if
        end if
      end if
c
      if (ifrsstp.eq.0)  return
c
c ****** Create a restart file name of the form: rs<runid>###.
c
      iseq=iseq+1
      rstname='rs'
      lrid=lenstr(runid)
      write (rstname(3:),'(a,i3.3)') runid(1:lrid),iseq
c
c ****** Write the restart file.
c
      call wrrstrt (rstname)
c
      return
      end
c#######################################################################
      subroutine tdiagcol
c
c-----------------------------------------------------------------------
c
c ****** Collect time history diagnostics.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Information about maximum speeds and time step limits.
c
      common /dtinfo/ vmax,vamax,csmax,dtcfl,dtadv,betamax
c
c-----------------------------------------------------------------------
c
c ****** Information about cell viscosity and resistivity.
c
      common /vcinfo/ vcmin,vcmax
      common /ecinfo/ ecmin,ecmax
c
c-----------------------------------------------------------------------
c
      real jxbr(nr,nt,np),jxbt(nr,nt,np),jxbp(nr,nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Calculate J x B.
c
      do 160 k=1,np
        do 110 j=2,ntm1
        do 100 i=2,nrm2
          avbt=AVG  (bt ,i  ,j  ,k)
          avbp=AVGT (bp ,i  ,j  ,k)
          avjt=AVGRT(fjt,i+1,j  ,k)
          avjp=AVGR (fjp,i+1,j  ,k)
          jxbr(i,j,k)=avjt*avbp-avjp*avbt
  100   continue
        jxbr(   1,j,k)=jxbr(   2,j,k)
        jxbr(nrm1,j,k)=jxbr(nrm2,j,k)
  110   continue
        do 130 j=2,ntm2
        do 120 i=2,nrm1
          avbr=AVG  (br ,i  ,j  ,k)
          avbp=AVGR (bp ,i  ,j  ,k)
          avjr=AVGRT(fjr,i  ,j+1,k)
          avjp=AVGT (fjp,i  ,j+1,k)
          jxbt(i,j,k)=avjp*avbr-avjr*avbp
  120   continue
  130   continue
        do 135 i=2,nrm1
          jxbt(i,   1,k)=jxbt(i,   2,k)
          jxbt(i,ntm1,k)=jxbt(i,ntm2,k)
  135   continue
        do 150 j=2,ntm1
        do 140 i=2,nrm1
          avbr=AVGT (br ,i  ,j  ,k)
          avbt=AVGR (bt ,i  ,j  ,k)
          avjr=AVGR (fjr,i  ,j  ,k)
          avjt=AVGT (fjt,i  ,j  ,k)
          jxbp(i,j,k)=avjr*avbt-avjt*avbr
  140   continue
  150   continue
  160 continue
c
      sum1=0.
      sum2=0.
      sum3=0.
      sum4=0.
      sum5=0.
      do 220 k=1,np
      do 210 j=2,ntm1
      do 200 i=2,nrm1
        avbr =AVGT (br ,i  ,j  ,k)
        avbt =AVGR (bt ,i  ,j  ,k)
        avbp =AVGRT(bp ,i  ,j  ,k)
        avjr =AVGR (fjr,i  ,j  ,k)
        avjt =AVGT (fjt,i  ,j  ,k)
        avjp =AVG  (fjp,i  ,j  ,k)
        fjdotb=abs(avbr*avjr+avbt*avjt+avbp*avjp)
        fjxbr=AVGR (jxbr,i,j,k)
        fjxbt=AVGT (jxbt,i,j,k)
        fjxbp=AVG  (jxbp,i,j,k)
        fjxb=sqrt(fjxbr**2+fjxbt**2+fjxbp**2)
        dv=rh(i)**2*drh(i)*dth(j)*sth(j)
        sum1=sum1+avbr**2*dv
        sum2=sum2+avbt**2*dv
        sum3=sum3+avbp**2*dv
        sum4=sum4+fjdotb*dv
        sum5=sum5+fjxb*dv
  200 continue
  210 continue
  220 continue
      bsqr=sum1*dphi
      bsqt=sum2*dphi
      bsqp=sum3*dphi
      fjdotb=sum4*dphi
      fjxb=sum5*dphi
c
      sum1=0.
      sum2=0.
      sum3=0.
      sum4=0.
      do 320 k=1,np
      do 310 j=2,ntm1
      do 300 i=2,nrm1
        avvr =AVGR (vr ,i  ,j  ,k)
        avvt =AVGT (vt ,i  ,j  ,k)
        avvp =AVG  (vp ,i  ,j  ,k)
        avrho=AVG  (rho,i  ,j  ,k)
        avp  =AVG  (p  ,i  ,j  ,k)
        dv=rh(i)**2*drh(i)*dth(j)*sth(j)
        sum1=sum1+avrho*avvr**2*dv
        sum2=sum2+avrho*avvt**2*dv
        sum3=sum3+avrho*avvp**2*dv
        sum4=sum4+avp*dv/(gammarh(i)-1.)
  300 continue
  310 continue
  320 continue
      vsqr=sum1*dphi
      vsqt=sum2*dphi
      vsqp=sum3*dphi
      etherm=sum4*dphi
c
      thist(ihist,1)=time
      thist(ihist,2)=bsqr
      thist(ihist,3)=bsqt
      thist(ihist,4)=bsqp
      thist(ihist,5)=vsqr
      thist(ihist,6)=vsqt
      thist(ihist,7)=vsqp
      thist(ihist,8)=fjxb
      thist(ihist,9)=fjdotb
      thist(ihist,10)=vmag0
      thist(ihist,11)=.5*(bsqr+bsqt+bsqp)
      thist(ihist,12)=.5*(vsqr+vsqt+vsqp)
      thist(ihist,13)=etherm
c
c ****** Compute v-parallel and v-perp.
c
      sum1=0.
      sum2=0.
      sum3=0.
      sum4=0.
      sum5=0.
      sum6=0.
      do 420 k=1,np
      do 410 j=2,ntm1
      do 400 i=2,nrm1
        avvr =AVGR (vr ,i  ,j  ,k)
        avvt =AVGT (vt ,i  ,j  ,k)
        avvp =AVG  (vp ,i  ,j  ,k)
        avbr =AVGT (br ,i  ,j  ,k)
        avbt =AVGR (bt ,i  ,j  ,k)
        avbp =AVGRT(bp ,i  ,j  ,k)
        avrho=AVG  (rho,i  ,j  ,k)
        bsq=avbr**2+avbt**2+avbp**2
        vdotb=avvr*avbr+avvt*avbt+avvp*avbp
        vparr=vdotb*avbr/max(bsq,1.e-30)
        vpart=vdotb*avbt/max(bsq,1.e-30)
        vparp=vdotb*avbp/max(bsq,1.e-30)
        vperpr=avvr-vparr
        vperpt=avvt-vpart
        vperpp=avvp-vparp
        dv=rh(i)**2*drh(i)*dth(j)*sth(j)
        sum1=sum1+avrho*vparr**2*dv
        sum2=sum2+avrho*vpart**2*dv
        sum3=sum3+avrho*vparp**2*dv
        sum4=sum4+avrho*vperpr**2*dv
        sum5=sum5+avrho*vperpt**2*dv
        sum6=sum6+avrho*vperpp**2*dv
  400 continue
  410 continue
  420 continue
      vparsqr=sum1*dphi
      vparsqt=sum2*dphi
      vparsqp=sum3*dphi
      vperpsqr=sum4*dphi
      vperpsqt=sum5*dphi
      vperpsqp=sum6*dphi
c
      thist(ihist,14)=vparsqr
      thist(ihist,15)=vparsqt
      thist(ihist,16)=vparsqp
      thist(ihist,17)=vperpsqr
      thist(ihist,18)=vperpsqt
      thist(ihist,19)=vperpsqp
c
      thist(ihist,20)=vr(ihst,jhst,khst)
      thist(ihist,21)=vt(ihst,jhst,khst)
      thist(ihist,22)=vp(ihst,jhst,khst)
c
c ****** Store the timestep and the CFL number.
c
      thist(ihist,23)=dtime
      thist(ihist,24)=.5*betamax
c
c ****** Store the maximum velocity magnitude.
c
      thist(ihist,25)=vmax
c
c ****** Store the maximum cell viscosity and resistivity.
c
      thist(ihist,26)=vcmax
      thist(ihist,27)=ecmax
c
      return
      end
c#######################################################################
      subroutine histplot
c
c-----------------------------------------------------------------------
c
c ****** Plot time histories.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (ihist.le.0) return
c
c ****** Dump time history information.
c
      call dumphist
c
      return
      end
c#######################################################################
      subroutine dumphist
c
c-----------------------------------------------------------------------
c
c ****** Write out time history information.
c
c-----------------------------------------------------------------------
c
      use ident
      use comm
c
c-----------------------------------------------------------------------
c
      character, parameter :: TAB=achar(9)
c
      character(*), parameter :: FMT='(100(1pe15.8,a))'
c
c-----------------------------------------------------------------------
c
c ****** History file name and sequence number.
c
      character*16 hstname
      save iseq
      data iseq/0/
c
c-----------------------------------------------------------------------
c
c ****** Create a history file name of the form: h<runid>##.
c
      iseq=iseq+1
      if (iseq.gt.99) iseq=iseq-99
      hstname='h'
      lrid=lenstr(runid)
      write (hstname(2:),'(a,i2.2)') runid(1:lrid),iseq
c
c ****** Write histories to the file.
c
      open (11,file=hstname)
      write (11,'(100a)') 'Time'     ,TAB,
     &                    'Br**2'    ,TAB,
     &                    'Bt**2'    ,TAB,
     &                    'Bp**2'    ,TAB,
     &                    'rho*vr**2',TAB,
     &                    'rho*vt**2',TAB,
     &                    'rho*vp**2',TAB,
     &                    '|JxB|'    ,TAB,
     &                    '|J.B|'    ,TAB,
     &                    'vmag0'    ,TAB,
     &                    'W'        ,TAB,
     &                    'K'        ,TAB,
     &                    'E'
      do i=1,ihist
        write (11,FMT) thist(i,1),(TAB,thist(i,j),j=2,13)
      enddo
      close (11)
c
c ****** Create a v-history file name of the form: v<runid>##.
c
      hstname(1:1)='v'
c
c ****** Write v histories to the file.
c
      open (11,file=hstname)
      write (11,'(100a)') 'Time',TAB,'rho*vpar_r**2',
     &                           TAB,'rho*vpar_t**2',
     &                           TAB,'rho*vpar_p**2',
     &                           TAB,'rho*vperp_r**2',
     &                           TAB,'rho*vperp_t**2',
     &                           TAB,'rho*vperp_p**2',
     &                           TAB,'vr(diag)',
     &                           TAB,'vt(diag)',
     &                           TAB,'vp(diag)',
     &                           TAB,'dt',
     &                           TAB,'CFL',
     &                           TAB,'vmax'
      do i=1,ihist
        write (11,FMT) thist(i,1),TAB,thist(i,14),
     &                            TAB,thist(i,15),
     &                            TAB,thist(i,16),
     &                            TAB,thist(i,17),
     &                            TAB,thist(i,18),
     &                            TAB,thist(i,19),
     &                            TAB,thist(i,20),
     &                            TAB,thist(i,21),
     &                            TAB,thist(i,22),
     &                            TAB,thist(i,23),
     &                            TAB,thist(i,24),
     &                            TAB,thist(i,25)
      enddo
      close (11)
c
c ****** Create a q-history file name of the form: q<runid>##.
c
      hstname(1:1)='q'
c
c ****** Write histories to the file.
c
      open (11,file=hstname)
      write (11,'(100a)') 'Time',TAB,'cell_visc_max',
     &                           TAB,'cell_eta_max'
      do i=1,ihist
        write (11,FMT) thist(i,1),TAB,thist(i,26),
     &                            TAB,thist(i,27)
      enddo
      close (11)
c
c ****** Create a mode history file name of the form: m<runid>##.
c
      hstname(1:1)='m'
c
      lf=lenstr(hstname)
c
c ****** Find the number of m-modes with histories.
c
      mm=0
      do i=1,maxmodes
        m=mmodes(i)
        if (m.ge.0.and.m.le.np/2) mm=mm+1
      enddo
cccc RL commented out: no wrabs 
cccc ****** Open the history file.
cccc
ccc      call bopen (4,hstname,'rw',ierr)
cccc
ccc      if (ierr.ne.0) then
ccc        write (*,*)
ccc        write (*,*) '### ERROR in DUMPHIST:'
ccc        write (*,*) '### Error while opening a mode history file.'
ccc        write (*,*) 'File name: ',hstname(1:lf)
ccc        write (*,*) 'IERR (from BOPEN) = ',ierr
ccc        write (*,*) 'NTIME = ',ntime
ccc        write (*,*) 'TIME = ',time
ccc        write (9,*)
ccc        write (9,*) '### ERROR in DUMPHIST:'
ccc        write (9,*) '### Error while opening a mode history file.'
ccc        write (9,*) 'File name: ',hstname(1:lf)
ccc        write (9,*) 'IERR (from BOPEN) = ',ierr
ccc        write (9,*) 'NTIME = ',ntime
ccc        write (9,*) 'TIME = ',time
ccc        call wrrstrt (rsofile)
ccc        call exit (1)
ccc      end if
cccc
cccc ****** Write histories to the file.
cccc
cccc ****** First, write the time histories to the file.
cccc ****** (These are written to the "h" and "v" files already;
cccc ****** they are written here again in binary form).
cccc
ccc      laddr=0
cccc
ccc      call wrabs (4,ihist,1,laddr)
ccc      laddr=laddr+1
cccc
ccc      call wrabs (4,nhq,1,laddr)
ccc      laddr=laddr+1
cccc
ccc      do i=1,nhq
ccc        call wrabs (4,thist(1,i),ihist,laddr)
ccc        laddr=laddr+ihist
ccc      enddo
cccc
cccc ****** Next, write the mode histories.
cccc
ccc      call wrabs (4,mm,1,laddr)
ccc      laddr=laddr+1
cccc
ccc      if (mm.gt.0) then
ccc        do i=1,maxmodes
ccc          m=mmodes(i)
ccc          if (m.ge.0.and.m.le.np/2) then
ccc            call wrabs (4,m,1,laddr)
ccc            laddr=laddr+1
ccc            call wrabs (4,bsqmhst(1,i),ihist,laddr)
ccc            laddr=laddr+ihist
ccc            call wrabs (4,vsqmhst(1,i),ihist,laddr)
ccc            laddr=laddr+ihist
ccc          end if
ccc        enddo
ccc      end if
cccc
ccc      call bclose (4,ierr)
cccc
ccc      if (ierr.ne.0) then
ccc        write (*,*)
ccc        write (*,*) '### ERROR in DUMPHIST:'
ccc        write (*,*) '### Error while closing a mode history file.'
ccc        write (*,*) 'File name: ',hstname(1:lf)
ccc        write (*,*) 'IERR (from BCLOSE) = ',ierr
ccc        write (*,*) 'NTIME = ',ntime
ccc        write (*,*) 'TIME = ',time
ccc        write (9,*)
ccc        write (9,*) '### ERROR in DUMPHIST:'
ccc        write (9,*) '### Error while closing a mode history file.'
ccc        write (9,*) 'File name: ',hstname(1:lf)
ccc        write (9,*) 'IERR (from BCLOSE) = ',ierr
ccc        write (9,*) 'NTIME = ',ntime
ccc        write (9,*) 'TIME = ',time
ccc        call wrrstrt (rsofile)
ccc        call exit (1)
ccc      end if
c
      write (9,*)
      write (9,*) '### COMMENT from DUMPHIST:'
      write (9,*) 'Wrote time histories to file: ','h'//hstname(2:)
      write (9,*) 'Wrote v histories to file:    ','v'//hstname(2:)
ccc      write (9,*) 'Wrote mode histories to file: ','m'//hstname(2:)
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
      write (9,*) 'IHIST = ',ihist
c
      return
      end
c#######################################################################
      subroutine modehst (bcrr,bcri,bctr,bcti,bcpr,bcpi,flag)
c
c-----------------------------------------------------------------------
c
c ****** Get B and V mode histories.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) flag
c
c-----------------------------------------------------------------------
c
c ****** If on a time-history collection step, get the average B**2 for
c ****** requested modes from (complex) B, when FLAG='b', and
c ****** similarly for v when FLAG='v'.
c
      if (ifhststp.eq.0) go to 200
      indx=0
  100 continue
      indx=indx+1
c     Check for out-of-range modes.  Do not exceed allocated space.
      if (indx.gt.maxmodes) go to 200
      m=mmodes(indx)
      if (m.lt.0.or.m.gt.np/2) go to 100
      if (flag.eq.'b')  then
        bsqmhst(ihist,indx)=bsqm(bcrr,bcri,bctr,bcti,bcpr,bcpi,m)
      else
        vsqmhst(ihist,indx)=vsqm(m)
      end if
      go to 100
  200 continue
c
      return
      end
c#######################################################################
      function vsqm (m)
c
c-----------------------------------------------------------------------
c
c ****** Get the volume-integrated value of v**2 for mode M.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (m.eq.0.or.m.eq.np/2) then
        fac=2.*pi
      else
        fac=4.*pi
      end if 
c
      k=m+1
c
      sum=0.
      do j=2,ntm1
        do i=2,nrm1
          avvrr=AVGR (vrr,i  ,j  ,k)
          avvtr=AVGT (vtr,i  ,j  ,k)
          avvpr=AVG  (vpr,i  ,j  ,k)
          avvri=AVGR (vri,i  ,j  ,k)
          avvti=AVGT (vti,i  ,j  ,k)
          avvpi=AVG  (vpi,i  ,j  ,k)
          vsq=avvrr**2+avvri**2+avvtr**2+avvti**2+avvpr**2+avvpi**2
          dv=rh(i)**2*drh(i)*dth(j)*sth(j)
          sum=sum+vsq*dv
        enddo
      enddo
      vsqm=fac*sum
c
      return
      end
c#######################################################################
      function bsqm (bcrr,bcri,bctr,bcti,bcpr,bcpi,m)
c
c-----------------------------------------------------------------------
c
c ****** Get the volume-integrated value of B**2 for mode M.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real bcrr(nr,nt,npd3),bcri(nr,nt,npd3),
     &     bctr(nr,nt,npd3),bcti(nr,nt,npd3),
     &     bcpr(nr,nt,npd3),bcpi(nr,nt,npd3)
c
c-----------------------------------------------------------------------
c
      if (m.eq.0.or.m.eq.np/2) then
        fac=2.*pi
      else
        fac=4.*pi
      end if 
c
      k=m+1
c
      sum=0.
      do j=2,ntm1
        do i=2,nrm1
          avbrr=AVGT (bcrr,i  ,j  ,k)
          avbtr=AVGR (bctr,i  ,j  ,k)
          avbpr=AVGRT(bcpr,i  ,j  ,k)
          avbri=AVGR (bcri,i  ,j  ,k)
          avbti=AVGT (bcti,i  ,j  ,k)
          avbpi=AVG  (bcpi,i  ,j  ,k)
          bsq=avbrr**2+avbri**2+avbtr**2+avbti**2+avbpr**2+avbpi**2
          dv=rh(i)**2*drh(i)*dth(j)*sth(j)
          sum=sum+bsq*dv
        enddo
      enddo
      bsqm=fac*sum
c
      return
      end
c#######################################################################
      subroutine pdump
c
c-----------------------------------------------------------------------
c
c ****** Dump requested fields at the present time.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*32 fname
      character*3 ch3
c
      save iseq
      data iseq/0/
c
      logical plotall
c
c-----------------------------------------------------------------------
c
      external has
      logical has
c
c-----------------------------------------------------------------------
c
c ****** Array to hold the inverse FFT of a component of A.
c
      real a(nrtp)
c
c-----------------------------------------------------------------------
c
      common /gbc0com/ ub(nt,np),h1(nt,np)
c
c-----------------------------------------------------------------------
c
c ****** Emerging flux tangential electric field at r=R0.
c
      common /efluxcom/ et0ef(nt,npd3,2),ep0ef(nt,npd3,2)
c
c-----------------------------------------------------------------------
c
      iseq=iseq+1
      if (iseq.gt.999) iseq=iseq-999
      write (ch3,'(i3.3)') iseq
c
c ****** Check if all quanatities are to be plotted.
c
      if (has(nplist,plotlist,'all')) then
        plotall=.true.
      else
        plotall=.false.
      end if
c
      if (plotall.or.has(nplist,plotlist,'p')) then
        fname='p'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,np,p  ,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'rho')) then
        fname='rho'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,np,rho,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vr')) then
        fname='vr'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,np,vr ,r ,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vt')) then
        fname='vt'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,np,vt ,rh,t ,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vp')) then
        fname='vp'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,np,vp ,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ar')) then
        fname='ar'//ch3
        call fft (a,arr,ari,0)
        call wrfld (fname,nr,nt,nrm1,nt  ,np,a  ,r ,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'at')) then
        fname='at'//ch3
        call fft (a,atr,ati,0)
        call wrfld (fname,nr,nt,nr  ,ntm1,np,a  ,rh,t ,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ap')) then
        fname='ap'//ch3
        call fft (a,apr,api,0)
        call wrfld (fname,nr,nt,nr  ,nt  ,np,a  ,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'br')) then
        fname='br'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,np,br ,rh,t ,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'bt')) then
        fname='bt'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,np,bt ,r ,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'bp')) then
        fname='bp'//ch3
        call wrfld (fname,nr,nt,nrm1,ntm1,np,bp ,r ,t ,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'jr')) then
        fname='jr'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,np,fjr,r ,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'jt')) then
        fname='jt'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,np,fjt,rh,t ,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'jp')) then
        fname='jp'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,np,fjp,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'arr')) then
        fname='arr'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,npd3,arr,r ,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ari')) then
        fname='ari'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,npd3,ari,r ,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'atr')) then
        fname='atr'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,npd3,atr,rh,t ,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ati')) then
        fname='ati'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,npd3,ati,rh,t ,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'apr')) then
        fname='apr'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,npd3,apr,rh,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'api')) then
        fname='api'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,npd3,api,rh,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vri')) then
        fname='vri'//ch3
        call wrfld (fname,nr,nt,nrm1,nt  ,npd3,vri,r ,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vtr')) then
        fname='vtr'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,npd3,vtr,rh,t ,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vti')) then
        fname='vti'//ch3
        call wrfld (fname,nr,nt,nr  ,ntm1,npd3,vti,rh,t ,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vpr')) then
        fname='vpr'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,npd3,vpr,rh,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'vpi')) then
        fname='vpi'//ch3
        call wrfld (fname,nr,nt,nr  ,nt  ,npd3,vpi,rh,th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'t')) then
        fname='t'//ch3
        call wrfld (fname,nr,nt,nr,nt,np,temp,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ep')) then
        fname='ep'//ch3
        call wrfld (fname,nr,nt,nr,nt,np,ep,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'em')) then
        fname='em'//ch3
        call wrfld (fname,nr,nt,nr,nt,np,em,rh,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ub')) then
        if (ntime.gt.0) then
          fname='ub'//ch3
          call wr2d (fname,nt,ntm2,np,ub(2,1),th(2),phi)
        end if
      end if
c
      if (plotall.or.has(nplist,plotlist,'rhor0')) then
        if (ntime.gt.0) then
          fname='rhor0'//ch3
          call wr2d (fname,nt,ntm2,np,rhogbc0(2,1),th(2),phi)
        end if
      end if
c
      if (plotall.or.has(nplist,plotlist,'h1')) then
        if (ntime.gt.0) then
          fname='h1'//ch3
          call wr2d (fname,nt,ntm2,np,h1(2,1),th(2),phi)
        end if
      end if
c
      if (plotall.or.has(nplist,plotlist,'visc')) then
        fname='visc'//ch3
        call wr2d (fname,nrm1,nrm1,ntm1,visfac,r,t)
      end if
c
      if (plotall.or.has(nplist,plotlist,'visccell')) then
        fname='visccell'//ch3
        call wr2d (fname,nrm1,nrm1,ntm1,viscel,r,t)
      end if
c
      if (plotall.or.has(nplist,plotlist,'eta')) then
        fname='eta'//ch3
        call wr2d (fname,nrm1,nrm1,ntm1,etafac,r,t)
      end if
c
      if (plotall.or.has(nplist,plotlist,'etacell')) then
        fname='etacell'//ch3
        call wr2d (fname,nrm1,nrm1,ntm1,etacel,r,t)
      end if
c
      if (plotall.or.has(nplist,plotlist,'et0ef')) then
        fname='et0efr'//ch3
        call wr2d (fname,nt,ntm1,npd3,et0ef(1,1,1),t,fm)
        fname='et0efi'//ch3
        call wr2d (fname,nt,ntm1,npd3,et0ef(1,1,2),t,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ep0ef')) then
        fname='ep0efr'//ch3
        call wr2d (fname,nt,nt,npd3,ep0ef(1,1,1),th,fm)
        fname='ep0efi'//ch3
        call wr2d (fname,nt,nt,npd3,ep0ef(1,1,2),th,fm)
      end if
c
      if (plotall.or.has(nplist,plotlist,'er0')) then
        fname='er0'//ch3
        call wr2d (fname,nt,nt,np,er0,th,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'et0')) then
        fname='et0'//ch3
        call wr2d (fname,nt,ntm1,np,et0,t,phi)
      end if
c
      if (plotall.or.has(nplist,plotlist,'ep0')) then
        fname='ep0'//ch3
        call wr2d (fname,nt,nt,np,ep0,th,phi)
      end if
c
      write (9,*)
      write (9,*) '### COMMENT from PDUMP:'
      write (9,*) '### Wrote out the requested fields.'
      write (9,*) 'File sequence number = ',iseq
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
c
      return
      end
c#######################################################################
      subroutine uwpdm (n,ndim1,f,v,upwind,uw)
c
c-----------------------------------------------------------------------
c
c ****** Calculate the "partial donor cell" upwind coefficient.
c
c-----------------------------------------------------------------------
c
c ****** Calculate the upwind coefficient for quantity F, based
c ****** on the velocity V.
c ****** Return the upwind coefficient in array UW.
c
c ****** N is the number of points at which V is defined.
c ****** It is assumed that V is staggered with respect to F
c ****** and that it is defined at points 1 through N
c ****** (although V(1) and V(N) are not used here).
c ****** F is defined at points 1 through N+1.
c
c ****** The upwind coefficient is defined on the same mesh
c ****** as V is, and is loaded for points 1 through N.
c
c ****** NDIM1 is the stride between neighboring elements of
c ****** arrays F, V, and UW.  For arrays with consecutive data,
c ****** use NDIM1=1.
c
c ****** The coefficient PDMA is the value of "A" for the
c ****** PDM method.  PDMA=1. is a typical value.  Setting
c ****** PDMA lower increases the diffusion (0.8 is sometimes
c ****** used), and PDMA=0. is equivalent to full upwind.
c ****** PDMA=2. is the maximum value that gives theoretical
c ****** stability.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      real f(ndim1,n+1),v(ndim1,n),uw(ndim1,n)
c
c-----------------------------------------------------------------------
c
c ****** Compute the upwind coefficient everywhere on the grid.
c
c ****** The maximum value (1.) means that full upwind
c ****** differencing is performed.
c ****** The minimum value (0.) means that centered
c ****** differencing is performed.
c
c ****** If UPWIND is non-negative, the upwind coefficient is
c ****** set to this value everywhere.  If UPWIND is negative,
c ****** compute the upwind coefficient using the PDM operator.
c
      if (upwind.ge.0.) then
        do 100 i=1,n
          uw(1,i)=upwind
  100   continue
        return
      end if
c
c ****** Loop over the internal V (and UW) locations.
c
      do 200 i=2,n-1
        del0=f(1,i+1)-f(1,i  )
        delp=f(1,i+2)-f(1,i+1)
        delm=f(1,i  )-f(1,i-1)
        term1=abs(del0)
        term2=.25*pdma*(1.-sign(1.,v(1,i)))
     &        *abs(sign(1.,del0)+sign(1.,delp))*abs(delp)
        term3=.25*pdma*(1.+sign(1.,v(1,i)))
     &        *abs(sign(1.,delm)+sign(1.,del0))*abs(delm)
        uw(1,i)=max(0.,term1-term2-term3)/(term1+1.e-20)
  200 continue
c
c ****** Set boundary points.
c
      uw(1,1)=uw(1,2)
      uw(1,n)=uw(1,n-1)
c
      return
      end
c#######################################################################
      subroutine zb1 (f,n)
c
c-----------------------------------------------------------------------
c
c ****** Zero out the boundary points in array F.
c ****** (Vr mesh)
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      dimension f(nr,nt,n)
c
c-----------------------------------------------------------------------
c
      do 300 k=1,n
        do 100 i=1,nr
          f(i, 1,k)=0.
          f(i,nt,k)=0.
  100   continue
        do 200 j=2,ntm1
          f(   1,j,k)=0.
          f(nrm1,j,k)=0.
          f(  nr,j,k)=0.
  200   continue
  300 continue
c
      return
      end
c#######################################################################
      subroutine zb2 (f,n)
c
c-----------------------------------------------------------------------
c
c ****** Zero out the boundary points in array F.
c ****** (Vt mesh)
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      dimension f(nr,nt,n)
c
c-----------------------------------------------------------------------
c
      do 300 k=1,n
        do 100 i=2,nrm1
          f(i,   1,k)=0.
          f(i,ntm1,k)=0.
          f(i,  nt,k)=0.
  100   continue
        do 200 j=1,nt
          f( 1,j,k)=0.
          f(nr,j,k)=0.
  200   continue
  300 continue
c
      return
      end
c#######################################################################
      subroutine zb3 (f,n)
c
c-----------------------------------------------------------------------
c
c ****** Zero out the boundary points in array F.
c ****** (Vp mesh)
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      dimension f(nr,nt,n)
c
c-----------------------------------------------------------------------
c
      do 300 k=1,n
        do 100 j=2,ntm1
          f( 1,j,k)=0.
          f(nr,j,k)=0.
  100   continue
        do 200 i=1,nr
          f(i, 1,k)=0.
          f(i,nt,k)=0.
  200   continue
  300 continue
c
      return
      end
c#######################################################################
      subroutine zb4 (f,n)
c
c-----------------------------------------------------------------------
c
c ****** Zero out the boundary points in array F.
c ****** (Bp mesh)
c
c-----------------------------------------------------------------------
c
      use globals
c
c-----------------------------------------------------------------------
c
      dimension f(nr,nt,n)
c
c-----------------------------------------------------------------------
c
      do 300 k=1,n
        do 100 j=1,ntm1
          f(nr,j,k)=0.
  100   continue
        do 200 i=1,nr
          f(i,nt,k)=0.
  200   continue
  300 continue
c
      return
      end
c#######################################################################
      subroutine wrrstrt (fname)
c
c      implicit none
c
c-----------------------------------------------------------------------
c
c ****** Write a restart file in file FNAME.
c
c uk2 change
c
c This has been completely rewritten to write platform-independent 
c HDF Scientific Dataset format files. 
c NOTE: There is absolutely no compatibility with this format 
c and the previous format. To write old-style restart files, 
c this routine must be replaced in its entirety with the previous 
c version. 
c To read old-style restart files, the routine rdrstrt must also be 
c replaced.
c  
c-----------------------------------------------------------------------
c
      integer sd_id, DFACC_CREATE,dims(2),FAIL,DFNT_CHAR8,
     &          n_values,DFNT_INT32,DFNT_FLOAT64,istat,
     &          sds_id
      integer sfstart, sfcreate, sfwdata, sfendacc, sfend
      integer heprnt,status,i64set,dssnt,sfscatt
      character*(*) fname
      parameter (DFACC_CREATE=4,FAIL=-1,DFNT_CHAR8=4, 
     &           DFNT_INT32=24,DFNT_FLOAT64=6)
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
c ****** Common block to save the initial boundary values
c ****** of vr, rho, and T.
c
      common /rtbc_ts/ vrbcr0re_ts(nt,np),
     &                 rhobcr0re_ts(nt,np),
     &                 tbcr0re_ts(nt,np)
c-----------------------------------------------------------------------
c
c We want the restart files to be written in 64 bit, but the 
c HDF output files to be written in 32 bit. So we call 
c dssnt here each time to specify 64 bit, and use dsclear()
c in the HDF output dumps to give 32 bit.
c
c uk2 change ... call to dssnt to set 64 bit data type.
c set argument to 6 for 64 bit floats.
c
      i64set = dssnt(6)
c
      if (i64set == -1) then
         write(6,*) '64 bit could not be set. stopping in wrrstrt'
         stop
      endif
c
c
c ****** Open the restart file.
c
      lf=lenstr(fname)
      fname(lf+1:)='.hdf'
c
c Create and open the file and initiate the SD interface.
c
      sd_id = sfstart(fname, DFACC_CREATE)
c
      if (sd_id .eq. FAIL) then
         write (*,*)
         write (*,*) '### ERROR in WRRSTRT:'
         write (*,*) '### Error while opening the restart file.'
         write (*,*) '.....sfstart failed.'
         write (*,*) 'File name: ',fname(1:lf)
         write (9,*)
         write (9,*) '### ERROR in WRRSTRT:'
         write (9,*) '### Error while opening the restart file.'
         write (9,*) 'File name: ',fname(1:lf)
         call exit (1)
      end if
c
      write (9,*)
      write (9,*) '### COMMENT from WRRSTRT:'
      write (9,*) 'Writing to restart file: ',fname(1:lf)
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
c
c write variables to HDF file
c
c First we will write some general file attributes, 
c including some dimensions and time and Br00 that 
c are needed to restart the simulation.
c The dimensions can be obtained from the individual 
c data sets, but for completeness, as well as some 
c crossover from the old format, we'll write them 
c as file attributes as well.
c
      nrout    = nr
      ntout    = nt
      npout    = np

      n_values = 1
c
      status = sfscatt(sd_id,'FILE_DESCRIPTION',DFNT_CHAR8,
     &                        18,'MASIP Restart file')
c
      if (status .eq. FAIL) print*,'sfscatt 1 failed.'

      status = sfscatt(sd_id,'nrout',DFNT_INT32,n_values,nrout)
      status = sfscatt(sd_id,'ntout',DFNT_INT32,n_values,ntout)
      status = sfscatt(sd_id,'npout',DFNT_INT32,n_values,npout)
      status = sfscatt(sd_id,'time',DFNT_FLOAT64,n_values,time)
      status = sfscatt(sd_id,'br00',DFNT_FLOAT64,n_values,br00)
c
c call 1,2,3-D routines to write the data to the file
c
      CALL DUMP_SDS_1D(sd_id,'r',nrm1,r)
      CALL DUMP_SDS_1D(sd_id,'t',ntm1,t)
      CALL DUMP_SDS_2D(sd_id,'etafac',nrm1,ntm1,etafac)
      CALL DUMP_SDS_2D(sd_id,'visfac',nrm1,ntm1,visfac)
      CALL DUMP_SDS_3D(sd_id,'arr',nr,nt,npd3,arr)
      CALL DUMP_SDS_3D(sd_id,'ari',nr,nt,npd3,ari)
      CALL DUMP_SDS_3D(sd_id,'atr',nr,nt,npd3,atr)
      CALL DUMP_SDS_3D(sd_id,'ati',nr,nt,npd3,ati)
      CALL DUMP_SDS_3D(sd_id,'apr',nr,nt,npd3,apr)
      CALL DUMP_SDS_3D(sd_id,'api',nr,nt,npd3,api)
      CALL DUMP_SDS_3D(sd_id,'vrr',nr,nt,npd3,vrr)
      CALL DUMP_SDS_3D(sd_id,'vri',nr,nt,npd3,vri)
      CALL DUMP_SDS_3D(sd_id,'vtr',nr,nt,npd3,vtr)
      CALL DUMP_SDS_3D(sd_id,'vti',nr,nt,npd3,vti)
      CALL DUMP_SDS_3D(sd_id,'vpr',nr,nt,npd3,vpr)
      CALL DUMP_SDS_3D(sd_id,'vpi',nr,nt,npd3,vpi)
      CALL DUMP_SDS_3D(sd_id,'rhr',nr,nt,npd3,rhr)
      CALL DUMP_SDS_3D(sd_id,'rhi',nr,nt,npd3,rhi)
      CALL DUMP_SDS_3D(sd_id,'prr',nr,nt,npd3,prr)
      CALL DUMP_SDS_3D(sd_id,'pri',nr,nt,npd3,pri)
      CALL DUMP_SDS_3D(sd_id,'vrbcr0',nt,npd3,2,vrbcr0)
      CALL DUMP_SDS_3D(sd_id,'vrbcr1',nt,npd3,2,vrbcr1)
      CALL DUMP_SDS_3D(sd_id,'vtbcr0',nt,npd3,2,vtbcr0)
      CALL DUMP_SDS_3D(sd_id,'vtbcr1',nt,npd3,2,vtbcr1)
      CALL DUMP_SDS_3D(sd_id,'vpbcr0',nt,npd3,2,vpbcr0)
      CALL DUMP_SDS_3D(sd_id,'vpbcr1',nt,npd3,2,vpbcr1)
      CALL DUMP_SDS_3D(sd_id,'rhobcr0',nt,npd3,2,rhobcr0)
      CALL DUMP_SDS_3D(sd_id,'rhobcr1',nt,npd3,2,rhobcr1)
      CALL DUMP_SDS_3D(sd_id,'pbcr0',nt,npd3,2,pbcr0)
      CALL DUMP_SDS_3D(sd_id,'pbcr1',nt,npd3,2,pbcr1)
      CALL DUMP_SDS_3D(sd_id,'epr',nr,nt,npd3,epr)
      CALL DUMP_SDS_3D(sd_id,'epi',nr,nt,npd3,epi)
      CALL DUMP_SDS_3D(sd_id,'emr',nr,nt,npd3,emr)
      CALL DUMP_SDS_3D(sd_id,'emi',nr,nt,npd3,emi)
c
c To restart the IP solutions, we also need to dump the 
c boundary gas variables
c 
      if (eqtype.eq.'interplanetary') then
          CALL DUMP_SDS_2D(sd_id,'vrbcr0re_ts',nt,np,vrbcr0re_ts)
          CALL DUMP_SDS_2D(sd_id,'rhobcr0re_ts',nt,np,rhobcr0re_ts)
          CALL DUMP_SDS_2D(sd_id,'tbcr0re_ts',nt,np,tbcr0re_ts)
      endif
c
c Terminate access to the SD interface and close the file.
c
      istat = sfend(sd_id)
      if (istat .eq. FAIL) then
        write (*,*)
        write (*,*) '### ERROR in WRRSTRT:'
        write (*,*) '### Error while closing the restart file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in WRRSTRT:'
        write (9,*) '### Error while closing the restart file.'
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
      return
      end
c
c#######################################################################
c
      SUBROUTINE DUMP_SDS_1D(sd_id,name,nxx,array_data)
c
      implicit none
c
      integer sd_id, sds_id, rank
      integer dims(1), start(1), edges(1), stride(1)
      integer istat
      integer sfcreate, sfwdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,FAIL
      real array_data(nxx)
      character*(*) name

      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the rank and dimensions of the data set to be created.
c
      rank = 1
      dims(1) = nxx
c
c Create the data set.
c
      sds_id = sfcreate(sd_id,name,DFNT_FLOAT64,rank,dims)
      if (sds_id .eq. FAIL) then
         print*,'sfcreate failed.'
      end if
c
c Define the location, pattern, and size of the data set
c that will be written to.
c
      start(1) = 0
      edges(1) = dims(1)
      stride(1) = 1
c
c Write the stored data to the "name" data set.
c
      istat = sfwdata(sds_id,start,stride,edges,array_data)
      if (istat .eq. FAIL) then
         print*,'sfwdata failed.'
      end if
c
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'sfendacc failed.'
      end if

      RETURN
      END
c#######################################################################
c
      SUBROUTINE DUMP_SDS_2D(sd_id,name,nxx,nyy,array_data)
c
      implicit none
c
      integer sd_id, sds_id, rank
      integer dims(2), start(2), edges(2), stride(2)
      integer i, j, istat
      integer sfcreate, sfwdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,nyy,FAIL
      real array_data(nxx,nyy)
      character*(*) name

      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the rank and dimensions of the data set to be created.
c
      rank = 2
      dims(1) = nxx
      dims(2) = nyy 
c
c Create the data set.
c
      sds_id = sfcreate(sd_id,name,DFNT_FLOAT64,rank,dims)
      if (sds_id .eq. FAIL) then
         print*,'sfcreate failed.'
      end if
c
c Define the location, pattern, and size of the data set
c that will be written to.
c
      start(1) = 0
      start(2) = 0
      edges(1) = dims(1)
      edges(2) = dims(2)
      stride(1) = 1
      stride(2) = 1
c
c Write the stored data to the "name" data set.
c
      istat = sfwdata(sds_id,start,stride,edges,array_data)
      if (istat .eq. FAIL) then
         print*,'sfwdata failed.'
      end if
c
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'sfendacc failed.'
      end if

      RETURN
      END
c#######################################################################
c
      SUBROUTINE DUMP_SDS_3D(sd_id,name,nxx,nyy,nzz,array_data)
c
      implicit none
c
      integer sd_id, sds_id, rank
      integer dims(3), start(3), edges(3), stride(3)
      integer i, j, istat
      integer sfcreate, sfwdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,nyy,nzz,FAIL
      real array_data(nxx,nyy,nzz)
      character*(*) name

      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the rank and dimensions of the data set to be created.
c
      rank = 3
      dims(1) = nxx
      dims(2) = nyy 
      dims(3) = nzz
c
c Create the data set.
c
      sds_id = sfcreate(sd_id,name,DFNT_FLOAT64,rank,dims)
      if (sds_id .eq. FAIL) then
         print*,'sfcreate failed.'
      end if
c
c Define the location, pattern, and size of the data set
c that will be written to.
c
      start(1) = 0
      start(2) = 0
      start(3) = 0
      edges(1) = dims(1)
      edges(2) = dims(2)
      edges(3) = dims(3)
      stride(1) = 1
      stride(2) = 1
      stride(3) = 1
c
c Write the stored data to the "name" data set.
c
      istat = sfwdata(sds_id,start,stride,edges,array_data)
      if (istat .eq. FAIL) then
         print*,'sfwdata failed.'
      end if
c
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'sfendacc failed.'
      end if

      RETURN
      END

c#######################################################################
c
      SUBROUTINE READ_SDS_1D(sd_id,name,nxx,array_data)
c
      implicit none
c
      integer sd_id, sds_id
      integer dims(1), start(1), edges(1), stride(1)
      integer i, j, istat, sds_index,sfselect
      integer sfn2index, sfrdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,FAIL
      real array_data(nxx)
      character*(*) name
c
      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the dimensions of the data set to be read.
c
      dims(1) = nxx
c
c Define the location, pattern, and size of the data set
c that will be read.
c
      start(1) = 0
      edges(1) = dims(1)
      stride(1) = 1
c
c grab the correct sds index for the particular data set
c
      sds_index = sfn2index(sd_id,name)
c
c select the data set
c
      sds_id = sfselect (sd_id, sds_index) 
c
      if (sds_id .eq. FAIL) then 
         print*,'Restart failure: sfselect'
         call exit(1)
      end if
c
      istat = sfrdata (sds_id,start,stride,edges,array_data) 
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfrdata'
         call exit(1)
      end if
c      
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfendacc'
      end if

      RETURN
      END
c#######################################################################
c
      SUBROUTINE READ_SDS_2D(sd_id,name,nxx,nyy,array_data)
c
      implicit none
c
      integer sd_id, sds_id
      integer dims(2), start(2), edges(2), stride(2)
      integer i, j, istat, sds_index,sfselect
      integer sfn2index, sfrdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,nyy,FAIL
      real array_data(nxx,nyy)
      character*(*) name
c
      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the dimensions of the data set to be read.
c
      dims(1) = nxx
      dims(2) = nyy 
c
c Define the location, pattern, and size of the data set
c that will be read.
c
      start(1) = 0
      start(2) = 0
      edges(1) = dims(1)
      edges(2) = dims(2)
      stride(1) = 1
      stride(2) = 1
c
c grab the correct sds index for the particular data set
c
      sds_index = sfn2index(sd_id,name)
c
c select the data set
c
      sds_id = sfselect (sd_id, sds_index) 
c
      if (sds_id .eq. FAIL) then 
         print*,'Restart failure: sfselect'
         call exit(1)
      end if
c
      istat = sfrdata (sds_id,start,stride,edges,array_data) 
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfrdata'
         call exit(1)
      end if
c      
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfendacc'
      end if

      RETURN
      END
c#######################################################################
c
      SUBROUTINE READ_SDS_3D(sd_id,name,nxx,nyy,nzz,array_data)
c
      implicit none
c
      integer sd_id, sds_id
      integer dims(3), start(3), edges(3), stride(3)
      integer i, j, istat, sds_index,sfselect
      integer sfn2index, sfrdata, sfendacc
      integer DFNT_INT32,DFNT_FLOAT64
      integer nxx,nyy,nzz,FAIL
      real array_data(nxx,nyy,nzz)
      character*(*) name
c
      parameter (DFNT_INT32=24,DFNT_FLOAT64=6,FAIL=-1)
c
c Define the dimensions of the data set to be read.
c
      dims(1) = nxx
      dims(2) = nyy 
      dims(3) = nzz
c
c Define the location, pattern, and size of the data set
c that will be read.
c
      start(1) = 0
      start(2) = 0
      start(3) = 0
      edges(1) = dims(1)
      edges(2) = dims(2)
      edges(3) = dims(3)
      stride(1) = 1
      stride(2) = 1
      stride(3) = 1
c
c grab the correct sds index for the particular data set
c
      sds_index = sfn2index(sd_id,name)
c
c select the data set
c
      sds_id = sfselect (sd_id, sds_index) 
c
      if (sds_id .eq. FAIL) then 
         print*,'Restart failure: sfselect'
         call exit(1)
      end if
c
      istat = sfrdata (sds_id,start,stride,edges,array_data) 
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfrdata'
         call exit(1)
      end if
c      
c Terminate access to the array.
c
      istat = sfendacc(sds_id)
      if (istat .eq. FAIL) then
         print*,'Restart failure: sfendacc'
      end if

      RETURN
      END

c#######################################################################
      logical function letter (ch)
c
c-----------------------------------------------------------------------
c
c ****** Check if character CH is a letter.
c
c-----------------------------------------------------------------------
c
      character ch
c
c-----------------------------------------------------------------------
c
      letter=.false.
c
      ich=ichar(ch)
c
      if ((ich.ge.65.and.ich.le. 90).or.
     &    (ich.ge.97.and.ich.le.122)) letter=.true.
c
      return
      end
c#######################################################################
      logical function digit (ch)
c
c-----------------------------------------------------------------------
c
c ****** Check if character CH is a digit.
c
c-----------------------------------------------------------------------
c
      character ch
c
c-----------------------------------------------------------------------
c
      digit=.false.
c
      ich=ichar(ch)
c
      if (ich.ge.48.and.ich.le.57) digit=.true.
c
      return
      end
c#######################################################################
      logical function alpha (ch)
c
c-----------------------------------------------------------------------
c
c ****** Check if CH is an alphanumeric character.
c
c-----------------------------------------------------------------------
c
      character ch
c
c-----------------------------------------------------------------------
c
      external letter,digit
      logical letter,digit
c
c-----------------------------------------------------------------------
c
      alpha=.false.
c
      if (letter(ch).or.digit(ch)) alpha=.true.
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do 100 i=len(str),1,-1
        if (str(i:i).ne.' ') go to 200
  100 continue
      i=1
  200 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      subroutine adjstrl (str)
c
c-----------------------------------------------------------------------
c
c ****** Left-adjust the character string STR; i.e., shift the
c ****** characters so there are no blanks on the left end of
c ****** the string.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      lstr=len(str)
c
c ****** Find the first nonblank character.
c
      do 100 i=1,lstr
        if (str(i:i).ne.' ') go to 200
  100 continue
c
c ****** All-blank string; leave untouched.
c
      return
c
c ****** Adjust to the left.
c
  200 continue
c
      if (i.gt.1) str(1:)=str(i:lstr)
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in FFOPEN:'
        write (9,*) '### Invalid MODE requested.'
        write (9,*) 'MODE = ',mode
        write (9,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      write (9,*)
      write (9,*) '### ERROR in FFOPEN:'
      write (9,*) '### Error while opening file.'
      write (9,*) 'File name = ',fname(1:lf)
      write (9,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      subroutine bcast (x,c,n)
c
c-----------------------------------------------------------------------
c
c ****** "Broadcast" the constant C into array X.
c
c-----------------------------------------------------------------------
c
      real x(*)
c
c-----------------------------------------------------------------------
c
      do 100 i=1,n
        x(i)=c
  100 continue
c
      return
      end
c#######################################################################
      logical function rdlin (iun,line)
c
c-----------------------------------------------------------------------
c
c ****** Read a line of text from unit IUN.
c
c ****** If a line was read successfully, return with
c ****** RDLIN=.true.; otherwise, set RDLIN=.false.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      read (iun,'(a)',end=100,err=100) line
c
      rdlin=.true.
      return
c
  100 continue
c
c ****** Error while reading the line.
c
      rdlin=.false.
c
      return
      end
c#######################################################################
      subroutine getlun (ilun,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Return an unused logical unit identifier.
c
c-----------------------------------------------------------------------
c
c ****** Upon successful completion (IERR=0), the first
c ****** unused logical unit number between MINLUN and
c ****** MAXLUN, inclusive, is returned in variable ILUN.
c ****** If all units between these limits are busy,
c ****** IERR=1 is returned.
c
c-----------------------------------------------------------------------
c
      integer ilun,ierr
c
c-----------------------------------------------------------------------
c
c ****** Range of valid units.
c
      parameter (minlun=30,maxlun=49)
      logical busy
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Find an unused unit number.
c
      do 100 i=minlun,maxlun
        inquire (unit=i,opened=busy)
        if (.not.busy) go to 200
  100 continue
c
c ****** Fall through here if all units are busy.
c
      ierr=1
      return
c
c ****** An unused unit was found.
c
  200 continue
c
      ilun=i
c
      return
      end
c#######################################################################
      logical function has (n,table,item)
c
c-----------------------------------------------------------------------
c
c ****** Check whether TABLE(N) contains ITEM.
c
c-----------------------------------------------------------------------
c
      character*(*) table(n),item
c
c-----------------------------------------------------------------------
c
      has=.true.
c
      do 100 i=1,n
        if (table(i).eq.item) return
  100 continue
c
      has=.false.
c
      return
      end
c#######################################################################
      subroutine wrfld (fname,ndim1,ndim2,n1,n2,n3,f,d1,d2,d3)
c
c-----------------------------------------------------------------------
c
c ****** Write a field to a file.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
c
c-----------------------------------------------------------------------
c
      if (n3.eq.1) then
        call wr2d (fname,ndim1,n1,n2,f,d1,d2)
      else
        call wr3d (fname,ndim1,ndim2,n1,n2,n3,f,d1,d2,d3)
      end if
c
      return
      end
c#######################################################################
      subroutine wr2d (fname,ndim1,nx,ny,f,x,y)
c
c-----------------------------------------------------------------------
c
c ****** Write a 2D field to a file.
c
c-----------------------------------------------------------------------
c
c ****** The format of the output file is controlled by
c ****** variable DFORMAT.
c
c ****** FNAME should have the root of the file name.
c ****** When DFORMAT='text', the text file is named <root>.dat.
c ****** When DFORMAT='hdf', the hdf file is named <root>.hdf.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(ndim1,ny),x(nx),y(ny) 
c
c-----------------------------------------------------------------------
c
c ****** Array to hold the contiguous HDF data.
c
      real*4 fhdf(nx,ny),x1(nx),y1(ny)
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (dformat.eq.'text') then
c
        fname(lf+1:)='.dat'
        call wr2dtxt (fname,ndim1,nx,ny,f,x,y)
c
      else if (dformat.eq.'hdf') then
c
        fname(lf+1:)='.hdf'
c
c ****** Copy the data into array FHDF.  This is necessary
c ****** because the HDF library requires the data to be
c ****** stored contiguously in the HDF file.
c
        do 110 j=1,ny
        do 100 i=1,nx
          fhdf(i,j)=f(i,j)
  100   continue
  110   continue
        x1(1:nx)=x(1:nx)
        y1(1:ny)=y(1:ny)
c
        call wr2dhdf (fname,nx,ny,fhdf,x1,y1)
c
      else
c
        write (*,*)
        write (*,*) '### ERROR in WR2D:'
        write (*,*) '### Invalid output file format requested.'
        write (*,*) 'DFORMAT = ',dformat
        write (9,*)
        write (9,*) '### ERROR in WR2D:'
        write (9,*) '### Invalid output file format requested.'
        write (9,*) 'DFORMAT = ',dformat
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine wr3d (fname,ndim1,ndim2,n1,n2,n3,f,d1,d2,d3)
c
c-----------------------------------------------------------------------
c
c ****** Write a 3D field to a file.
c
c-----------------------------------------------------------------------
c
c ****** The format of the output file is controlled by
c ****** variable DFORMAT.
c
c ****** FNAME should have the root of the file name.
c ****** When DFORMAT='text', the text file is named <root>.dat.
c ****** When DFORMAT='hdf', the hdf file is named <root>.hdf.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(ndim1,ndim2,*),d1(*),d2(*),d3(*)
c
c-----------------------------------------------------------------------
c
c ****** Array to hold the contiguous HDF data.
c
      real*4 fhdf(n1,n2,n3),d1_(n1),d2_(n2),d3_(n3)
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (dformat.eq.'text') then
c
        fname(lf+1:)='.dat'
        call wr3dtxt (fname,ndim1,ndim2,n1,n2,n3,f,d1,d2,d3)
c
      else if (dformat.eq.'hdf') then
c
        fname(lf+1:)='.hdf'
c
c ****** Copy the data into array FHDF.  This is necessary
c ****** because the HDF library requires the data to be
c ****** stored contiguously in the HDF file.
c
        do 120 k=1,n3
          do 110 j=1,n2
            do 100 i=1,n1
              fhdf(i,j,k)=f(i,j,k)
  100       continue
  110     continue
  120   continue
        d1_(1:n1)=d1(1:n1)
        d2_(1:n2)=d2(1:n2)
        d3_(1:n3)=d3(1:n3)
c
        call wr3dhdf (fname,n1,n2,n3,fhdf,d1_,d2_,d3_)
c
      else
c
        write (*,*)
        write (*,*) '### ERROR in WR3D:'
        write (*,*) '### Invalid output file format requested.'
        write (*,*) 'DFORMAT = ',dformat
        write (9,*)
        write (9,*) '### ERROR in WR3D:'
        write (9,*) '### Invalid output file format requested.'
        write (9,*) 'DFORMAT = ',dformat
        call exit (1)
c
      end if
c
      return
      end
c#######################################################################
      subroutine wr2dtxt (fname,ndim1,nx,ny,f,x,y)
c
c-----------------------------------------------------------------------
c
c ****** Write the 2D field in array F into a file named FNAME.
c
c ****** The scales in X and Y are written also.
c ****** The file is written as a formatted text file.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(ndim1,*),x(nx),y(ny)
c
c-----------------------------------------------------------------------
c
      call ffopen (21,fname,'rw')
      write (21,*) 2
      write (21,*) nx
      write (21,*) ny
      write (21,*) 1
      write (21,100) (x(i),i=1,nx)
      write (21,100) (y(j),j=1,ny)
      write (21,100) ((f(i,j),i=1,nx),j=1,ny)
  100 format (5(1x,1pe13.6))
      close (21)
c
      return
      end
c#######################################################################
      subroutine wr2dhdf (fname,nx,ny,f,x,y)
 
c uk2 change ... call dssnt to tell HDF read that the 
c data to be written are 64 bit.
c
c-----------------------------------------------------------------------
c
c ****** Write the 2D data in array F into an HDF file.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real*4 f(nx,ny),x(nx),y(ny)
c
c-----------------------------------------------------------------------
c
      integer idim(2), i64set, i32set, dsclear
c
c-----------------------------------------------------------------------
c
      idim(1)=nx
      idim(2)=ny
c
c uk2 change ... call to dssnt to set 64 bit data type.
c set argument to 6 for 64 bit floats.
c
c uk2 change ... let's not call dssnt because we want these
c files to be 32 bit. Instead, we'll call dsclear. 
c      i64set = dssnt(6)
c
c      if (i64set == -1) then
c         write(6,*) '64 bit could not be set. stopping in wr2dhdf'
c         stop
c      endif

      i32set = dsclear()
      if (i32set == -1) then
         write(6,*) '32 bit could not be set. stopping in wr2dhdf'
         stop
      endif
      
      call dssdims (2,idim)
      call dssdisc (1,idim(1),x)
      call dssdisc (2,idim(2),y)
      call dspdata (fname,2,idim,f)
c
      return
      end
c#######################################################################
      subroutine wr3dtxt (fname,ndim1,ndim2,nx,ny,nz,f,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Write the 3D field in array F into a file named FNAME.
c
c ****** The scales in X, Y, and Z are written also.
c ****** The file is written as a formatted text file.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(ndim1,ndim2,*),x(nx),y(ny),z(nz)
c
c-----------------------------------------------------------------------
c
      call ffopen (21,fname,'rw')
      write (21,*) 3
      write (21,*) nx
      write (21,*) ny
      write (21,*) nz
      write (21,*) 1
      write (21,100) (x(i),i=1,nx)
      write (21,100) (y(j),j=1,ny)
      write (21,100) (z(k),k=1,nz)
      write (21,100) (((f(i,j,k),i=1,nx),j=1,ny),k=1,nz)
  100 format (5(1x,1pe13.6))
      close (21)
c
      return
      end
c#######################################################################
      subroutine wr3dhdf (fname,nx,ny,nz,f,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Write the 3D data in array F into an HDF file.
c
c uk2 change ... call dssnt to tell HDF that the data sets 
c to be read are 64 bit.
c
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real*4 f(nx,ny,nz),x(nx),y(ny),z(nz)
c
c-----------------------------------------------------------------------
c
      integer idim(3), i64set, i32set, dsclear
c
c-----------------------------------------------------------------------
c
      idim(1)=nx
      idim(2)=ny
      idim(3)=nz
c
c uk2 change ... call to dssnt to set 64 bit data type.
c set argument to 6 for 64 bit floats.
c
c uk2 change ... let's not call dssnt because we want these
c files to be 32 bit. Instead, we'll call dsclear. 
c      i64set = dssnt(6)
c
c      if (i64set == -1) then
c         write(6,*) '64 bit could not be set. stopping in wr2dhdf'
c         stop
c      endif
c
      i32set = dsclear()
      if (i32set == -1) then
         write(6,*) '32 bit could not be set. stopping in wr2dhdf'
         stop
      endif
c
      call dssdims (3,idim)
      call dssdisc (1,idim(1),x)
      call dssdisc (2,idim(2),y)
      call dssdisc (3,idim(3),z)
      call dspdata (fname,3,idim,f)
c
      return
      end
c#######################################################################
      subroutine load_thprof
c
c-----------------------------------------------------------------------
c
c ****** Load the latitudinal heating profile into array THPROF.
c
c-----------------------------------------------------------------------
c
c ****** Smooth latitude profile :
c
c                _________              1
c    ___________/         \___________  0
c    north pole   equator   south pole
c
c ****** The variables THPROF_TH and THPROF_DTH control the profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (thprof_dth.eq.0.) then
c
        do j=1,nt
          thprof(j)=sth(j)**2
        enddo
c
      else
c
        do j=1,nt
          if (th(j).lt..5*pi) then
            angle=th(j)
          else
            angle=pi-th(j)
          endif
          thprof(j)=.5*(1.+tanh((angle-thprof_th)/thprof_dth))
        enddo
c
      endif
      
      return
      end
c#######################################################################
      subroutine load_awthprof
c
c-----------------------------------------------------------------------
c
c ****** Load the latitudinal Alfven wave profile into array THPROF.
c
c-----------------------------------------------------------------------
c
c ****** Smooth latitude profile :
c
c    ___________           ___________  0
c               \_________/             1
c    north pole   equator   south pole
c
c ****** The variables AWTHPROF_TH and AWTHPROF_DTH control the profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      if (awthprof_dth.eq.0.) then
c
        do j=1,nt
          awthprof(j)=1.-sth(j)**2
        enddo
c
      else
c
        do j=1,nt
          if (th(j).lt..5*(awthprof2_th+awthprof_th)) then
            awthprof(j)=
     &      1.-.5*(1.+tanh((th(j)-awthprof_th)/awthprof_dth))
          else
            awthprof(j)=
     &      1.-.5*(1.-tanh((th(j)-awthprof2_th)/awthprof_dth))
          endif
        enddo
c
      endif
      
      return
      end
c#######################################################################
      subroutine initbheat (fname)
c
c-----------------------------------------------------------------------
c
c ****** Read in the heating profile and interpolate it into
c ****** array BRBYB2.
c
c ****** FNAME is the name of the file to read.
c
c-----------------------------------------------------------------------
c
      use rdtxt2d_arrays
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*(*) fname
c
c-----------------------------------------------------------------------
c
c ****** Heating profile array and its scales.
c
      real,pointer:: hp(:,:),thp(:),php(:)

c-----------------------------------------------------------------------
c
c ****** Read in the heating profile.
c
c ****** The routine RDTXT2D communicates the arrays read in through
c ****** module RDTXT2D_ARRAYS.
c
      lf=lenstr(fname)
c
      write (9,*)
      write (9,*) '### COMMENT from INITBHEAT:'
      write (9,*) '### Reading heat profile file: ',fname(1:lf)
      write (9,*) 'NTIME = ',ntime
      write (9,*) 'TIME = ',time
c
      call rdtxt2d (fname,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in INITBHEAT:'
        write (*,*) '### The heat profile file has the wrong format.'
        write (*,*) 'IERR (from RDTXT2D) = ',ierr
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in INITBHEAT:'
        write (9,*) '### The heat profile file has the wrong format.'
        write (9,*) 'IERR (from RDTXT2D) = ',ierr
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** Set the heating profile arrays to point to the data read in
c ****** by RDTXT2D.
c
      hp=>f_rdtxt2d
      thp=>x_rdtxt2d
      php=>y_rdtxt2d
c
c ****** Get the dimensions of the heating profile array read in.
c
      nthp=size(thp)
      nphp=size(php)
c
c ****** Interpolate the field to the code mesh (into array BRBYB2).
c
      call intrp2d_internal
     &   (nthp,nphp,thp,php,hp,nt,np,th,phi,brbyb2,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in INITBHEAT:'
        write (*,*) '### The scales are invalid.'
        write (*,*) 'File name: ',fname(1:lf)
        write (9,*)
        write (9,*) '### ERROR in INITBHEAT:'
        write (9,*) '### The scales are invalid.'
        write (9,*) 'File name: ',fname(1:lf)
        call exit (1)
      end if
c
c ****** De-allocate the memory for the RDTXT2D arrays.
c
      deallocate (f_rdtxt2d)
      deallocate (x_rdtxt2d)
      deallocate (y_rdtxt2d)
c
      return
      end
c#######################################################################
      subroutine loadgamma
c
c-----------------------------------------------------------------------
c
c ****** Load the viscosity profile.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      character*48 errmsg
c
      if (drgamma.eq.0.) then
        gammar=gamma
        gammarh=gamma
        return
      endif

      do i=1,nrm1
        s=.5*(1.+tanh((r(i)-rgamma)/drgamma))
        gammar(i)=gamma1*(1.-s)+gamma2*s
      enddo
      do i=2,nrm1
        gammarh(i)=.5*(gammar(i)+gammar(i-1))
      enddo
c
      write (9,*) '### COMMENT from LOADGAMMA:'

      write (9,'(/,1x,a,/)') 'GAMMA on the integer mesh:'
      do 900 i=1,nrm1
        write (9,910) i,r(i),gammar(i)
  900 continue
      write (9,'(/,1x,a,/)') 'GAMMA on the semi-integer mesh:'
      do i=2,nrm1
        write (9,910) i,rh(i),gammarh(i)
      enddo
  910 format (1x,i4,f11.5,f13.5)
  
      return
      end
c#######################################################################
      subroutine scopy (n,f,incf,g,incg)
      real f(n),g(n)
c
      if (incf.ne.1.or.incg.ne.1) then
        write (*,*) 
        write (*,*) '### ERROR in SCOPY:'
        write (*,*) '### Nonzero increment.'
        write (*,*) '### Buy another SCOPY!'
        call exit (1)
      end if
c
      g(:)=f(:)
c
      return
      end     
c#######################################################################
      function second ()
c
c-----------------------------------------------------------------------
c
c ****** Return the number of CPU seconds elapsed since the
c ****** program started.
c
c ****** This routine is used on Linux systems with PGF90.
c
c ****** Uses the system routine ETIME.
c
c-----------------------------------------------------------------------
c
      implicit none
c
c-----------------------------------------------------------------------
c
      real :: second
c
c-----------------------------------------------------------------------
c
      real(4), dimension(2) :: t
      real :: tot,cpu,sys
c
c-----------------------------------------------------------------------
c
      real(4), external :: etime
c
c-----------------------------------------------------------------------
c
c ****** Note that ETIME returns the total (CPU + system) time used
c ****** as the function value.
c
      tot=etime(t)
      cpu=t(1)
      sys=t(2)
c
      second=cpu
c
      return
      end
c#######################################################################
      subroutine tremain (tleft)
c
c-----------------------------------------------------------------------
c
c ****** Stub for Unicos tremain
c
c-----------------------------------------------------------------------
      real :: tleft
c-----------------------------------------------------------------------
      tleft=100.
      return
      end
c#######################################################################
      subroutine check_if_stoprun
c
c-----------------------------------------------------------------------
c
c ****** Stop the present run if a file named "STOPRUN" exists in
c ****** the run directory.
c
c ****** If it does, set the abort flag IFABORT=.true. and return.
c
c-----------------------------------------------------------------------
c
      use comm
c
c-----------------------------------------------------------------------
c
      logical :: exists
c
c-----------------------------------------------------------------------
c
c ****** Check if the user has requested to stop the run
c ****** prematurely.
c
      inquire (file='STOPRUN',exist=exists)
      if (exists) then
        write (*,*)
        write (*,*) '### WARNING from MAS:'
        write (*,*) '### The run was stopped prematurely'
     &              //' via STOPRUN ...'
        write (*,*) 'NTIME = ',ntime
        write (*,*) 'TIME = ',time
        write (9,*)
        write (9,*) '### WARNING from MAS:'
        write (9,*) '### The run was stopped prematurely'
     &              //' via STOPRUN ...'
        write (9,*) 'NTIME = ',ntime
        write (9,*) 'TIME = ',time
        ifabort=.true.
      end if
c
      return
      end
c#######################################################################
c
c     Bug fixes and enhancements:
c
c
c ### Version 0.10k, 05/13/92, file MAS01, modified by ZM and JL:
c
c       - First version of spherical code.
c         Generated from MAC version 0.77 of 11/27/91 (MAC25).
c
c ### Version 0.20a, 06/23/92, file MAS02, modified by ZM:
c
c       - Split the viscosity and semi implicit advances.
c         This ought to allow the use of smaller physical
c         viscosity.
c       - Added diagnostics of the parallel and perpendicular
c         (to B) kinetic energies versus time.
c       - Added a time-history diagnostic point (at which
c         vr, vt, and vp are stored versus time).
c       - Changed routine BOPEN to check for the existence of a
c         file when using read-only mode.  This makes the error
c         more sensible if a file is missing.
c
c ### Version 0.20b, 07/16/92, file mas03.f, modified by ZM:
c
c       - Added the ability to write a restart file every time a
c         plot dump is made.  Use IFRSDUMP=1 to request this
c         feature.
c
c ### Version 0.20c, 08/03/92, file mas04.f, modified by ZM:
c
c       - Modified the restart file dump to use a different
c         interval than the plot dump.  One can now request a
c         restart file dump every IRSDUMP time steps or every
c         TRSDUMP Alfven times.  The default is not to dump
c         the restart file.  The switch IFRSDUMP has been removed.
c       - Added an option to include the resistive heating term
c         (ohmic heating) in the pressure equation.  Use IFOHMIC=1
c         to request this.
c       - Added a message to the output file when DTIME becomes
c         smaller than DTMIN.
c       - Added logic to end the run when the CPU time left is
c         less than a given value (currently 60 sec).
c
c ### Version 0.20d, 08/06/92, file mas05.f, modified by ZM and JL:
c
c       - Fixed a bug in the algorithm for calculating v-dot-grad-v.
c         We re-discovered this bug while analyzing the apparent
c         lack of upwind diffusion in the current sheet occuring
c         in the arcade problem.  Currently, PDM has been disabled
c         for the computation of the v-dot-grad-v term, i.e., UPWINDV
c         is used directly.
c       - Fixed an anomaly in START having to do with PLOTLIST.
c       - Fixed an (inconsequential) dimension error in INTCHI.
c
c ### Version 0.20e, 11/06/92, file mas06.f, modified by ZM:
c
c       - Put in a printout of TIME when reading a restart file.
c       - Changed PRECOND to vectorize the main loop when IFPREC=1.
c       - Changed the switch which controls whether the pressure is
c         advanced.  A new variable IFP is now used, instead of using
c         BETA0.  When IFP=0, the pressure is not advanced.
c         When IFP=1, the default, the pressure is advanced.
c         (This is similar to the use of IFRHO and IFVDGV.)
c
c ### Version 0.20f, 01/19/93, file mas07.f, modified by ZM:
c
c       - Added the capability for setting an "open" boundary
c         condition at the outer radial boundary.  The variable
c         BCR1TYPE was added.  To request this new open boundary
c         condition, set BCR1TYPE='open'.  The default is as before.
c         With the open BC, quantities are advected out with the
c         radial velocity.
c         Note that GETBC1 does not work after this change.
c       - Added a local vicosity which is set to give a specified
c         cell-Reynolds number, RECELL.  By default this is turned
c         off (RECELL=0.).
c
c ### Version 0.20g, 01/26/93, file mas08.f, modified by JL:
c
c       - Modified the boundary conditions to allow for streamer
c         simulatons: set BCR0TYPE and BCR1TYPE to '1dchar'.
c         With these options, getbc0 computes vr and vt at the
c         lower boundary.  v is computed as an approximation of
c         v parallel to B.   getbc1 computes rho, P, and vr
c         at the upper boundary.  vt and vp are advected out as
c         in the open boundary.
c         Reorganized boundary condition implementation in setbcg
c         (formerly setvbc).  Different bcrtypes can now be set
c         for different options.
c
c ### Version 0.20h, 02/20/93, file mas08xx.f, modified by ZM:
c
c       - Kluged version with special density profile (1/r**4) for
c         magnetic dipole run.  This version has "open" BCs and the
c         special treatment of the equatorial current sheet.
c
c ### Version 0.20i, 02/25/93, file mas08z.f, modified by ZM:
c
c       - Added the option of setting the density to give a uniform
c         Alfven speed.  To use this option, set ISETRHO to the time
c         step interval at which to do this.  The parameter OMEGARHO
c         controls what fraction of the density to update.
c
c ### Version 0.20j, 03/02/93, file mas08zz.f, modified by ZM:
c
c       - Changed the treatment of the equatorial the current sheet.
c         Put in option IFEQCS to turn on special treatment of the
c         current sheet.  The new scheme only changes the points
c         next to the equator in the v-theta equation when NT is odd.
c       - Added variables UPWINDAR and UPWINDAT to control
c         the r and theta upwind derivatives separately.  The default
c         is to use the value of UPWINDA for both.
c
c ### Version 0.20k, 08/19/93, file mas09.F, modified by ZM:
c
c       - This version is identical to version 0.20j, of 03/02/93,
c         from file mab08zz.f.
c
c ### Version 0.20l, 08/19/93, file mas10.F, modified by JL:
c
c       - This version includes an option for viscous heating
c         in the pressure equation (ifhvisc = 1).  Only the m=0
c         case is considered.
c
c ### Version 0.20m, 3/21/94, file mas11.F, modified by JL:
c
c       - modified the convergence criteria in getbc0 to use
c         the maximum of the velocity ub or the sound speed cs.
c         A new namelist variable (gbc0err) was added for specifying
c         the error, default is 1e-5.
c
c ### Version 0.20n, 04/25/94, file mas12.F, modified by ZM:
c
c       - Temporary version to investigate improved characteristic
c         BCs at r=1.
c
c ### Version 0.30, 12/02/94, file mas13.F, modified by ZM:
c
c       - First crack at a three-dimensional version.
c       - Fixed a bug in SETBCV on the BCs at the poles
c         for m=0 on vp.
c       - Removed input parameter IFRSIN.  A restart file is used
c         if the variable RSIFILE is not blank (default).  To use
c         a restart file, set RSIFILE equal to the restart file
c         name.
c       - Cleaned up a bit (removed unused variables).
c       - Added an option to filter the mesh point distributions
c         in an attempt to produce meshes with a more smoothly
c         varying mesh.  The new variables NFRMESH and NFTMESH
c         control the number of times to apply a "(1,2,1)/4"
c         digital filter to the r and theta mesh arrays.
c         The default is NFRMESH=0 and NFTMESH=0, which
c         does not introduce any filtering (as before).
c       - Added diagnostics of the time step and the CFL ratio
c         (i.e., ratio of the time step to the CFL limit) to
c         v-diagnostic file.
c       - Updated RDABS and WRABS to work on long arrays (by
c         introducing a buffer).
c
c ### Version 0.31, 01/03/95, file mas14.F, modified by ZM:
c
c       - Made the resistivity implicit.
c       - Deleted unused variables IFROWEQ and IFSTAB.
c       - Changed the names of CG variables NWRITE, EPS, and ITMAX,
c         to NCGHIST, EPSCG, and NCGMAX, respectively.
c
c ### Version 0.32, 01/13/95, file mas15.F, modified by ZM:
c
c       - Put in the 3D potential field solver.
c       - Removed unused variable BETA0.
c       - Changed the way a case is selected.
c         The new scheme is as follows.  The variable OPTION
c         should be selected to be one of the following:
c
c           OPTION='wave1' or 'wave2' : initialize a test wave;
c           OPTION='streamer'         : do a streamer run;
c           OPTION='dipole'           : do a dipole run.
c
c         For streamer or dipole runs, one can select the initial
c         equilibrium according to the variable EQTYPE:
c
c           EQTYPE='hydrostatic' : load a hydrostatic equilibrium;
c           EQTYPE='zero-beta'   : load a zero beta equilibrium;
c           EQTYPE='parker'      : load a 1D Parker solar wind
c                                  solution from the file whose
c                                  name is specified in ONEDFILE.
c
c         The initial magnetic field can be selected according
c         to the variable FLDTYPE:
c
c           FLDTYPE='dipole'    : load a dipole (optionally tilted);
c           FLDTYPE='potential' : load a potential field with a
c                                 photospheric Br field read in
c                                 from the file whose name is
c                                 specified in BNFILE.
c
c ### Version 0.33, 01/31/95, file mas16.F, modified by ZM:
c
c       - Changed the computation of the advective time step limit.
c         The new version uses "k-dot-v" rather than "|k|-times-|v|".
c       - Changed the name of variables RHOHS0 and PHS0 to the
c         new names RHOR0 and PR0.
c         The value RHOR0 is used to set the base density at r=R0
c         for 1D Parker solar wind equilibria, and for zero-beta
c         equilibria.
c         The values of RHOR0 and PR0 are also used to set the
c         base values at r=R0 for the initial density and pressure
c         profiles for hydrostatic equilibria.
c       - Put in a check to detect when the pressure or density
c         becomes negative.  The run is aborted when this happens.
c       - Made aborts more graceful.  The ouput diagnostics
c         (final spatial diagnostics and time histories) are
c         written upon an abort whenever possible.
c       - Cleaned up the dump of mode histories.  The "m" history
c         files now also have the binary form of the "h" and "v"
c         histories that are written to text files.  This may
c         be useful for post-processing (esp. when the number of
c         significant figures in the text "h" and "v" files is
c         not sufficient).
c
c ### Version 0.34, 02/23/95, file mas17.F, modified by ZM:
c
c       - Improved the formulation in GETBC0, the lower-boundary
c         characteristic-equation solver.  The improved solution
c         method uses a fully consistent formulation in which the
c         gas characteristics are integrated assuming that the
c         flow is along B.
c       - Changed the default to IFPC=1 (predictor-corrector
c         switch for advection).  This turns the predictor-
c         corrector on (as required for 3D runs).
c       - Added the ability to plot all fields.  To do this, set
c         PLOTLIST='all'.
c
c ### Version 0.35, 03/10/95, file mas18.F, modified by ZM:
c
c       - Added time filtering of the boundary velocity
c         in routine GETBC0.  This is an attempt to damp
c         high-frequency oscillations in the boundary velocity.
c
c ### Version 0.36, 10/30/95, file mas19.F, modified by ZM:
c
c       - Corrected a bug in the AXA routine.  The boundary
c         and guard cells of the result vector (y) were not
c         being set to zero.  This prevented the CG solver
c         from converging in certain cases, in an unpredictable
c         manner, due to the dependence on the contents of the
c         boundary values of the CG common block arrays
c         from previous solves.
c       - Improved the CG solution of the semi-implicit equations.
c         Previously, v(n) was used as a guess (to v(n+1)) for the
c         CG solution.  This had the undesirable effect of not
c         adding the explicit part of the solution if the CG
c         solver converged in zero steps.  This could occur if EPSCG
c         were not small enough and the solution was going towards
c         a steady state (with a small time step for which the
c         semi-implicit term was zero). 
c         The new guess uses the velocity advanced with the
c         explicit part of the solution.  This ought to converge
c         in zero steps for an explicit run.
c       - Improved the guess for the CG solution of the
c         implicit resistivity equations.  We now use the
c         explicit part of the solution as an initial guess.
c         This ought to converge rapidly when the resistivity
c         is small (and for ideal MHD).
c       - Improved the guess for the CG solution of the
c         implicit viscosity equations.  We now use the
c         explicit part of the solution as an initial guess.
c         This ought to converge rapidly when the viscosity
c         is small.
c       - Set the default convergence criterion for the
c         CG solver to EPSCG=1.e-8.  This is a more conservative
c         default.  The previous default, EPSCG=1.e-6, may be
c         too large for satisfactory convergence in some cases.
c
c ### Version 0.37, 11/10/95, file mas20.F, modified by ZM and JAC:
c
c       - Put in coronal heating and parallel thermal conduction.
c
c ### Version 0.38, 01/17/96, file mas21.F, modified by ZM and JAC:
c
c       - Put in the capability to control the temperature at
c         the lower radial boundary.  Set the switch USE_TBCR0=.true.
c         to use this option.  The arrays TNODE_T and FNODE_T to
c         specify the temperature time-profile at the lower
c         radial boundary.
c       - Added the ability to dump the temperature at the
c         plotting intervals.  Use PLOTLIST='t' to dump the
c         temperature.
c
c ### Version 0.39, 01/18/96, file mas22.F, modified by ZM and JAC:
c
c       - Generalized the coronal heating term to include the
c         capability to specify the strength of the heating term
c         and the scale length as a function of latitude.
c         The variables HLEN_POLE and HLEN_EQ specify
c         the scale length at the pole and equator, respectively.
c         The strength of the heating term at the photosphere
c         at the pole and equator is specified using the variables
c         H_POLE and H_EQ, respectively.
c         The average heat flux (in ergs/cm**2/sec) is specified
c         by Q0PHYSAV.
c
c ### Version 0.40, 03/15/96, file mas23.F, modified by ZM:
c
c       - Added the ability to use either of two formulations
c         for the characteristic solve at the lower radial boundary.
c         The default, selected by PARCHAR=.false., is to use
c         the "radial gas characteristics" (from version MAS16).
c         Setting PARCHAR=.true. selects the routine that solves
c         the characteristics along B (from version MAS19).
c       - Added the ability to set the parallel velocity in
c         the lower radial boundary characteristic solve to
c         zero if the flow is into the Sun.  This can be selected
c         by setting UBZERO=.true..  The default, UBZERO=.false.,
c         allows flows into the Sun, and sometimes leads to
c         parallel "siphon" flows.
c       - Added the ability to spatially filter the parallel
c         velocity, UB, in the lower radial boundary characteristic
c         solve.  The filtering is applied NFILTUB times.
c         The default, NFILTUB=0, does not apply any filtering.
c       - Changed the assumed format for the 1D initial state
c         input file (specified by ONEDFILE).  The first line of
c         the initial state file is assumed to be a header line,
c         in conformance with our usual convention.
c       - Changed the formulation of the energy equation.
c         The equation for temperature is now solved, rather
c         than for pressure.
c       - Added radiation in the energy equation.
c
c ### Version 0.41, 03/25/96, file mas24.F, modified by ZM:
c
c       - Corrected version.
c       - Changed variable RHOBCR00 to be real.
c
c ### Version 0.41b, 04/26/96, file mas24b.F, modified by ZM:
c
c       - Added in the ability to specify HLEN, H, and Q0PHYS
c         as a fuction of time.
c
c ### Version 0.42, 05/16/96, file mas25.F, modified by ZM:
c
c       - Added Alfven waves.
c       - Added a printout of the average time step used during
c         a run.
c
c ### Version 0.43, 06/13/96, file mas26.F, modified by ZM:
c
c       - Changed the Alfven wave input energy density at the base
c         to be a function of time, QALFVEN_CH.  Note that at
c         present this variable sets the wave pressure at the
c         base in normalized units.
c       - Fixed the reading of 1D input files.  The 1D files
c         contain the electron number density.  This needs to
c         be converted properly to mass density when the
c         Helium fraction is not 0.
c       - Changed the definition of the coronal heating function
c         parameters.  The dependence of the heat flux on
c         HLEN has been taken out, assuming a primary 1D radial
c         dependence.  This is only applies for heating profiles
c         that depend on latitude (when HLEN_POLE_CH and HLEN_EQ_CH
c         are not equal).  The new variables Q0_POLE_CH and Q0_EQ_CH
c         specify the profile of the (approximate) heat flux
c         at r=Rsun, and replace the variables H_POLE_CH and
c         H_EQ_CH.
c
c ### Version 0.44, 07/31/96, file mas27.F, modified by ZM and JL:
c
c       - Unified the treatment of boundary conditions on the
c         electric field at the boundaries.
c       - Introduced the variable ISHEARPROF, which controls the
c         type of shear profile.  When ISHEARPROF=1 (default),
c         use the old-style shear profile, with parameter DTHMAX.
c         When ISHEARPROF=2, do a differential-rotation type of
c         shear profile, with parameters AROTATE, BROTATE,
c         and CROTATE.
c
c ### Version 0.45, 08/08/96, file mas28.F, modified by ZM:
c
c       -Improved the specification of the cell-Reynolds number
c        viscosity and added more options.  The previous version
c        used only the m=0 parts of the flow velocity in the r
c        and theta directions.  The new version uses all components
c        of v in real space.  The new parameters RECELL_FACR, 
c        RECELL_FACT, and RECELL_FACP, can be used to
c        specify only the r, theta, and phi components of the
c        velocity in calculating the flow speed for the
c        Reynolds number.  The default is RECELL_FACR=1.,
c        RECELL_FACT=1., and RECELL_FACP=1.  The new variable
c        RECELL_VISCMAX specifies the maximum cell viscosity,
c        and may be used to limit the cell-Reynolds number
c        viscosity be be less than or equal to RECELL_VISCMAX.
c        The variable RECELL_UNIFT may be used to make the cell
c        viscosity uniform in theta (by setting RECELL_UNIFT=.true.),
c        in which case the viscosity is set to the maximum value
c        in theta for each radius.  The default is to have
c        RECELL_UNIFT=.false.  See routine CELLVISC for details.
c       -Added the ability to specify a cell-magnetic-Reynolds number
c        resistivity.  This is specified similarly to the viscosity,
c        with parameters RMCELL, RMCELL_FACR, RMCELL_FACT,
c        RMCELL_FACP, RMCELL_ETAMAX, RMCELL_UNIFT.
c      - Added the ability to dump the resistivity and viscosity
c        profiles, as well as the cell viscosity and resistivity
c        profiles, at the diagnostic interval.  Add the strings
c        'visc','visccell','eta', and 'etacell' to PLOTLIST to
c        request these.
c      - Added a printout of the maximum velocity to the
c        v-history file.
c      - Added a printout of the maximum cell viscosity and
c        resistivity to a new "q-history" file.
c
c ### Version 0.46, 11/19/96, file mas29.F, modified by ZM:
c
c      - Changed the check of p and T in the advance routine.
c        The new check is for negative values, rather than for
c        non-positive vaues, since a zero-beta case has zero p and T.
c      - Added the option of specifying Br at the photosphere in
c        Gauss.  Set BINGAUSS=.true. to use this option.
c        In this case, the value of B0 is ignored.
c      - Disabled the checking of negative density at the first
c        (ghost) radial cell.
c
c ### Version 0.47, 12/06/96, file mas30.F, modified by ZM:
c
c      - Corrected a bug in routine SETP.  The upper radial
c        boundary value of p was being set incorrectly.
c      - Added the collisionless parallel electron heat flux
c        (a la Withbroe and Hollweg).
c        The collisionless heat flux equals ALPHA_NOCOLL times
c        the internal energy density times the velocity parallel
c        to B.  The default is ALPHA_NOCOLL=0.
c        The collisionless heat flux is turned on smoothly at a
c        radius TC_NOCOLL_R, over a distance TC_NOCOLL_DR.
c
c ### Version 0.48, 12/12/96, file mas31.F, modified by ZM and JL:
c
c      - Added the ability of doing upwind differencing in the
c        phi direction in the advection of rho and T, and in
c        the v-dot-grad-v term.  To select this option, set
c        the variable UPWINDPHI to a positive value.  For full
c        upwind differencing, use a value UPWINDPHI=1.; for
c        partial upwind differencing, use UPWINDPHI between 0.
c        and 1.  To use spectral phi derivatives
c        (i.e., fully centered, as in pervious versions),
c        set UPWINDPHI to any negative value.  The default is
c        UPWINDPHI=-1., which gives the previous (centered)
c        advection algorithm.
c
c ### Version 0.49, 01/10/97, file mas32.F, modified by ZM:
c
c      - This version does not have de-aliasing in the azimuthal
c        direction.
c
c ### Version 0.50, 01/13/97, file mas33.F, modified by ZM:
c
c      - Changed the definition of UB in the GETBC0 routine.
c        UB is now the velocity parallel to B, directed into
c        the domain.
c
c ### Version 0.51, 01/13/97, file mas34.F, modified by ZM:
c
c      - Put in upwinding in the PHI direction in the A equation.
c        The new variable UPWINDAP can be used to control this.
c        The default is to set UPWINDAP to the value of UPWINDA,
c        unless UPWINDA=0., in which case it is assumed that
c        UPWINDAP has been set directly.
c
c ### Version 0.52, 01/28/97, file mas35.F, modified by ZM:
c
c      - Fixed the evaluation of the T*div(v) term in the temperature
c        equation.  When UPWINDPHI.ge.0., this term is now done using
c        centered differences in phi.  When UPWINDPHI.lt.0., this
c        term is evaluated using spectral phi derivatives, as before.
c      - Added the ability to filter the parallel boundary velocity
c        computed in GETBC0.  Previously, the UB velocity was
c        filtered and the parallel velocity was computed from
c        this filtered UB.  If B was changing rapidly in the
c        photosphere, this computed parallel boundary velocity
c        was not smooth.
c
c ### Version 0.53, 06/05/97, file mas36.F, modified by ZM:
c
c      - Changed the CG solver for the parallel thermal conductivity
c        equation.  The new solver solves for the temperature
c        on the usual temperature mesh, and ought to be more
c        accurate.
c
c ### Version 0.54, 07/17/97, file mas37.F, modified by ZM:
c
c      - Added the variable NERADMIN to prevent the transition
c        region radiation boundary condition from giving a
c        negative mass density.  The default value is
c        NERADMIN=-1.e10, which is not expected to limit the
c        density (i.e., this feature is turned off by default).
c
c ### Version 0.55, 07/24/97, file mas38.F, modified by ZM:
c
c      - Changed the averaging in the calculation of the
c        velocity parallel to B in routine GETVPAR.  There
c        was a problem near the current sheet in the old
c        way of averaging B.
c      - This is a temporary version which uses plain old v
c        in the collisionless heat flux.
c
c ### Version 0.56, 09/24/97, file mas39.F, modified by ZM:
c
c      - Cleaned up the use of scratch space.  Scratch arrays
c        are now put in local storage in each subroutine.
c        This ought to simplify porting of the code to
c        a massively parallel computer.
c      - Eliminated many of the pointers that were used
c        purely for convenience (especially those used to
c        make the argument lists of subroutines short).
c
c ### Version 0.57, 10/21/97, file mas40.F, modified by RL:
c
c      - New radiation boundary conditions in advtemp.
c      - New pressure boundary conditions at r=R0
c      - New profile for the heating term
c
c ### Version 0.56b, 10/31/97, file mas39b.F, modified by RL:
c
c      - Took out the pressure BC "fix" at r=R0.
c
c ### Version 0.56e, 01/21/98, file mas39e.F, modified by RL:
c
c      - Introduced wave terms in the predictors and 
c      - SI term in the predictor for v.
c
c ### Version 0.56f, 01/26/98, file mas39f.F, modified by RL:
c
c      - Taken away V.grad.A in the predictor for A.
c
c ### Version 0.56g, 01/27/98, file mas39g.F, modified by ZM:
c
c      - Updated this version with the changes that updated
c        mas39.F to mas39a.F (i.e., fixed assumptions about
c        contiguousness of arrays in memory).
c
c ### Version 0.56h, 02/12/98, file mas39h.F, modified by ZM:
c
c      - Extended the thermal conduction solve to 3D.
c
c ### Version 0.56i, 02/12/98, file mas39i.F, modified by ZM:
c
c      - Added the ability to spatially filter the
c        radiation-balance boundary condition density, NERAD.
c        The filtering is applied NFILTNE0 times.
c        The default, NFILTNE0=0, does not apply any filtering.
c      - Put in separate counters for the various CG solves.
c
c ### Version 0.60, 03/18/98, file mas41.F, modified by ZM:
c
c      - Corrected a bug in the application of the boundary
c        condition on Br at r=R0 in the potential field solve.
c        The averaging of the vector potential at r=R0 was not
c        done correctly, introducing a small difference
c        between the specified Br and the actual Br at r=R0.
c      - Added the capability to perform emerging flux runs.
c        Set EMGFLUX=.true. to use this option.  The sequence
c        of flux files (with Br in them) are specified using
c        the arrays TNODE_EF and BRFILE_EF.
c      - Added output to unit 9 (the run output file) for
c        all abort messages.  Previously, some (most?) error
c        messages only went to the TTY.
c      - Added shear profile type ISHEARPROF=3 to shear
c        streamers that are not symmetric about the equator.
c
c ### Version 0.61, 03/24/98, file mas42.F, modified by ZM:
c
c      - Changed the treatment of boundary conditions on A
c        at r=R0 for emerging flux runs.  Now, the tangential
c        electric field will be zero (i.e., roundoff) when
c        Br stops changing at r=R0.  Note that this changes
c        the definition of the electric field at r=R0, so the
c        new code is not exactly compatible with previous
c        versions (when shearing is employed).  However,
c        note that the electric field is the same in the limit
c        of dr going to zero.
c
c ### Version 0.62, 04/28/98, file mas43.F, modified by ZM:
c
c      - Corrected the boundary conditions on the electric
c        field in routine NEWFLUX.
c      - Added the ability to set the velocity for emerging
c        flux runs equal to the "E x B/B**2" velocity.
c
c ### Version 0.62a, 10/14/98, file mas43a.F, modified by ZM:
c
c      - Changed the name of WRRSFILE to WRRSTRT.
c      - Corrected assumptions about contiguousness of arrays
c        in memory.  It was assumed that arrays were
c        consecutive in memory in routines WRRSTRT and
c        RDRSTRT.
c
c ### Version 0.62b, 10/20/98, file mas43b.F, modified by ZM:
c
c      - Corrected an anomaly in routine ADVTEMP, in which
c        the thermal conductivity was being calculated using a
c        partially-updated temperature (instead of the
c        temperature at the old time step, as intended).
c
c ### Version 0.62c, 10/30/98, file mas43c.F, modified by ZM and AT:
c
c      - Corrected a bug in the POTFLD routine.  The BCs on
c        A-phi were not being set properly at the poles.
c
c ### Version 0.63, 11/05/98, file mas44.F, modified by RL and ZM:
c
c      - Introduced the ability to specify a heating profile
c        from a file.  Use the switch IFBHEAT=1 to select this
c        option, and specify the file name for the (theta,phi)
c        profile of the heating scale length and the
c        heating source strength in BHEATFILE.  The default
c        is IFBHEAT=0, which uses the profiles specified by
c        THPROF_TH and THPROF_DTH.
c      - Introduced a switch, IFRADBC, for radiation boundary
c        conditions.  The default is IFRADBC=1 (radiation
c        boundary conditions used).
c        IFRADBC=0 gives a uniform density at r=R0.
c      - Changed the calculation of the sound speed in
c        SETDT and GETBCG0 to use the temperature (rather than
c        the pressure and density).
c      - The routines to read/write restart files were updated
c        to include the Alfven wave pressure.  Note that
c        as a consequence it is no longer possible to read
c        restart files from previous versions of the code.
c
c ### Version 0.64, 12/02/98, file mas45.F, modified by ZM:
c
c      - Cleaned up the use of the Fourier mode "m" in all
c        phi derivatives to use the arrays FM and FM2.  This
c        now allows for various ways to differentiate in
c        Fourier space.  In particular, by loading FM and FM2
c        properly, it is possible to mimic finite-difference
c        derivatives in phi.
c        Set FDPHI=.true. to mimic finite differences in phi.
c      - Fixed a bug in common block /norm1/.  There was a comma
c        missing.  (Luckily, this bug did not have any effects
c        at present, but had the potential for introducing bugs
c        in the future.)
c
c ### Version 0.65, 02/26/99, file mas46.F, modified by ZM:
c
c      - Replaced the CG solver with one that is more explicitly
c        connected to the 3D mesh.  The pointers were taken
c        out.  This should simplify the parallelization of the
c        code.  
c      - Changed routines READBR and RDTXT2D to use F90
c        allocatable arrays.  This eliminated the use of
c        pointers.
c      - Changed routine INITBHEAT to use the new RDTXT2D
c        structure.
c      - Fixed a bug in ADVTEMP involving derivatives in phi.
c        The code was still using the old-style spectral
c        derivatives in computing a phi-derivative near
c        the r=R0 boundary for the radiation boundary condition.
c        This one was missed in the update of MAS44 to MAS45.
c
c ### Version 0.66, 03/03/99, file mas47.F, modified by ZM and RL:
c
c      - Corrected a bug in routine ADVTEMP.  The module
c        SOLVER_TYPES was not being used.
c      - Changed the way THPROF is used.  The old way made the
c        loops that used THPROF scalar loops, which was
c        inefficient.
c
c ### Version 0.66b, 05/06/1999, file mas47b.F, modified by RL:
c
c      - Made MAS Y2K compliant :-)
c      - New robust characteristic b. c. in GETBC0G
c      - New implicit treatement of radiation losses 
c      - Radiation losses law according to Yung Mok hard wired
c        WARNING: RADINT has still Rosner's law. 
c
c ### Version 0.66c, 07/12/1999, file mas47c.F, modified by RL:
c
c      - Find tipe step limit from radiation law
c      - Introduced RADINT for Athay's (Mok's) radiation law.
c
c ### Version 0.66d, 07/20/1999, file mas47d.F, modified by RL:
c
c      - Introduced IFIMPLRAD flag to choose implicit treatment
c        of radiation law. Default=1 (true)
c      - Fixed bug in DINV_T introduced in mas47b.F by RL (mea culpa!)
c      - Fixed routines WR2D and WR3D to take out the unnecessary
c        parameter statement for the HDF array dimension.
c        This change is made possible by the FORTRAN90 language.
c
c ### Version 0.66d_zm, 10/27/1999, file mas47d_zm.F, modified by ZM:
c
c      - Added an option to modify the thermal conductivity
c        dependence on temperature.  The default is as before
c        (Spitzer formula).  Use ALPHA_TC and T0_TC to modify the
c        dependence of kappa on temperature (see routine GETKAPPA).
c
c ### Version 0.66d_zm2, 01/18/2000, file mas47d_zm2.F, modified by RL:
c
c      - Added  dt0_tc to the modify thermal condication to 
c        change smoothly from one regime to another.
c
c ### Version 0.66d_zm3, 03/27/2001, file mas47d_zm3.F,
c ### modified by RL and ZM:
c
c      - Corrected a bug in GETBC0G in which the wrong theta mesh
c        was used in computing FKSQ.
c      - Added the ability to filter the parallel thermal
c        conductivity tensor.  Use variable NFILT_KAPPA to specify
c        how many times a digital "(1,2,1)/4" filter should be
c        applied to the components of kappa.  By default,
c        NFILT_KAPPA=0, which implies no filtering.
c
c ### Version 0.66d_zm4, 04/10/2001, file mas47d_zm4.F,
c ### modified by ZM:
c
c      - Changed boundary conditions at the poles.  Fourier
c        modes with m larger than or equal to 2 are now set
c        to zero for scalars (temperature, desnity, pressure)
c        and radial vector components.
c
c ### Version 0.66d_zm5, 04/21/2001, file mas47d_zm5.F,
c ### modified by ZM:
c
c      - Added the ability to solve for the viscosity using a
c        split or unsplit formulation.  Use the logical flag
c        SPLIT_VISC to select this option.
c
c ### Version 0.66d_zm6, 04/23/2001, file mas47d_zm6.F,
c ### modified by ZM:
c
c      - Corrected a bug in the implementation of collisionless
c        heat flux.  The boundary points were not set before
c        advecting (nT) in routine ADVTEMP.
c
c ### Version 0.66d_zm6a, 05/18/2001, file mas47d_zm6a.F,
c ### modified by ZM:
c
c      - Fixed the two bugs in the potential solve that caused
c        the current to be nonzero at the highest mode.
c
c ### Version 0.66d_zm6b, 01/23/2002, file mas47d_zm6b.F,
c ### modified by RL:
c
c      - Introduced option for gamma dependent on r 
c      - Fixed bug in DELSQ (in calculating dsqvrr/i there was
c        an r(i) at denominator that should not have been there).
c
c ### Version 0.66d_zm6e, 03/11/2002, file mas47d_zm6e.F,
c ### modified by RL:
c
c      - Made upwindphi=1 by default
c      - Use full predictor VxB in ADVA.
c
c ### Version 0.66d_zm6f, 03/27/2002, file mas47d_zm6f.F,
c ### modified by RL:
c
c      - Fixed bug in SBCT (2D case gave negative values at the poles)
c
c ### Version 0.66d_zm6g, 04/17/2002, file mas47d_zm6g.F,
c ### modified by RL:
c
c      - Use half predictor in VxB in ADVA
c
c ### Version 0.66d_zm6h, 04/18/2002, file mas47d_zm6h.F,
c ### modified by RL:
c
c      - Modified routine CELLETA to compute dynamic resistivity.
c        There are 3 options:
c        IFETA_DYN = 1, set by value of current (beyond a threshold).
c        IFETA_DYN = 2, set proportional to current (beyond a threshold).
c        IFETA_DYN = 3, set proportional to current (beyond a threshold).
c        The new version of CELLETA no longer computes resistivity
c        based on the flow velocity.
c      - Added EPSCGA, a specific convergence criterion for the A 
c        equation.  EPSCGA=EPSCG=1e-8 is the default.
c
c ### Version 0.66d_zm6i 05/09/2002, file mas47d_zm6i.F
c ### modified by RL:
c
c      - Two HDF function calls (DSSNT and DSCLEAR) were diclared
c        integer
c      - Use MNMXPRES instead of MNMX.
c
c ### Version 0.66d_zm6j 05/17/2002, file mas47d_zm6j.F
c ### modified by RL:
c
c      - Modified QDIVV to have the same algorithm as in ADVTEMP
c
c ### Version 0.66d_zm6k, 11/27/2002, file mas47d_zm6k_linux.F,
c ### modified by ZM:
c
c      - Changed the SECOND routine to use the Linux ETIME call.
c      - Added the ability to stop the run prematurely using
c        a STOPRUN file.
c      - Fixed WR2D and WR3D to allocate storage dynamically for
c        the scracth array.  They now work for arbitrary array
c        sizes.
c
c ### Version 0.66d_zm6l 03/04/2003, file mas47d_zm6l_linux.F,
c ### modified by RL:
c 
c      - floor value for density is set to negative.
c
c ### Version 0.66d_ccmc, 04/29/2003, file mas47d_ccmc.F,
c ### modified by RL:
c 
c      - Put back the old GETBC0G from MAS47 that uses an
c        iterative solution.  Another step backward ...
c      - Added an Alfven wave pressure profile. The variables
c        AWTHPROF_TH and AWTHPROF_DTH control the profile,
c        which varies between 1 at the poles and 0 at
c        the equator.
c      - Added AWTHPROF2_DTH to produce non-symmetric Alfven
c        wave profile.
c      - This is the same version as MAS47D_ZM6M_TEST2.F.
c
c ### Version 0.66d_ccmc_v01, 04/30/2003, file mas47d_ccmc_v01.F,
c ### modified by ZM:
c 
c      - Fixed the formatting of the history files in DUMPHIST.
c      - Fixed SECOND to use REAL(4) data types for calls to the
c        system routine ETIME.
c
c#######################################################################
